#!/bin/bash
#
# create_config.sh
#
# Script to create config.h for compatibility with
# different asterisk versions.
#
# Based on script made by Cytronics & Melware
# Armin Schindler <armin@melware.de>
#

CONFIGFILE="config.h"
rm -f "$CONFIGFILE"

VER=1_2

if [ $# -lt 2 ]; then
	echo >&2 "Missing argument"
	exit 1
fi

INCLUDEDIR="$1/asterisk"
INCLUDEDIR_LIBPRI="$2"
ASTMAINDIR="/usr/src/asterisk/main"

if [ ! -d "$INCLUDEDIR" ]; then
	echo >&2 "Include directory '$INCLUDEDIR' does not exist"
	exit 1
fi

if [ ! -d "$INCLUDEDIR_LIBPRI" ]; then
	echo >&2 "Include directory '$INCLUDEDIR_LIBPRI' does not exist"
	exit 1
fi

#echo -n "Checking VoicerLib version... "
VVERSION=`sed -n '/.*VOICERLIB_VERSIONNUMBER /s/^.*VOICERLIB_VERSIONNUMBER //p' /usr/include/voicerlib/version.h`
VVERSION=`echo $VVERSION | sed 's/\"//g'`

if [ -z "$VVERSION" ]; then
    VVERSION="1000"
fi
echo "$VVERSION"

echo -n "Checking Asterisk version at ${INCLUDEDIR}... "
AVERSION=`sed -n '/.*ASTERISK_VERSION /s/^.*ASTERISK_VERSION //p' $INCLUDEDIR/version.h`
AVERSION=`echo $AVERSION | sed 's/\"//g'`
echo $AVERSION

echo "/*" >$CONFIGFILE
echo " * automatically generated by $0 `date`" >>$CONFIGFILE
echo " */" >>$CONFIGFILE
echo >>$CONFIGFILE
echo "#ifndef CHAN_DGV_CONFIG_H" >>$CONFIGFILE
echo "#define CHAN_DGV_CONFIG_H" >>$CONFIGFILE
echo >>$CONFIGFILE

#voicerlib version define
echo "#define VOICERLIB_VERSIONNUMBER $VVERSION">>$CONFIGFILE


if grep -q "ASTERISK_VERSION_NUM 0104" $INCLUDEDIR/version.h; then
	echo "#define CC_AST_HAS_VERSION_1_4" >>$CONFIGFILE
	echo " * found Asterisk version 1.4"
	VER=1_4
else
	if grep -q "ASTERISK_VERSION_NUM 104" $INCLUDEDIR/version.h; then
		echo "#define CC_AST_HAS_VERSION_1_4" >>$CONFIGFILE
		echo " * assuming Asterisk version 1.4"
		VER=1_4
	else
		echo "#undef CC_AST_HAS_VERSION_1_4" >>$CONFIGFILE
	fi
fi

if grep -q "ASTERISK_VERSION_NUM 106" $INCLUDEDIR/version.h; then
	echo "#define CC_AST_HAS_VERSION_1_6" >>$CONFIGFILE
	echo " * found Asterisk version 1.6"
	VER=1_6

	if grep -q "ASTERISK_VERSION_NUM 1060[1-2]" $INCLUDEDIR/version.h; then
		echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
		echo " * found data pointer in ast_frame"
	else
		echo "#undef CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
    		echo " * no data pointer in ast_frame"
	fi
else
	echo "#undef CC_AST_HAS_VERSION_1_6" >>$CONFIGFILE
fi

if grep -q "ASTERISK_VERSION_NUM 108" $INCLUDEDIR/version.h; then
        echo "#define CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
        echo " * found Asterisk version 1.8"
        VER=1_8
	echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
	echo " * found data pointer in ast_frame"
else
        echo "#undef CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
fi

if grep -q "ASTERISK_VERSION_NUM 10[0-1]" $INCLUDEDIR/version.h; then
        echo "#define CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
        echo "#define CC_AST_HAS_VERSION_10" >>$CONFIGFILE
        echo " * found Asterisk version 10"
        VER=10
	echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
	echo " * found data pointer in ast_frame"
else
        echo "#undef CC_AST_HAS_VERSION_10" >>$CONFIGFILE
fi

if grep -q "use 'asterisk\/ast_version\.h' instead" $INCLUDEDIR/version.h; then
	echo "Checking Asterisk version at ${ASTMAINDIR}..."
	if grep -q "asterisk_version_num\[\] = \"11" $ASTMAINDIR/version.c; then
		echo "#define CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_10" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_11" >>$CONFIGFILE
		echo " * found Asterisk version 11"
		VER=11
		echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
		echo " * found data pointer in ast_frame"
	else
		echo "#undef CC_AST_HAS_VERSION_11" >>$CONFIGFILE
	fi

	if grep -q "asterisk_version_num\[\] = \"12" $ASTMAINDIR/version.c; then
		echo "#define CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_10" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_11" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_12" >>$CONFIGFILE
		echo " * found Asterisk version 12"
		VER=12
		echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
		echo " * found data pointer in ast_frame"
	else
		echo "#undef CC_AST_HAS_VERSION_12" >>$CONFIGFILE
	fi

	if grep -q "asterisk_version_num\[\] = \"13" $ASTMAINDIR/version.c; then
		echo "#define CC_AST_HAS_VERSION_1_8" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_10" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_11" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_12" >>$CONFIGFILE
		echo "#define CC_AST_HAS_VERSION_13" >>$CONFIGFILE
		echo " * found Asterisk version 13"
		VER=13
		echo "#define CC_AST_HAS_DATA_PTR" >>$CONFIGFILE
		echo " * found data pointer in ast_frame"
	else
		echo "#undef CC_AST_HAS_VERSION_13" >>$CONFIGFILE
	fi
fi

if grep -q "assignedids" $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL" >>$CONFIGFILE
	echo " * found assignedids in ast_channel"
else
	echo "#undef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL" >>$CONFIGFILE
	echo " * no assignedids in ast_channel"
fi

if grep -q "AST_STRING_FIELD(name)" $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_STRINGFIELD_IN_CHANNEL" >>$CONFIGFILE
	echo " * found stringfield in ast_channel"
else
	echo "#undef CC_AST_HAS_STRINGFIELD_IN_CHANNEL" >>$CONFIGFILE
	echo " * no stringfield in ast_channel"
fi

if grep -q "const show_alarms" $INCLUDEDIR/channel.h; then
        echo "#define CC_AST_HAS_SNMP" >>$CONFIGFILE
        echo " * found snmp in ast_channel"
else
        echo "#undef CC_AST_HAS_SNMP" >>$CONFIGFILE
        echo " * no snmp in ast_channel"
fi

if [ -e $INCLUDEDIR/select.h ]; then
	if grep -q "ast_fdset" $INCLUDEDIR/select.h; then
        	echo "#define CC_AST_HAS_FDSET" >>$CONFIGFILE
	        echo " * found ast_fdset in ast_select"
	else
	        echo "#undef CC_AST_HAS_FDSET" >>$CONFIGFILE
	        echo " * no ast_fdset in ast_select"
	fi
else
        echo "#undef CC_AST_HAS_FDSET" >>$CONFIGFILE
        echo " * no select.h file found"
fi

if grep -q "const get_faxstate" $INCLUDEDIR/channel.h; then
        echo "#define CC_AST_HAS_FAX" >>$CONFIGFILE
        echo " * found get fax state in ast_channel"
else
        echo "#undef CC_AST_HAS_FAX" >>$CONFIGFILE
        echo " * no get fax state in ast_channel"
fi

if grep -q "const indicate.*datalen" $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_INDICATE_DATA" >>$CONFIGFILE
	echo " * found 'indicate' with data"
else
	echo "#undef CC_AST_HAS_INDICATE_DATA" >>$CONFIGFILE
	echo " * no data on 'indicate'"
fi

if grep -q "ast_channel_alloc.*amaflag" $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_EXT2_CHAN_ALLOC" >>$CONFIGFILE
	echo " * found second extended ast_channel_alloc"
else
	echo "#undef CC_AST_HAS_EXT2_CHAN_ALLOC" >>$CONFIGFILE
	echo " * no second extended ast_channel_alloc"
fi

if grep -q "AST_FRFLAG_HAS_TIMING_INFO" $INCLUDEDIR/frame.h; then
	echo "#define CC_AST_HAS_TIMING_INFO_IN_FRAME" >>$CONFIGFILE
	echo " * found clear flag HAS_TIMING_INFO in ast_channel"
else
	echo "#undef CC_AST_HAS_TIMING_INFO_IN_FRAME" >>$CONFIGFILE
	echo " * no clear flag HAS_TIMING_INFO in ast_channel"
fi

if grep -q "reversecharge" $INCLUDEDIR_LIBPRI/q931.c; then
	echo "#define CC_PRI_HAS_REVERSECHARGE" >>$CONFIGFILE
	echo " * found reverse charging detection in libpri"
else
	echo "#undef CC_PRI_HAS_REVERSECHARGE" >>$CONFIGFILE
	echo " * no reverse charging detection in libpri"	
fi

if grep -q "ast_channel_alloc.*cid_num," $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_EXT_CHAN_ALLOC" >>$CONFIGFILE
	echo " * found extended ast_channel_alloc"
else
	echo "#undef CC_AST_HAS_EXT_CHAN_ALLOC" >>$CONFIGFILE
	echo " * no extended ast_channel_alloc"
fi

if grep -q "send_digit_end.*duration" $INCLUDEDIR/channel.h; then
	echo "#define CC_AST_HAS_SEND_DIGIT_END_DURATION" >>$CONFIGFILE
	echo " * found send_digit_end with duration"
else
	echo "#undef CC_AST_HAS_SEND_DIGIT_END_DURATION" >>$CONFIGFILE
	echo " * no duration with send_digit_end"
fi

if [ "$VER" = "1_2" ]; then
if grep -q "AST_JB" $INCLUDEDIR/channel.h; then
	if [ ! -f "$INCLUDEDIR/../../lib/asterisk/modules/chan_sip.so" ]; then
		echo "/* AST_JB */" >>$CONFIGFILE
		echo "#define CC_AST_HAS_JB_PATCH" >>$CONFIGFILE
		echo " * assuming generic jitter-buffer patch"
	else
		if grep -q "ast_jb" "$INCLUDEDIR/../../lib/asterisk/modules/chan_sip.so"; then
			echo "/* AST_JB */" >>$CONFIGFILE
			echo "#define CC_AST_HAS_JB_PATCH" >>$CONFIGFILE
			echo " * found generic jitter-buffer patch"
		else
			echo "#undef CC_AST_HAS_JB_PATCH" >>$CONFIGFILE
			echo " * found DISABLED generic jitter-buffer patch"
		fi
	fi
else
	echo "#undef CC_AST_HAS_JB_PATCH" >>$CONFIGFILE
	echo " * without generic jitter-buffer patch"
fi
fi

echo "" >>$CONFIGFILE
echo "#endif /* CHAN_DGV_CONFIG_H */" >>$CONFIGFILE
echo "" >>$CONFIGFILE

echo "config.h complete."
exit 0

