
/***************************************************************************
 *                                                                         *
 *   chan_dgv.c - Channel Driver for Asterisk (c)                          *
 *                                                                         *
 *   Copyright (c) 2006-2015, DigiVoice Tecnologia em Eletronica Ltda      *
 *                                                                         *
 *   Module: Asterisk Channel for DigiVoice Cards                          *
 *                                                                         *
 *   Author: DigiVoice Tecnologia em Eletronica Ltda                       *
 *   desenvolvimento@digivoice.com.br                                      *
 *                                                                         *
 *   This software is licensed under GPL                                   *
 *   Check LICENSE file for details                                        *
 *                                                                         *
 ***************************************************************************/

//TO USE THE CHANNEL WITHOUT ISDN SIGNALLING COMMENT THE LINE "#define USE_LIBPRI" LIKE "//#define USE_LIBPRI" AND RECOMPILE ALL

#define USE_LIBPRI

#define CHANDGV_VERSION      "1.1.8"
#define VOICERLIB_DEP        4256/*4400 new_vlib_change*/               //VoicerLib dependency Version

//#define DEBUG_Q

#ifdef USE_LIBPRI
	//#define USERUSER_INFO  1
	//#define DEBUG_PRI		 1
	//#define DEBUG_PRI_STEP 1
#endif//#ifdef USE_LIBPRI

//#define DEBUG_LOCKS        1
//#define DEBUG_LAST_LOCK    1

#define DETAILED_VERBOSE     8

#define	DATALEN              160

#define	GSM_DATALEN          33

#define TX_CUT_FACTOR        33                  //8 = 160ms, 16 = 320ms, etc...

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <voicerlib/voicerlib.h>

#include <asterisk.h>

#define NUM_SPANS            12

#define SILENCE_TIMEOUT      3000

#define DEFAULT_MOHINTERPRET "default"

#include "config.h"

#ifdef VOICERLIB_VERSIONNUMBER
    #if VOICERLIB_VERSIONNUMBER < VOICERLIB_DEP
    	#error "**** Incompatible VoicerLib Version. Please check the dependencies ****"
    #endif//#if VOICERLIB_VERSIONNUMBER < VOICERLIB_DEP
#else//#ifdef VOICERLIB_VERSIONNUMBER
    #error "**** Incompatible VoicerLib Version. Please check the dependencies ****"
#endif//#ifdef VOICERLIB_VERSIONNUMBER

u32 last_time_pri;

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 9267 $")

#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <asterisk/lock.h>
#include <asterisk/utils.h>
#include <asterisk/channel.h>
#include <asterisk/config.h>
#include <asterisk/logger.h>
#include <asterisk/module.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/callerid.h>
#include <asterisk/dsp.h>
#include <asterisk/features.h>
#include <asterisk/causes.h>
#include <asterisk/cli.h>
#include <asterisk/sched.h>
#include <asterisk/translate.h>
#include <asterisk/app.h>
#include <asterisk/transcap.h>
#include <asterisk/musiconhold.h>
#include <asterisk/manager.h>
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	#include <asterisk/abstract_jb.h>
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#ifdef USE_LIBPRI
    #include <libpri.h>
#endif//#ifdef USE_LIBPRI

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	#include "asterisk/abstract_jb.h"
	#include "asterisk/devicestate.h"
	#define AST_MODULE "chan_dgv"	
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	static const char desc[] = "DigiVoice Boards Support";
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#ifdef CC_AST_HAS_VERSION_12
	#include "asterisk/pickup.h"
	#include "asterisk/features_config.h"
#endif//#ifdef CC_AST_HAS_VERSION_12

#ifdef CC_AST_HAS_VERSION_13
	#include "asterisk/format_cache.h"
	#include "asterisk/format_compatibility.h"
	#include "asterisk/format_cap.h"
#endif//#ifdef CC_AST_HAS_VERSION_13

#include "dgvchannel.h"

//static const char desc[] = "DigiVoice Boards Support";
static const char ttype[] = "DGV";
static const char tdesc[] = "DiGiVoice voice cards driver";
static const char config[] = "digivoice.conf";
static const char sms_header[] = "DGV_GSM_SMS";
static const char cmd_header[] = "DGV_GET_STATISTICS";
static const char cmd_indicate_header[] = "DGV_SEND_INDICATE";
static const char generic_sep[] = "~";
static const char sms_lock[] = "lock";
//static const char gsm_signalsep[] = "|";
static const char gsm_signalsep = '|';
#define DEFAULT_GSM_REQ_OPERATOR "AT+COPS?"
#define DEFAULT_GSM_REQ_SIMID "AT^SCID"
#define DEFAULT_GSM_REQ_SIMSTATUS "AT^SCKS?"
#define DEFAULT_SMS_EXTEN "dgv_sms"
#define DEFAULT_SMS_ACK_EXTEN "dgv_sms_ack"
#define DEFAULT_USSD_EXTEN "dgv_ussd"
#define DEFAULT_SMS_CHANNEL "-SMS-"
#define DEFAULT_GSM_READ_SMSC_ADDRESS "AT+CSCA?"
#define DEFAULT_GSM_WRITE_SMSC_ADDRESS "AT+CSCA="

#define START_PBX		1
#define DONT_START_PBX	0

//default echo cancellation taps
#define DEFAULT_TAPS    ECHO_TAPS_64

#ifdef CC_AST_HAS_VERSION_13
	#ifndef AST_FORMAT_SLINEAR
		#define AST_FORMAT_SLINEAR AST_FORMAT_SLIN
	#endif//#ifndef AST_FORMAT_SLINEAR
#endif//#ifdef CC_AST_HAS_VERSION_13

static ast_cond_t timer_pending_cond;

static const char *auto_scan_port_desc[] = {"auto","manual"};

static const char *echo_taps_desc[] = {"ECHO OFF","ECHO_TAPS_32","ECHO_TAPS_64","ECHO_TAPS_128","ECHO_TAPS_256","ECHO_TAPS_512","ECHO_TAPS_1024"};

static const char *echo_type_desc[] = {"Hardware Echo Cancellation","Software Echo Cancellation (vlibd)","Software Echo Cancellation"};

static const char *echo_nlp_desc[] = {"OFF","MUTE method","CLIP method","CNG (Confort noise generator) method"};

static const char *sync_mode_desc[] = {"","SYNC_LINE_A","SYNC_LINE_B","SYNC_INTERNAL"};

static const char *consolelanguage_desc[] = {"English", "Portuguese"};

static const char *showalarmstatus_desc[] = {"Off", "On "};

static const char *showcallquality_desc[] = {"Unknown", "Low", "Medium", "High"};

/*const short senoide[] = { 0x0000, 0x09cb, 0x121a, 0x17a6, 0x1999, 0x17a6, 0x121a, 
                                 0x09cb, 0x0000, 0xf635, 0xede6, 0xe85a, 
                                 0xe667, 0xe85a, 0xede6, 0xf635 };*/

/*const char senoide[] = { 0xd5,0x9d,0x83,0x88,
                          0xb5,0x88,0x83,0x9d,
                          0x55,0x1d,0x03,0x08,
                          0x35,0x08,0x03,0x1d }; senoide 500hz alaw   */

const char senoide[] = { 0xd5, 0xed, 0x9d, 0x84, 0x83, 0x8f, 0x88, 0x8a, 0x8a, 0x8a, 0x88, 0x8f, 0x83, 0x84, 0x9d, 0xed, 
                         0x55, 0x6d, 0x1d, 0x04, 0x03, 0x0f, 0x08, 0x0a, 0x35, 0x0a, 0x08, 0x0f, 0x03, 0x04, 0x1d, 0x6d };

/* Default language */
static char global_language[MAX_LANGUAGE] = "";

/* used to identify span interface type - ccs or cas */
enum {
	DG_SPAN_CCS,
    DG_SPAN_CAS
};

//enum used to index span structure
enum {
    DG_SPAN_NUMBER,
    DG_FRAME_TYPE,
    DG_SIDE,
    DG_SWITCHTYPE,
    DG_OVERLAPDIAL,
    DG_PRIDIALPLAN,
    DG_PRILOCALDIALPLAN,
    DG_FACILITYENABLE,
    DG_DISCARDREMOTEHOLDRETRIEVAL
};

#ifdef CC_AST_HAS_VERSION_10
	struct ast_sched_context *sched;     /*!< The scheduling context */
#else//#ifdef CC_AST_HAS_VERSION_10
	static struct sched_context *sched;  /*!< The scheduling context */
#endif//#ifdef CC_AST_HAS_VERSION_10

static short channel_up = 0;		//flag indicating that channel is up and running

static int usecnt =0;

int supported_fmts = AST_FORMAT_SLINEAR | AST_FORMAT_ULAW | AST_FORMAT_ALAW | AST_FORMAT_GSM;

#ifndef CC_AST_HAS_VERSION_1_4
	#ifndef CC_AST_HAS_VERSION_1_6
		#ifndef CC_AST_HAS_VERSION_1_8
			AST_MUTEX_DEFINE_STATIC(usecnt_lock);
		#endif//#ifndef CC_AST_HAS_VERSION_1_8
	#endif//#ifndef CC_AST_HAS_VERSION_1_6		
#endif//#ifndef CC_AST_HAS_VERSION_1_4

/* Protect the interface list (of ast_modem_pvt's) */
ast_mutex_t iflock;

/* these are used to wake up the timer thread when there's work to do */
//AST_MUTEX_DEFINE_STATIC(timer_pending_lock);
ast_mutex_t timer_pending_lock;

static short span_type[NUM_SPANS];

#ifdef USE_LIBPRI
	/* global variables to config cards */
	static short span_pritype[NUM_SPANS];
	static short span_switchtype[NUM_SPANS];
	static short span_overlapdial[NUM_SPANS];
	static short span_pridialplan[NUM_SPANS];
	static short span_prilocaldialplan[NUM_SPANS];
	static short span_facilityenable[NUM_SPANS];
	static short span_discardremoteholdretrieval[NUM_SPANS];
	
	static int thd_socket_running = 0;
	static int thd_pri_running = 0;

	ast_mutex_t ccs_lock[MAX_CARDS*NUM_SPANS_PER_CARD];
	
#ifdef DEBUG_PRI_STEP
		int ccs_lock_count[MAX_CARDS*NUM_SPANS_PER_CARD];
#endif//#ifdef DEBUG_PRI_STEP
	
	//static int dialplan = PRI_NATIONAL_ISDN + 1;
	//static int localdialplan = PRI_NATIONAL_ISDN + 1;

	static int dev_idx = 0;//used only in set_card_config
#endif//#ifdef USE_LIBPRI

/*! \brief How long to wait for following digits */
static int gendigittimeout = 7000;

/*! \brief How long to wait for an extra digit, if there is an ambiguous match */
static int matchdigittimeout = 3000;

static int ringtimeout = 7000;                   //How long to wait for other ring (FXO logic)

//static const int trycnt_avoiddl = 10;          //Max try lock count, after this release and avoid a possible deadlock

static const int max_ndt_count = 3;
static int alarmNDTtimeout = 60000*10;           //How long to wait for enable channel again

static int reservedtimeout = 95000;              //How long to wait for a R2 IDLE (9)

static int gsmgenerictimeout = 45000;            //GSM Thread Configuration
static int gsmerrortimeout = 30000;              //How long to wait for enable channel again
static int gsmsimdetectedtimeout = 1000;         //How long to wait for enable channel again
static int gsmgetsignaltimeout = 60000;          //How long to wait for signal response
static int gsmclearsmstimeout = 40000;           //How long to wait to call dg_GSMClearAllSMS command

static struct {
	short auto_scan_port;						 //1- auto allocates ports (default) - 0 - manual allocations (future implementation)
	//types
	short default_dialtype;
	short default_detectiontype;

	char szDefaultCallProgressFile[TAMSTR];

	short afterdialpause;
	short ringbacktone1;
	short ringbacktone2;	
	char szRingbacktone1Name[TAMSTR];
	char szRingbacktone2Name[TAMSTR];
	char szBusytone1Name[TAMSTR];
	char szBusytone2Name[TAMSTR];
	short busytone1;
	short busytone2;
	char szSubchannelringtone1Name[TAMSTR];
	char szSubchannelringtone2Name[TAMSTR];
	short subchannelringtone1;
	short subchannelringtone2;	
  	unsigned int ringback_times[4];			     //time for ringback
	unsigned int busy_times[4];					 //times for busy
  	unsigned int subchannelring_times[4];	     //time for subchannel ring
	//E1 configurations
	char szDefPortID[TAMSTR];
	//-----
	
    //codec available
	short allow_slinear;
	short allow_ulaw;
	short allow_alaw;
	short allow_gsm;
	
	short consolelanguage;                       //console language
	char deny_prefix[AST_MAX_EXTENSION];         /* this dialed prefix will be denied */

	short gsm_signal_update_interval;            /* Interval to update GSM signal */
} dgv_conf;

typedef struct {
	int nALARM_RSLIP;
    int nALARM_RAIS;
    int nALARM_AISS;
    int nALARM_AIS16S;
    int nALARM_LOSS;
    int nALARM_CRC4SYNC;
    int nALARM_MFSYNC;
    int nALARM_SYNC;
#ifdef USE_LIBPRI
		int nALARM_LINKDOWN;
#endif//#ifdef USE_LIBPRI
} tddgv_e1alarms;

//e1											
tddgv_e1alarms dgv_e1alarms[MAX_CARDS][2];
tddgv_e1alarms dgv_e1alarmsCount[MAX_CARDS][2];
int dgv_sync_conf[MAX_CARDS];
int dgv_sync_alternative_conf[MAX_CARDS];

//gsm
int dgv_gsmsignal[MAX_CARDS][4][2];//Signal and Call Quality
typedef struct {
	int nTotalDialError;
	int nTempDialError;
	int nFlagDialError;
} tddgv_gsmDEalarms;
tddgv_gsmDEalarms dgv_gsmDECount[MAX_CARDS][4];//Dial Error Count

//fxo
int dgv_fxoNDTCount[MAX_CARDS][60];//No Dial Tone Count

//all
char LastAlarmReset[MAX_CARDS][4][256];

static struct {
	char fxs_atxfer[5];                          //atxfer
} features_conf;		

//static char context[AST_MAX_EXTENSION] = "default";

/* functions */

static char *GetCardName(short nCard);
void dgv_cleanup(void);
static struct dgv_pvt *find_ch_port(short port);
static struct dgv_pvt *find_free_port(int group_port[], short is_group[], short backwards[], short dc, short use_roundrobin[], short use_serialnumber[], short use_serialnumber_port[], short use_talktime[], short use_serialnumber_talktime[], short find_sms_reserved);
static struct dgv_pvt *find_free_sms_channel(struct dgv_pvt *tmp);
static struct dgv_pvt *initialize_ports(short port);
void start_dial(struct dgv_pvt *p);
#ifdef USE_LIBPRI
	static inline void start_pri_dial(struct dgv_pvt *p, struct ast_channel *ast);
#endif//#ifdef USE_LIBPRI

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor);
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
		static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability, const char *linkedid);
	#else//#ifdef CC_AST_HAS_VERSION_1_8
		static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause);
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_11
		static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_channel *requestor, const char *data, int *cause);
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
			static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_channel *requestor, void *data, int *cause);
		#else//#ifdef CC_AST_HAS_VERSION_10
			#ifdef CC_AST_HAS_VERSION_1_8
				static struct ast_channel *dgv_request(const char *type, format_t format, const struct ast_channel *requestor, void *data, int *cause);
			#else//#ifdef CC_AST_HAS_VERSION_1_8
				static struct ast_channel *dgv_request(const char *type, int format, void *data, int *cause);
			#endif//#ifdef CC_AST_HAS_VERSION_1_8
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	static int dgv_digit_begin(struct ast_channel *ast, char digit);

	#ifdef CC_AST_HAS_VERSION_11
		static int dgv_devicestate(const char *data);
	#else//#ifdef CC_AST_HAS_VERSION_11
		static int dgv_devicestate(void *data);
	#endif//#ifdef CC_AST_HAS_VERSION_11

	#ifdef CC_AST_HAS_SNMP
		static int dgv_snmp_show_alarms(const char *type, int card, char *alarms);
	#endif//#ifdef CC_AST_HAS_SNMP
	#ifdef CC_AST_HAS_FAX
		static int dgv_faxstate(struct ast_channel *chan);
	#endif//#ifdef CC_AST_HAS_FAX
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

static int load_statistics(void);
static void update_statistics_call(void *data);
static void update_statistics_sms(void *data);
static void reset_statistics(void *data);

#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
	static int dgv_digit(struct ast_channel *ast, char digit, unsigned int duration);
#else//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
	static int dgv_digit(struct ast_channel *ast, char digit);
#endif//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)

#ifdef CC_AST_HAS_VERSION_11
	static int dgv_call(struct ast_channel *ast, const char *dest, int timeout);
#else//#ifdef CC_AST_HAS_VERSION_11
	static int dgv_call(struct ast_channel *ast, char *dest, int timeout);
#endif//#ifdef CC_AST_HAS_VERSION_11

static int dgv_hangup(struct ast_channel *ast);
static int dgv_answer(struct ast_channel *ast);

#ifdef CC_AST_HAS_INDICATE_DATA
	static int dgv_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
	static int dgv_indicate(struct ast_channel *ast, int condition);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA

static int dgv_write(struct ast_channel *ast, struct ast_frame *frame);
static int dgv_fixup(struct ast_channel *oldchan, struct ast_channel *newchan);
static int dgv_sendtext(struct ast_channel *ast, const char *text);

static struct ast_frame *dgv_read(struct ast_channel *ast);

short set_card_config(char *s, short sync, short sync_alternative, short use_crc4);
short set_port_config(short reload,
                      char *s,
                      char *port_id,
                      char *port_name,
                      char *context,
                      short category,
                      short category_pass_through,
                      short min_rx,
					  short max_rx,
					  short interdigit_timeout,
					  short txgain,
					  short rxgain,
					  short cid_pos,
                      short echo_can,
                      char *accountcode,
                      short signalling,
                      short b_subscriber_hangup,
                      short ccb_pass_through,
                      short auto_dgccb,
                      char *deny_prefix,
                      short vconf_pass_through,
                      short group_b_value,
                      short wait_group_b,
                      short silence_threshold,
                      char *musicclass,
                      char *callgroup,
                      char *pickupgroup,
                      short gen_dtmf,
                      short gen_ringback,
                      char *callprogress,
                      char *language,
                      short dtmf_to_asterisk,
                      short dialtone_timeout,
                      short wait_dialtone,
                      short gen_busy,
                      short drop_tx_samples_factor,
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
						  struct ast_jb_conf jbconf,
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
                      short relaxdtmf,
                      short echo_type,
                      short echo_training,
                      short echo_nlp,
                      short ring_type,
                      short flash_min_time,
                      short flash_max_time,
                      short immediate,
                      char *mailbox,
                      short fax_detection,
                      short fax_pass_through,
                      short r2_country,
                      short polarity,
                      short cid_detection_type,
                      short second_call,
                      char *second_call_digits,
                      short gsm_hide_number,
                      short gsm_sms_confirmation,
                      short gsm_sms_flashsms,
                      short gsm_displaycallwaiting,
                      char *gsm_pin_number);
short set_port_group(short group, char *s);
short tone_lookup(char *tone_str);
unsigned int dgv_gettick(void);
int __reload(short reset);
static int __unload_module(int reload);
int reload(void);
int unload_module(void);
int load_module(void);
static int digit_timeout_callback(const void *data);
static int dialtone_timeout_callback(const void *data);
static int alarmNDT_timeout_callback(const void *data);
static int ring_timeout_callback(const void *data);
static int all_echotraining_callback(const void *data);
static int second_call_callback(const void *data);
static int need_sc_ring_callback(const void *data);
static int senddtmf_dialtone_timeout_callback(const void *data);
static int reserved_timeout_callback(const void *data);
static int gsmerror_timeout_callback(const void *data);

static void *fv_thread(void *data); /*Send frametype as AST_FRAME_VOICE; ***zt_read*/
static void *fxss_thread(void *data);//thread used to outgoing call(fxs card)
static void *dialplan_echo_thread(void *data);//enable echo cancellation
static void *vconf_thread(void *data);//enable vconf
static void *ccb_thread(void *data);//wait for a ccb_pass_through
static void *gsm_getsignal_thread(void *data);//update gsm signal

static int send_sms_message(struct ast_channel *ast, int fd, char *text);
static int set_gsm_conf(struct dgv_pvt *p);

#ifdef USE_LIBPRI
	#ifdef CC_AST_HAS_FDSET
		#define dg_fd_set ast_fdset
		#define dg_select(a, b, c, d, e) ast_select(a, b, c, d, e)
	#else//#ifdef CC_AST_HAS_FDSET
		#define dg_fd_set fd_set
		#define dg_select(a, b, c, d, e) select(a, b, c, d, e)
	#endif//#ifdef CC_AST_HAS_FDSET

	static void *ss_thread(void *data);

	int dg_pri_findindex(short card, short span);
	int dg_pri_findpri(struct pri *pri);
	int dg_pri_findfds(int fds);
	int dg_pri_to_port(int pri_channel);
	int port_to_pri(int port);
	static struct dgv_pvt *fixup_pri_channel(int dc, struct dgv_pvt *p, q931_call *c, short clear_old_call);

	void ccs_handler(short card, short span, short size, void *SampleData);

	static pthread_t thd_sock;
	static pthread_t thd_pri;

	static char internationalprefix[10] = "";
	static char nationalprefix[10] = "";
	static char localprefix[20] = "";
	static char privateprefix[20] = "";
	static char unknownprefix[20] = "";
#endif//#ifdef USE_LIBPRI

static int use_callerid = 1;
static char customcascfg[TAMSTR] = "avaya-definity-line-side.cfg";
static char featurescfg[TAMSTR] = "features.conf";

static short set_tone_thread_values(int port);
static void *ToneThread(void *param);
static void *StutterToneThread(void *param);
static void *SubChannelToneThread(void *param);
static void *BeepToneThread(void *param);

int CleanVarSubChannelToneThread(struct dgv_pvt *p, int endStep);

void save_log(short port, char *msg);

void QueueFrame(struct dgv_pvt *p, struct ast_frame *fp);

//Callbacks
void event_handler(void *context_data);
void/*short new_vlib_change*/ read_samples(short port, short size, void *SampleData);
#define ECHO_SW_LEC_U ECHO_SW_LEC//will be removed

//voicerlib definitions & functions
dg_event_data_structure event_context;
int setup_dgv(int reload);

static pthread_t timer_thread = AST_PTHREADT_NULL;

static struct ast_channel_tech dgv_tech = {
	.type = ttype,
	.description = tdesc,
#ifndef CC_AST_HAS_VERSION_10
		.capabilities = AST_FORMAT_ALAW,
#endif//#ifndef CC_AST_HAS_VERSION_10
	.requester = dgv_request,
	.call = dgv_call,
	.hangup = dgv_hangup,
	.answer = dgv_answer,
	.read = dgv_read,
	.write = dgv_write,
#ifndef CC_AST_HAS_VERSION_12
		.bridged_channel = NULL,
		.bridge = NULL,
#endif//#ifndef CC_AST_HAS_VERSION_12
    .fixup = dgv_fixup,
	#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
    	.send_digit_begin = dgv_digit_begin,
    	.send_digit_end = dgv_digit,
    	.devicestate = dgv_devicestate,
		#ifdef CC_AST_HAS_SNMP
			.show_alarms = dgv_snmp_show_alarms,
    	#endif//#ifdef CC_AST_HAS_SNMP
		#ifdef CC_AST_HAS_FAX
			.get_faxstate = dgv_faxstate,
		#endif//#ifdef CC_AST_HAS_FAX
	#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
   		.send_digit = dgv_digit,
	#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	.indicate = dgv_indicate,
	.send_text = dgv_sendtext
};

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	/*! Global jitterbuffer configuration - by default, jb is disabled */
	static struct ast_jb_conf default_jbconf =
	{
		.flags = 0,
		.max_size = -1,
		.resync_threshold = -1,
		.impl = ""
	};
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#ifdef DEBUG_LOCKS
	#define ast_dgv_trylock(a, port) __ast_dgv_mutex_trylock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
	static inline int __ast_dgv_mutex_trylock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
	{
		int res;
		ast_log(LOG_DEBUG, "ast_dgv_trylock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		res = ast_mutex_trylock(t);
		ast_log(LOG_DEBUG, "ast_dgv_trylock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		return res;
	}
#else
//	#ifdef CC_AST_HAS_VERSION_1_6

	
//	#else//#ifdef CC_AST_HAS_VERSION_1_6
		static inline int ast_dgv_trylock(ast_mutex_t *t, int port)	
		{
			return ast_mutex_trylock(t);
		}
	//#endif//#ifdef CC_AST_HAS_VERSION_1_6		
#endif//#ifdef DEBUG_LOCKS

#ifdef DEBUG_LOCKS
	#define ast_dgv_lock(a, port) __ast_dgv_mutex_lock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
	static inline int __ast_dgv_mutex_lock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
	{
		int res;
		ast_log(LOG_DEBUG, "ast_dgv_lock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		res = ast_mutex_lock(t);
		ast_log(LOG_DEBUG, "ast_dgv_lock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		return res;
	}
#else
	#ifdef DEBUG_LAST_LOCK
		#define ast_dgv_lock(a, port) __ast_dgv_mutex_last_lock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
		static inline int __ast_dgv_mutex_last_lock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
		{
			int res;
			struct dgv_pvt *tmp;

			res = ast_mutex_lock(t);
			if (port > 0)
			{
				tmp = find_ch_port(port);
				if (tmp)
				{
					tmp->lock_last_lineno = lineno;
					ast_copy_string(tmp->lock_last_func, func, sizeof(tmp->lock_last_func));
					ast_copy_string(tmp->lock_last_mutex_name, mutex_name, sizeof(tmp->lock_last_mutex_name));
					tmp->lock_last_port = port;
					tmp->lock_last_res = res;
				}
				else
					ast_log(LOG_DEBUG, "ast_dgv_lock: cannot set lastlock for port %d when p is null\n", port);
			}			
			return res;
		}
	#else//#ifdef DEBUG_LAST_LOCK
		static inline int ast_dgv_lock(ast_mutex_t *t, int port)
		{		
			return ast_mutex_lock(t);
		}
	#endif//#ifdef DEBUG_LAST_LOCK
#endif//#ifdef DEBUG_LOCKS	

#ifdef DEBUG_LOCKS
	#define ast_dgv_unlock(a, port) __ast_dgv_mutex_unlock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
	static inline int __ast_dgv_mutex_unlock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
	{
		int res;
		ast_log(LOG_DEBUG, "ast_dgv_unlock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		res = ast_mutex_unlock(t);
		ast_log(LOG_DEBUG, "ast_dgv_unlock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
		return res;
	}
#else
	static inline int ast_dgv_unlock(ast_mutex_t *t, int port)
	{
		return ast_mutex_unlock(t);
	}
#endif//#ifdef DEBUG_LOCKS

/*
#define ast_dgv_trylock(a, port) __ast_dgv_mutex_trylock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
static inline int __ast_dgv_mutex_trylock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
{
	int res;
//ast_channel_lock(chan);
#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_trylock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS

	res = ast_mutex_trylock(t);

#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_trylock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS
	return res;
}

#define ast_dgv_lock(a, port) __ast_dgv_mutex_lock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
static inline int __ast_dgv_mutex_lock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
{
	int res;
//ast_channel_lock(chan);
#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_lock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS

	res = ast_mutex_lock(t);

#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_lock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS
	return res;
}

#define ast_dgv_unlock(a, port) __ast_dgv_mutex_unlock(a, __LINE__, __PRETTY_FUNCTION__, #a, port)
static inline int __ast_dgv_mutex_unlock(ast_mutex_t *t, int lineno, const char *func, const char* mutex_name, int port)
{
	int res;
//ast_channel_lock(chan);
#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_unlock(before): line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS

	res = ast_mutex_unlock(t);

#ifdef DEBUG_LOCKS
	ast_log(LOG_DEBUG, "ast_dgv_unlock(after):  line %05d(%s), port %03d, mutex '%s'\n", lineno, func, port, mutex_name);
#endif//#ifdef DEBUG_LOCKS
	return res;
}
*/

static short statistics_to_file(short channel, short locked, struct timeval total_time, int calls_answered, int sms_sent, char *last_reset)
{
	FILE *f;
	char szFile[100];
	char buftotal[80];

	snprintf(buftotal, sizeof(buftotal), "%ld", (long) total_time.tv_sec);

	//updating file control
	sprintf(szFile, "/var/lib/voicerlib/statistics/dgv_%d", channel);
    if ((f = fopen(szFile, "w")) == NULL)
    	return 0;
    else
    {
    	fprintf(f, "Please, do not edit this file!\n");
		fprintf(f, "%d\n", locked);
		fprintf(f, "%s\n", buftotal);
		fprintf(f, "%d\n", calls_answered);
		fprintf(f, "%d\n", sms_sent);
		fprintf(f, "%s", last_reset);
    	fflush(f);
    	fclose(f);
    }

	return 1;
}

#ifdef USE_LIBPRI
	//message handler
	static void primsg(struct pri *pri, char *s)
	{
		int i;
		int dchan = -1;

		FILE *f;
		char szTemp[CCS_DATA_SIZE*4];
		char szFile[100];

		struct tm *now_dbg;
		time_t curSecs;
		struct timeval tv;

		if (pri)
		{
		    for (i = 0; i < dchans_count; i++)
	    	{
	        	if (dchans[i].pri==pri)
	        	{
	        		dchan = i;
	        		break;
	        	}
	    	}
			if (dchan >= 0)
			{
				gettimeofday(&tv, NULL);
				u32 timenow =  tv.tv_sec*1000L+tv.tv_usec/1000L;

				curSecs = time(NULL);
				now_dbg = localtime(&curSecs);

				sprintf(szTemp, "[%02d]*%02d:%02d:%02d.%06d*%s", dchan, now_dbg->tm_hour, now_dbg->tm_min, now_dbg->tm_sec, timenow, s);
				sprintf(szFile, "/var/log/voicerlib/digi_pri_%02d_%d%02d%02d", dchan, now_dbg->tm_year%100, now_dbg->tm_mon+1, now_dbg->tm_mday);

				f = fopen(szFile, "a+");
				fprintf(f, "%s", szTemp);
				fflush(f);
				fclose(f);

				last_time_pri = timenow;
			}
			else
				ast_log(LOG_WARNING, "Invalid dchan at primsg, do not print this message!\n");
	    }
		else
			ast_log(LOG_WARNING, "Invalid pri received at primsg, do not print this message!\n");
	}

	//error message handler
	static void prierr(struct pri *pri, char *s)
	{
		int i;
		int dchan = -1;

		char szTemp[CCS_DATA_SIZE*4];

		if (pri)
		{
		    for (i = 0; i < dchans_count; i++)
	    	{
	        	if (dchans[i].pri==pri)
	        	{
	        		dchan = i;
	        		break;
	        	}
	    	}
			if (dchan >= 0)
			{
				sprintf(szTemp, "dgv_pri_err: [%02d]%s", dchan, s);

				if (option_verbose >= 3)
					ast_log(LOG_WARNING, "%s", szTemp);
			}
			else
				ast_log(LOG_WARNING, "Invalid dchan at prierr, do not print this message!\n");
	    }
		else
			ast_log(LOG_WARNING, "Invalid pri received at prierr, do not print this message!\n");
	}

	static inline int dgv_trylock_ccs(struct dgv_pvt *p, int dc)
	{
		int res;
		/* Grab the lock first */
		do {
			res = ast_dgv_trylock(&ccs_lock[dc], 0);
			if (res)
			{
				ast_dgv_unlock(&p->lock, p->port);
				/* Release the lock and try again */
				usleep(1);
				ast_dgv_lock(&p->lock, p->port);
			}
		} while (res);

		/* Then break the poll */
		//pthread_kill(pri->master, SIGURG);
		return 0;
	}

	static inline void dgv_releaselock_ccs(int dc)
	{
		ast_dgv_unlock(&ccs_lock[dc], 0);
	}

#endif//#ifdef USE_LIBPRI

static char *GetCardName(short nCard)
{
	/* Old version, before 1.1.7
	switch(dg_GetCardType(nCard))
	{
		case VBE13060PCI:
			return "E13060 rv.1\0";
	    case VBE13030PCI:
    	    return "E13030\0";
		case VB0408PCI:
			return "0408PCI\0";
		case VBE13060PCI_R:
			return "E13060 rv.2\0";
		case VBE16060PCI:
		case VBE16060PCI_R:
			return "E16060 slim\0";
		case VB0404FX:
			return "0404FX\0";
		case VB0404FX_R:
			return "0404FX rv.2\0";
		case VB3030PCIE:
			return "E13030PCIe\0";
		case VB6060PCIE:
			return "E16060PCIe\0";
		case VB0404GSM:
			return "0404GSM\0";			
		case VB0408PCIE:
			return "0408PCIe\0";
		case DGV_2E1F:
			return "DGV-2E1F\0";
		case DGV_1E1F:
			return "DGV-1E1F\0";
		default:
			return "Unknown type\0";
	}*/

	switch(dg_GetCardType(nCard))
	{
		case VBE13060PCI:
			return "VB3060PCI\0";
	    case VBE13030PCI:
    	    return "VB3030PCI\0";
		case VB0408PCI:
			return "VB0408PCI\0";
		case VBE13060PCI_R:
			return "VB3060PCI\0";
		case VBE16060PCI:
		case VBE16060PCI_R:
			return "VB6060PCI\0";
		case VB0404FX:
		case VB0404FX_R:
			return "VB0404FX\0";
		case VB3030PCIE:
			return "VB3030PCIe\0";
		case VB6060PCIE:
			return "VB6060PCIe\0";
		case VB0404GSM:
			return "VB0404GSM\0";
		case VB0408PCIE:
			return "VB0408PCIe\0";
		case DGV_2E1F:
			return "DGV-2E1F\0";
		case DGV_1E1F:
			return "DGV-1E1F\0";
		default:
			return "Unknown type\0";
	}
}

//------------------------------------------------------------
// Convert audio format constants - return string description
//------------------------------------------------------------
static inline char * ast2dgvformatname(int ast_format)
{
#ifdef CC_AST_HAS_VERSION_13
		if (ast_format == ast_format_get_codec_id(ast_format_alaw))
			return "AST_FORMAT_ALAW:ffWaveALaw";
		else
			if (ast_format == ast_format_get_codec_id(ast_format_slin))
				return "AST_FORMAT_SLINEAR:ffWavePCM";
			else
				if (ast_format == ast_format_get_codec_id(ast_format_ulaw))
					return "AST_FORMAT_ULAW:ffWaveULaw";
				else
					if (ast_format == ast_format_get_codec_id(ast_format_gsm))
						return "AST_FORMAT_GSM:ffGsm610";
					else
						return "UNKN:UNKN";
#else//#ifdef CC_AST_HAS_VERSION_13
		switch(ast_format) {
			case AST_FORMAT_ALAW:
				return "AST_FORMAT_ALAW:ffWaveALaw";
			case AST_FORMAT_SLINEAR:
				return "AST_FORMAT_SLINEAR:ffWavePCM";
			case AST_FORMAT_ULAW:
				return "AST_FORMAT_ULAW:ffWaveULaw";
			case AST_FORMAT_GSM:
				return "AST_FORMAT_GSM:ffGsm610";
			default:
				return "UNKN:UNKN";
		}
#endif//#ifdef CC_AST_HAS_VERSION_13
}

//------------------------------------------------------------
// Convert signalling constants - return string description
//------------------------------------------------------------
static inline char *sig2str(int sig)
{
    switch (sig) {
        case DG_SIG_FXO:
            return "FXO";
        case DG_SIG_R2_NDIS_FIXED:
            return "E1 R2D/MFC - NDIS Fixed";
        case DG_SIG_R2_NDIS_VARIABLE:
            return "E1 R2D/MFC - NDIS Variable";
        case DG_SIG_PRI:
            return "E1 ISDN/PRI";
        case DG_SIG_CUSTOMCAS:
            return "E1 CustomCAS";
        case DG_SIG_FXS:
            return "FXS";
        case DG_SIG_CB_FXS:    
            return "E1 R2D - ChannelBank FXS";
        case DG_SIG_CB_FXO:    
            return "E1 R2D - ChannelBank FXO";
        case DG_SIG_GSM:
        	return "GSM";
        default:
            return "Unknown";
   }
}

//------------------------------------------------------------
// Swap ast_channel between pvt and subchannel pvt struct
//------------------------------------------------------------
static void swap_sc(struct dgv_pvt *p, struct dgv_pvt *psc)
{
	struct ast_channel *old_ast_ch;
	
	if ((p) && (psc))
	{
		if ((p->verbosity & DG_LOG_FIND_PORT) != 0)
			ast_log(LOG_NOTICE, "swap_sc: swapping ast_channel on port %d\n", p->port);

		if ((p->ast_ch) && (psc->ast_ch))
		{		
			old_ast_ch  = p->ast_ch;
			p->ast_ch   = psc->ast_ch;
			psc->ast_ch = old_ast_ch;
		}
		else
		{
			if (option_debug == 20)
			{
				if (!p->ast_ch)
					ast_log(LOG_WARNING, "swap_sc: !p->ast_ch\n");
				if (!psc->ast_ch)
					ast_log(LOG_WARNING, "swap_sc: !psc->ast_ch\n");
			}
		}			
	}
	else
	{
		if (option_debug == 20)
		{
			if (!p)
				ast_log(LOG_WARNING, "swap_sc: !p\n");
			if (!psc)
				ast_log(LOG_WARNING, "swap_sc: !psc\n");
		}
	}
}

//------------------------------------------------------------
// Swap ast_channel tech between pvt and subchannel pvt struct
//------------------------------------------------------------
static void swap_sc_tech(struct dgv_pvt *p, struct dgv_pvt *psc)
{
	struct ast_channel *old_tech_pvt;
	
	if ((p) && (psc))
	{
		if ((p->verbosity & DG_LOG_FIND_PORT) != 0)
			ast_log(LOG_NOTICE, "swap_sc_tech: swapping ast_channel tech on port %d\n", p->port);

		if ((p->ast_ch) && (psc->ast_ch))
		{
			if ((dgv_channel_tech_pvt(p->ast_ch)) && (dgv_channel_tech_pvt(psc->ast_ch)))
			{			
				old_tech_pvt = dgv_channel_tech_pvt(p->ast_ch);
				dgv_channel_tech_pvt_set(p->ast_ch, dgv_channel_tech_pvt(psc->ast_ch));
				dgv_channel_tech_pvt_set(psc->ast_ch, old_tech_pvt);
			}
			else
			{
				if (option_debug == 20)
				{
					if (!dgv_channel_tech_pvt(p->ast_ch))
						ast_log(LOG_WARNING, "swap_sc_tech: !dgv_channel_tech_pvt(p->ast_ch)\n");
					if (!dgv_channel_tech_pvt(psc->ast_ch))
						ast_log(LOG_WARNING, "swap_sc_tech: !dgv_channel_tech_pvt(psc->ast_ch)\n");
				}				
			}
		}
		else
		{
			if (option_debug == 20)
			{
				if (!p->ast_ch)
					ast_log(LOG_WARNING, "swap_sc_tech: !p->ast_ch\n");
				if (!psc->ast_ch)
					ast_log(LOG_WARNING, "swap_sc_tech: !psc->ast_ch\n");
			}
		}			
	}
	else
	{
		if (option_debug == 20)
		{
			if (!p)
				ast_log(LOG_WARNING, "swap_sc_tech: !p\n");
			if (!psc)
				ast_log(LOG_WARNING, "swap_sc_tech: !psc\n");
		}
	}
}

//linetone timeout - hangup and free channel
static int dialtone_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
    {
        if (p->call_state == DG_AST_WAITFORDIALTONE)
        {
            //no dial tone
            dg_DisableCallProgress(p->port);

            p->linetone_timeout_sched = -1;

            switch (p->sig)
            {    
                case DG_SIG_GSM:
                	p->wait_dialtone = 0;
                    break;
                //0.9.8 default:
    				//0.9.8 break;
            }

            if (p->wait_dialtone)
            {
            	if (option_verbose >= 3)
                	ast_verbose(VERBOSE_PREFIX_3 "Dgv/%d: No dial tone.\n", p->port);
                
                switch (p->sig)
                {
                    case DG_SIG_FXO:
                    case DG_SIG_CB_FXO:                        	
                        dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1]++;                        
                    	break;
                }

                ast_dgv_lock(&p->lock, p->port);
                {
                	if (p->ast_ch)
                		ast_softhangup(p->ast_ch, AST_SOFTHANGUP_TIMEOUT);
                	else
                	{
                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        	ast_log(LOG_WARNING, "dialtone_timeout_callback: p->ast_ch does not exist on port %d\n", p->port);
                	}
                }
                ast_dgv_unlock(&p->lock, p->port);
            }
            else
            {
                //dial anyway
                start_dial(p);
            }
        }
        else
           	ast_log(LOG_WARNING, "dialtone_timeout_callback: Invalid call state (%d) on port %d, going out...\n", p->call_state, p->port);
    }
    return 0;
}

//after n minutes, try to enable channel again
static int alarmNDT_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
    {		
		p->alarmNDT_timeout_sched = -1;

        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	ast_verbose(VERBOSE_PREFIX_3 "Dgv/%d: Trying to enable channel again.\n", p->port);

        dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1]--;
        p->alarmed = 0;
    }
    return 0;
}

//ring timeout - hangup and free channel
static int ring_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
    {
    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	ast_log(LOG_NOTICE, "ring_timeout_callback on port %d \n", p->port);
        	
     	p->ring_timeout_sched = -1;
    	
        if (p->call_state == DG_AST_RINGING)
        {
            switch(p->sig)
            { 
                case DG_SIG_FXO:
                //case DG_SIG_CB_FXO:
                case DG_SIG_GSM:
                  	if (dg_GetCardInterface(dg_GetCardNumber(p->port)) == DG_FX_INTERFACE && p->sig == DG_SIG_FXO)
                    	dg_SetTwist(p->port, 12, 0 );

                	ast_dgv_lock(&p->lock, p->port);
                	{
                		if (p->ast_ch)
                			ast_softhangup(p->ast_ch, AST_SOFTHANGUP_TIMEOUT);
                		else
                		{
                			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                				ast_log(LOG_WARNING, "ring_timeout_callback: p->ast_ch does not exist on port %d\n", p->port);
                		}
                	}
                	ast_dgv_unlock(&p->lock, p->port);
                    break;
            }
        }
    }
    
    return 0;
}

//timeout function
static int digit_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
	{
		if ((p->sig != DG_SIG_FXS) && (p->sig != DG_SIG_CB_FXS))
		{
	        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    	        ast_log(LOG_WARNING,"********** Port %d receives schedule - id %x!!!!\n",p->port,p->digit_timeout_sched);
        	p->digit_timeout_sched = -1;
	        if (ast_exists_extension(NULL, p->context, p->exten, 1, p->cid_num))
    	    {
        	    //extension found! accept call
            	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                	ast_log(LOG_NOTICE,"digit_timeout: Extension (%s@%s) found!!!.\n", p->exten, p->context);
	            //when send groupB you're saying that you receive enough numbers
    	        dg_R2AskForGroupII(p->port, DG_A3_PULSED);
        	}
	        else
    	    {
        	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            	    ast_log(LOG_NOTICE,"digit_timeout (%d): No more possible extensions. Dropping call.\n", p->port);
            	dg_HangUp(p->port);
	            if ((p->sig==DG_SIG_FXO) || (p->sig==DG_SIG_CUSTOMCAS) || (p->sig==DG_SIG_CB_FXO) || (p->sig==DG_SIG_GSM))
    	            p->reserved=0;
        	    else
        	    {
            	    p->reserved=2;      //wait receiving idle
            	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                	    ast_log(LOG_NOTICE, "digit_timeout (%d): Changing reserved to 2.\n", p->port);
        	    }	            
	        }
		}
	}
	
	return 0;
}

//all_echotraining_callback function
static int all_echotraining_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p)
	{
		if (dg_GetPortCardType(p->port) == DGV_2E1F || dg_GetPortCardType(p->port) == DGV_1E1F)
		{
			//nothing to do, this boards has a Zarlink Echo Canceller
		}
		else
		{
			if (p->call_state == DG_AST_CONVERSATION)
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "all_echotraining (%d): Starting dg_SetEchoCancellationAdapt(4)\n", p->port);

				dg_SetEchoCancelationAdapt(p->port, 4, 0, 0);
			}
		}
		p->all_echotraining_sched = -1;
	}
	return 0;
}

//second_call_callback function
static int second_call_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
	{
		ast_dgv_lock(&p->lock, p->port);
		{
	   		if (option_verbose >= 3)	         	
	   			ast_log(LOG_NOTICE, "second_call_callback: SecondCall digit timeout on port %d\n", p->port);

	        int nSCDialLen = strlen(p->sc_save_digits);
			char cSCDial;
		    int nForLen;	   		
	   		
	        for (nForLen = 0; nForLen < nSCDialLen; nForLen++)
	        {
	        	cSCDial = p->sc_save_digits[nForLen];
	
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "second_call_callback: Sending DTMF %d on port %d \n", cSCDial, p->port);
	
				struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
					f.subclass.integer = cSCDial;
#else//#ifdef CC_AST_HAS_VERSION_1_8
					f.subclass = cSCDial;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
				f.src = "chan_dgv";
				f.datalen = 0;
	            //ast_dgv_lock(&p->lock, p->port);
		        //{
				QueueFrame(p, &f);
				//}
	    		//ast_dgv_unlock(&p->lock, p->port);                        
			}
	
			p->sc_save_digits[0] = '\0';
			p->sc_save_digits[1] = '\0';
			p->sc_save_digits[2] = '\0';
			p->sc_save_digits[3] = '\0';
			p->sc_save_digits[4] = '\0';
			
			p->scd_step = 0;
		    p->second_call_sched = -1;
		}
		ast_dgv_unlock(&p->lock, p->port);
	}
	return 0;
}

//need_sc_ring_callback function
static int need_sc_ring_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p)
	{
   		if (option_verbose >= 3)	         	
   			ast_log(LOG_NOTICE, "need_sc_ring_callback: SecondCallRing timeout on port %d\n", p->port);

		ast_dgv_lock(&p->lock, p->port);
		{
	   		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	   			ast_log(LOG_NOTICE, "need_sc_ring_callback: p->need_sc_ring %d, port %d\n", p->need_sc_ring, p->port);
	   		
			if (p->need_sc_ring >= 1)
			{
				if (p->subchannel)	
				{
					if (p->ast_ch)	
					{
						swap_sc(p, p->subchannel);
					
						ast_setstate(p->ast_ch, AST_STATE_RINGING);
					
						p->call_state = DG_AST_CALLING;
						p->subchannel->call_state = DG_AST_HANGINGUP;
						
						p->cid_num[0] = '\0';
						
 						dg_PickUp(p->port, 20);
	           			
	           			if (p->need_sc_ring == 2)
	           				swap_sc_tech(p, p->subchannel);

						p->subchannel_audio_index = 1;
						p->subchannel->subchannel_audio_index = 1;

						struct ast_frame f = { AST_FRAME_CONTROL, };
						f.src = "chan_dgv";
						f.datalen = 0;
#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						QueueFrame(p->subchannel, &f);
                	}
            		else
            		{
            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_WARNING, "need_sc_ring_callback: p->ast_ch does not exist on port %d\n", p->port);
            		}
            	}
            	else
            	{
            		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            			ast_log(LOG_WARNING, "need_sc_ring_callback: p->subchannel does not exist on port %d\n", p->port);
            		
            		if ((p->ast_ch) && (dg_GetPortStatus(p->port) == spNone))
            		{
						struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
							fmoh.subclass.integer = AST_CONTROL_HOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							fmoh.subclass = AST_CONTROL_HOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						fmoh.src = "chan_dgv";
#ifdef CC_AST_HAS_DATA_PTR						
							fmoh.data.ptr = DEFAULT_MOHINTERPRET;
#else//#ifdef CC_AST_HAS_DATA_PTR
							fmoh.data = DEFAULT_MOHINTERPRET;
#endif//#ifdef CC_AST_HAS_DATA_PTR
						fmoh.datalen = strlen(DEFAULT_MOHINTERPRET) + 1;
						QueueFrame(p, &fmoh);            			

						ast_setstate(p->ast_ch, AST_STATE_RINGING);
						p->call_state = DG_AST_CALLING;
						
						p->cid_num[0] = '\0';
						
						dg_PickUp(p->port, 20);
	           			
						p->subchannel_audio_index = 1;
            		}
            	}
			}
		}
		ast_dgv_unlock(&p->lock, p->port);

	    p->need_sc_ring_sched = -1;
	}
	return 0;
}

static int senddtmf_dialtone_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p)
	{
   		if (option_verbose >= 3)
   			ast_log(LOG_NOTICE, "senddtmf_dialtone_timeout_callback: dialtone timeout on port %d\n", p->port);

   		dg_DisableCallProgress(p->port);
   		
   		if (p->call_state == DG_AST_WAITFORDIALTONE)
   		{
   			ast_dgv_lock(&p->lock, p->port);
   			{
   				p->call_state = DG_AST_CALLING;
   			}
   			ast_dgv_unlock(&p->lock, p->port);
   			
			dg_Dial(p->port, p->dialplan_senddtmf, 20, dtDirectMF);

        	ast_log(LOG_NOTICE, "Dialing using port %d to %s\n", p->port, p->dialplan_senddtmf);
   		}
   		else
   		{
   			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
   				ast_log(LOG_WARNING, "senddtmf_dialtone_timeout_callback: p->call_state != DG_AST_WAITOFORDIALTONE on port %d\n", p->port);	
   		}
	    p->senddtmf_dialtone_timeout_sched = -1;
	}
	return 0;
}

//reserved 2 timeout - set reserved 0
static int reserved_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p) 
    {
    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	ast_log(LOG_NOTICE, "reserved_timeout_callback on port %d \n", p->port);
        	
     	p->reserved_timeout_sched = -1;
    	
        if (p->call_state == DG_AST_NONE && p->ast_ch == NULL && p->reserved == 2)
        {
        	ast_dgv_lock(&p->lock, p->port);
        	{
        		p->reserved = 0;
        	}
        	ast_dgv_unlock(&p->lock, p->port);
        	
        	if (option_verbose >= 3)
        		ast_log(LOG_NOTICE, "reserved_timeout_callback: Changing reserved to 0 on port %d\n", p->port);                	

        }
    }

    return 0;
}

//callback for EV_GSMTIMEOUT or EV_GSMERROR
static int gsmerror_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;
	short ret;
	short nForTryCount;

	if (p)
    {
    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	ast_log(LOG_NOTICE, "gsmerror_timeout_callback on port %d\n", p->port);

    	ast_dgv_lock(&p->lock, p->port);
    	{
    		p->gsmerror_timeout_sched = -1;

    		if (p->alarmed == 99)
    		{
    			p->reserved = p->sms_reserved = 0;
    			p->sms_send_to[0] = '\0';
    			p->sms_send_messagereference = -1;
    			p->sms_send_custom_id[0] = '\0';
    			p->sms_sent_status = 0;

				p->alarmed = 2;
				ast_copy_string(p->gsm_operator_name, "Unknown", sizeof(p->gsm_operator_name));
				ast_copy_string(p->gsm_simcard_id, "Unknown", sizeof(p->gsm_simcard_id));

				ast_dgv_unlock(&p->lock, p->port);

				ret = dg_DestroyGSMThread(p->port);
				if (ret!=DG_EXIT_SUCCESS)
					ast_log(LOG_NOTICE, "Error destroying GSM thread for port %d\n",p->port);
				else
					if ((option_verbose >= 3 && p->gsm_start_log) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
						ast_log(LOG_NOTICE, "Destroying GSM thread for port %d\n",p->port);

				for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)
				{
					sleep(1);

					if (!ast_strlen_zero(p->gsm_pin_number))
					{
						dg_GSMSetPinNumber(p->port, p->gsm_pin_number, NULL);
						if ((option_verbose >= 3 && p->gsm_start_log) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
							ast_log(LOG_NOTICE, "GSM port=%d, PIN=%s\n", p->port, p->gsm_pin_number);
					}

					ret = dg_CreateGSMThread(p->port);
					if (ret==DG_EXIT_SUCCESS)
					{
						if ((option_verbose >= 3 && p->gsm_start_log) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
							ast_log(LOG_NOTICE, "gsmerror_timeout_callback: Re-creating GSM thread for port %d\n", p->port);

						break;
					}
					else
						if (option_debug==50)
							ast_log(LOG_DEBUG, "Error creating GSM thread for port %d - cod %x - step %d\n", p->port, ret, nForTryCount);
				} //for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)

				if (ret!=DG_EXIT_SUCCESS)
					ast_log(LOG_ERROR, "Error creating GSM thread for port %d - cod %x\n", p->port, ret);

				ast_dgv_lock(&p->lock, p->port);
    		} //if (p->alarmed == 99)
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    }

    return 0;
}

//callback for dg_GSMCheckSignalQuality
static int gsmgetsignal_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;
	int res_sched;

	if (p)
    {
    	if (option_debug==100)
    		ast_log(LOG_DEBUG, "gsmgetsignal_timeout_callback on port %d\n", p->port);

    	ast_dgv_lock(&p->lock, p->port);
    	{
    		p->gsmgetsignal_timeout_sched = -1;

       		if (option_verbose >= 3)
       			ast_log(LOG_NOTICE, "No GSM Signal received, restarting GSM port %d\n", p->port);

			if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "gsmgetsignal_timeout_callback (%d) - deleting gsmerror schedule\n", p->port);

				res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
				if (res_sched == -1)
					ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

				p->gsmerror_timeout_sched = -1;
			}

			p->alarmed=99;

			//set gsmerror timeout
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "gsmgetsignal_timeout_callback (%d) - Creating gsmerror schedule\n", p->port);

			p->gsmerror_timeout_sched = ast_sched_add(sched, gsmerrortimeout, gsmerror_timeout_callback, p);

			if (p->gsmclearsms_timeout_sched > -1)
			{
				res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
		    	if (res_sched == -1)
		    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

				p->gsmclearsms_timeout_sched = -1;
			}
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    }

    return 0;
}

//callback for dg_GSMClearAllSMS
static int gsmclearsms_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;
	int ret;

	if (p)
    {
    	if (option_debug==100)
    		ast_log(LOG_DEBUG, "gsmclearsms_timeout_callback on port %d\n", p->port);

    	ast_dgv_lock(&p->lock, p->port);
    	{
    		p->gsmclearsms_timeout_sched = -1;

       		if (option_verbose >= 3)
       			ast_log(LOG_NOTICE, "Reading and Clearing All SMS of GSM port %d\n", p->port);

       		if (p->call_state != DG_AST_CALLING && p->sms_reserved == 0 && p->alarmed == 0 && p->reserved == 0)
       		{
           		if (option_verbose >= 3)
           			ast_log(LOG_NOTICE, "During Read and Clear SMS process, the port %d will be reserved\n", p->port);
                                ast_dgv_unlock(&p->lock, p->port);
				ret = dg_GSMClearAllSMS(p->port, 2); /* request all sim card messages again */
                                ast_dgv_lock(&p->lock, p->port);
				if (ret != EXIT_SUCCESS)
					ast_log(LOG_NOTICE, "Error requesting to read and clear all SMS for card %d, port %d - error code %x\n", p->board, p->port, ret);
				else
					p->reserved = 2;
       		}
			else
			{
				if (option_debug==100)
					ast_log(LOG_DEBUG, "Cannot Read and Clear All SMS for card %d, port %d (call_state %d, sms_reserved %d, alarmed %d, reserved %d)\n", p->board, p->port, p->call_state, p->sms_reserved, p->alarmed, p->reserved);

				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "gsmclearsms_timeout_callback (%d) - Creating gsmclearsms schedule\n", p->port);

				p->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, p);
			}
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    }

    return 0;
}

//callback for dg_GSMSendSMS (without lock_console)
static int gsmsendsms_timeout_callback(const void *data)
{
	struct dgv_pvt *p = (struct dgv_pvt *)data;

	if (p)
    {
    	if (option_debug==100)
    		ast_log(LOG_DEBUG, "gsmsendsms_timeout_callback on port %d\n", p->port);

    	ast_dgv_lock(&p->lock, p->port);
    	{
    		p->gsmsendsms_timeout_sched = -1;

    		if (p->sms_reserved == 1)
    		{
    		    if (option_verbose >= 3)
    		        ast_log(LOG_NOTICE, "Port %d got timeout during Send SMS Message!\n", p->port);

           		manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
                            				 	 "SendTo: %s\r\n"
                            				 	 "Group: %d\r\n"
                            					 "Port: %d\r\n"
                            					 "SIMCardID: %s\r\n"
                            					 "Status: %s\r\n"
                            					 "TimeOut(ms): %d\r\n"
                            					 "CustomID: %s\r\n",
                            					 p->sms_send_to, p->group, p->port, p->gsm_simcard_id, "TIMEOUT", gsmerrortimeout, p->sms_send_custom_id
                              );

				p->sms_reserved = 0;
				p->sms_send_to[0] = '\0';
				p->sms_send_messagereference = -1;
				p->sms_send_custom_id[0] = '\0';
				p->sms_sent_status = 0;
    		}
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    }

    return 0;
}

#ifdef USE_LIBPRI
	static int alerting_timeout_callback(const void *data)
	{
		struct dgv_pvt *p = (struct dgv_pvt *)data;
		int dc;

		if (p)
		{
			if (p->ch_board > 30)
				dc = dg_pri_findindex(p->board, 2);
			else
				dc = dg_pri_findindex(p->board, 1);

			if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
				ast_log(LOG_NOTICE, "alerting_timeout_callback on port %d \n", p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				p->alerting_sig_timeout_sched = -1;

				if (p->pri)
					if (p->call)
						if (p->call_state != DG_AST_CONVERSATION)
							if (p->ast_ch)
								if (dgv_channel_state(p->ast_ch) != AST_STATE_UP)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "alerting_timeout_callback: sending pri_acknowledge port %d, pri_channel %d\n", p->port, p->pri_channel);

									if (!dgv_trylock_ccs(p, dc))
									{
										pri_acknowledge(p->pri, p->call, p->pri_channel, !p->digital);

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "Sending pri_acknowledge %d/%d\n", p->pri_channel, p->port);

										dgv_releaselock_ccs(dc);
									}
									else
										ast_log(LOG_WARNING, "alerting_timeout_callback: Unable to grab ccs_lock on port %d\n", p->port);
								}
			}
			ast_dgv_unlock(&p->lock, p->port);
		}

		return 0;
	}

	static int proceeding_timeout_callback(const void *data)
	{
		struct dgv_pvt *p = (struct dgv_pvt *)data;
		int dc;

		if (p)
		{
			if (p->ch_board > 30)
				dc = dg_pri_findindex(p->board, 2);
			else
				dc = dg_pri_findindex(p->board, 1);

			if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
				ast_log(LOG_NOTICE, "proceeding_timeout_callback on port %d \n", p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				p->proceeding_sig_timeout_sched = -1;

				if (p->pri)
					if (p->call)
						if (p->call_state != DG_AST_CONVERSATION)
							if (p->ast_ch)
								if (dgv_channel_state(p->ast_ch) != AST_STATE_UP)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "proceeding_timeout_callback: sending pri_proceeding port %d, pri_channel %d\n", p->port, p->pri_channel);

									if (!dgv_trylock_ccs(p, dc))
									{
										pri_proceeding(p->pri, p->call, p->pri_channel, !p->digital);

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "Sending pri_proceeding %d/%d\n", p->pri_channel, p->port);

										dgv_releaselock_ccs(dc);
									}
									else
										ast_log(LOG_WARNING, "proceeding_timeout_callback: Unable to grab ccs_lock on port %d\n", p->port);
								}
			}
			ast_dgv_unlock(&p->lock, p->port);
		}

		return 0;
	}

	static int progress_timeout_callback(const void *data)
	{
		struct dgv_pvt *p = (struct dgv_pvt *)data;
		int dc;

		if (p)
		{
			if (p->ch_board > 30)
				dc = dg_pri_findindex(p->board, 2);
			else
				dc = dg_pri_findindex(p->board, 1);

			if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
				ast_log(LOG_NOTICE, "progress_timeout_callback on port %d \n", p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				p->progress_sig_timeout_sched = -1;

				if (p->pri)
					if (p->call)
						if (p->call_state != DG_AST_CONVERSATION)
							if (p->ast_ch)
								if (dgv_channel_state(p->ast_ch) != AST_STATE_UP)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "progress_timeout_callback: sending pri_progress port %d, pri_channel %d\n", p->port, p->pri_channel);

									if (!dgv_trylock_ccs(p, dc))
									{
										pri_progress(p->pri, p->call, p->pri_channel, 1);

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "Sending pri_progress %d/%d\n", p->pri_channel, p->port);

										dgv_releaselock_ccs(dc);
									}
									else
										ast_log(LOG_WARNING, "progress_timeout_callback: Unable to grab ccs_lock on port %d\n", p->port);
								}
			}
			ast_dgv_unlock(&p->lock, p->port);
		}

		return 0;
	}
#endif//#ifdef USE_LIBPRI

//---------------------------------------------------------------------------------
// Save signalling into a file
//---------------------------------------------------------------------------------
void save_log(short port, char *msg)
{
    FILE *f;
    struct dgv_pvt *p=NULL;

    char file_name[TAMSTR];
    
   	p = find_ch_port(port);
    
    if (p) {
        ast_dgv_lock(&p->lock, p->port); 
        {
            sprintf(file_name,"/var/log/voicerlib/dgvchannel-%d",port);
            f=fopen(file_name,"a+");
            if (f!=NULL) {
                fprintf(f,"%s\n",msg);
                fclose(f);
            }
        } 
        ast_dgv_unlock(&p->lock, p->port);
    }
}

//-------------------------------------------------------------------------------------------
// Console Functions
//-------------------------------------------------------------------------------------------

//enables/disables audio debugging messages
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_audio(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_audio(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short port;
	short enable=0;
	struct dgv_pvt *p;

	if (argc != 5)
		return RESULT_SHOWUSAGE;

	//gets port number
	port = atoi(argv[3]);

	if (port<0 || port>=dg_GetPortsCount())
    {
        ast_cli(fd, "ERROR for port %d \n", port);
		return RESULT_SHOWUSAGE;
    }

	if (!strncmp(argv[4], "on", 2))
		enable = 1;
	else
		enable = 0;

	p = find_ch_port(port);
	
	if (p) 
	{
		p->audio_debug = enable;
		ast_cli(fd, "\n       %s messages for audio debugging on port %d \n\n", ((enable) ? "Enabling":"Disabling"), port);
	}
	else
		return RESULT_SHOWUSAGE;

	return RESULT_SUCCESS;
}

//TAPS parameter must be: 
/*	ECHO OFF = 0,
	ECHO_TAPS_32 = 1,
	ECHO_TAPS_64 = 2,
	ECHO_TAPS_128 = 3,
	ECHO_TAPS_256 = 4,
	ECHO_TAPS_512 = 5,
	ECHO_TAPS_1024 = 6
*/
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_echo_can(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_echo_can(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	//Generic port (any cards)
    short port;

    //DGV_2E1F and DGV_1E1F
	int subcommand, param1, param2;

	//else, other cards
#ifdef K_ECHO
		short type;
#endif//#ifdef K_ECHO
	short taps;
    short ret;

    if (argc >= 3)
    	port = atoi(argv[2]);
    else
    	return RESULT_SHOWUSAGE;

    if (dg_GetPortCardType(port) == DGV_2E1F || dg_GetPortCardType(port) == DGV_1E1F)
    {
		if (argc != 6)
			return RESULT_SHOWUSAGE;

		subcommand = atoi(argv[3]);
		param1 = atoi(argv[4]);
		param2 = atoi(argv[5]);

		switch (subcommand)
		{
			case ECHO_FREE:
				ret = dg_DisableEchoCancelation(port);
				if (ret != DG_EXIT_SUCCESS)
					ast_cli(fd, "Error turning off echo cancellation for port %d - code %x\n", port, ret);
				else
					ast_cli(fd, "Turning off echo cancellation for port %d\n", port);
				break;
			case ECHO_ALLOC_DEFAULT:
			case ECHO_ALLOC:
#ifdef K_ECHO
					ret = dg_EnableEchoCancelation(port, type, param1, 0, 0);
#else//#ifdef K_ECHO
					ret = dg_EnableEchoCancelation(port, param1, 0, 0);
#endif//#ifdef K_ECHO
				if (ret != DG_EXIT_SUCCESS)
					ast_cli(fd, "Error starting echo cancellation for port %d - code %x\n", port, ret);
				else
					ast_cli(fd, "Turning on DSP Echo Cancellation for port %d - taps: %d - (%s)\n", port, param1, echo_taps_desc[param1]);
				break;
			default:
				ret = dg_SetEchoCancelationAdapt(port, subcommand, param1, param2);
				if (ret != DG_EXIT_SUCCESS)
					ast_cli(fd, "Error setting echo cancellation adapt for port %d - code %x\n", port, ret);
				break;
		}
    }
    else //if (dg_GetPortCardType(port) == DGV_2E1F || dg_GetPortCardType(port) == DGV_1E1F)
    {
#ifdef K_ECHO
		if (argc != 5)
#else//#ifdef K_ECHO
		if (argc != 4)
#endif//#ifdef K_ECHO
        return RESULT_SHOWUSAGE;

#ifdef K_ECHO
		type = atoi(argv[3]);
		taps = atoi(argv[4]);
#else//#ifdef K_ECHO
		taps = atoi(argv[3]);
#endif//#ifdef K_ECHO

		if (taps==0) {
			ast_cli(fd, "Turning off echo cancellation for port %d\n", port);
			dg_DisableEchoCancelation(port);
		}
		else
		{
			if (taps<=6)
			{
#ifdef K_ECHO
					ret = dg_EnableEchoCancelation(port, type, taps, 800, 3);
					if (ret!=DG_EXIT_SUCCESS) {
						ast_cli(fd, "Error starting echo cancellation for port %d - code %x\n", port,ret);
					}
					else
						if (type == 0)
							ast_cli(fd, "Turning on DSP Echo Cancellation for port %d - taps: %d - (%s)\n", port, taps, echo_taps_desc[taps]);
						else
							ast_cli(fd, "Turning on SW Echo Cancellation for port %d - taps: %d - (%s)\n", port, taps, echo_taps_desc[taps]);
#else//#ifdef K_ECHO
					ret = dg_EnableEchoCancelation(port, taps, 800, 3);
					if (ret!=DG_EXIT_SUCCESS) {
						ast_cli(fd, "Error starting echo cancellation for port %d - code %x\n", port,ret);
					}
					else
						ast_cli(fd, "Turning on DSP Echo Cancellation for port %d - taps: %d - (%s)\n", port, taps, echo_taps_desc[taps]);
#endif//#ifdef K_ECHO
			}
			else
			{
				if (dg_GetPortCardType(port) == DGV_2E1F || dg_GetPortCardType(port) == DGV_1E1F)
				{

				}
				else
				{
					switch (taps)
					{
						case 7:
							dg_SetEchoCancelationAdapt(port, 0, 0, 0);
							break;
						case 8:
							dg_SetEchoCancelationAdapt(port, 1, 0, 0);
							break;
						case 9:
#ifdef K_ECHO
								dg_EnableEchoCancelation(port, type, 3, 888, 3);
#else//#ifdef K_ECHO
								dg_EnableEchoCancelation(port, 3, 888, 3);
#endif//#ifdef K_ECHO
							break;
						case 10:
							dg_SetEchoCancelationAdapt(port, 3, 0, 0);
							break;
						case 11:
							dg_SetEchoCancelationAdapt(port, 4, 0, 0);
							break;
						case 20:
#ifdef K_ECHO
								dg_EnableEchoCancelation(port, type, 5, 800, 0);
#else//#ifdef K_ECHO
								dg_EnableEchoCancelation(port, 5, 800, 0);
#endif//#ifdef K_ECHO
							break;
						case 21:
#ifdef K_ECHO
								dg_EnableEchoCancelation(port, type, 5, 800, 1);
#else//#ifdef K_ECHO
								dg_EnableEchoCancelation(port, 5, 800, 1);
#endif//#ifdef K_ECHO
							break;
						case 22:
#ifdef K_ECHO
								dg_EnableEchoCancelation(port, type, 5, 800, 2);
#else//#ifdef K_ECHO
								dg_EnableEchoCancelation(port, 5, 800, 2);
#endif//#ifdef K_ECHO
							break;
						case 23:
#ifdef K_ECHO
								dg_EnableEchoCancelation(port, type, 5, 800, 3);
#else//#ifdef K_ECHO
								dg_EnableEchoCancelation(port, 5, 800, 3);
#endif//#ifdef K_ECHO
							break;
					}
				}
			}
		}
    } //if (dg_GetPortCardType(port) == DGV_2E1F || dg_GetPortCardType(port) == DGV_1E1F)

    return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_r2_log(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_r2_log(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short port;
	struct dgv_pvt *p;

    if (argc != 4)
        return RESULT_SHOWUSAGE;

    if (!strncmp(argv[3],"on",2))
    {
        dg_EnableDebug(1);
        ast_cli(fd, "\n       All R2/MFC signalling will be saved in /var/log/voicerlib/debug-<ch>\n\n");
    }
    else
    {
    	if (!strncmp(argv[3], "reload", 6))
    	{
            for (port = 1; port <= dg_GetPortsCount(); port++) 
            {
           		p = find_ch_port(port);

                if (p) 
                {
		    		switch (p->sig)
		    		{
		    			case DG_SIG_R2_NDIS_FIXED:
		    			case DG_SIG_R2_NDIS_VARIABLE:
		    			case DG_SIG_CUSTOMCAS:
		    			case DG_SIG_CB_FXS:    
		    			case DG_SIG_CB_FXO:
		    				dg_SendR2Command(p->port, R2_ENABLE);		
		    				
		    		} //switch (p->sig)
                } //if (p)
            } //for (port = 1; port <= dg_GetPortsCount(); port++)
            ast_cli(fd, "\n       Asking for R2 state, it will be saved in /var/log/voicerlib/debug-<ch>\n\n");
    	}
    	else
        {
        	if (!strncmp(argv[3], "reset", 5))
        	{
                for (port = 1; port <= dg_GetPortsCount(); port++) 
                {
               		p = find_ch_port(port);

                    if (p) 
                    {
    		    		switch (p->sig)
    		    		{
    		    			case DG_SIG_R2_NDIS_FIXED:
    		    			case DG_SIG_R2_NDIS_VARIABLE:
    		    			case DG_SIG_CUSTOMCAS:
    		    			case DG_SIG_CB_FXS:    
    		    			case DG_SIG_CB_FXO:    		    				
    		    				 dg_SendR2Command(p->port, R2_IDLE);
    		    		} //switch (p->sig)
                    } //if (p)
                } //for (port = 1; port <= dg_GetPortsCount(); port++)
                ast_cli(fd, "\n       Sending R2_IDLE...\n\n");

                digivoice_sleep(500);

                for (port = 1; port <= dg_GetPortsCount(); port++) 
                {
               		p = find_ch_port(port);

                    if (p) 
                    {
    		    		switch (p->sig)
    		    		{
    		    			case DG_SIG_R2_NDIS_FIXED:
    		    			case DG_SIG_R2_NDIS_VARIABLE:
    		    			case DG_SIG_CUSTOMCAS:
    		    			case DG_SIG_CB_FXS:    
    		    			case DG_SIG_CB_FXO:    		    				
    		    				dg_SendR2Command(p->port, R2_BLOCKED);
    		    		} //switch (p->sig)
                    } //if (p)
                } //for (port = 1; port <= dg_GetPortsCount(); port++)
                ast_cli(fd, "\n       Sending R2_BLOCKED...\n\n");

                digivoice_sleep(1000);
                
                for (port = 1; port <= dg_GetPortsCount(); port++) 
                {
               		p = find_ch_port(port);

                    if (p) 
                    {
    		    		switch (p->sig)
    		    		{
    		    			case DG_SIG_R2_NDIS_FIXED:
    		    			case DG_SIG_R2_NDIS_VARIABLE:
    		    			case DG_SIG_CUSTOMCAS:
    		    			case DG_SIG_CB_FXS:    
    		    			case DG_SIG_CB_FXO:    		    				
    		    				dg_SendR2Command(p->port, R2_IDLE);
    		    		} //switch (p->sig)
                    } //if (p)
                } //for (port = 1; port <= dg_GetPortsCount(); port++)
                ast_cli(fd, "\n       Sending R2_IDLE...\n\n");
                ast_cli(fd, "\n       All R2 state reseted\n\n");
        	}
        	else    
        	{
        		dg_DisableDebug();
        		ast_cli(fd, "\n       R2/MFC signalling log disabled.\n\n");
        	}
        }
    }
    return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_log(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_log(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	if (argc < 4)
		return RESULT_SHOWUSAGE;

	if (!strncmp(argv[3], "on", 2))
	{
		dg_EnableGSMDebug();
		ast_cli(fd, "\n       All GSM signalling will be saved in /var/log/voicerlib/gsm-<ch>.log\n\n");
	}
	else
		if (!strncmp(argv[3], "off", 3))
		{
			dg_DisableGSMDebug();
			ast_cli(fd, "\n       All GSM signalling debug will be disabled\n\n");
		}/*
		else
			if (!strncmp(argv[3], "1", 1))
			{
				dg_CreateGSMThread(atoi(argv[4]));
				ast_cli(fd, "\n       dg_CreateGSMThread(%d)\n\n", atoi(argv[4]));
			}
			else
				if (!strncmp(argv[3], "2", 1))
				{
					dg_DestroyGSMThread(atoi(argv[4]));
					ast_cli(fd, "\n       dg_DestroyGSMThread(%d)\n\n", atoi(argv[4]));
				}
				else
					if (!strncmp(argv[3], "3", 1))
					{
						dg_EnableGSMThread(atoi(argv[4]));
						ast_cli(fd, "\n       dg_EnableGSMThread(%d)\n\n", atoi(argv[4]));
					}
					else
						if (!strncmp(argv[3], "4", 1))
						{
							dg_DisableGSMThread(atoi(argv[4]));
							ast_cli(fd, "\n       dg_DisableGSMThread(%d)\n\n", atoi(argv[4]));
						}*/

	return RESULT_SUCCESS;
}

//dgv verbose <port> <level> <on/off>
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_verbose_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_verbose_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	struct dgv_pvt *p;
	short port;	
	int nForSMSChannel;
	int nArgvReceived;
	int nTempVerbosity;

	if (!channel_up)
	{
		ast_cli(fd, "\n       *** Channel not ready to set verbosity level! ***\n");
		return RESULT_SUCCESS;
	}

	if ((argc != 4) && (argc != 5))
		return RESULT_SHOWUSAGE;

	nTempVerbosity = DG_LOG_ALL_DISABLED;

	nArgvReceived = atoi(argv[3]);

	switch (nArgvReceived)
	{
		case 10:
			nTempVerbosity = DG_LOG_START_END_CALL;
			break;
		case 20:
			nTempVerbosity = DG_LOG_FIND_PORT;
			break;
		case 30:
			nTempVerbosity = DG_LOG_ISDN_EVENTS;
			break;
		case 40:
			nTempVerbosity = DG_LOG_THREADS;
			break;
		case 50:
			nTempVerbosity = DG_LOG_GENERAL_EVENTS;
			break;
		case 60:
			nTempVerbosity = DG_LOG_READ_SAMPLES;
			break;
		case 70:
			nTempVerbosity = DG_LOG_AST_QUEUE_FRAME;
			break;
		case 80:
			nTempVerbosity = DG_LOG_WRITE_SAMPLES;
			break;
		case 90:
			nTempVerbosity = DG_LOG_LOAD_RESET_UNLOAD;
			break;
		case 100:
			nTempVerbosity = DG_LOG_AST_INDICATE;
			break;
		case 110:
			nTempVerbosity = DG_LOG_PAUSEINPUTBUFFER;
			break;
		default:
			nTempVerbosity = DG_LOG_ALL_DISABLED;
	}

	//gets port number
    if (!strncmp(argv[2], "all", 3))
    {
        //all ports
        for (port = 1; port <= dg_GetPortsCount(); port++)
        {
       		p = find_ch_port(port);
        	
            if (p)
            {
				if (nTempVerbosity == DG_LOG_ALL_DISABLED)
				{
					p->verbosity = 0;

					if (p->subchannel)
						p->subchannel->verbosity = 0;

					if (p->sig == DG_SIG_GSM)
						for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
							if (p->smschannel[nForSMSChannel])
								p->smschannel[nForSMSChannel]->verbosity = 0;
				}
				else
					if ((argc == 4) || ((argc == 5) && (!strncmp(argv[4], "on", 2))))
					{
						p->verbosity |= nTempVerbosity;

						if (p->subchannel)
							p->subchannel->verbosity |= p->verbosity;

						if (p->sig == DG_SIG_GSM)
							for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
								if (p->smschannel[nForSMSChannel])
									p->smschannel[nForSMSChannel]->verbosity |= p->verbosity;
					}
					else
					{
						p->verbosity &= ~nTempVerbosity;

						if (p->subchannel)
							p->subchannel->verbosity &= ~p->verbosity;

						if (p->sig == DG_SIG_GSM)
							for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
								if (p->smschannel[nForSMSChannel])
									p->smschannel[nForSMSChannel]->verbosity &= ~p->verbosity;
					}
            }
            else
                return RESULT_SHOWUSAGE;
        } //for (port = 1; port <= dg_GetPortsCount(); port++)

		if ((nTempVerbosity == DG_LOG_ALL_DISABLED) || ((argc == 4) || ((argc == 5) && (!strncmp(argv[4], "on", 2)))))
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Set verbosity %d on ALL ports. \n\n", nArgvReceived);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Verbosidade %d ligada em TODAS as portas. \n\n", nArgvReceived);
					break;
				default:
					ast_cli(fd, "\n       Set verbosity %d on ALL ports. \n\n", nArgvReceived);
			}
		}
		else
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Unset verbosity %d on ALL ports. \n\n", nArgvReceived);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Verbosidade %d desligada em TODAS as portas. \n\n", nArgvReceived);
					break;
				default:
					ast_cli(fd, "\n       Unset verbosity %d on ALL ports. \n\n", nArgvReceived);
			}
		}
	}
    else //if (!strncmp(argv[2], "all", 3))
    {
        port = atoi(argv[2]);

    	if (port < 1 || port > dg_GetPortsCount())
        {
			switch (dgv_conf.consolelanguage)
			{
				case 0://English		
        			ast_cli(fd, "       ERROR for port %d. \n", port);
					break;
				case 1://Portuguese
					ast_cli(fd, "       ERRO para porta %d. \n", port);
					break;
				default:
					ast_cli(fd, "       ERROR for port %d. \n", port);
			}            
            
    		return RESULT_SHOWUSAGE;
        }

    	p = find_ch_port(port);        
        
        if (p)
        {
        	if (nTempVerbosity == DG_LOG_ALL_DISABLED)
        	{
				p->verbosity = 0;

				if (p->subchannel)
					p->subchannel->verbosity = 0;

				if (p->sig == DG_SIG_GSM)
					for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
						if (p->smschannel[nForSMSChannel])
							p->smschannel[nForSMSChannel]->verbosity = 0;

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
	        			ast_cli(fd, "\n       Set verbosity %d on port %d. \n\n", nArgvReceived, port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Verbosidade %d ligada na porta %d. \n\n", nArgvReceived, port);
						break;
					default:
						ast_cli(fd, "\n       Set verbosity %d on port %d. \n\n", nArgvReceived, port);
				}
        	}
			else
				if ((argc == 4) || ((argc == 5) && (!strncmp(argv[4], "on", 2))))
				{
					p->verbosity |= nTempVerbosity;

					if (p->subchannel)
						p->subchannel->verbosity |= p->verbosity;

					if (p->sig == DG_SIG_GSM)
						for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
							if (p->smschannel[nForSMSChannel])
								p->smschannel[nForSMSChannel]->verbosity |= p->verbosity;

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
		        			ast_cli(fd, "\n       Set verbosity %d on port %d. \n\n", nArgvReceived, port);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Verbosidade %d ligada na porta %d. \n\n", nArgvReceived, port);
							break;
						default:
							ast_cli(fd, "\n       Set verbosity %d on port %d. \n\n", nArgvReceived, port);
					}
				}
				else
				{
					p->verbosity &= ~nTempVerbosity;

					if (p->subchannel)
						p->subchannel->verbosity &= ~p->verbosity;

					if (p->sig == DG_SIG_GSM)
						for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
							if (p->smschannel[nForSMSChannel])
								p->smschannel[nForSMSChannel]->verbosity &= ~p->verbosity;

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
		        			ast_cli(fd, "\n       Unset verbosity %d on port %d. \n\n", nArgvReceived, port);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Verbosidade %d desligada na porta %d. \n\n", nArgvReceived, port);
							break;
						default:
							ast_cli(fd, "\n       Unset verbosity %d on port %d. \n\n", nArgvReceived, port);
					}
				}
        }
        else
            return RESULT_SHOWUSAGE;
    } //if (!strncmp(argv[2], "all", 3))

	return RESULT_SUCCESS;
}

//dgv debug voicerlib <card> <level>
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_voicerlib(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_voicerlib(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short card;
	short level;
	short ret=0;

	if (argc != 5)
		return RESULT_SHOWUSAGE;

	level = atoi(argv[4]);
	
	if (level < 0 || level > 5)
    {
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
    			ast_cli(fd, "       ERROR for level %d.\n", level);
				break;
			case 1://Portuguese
				ast_cli(fd, "       ERRO para nivel %d.\n", level);
				break;
			default:
				ast_cli(fd, "       ERROR for level %d.\n", level);
		}
        
		return RESULT_SHOWUSAGE;
    }	
	
	//gets card number
    if (!strncmp(argv[3], "all", 3))
    {
        //all cards
        for (card = 1; card <= dg_GetCardsCount(); card++)
        	ret = dg_SetVerboseLevel(card, level);

		if (ret == DG_FEATURE_NOT_SUPPORTED)
		{
			ast_cli(fd, "\n       dg_SetVerboseLevel error, you need to compile VoicerLib with DEBUG_LEVEL option!\n\n");
			return RESULT_SUCCESS;
		}
		
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
	    		ast_cli(fd, "\n       Set VoicerLib verbosity level %d on ALL cards.\n\n", level);
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Setada Verbosidade da VoicerLib nivel %d em TODAS as placas.\n\n", level);
				break;
			default:
				ast_cli(fd, "\n       Set VoicerLib verbosity level %d on ALL cards.\n\n", level);
		}
	}
    else //if (!strncmp(argv[3], "all", 3))
    {
    	card = atoi(argv[3]);

    	if (card < 1 || card > dg_GetCardsCount())
        {            
			switch (dgv_conf.consolelanguage)
			{
				case 0://English		
        			ast_cli(fd, "       ERROR for card %d. \n", card);
					break;
				case 1://Portuguese
					ast_cli(fd, "       ERRO para placa %d. \n", card);
					break;
				default:
					ast_cli(fd, "       ERROR for card %d. \n", card);
			}

    		return RESULT_SHOWUSAGE;
        }

	    ret = dg_SetVerboseLevel(card, level);
		if (ret == DG_FEATURE_NOT_SUPPORTED)
		{
			ast_cli(fd, "\n       dg_SetVerboseLevel error, you need to compile VoicerLib with DEBUG_LEVEL option!\n\n");
			return RESULT_SUCCESS;
		}
    		
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
	    		ast_cli(fd, "\n       Set VoicerLib verbosity level %d on card %d.\n\n", level, card);
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Setada Verbosidade da VoicerLib nivel %d na placa %d.\n\n", level, card);
				break;
			default:
				ast_cli(fd, "\n       Set VoicerLib verbosity level %d on card %d.\n\n", level, card);		
		}
    } //if (!strncmp(argv[3], "all", 3))

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_isdn(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_debug_isdn(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short pri;
	short enable = 0;
	short i;
	short priCount = 0;

	if (argc != 5)
		return RESULT_SHOWUSAGE;

	if (!strncmp(argv[3], "all", 3))
	{
	    for (i = 0; i < dchans_count; i++)
    	{
			if (!dchans[i].pri)
			{
				ast_cli(fd, "\n       Error Enabling/Disabling messages for ISDN debugging on ISDN PRI %d\n", i);
				return RESULT_SHOWUSAGE;
			}

			if (!strncmp(argv[4], "on", 2))
			{
				pri_set_debug(dchans[i].pri, PRI_DEBUG_ALL);
				enable = 1;
			}
			else
				pri_set_debug(dchans[i].pri, 0);

			ast_cli(fd, "\n       %s messages for ISDN debugging on ISDN PRI %d", ((enable) ? "Enabling":"Disabling"), i);//ja foi

			priCount++;
    	} //for (i = 0; i < dchans_count; i++)

		if (priCount == 0)
		{
			ast_cli(fd, "\n       Error Enabling/Disabling messages for ISDN debugging, no ISDN PRI found\n");
			return RESULT_SHOWUSAGE;
		}

		ast_cli(fd, "\n\n");
	}
	else //if (!strncmp(argv[3], "all", 3))
	{
		pri = atoi(argv[3]);

		if (pri < 0 || pri >= dchans_count || !dchans[pri].pri)
		{
			ast_cli(fd, "\n       Error Enabling/Disabling messages for ISDN debugging on ISDN PRI %d\n", pri);
			return RESULT_SHOWUSAGE;
		}

		if (!strncmp(argv[4], "on", 2))
		{
			pri_set_debug(dchans[pri].pri, PRI_DEBUG_ALL);
			enable = 1;
		}
		else
			pri_set_debug(dchans[pri].pri, 0);

		ast_cli(fd, "\n       %s messages for ISDN debugging on ISDN PRI %d \n\n", ((enable) ? "Enabling":"Disabling"), pri);
	} //if (!strncmp(argv[3], "all", 3))

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_channels(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_channels(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	//#define FORMAT "%7s %7s %-10.10s %-15.15s %-16.16s %-12.12s %-9.9s\n"
	#define FORMAT_SHOW_CHANNELS "%5s %3s %-16.16s %-12.12s %5s %5s %-4.4s %-11.11s %-12.12s %-9.9s\n" //with Rsrvd
	
	int group=0;
	struct dgv_pvt *tmp = NULL;
	char tmps[20] = "";
	char grp_tmps[20] = "";
	char rsrvd_tmps[20] = "";
	char alarmed_tmps[20] = "";
	struct dgv_pvt *start;
	int lin=20;
	int show=0;
	int nForText;
	char textchannels[50000] = "";
	char temptextchannels[1000] = "";

	if (argc == 4)
	{
	    if (!strncmp(argv[3], "all", 3))
	    {
			ast_cli(fd, "\n<");

			for (nForText = 0; nForText <= sizeof(textchannels) - 1; nForText++)
				textchannels[nForText] = '\0';

			sprintf(textchannels, "CHANNELS;CHANNELS_COUNT:%d;", dg_GetPortsCount());

			ast_dgv_lock(&iflock, 0);
			{
				start = iffirst;
				tmp = start;
				while (tmp)
				{
					ast_dgv_lock(&tmp->lock, tmp->port);
					{
						if (tmp->port > 0) {
							snprintf(tmps, sizeof(tmps), "%d", tmp->port);
						} else
							ast_copy_string(tmps, "pseudo", sizeof(tmps));

						snprintf(grp_tmps, sizeof(grp_tmps), "%d", tmp->group);

						snprintf(rsrvd_tmps, sizeof(rsrvd_tmps), "%d", tmp->reserved);

						snprintf(alarmed_tmps, sizeof(alarmed_tmps), "%d", tmp->alarmed);

						switch (dg_GetCardInterface(tmp->board))
						{
							case DG_DIGITAL_INTERFACE:
								sprintf(temptextchannels, "%s:%s,%s,%s,%s,%s,%s,%s,%s,%s;",
										tmps,
										grp_tmps,
										tmp->context,
										tmp->szPortID,
										rsrvd_tmps,
										alarmed_tmps,
										((tmp->locked == 1) ? "yes" : "no"),
										tmp->exten,
										GetCardName(tmp->board),
										"R2/ISDN");
								break;
							case DG_FXO_INTERFACE:
								sprintf(temptextchannels, "%s:%s,%s,%s,%s,%s,%s,%s,%s,%s;",
										tmps,
										grp_tmps,
										tmp->context,
										tmp->szPortID,
										rsrvd_tmps,
										alarmed_tmps,
										((tmp->locked == 1) ? "yes" : "no"),
										tmp->exten,
										GetCardName(tmp->board),
										"FXO");
								break;
							case DG_FX_INTERFACE:
								sprintf(temptextchannels, "%s:%s,%s,%s,%s,%s,%s,%s,%s,%s;",
										tmps,
										grp_tmps,
										tmp->context,
										tmp->szPortID,
										rsrvd_tmps,
										alarmed_tmps,
										((tmp->locked == 1) ? "yes" : "no"),
										tmp->exten,
										GetCardName(tmp->board),
										"FXO/FXS");
								break;
							case DG_GSM_INTERFACE:
								sprintf(temptextchannels, "%s:%s,%s,%s,%s,%s,%s,%s,%s,%s;",
										tmps,
										grp_tmps,
										tmp->context,
										tmp->szPortID,
										rsrvd_tmps,
										((tmp->alarmed < 2) ? alarmed_tmps : (tmp->alarmed == 2) ? "(S) 1" : "(W) 1"),
										((tmp->locked == 1) ? "yes" : "no"),
										tmp->exten,
										GetCardName(tmp->board),
										"GSM");
								break;
						}
					}
					ast_dgv_unlock(&tmp->lock, tmp->port);

					strcat(textchannels, temptextchannels);
					tmp = tmp->next;
				}//while (tmp)
			}
			ast_dgv_unlock(&iflock, 0);

			textchannels[strlen(textchannels) - 1] = '\0';
			strcat(textchannels, "> \n\n");
			ast_cli(fd, "%s", textchannels);

			return RESULT_SUCCESS;
		}
		else
			if (!strncmp(argv[3], "json", 4))
			{
				ast_cli(fd, "\n");

				for (nForText = 0; nForText <= sizeof(textchannels) - 1; nForText++)
					textchannels[nForText] = '\0';

				ast_copy_string(textchannels,
						        "{\"parameters\":{\"c\":\"channel\",\"g\":\"group\",\"d\":\"context\",\"p\":\"portid\",\"r\":\"reserved\",\"a\":\"alarmed\",\"k\":\"locked\",\"e\":\"extension\",\"y\":\"cardtype\",\"i\":\"interface\"},\"values\":[",
						        sizeof(textchannels));

				ast_dgv_lock(&iflock, 0);
				{
					start = iffirst;
					tmp = start;
					while (tmp)
					{
						ast_dgv_lock(&tmp->lock, tmp->port);
						{
							if (tmp->port > 0) {
								snprintf(tmps, sizeof(tmps), "%d", tmp->port);
							} else
								ast_copy_string(tmps, "pseudo", sizeof(tmps));

							snprintf(grp_tmps, sizeof(grp_tmps), "%d", tmp->group);

							snprintf(rsrvd_tmps, sizeof(rsrvd_tmps), "%d", tmp->reserved);

							snprintf(alarmed_tmps, sizeof(alarmed_tmps), "%d", tmp->alarmed);

							switch (dg_GetCardInterface(tmp->board))
							{
								case DG_DIGITAL_INTERFACE:
									sprintf(temptextchannels, "{\"c\":\"%s\",\"g\":\"%s\",\"d\":\"%s\",\"p\":\"%s\",\"r\":\"%s\",\"a\":\"%s\",\"k\":\"%s\",\"e\":\"%s\",\"y\":\"%s\",\"i\":\"%s\"},",
											tmps,
											grp_tmps,
											tmp->context,
											tmp->szPortID,
											rsrvd_tmps,
											alarmed_tmps,
											((tmp->locked == 1) ? "yes" : "no"),
											tmp->exten,
											GetCardName(tmp->board),
											"R2/ISDN");
									break;
								case DG_FXO_INTERFACE:
									sprintf(temptextchannels, "{\"c\":\"%s\",\"g\":\"%s\",\"d\":\"%s\",\"p\":\"%s\",\"r\":\"%s\",\"a\":\"%s\",\"k\":\"%s\",\"e\":\"%s\",\"y\":\"%s\",\"i\":\"%s\"},",
											tmps,
											grp_tmps,
											tmp->context,
											tmp->szPortID,
											rsrvd_tmps,
											alarmed_tmps,
											((tmp->locked == 1) ? "yes" : "no"),
											tmp->exten,
											GetCardName(tmp->board),
											"FXO");
									break;
								case DG_FX_INTERFACE:
									sprintf(temptextchannels, "{\"c\":\"%s\",\"g\":\"%s\",\"d\":\"%s\",\"p\":\"%s\",\"r\":\"%s\",\"a\":\"%s\",\"k\":\"%s\",\"e\":\"%s\",\"y\":\"%s\",\"i\":\"%s\"},",
											tmps,
											grp_tmps,
											tmp->context,
											tmp->szPortID,
											rsrvd_tmps,
											alarmed_tmps,
											((tmp->locked == 1) ? "yes" : "no"),
											tmp->exten,
											GetCardName(tmp->board),
											"FXO/FXS");
									break;
								case DG_GSM_INTERFACE:
									sprintf(temptextchannels, "{\"c\":\"%s\",\"g\":\"%s\",\"d\":\"%s\",\"p\":\"%s\",\"r\":\"%s\",\"a\":\"%s\",\"k\":\"%s\",\"e\":\"%s\",\"y\":\"%s\",\"i\":\"%s\"},",
											tmps,
											grp_tmps,
											tmp->context,
											tmp->szPortID,
											rsrvd_tmps,
											((tmp->alarmed < 2) ? alarmed_tmps : (tmp->alarmed == 2) ? "(S) 1" : "(W) 1"),
											((tmp->locked == 1) ? "yes" : "no"),
											tmp->exten,
											GetCardName(tmp->board),
											"GSM");
									break;
							}
						}
						ast_dgv_unlock(&tmp->lock, tmp->port);

						strcat(textchannels, temptextchannels);
						tmp = tmp->next;
					}//while (tmp)
				}
				ast_dgv_unlock(&iflock, 0);

				textchannels[strlen(textchannels) - 1] = '\0';
				strcat(textchannels, "]} \n\n");
				ast_cli(fd, "%s", textchannels);

				return RESULT_SUCCESS;
			}
	    	else
			{
				//group selected
				group = atoi(argv[3]);
			}
	}

	ast_cli(fd, "List of available DigiVoice Channels");
	
	if (group!=0)
		ast_cli(fd, " of group %d\n",group);
	else
		ast_cli(fd, "\n");

	ast_dgv_lock(&iflock, 0);
	{
		start = iffirst;
		tmp = start;		
		while (tmp) 
		{
			if (lin>=20)
			{
				ast_cli(fd, "\n");
				ast_cli(fd, FORMAT_SHOW_CHANNELS, "Chan", "Grp", "Context", "PortId", "Rsrvd", "Alrmd", "Lckd", "Extension", "CardType", "Intrf.");
				ast_cli(fd, "\n");
				lin=0;
			}
			
			if (tmp->port > 0) {
				snprintf(tmps, sizeof(tmps), "%d", tmp->port);
			} else
				ast_copy_string(tmps, "pseudo", sizeof(tmps));
	
			snprintf(grp_tmps, sizeof(grp_tmps), "%d", tmp->group);
			
			snprintf(rsrvd_tmps, sizeof(rsrvd_tmps), "%d", tmp->reserved);
			
			snprintf(alarmed_tmps, sizeof(alarmed_tmps), "%d", tmp->alarmed);

			if (group==0)
				show=1;
			else
				if (group == tmp->group)
					show=1;
				else
					show=0;
	
			if (show) 
			{
				switch (dg_GetCardInterface(tmp->board))
				{ 
					case DG_DIGITAL_INTERFACE:
						ast_cli(fd, FORMAT_SHOW_CHANNELS,
								tmps,
								grp_tmps,
								tmp->context,
								tmp->szPortID,
								rsrvd_tmps,
								alarmed_tmps,
								((tmp->locked == 1) ? "yes" : "no"),
								tmp->exten,
								GetCardName(tmp->board),
								"R2/ISDN");
					 	break;				            					
					case DG_FXO_INTERFACE:
						ast_cli(fd, FORMAT_SHOW_CHANNELS,
								tmps,
								grp_tmps,
								tmp->context,
								tmp->szPortID,
								rsrvd_tmps,
								alarmed_tmps,
								((tmp->locked == 1) ? "yes" : "no"),
								tmp->exten,
								GetCardName(tmp->board),
								"FXO");
					    break;
					case DG_FX_INTERFACE:
						ast_cli(fd, FORMAT_SHOW_CHANNELS,
								tmps,
								grp_tmps,
								tmp->context,
								tmp->szPortID,
								rsrvd_tmps,
								alarmed_tmps,
								((tmp->locked == 1) ? "yes" : "no"),
								tmp->exten,
								GetCardName(tmp->board),
								"FXO/FXS");
					    break;
					case DG_GSM_INTERFACE:
						ast_cli(fd, FORMAT_SHOW_CHANNELS,
								tmps,
								grp_tmps,
								tmp->context,
								tmp->szPortID,
								rsrvd_tmps,
								((tmp->alarmed < 2) ? alarmed_tmps : (tmp->alarmed == 2) ? "(S) 1" : "(W) 1"),
								((tmp->locked == 1) ? "yes" : "no"),
								tmp->exten,
								GetCardName(tmp->board),
								"GSM");
					    break;
				}
				lin++;	
			}
	
			tmp = tmp->next;
		}//while (tmp)	
	}
	ast_dgv_unlock(&iflock, 0);
	
	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_cards(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_cards(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	#define FORMAT_SHOW_CARDS "%5s %8s %-12.12s %-20.20s %-12.12s %-13.13s %-9.9s %5s %5s %4s %4s\n"

	struct dgv_pvt *tmp = NULL;
	struct dgv_pvt *tmp_ports = NULL;
	short c, p;
	int nForText;
	char cardnum_tmps[20] = "";
	char portscnt_tmps[20] = "";
	char serial_tmps[20] = "";
	char irq_tmps[20] = "";
	char rsrvd_tmps[20] = "";
	char alrmd_tmps[20] = "";
	char lckd_tmps[20] = "";
	char free_tmps[20] = "";
	char textcards[30000] = "";
	char temptextcards[1000] = "";

	int total_rsrvd = 0;
	int total_alrmd = 0;
	int total_lckd = 0;
	int total_free = 0;

	if (argc == 4)
	{
	    if (!strncmp(argv[3], "all", 3))
		{
			ast_cli(fd, "\n<");

			for (nForText = 0; nForText <= sizeof(textcards) - 1; nForText++)
				textcards[nForText] = '\0';

			sprintf(textcards, "CARDS;CARDS_COUNT:%d;", dg_GetCardsCount());

			for (c = 1; c <= dg_GetCardsCount(); c++)
			{
				tmp = find_ch_port(dg_GetAbsolutePortNumber(c, 1));
				if (tmp)
				{
					if (tmp->serial_num)
						snprintf(serial_tmps, sizeof(serial_tmps), "%05d", tmp->serial_num);
					else
						snprintf(serial_tmps, sizeof(serial_tmps), "%s", "Unknown");

					if (tmp->irq_num)
						snprintf(irq_tmps, sizeof(irq_tmps), "%d", tmp->irq_num);
					else
						snprintf(irq_tmps, sizeof(irq_tmps), "%s", "Unknown");

					total_rsrvd = 0;
					total_alrmd = 0;
					total_lckd = 0;
					total_free = 0;

					for (p = 1; p <= dg_GetCardPortsCount(c); p++)
					{
						tmp_ports = find_ch_port(dg_GetAbsolutePortNumber(c, p));
						if (tmp_ports)
						{
							if ((tmp_ports->reserved) || (tmp_ports->alarmed) || (tmp_ports->locked))
							{
								if (tmp_ports->reserved)
									total_rsrvd++;

								if (tmp_ports->alarmed)
									total_alrmd++;

								if (tmp_ports->locked)
									total_lckd++;
							}
							else
								total_free++;
						}
						else
							ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
					} //for (c = 1; c <= dg_GetCardsCount(); c++)

					switch (dg_GetCardInterface(c))
					{
						case DG_DIGITAL_INTERFACE:
							sprintf(temptextcards, "%d:%d,%s,%s,%s,%s,%s,%d,%d,%d,%d;", c, dg_GetCardPortsCount(c), serial_tmps, "DG_DIGITAL_INTERFACE", GetCardName(tmp->board), "R2/ISDN", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
							break;
						case DG_FXO_INTERFACE:
							sprintf(temptextcards, "%d:%d,%s,%s,%s,%s,%s,%d,%d,%d,%d;", c, dg_GetCardPortsCount(c), serial_tmps, "DG_FXO_INTERFACE", GetCardName(tmp->board), "FXO", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
							break;
						case DG_FX_INTERFACE:
							sprintf(temptextcards, "%d:%d,%s,%s,%s,%s,%s,%d,%d,%d,%d;", c, dg_GetCardPortsCount(c), serial_tmps, "DG_FX_INTERFACE", GetCardName(tmp->board), "FXO/FXS", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
							break;
						case DG_GSM_INTERFACE:
							sprintf(temptextcards, "%d:%d,%s,%s,%s,%s,%s,%d,%d,%d,%d;", c, dg_GetCardPortsCount(c), serial_tmps, "DG_GSM_INTERFACE", GetCardName(tmp->board), "GSM", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
							break;
					}
					strcat(textcards, temptextcards);
				}
				else
					ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
			} //for (c = 1; c <= dg_GetCardsCount(); c++)

			textcards[strlen(textcards) - 1] = '\0';
			strcat(textcards, "> \n\n");
			ast_cli(fd, "%s", textcards);

			return RESULT_SUCCESS;
		}
		else
			if (!strncmp(argv[3], "json", 4))
			{
				ast_cli(fd, "\n");

				for (nForText = 0; nForText <= sizeof(textcards) - 1; nForText++)
					textcards[nForText] = '\0';

				ast_copy_string(textcards,
						        "{\"parameters\":{\"c\":\"card\",\"t\":\"channels\",\"s\":\"serialnumber\",\"i\":\"cardinterface\",\"y\":\"cardtype\",\"g\":\"siginterface\",\"n\":\"irqnumber\",\"r\":\"reserved\",\"a\":\"alarmed\",\"l\":\"locked\",\"f\":\"free\"},\"values\":[",
						        sizeof(textcards));

				for (c = 1; c <= dg_GetCardsCount(); c++)
				{
					tmp = find_ch_port(dg_GetAbsolutePortNumber(c, 1));
					if (tmp)
					{
						if (tmp->serial_num)
							snprintf(serial_tmps, sizeof(serial_tmps), "%05d", tmp->serial_num);
						else
							snprintf(serial_tmps, sizeof(serial_tmps), "%s", "Unknown");

						if (tmp->irq_num)
							snprintf(irq_tmps, sizeof(irq_tmps), "%d", tmp->irq_num);
						else
							snprintf(irq_tmps, sizeof(irq_tmps), "%s", "Unknown");

						total_rsrvd = 0;
						total_alrmd = 0;
						total_lckd = 0;
						total_free = 0;

						for (p = 1; p <= dg_GetCardPortsCount(c); p++)
						{
							tmp_ports = find_ch_port(dg_GetAbsolutePortNumber(c, p));
							if (tmp_ports)
							{
								if ((tmp_ports->reserved) || (tmp_ports->alarmed) || (tmp_ports->locked))
								{
									if (tmp_ports->reserved)
										total_rsrvd++;

									if (tmp_ports->alarmed)
										total_alrmd++;

									if (tmp_ports->locked)
										total_lckd++;
								}
								else
									total_free++;
							}
							else
								ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
						} //for (c = 1; c <= dg_GetCardsCount(); c++)

						switch (dg_GetCardInterface(c))
						{
							case DG_DIGITAL_INTERFACE:
								sprintf(temptextcards, "{\"c\":\"%d\",\"t\":\"%d\",\"s\":\"%s\",\"i\":\"%s\",\"y\":\"%s\",\"g\":\"%s\",\"n\":\"%s\",\"r\":\"%d\",\"a\":\"%d\",\"l\":\"%d\",\"f\":\"%d\"},", c, dg_GetCardPortsCount(c), serial_tmps, "DG_DIGITAL_INTERFACE", GetCardName(tmp->board), "R2/ISDN", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
								break;
							case DG_FXO_INTERFACE:
								sprintf(temptextcards, "{\"c\":\"%d\",\"t\":\"%d\",\"s\":\"%s\",\"i\":\"%s\",\"y\":\"%s\",\"g\":\"%s\",\"n\":\"%s\",\"r\":\"%d\",\"a\":\"%d\",\"l\":\"%d\",\"f\":\"%d\"},", c, dg_GetCardPortsCount(c), serial_tmps, "DG_FXO_INTERFACE", GetCardName(tmp->board), "FXO", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
								break;
							case DG_FX_INTERFACE:
								sprintf(temptextcards, "{\"c\":\"%d\",\"t\":\"%d\",\"s\":\"%s\",\"i\":\"%s\",\"y\":\"%s\",\"g\":\"%s\",\"n\":\"%s\",\"r\":\"%d\",\"a\":\"%d\",\"l\":\"%d\",\"f\":\"%d\"},", c, dg_GetCardPortsCount(c), serial_tmps, "DG_FX_INTERFACE", GetCardName(tmp->board), "FXO/FXS", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
								break;
							case DG_GSM_INTERFACE:
								sprintf(temptextcards, "{\"c\":\"%d\",\"t\":\"%d\",\"s\":\"%s\",\"i\":\"%s\",\"y\":\"%s\",\"g\":\"%s\",\"n\":\"%s\",\"r\":\"%d\",\"a\":\"%d\",\"l\":\"%d\",\"f\":\"%d\"},", c, dg_GetCardPortsCount(c), serial_tmps, "DG_GSM_INTERFACE", GetCardName(tmp->board), "GSM", irq_tmps, total_rsrvd, total_alrmd, total_lckd, total_free);
								break;
						}
						strcat(textcards, temptextcards);
					}
					else
						ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
				} //for (c = 1; c <= dg_GetCardsCount(); c++)

				textcards[strlen(textcards) - 1] = '\0';
				strcat(textcards, "]} \n\n");
				ast_cli(fd, "%s", textcards);

				return RESULT_SUCCESS;
			}
	}

	ast_cli(fd, "List of available DigiVoice Cards\n");

	ast_cli(fd, "\n");
	ast_cli(fd, FORMAT_SHOW_CARDS, "Card", "Channels", "SerialNumber", "CardInterface", "CardType", "Sig.Interface", "IRQNumber", "Rsrvd", "Alrmd", "Lckd", "Free");
	ast_cli(fd, "\n");

	for (c = 1; c <= dg_GetCardsCount(); c++)
	{
		tmp = find_ch_port(dg_GetAbsolutePortNumber(c, 1));
		if (tmp)
		{
			snprintf(cardnum_tmps, sizeof(cardnum_tmps), "%d", c);
			snprintf(portscnt_tmps, sizeof(portscnt_tmps), "%d", dg_GetCardPortsCount(c));

			if (tmp->serial_num)
				snprintf(serial_tmps, sizeof(serial_tmps), "%05d", tmp->serial_num);
			else
				snprintf(serial_tmps, sizeof(serial_tmps), "%s", "Unknown");

			if (tmp->irq_num)
				snprintf(irq_tmps, sizeof(irq_tmps), "%d", tmp->irq_num);
			else
				snprintf(irq_tmps, sizeof(irq_tmps), "%s", "Unknown");

			total_rsrvd = 0;
			total_alrmd = 0;
			total_lckd = 0;
			total_free = 0;

			for (p = 1; p <= dg_GetCardPortsCount(c); p++)
			{
				tmp_ports = find_ch_port(dg_GetAbsolutePortNumber(c, p));
				if (tmp_ports)
				{
					if ((tmp_ports->reserved) || (tmp_ports->alarmed) || (tmp_ports->locked))
					{
						if (tmp_ports->reserved)
							total_rsrvd++;

						if (tmp_ports->alarmed)
							total_alrmd++;

						if (tmp_ports->locked)
							total_lckd++;
					}
					else
						total_free++;
				}
				else
					ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
			} //for (c = 1; c <= dg_GetCardsCount(); c++)

			snprintf(rsrvd_tmps, sizeof(rsrvd_tmps), "%d", total_rsrvd);
			snprintf(alrmd_tmps, sizeof(alrmd_tmps), "%d", total_alrmd);
			snprintf(lckd_tmps, sizeof(lckd_tmps), "%d", total_lckd);
			snprintf(free_tmps, sizeof(free_tmps), "%d", total_free);

			switch (dg_GetCardInterface(c))
			{
				case DG_DIGITAL_INTERFACE:
					ast_cli(fd, FORMAT_SHOW_CARDS,
							cardnum_tmps,
							portscnt_tmps,
							serial_tmps,
							"DG_DIGITAL_INTERFACE",
							GetCardName(tmp->board),
							"R2/ISDN",
							irq_tmps,
							rsrvd_tmps,
							alrmd_tmps,
							lckd_tmps,
							free_tmps
					);
					break;
				case DG_FXO_INTERFACE:
					ast_cli(fd, FORMAT_SHOW_CARDS,
							cardnum_tmps,
							portscnt_tmps,
							serial_tmps,
							"DG_FXO_INTERFACE",
							GetCardName(tmp->board),
							"FXO",
							irq_tmps,
							rsrvd_tmps,
							alrmd_tmps,
							lckd_tmps,
							free_tmps
					);
					break;
				case DG_FX_INTERFACE:
					ast_cli(fd, FORMAT_SHOW_CARDS,
							cardnum_tmps,
							portscnt_tmps,
							serial_tmps,
							"DG_FX_INTERFACE",
							GetCardName(tmp->board),
							"FXO/FXS",
							irq_tmps,
							rsrvd_tmps,
							alrmd_tmps,
							lckd_tmps,
							free_tmps
					);
					break;
				case DG_GSM_INTERFACE:
					ast_cli(fd, FORMAT_SHOW_CARDS,
							cardnum_tmps,
							portscnt_tmps,
							serial_tmps,
							"DG_GSM_INTERFACE",
							GetCardName(tmp->board),
							"GSM",
							irq_tmps,
							rsrvd_tmps,
							alrmd_tmps,
							lckd_tmps,
							free_tmps
					);
					break;
			}
		}
		else
			ast_cli(fd, "Cannot show informations for card %d when tmp is null\n", c);
	} //for (c = 1; c <= dg_GetCardsCount(); c++)

	ast_cli(fd, "\n");

	return RESULT_SUCCESS;
}

/*
//---- reset all channels -  it's a kind of reload with recreation of all control threads
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_reset(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_reset(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short ch, ret, ch_board, nForCard;
	int res_sched;
	struct dgv_pvt *p;	

	if (argc != 2)
		return RESULT_SHOWUSAGE;

	channel_up = 0;

	for (ch=1; ch<=dg_GetPortsCount();ch++)
	{
		p = find_ch_port(ch);
		
		if (p)
		{
			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->ast_ch!=NULL)
					ast_softhangup(p->ast_ch, AST_SOFTHANGUP_APPUNLOAD);
				else
				{
					if ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
						ast_log(LOG_WARNING, "dgv_reset: p->ast_ch does not exist on port %d\n", p->port);
				}
			}
			ast_dgv_unlock(&p->lock, p->port);
			
            p->reserved=0;            
            p->sms_reserved=0;
            p->sms_send_to[0] = '\0';
            p->sms_send_messagereference = -1;
            p->sms_send_custom_id[0] = '\0';
            p->sms_sent_status = 0;
            p->alarmed=0;            
            p->vconf_established=0;
            p->fax_established=0;

            if (p->alarmNDT_timeout_sched > -1) 
			{
				res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
		    	if (res_sched == -1)
		    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

				p->alarmNDT_timeout_sched = -1;
			}
			
    		if (p->reserved_timeout_sched > -1)//stop reserved timeout
    		{
    			res_sched = ast_sched_del(sched, p->reserved_timeout_sched);
    	    	if (res_sched == -1)
    	    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

    			p->reserved_timeout_sched = -1;
    		}

    		if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
    		{
    			res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
    	    	if (res_sched == -1)
    	    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

    			p->gsmerror_timeout_sched = -1;
    		}

	        switch (p->sig)
			{
	            case DG_SIG_R2_NDIS_FIXED:
	            case DG_SIG_R2_NDIS_VARIABLE:
	    			ret=dg_DestroyE1Thread(ch);
	    			if ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
	    				ast_log(LOG_NOTICE, "Destroying E1 thread for port %d - cod %x\n",ch,ret);		
	    			break;
	            case DG_SIG_PRI:
	                break;
				case DG_SIG_CUSTOMCAS:
	    			ret=dg_DestroyCustomCAS(ch);
	    			if ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
	    				ast_log(LOG_NOTICE, "Destroying Custom CAS thread for port %d - cod %x\n",ch,ret);
	    				
					break;
			    case DG_SIG_FXO:
			    	//dg_IdleAbort(p->port);
			        break;
			    case DG_SIG_CB_FXO:
					//dg_IdleAbort(p->port);
			    case DG_SIG_CB_FXS:
	    			ret=dg_DestroyCBThread(ch);
	    			if ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
	    				ast_log(LOG_NOTICE, "Destroying CB FXO/FXS thread for port %d - cod %x\n",ch,ret);		
	                break;
				case DG_SIG_GSM:
			        ret=dg_DestroyGSMThread(ch);
			        if ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
			            ast_log(LOG_NOTICE, "Destroying GSM thread for port %d - cod %x\n",ch,ret);
			        break;
			} //switch (p->sig)

	        if ((p->vconf_pass_through) || (p->fax_pass_through))
			{
				ch_board = dg_GetRelativeChannelNumber(p->port);

				if ((ch_board > 0) && (ch_board < 31))
				{
					ret=dg_LocalBridgeDisconnect(p->port, p->vconf_port_bridged);
					if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0))
						ast_log(LOG_NOTICE, "Disconnecting vconf for port %d and port %d - cod %x\n", p->port, p->vconf_port_bridged, ret);
				}
			}
		}

		//calls all destructor functions
		dg_DisableInputBuffer(ch);
		dg_StopPlayBuffer(ch);
		dg_DestroyCallProgress(ch);
	}
	
	//sets alarmmode for each card
	for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
	{
		if (dg_GetCardInterface(nForCard) == DG_DIGITAL_INTERFACE)
			dg_SetAlarmMode(nForCard, ALARM_MANUAL_NOTIFY);
	}

#ifdef USE_LIBPRI
	    if (dchans_count>0) {
    	    thd_socket_running = 0;
        	thd_pri_running = 0;

        	for (ret=0; ret<dchans_count; ret++) {
	            //closes ccs device drivers
            	close(dchans[ret].fds[TX]);
            	close(dchans[ret].fds[RX]);
        	}
    	}

		if (thd_sock)
		{
			pthread_cancel(thd_sock);
			pthread_kill(thd_sock, SIGURG);
			pthread_join(thd_sock, NULL);
		}
		thd_sock = AST_PTHREADT_STOP;

		if (thd_pri)
		{
			pthread_cancel(thd_pri);
			pthread_kill(thd_pri, SIGURG);
			pthread_join(thd_pri, NULL);
		}
		thd_pri = AST_PTHREADT_STOP;

		dev_idx = 0;
		dchans_count = 0;
#endif//#ifdef USE_LIBPRI

	setup_dgv(1);

	return RESULT_SUCCESS;
}*/

//---- Reset specific gsm channel
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_reset(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_reset(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short nForChannel, ch;
	int res_sched;
	short ret;
	short nForTryCount;
	short nGSMCount;
	struct dgv_pvt *tmp;

	if (argc != 4)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;
		}
		return RESULT_SHOWUSAGE;
	}

	if (!strncmp(argv[3], "all", 3))
	{
		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "\n       Resetting GSM Control (All Channels)... \n\n");
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Resetando Controle GSM (Todos Canais)... \n\n");
				break;
			default:
				ast_cli(fd, "\n       Resetting GSM Control (All Channels)... \n\n");
		}

		nGSMCount = 0;

		for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
		{
			if (dg_GetPortInterface(nForChannel) == DG_GSM_INTERFACE)
			{
				nGSMCount++;

				tmp = find_ch_port(nForChannel);

				if (tmp)
				{
					ast_dgv_lock(&tmp->lock, tmp->port);
					{
						if (tmp->gsmerror_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->gsmerror_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->gsmerror_timeout_sched = -1;
						}

						if (tmp->gsmgetsignal_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->gsmgetsignal_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->gsmgetsignal_timeout_sched = -1;
						}

						if (tmp->gsmclearsms_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->gsmclearsms_timeout_sched = -1;
						}

						if (tmp->gsmsendsms_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->gsmsendsms_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->gsmsendsms_timeout_sched = -1;
						}

						if (tmp->ring_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->ring_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->ring_timeout_sched = -1;
						}

						tmp->reserved = tmp->sms_reserved = 0;
						tmp->sms_send_to[0] = '\0';
						tmp->sms_send_messagereference = -1;
						tmp->sms_send_custom_id[0] = '\0';
						tmp->sms_sent_status = 0;
						tmp->alarmed = 2;
						ast_copy_string(tmp->gsm_operator_name, "Unknown", sizeof(tmp->gsm_operator_name));
						ast_copy_string(tmp->gsm_simcard_id, "Unknown", sizeof(tmp->gsm_simcard_id));

						ret = dg_DestroyGSMThread(tmp->port);
						if (ret!=DG_EXIT_SUCCESS)
							ast_cli(fd, "\n       Error destroying GSM thread for port %d\n", tmp->port);
						else
							if (option_verbose >= 3)
								ast_cli(fd, "\n       Destroying GSM thread for port %d\n", tmp->port);

						for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)
						{
							sleep(1);

							if (!ast_strlen_zero(tmp->gsm_pin_number))
							{
								dg_GSMSetPinNumber(tmp->port, tmp->gsm_pin_number, NULL);
								ast_cli(fd, "\n       GSM port=%d, PIN=%s\n", tmp->port, tmp->gsm_pin_number);
							}

							ret = dg_CreateGSMThread(tmp->port);
							if (ret==DG_EXIT_SUCCESS)
							{
								ast_cli(fd, "\n       Re-creating GSM thread for port %d\n", tmp->port);
								break;
							}
							else
								if (option_debug==100)
									ast_log(LOG_DEBUG, "Error creating GSM thread for port %d - cod %x - step %d\n", tmp->port, ret, nForTryCount);
						} //for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)

						if (ret!=DG_EXIT_SUCCESS)
							ast_cli(fd, "\n       Error creating GSM thread for port %d - cod %x\n", tmp->port, ret);
					}
					ast_dgv_unlock(&tmp->lock, tmp->port);
				}
				else //if (tmp)
					ast_cli(fd, "\n       Channel %d cannot be reseted\n", nForChannel);

			} //if (dg_GetPortInterface(nForChannel) == DG_GSM_INTERFACE)
		} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)

		if (nGSMCount==0)
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
        			ast_cli(fd, "       Error: No GSM Cards Found. \n");
					break;
				case 1://Portuguese
					ast_cli(fd, "       Erro: Nenhuma Placa GSM encontrada. \n");
					break;
				default:
					ast_cli(fd, "       Error: No GSM Cards Found. \n");
			}
		}
	}
	else //if (!strncmp(argv[3], "all", 3))
	{
		ch = atoi(argv[3]);

		tmp = find_ch_port(ch);

		if (tmp)
		{
			if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Resetting GSM Control (Channel %d)... \n\n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Resetando Controle GSM (Canal %d)... \n\n", tmp->port);
						break;
					default:
						ast_cli(fd, "\n       Resetting GSM Control (Channel %d)... \n\n", tmp->port);
				}

				ast_dgv_lock(&tmp->lock, tmp->port);
				{
					if (tmp->gsmerror_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->gsmerror_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->gsmerror_timeout_sched = -1;
					}

					if (tmp->gsmgetsignal_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->gsmgetsignal_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->gsmgetsignal_timeout_sched = -1;
					}

					if (tmp->gsmclearsms_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->gsmclearsms_timeout_sched = -1;
					}

					if (tmp->gsmsendsms_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->gsmsendsms_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->gsmsendsms_timeout_sched = -1;
					}

					if (tmp->ring_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->ring_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->ring_timeout_sched = -1;
					}

					tmp->reserved = tmp->sms_reserved = 0;
					tmp->sms_send_to[0] = '\0';
					tmp->sms_send_messagereference = -1;
					tmp->sms_send_custom_id[0] = '\0';
					tmp->sms_sent_status = 0;
					tmp->alarmed = 2;
					ast_copy_string(tmp->gsm_operator_name, "Unknown", sizeof(tmp->gsm_operator_name));
					ast_copy_string(tmp->gsm_simcard_id, "Unknown", sizeof(tmp->gsm_simcard_id));

					ret = dg_DestroyGSMThread(tmp->port);
					if (ret!=DG_EXIT_SUCCESS)
						ast_cli(fd, "\n       Error destroying GSM thread for port %d\n", tmp->port);
					else
						if (option_verbose >= 3)
							ast_cli(fd, "\n       Destroying GSM thread for port %d\n", tmp->port);

					for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)
					{
						sleep(1);

						if (!ast_strlen_zero(tmp->gsm_pin_number))
						{
							dg_GSMSetPinNumber(tmp->port, tmp->gsm_pin_number, NULL);
							ast_cli(fd, "\n       GSM port=%d, PIN=%s\n", tmp->port, tmp->gsm_pin_number);
						}

						ret = dg_CreateGSMThread(tmp->port);
						if (ret==DG_EXIT_SUCCESS)
						{
							ast_cli(fd, "\n       Re-creating GSM thread for port %d\n", tmp->port);
							break;
						}
						else
							if (option_debug==100)
								ast_log(LOG_DEBUG, "Error creating GSM thread for port %d - cod %x - step %d\n", tmp->port, ret, nForTryCount);
					} //for (nForTryCount = 1; nForTryCount <= 60; nForTryCount++)

					if (ret!=DG_EXIT_SUCCESS)
						ast_cli(fd, "\n       Error creating GSM thread for port %d - cod %x\n", tmp->port, ret);
				}
				ast_dgv_unlock(&tmp->lock, tmp->port);
			}
			else //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
			{
				ast_cli(fd, "\n");

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "       Error: Channel %d isn't a GSM Channel... \n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "       Erro: Canal %d nao eh um Canal GSM... \n", tmp->port);
						break;
					default:
						ast_cli(fd, "       Error: Channel %d isn't a GSM Channel... \n", tmp->port);
				}
			} //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
		}
		else //if (tmp)
			ast_cli(fd, "\n       Channel %d cannot be reseted\n", ch);

	} //if (!strncmp(argv[3], "all", 3))

	return RESULT_SUCCESS;
}

//---- Shows channel version
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_version(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_version(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
    int p;
    int nForText;
    char tmpVersionInfo[100];
    char tmpVersionInfoNum[100];
    struct dgv_pvt *tmp;
	int tmpFirmware_vr_board = -1;
	int tmpLec_count_board = -1;

	for (nForText = 0; nForText <= sizeof(tmpVersionInfo) - 1; nForText++)
	{
		tmpVersionInfo[nForText] = '\0';
		tmpVersionInfoNum[nForText] = '\0';
	}

#ifdef DEBUG_LOCKS
    	ast_cli(fd, "\n Channel Driver Version: %s, DEBUG_LOCKS: Enabled \n", CHANDGV_VERSION);
#else//#ifdef DEBUG_LOCKS
	#ifdef DEBUG_LAST_LOCK
    		ast_cli(fd, "\n Channel Driver Version: %s, DEBUG_LAST_LOCK: Enabled \n", CHANDGV_VERSION);
	#else//#ifdef DEBUG_LAST_LOCK
    		ast_cli(fd, "\n Channel Driver Version: %s \n", CHANDGV_VERSION);
	#endif//#ifdef DEBUG_LAST_LOCK    		
#endif//#ifdef DEBUG_LOCKS    
    
#ifdef VOICERLIB_VERSIONNUMBER
    if (dg_GetLibVersion(tmpVersionInfoNum, 1) == DG_EXIT_SUCCESS)
    {
    	if (dg_GetLibVersion(tmpVersionInfo, 0) == DG_EXIT_SUCCESS)
    		ast_cli(fd, " VoicerLib Version.....: %s (%s) \n", tmpVersionInfo, tmpVersionInfoNum);
    	else
    		ast_cli(fd, " VoicerLib Version.....: %s \n", tmpVersionInfoNum);
    }
#endif//#ifdef VOICERLIB_VERSIONNUMBER

    ast_cli(fd, " Firmware Versions.....: \n");

    for (p=1;p<=dg_GetCardsCount();p++) 
    {
    	if (dg_GetCardType(p) == DGV_2E1F || dg_GetCardType(p) == DGV_1E1F)
    	{
			tmp = find_ch_port(dg_GetAbsolutePortNumber(p, 1));
			if (tmp)
			{
				tmpFirmware_vr_board = tmp->firmware_vr_board;
				tmpLec_count_board = tmp->lec_count_board;
			}

    		ast_cli(fd, "    |_ DigiVoice Card %d: %xh (LEC %d)\n", p, tmpFirmware_vr_board, tmpLec_count_board);
    	}
    	else
    		ast_cli(fd, "    |_ DigiVoice Card %d: %xh\n", p, dg_GetVersion(p));
    }
    
    ast_cli(fd, "\n");    
    
    return RESULT_SUCCESS;
}

//---- Shows specific channel information
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	#define FORMAT_SMS_CHANNEL "%11s %11s %09s"

	struct dgv_pvt *tmp;
	int channel;

	struct dgv_pvt *tmp_smschannel = NULL;

    if (argc != 4)
        return RESULT_SHOWUSAGE;

	channel = atoi(argv[3]);

	tmp = find_ch_port(channel);
	
	if (tmp!=NULL)
	{
		ast_cli(fd, "\n");
		
		ast_cli(fd, " Channel: %d\n", tmp->port);
		ast_cli(fd, " Group: %d\n", tmp->group);
		ast_cli(fd, " Card Number: %d\n", tmp->board);

		if (tmp->serial_num)
			ast_cli(fd, " Card Serial Number: %05d\n", tmp->serial_num);
		else
			ast_cli(fd, " Card Serial Number: %s\n", "Unknown");

		if (tmp->irq_num)
			ast_cli(fd, " Card IRQ Number: %d\n", tmp->irq_num);
		else
			ast_cli(fd, " Card IRQ Number: %s\n", "Unknown");

		ast_cli(fd, " Card Model: %s\n", GetCardName(tmp->board));
		switch (dg_GetCardInterface(tmp->board))
		{
			case DG_DIGITAL_INTERFACE:
				ast_cli(fd, " Interface Type: R2/ISDN\n");
				break;
			case DG_FXO_INTERFACE:
				ast_cli(fd, " Interface Type: FXO\n");
				break;
			case DG_FX_INTERFACE:
				ast_cli(fd, " Interface Type: FXO/FXS\n");
				break;
			case DG_GSM_INTERFACE:
				ast_cli(fd, " Interface Type: GSM\n");
				break;
		}			
		
		ast_cli(fd, " Extension: %s\n", tmp->exten);
		ast_dgv_lock(&tmp->lock, tmp->port);
		{
			ast_cli(fd, " Owner: %s\n", (tmp->ast_ch) ? dgv_channel_name(tmp->ast_ch) : "<None>");

#ifdef CC_AST_HAS_VERSION_12
				RAII_VAR(struct ast_channel *, dgv_bridge_ch, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
				struct ast_channel *dgv_bridge_ch = NULL;
#endif//#ifdef CC_AST_HAS_VERSION_12
			if (tmp->ast_ch)
				dgv_bridge_ch = dgv_bridged_channel(tmp->ast_ch);
			ast_cli(fd, " Bridge: %s\n", (dgv_bridge_ch) ? dgv_channel_name(dgv_bridge_ch) : "<None>");
		}
		ast_dgv_unlock(&tmp->lock, tmp->port);
		ast_cli(fd, " Reserved: %d\n", tmp->reserved);
		
		switch (tmp->alarmed)
		{
			case 0:
			case 1:
				ast_cli(fd, " Alarmed: %d\n", tmp->alarmed);
				break;
			case 2:
				ast_cli(fd, " Alarmed: %d (Starting)\n", 1);
				break;
			case 99:
				ast_cli(fd, " Alarmed: %d (Waiting)\n", 1);
				break;
		}

		ast_cli(fd, " Locked: %s\n", tmp->locked ? "yes":"no");

		if (tmp->sig == DG_SIG_R2_NDIS_FIXED || tmp->sig == DG_SIG_R2_NDIS_VARIABLE)
			ast_cli(fd, " Last R2 Signal Received: %x\n", tmp->last_r2_signal);

#ifdef USE_LIBPRI
            if (tmp->sig == DG_SIG_PRI)
                ast_cli(fd, " ISDN Outgoing Call: %s\n", tmp->outgoing ? "yes":"no");
#endif//#ifdef USE_LIBPRI

#ifdef CC_AST_HAS_VERSION_13
            ast_cli(fd, " Current RX Codec: %s(%lld)\n", ast2dgvformatname(tmp->read_last_format), tmp->read_last_format);
            ast_cli(fd, " Current TX Codec: %s(%lld)\n", ast2dgvformatname(tmp->write_last_format), tmp->write_last_format);
#else//#ifdef CC_AST_HAS_VERSION_13
        	ast_cli(fd, " Current RX Codec: %s(%d)\n", ast2dgvformatname(tmp->read_last_format), tmp->read_last_format);
        	ast_cli(fd, " Current TX Codec: %s(%d)\n", ast2dgvformatname(tmp->write_last_format), tmp->write_last_format);
#endif//#ifdef CC_AST_HAS_VERSION_13

		ast_cli(fd, " Context: %s\n", tmp->context);
        ast_cli(fd, " Language: %s\n", tmp->language);
		ast_cli(fd, " Account Code: %s\n", (strlen(tmp->accountcode)>0)  ?  tmp->accountcode :  "<None>");
        ast_cli(fd, " MusicOnHold Class: %s\n", (ast_strlen_zero(tmp->musicclass)) ? "<default>" : tmp->musicclass);

		char sCodecs[AST_MAX_EXTENSION] = "";
		sCodecs[0] = '\0';
        if (dgv_conf.allow_slinear)        	
        	strcat(sCodecs, "SLINEAR, ");
        if (dgv_conf.allow_ulaw)        
        	strcat(sCodecs, "U_LAW, ");        
        if (dgv_conf.allow_alaw)
        	strcat(sCodecs, "A_LAW, ");            	
        if (dgv_conf.allow_gsm)            	
        	strcat(sCodecs, "GSM, ");
        sCodecs[strlen(sCodecs) - 2] = '\0';
        ast_cli(fd, " Selected Codecs: %s\n", sCodecs);
        
		ast_cli(fd, " TX Gain: %d\n", tmp->tx_gain);
		ast_cli(fd, " RX Gain: %d\n", tmp->rx_gain);
		ast_cli(fd, " Using Echo Cancellation: %s\n", tmp->echo_can ? "yes":"no");

		switch (dg_GetCardInterface(tmp->board))
		{
			case DG_DIGITAL_INTERFACE:
				if (dg_GetPortCardType(tmp->port) == DGV_2E1F || dg_GetPortCardType(tmp->port) == DGV_1E1F)
				{
					if ((tmp->echo_type == ECHO_HW_DSP) && (tmp->echo_can>0 && tmp->echo_can<7))
					{
						ast_cli(fd, "    |_ Using Echo Cancellation Type = %s\n", echo_type_desc[tmp->echo_type]);
						ast_cli(fd, "    |_ Using Echo Cancellation Taps = %s\n", echo_taps_desc[tmp->echo_can]);
					}
				}
				else
				{
					if ((tmp->echo_type == ECHO_HW_DSP) && (tmp->echo_can>0 && tmp->echo_can<6))
					{
						ast_cli(fd, "    |_ Using Echo Cancellation Type = %s\n", echo_type_desc[tmp->echo_type]);
						ast_cli(fd, "    |_ Using Echo Cancellation Taps = %s\n", echo_taps_desc[tmp->echo_can]);
						ast_cli(fd, "    |_ Using Echo Cancellation Training After %d ms\n", tmp->echo_training);
					}
					else
					{
						if (((tmp->echo_type == ECHO_SW_LEC) || (tmp->echo_type == ECHO_SW_LEC_U)) && (tmp->echo_can>0 && tmp->echo_can<7))
						{
							ast_cli(fd, "    |_ Using Echo Cancellation Type = %s\n", echo_type_desc[tmp->echo_type]);
							ast_cli(fd, "    |_ Using Echo Cancellation Taps = %s\n", echo_taps_desc[tmp->echo_can]);
							//ast_cli(fd, "    |_ Using Echo Cancellation NLP = %s\n", echo_nlp_desc[tmp->echo_nlp]);
						}
					}
				}
				break;
			case DG_FXO_INTERFACE:
			case DG_FX_INTERFACE:
			case DG_GSM_INTERFACE:
				if ((tmp->echo_type == ECHO_HW_DSP) && (tmp->echo_can>0 && tmp->echo_can<6))
				{
					ast_cli(fd, "    |_ Using Echo Cancellation Type = %s\n", echo_type_desc[tmp->echo_type]);
					ast_cli(fd, "    |_ Using Echo Cancellation Taps = %s\n", echo_taps_desc[tmp->echo_can]);
					ast_cli(fd, "    |_ Using Echo Cancellation NLP = %s\n", echo_nlp_desc[tmp->echo_nlp]);
				}
				else
				{
					if (((tmp->echo_type == ECHO_SW_LEC) || (tmp->echo_type == ECHO_SW_LEC_U)) && (tmp->echo_can>0 && tmp->echo_can<7))
					{
						ast_cli(fd, "    |_ Using Echo Cancellation Type = %s\n", echo_type_desc[tmp->echo_type]);
						ast_cli(fd, "    |_ Using Echo Cancellation Taps = %s\n", echo_taps_desc[tmp->echo_can]);
						//ast_cli(fd, "    |_ Using Echo Cancellation NLP = %s\n", echo_nlp_desc[tmp->echo_nlp]);
					}
				}
				break;
		}

        ast_cli(fd, " Callprogress Configuration File: %s\n", tmp->callprogress);		
        ast_cli(fd, " Silence Threshold: %d\n", tmp->silence_threshold);
        ast_cli(fd, " Relax DTMF Level: %d\n", (int)tmp->relaxdtmf);
        ast_cli(fd, " Regenerate Detected DTMF: %s\n", tmp->gen_dtmf  ? "yes":"no");
        ast_cli(fd, " Generate Ringback Tone: %s\n", tmp->gen_ringback ? "yes":"no");
        ast_cli(fd, " Generate Busy Tone: %s\n", tmp->gen_busy ? "yes":"no");
        ast_cli(fd, " Detects DTMF and Pass It To Asterisk: %d\n", (int)tmp->dtmf_to_asterisk);
		ast_cli(fd, " Fax Detection: %s\n", tmp->fax_detection ? "yes":"no");
        ast_cli(fd, " Drop TX Samples Factor: %d\n", (int)tmp->drop_tx_samples_factor);

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			if (ast_test_flag(&tmp->jbconf, AST_JB_ENABLED))
			{
				ast_cli(fd, " JitterBuffer Enabled: %s\n", ast_test_flag(&tmp->jbconf, AST_JB_ENABLED) ? "yes":"no");
				ast_cli(fd, "    |_ JitterBuffer Forced: %s\n", ast_test_flag(&tmp->jbconf, AST_JB_FORCED) ? "yes":"no");
				ast_cli(fd, "    |_ JitterBuffer Max Size: %ld\n", tmp->jbconf.max_size);
				ast_cli(fd, "    |_ JitterBuffer Resync: %ld\n", tmp->jbconf.resync_threshold);
				ast_cli(fd, "    |_ JitterBuffer Impl: %s\n", tmp->jbconf.impl);
				ast_cli(fd, "    |_ JitterBuffer Log: %s\n", ast_test_flag(&tmp->jbconf, AST_JB_LOG) ? "yes":"no");
			}
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

		char sVerbosity[100] = "";
        sVerbosity[0] = '\0';
		if ((tmp->verbosity & DG_LOG_START_END_CALL) != 0)
        	strcat(sVerbosity, "10, ");
		if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
        	strcat(sVerbosity, "20, ");
		if ((tmp->verbosity & DG_LOG_ISDN_EVENTS) != 0)
        	strcat(sVerbosity, "30, ");
		if ((tmp->verbosity & DG_LOG_THREADS) != 0)
        	strcat(sVerbosity, "40, ");
		if ((tmp->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	strcat(sVerbosity, "50, ");
		if ((tmp->verbosity & DG_LOG_READ_SAMPLES) != 0)
        	strcat(sVerbosity, "60, ");
		if ((tmp->verbosity & DG_LOG_AST_QUEUE_FRAME) != 0)
        	strcat(sVerbosity, "70, ");
		if ((tmp->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
        	strcat(sVerbosity, "80, ");
		if ((tmp->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
        	strcat(sVerbosity, "90, ");
		if ((tmp->verbosity & DG_LOG_AST_INDICATE) != 0)
        	strcat(sVerbosity, "100, ");
		if ((tmp->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
        	strcat(sVerbosity, "110, ");
		if (strlen(sVerbosity) >= 2)
			sVerbosity[strlen(sVerbosity) - 2] = '\0';
		ast_cli(fd, " Verbosity Level Enabled: %s\n", (strlen(sVerbosity) > 0) ? sVerbosity : "0");

        ast_cli(fd, " Signalling: %s\n", sig2str(tmp->sig));
        switch (tmp->sig)
        {
            case DG_SIG_R2_NDIS_FIXED:
            	//R2 Fixed information
            	switch (tmp->r2_country)
            	{
            		case R2_COUNTRY_BR:
                    	ast_cli(fd, "    |_ R2 Country: Brazil\n");
            			break;
            		case R2_COUNTRY_AR:
                    	ast_cli(fd, "    |_ R2 Country: Argentine\n");
            			break;            			
            		case R2_COUNTRY_MX:
                    	ast_cli(fd, "    |_ R2 Country: Mexico\n");
            			break;
            	} //switch (tmp->r2_country)
                ast_cli(fd, "    |_ Max RX Digits For Incoming Calls: %d\n", tmp->max_digits_rx);
                ast_cli(fd, "    |_ Ask For CID Position: %d\n", tmp->ask_for_cid_position);
                ast_cli(fd, "    |_ Category: %d\n", tmp->category);
                ast_cli(fd, "    |_ Category Pass Through: %s\n", tmp->category_pass_through ? "yes":"no");
                ast_cli(fd, "    |_ Group B Value: %d\n", tmp->group_b_value);
                ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ B Subscriber Hangup: %s\n", tmp->b_subscriber_hangup ? "yes":"no");
                ast_cli(fd, "    |_ Detect CCB and Pass It To PSTN: %s\n", tmp->ccb_pass_through ? "yes":"no");
                ast_cli(fd, "    |_ Auto CCB For Incoming Calls: %s\n", tmp->auto_dgccb ? "yes":"no");
                ast_cli(fd, "    |_ Wait For Dialtone: %s\n", tmp->wait_dialtone ? "yes":"no");
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");
                
				if (tmp->vconf_pass_through)
				{
					ast_cli(fd, "    |_ VConf Pass Through: %s\n", "yes");
					ast_cli(fd, "       |_ VConf Port Bridged: %d\n", tmp->vconf_port_bridged);
				}

				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "       |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "       |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

				if (tmp->category_port_bridged > 0 && tmp->category_port_bridged <= dg_GetPortsCount())
					ast_cli(fd, "    |_ Category Port Bridged: %d\n", tmp->category_port_bridged);
				else
					ast_cli(fd, "    |_ Category Port Bridged: %s\n", "no one is bridged");

				if (tmp->group_b_port_bridged > 0 && tmp->group_b_port_bridged <= dg_GetPortsCount())
					ast_cli(fd, "    |_ Group B Port Bridged: %d\n", tmp->group_b_port_bridged);
				else
					ast_cli(fd, "    |_ Group B Port Bridged: %s\n", "no one is bridged");

            	break;
            case DG_SIG_R2_NDIS_VARIABLE:
            	//R2 Variable information
            	switch (tmp->r2_country)
            	{
            		case R2_COUNTRY_BR:
                    	ast_cli(fd, "    |_ R2 Country: Brazil\n");
            			break;
            		case R2_COUNTRY_AR:
                    	ast_cli(fd, "    |_ R2 Country: Argentine\n");
            			break;            			
            		case R2_COUNTRY_MX:
                    	ast_cli(fd, "    |_ R2 Country: Mexico\n");
            			break;
            	} //switch (tmp->r2_country)
            	ast_cli(fd, "    |_ Min RX Digits: %d\n", tmp->min_digits_rx);
                ast_cli(fd, "    |_ Max RX Digits: %d\n", tmp->max_digits_rx);
                ast_cli(fd, "    |_ Interdigit Timeout: %d\n", tmp->interdigit_timeout);
                ast_cli(fd, "    |_ Ask For CID Position: %d\n", tmp->ask_for_cid_position);
                ast_cli(fd, "    |_ Category: %d\n", tmp->category);
                ast_cli(fd, "    |_ Group B Value: %d\n", tmp->group_b_value);
				ast_cli(fd, "    |_ Wait For Group B Information: %s\n", tmp->wait_group_b ? "yes":"no");
                ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ B Subscriber Hangup: %s\n", tmp->b_subscriber_hangup ? "yes":"no");
                ast_cli(fd, "    |_ Detect CCB and Pass It To PSTN: %s\n", tmp->ccb_pass_through ? "yes":"no");
                ast_cli(fd, "    |_ Auto CCB For Incoming Calls: %s\n", tmp->auto_dgccb ? "yes":"no");
                ast_cli(fd, "    |_ Wait For Dialtone: %s\n", tmp->wait_dialtone ? "yes":"no");
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");

				if (tmp->vconf_pass_through)
				{
					ast_cli(fd, "    |_ VConf Pass Through: %s\n", "yes");
					ast_cli(fd, "       |_ VConf Port Bridged: %d\n", tmp->vconf_port_bridged);
				}

				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "       |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "       |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

				if (tmp->category_port_bridged > 0 && tmp->category_port_bridged <= dg_GetPortsCount())
					ast_cli(fd, "    |_ Category Port Bridged: %d\n", tmp->category_port_bridged);
				else
					ast_cli(fd, "    |_ Category Port Bridged: %s\n", "no one is bridged");

				if (tmp->group_b_port_bridged > 0 && tmp->group_b_port_bridged <= dg_GetPortsCount())
					ast_cli(fd, "    |_ Group B Port Bridged: %d\n", tmp->group_b_port_bridged);
				else
					ast_cli(fd, "    |_ Group B Port Bridged: %s\n", "no one is bridged");

                break;                
            case DG_SIG_CUSTOMCAS:
                //CUSTOMCAS information
                ast_cli(fd, "    |_ Default Custom CAS Configuration File: %s\n", tmp->customcascfg);
                ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ B Subscriber Hangup: %s\n", tmp->b_subscriber_hangup ? "yes":"no");
                ast_cli(fd, "    |_ Wait For Dialtone: %s\n", tmp->wait_dialtone ? "yes":"no");
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");
                
				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "    |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "    |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

                break;
#ifdef USE_LIBPRI
            case DG_SIG_PRI:
            	//ISDN information
            	ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ Immediate: %s\n", tmp->immediate ? "yes":"no");
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");

#ifdef CC_PRI_HAS_REVERSECHARGE
                	ast_cli(fd, "    |_ Detect ISDN Reverse Charging: yes\n");
#else//#ifdef CC_PRI_HAS_REVERSECHARGE
                	ast_cli(fd, "    |_ Detect ISDN Reverse Charging: no\n");
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE                	

				if (tmp->vconf_pass_through)
				{
					ast_cli(fd, "    |_ VConf Pass Through: %s\n", "yes");
					ast_cli(fd, "    |_ VConf Port Bridged: %d\n", tmp->vconf_port_bridged);
				}

				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "    |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "    |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

                break;
#endif//#ifdef USE_LIBPRI
            case DG_SIG_CB_FXS:
            	//ChannelBank FXS Information
                ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ Port Name: %s\n", tmp->szPortName);
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Interdigit Timeout: %d\n", tmp->interdigit_timeout);
                ast_cli(fd, "    |_ MailBox: %s\n", (strlen(tmp->mailbox) > 0) ? tmp->mailbox : "<None>");
                ast_cli(fd, "    |_ Immediate: %s\n", tmp->immediate ? "yes":"no");
                ast_cli(fd, "    |_ Call Group: %s\n", (strlen(tmp->callgroupcfg) > 0) ? tmp->callgroupcfg : "<None>");
                ast_cli(fd, "    |_ Pickup Group: %s\n", (strlen(tmp->pickupgroupcfg) > 0) ? tmp->pickupgroupcfg : "<None>");
                if (tmp->second_call)
                	ast_cli(fd, "    |_ Second Call (%s): %s\n", tmp->second_call_digits, tmp->second_call ? "yes":"no");
                else
                	ast_cli(fd, "    |_ Second Call: %s\n", tmp->second_call ? "yes":"no");

				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "    |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "    |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

            	break;            	
            case DG_SIG_CB_FXO:
            	//ChannelBank FXO Information
            	ast_cli(fd, "    |_ Wait For Dialtone: %s\n", tmp->wait_dialtone ? "yes":"no");
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");
                ast_cli(fd, "    |_ Polarity: %d\n", (int)tmp->polarity);
                ast_cli(fd, "    |_ No DialTone Count: %d\n", dgv_fxoNDTCount[(dg_GetCardNumber(tmp->port)) - 1][(dg_GetRelativeChannelNumber(tmp->port)) - 1]);
            	
				if (tmp->fax_pass_through)
				{
					ast_cli(fd, "    |_ Fax Pass Through: %s\n", "yes");
					if (tmp->fax_established)
						ast_cli(fd, "    |_ Fax Port Bridged: %d\n", tmp->fax_port_bridged);
					else
						ast_cli(fd, "    |_ Fax Port Bridged: %s\n", "no one is bridged");
				}

            	break;
            case DG_SIG_FXO:
                //FXO information
            	ast_cli(fd, "    |_ Auto CCB For Incoming Calls: %s\n", tmp->auto_dgccb ? "yes":"no");
                ast_cli(fd, "    |_ Wait For Dialtone: %s\n", tmp->wait_dialtone ? "yes":"no");
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");
                ast_cli(fd, "    |_ CID Detection Type: %s\n", (tmp->cid_detection_type == 2) ? "FSK":"DTMF");
                ast_cli(fd, "    |_ No DialTone Count: %d\n", dgv_fxoNDTCount[(dg_GetCardNumber(tmp->port)) - 1][(dg_GetRelativeChannelNumber(tmp->port)) - 1]);

                break;
            case DG_SIG_FXS:
            	//FXS information
                ast_cli(fd, "    |_ Port Id: %s\n", tmp->szPortID);
                ast_cli(fd, "    |_ Port Name: %s\n", tmp->szPortName);
                ast_cli(fd, "    |_ DialTone Timeout: %d\n", (int)tmp->dialtone_timeout);
                ast_cli(fd, "    |_ Interdigit Timeout: %d\n", tmp->interdigit_timeout);
                ast_cli(fd, "    |_ Flash Min Time: %d\n", (int)tmp->flash_min_time);
                ast_cli(fd, "    |_ Flash Max Time: %d\n", (int)tmp->flash_max_time);
                ast_cli(fd, "    |_ Ring Type: %d\n", (int)tmp->ring_type);                
                ast_cli(fd, "    |_ MailBox: %s\n", (strlen(tmp->mailbox) > 0) ? tmp->mailbox : "<None>");
                ast_cli(fd, "    |_ Immediate: %s\n", tmp->immediate ? "yes":"no");
                ast_cli(fd, "    |_ Call Group: %s\n", (strlen(tmp->callgroupcfg) > 0) ? tmp->callgroupcfg : "<None>");
                ast_cli(fd, "    |_ Pickup Group: %s\n", (strlen(tmp->pickupgroupcfg) > 0) ? tmp->pickupgroupcfg : "<None>");                
                if (tmp->second_call)
                	ast_cli(fd, "    |_ Second Call (%s): %s\n", tmp->second_call_digits, tmp->second_call ? "yes":"no");
                else
                	ast_cli(fd, "    |_ Second Call: %s\n", tmp->second_call ? "yes":"no");                

                break;
            case DG_SIG_GSM:
                //GSM information
            	ast_cli(fd, "    |_ Operator Name: %s\n", (strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown");
            	ast_cli(fd, "    |_ SIM Card ID: %s\n", (strlen(tmp->gsm_simcard_id) > 1) ? tmp->gsm_simcard_id : "Unknown");
                ast_cli(fd, "    |_ Hide My Number: %s\n", tmp->gsm_hide_number ? "yes":"no");
                ast_cli(fd, "    |_ PIN Number: %s\n", (strlen(tmp->gsm_pin_number) > 0) ? tmp->gsm_pin_number : "<None>");
                ast_cli(fd, "    |_ Prefix Denied: %s\n", (strlen(tmp->deny_prefix) > 0) ? tmp->deny_prefix : "<None>");
                ast_cli(fd, "    |_ GSM Dial Error: %s (Count %d)\n", dgv_gsmDECount[(dg_GetCardNumber(tmp->port)) - 1][(dg_GetRelativeChannelNumber(tmp->port)) - 1].nFlagDialError ? "yes":"no", dgv_gsmDECount[(dg_GetCardNumber(tmp->port)) - 1][(dg_GetRelativeChannelNumber(tmp->port)) - 1].nTotalDialError);
                ast_cli(fd, "    |_ GSM Need Reload: %s\n", tmp->gsm_need_reload ? "yes":"no");
                ast_cli(fd, "    |_ GSM Signal Update Interval: %d ms\n", dgv_conf.gsm_signal_update_interval);
                ast_cli(fd, "    |_ SMS Confirmation: %s\n", tmp->gsm_sms_confirmation ? "yes":"no");
                ast_cli(fd, "    |_ Flash SMS: %s\n", tmp->gsm_sms_flashsms ? "yes":"no");
                if (!tmp->sms_reserved)
                	ast_cli(fd, "    |_ SMS Reserved (Outgoing): %d\n", tmp->sms_reserved);
                else
                	ast_cli(fd, "    |_ SMS Reserved (Outgoing to %s): %d\n", (strlen(tmp->sms_send_to) > 0) ? tmp->sms_send_to : "<None>", tmp->sms_reserved);
                ast_cli(fd, "    |_ Max SMS Channel Reservation (Incoming): %d\n", NUM_SMSCHANNEL);
                tmp_smschannel = tmp->iffirstsmschannel;                
                while (tmp_smschannel)
                {
                	ast_cli(fd, "       |_ SMS Channel: %02d, SMS Reserved: %d\n", tmp_smschannel->smschannel_index + 1, tmp_smschannel->sms_reserved);
                	tmp_smschannel = tmp_smschannel->next;
                }

                break;
        } //switch (tmp->sig)
        ast_cli(fd, "\n");        
	} //if (tmp!=NULL)
	return RESULT_SUCCESS;
}

//only show the alarm status
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_alarms(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_alarms(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	#define FORMAT_GSM_ALARMS "   | %3s %8s %s%-31.31s%s %4s %11s %7s %10s  |   \n"

	short c, p, nForSignal, nForOperator;
	int card=0;
	short nDigitalCount=0;
	short nFXOCount=0;
	short ret=0;
	char tmps[20] = "";
	char signalbar[31] = ""; //| array[30]
	char signaldBm[5] = "";
	char signalBER[20] = "";
	char operator[9] = "";
	char dialerrorflag[20], dialerrorcount[20] = "";
	short nPosText;
	int nTemp_gsmsignal;
	struct dgv_pvt *tmp, *tmp2;
	
	char textalarm[10000] = "";
	char temptextalarm[1000] = "";
	short nForText;
	short nPrintType;

    if (argc != 4)
    {
        ast_cli(fd, "\n");
        
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;		
		}         	    	
        return RESULT_SHOWUSAGE;
    }

    if (!strncmp(argv[3], "all", 3))
    	nPrintType = 1;
    else
    	if (!strncmp(argv[3], "json", 4))
				nPrintType = 2;
			else
				nPrintType = 0;

   	if (nPrintType > 0)
   	{
		ast_cli(fd, "\n");
		
    	//show only to parse string
		for (c = 1; c <= dg_GetCardsCount(); c++)
		{
			switch (dg_GetCardInterface(c))
			{
				case DG_DIGITAL_INTERFACE:
					nDigitalCount++;
				
					if (dg_GetCardPortsCount(c) == 60)
					{
						if (nPrintType == 1)
							ast_cli(fd, "<"
										"CARD:%d,DG_DIGITAL_INTERFACE,E1;"
										"PORTS_COUNT:%d;"
										"SPAN:%d;"
										"ALARM_LOSS:%d,%d;"
										"ALARM_AISS:%d,%d;"
										"ALARM_AIS16S:%d,%d;"
										"ALARM_SYNC:%d,%d;"
										"ALARM_MFSYNC:%d,%d;"
										"ALARM_RAIS:%d,%d;"
										"ALARM_CRC4SYNC:%d,%d;"
#ifdef USE_LIBPRI
											"ALARM_LINKDOWN:%d,%d;"
#endif//#ifdef USE_LIBPRI
										"ALARM_RSLIP:%d;"
										"LAST_RESET:%s"
										"> \n\n",
										c,
										60,
										1,
										dgv_e1alarms[c - 1][0].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][0].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][0].nALARM_AISS    , dgv_e1alarmsCount[c - 1][0].nALARM_AISS    ,
										dgv_e1alarms[c - 1][0].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][0].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][0].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][0].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][0].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][0].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][0].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][0].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][0].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][0].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][0].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][0].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][0].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][0]
									);
						else
						{
							if (nDigitalCount == 1)
								ast_cli(fd, "[");
							else
								ast_cli(fd, ",");
							ast_cli(fd, "{"
										"\"CARD\":\"%d,DG_DIGITAL_INTERFACE,E1\","
										"\"PORTS_COUNT\":\"%d\","
										"\"SPAN\":\"%d\","
										"\"ALARM_LOSS\":\"%d,%d\","
										"\"ALARM_AISS\":\"%d,%d\","
										"\"ALARM_AIS16S\":\"%d,%d\","
										"\"ALARM_SYNC\":\"%d,%d\","
										"\"ALARM_MFSYNC\":\"%d,%d\","
										"\"ALARM_RAIS\":\"%d,%d\","
										"\"ALARM_CRC4SYNC\":\"%d,%d\","
#ifdef USE_LIBPRI
											"\"ALARM_LINKDOWN\":\"%d,%d\","
#endif//#ifdef USE_LIBPRI
										"\"ALARM_RSLIP\":\"%d\","
										"\"LAST_RESET\":\"%s\""
										"}",
										c,
										60,
										1,
										dgv_e1alarms[c - 1][0].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][0].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][0].nALARM_AISS    , dgv_e1alarmsCount[c - 1][0].nALARM_AISS    ,
										dgv_e1alarms[c - 1][0].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][0].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][0].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][0].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][0].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][0].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][0].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][0].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][0].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][0].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][0].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][0].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][0].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][0]
									);
						}
						
						if (nPrintType == 1)
							ast_cli(fd, "<"
										"CARD:%d,DG_DIGITAL_INTERFACE,E1;"
										"PORTS_COUNT:%d;"
										"SPAN:%d;"
										"ALARM_LOSS:%d,%d;"
										"ALARM_AISS:%d,%d;"
										"ALARM_AIS16S:%d,%d;"
										"ALARM_SYNC:%d,%d;"
										"ALARM_MFSYNC:%d,%d;"
										"ALARM_RAIS:%d,%d;"
										"ALARM_CRC4SYNC:%d,%d;"
#ifdef USE_LIBPRI
											"ALARM_LINKDOWN:%d,%d;"
#endif//#ifdef USE_LIBPRI
										"ALARM_RSLIP:%d;"
										"LAST_RESET:%s"
										"> \n\n",
										c,
										60,
										2,
										dgv_e1alarms[c - 1][1].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][1].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][1].nALARM_AISS    , dgv_e1alarmsCount[c - 1][1].nALARM_AISS    ,
										dgv_e1alarms[c - 1][1].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][1].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][1].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][1].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][1].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][1].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][1].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][1].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][1].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][1].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][1].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][1].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][1].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][1]
									);
						else
						{
							ast_cli(fd, ",");
							ast_cli(fd, "{"
										"\"CARD\":\"%d,DG_DIGITAL_INTERFACE,E1\","
										"\"PORTS_COUNT\":\"%d\","
										"\"SPAN\":\"%d\","
										"\"ALARM_LOSS\":\"%d,%d\","
										"\"ALARM_AISS\":\"%d,%d\","
										"\"ALARM_AIS16S\":\"%d,%d\","
										"\"ALARM_SYNC\":\"%d,%d\","
										"\"ALARM_MFSYNC\":\"%d,%d\","
										"\"ALARM_RAIS\":\"%d,%d\","
										"\"ALARM_CRC4SYNC\":\"%d,%d\","
#ifdef USE_LIBPRI
											"\"ALARM_LINKDOWN\":\"%d,%d\","
#endif//#ifdef USE_LIBPRI
										"\"ALARM_RSLIP\":\"%d\","
										"\"LAST_RESET\":\"%s\""
										"}",
										c,
										60,
										2,
										dgv_e1alarms[c - 1][1].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][1].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][1].nALARM_AISS    , dgv_e1alarmsCount[c - 1][1].nALARM_AISS    ,
										dgv_e1alarms[c - 1][1].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][1].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][1].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][1].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][1].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][1].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][1].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][1].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][1].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][1].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][1].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][1].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][1].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][1]
									);
						}
					}
					else //if (dg_GetCardPortsCount(c) == 60)
					{
						if (nPrintType == 1)
							ast_cli(fd, "<"
										"CARD:%d,DG_DIGITAL_INTERFACE,E1;"
										"PORTS_COUNT:%d;"
										"SPAN:%d;"
										"ALARM_LOSS:%d,%d;"
										"ALARM_AISS:%d,%d;"
										"ALARM_AIS16S:%d,%d;"
										"ALARM_SYNC:%d,%d;"
										"ALARM_MFSYNC:%d,%d;"
										"ALARM_RAIS:%d,%d;"
										"ALARM_CRC4SYNC:%d,%d;"
#ifdef USE_LIBPRI
											"ALARM_LINKDOWN:%d,%d;"
#endif//#ifdef USE_LIBPRI
										"ALARM_RSLIP:%d;"
										"LAST_RESET:%s"
										"> \n\n",
										c,
										30,
										1,
										dgv_e1alarms[c - 1][0].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][0].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][0].nALARM_AISS    , dgv_e1alarmsCount[c - 1][0].nALARM_AISS    ,
										dgv_e1alarms[c - 1][0].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][0].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][0].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][0].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][0].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][0].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][0].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][0].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][0].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][0].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][0].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][0].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][0].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][0]
									);
						else
						{
							if (nDigitalCount == 1)
								ast_cli(fd, "[");
							else
								ast_cli(fd, ",");
							ast_cli(fd, "{"
										"\"CARD\":\"%d,DG_DIGITAL_INTERFACE,E1\","
										"\"PORTS_COUNT\":\"%d\","
										"\"SPAN\":\"%d\","
										"\"ALARM_LOSS\":\"%d,%d\","
										"\"ALARM_AISS\":\"%d,%d\","
										"\"ALARM_AIS16S\":\"%d,%d\","
										"\"ALARM_SYNC\":\"%d,%d\","
										"\"ALARM_MFSYNC\":\"%d,%d\","
										"\"ALARM_RAIS\":\"%d,%d\","
										"\"ALARM_CRC4SYNC\":\"%d,%d\","
#ifdef USE_LIBPRI
											"\"ALARM_LINKDOWN\":\"%d,%d\","
#endif//#ifdef USE_LIBPRI
										"\"ALARM_RSLIP\":\"%d\","
										"\"LAST_RESET\":\"%s\""
										"}",
										c,
										30,
										1,
										dgv_e1alarms[c - 1][0].nALARM_LOSS    , dgv_e1alarmsCount[c - 1][0].nALARM_LOSS    ,
										dgv_e1alarms[c - 1][0].nALARM_AISS    , dgv_e1alarmsCount[c - 1][0].nALARM_AISS    ,
										dgv_e1alarms[c - 1][0].nALARM_AIS16S  , dgv_e1alarmsCount[c - 1][0].nALARM_AIS16S  ,
										dgv_e1alarms[c - 1][0].nALARM_SYNC    , dgv_e1alarmsCount[c - 1][0].nALARM_SYNC    ,
										dgv_e1alarms[c - 1][0].nALARM_MFSYNC  , dgv_e1alarmsCount[c - 1][0].nALARM_MFSYNC  ,
										dgv_e1alarms[c - 1][0].nALARM_RAIS    , dgv_e1alarmsCount[c - 1][0].nALARM_RAIS    ,
										dgv_e1alarms[c - 1][0].nALARM_CRC4SYNC, dgv_e1alarmsCount[c - 1][0].nALARM_CRC4SYNC,
#ifdef USE_LIBPRI
											dgv_e1alarms[c - 1][0].nALARM_LINKDOWN, dgv_e1alarmsCount[c - 1][0].nALARM_LINKDOWN,
#endif//#ifdef USE_LIBPRI
										dgv_e1alarms[c - 1][0].nALARM_RSLIP   ,
										LastAlarmReset[c - 1][0]
									);
						}

					} //if (dg_GetCardPortsCount(c) == 60)

					nFXOCount = 0;
					for (p = 1; p <= dg_GetCardPortsCount(c); p++)
					{
						tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
						if (tmp)
						{
							if (tmp->sig == DG_SIG_CB_FXO || tmp->sig == DG_SIG_CB_FXS)
							{
								nFXOCount = 1;
								break;
							}
						}
						else
						{
							if (nPrintType == 1)
								ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
							else
								ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
							break;
						}
					}

					if (nFXOCount)
					{
						if (dg_GetCardPortsCount(c) == 60)
						{
							for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
								textalarm[nForText] = '\0';
		
							if (nPrintType == 1)
							{
								ast_copy_string(textalarm, "<", sizeof(textalarm));
								sprintf(temptextalarm, "CARD:%d,DG_DIGITAL_INTERFACE,CB;", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "PORTS_COUNT:%d;", 60);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "SPAN:%d;", 1);
								strcat(textalarm, temptextalarm);
							}
							else
							{
								ast_copy_string(textalarm, ",{", sizeof(textalarm));
								sprintf(temptextalarm, "\"CARD\":\"%d,DG_DIGITAL_INTERFACE,CB\",", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"PORTS_COUNT\":\"%d\",", 60);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"SPAN\":\"%d\",", 1);
								strcat(textalarm, temptextalarm);
							}
							
							if (nPrintType == 1)
							{
								for (p = 1; p <= 30; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "CHANNEL:%d,%d,%d;", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "CHANNEL:%d,-1,-1;", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)


								sprintf(temptextalarm, "LAST_RESET:%s", LastAlarmReset[c - 1][0]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "> \n\n");
							}
							else //if (nPrintType == 1)
							{
								sprintf(temptextalarm, "\"CHANNEL\":[");
								strcat(textalarm, temptextalarm);

								for (p = 1; p <= 30; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"%d\",\"alarm\":\"%d\"},", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"-1\",\"alarm\":\"-1\"},", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

								textalarm[strlen(textalarm) - 1] = '\0';

								sprintf(temptextalarm, "],");
								strcat(textalarm, temptextalarm);

								sprintf(temptextalarm, "\"LAST_RESET\":\"%s\"", LastAlarmReset[c - 1][0]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "}");
							} //if (nPrintType == 1)

							ast_cli(fd, "%s", textalarm);
							
							for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
								textalarm[nForText] = '\0';
		
							if (nPrintType == 1)
							{
								ast_copy_string(textalarm, "<", sizeof(textalarm));
								sprintf(temptextalarm, "CARD:%d,DG_DIGITAL_INTERFACE,CB;", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "PORTS_COUNT:%d;", 60);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "SPAN:%d;", 2);
								strcat(textalarm, temptextalarm);
							}
							else
							{
								ast_copy_string(textalarm, ",{", sizeof(textalarm));
								sprintf(temptextalarm, "\"CARD\":\"%d,DG_DIGITAL_INTERFACE,CB\",", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"PORTS_COUNT\":\"%d\",", 60);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"SPAN\":\"%d\",", 2);
								strcat(textalarm, temptextalarm);
							}
							
							if (nPrintType == 1)
							{
								for (p = 31; p <= 60; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "CHANNEL:%d,%d,%d;", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "CHANNEL:%d,-1,-1;", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

								sprintf(temptextalarm, "LAST_RESET:%s", LastAlarmReset[c - 1][1]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "> \n\n");
								ast_cli(fd, "%s", textalarm);
							}
							else //if (nPrintType == 1)
							{
								sprintf(temptextalarm, "\"CHANNEL\":[");
								strcat(textalarm, temptextalarm);

								for (p = 31; p <= 60; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"%d\",\"alarm\":\"%d\"},", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"-1\",\"alarm\":\"-1\"},", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

								textalarm[strlen(textalarm) - 1] = '\0';

								sprintf(temptextalarm, "],");
								strcat(textalarm, temptextalarm);

								sprintf(temptextalarm, "\"LAST_RESET\":\"%s\"", LastAlarmReset[c - 1][0]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "}");
							} //if (nPrintType == 1)
						}
						else //if (dg_GetCardPortsCount(c) == 60)
						{
							for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
								textalarm[nForText] = '\0';
		
							if (nPrintType == 1)
							{
								ast_copy_string(textalarm, "<", sizeof(textalarm));
								sprintf(temptextalarm, "CARD:%d,DG_DIGITAL_INTERFACE,CB;", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "PORTS_COUNT:%d;", 30);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "SPAN:%d;", 1);
								strcat(textalarm, temptextalarm);
							}
							else
							{
								ast_copy_string(textalarm, ",{", sizeof(textalarm));
								sprintf(temptextalarm, "\"CARD\":\"%d,DG_DIGITAL_INTERFACE,CB\",", c);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"PORTS_COUNT\":\"%d\",", 30);
								strcat(textalarm, temptextalarm);
								sprintf(temptextalarm, "\"SPAN\":\"%d\",", 1);
								strcat(textalarm, temptextalarm);
							}
							
							if (nPrintType == 1)
							{
								for (p = 1; p <= 30; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "CHANNEL:%d,%d,%d;", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "CHANNEL:%d,-1,-1;", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)


								sprintf(temptextalarm, "LAST_RESET:%s", LastAlarmReset[c - 1][0]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "> \n\n");
							}
							else //if (nPrintType == 1)
							{
								sprintf(temptextalarm, "\"CHANNEL\":[");
								strcat(textalarm, temptextalarm);

								for (p = 1; p <= 30; p++)
								{
									tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
									if (tmp)
									{
										if (tmp->sig == DG_SIG_CB_FXO)
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"%d\",\"alarm\":\"%d\"},", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
										else
											sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"-1\",\"alarm\":\"-1\"},", p);
										strcat(textalarm, temptextalarm);
									}
									else
									{
										if (nPrintType == 1)
											ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
										else
											ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
										break;
									}
								} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

								textalarm[strlen(textalarm) - 1] = '\0';

								sprintf(temptextalarm, "],");
								strcat(textalarm, temptextalarm);

								sprintf(temptextalarm, "\"LAST_RESET\":\"%s\"", LastAlarmReset[c - 1][0]);
								strcat(textalarm, temptextalarm);
								strcat(textalarm, "}");
							} //if (nPrintType == 1)

							ast_cli(fd, "%s", textalarm);
						} //if (dg_GetCardPortsCount(c) == 60)
					} //if (nFXOCount)
					
					break;
				case DG_GSM_INTERFACE:
					nDigitalCount++;
					
					for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
						textalarm[nForText] = '\0';

					if (nPrintType == 1)
					{
						ast_copy_string(textalarm, "<", sizeof(textalarm));
						sprintf(temptextalarm, "CARD:%d,DG_GSM_INTERFACE;", c);
						strcat(textalarm, temptextalarm);
						sprintf(temptextalarm, "PORTS_COUNT:%d;", dg_GetCardPortsCount(c));
						strcat(textalarm, temptextalarm);

						for (p = 1; p <= dg_GetCardPortsCount(c); p++)
						{
							tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
							if (tmp)
							{
								sprintf(temptextalarm, "CHANNEL:%d,%d,%d,%d,%s,%d,%d;",
										p,
										((dgv_gsmsignal[c - 1][p - 1][0] >= 99) ? dgv_gsmsignal[c - 1][p - 1][0] : ((dgv_gsmsignal[c - 1][p - 1][0] < 5) ? 0 : (dgv_gsmsignal[c - 1][p - 1][0] - 5))),
										dgv_gsmsignal[c - 1][p - 1][1],
										(tmp->alarmed < 2) ? tmp->alarmed : 1,
										((strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown"),
										dgv_gsmDECount[c - 1][p - 1].nFlagDialError,
										dgv_gsmDECount[c - 1][p - 1].nTotalDialError);
								strcat(textalarm, temptextalarm);
							}
							else
							{
								if (nPrintType == 1)
									ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
								else
									ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
								break;
							}
						} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

						sprintf(temptextalarm, "LAST_RESET:%s", LastAlarmReset[c - 1][0]);
						strcat(textalarm, temptextalarm);
						strcat(textalarm, "> \n\n");
					}
					else //if (nPrintType == 1)
					{
						if (nDigitalCount == 1)
							ast_copy_string(textalarm, "[{", sizeof(textalarm));
						else
							ast_copy_string(textalarm, ",{", sizeof(textalarm));
						sprintf(temptextalarm, "\"CARD\":\"%d,DG_GSM_INTERFACE\",", c);
						strcat(textalarm, temptextalarm);
						sprintf(temptextalarm, "\"PORTS_COUNT\":\"%d\",", dg_GetCardPortsCount(c));
						strcat(textalarm, temptextalarm);

						sprintf(temptextalarm, "\"CHANNEL\":[");
						strcat(textalarm, temptextalarm);

						for (p = 1; p <= dg_GetCardPortsCount(c); p++)
						{
							tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
							if (tmp)
							{
								sprintf(temptextalarm, "{\"channel\":\"%d\",\"signalquality\":\"%d\",\"callquality\":\"%d\",\"alarm\":\"%d\",\"operator\":\"%s\",\"dialerror\":\"%d\",\"dialerrorcount\":\"%d\"},",
										p,
										((dgv_gsmsignal[c - 1][p - 1][0] >= 99) ? dgv_gsmsignal[c - 1][p - 1][0] : ((dgv_gsmsignal[c - 1][p - 1][0] < 5) ? 0 : (dgv_gsmsignal[c - 1][p - 1][0] - 5))),
										dgv_gsmsignal[c - 1][p - 1][1],
										(tmp->alarmed < 2) ? tmp->alarmed : 1,
										((strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown"),
										dgv_gsmDECount[c - 1][p - 1].nFlagDialError,
										dgv_gsmDECount[c - 1][p - 1].nTotalDialError);
								strcat(textalarm, temptextalarm);
							}
							else
							{
								if (nPrintType == 1)
									ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
								else
									ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
								break;
							}
						} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

						if (dg_GetCardPortsCount(c) > 0)
							textalarm[strlen(textalarm) - 1] = '\0';

						sprintf(temptextalarm, "],");
						strcat(textalarm, temptextalarm);

						sprintf(temptextalarm, "\"LAST_RESET\":\"%s\"", LastAlarmReset[c - 1][0]);
						strcat(textalarm, temptextalarm);
						strcat(textalarm, "}");

					} //if (nPrintType == 1)
					ast_cli(fd, "%s", textalarm);
					
					break;
				case DG_FXO_INTERFACE:
				case DG_FX_INTERFACE:					
					nFXOCount = 0;
					for (p = 1; p <= dg_GetCardPortsCount(c); p++)
					{
						tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
						if (tmp)
						{
							if (tmp->sig == DG_SIG_FXO)
							{
								nFXOCount = 1;
								break;
							}
						}
						else
						{
							if (nPrintType == 1)
								ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
							else
								ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
							break;
						}
					}
					
					if (nFXOCount)
					{				
						nDigitalCount++;
						
						for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
							textalarm[nForText] = '\0';
						
						if (nPrintType == 1)
						{
							ast_copy_string(textalarm, "<", sizeof(textalarm));
							if (dg_GetCardInterface(c) == DG_FXO_INTERFACE)
								sprintf(temptextalarm, "CARD:%d,DG_FXO_INTERFACE;", c);
							else
								sprintf(temptextalarm, "CARD:%d,DG_FX_INTERFACE;", c);
							strcat(textalarm, temptextalarm);
							sprintf(temptextalarm, "PORTS_COUNT:%d;", dg_GetCardPortsCount(c));
							strcat(textalarm, temptextalarm);

							for (p = 1; p <= dg_GetCardPortsCount(c); p++)
							{
								tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
								if (tmp)
								{
									if (tmp->sig == DG_SIG_FXO)
										sprintf(temptextalarm, "CHANNEL:%d,%d,%d;", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
									else
										sprintf(temptextalarm, "CHANNEL:%d,-1,-1;", p);
									strcat(textalarm, temptextalarm);
								}
								else
								{
									if (nPrintType == 1)
										ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
									else
										ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
									break;
								}
							} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

							sprintf(temptextalarm, "LAST_RESET:%s", LastAlarmReset[c - 1][0]);
							strcat(textalarm, temptextalarm);
							strcat(textalarm, "> \n\n");
						}
						else //if (nPrintType == 1)
						{
							if (nDigitalCount == 1)
								ast_copy_string(textalarm, "[{", sizeof(textalarm));
							else
								ast_copy_string(textalarm, ",{", sizeof(textalarm));

							if (dg_GetCardInterface(c) == DG_FXO_INTERFACE)
								sprintf(temptextalarm, "\"CARD\":\"%d,DG_FXO_INTERFACE\",", c);
							else
								sprintf(temptextalarm, "\"CARD\":\"%d,DG_FX_INTERFACE\",", c);
							strcat(textalarm, temptextalarm);
							sprintf(temptextalarm, "\"PORTS_COUNT\":\"%d\",", dg_GetCardPortsCount(c));
							strcat(textalarm, temptextalarm);

							sprintf(temptextalarm, "\"CHANNEL\":[");
							strcat(textalarm, temptextalarm);

							for (p = 1; p <= dg_GetCardPortsCount(c); p++)
							{
								tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
								if (tmp)
								{
									if (tmp->sig == DG_SIG_FXO)
										sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"%d\",\"alarm\":\"%d\"},", p, dgv_fxoNDTCount[c - 1][p - 1], tmp->alarmed);
									else
										sprintf(temptextalarm, "{\"channel\":\"%d\",\"ndtc\":\"-1\",\"alarm\":\"-1\"},", p);
									strcat(textalarm, temptextalarm);
								}
								else
								{
									if (nPrintType == 1)
										ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", c, p);
									else
										ast_cli(fd, "{\"Error\":\"Cannot show alarm notifications for card %d, port %d when tmp is null\"}\n", c, p);
									break;
								}
							} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

							if (dg_GetCardPortsCount(c) > 0)
								textalarm[strlen(textalarm) - 1] = '\0';

							sprintf(temptextalarm, "],");
							strcat(textalarm, temptextalarm);

							sprintf(temptextalarm, "\"LAST_RESET\":\"%s\"", LastAlarmReset[c - 1][0]);
							strcat(textalarm, temptextalarm);
							strcat(textalarm, "}");
						} //if (nPrintType == 1)

						ast_cli(fd, "%s", textalarm);
					}
					
					break;
			} //switch (dg_GetCardInterface(c))
		}//for (c = 1; c <= dg_GetCardsCount(); c++)		
		
		if (nDigitalCount==0)
		{
			ast_cli(fd, "\n");

			if (nPrintType == 1)
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "       Error: No E1/GSM/FXO Cards Found. \n");
						break;
					case 1://Portuguese
						ast_cli(fd, "       Erro: Nenhuma Placa E1/GSM/FXO encontrada. \n");
						break;
					default:
						ast_cli(fd, "       Error: No E1/GSM/FXO Cards Found. \n");
						//0.9.8 break;
				}
			else
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "{\"Error\":\"No E1/GSM/FXO Cards Found.\"}\n");
						break;
					case 1://Portuguese
						ast_cli(fd, "{\"Error\":\"Nenhuma Placa E1/GSM/FXO encontrada.\"}\n");
						break;
					default:
						ast_cli(fd, "{\"Error\":\"No E1/GSM/FXO Cards Found.\"}\n");
						//0.9.8 break;
				}
		}
		else
			if (nPrintType == 2)
				ast_cli(fd, "]\n");

		return RESULT_SUCCESS;  	
    } //if (nPrintType > 0)
   
    card = atoi(argv[3]);
	
	if (card == 99)
	{
		for (c = 1; c <= dg_GetCardsCount(); c++)
		{
			if (dg_GetCardInterface(c) == DG_DIGITAL_INTERFACE)
			{
				ast_cli(fd, "\n       Requesting alarm notifications(card %d)...\n", c);
				
				ret = dg_GetAlarmStatus(c);
				if (ret != DG_EXIT_SUCCESS)
					ast_cli(fd, "Error requesting alarm notifications for card %d - error code %x\n", c, ret);
			}
			else
				if (dg_GetCardInterface(c) == DG_GSM_INTERFACE)
				{
					//date and time  
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
						struct ast_tm tm;
						struct timeval now = ast_tvnow();
						ast_localtime(&now, &tm, NULL);
						ast_strftime(LastAlarmReset[c - 1][0], sizeof(LastAlarmReset[c - 1][0]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
						time_t t;
						struct tm tm;
						time(&t);
						ast_localtime(&t, &tm, NULL);//old localtime_r
						strftime(LastAlarmReset[c - 1][0], sizeof(LastAlarmReset[c - 1][0]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					
					ast_cli(fd, "\n       Requesting alarm notifications(card %d)...\n", c);
					
					for (p = 1; p <= dg_GetCardPortsCount(c); p++)
					{
						tmp = find_ch_port(dg_GetAbsolutePortNumber(c, p));
						if (tmp != NULL)
						{
							if (tmp->call_state != DG_AST_CALLING && tmp->sms_reserved == 0 && tmp->alarmed == 0)
							{
								ret = dg_GSMCheckSignalQuality(dg_GetAbsolutePortNumber(c, p));
								if (ret != DG_EXIT_SUCCESS)
									ast_cli(fd, "Error requesting alarm notifications for card %d, port %d - error code %x\n", c, p, ret);
							}
							else
							{
								if (option_debug==100)
									ast_log(LOG_DEBUG, "Cannot request alarm notifications for card %d, port %d (call_state %d, sms_reserved %d)\n", c, p, tmp->call_state, tmp->sms_reserved);
							}
						}
						else
						{
							ast_cli(fd, "Cannot request alarm notifications for card %d, port %d when tmp is null\n", c, p);
							break;
						}
					} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)
				} //if (dg_GetCardInterface(c) == DG_GSM_INTERFACE)
		} //for (c = 1; c <= dg_GetCardsCount(); c++)
		ast_cli(fd, "\n");

		return RESULT_SUCCESS;
	}
	
	switch (dg_GetCardType(card))
	{
		case VB0408PCI:
		case VB0408PCIE:
		case VB0404FX:
		case VB0404FX_R:
			nDigitalCount = 0;
			
			nFXOCount = 0;
			for (p = 1; p <= dg_GetCardPortsCount(card); p++)
			{
				tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
				if (tmp)
				{
					if (tmp->sig == DG_SIG_FXO)
					{
						nFXOCount = 1;
						break;
					}
				}
				else
				{
					ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
					break;
				}
			}
			
			if (nFXOCount)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English		
						ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - FXO Card: %d \n\n", card);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       DigiVoice - Alarmes de Ausencia de Tom de Discagem - Placa FXO: %d \n\n", card);		
						break;
					default:
						ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - FXO Card: %d \n\n", card);
				}
				break;
			}
			else
			{
				ast_cli(fd, "\n");
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English		
	        			ast_cli(fd, "       Error: This card isn't an E1/GSM/FXO Card. \n");
						break;
					case 1://Portuguese
						ast_cli(fd, "       Erro: Esta placa nao eh uma Placa E1/GSM/FXO. \n");
						break;
					default:
						ast_cli(fd, "       Error: This card isn't an E1/GSM/FXO Card. \n");
						//0.9.8 break;		
				}       	    	
				return RESULT_SHOWUSAGE;						
			}
	    case VBE13030PCI:
		case VBE13060PCI:			    		
	    case VBE13060PCI_R:	    
		case VBE16060PCI:
		case VBE16060PCI_R:
		case VB3030PCIE:
		case VB6060PCIE:
		case DGV_1E1F:
		case DGV_2E1F:
			nDigitalCount = 1;

			break;
		case VB0404GSM:
			nDigitalCount = 0;

			//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				struct ast_tm tm;
				struct timeval now = ast_tvnow();
				ast_localtime(&now, &tm, NULL);
				ast_strftime(LastAlarmReset[card - 1][0], sizeof(LastAlarmReset[card - 1][0]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				time_t t;
				struct tm tm;
				time(&t);
				ast_localtime(&t, &tm, NULL);//old localtime_r
				strftime(LastAlarmReset[card - 1][0], sizeof(LastAlarmReset[card - 1][0]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

			switch (dgv_conf.consolelanguage)
			{
				case 0://English		
					ast_cli(fd, "\n       DigiVoice - GSM Alarms - GSM Card: %d \n\n", card);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       DigiVoice - Alarmes GSM - Placa GSM: %d \n\n", card);
					break;
				default:
					ast_cli(fd, "\n       DigiVoice - GSM Alarms - GSM Card: %d \n\n", card);
			}			
			break;
		default:
			ast_cli(fd, "\n");
			
			switch (dgv_conf.consolelanguage)
			{
				case 0://English		
        			ast_cli(fd, "       Error: This card doesn't exist... \n");
					break;
				case 1://Portuguese
					ast_cli(fd, "       Erro: Esta placa nao existe... \n");
					break;
				default:
					ast_cli(fd, "       Error: This card doesn't exist... \n");
					//0.9.8 break;		
			}       	    	
			return RESULT_SHOWUSAGE;			
	} //switch (dg_GetCardType(card))

	switch (dg_GetCardInterface(card))
	{
		case DG_DIGITAL_INTERFACE:
			if (dg_GetCardPortsCount(card) == 60)
			{
				nFXOCount = 0;
				for (p = 1; p <= dg_GetCardPortsCount(card); p++)
				{
					tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
					if (tmp)
					{
						if (tmp->sig == DG_SIG_CB_FXO)
						{
							nFXOCount = 1;
							break;
						}
					}
					else
					{
						ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
						break;
					}
				}			
				
				if (nFXOCount)
				{			
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - E1 Card: %d \n\n", card);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       DigiVoice - Alarmes de Ausencia de Tom de Discagem - Placa E1: %d \n\n", card);
							break;
						default:
							ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - E1 Card: %d \n\n", card);
					}					
					
					ast_cli(fd, "   +---[ Span: 1 ]----------------+---[ Span: 2 ]----------------+ \n");
					ast_cli(fd, "   |                              |                              | \n");
					
					for (p = 1; p <= dg_GetCardPortsCount(card) - 30; p++)
					{
						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
						tmp2 = find_ch_port(dg_GetAbsolutePortNumber(card, p + 30));
						if (!tmp)
						{
							ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
							break;
						}
						
						if (!tmp2)
						{
							ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p + 30);
							break;
						}							
						
						for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
							textalarm[nForText] = '\0';
						
						if (tmp->sig == DG_SIG_CB_FXO)
							sprintf(temptextalarm, "   |  Channel %03d:     %s   %03d  ", p, showalarmstatus_desc[tmp->alarmed], dgv_fxoNDTCount[card - 1][p - 1]);
						else
						{
							switch (dgv_conf.consolelanguage)
							{
								case 0://English
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  ", p);									
									break;
								case 1://Portuguese
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  ", p);		
									break;
								default:
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  ", p);
							}							
						}
						strcat(textalarm, temptextalarm);
						
						if (tmp2->sig == DG_SIG_CB_FXO)
							sprintf(temptextalarm, "|  Channel %03d:     %s   %03d  | \n", p + 30, showalarmstatus_desc[tmp2->alarmed], dgv_fxoNDTCount[card - 1][p + 29]);
						else
						{
							switch (dgv_conf.consolelanguage)
							{						
								case 0://English
									sprintf(temptextalarm, "|  Channel %03d:     ---------  | \n", p + 30);									
									break;
								case 1://Portuguese
									sprintf(temptextalarm, "|  Channel %03d:     ---------  | \n", p + 30);		
									break;
								default:
									sprintf(temptextalarm, "|  Channel %03d:     ---------  | \n", p + 30);
							}
						}
						strcat(textalarm, temptextalarm);
						ast_cli(fd, "%s", textalarm);
					} //for (p = 1; p <= dg_GetCardPortsCount(card); p++)
		
					ast_cli(fd, "   |                              |                              | \n");
					ast_cli(fd, "   +------------------------------+------------------------------+ \n\n");
				} //if (nFXOCount)

				switch (dgv_conf.consolelanguage)
				{
					case 0://English		
						ast_cli(fd, "\n       DigiVoice - E1 Alarms - E1 Card: %d \n\n", card);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       DigiVoice - Alarmes E1 - Placa E1: %d \n\n", card);
						break;
					default:
						ast_cli(fd, "\n       DigiVoice - E1 Alarms - E1 Card: %d \n\n", card);
				}				
				
				ast_cli(fd, "   +---[ Span: 1 ]----------------+---[ Span: 2 ]----------------+ \n");
				ast_cli(fd, "   |                              |                              | \n");
				ast_cli(fd, "   |  ALARM_LOSS       %s   %03d  |  ALARM_LOSS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_LOSS]    , dgv_e1alarmsCount[card - 1][0].nALARM_LOSS    , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_LOSS]    , dgv_e1alarmsCount[card - 1][1].nALARM_LOSS    );
				ast_cli(fd, "   |  ALARM_AISS       %s   %03d  |  ALARM_AISS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_AISS]    , dgv_e1alarmsCount[card - 1][0].nALARM_AISS    , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_AISS]    , dgv_e1alarmsCount[card - 1][1].nALARM_AISS    );
				ast_cli(fd, "   |  ALARM_AIS16S     %s   %03d  |  ALARM_AIS16S     %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_AIS16S]  , dgv_e1alarmsCount[card - 1][0].nALARM_AIS16S  , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_AIS16S]  , dgv_e1alarmsCount[card - 1][1].nALARM_AIS16S  );
				ast_cli(fd, "   |  ALARM_SYNC       %s   %03d  |  ALARM_SYNC       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_SYNC]    , dgv_e1alarmsCount[card - 1][0].nALARM_SYNC    , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_SYNC]    , dgv_e1alarmsCount[card - 1][1].nALARM_SYNC    );
				ast_cli(fd, "   |  ALARM_MFSYNC     %s   %03d  |  ALARM_MFSYNC     %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_MFSYNC]  , dgv_e1alarmsCount[card - 1][0].nALARM_MFSYNC  , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_MFSYNC]  , dgv_e1alarmsCount[card - 1][1].nALARM_MFSYNC  );
				ast_cli(fd, "   |  ALARM_RAIS       %s   %03d  |  ALARM_RAIS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_RAIS]    , dgv_e1alarmsCount[card - 1][0].nALARM_RAIS    , showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_RAIS]    , dgv_e1alarmsCount[card - 1][1].nALARM_RAIS    );
				ast_cli(fd, "   |  ALARM_CRC4SYNC   %s   %03d  |  ALARM_CRC4SYNC   %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_CRC4SYNC], dgv_e1alarmsCount[card - 1][0].nALARM_CRC4SYNC, showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_CRC4SYNC], dgv_e1alarmsCount[card - 1][1].nALARM_CRC4SYNC);
#ifdef USE_LIBPRI
					ast_cli(fd, "   |  ALARM_LINKDOWN   %s   %03d  |  ALARM_LINKDOWN   %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_LINKDOWN], dgv_e1alarmsCount[card - 1][0].nALARM_LINKDOWN, showalarmstatus_desc[dgv_e1alarms[card - 1][1].nALARM_LINKDOWN], dgv_e1alarmsCount[card - 1][1].nALARM_LINKDOWN);
#endif//#ifdef USE_LIBPRI
				ast_cli(fd, "   |  ALARM_RSLIP            %03d  |  ALARM_RSLIP            %03d  | \n",                    dgv_e1alarms[card - 1][0].nALARM_RSLIP    ,                                                                      dgv_e1alarms[card - 1][1].nALARM_RSLIP                                                    );
				ast_cli(fd, "   |                              |                              | \n");
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "   |  Last Reset:                 |  Last Reset:                 | \n");
						break;
					case 1://Portuguese
						ast_cli(fd, "   |  Ultimo Reset:               |  Ultimo Reset:               | \n");				
						break;
					default:
						ast_cli(fd, "   |  Last Reset:                 |  Last Reset:                 | \n");
						//0.9.8 break;
				}
				ast_cli(fd, "   |  %s         |  %s         | \n", LastAlarmReset[card - 1][0], LastAlarmReset[card - 1][1]);
				ast_cli(fd, "   |                              |                              | \n");
				ast_cli(fd, "   +------------------------------+------------------------------+ \n\n");
			}
			else //if (dg_GetCardPortsCount(card) == 60)
			{
				nFXOCount = 0;
				for (p = 1; p <= dg_GetCardPortsCount(card); p++)
				{
					tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
					if (tmp)
					{
						if (tmp->sig == DG_SIG_CB_FXO)
						{
							nFXOCount = 1;
							break;
						}
					}
					else
					{
						ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
						break;
					}
				}			
				
				if (nFXOCount)
				{			
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - E1 Card: %d \n\n", card);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       DigiVoice - Alarmes de Ausencia de Tom de Discagem - Placa E1: %d \n\n", card);
							break;
						default:
							ast_cli(fd, "\n       DigiVoice - No DialTone Alarms - E1 Card: %d \n\n", card);
					}					
					
					ast_cli(fd, "   +---[ Span: 1 ]----------------+ \n");
					ast_cli(fd, "   |                              | \n");
					
					for (p = 1; p <= dg_GetCardPortsCount(card); p++)
					{
						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));						
						if (!tmp)
						{
							ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
							break;
						}
						
						for (nForText = 0; nForText <= sizeof(textalarm) - 1; nForText++)
							textalarm[nForText] = '\0';
						
						if (tmp->sig == DG_SIG_CB_FXO)
							sprintf(temptextalarm, "   |  Channel %03d:     %s   %03d  | \n", p, showalarmstatus_desc[tmp->alarmed], dgv_fxoNDTCount[card - 1][p - 1]);
						else
						{
							switch (dgv_conf.consolelanguage)
							{
								case 0://English
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  | \n", p);									
									break;
								case 1://Portuguese
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  | \n", p);
									break;
								default:
									sprintf(temptextalarm, "   |  Channel %03d:     ---------  | \n", p);
							}							
						}
						strcat(textalarm, temptextalarm);
						ast_cli(fd, "%s", textalarm);
					} //for (p = 1; p <= dg_GetCardPortsCount(card); p++)
		
					ast_cli(fd, "   |                              | \n");
					ast_cli(fd, "   +------------------------------+ \n\n");
				} //if (nFXOCount)

				switch (dgv_conf.consolelanguage)
				{
					case 0://English		
						ast_cli(fd, "\n       DigiVoice - E1 Alarms - E1 Card: %d \n\n", card);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       DigiVoice - Alarmes E1 - Placa E1: %d \n\n", card);
						break;
					default:
						ast_cli(fd, "\n       DigiVoice - E1 Alarms - E1 Card: %d \n\n", card);
				}
				
				ast_cli(fd, "   +---[ Span: 1 ]----------------+ \n");
				ast_cli(fd, "   |                              | \n");
				ast_cli(fd, "   |  ALARM_LOSS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_LOSS]    , dgv_e1alarmsCount[card - 1][0].nALARM_LOSS    );
				ast_cli(fd, "   |  ALARM_AISS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_AISS]    , dgv_e1alarmsCount[card - 1][0].nALARM_AISS    );
				ast_cli(fd, "   |  ALARM_AIS16S     %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_AIS16S]  , dgv_e1alarmsCount[card - 1][0].nALARM_AIS16S  );
				ast_cli(fd, "   |  ALARM_SYNC       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_SYNC]    , dgv_e1alarmsCount[card - 1][0].nALARM_SYNC    );
				ast_cli(fd, "   |  ALARM_MFSYNC     %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_MFSYNC]  , dgv_e1alarmsCount[card - 1][0].nALARM_MFSYNC  );
				ast_cli(fd, "   |  ALARM_RAIS       %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_RAIS]    , dgv_e1alarmsCount[card - 1][0].nALARM_RAIS    );
				ast_cli(fd, "   |  ALARM_CRC4SYNC   %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_CRC4SYNC], dgv_e1alarmsCount[card - 1][0].nALARM_CRC4SYNC);
#ifdef USE_LIBPRI
					ast_cli(fd, "   |  ALARM_LINKDOWN   %s   %03d  | \n", showalarmstatus_desc[dgv_e1alarms[card - 1][0].nALARM_LINKDOWN], dgv_e1alarmsCount[card - 1][0].nALARM_LINKDOWN);
#endif//#ifdef USE_LIBPRI
				ast_cli(fd, "   |  ALARM_RSLIP            %03d  | \n",                     dgv_e1alarms[card - 1][0].nALARM_RSLIP                                                    );
				ast_cli(fd, "   |                              | \n");
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "   |  Last Reset:                 | \n");
						break;
					case 1://Portuguese
						ast_cli(fd, "   |  Ultimo Reset:               | \n");				
						break;
					default:
						ast_cli(fd, "   |  Last Reset:                 | \n");
						//0.9.8 break;
				}
				ast_cli(fd, "   |  %s         | \n", LastAlarmReset[card - 1][0]);
				ast_cli(fd, "   |                              | \n");
				ast_cli(fd, "   +------------------------------+ \n\n");
			} //if (dg_GetCardPortsCount(card) == 60)
			break;
		case DG_GSM_INTERFACE:
			ast_cli(fd, "   +-------------------------------------------------------------------------------------+ \n");
			ast_cli(fd, "   |                                                                                     | \n");
			ast_cli(fd, FORMAT_GSM_ALARMS, "Ch", "Operator", "", "          SignalQuality", "  ", "dBm", "CallQuality", "DialErr", "DialErrCnt");
			ast_cli(fd, "   |                                                                                     | \n");

			for (p = 1; p <= dg_GetCardPortsCount(card); p++)
			{
				for (nForSignal = 0; nForSignal <= sizeof(tmps) - 1; nForSignal++)
					tmps[nForSignal] = '\0';
				for (nForOperator = 0; nForOperator <= sizeof(operator) - 1; nForOperator++)
					operator[nForOperator] = '\0';
				for (nForSignal = 0; nForSignal <= sizeof(signalbar) - 1; nForSignal++)
					signalbar[nForSignal] = ' ';
				for (nForSignal = 0; nForSignal <= sizeof(signalBER) - 1; nForSignal++)
					signalBER[nForSignal] = '\0';
				for (nForSignal = 0; nForSignal <= sizeof(dialerrorflag) - 1; nForSignal++)
					dialerrorflag[nForSignal] = '\0';
				for (nForSignal = 0; nForSignal <= sizeof(dialerrorcount) - 1; nForSignal++)
					dialerrorcount[nForSignal] = '\0';

				snprintf(tmps, sizeof(tmps), "%d", p);
				signalbar[sizeof(signalbar) - 1] = '\0';
				nPosText = 0;

				if ((dgv_gsmsignal[card - 1][p - 1][0] == 99) || (dgv_gsmsignal[card - 1][p - 1][0] == 0))
				{
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_copy_string(signalbar, "      NO SIGNAL RECEIVED      ", sizeof(signalbar));
							break;
						case 1://Portuguese
							ast_copy_string(signalbar, "    NENHUM SINAL RECEBIDO     ", sizeof(signalbar));
							break;
						default:
							ast_copy_string(signalbar, "      NO SIGNAL RECEIVED      ", sizeof(signalbar));
					}
					snprintf(signaldBm, sizeof(signaldBm), "%d", 0);
					ast_copy_string(signalBER, showcallquality_desc[0], sizeof(signalBER));
					ast_copy_string(dialerrorflag, showalarmstatus_desc[0], sizeof(dialerrorflag));
					snprintf(dialerrorcount, sizeof(dialerrorcount), "%d", 0);
					ast_cli(fd, FORMAT_GSM_ALARMS, tmps, operator, "[", signalbar, "]", signaldBm, signalBER, dialerrorflag, dialerrorcount);

					tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
					if (tmp)
					{
						manager_event(EVENT_FLAG_SYSTEM, "Alarm",
														 "CARD: %d\r\n"
														 "CHANNEL: %d\r\n"
														 "SIGNALQUALITY: %d\r\n"
														 "CALLQUALITY: %d\r\n"
														 "ALARMED: %d\r\n"
														 "OPERATOR: %s\r\n"
								 	 	 	 	 	 	 "DIALERROR: %s\r\n"
								 	 	 	 	 	 	 "DIALERRORCOUNT: %d\r\n",
														 tmp->board,
														 p,
														 ((dgv_gsmsignal[card - 1][p - 1][0] >= 99) ? dgv_gsmsignal[card - 1][p - 1][0] : ((dgv_gsmsignal[card - 1][p - 1][0] < 5) ? 0 : (dgv_gsmsignal[card - 1][p - 1][0] - 5))),
														 dgv_gsmsignal[card - 1][p - 1][1],
														 (tmp->alarmed < 2) ? tmp->alarmed : 1,
														 ((strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown"),
														 (dgv_gsmDECount[card - 1][p - 1].nFlagDialError) ? "yes" : "no",
														 dgv_gsmDECount[card - 1][p - 1].nTotalDialError
									  );
					}
				}
				else
				{
					if (dgv_gsmsignal[card - 1][p - 1][0] == 100)
					{
						switch (dgv_conf.consolelanguage)
						{
							case 0://English
								ast_copy_string(signalbar, "     NO SIM CARD DETECTED     ", sizeof(signalbar));
								break;
							case 1://Portuguese
								ast_copy_string(signalbar, "   NENHUM SIM CARD DETECTADO  ", sizeof(signalbar));
								break;
							default:
								ast_copy_string(signalbar, "     NO SIM CARD DETECTED     ", sizeof(signalbar));
						}
						snprintf(signaldBm, sizeof(signaldBm), "%d", 0);
						ast_copy_string(signalBER, showcallquality_desc[0], sizeof(signalBER));
						ast_copy_string(dialerrorflag, showalarmstatus_desc[0], sizeof(dialerrorflag));
						snprintf(dialerrorcount, sizeof(dialerrorcount), "%d", 0);
						ast_cli(fd, FORMAT_GSM_ALARMS, tmps, operator, "[", signalbar, "]", signaldBm, signalBER, dialerrorflag, dialerrorcount);

						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
						if (tmp)
						{
							manager_event(EVENT_FLAG_SYSTEM, "Alarm",
															 "CARD: %d\r\n"
															 "CHANNEL: %d\r\n"
															 "SIGNALQUALITY: %d\r\n"
															 "CALLQUALITY: %d\r\n"
															 "ALARMED: %d\r\n"
															 "OPERATOR: %s\r\n"
			 	 	 	 	 	 	 	 	 	 	 	 	 "DIALERROR: %s\r\n"
			 	 	 	 	 	 	 	 	 	 	 	 	 "DIALERRORCOUNT: %d\r\n",
															 tmp->board,
															 p,
															 ((dgv_gsmsignal[card - 1][p - 1][0] >= 99) ? dgv_gsmsignal[card - 1][p - 1][0] : ((dgv_gsmsignal[card - 1][p - 1][0] < 5) ? 0 : (dgv_gsmsignal[card - 1][p - 1][0] - 5))),
															 dgv_gsmsignal[card - 1][p - 1][1],
															 (tmp->alarmed < 2) ? tmp->alarmed : 1,
															 ((strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown"),
															 (dgv_gsmDECount[card - 1][p - 1].nFlagDialError) ? "yes" : "no",
															 dgv_gsmDECount[card - 1][p - 1].nTotalDialError
										  );
						}
					}
					else
					{
						nTemp_gsmsignal = dgv_gsmsignal[card - 1][p - 1][0];

						if (nTemp_gsmsignal < 5)
							nTemp_gsmsignal = 0;
						else
							nTemp_gsmsignal = nTemp_gsmsignal - 5;

						for (nForSignal = 0; nForSignal <= nTemp_gsmsignal - 1; nForSignal++)
						{
							signalbar[nPosText] = gsm_signalsep;
							nPosText++;
							signalbar[nPosText] = gsm_signalsep;
							nPosText++;

							if (nForSignal == 15)
								break;
						}
						signalbar[sizeof(signalbar) - 1] = '\0';
						snprintf(signaldBm, sizeof(signaldBm), "%04d", (-113 + (2 * dgv_gsmsignal[card - 1][p - 1][0])));
						ast_copy_string(signalBER, showcallquality_desc[((dgv_gsmsignal[card - 1][p - 1][1] == 99) ? 0 : (dgv_gsmsignal[card - 1][p - 1][1] >= 6 ? 1 : (dgv_gsmsignal[card - 1][p - 1][1] >= 3 ? 2 : 3)))], sizeof(signalBER));
						ast_copy_string(dialerrorflag, showalarmstatus_desc[dgv_gsmDECount[card - 1][p - 1].nFlagDialError], sizeof(dialerrorflag));
						snprintf(dialerrorcount, sizeof(dialerrorcount), "%d", dgv_gsmDECount[card - 1][p - 1].nTotalDialError);

						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
						if (tmp)
						{
							if (strlen(tmp->gsm_operator_name) > 1)
							{
								ast_copy_string(operator, tmp->gsm_operator_name, sizeof(operator));

								for (nForOperator = strlen(operator); nForOperator <= sizeof(operator) - 1; nForOperator++)
									operator[nForOperator] = ' ';

								operator[sizeof(operator) - 1] = '\0';
							}
							else
							{
								for (nForOperator = 0; nForOperator <= sizeof(operator) - 1; nForOperator++)
									operator[nForOperator] = ' ';

								operator[sizeof(operator) - 1] = '\0';
							}

							manager_event(EVENT_FLAG_SYSTEM, "Alarm",
															 "CARD: %d\r\n"
															 "CHANNEL: %d\r\n"
															 "SIGNALQUALITY: %d\r\n"
															 "CALLQUALITY: %d\r\n"
															 "ALARMED: %d\r\n"
															 "OPERATOR: %s\r\n"
									 	 	 	 	 	 	 "DIALERROR: %s\r\n"
									 	 	 	 	 	 	 "DIALERRORCOUNT: %d\r\n",
															 tmp->board,
															 p,
															 ((dgv_gsmsignal[card - 1][p - 1][0] >= 99) ? dgv_gsmsignal[card - 1][p - 1][0] : ((dgv_gsmsignal[card - 1][p - 1][0] < 5) ? 0 : (dgv_gsmsignal[card - 1][p - 1][0] - 5))),
															 dgv_gsmsignal[card - 1][p - 1][1],
															 (tmp->alarmed < 2) ? tmp->alarmed : 1,
															 ((strlen(tmp->gsm_operator_name) > 1) ? tmp->gsm_operator_name : "Unknown"),
															 (dgv_gsmDECount[card - 1][p - 1].nFlagDialError) ? "yes" : "no",
															 dgv_gsmDECount[card - 1][p - 1].nTotalDialError
										  );
						}
						else //if (tmp)
						{
							ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
							break;
						} //if (tmp)

						ast_cli(fd, FORMAT_GSM_ALARMS, tmps, operator, "[", signalbar, "]", signaldBm, signalBER, dialerrorflag, dialerrorcount);
					}
				}
			}
			ast_cli(fd, "   |                                                                                     | \n");
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "   |  Last Reset:                                                                        | \n");
					break;
				case 1://Portuguese
					ast_cli(fd, "   |  Ultimo Reset:                                                                      | \n");
					break;
				default:
					ast_cli(fd, "   |  Last Reset:                                                                        | \n");
			}
			ast_cli(fd, "   |  %s                                                                | \n", LastAlarmReset[card - 1][0]);
			ast_cli(fd, "   |                                                                                     | \n");
			ast_cli(fd, "   +-------------------------------------------------------------------------------------+ \n\n");
			break;
		case DG_FXO_INTERFACE:
		case DG_FX_INTERFACE:
			ast_cli(fd, "   +------------------------------+ \n");
			ast_cli(fd, "   |                              | \n");
			
			for (p = 1; p <= dg_GetCardPortsCount(card); p++)
			{
				tmp = find_ch_port(dg_GetAbsolutePortNumber(card, p));
				if (tmp)
				{
					if (tmp->sig == DG_SIG_FXO)
						ast_cli(fd, "   |  Channel %03d:     %s   %03d  | \n", p, showalarmstatus_desc[tmp->alarmed], dgv_fxoNDTCount[card - 1][p - 1]);
					else						
					{
						switch (dgv_conf.consolelanguage)
						{
							case 0://English
								ast_cli(fd, "   |  Channel %03d:     ---------  | \n", p);
								break;
							case 1://Portuguese
								ast_cli(fd, "   |  Channel %03d:     ---------  | \n", p);
								break;
							default:
								ast_cli(fd, "   |  Channel %03d:     ---------  | \n", p);
						}
					}
				}
				else //if (tmp)
				{
					ast_cli(fd, "Cannot show alarm notifications for card %d, port %d when tmp is null\n", card, p);
					break;
				} //if (tmp)
			} //for (p = 1; p <= dg_GetCardPortsCount(card); p++)

			ast_cli(fd, "   |                              | \n");
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "   |  Last Reset:                 | \n");
					break;
				case 1://Portuguese
					ast_cli(fd, "   |  Ultimo Reset:               | \n");				
					break;
				default:
					ast_cli(fd, "   |  Last Reset:                 | \n");
					//0.9.8 break;
			}
			ast_cli(fd, "   |  %s         | \n", LastAlarmReset[card - 1][0]);
			ast_cli(fd, "   |                              | \n");
			ast_cli(fd, "   +------------------------------+ \n\n");							
			break;
	} //switch (dg_GetCardInterface(card))

	if (nDigitalCount >= 1)
	{
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
				ast_cli(fd, "       Alarms Descriptions: \n");
				ast_cli(fd, "        | \n");
				ast_cli(fd, "        |_ ALARM_LOSS     - Loss of Signal Status Indication. \n");			
				ast_cli(fd, "        |_ ALARM_AISS     - Alarm Indication Status Signal. \n");
				ast_cli(fd, "        |_ ALARM_AIS16S   - Alarm Indication Signal 16 Status. \n");
				ast_cli(fd, "        |_ ALARM_SYNC     - Basic Frame Alignment. \n");
				ast_cli(fd, "        |_ ALARM_MFSYNC   - Multiframe Alignment. \n");
				ast_cli(fd, "        |_ ALARM_RAIS     - Remote Alarm Indication Status. \n");						
				ast_cli(fd, "        |_ ALARM_CRC4SYNC - CRC-4 Synchronization. \n");
#ifdef USE_LIBPRI
					ast_cli(fd, "        |_ ALARM_LINKDOWN - ISDN Protocol Status Indication. \n");
#endif//#ifdef USE_LIBPRI
				ast_cli(fd, "        |_ ALARM_RSLIP    - Frame Slip. \n\n");			
				break;
			case 1://Portuguese
				ast_cli(fd, "       Descricoes dos Alarmes: \n");
				ast_cli(fd, "        | \n");			
				ast_cli(fd, "        |_ ALARM_LOSS     - Indicacao de Perda de Sinal. \n");			
				ast_cli(fd, "        |_ ALARM_AISS     - Status de Indicacao de Alarme. \n");
				ast_cli(fd, "        |_ ALARM_AIS16S   - Indicacao de Alarme no Canal 16. \n");
				ast_cli(fd, "        |_ ALARM_SYNC     - Perda de Sincronismo de Quadro. \n");
				ast_cli(fd, "        |_ ALARM_MFSYNC   - Perda de Sincronismo de Multiquadro. \n");
				ast_cli(fd, "        |_ ALARM_RAIS     - Indicacao de Alarme Remoto. \n");						
				ast_cli(fd, "        |_ ALARM_CRC4SYNC - Erro de CRC-4. \n");
#ifdef USE_LIBPRI
					ast_cli(fd, "        |_ ALARM_LINKDOWN - Indicacao do Status de Protocolo ISDN. \n");
#endif//#ifdef USE_LIBPRI
				ast_cli(fd, "        |_ ALARM_RSLIP    - Escorregamento. \n\n");		
				break;
			default:
				ast_cli(fd, "       Alarms Descriptions: \n");
				ast_cli(fd, "        | \n");
				ast_cli(fd, "        |_ ALARM_LOSS     - Loss of Signal Status Indication. \n");			
				ast_cli(fd, "        |_ ALARM_AISS     - Alarm Indication Status Signal. \n");
				ast_cli(fd, "        |_ ALARM_AIS16S   - Alarm Indication Signal 16 Status. \n");
				ast_cli(fd, "        |_ ALARM_SYNC     - Basic Frame Alignment. \n");
				ast_cli(fd, "        |_ ALARM_MFSYNC   - Multiframe Alignment. \n");
				ast_cli(fd, "        |_ ALARM_RAIS     - Remote Alarm Indication Status. \n");						
				ast_cli(fd, "        |_ ALARM_CRC4SYNC - CRC-4 Synchronization. \n");
#ifdef USE_LIBPRI
					ast_cli(fd, "        |_ ALARM_LINKDOWN - ISDN Protocol Status Indication. \n");
#endif//#ifdef USE_LIBPRI
				ast_cli(fd, "        |_ ALARM_RSLIP    - Frame Slip. \n\n");			
				//0.9.8 break;		
		} //switch (dgv_conf.consolelanguage)
	} //if (nDigitalCount >= 1)
	
	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_statistics(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_show_statistics(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	#define FORMAT_SHOW_STATISTICS "%8s %5s %-6.6s %-16.16s %-9.9s %13s %7s %-20.20s\n"

	struct dgv_pvt *tmp = NULL;
	char buftotal[80];
	char buflast[80];
	char textstatistics[30000] = "";
	char temptextstatistics[100] = "";
	short nForText;

	int group=0;

	char tmps[20] = "";
	char grp_tmps[20] = "";
	char lastansweredtime_tmps[20] = "";
	char totaltime_tmps[20] = "";
	char callsanswered_tmps[20] = "";
	char smssent_tmps[20] = "";
	char lastreset_tmps[20] = "";

	int hour=0, min=0, sec=0;

	struct dgv_pvt *start;
	int lin=20;
	int show=0;

	int res_total_time = 0;
	int res_calls_answered = 0;
	int res_sms_sent = 0;

	if (argc == 4)
	{
		if (!strncmp(argv[3], "all", 3))
		{
			ast_cli(fd, "\n<");

			for (nForText = 0; nForText <= sizeof(textstatistics) - 1; nForText++)
				textstatistics[nForText] = '\0';

			sprintf(textstatistics, "STATISTICS;PORTS_COUNT:%d;", dg_GetPortsCount());

			ast_dgv_lock(&iflock, 0);
			{
				start = iffirst;
				tmp = start;
				while (tmp)
				{
					ast_dgv_lock(&tmp->lock, tmp->port);
					{
						snprintf(buflast, sizeof(buflast), "%ld", (long) tmp->statistics.last_answered.tv_sec);
						snprintf(buftotal, sizeof(buftotal), "%ld", (long) tmp->statistics.total_time.tv_sec);

						sprintf(temptextstatistics, "%d:%d,%d,%s,%s,%d,%d,%s;", tmp->port, tmp->group, tmp->locked, buflast, buftotal, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset);
					}
					ast_dgv_unlock(&tmp->lock, tmp->port);

					strcat(textstatistics, temptextstatistics);
					tmp = tmp->next;
				} //while (tmp)
			}
			ast_dgv_unlock(&iflock, 0);

			textstatistics[strlen(textstatistics) - 1] = '\0';
			strcat(textstatistics, "> \n\n");
			ast_cli(fd, "%s", textstatistics);

			return RESULT_SUCCESS;
		}
		else
			if (!strncmp(argv[3], "json", 4))
			{
				ast_cli(fd, "\n");

				for (nForText = 0; nForText <= sizeof(textstatistics) - 1; nForText++)
					textstatistics[nForText] = '\0';

				ast_copy_string(textstatistics,
						        "{\"parameters\":{\"c\":\"channel\",\"g\":\"group\",\"k\":\"locked\",\"l\":\"lastansweredtime\",\"t\":\"totaltime\",\"a\":\"callsanswered\",\"s\":\"smssent\",\"r\":\"lastreset\"},\"values\":[",
						        sizeof(textstatistics));

				ast_dgv_lock(&iflock, 0);
				{
					start = iffirst;
					tmp = start;
					while (tmp)
					{
						ast_dgv_lock(&tmp->lock, tmp->port);
						{
							snprintf(buflast, sizeof(buflast), "%ld", (long) tmp->statistics.last_answered.tv_sec);
							snprintf(buftotal, sizeof(buftotal), "%ld", (long) tmp->statistics.total_time.tv_sec);

							sprintf(temptextstatistics, "{\"c\":\"%d\",\"g\":\"%d\",\"k\":\"%d\",\"l\":\"%s\",\"t\":\"%s\",\"a\":\"%d\",\"s\":\"%d\",\"r\":\"%s\"},", tmp->port, tmp->group, tmp->locked, buflast, buftotal, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset);
						}
						ast_dgv_unlock(&tmp->lock, tmp->port);

						strcat(textstatistics, temptextstatistics);
						tmp = tmp->next;
					} //while (tmp)
				}
				ast_dgv_unlock(&iflock, 0);

				textstatistics[strlen(textstatistics) - 1] = '\0';
				strcat(textstatistics, "]} \n\n");
				ast_cli(fd, "%s", textstatistics);

				return RESULT_SUCCESS;
			}
			else
			{
				//group selected
				group = atoi(argv[3]);
			}
	}
	ast_cli(fd, "Statistics of available DigiVoice Channels");

	if (group!=0)
		ast_cli(fd, " of group %d\n",group);
	else
		ast_cli(fd, "\n");

	ast_dgv_lock(&iflock, 0);
	{
		start = iffirst;
		tmp = start;
		while (tmp)
		{
			if (lin>=20)
			{
				ast_cli(fd, "\n");
			    ast_cli(fd, FORMAT_SHOW_STATISTICS, "Channel", "Group", "Locked", "LastAnsweredTime", "TotalTime", "CallsAnswered", "SMSSent", "LastReset");
				ast_cli(fd, "\n");
				lin=0;
			}

			if (tmp->port > 0) {
				snprintf(tmps, sizeof(tmps), "%d", tmp->port);
			} else
				ast_copy_string(tmps, "pseudo", sizeof(tmps));

			snprintf(grp_tmps, sizeof(grp_tmps), "%d", tmp->group);

			ast_dgv_lock(&tmp->lock, tmp->port);
			{
				hour = tmp->statistics.last_answered.tv_sec / 3600;
				min = (tmp->statistics.last_answered.tv_sec % 3600) / 60;
				sec = tmp->statistics.last_answered.tv_sec % 60;
				snprintf(lastansweredtime_tmps, sizeof(lastansweredtime_tmps), "%02d:%02d:%02d", hour, min, sec);

				hour = tmp->statistics.total_time.tv_sec / 3600;
				min = (tmp->statistics.total_time.tv_sec % 3600) / 60;
				sec = tmp->statistics.total_time.tv_sec % 60;
				snprintf(totaltime_tmps, sizeof(totaltime_tmps), "%02d:%02d:%02d", hour, min, sec);

				snprintf(callsanswered_tmps, sizeof(callsanswered_tmps), "%d", tmp->statistics.calls_answered);
				snprintf(smssent_tmps, sizeof(smssent_tmps), "%d", tmp->statistics.sms_sent);

				snprintf(lastreset_tmps, sizeof(lastreset_tmps), "%s", tmp->statistics.last_reset);

				if (group==0)
					show=1;
				else
					if (group == tmp->group)
						show=1;
					else
						show=0;

				if (show)
				{
					ast_cli(fd, FORMAT_SHOW_STATISTICS,
							tmps,
							grp_tmps,
							((tmp->locked == 1) ? "yes" : "no"),
							lastansweredtime_tmps,
							totaltime_tmps,
							callsanswered_tmps,
							smssent_tmps,
							lastreset_tmps);

					res_total_time = res_total_time + tmp->statistics.total_time.tv_sec;
					res_calls_answered = res_calls_answered + tmp->statistics.calls_answered;
					res_sms_sent = res_sms_sent + tmp->statistics.sms_sent;
					lin++;
				}
			}
			ast_dgv_unlock(&tmp->lock, tmp->port);

			tmp = tmp->next;
		} //while (tmp)
	}
	ast_dgv_unlock(&iflock, 0);

    hour = res_total_time / 3600;
    min = (res_total_time % 3600) / 60;
    sec = res_total_time % 60;
	ast_cli(fd, "\n Total Time Processed: %02d:%02d:%02d", hour, min, sec);
	ast_cli(fd, "\n Total Calls Answered: %d", res_calls_answered);
	ast_cli(fd, "\n Total SMS Sent: %d\n\n", res_sms_sent);

	return RESULT_SUCCESS;
}

//only reset the alarm status
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_alarms_reset(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_alarms_reset(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short card=0, span=0;
	short nForCard, nForSpan, ch;
	int res_sched;
	struct dgv_pvt *tmp;	
	
    if (argc != 5)
    {
        ast_cli(fd, "\n");
        
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;		
		}         	    	
        return RESULT_SHOWUSAGE;
    }
    
    if (!strncmp(argv[3], "all", 3))
    {
        if (!strncmp(argv[4], "all", 3))
        {
        	switch (dgv_conf.consolelanguage)
			{
				case 0://English		
					ast_cli(fd, "\n       Resetting Alarms Status (All Cards and All Spans/Channels)... \n\n");
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Resetando Status dos Alarmes (Todas Placas e Todos Spans/Canais)... \n\n");
					break;
				default:
					ast_cli(fd, "\n       Resetting Alarms Status (All Cards and All Spans/Channels)... \n\n");
					//0.9.8 break;		
			}
        	
            for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)
            {
				//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					struct ast_tm tm;
					struct timeval now = ast_tvnow();
					ast_localtime(&now, &tm, NULL);
					for (nForSpan = 0; nForSpan <= 3; nForSpan++)
						ast_strftime(LastAlarmReset[nForCard][nForSpan], sizeof(LastAlarmReset[nForCard][nForSpan]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					time_t t;
					struct tm tm;
					time(&t);
					ast_localtime(&t, &tm, NULL);//old localtime_r
					for (nForSpan = 0; nForSpan <= 3; nForSpan++)
						strftime(LastAlarmReset[nForCard][nForSpan], sizeof(LastAlarmReset[nForCard][nForSpan]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
            
            	switch (dg_GetCardInterface(nForCard + 1))
            	{
            		case DG_DIGITAL_INTERFACE:
    	                for (nForSpan = 0; nForSpan <= 1; nForSpan++)
    	                {
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_LOSS     = 0;
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_AISS     = 0;
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_AIS16S   = 0;
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_SYNC     = 0;
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_MFSYNC   = 0;
    	        			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_RAIS     = 0;  
    	          			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
								dgv_e1alarmsCount[nForCard][nForSpan].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI
    	                } //for (nForSpan = 0; nForSpan <= 1; nForSpan++)

            		case DG_FXO_INTERFACE:
            		case DG_FX_INTERFACE:		        
        		        for (ch = 0; ch <= 59 ; ch++) 
        		        {
        		        	dgv_fxoNDTCount[nForCard][ch] = 0;        		        	
							tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard + 1, ch + 1));
							if (tmp)
							{
								if (tmp->sig == DG_SIG_FXO || tmp->sig == DG_SIG_CB_FXO)
									tmp->alarmed = 0;
							
								if (tmp->alarmNDT_timeout_sched > -1) 
								{
									res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
							    	if (res_sched == -1)
							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

									tmp->alarmNDT_timeout_sched = -1;
								}
							}
        		        }
    	                
        		        break;
            		case DG_GSM_INTERFACE:
        		        for (ch = 0; ch <= 3 ; ch++)
        		        {
        		        	dgv_gsmDECount[nForCard][ch].nTotalDialError = 0;
        		        	dgv_gsmDECount[nForCard][ch].nTempDialError = 0;
        		        	dgv_gsmDECount[nForCard][ch].nFlagDialError = 0;
        		        }

            			break;				
            	} //switch (dg_GetCardInterface(nForCard + 1))
            } //for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)        	
        }
        else //if (!strncmp(argv[4], "all", 3))
        {
            span = atoi(argv[4]);

	    	switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Resetting Alarms Status (All Cards and Span/Channel %d)... \n\n", span);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Resetando Status dos Alarmes (Todas Placas and Span/Canal %d)... \n\n", span);
					break;
				default:
					ast_cli(fd, "\n       Resetting Alarms Status (All Cards and Span/Channel %d)... \n\n", span);
					//0.9.8 break;
			}

            for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)
            {
            	switch (dg_GetCardInterface(nForCard + 1))
            	{
            		case DG_DIGITAL_INTERFACE:
						if (span < 1 || span > 2)
						{
							ast_cli(fd, "\n");

							switch (dgv_conf.consolelanguage)
							{
								case 0://English
									ast_cli(fd, "       Error: This span doesn't exist... \n");
									break;
								case 1://Portuguese
									ast_cli(fd, "       Erro: Este span nao existe... \n");
									break;
								default:
									ast_cli(fd, "       Error: This span doesn't exist... \n");
									//0.9.8 break;
							}
							return RESULT_SHOWUSAGE;
						}

						dgv_e1alarmsCount[nForCard][span - 1].nALARM_LOSS     = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_AISS     = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_AIS16S   = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_SYNC     = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_MFSYNC   = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_RAIS     = 0;
						dgv_e1alarmsCount[nForCard][span - 1].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
							dgv_e1alarmsCount[nForCard][span - 1].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI

						//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
							struct ast_tm tm;
							struct timeval now = ast_tvnow();
							ast_localtime(&now, &tm, NULL);
							ast_strftime(LastAlarmReset[nForCard][span - 1], sizeof(LastAlarmReset[nForCard][span - 1]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
							time_t t;
							struct tm tm;
							time(&t);
							ast_localtime(&t, &tm, NULL);//old localtime_r
							strftime(LastAlarmReset[nForCard][span - 1], sizeof(LastAlarmReset[nForCard][span - 1]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

						for (ch = 0; ch <= 29 ; ch++)
						{
							if (span == 1)
							{
								dgv_fxoNDTCount[nForCard][ch] = 0;
								tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard + 1, ch + 1));
							}
							else
							{
								dgv_fxoNDTCount[nForCard][ch + 30] = 0;
								tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard + 1, ch + 31));
							}
							if (tmp)
							{
								if (tmp->sig == DG_SIG_CB_FXO)
									tmp->alarmed = 0;

								if (tmp->alarmNDT_timeout_sched > -1)
								{
									res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
									if (res_sched == -1)
										ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

									tmp->alarmNDT_timeout_sched = -1;
								}
							}
						}
						break;
            		case DG_FXO_INTERFACE:
            		case DG_FX_INTERFACE:
						if (span < 1 || span > 8)
						{
							ast_cli(fd, "\n");

							switch (dgv_conf.consolelanguage)
							{
								case 0://English
									ast_cli(fd, "       Error: This channel doesn't exist... \n");
									break;
								case 1://Portuguese
									ast_cli(fd, "       Erro: Este canal nao existe... \n");
									break;
								default:
									ast_cli(fd, "       Error: This channel doesn't exist... \n");
									//0.9.8 break;
							}
							return RESULT_SHOWUSAGE;
						}

						dgv_fxoNDTCount[nForCard][span - 1] = 0;
						tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard + 1, span));
						if (tmp)
						{
							if (tmp->sig == DG_SIG_FXO || tmp->sig == DG_SIG_CB_FXO)
								tmp->alarmed = 0;

							if (tmp->alarmNDT_timeout_sched > -1)
							{
								res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
								if (res_sched == -1)
									ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

								tmp->alarmNDT_timeout_sched = -1;
							}
						}
						break;
            		case DG_GSM_INTERFACE:
						if (span < 1 || span > 4)
						{
							ast_cli(fd, "\n");

							switch (dgv_conf.consolelanguage)
							{
								case 0://English
									ast_cli(fd, "       Error: This channel doesn't exist... \n");
									break;
								case 1://Portuguese
									ast_cli(fd, "       Erro: Este canal nao existe... \n");
									break;
								default:
									ast_cli(fd, "       Error: This channel doesn't exist... \n");
									//0.9.8 break;
							}
							return RESULT_SHOWUSAGE;
						}

						dgv_gsmDECount[nForCard][span - 1].nTotalDialError = 0;
						dgv_gsmDECount[nForCard][span - 1].nTempDialError = 0;
						dgv_gsmDECount[nForCard][span - 1].nFlagDialError = 0;
						break;
            	}
            } //for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)
        } //if (!strncmp(argv[4], "all", 3))
        
    	return RESULT_SUCCESS;        
    }//if (!strncmp(argv[3], "all", 3))

    card = atoi(argv[3]);
    
	switch (dg_GetCardType(card))
	{
		case VB0404GSM:
	    case VBE13030PCI:
		case VBE13060PCI:			    		
	    case VBE13060PCI_R:	    
		case VBE16060PCI:
		case VBE16060PCI_R:
		case VB3030PCIE:
		case VB6060PCIE:
		case DGV_1E1F:
		case DGV_2E1F:
		case VB0408PCI:
		case VB0408PCIE:
		case VB0404FX:
		case VB0404FX_R:			
			break;
		default:
			ast_cli(fd, "\n");
			
			switch (dgv_conf.consolelanguage)
			{
				case 0://English		
        			ast_cli(fd, "       Error: This card doesn't exist... \n");
					break;
				case 1://Portuguese
					ast_cli(fd, "       Erro: Esta placa nao existe... \n");
					break;
				default:
					ast_cli(fd, "       Error: This card doesn't exist... \n");
					//0.9.8 break;		
			}       	    	
			return RESULT_SHOWUSAGE;			
	} //switch (dg_GetCardType(card))
    
    if (!strncmp(argv[4], "all", 3))
    {
    	switch (dgv_conf.consolelanguage)
		{
			case 0://English		
				ast_cli(fd, "\n       Resetting Alarms Status (Card %d/All Spans/Channels)... \n\n", card);
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Resetando Status dos Alarmes (Placa %d/Todos Spans/Canais)... \n\n", card);
				break;
			default:
				ast_cli(fd, "\n       Resetting Alarms Status (Card %d/All Spans/Channels)... \n\n", card);
				//0.9.8 break;		
		}
	
		//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			struct ast_tm tm;
			struct timeval now = ast_tvnow();
			ast_localtime(&now, &tm, NULL);
		    for (nForSpan = 0; nForSpan <= 3; nForSpan++)
				ast_strftime(LastAlarmReset[card - 1][nForSpan], sizeof(LastAlarmReset[card - 1][nForSpan]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			time_t t;
			struct tm tm;
			time(&t);
			ast_localtime(&t, &tm, NULL);//old localtime_r
		    for (nForSpan = 0; nForSpan <= 3; nForSpan++)
				strftime(LastAlarmReset[card - 1][nForSpan], sizeof(LastAlarmReset[card - 1][nForSpan]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	    
    	switch (dg_GetCardInterface(card))
    	{
    		case DG_DIGITAL_INTERFACE:
                for (nForSpan = 0; nForSpan <= 1; nForSpan++)
                {
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_LOSS     = 0;
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_AISS     = 0;
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_AIS16S   = 0;
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_SYNC     = 0;
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_MFSYNC   = 0;
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_RAIS     = 0;  
        			dgv_e1alarmsCount[card - 1][nForSpan].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
						dgv_e1alarmsCount[card - 1][nForSpan].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI
                } //for (nForSpan = 0; nForSpan <= 1; nForSpan++)

    		case DG_FXO_INTERFACE:
    		case DG_FX_INTERFACE:		        
		        for (ch = 0; ch <= 59 ; ch++)
		        {
		        	dgv_fxoNDTCount[card - 1][ch] = 0;
					tmp = find_ch_port(dg_GetAbsolutePortNumber(card, ch + 1));
					if (tmp)
					{
						if (tmp->sig == DG_SIG_FXO || tmp->sig == DG_SIG_CB_FXO)
							tmp->alarmed = 0;
					
						if (tmp->alarmNDT_timeout_sched > -1) 
						{
							res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->alarmNDT_timeout_sched = -1;
						}						
					}
		        }		        
                
		        break;
    		case DG_GSM_INTERFACE:
		        for (ch = 0; ch <= 3 ; ch++)
		        {
		        	dgv_gsmDECount[card - 1][ch].nTotalDialError = 0;
		        	dgv_gsmDECount[card - 1][ch].nTempDialError = 0;
		        	dgv_gsmDECount[card - 1][ch].nFlagDialError = 0;
		        }

		        break;
    	} //switch (dg_GetCardInterface(card))
    }//if (!strncmp(argv[4], "all", 3))
    else
    {
    	span = atoi(argv[4]);
    	
    	switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "\n       Resetting Alarms Status (Card %d and Span/Channel %d)... \n\n", card, span);
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Resetando Status dos Alarmes (Placa %d e Span/Canal %d)... \n\n", card, span);
				break;
			default:
				ast_cli(fd, "\n       Resetting Alarms Status (Card %d and Span/Channel %d)... \n\n", card, span);
				//0.9.8 break;
		}

    	switch (dg_GetCardInterface(card))
    	{
    		case DG_DIGITAL_INTERFACE:
				if (span < 1 || span > 2)
				{
					ast_cli(fd, "\n");

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "       Error: This span doesn't exist... \n");
							break;
						case 1://Portuguese
							ast_cli(fd, "       Erro: Este span nao existe... \n");
							break;
						default:
							ast_cli(fd, "       Error: This span doesn't exist... \n");
							//0.9.8 break;
					}
					return RESULT_SHOWUSAGE;
				}

				dgv_e1alarmsCount[card - 1][span - 1].nALARM_LOSS     = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_AISS     = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_AIS16S   = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_SYNC     = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_MFSYNC   = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_RAIS     = 0;
				dgv_e1alarmsCount[card - 1][span - 1].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
					dgv_e1alarmsCount[card - 1][span - 1].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI

				//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					struct ast_tm tm;
					struct timeval now = ast_tvnow();
					ast_localtime(&now, &tm, NULL);
					ast_strftime(LastAlarmReset[card - 1][span - 1], sizeof(LastAlarmReset[card - 1][span - 1]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					time_t t;
					struct tm tm;
					time(&t);
					ast_localtime(&t, &tm, NULL);//old localtime_r
					strftime(LastAlarmReset[card - 1][span - 1], sizeof(LastAlarmReset[card - 1][span - 1]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

				for (ch = 0; ch <= 29 ; ch++)
				{
					if (span == 1)
					{
						dgv_fxoNDTCount[card - 1][ch] = 0;
						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, ch + 1));
					}
					else
					{
						dgv_fxoNDTCount[card - 1][ch + 30] = 0;
						tmp = find_ch_port(dg_GetAbsolutePortNumber(card, ch + 31));
					}
					if (tmp)
					{
						if (tmp->sig == DG_SIG_CB_FXO)
							tmp->alarmed = 0;

						if (tmp->alarmNDT_timeout_sched > -1)
						{
							res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
							if (res_sched == -1)
								ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

							tmp->alarmNDT_timeout_sched = -1;
						}
					}
				}
				break;
    		case DG_FXO_INTERFACE:
    		case DG_FX_INTERFACE:
				if (span < 1 || span > 8)
				{
					ast_cli(fd, "\n");

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "       Error: This channel doesn't exist... \n");
							break;
						case 1://Portuguese
							ast_cli(fd, "       Erro: Este canal nao existe... \n");
							break;
						default:
							ast_cli(fd, "       Error: This channel doesn't exist... \n");
							//0.9.8 break;
					}
					return RESULT_SHOWUSAGE;
				}

				dgv_fxoNDTCount[card - 1][span - 1] = 0;
				tmp = find_ch_port(dg_GetAbsolutePortNumber(card, span));
				if (tmp)
				{
					if (tmp->sig == DG_SIG_FXO || tmp->sig == DG_SIG_CB_FXO)
						tmp->alarmed = 0;

					if (tmp->alarmNDT_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, tmp->alarmNDT_timeout_sched);
						if (res_sched == -1)
							ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", tmp->port);

						tmp->alarmNDT_timeout_sched = -1;
					}
				}
				break;
    		case DG_GSM_INTERFACE:
				if (span < 1 || span > 4)
				{
					ast_cli(fd, "\n");

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "       Error: This channel doesn't exist... \n");
							break;
						case 1://Portuguese
							ast_cli(fd, "       Erro: Este canal nao existe... \n");
							break;
						default:
							ast_cli(fd, "       Error: This channel doesn't exist... \n");
							//0.9.8 break;
					}
					return RESULT_SHOWUSAGE;
				}

				dgv_gsmDECount[card - 1][span - 1].nTotalDialError = 0;
				dgv_gsmDECount[card - 1][span - 1].nTempDialError = 0;
				dgv_gsmDECount[card - 1][span - 1].nFlagDialError = 0;
				break;
    	}
    }
	
	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_statistics_reset(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_statistics_reset(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short nForChannel, ch;
	struct dgv_pvt *tmp;

	if (argc != 4)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;
		}
		return RESULT_SHOWUSAGE;
	}

	if (!strncmp(argv[3], "all", 3))
	{
		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "\n       Resetting statistics (All Channels)... \n\n");
				break;
			case 1://Portuguese
				ast_cli(fd, "\n       Resetando estatisticas (Todos Canais)... \n\n");
				break;
			default:
				ast_cli(fd, "\n       Resetting statistics (All Channels)... \n\n");
		}

		for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
		{
			tmp = find_ch_port(nForChannel);

			if (tmp)
			{
				ast_dgv_lock(&tmp->lock, tmp->port);
				{
					reset_statistics(tmp);
				}
				ast_dgv_unlock(&tmp->lock, tmp->port);

			}
			else //if (tmp)
				ast_cli(fd, "\n       Statistics of channel %d cannot be reseted\n", nForChannel);

		} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)
	}
	else //if (!strncmp(argv[3], "all", 3))
	{
		ch = atoi(argv[3]);

		tmp = find_ch_port(ch);

		if (tmp)
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Resetting statistics (Channel %d)... \n\n", tmp->port);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Resetando estatisticas (Canal %d)... \n\n", tmp->port);
					break;
				default:
					ast_cli(fd, "\n       Resetting statistics (Channel %d)... \n\n", tmp->port);
			}

			ast_dgv_lock(&tmp->lock, tmp->port);
			{
				reset_statistics(tmp);
			}
			ast_dgv_unlock(&tmp->lock, tmp->port);
		}
		else //if (tmp)
			ast_cli(fd, "\n       Statistics of channel %d cannot be reseted\n", ch);

	} //if (!strncmp(argv[3], "all", 3))

	return RESULT_SUCCESS;
}

/* debug using */
#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_drop(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_drop(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
    short port;
    short times=0;
    short txrx=0;  //0-tx 1 - rx
    struct dgv_pvt *tmp;


    if (argc != 5)
    {
        ast_cli(fd, "\n");    	    	
        ast_cli(fd, "Error: You passed only %d args \n", argc);    
        return RESULT_SHOWUSAGE;
    }


    if (!strncmp(argv[2],"tx",2))
        txrx=0;
    else
        txrx=1;

    port = atoi(argv[3]);
    times = atoi(argv[4]);

   	tmp = find_ch_port(port);
    
    if (tmp!=NULL)
    {
        ast_cli(fd, "Forcing dropping %s samples %d times\n", (txrx ? "rx":"tx"), times);
        if (txrx)
            tmp->force_drop_rx = times;
        else
            tmp->force_drop_tx = times;
    }

    return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_record(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_record(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short record_ch=0;
	char tmps[50] = "";
	int nRet;
	
    if (argc < 3)
    {
        ast_cli(fd, "\n");
        
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;		
		}         	    	
        return RESULT_SHOWUSAGE;
    }
	
	record_ch = atoi(argv[2]);

	//dg_SetRecordFormat(record_ch, ffWaveALaw);
	snprintf(tmps, sizeof(tmps), "/var/log/voicerlib/recChanDGV%d.wav", record_ch);

	if (dg_IsRecording(record_ch))
	{
		nRet = dg_StopRecordFile(record_ch);
		ast_cli(fd, "  dgv_record: Stop recording file %s (Result: %d).\n", tmps, nRet);
	}
	else	
	{
		nRet = dg_RecordFile(record_ch, tmps, "");
		ast_cli(fd, "  dgv_record: Start recording on port %d (Result: %d)...\n", record_ch, nRet);
	}

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_test_lock(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_test_lock(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	#define FORMAT_FIND_LOCK "%8s %7s %7s %9s %6s %12s"
	
	struct dgv_pvt *tmp = NULL;
	struct dgv_pvt *start;
	char tmps[20] = "";
	char grp_tmps[20] = "";
	char rsrvd_tmps[20] = "";
	char alarmed_tmps[20] = "";
	char locked_tmps[20] = "";

	if (argc < 3)
    {
        ast_cli(fd, "\n");
        
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				//0.9.8 break;		
		}         	    	
        return RESULT_SHOWUSAGE;
    }

    if (!strncmp(argv[2], "lock", 4))
    {
		ast_dgv_lock(&iflock, 0);
		{
			start = iffirst;
			tmp = start;
	
			ast_cli(fd, "\n");
			ast_cli(fd, FORMAT_FIND_LOCK, "Channel", "Group", "Rsrvd", "Alarmed", "Locked", "Lock Test\n");
			ast_cli(fd, "\n");
	
			while (tmp)
			{
				snprintf(tmps, sizeof(tmps), "%d", tmp->port);	
				snprintf(grp_tmps, sizeof(grp_tmps), "%d", tmp->group);			
				snprintf(rsrvd_tmps, sizeof(rsrvd_tmps), "%d", tmp->reserved);
				snprintf(alarmed_tmps, sizeof(alarmed_tmps), "%d", tmp->alarmed);
				snprintf(locked_tmps, sizeof(locked_tmps), "%d", tmp->locked);
	
				ast_cli(fd, FORMAT_FIND_LOCK,
						tmps,
						grp_tmps,
						rsrvd_tmps,
						alarmed_tmps,
						locked_tmps,
						"trying to ");
						
				ast_cli(fd, "LOCK...");
				ast_dgv_lock(&tmp->lock, tmp->port);
				{
					ast_cli(fd, "LOCKED...");
				}
				ast_dgv_unlock(&tmp->lock, tmp->port);
				
				ast_cli(fd, "UNLOCKED\n");
	
				tmp = tmp->next;
			} //while (tmp)
			ast_cli(fd, "\n");
		}
		ast_dgv_unlock(&iflock, 0);
    }
    else
    	return RESULT_SHOWUSAGE;

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_send_generic_cmd(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_send_generic_cmd(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
    int ch;
	char cmdtosend[300];
	short param1, param2, param3, param4, param5, param6;
	struct dgv_pvt *tmp;

	if (argc < 5)
    {
        ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}
        return RESULT_SHOWUSAGE;
    }

	if (strlen(argv[4]))
	{
		ch = atoi(argv[3]);

		tmp = find_ch_port(ch);

		if (tmp)
		{
			if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
			{
			    if (!strncmp(argv[4], "AT", 2))
			    {
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       Sending an AT Command (Channel %d)... \n\n", tmp->port);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Enviando um Comando AT (Canal %d)... \n\n", tmp->port);
							break;
						default:
							ast_cli(fd, "\n       Sending an AT Command (Channel %d)... \n\n", tmp->port);
					}

			    	ast_copy_string(cmdtosend, argv[4], sizeof(cmdtosend));
			    	dg_GSMSendCommand(ch, cmdtosend);
			    }
			    else
			    {
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       Sending a Generic Command (Channel %d)... \n\n", tmp->port);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Enviando um Comando Generico (Canal %d)... \n\n", tmp->port);
							break;
						default:
							ast_cli(fd, "\n       Sending a Generic Command (Channel %d)... \n\n", tmp->port);
					}

			    	param1 = atoi(argv[4]);
			    	param2 = atoi(argv[5]);
			    	param3 = atoi(argv[6]);
			    	param4 = atoi(argv[7]);
			    	param5 = atoi(argv[8]);
			    	param6 = atoi(argv[9]);
			    	dg_send_command(dg_GetCardNumber(ch), param1, param2, param3, param4, param5, param6);
			    }
			}
			else //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Sending a Generic Command (Channel %d)... \n\n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Enviando um Comando Generico (Canal %d)... \n\n", tmp->port);
						break;
					default:
						ast_cli(fd, "\n       Sending a Generic Command (Channel %d)... \n\n", tmp->port);
				}

		    	param1 = atoi(argv[4]);
		    	param2 = atoi(argv[5]);
		    	param3 = atoi(argv[6]);
		    	param4 = atoi(argv[7]);
		    	param5 = atoi(argv[8]);
		    	param6 = atoi(argv[9]);
		    	dg_send_command(dg_GetCardNumber(ch), param1, param2, param3, param4, param5, param6);
			} //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
		}
		else //if (tmp)
			ast_cli(fd, "\n       Command cannot be send on port %d\n", ch);
    }
    else
    	return RESULT_SHOWUSAGE;

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_send_sms(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_send_sms(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
    char resource[20];
	char send_to[40];
	char text_message[200];
	char lock_console[5];
	char custom_id[40];
	
	char texttosend[305];
	
	int ret;

	if (argc < 6)
    {
        ast_cli(fd, "\n");
        
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
        		ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}         	    	
        return RESULT_SHOWUSAGE;
    }

	if (strlen(argv[3]) && strlen(argv[4]) && strlen(argv[5]))
	{
	    if (option_verbose >= 3)
	    {
	    	if (argc == 7)
	    		ast_verbose(VERBOSE_PREFIX_3 "dgv_send_sms, a text was received: '%s' '%s' '%s' '%s'\n", argv[3], argv[4], argv[5], argv[6]);
	    	else
	    		if (argc == 8)
	    			ast_verbose(VERBOSE_PREFIX_3 "dgv_send_sms, a text was received: '%s' '%s' '%s' '%s' '%s'\n", argv[3], argv[4], argv[5], argv[6], argv[7]);
	    		else
	    			ast_verbose(VERBOSE_PREFIX_3 "dgv_send_sms, a text was received: '%s' '%s' '%s'\n", argv[3], argv[4], argv[5]);
	    }
	    
	    ast_copy_string(resource    , argv[3], sizeof(resource));
	    ast_copy_string(send_to     , argv[4], sizeof(send_to));
	    ast_copy_string(text_message, argv[5], sizeof(text_message));
	    if (argc == 7 || argc == 8)
	    	if (strlen(argv[6]))
				ast_copy_string(lock_console, argv[6], sizeof(lock_console));
	    if (argc == 8)
	    	if (strlen(argv[7]))
				ast_copy_string(custom_id, argv[7], sizeof(custom_id));

		ast_copy_string(texttosend, sms_header, sizeof(texttosend));
		strcat(texttosend, generic_sep);
		strcat(texttosend, resource);
		strcat(texttosend, generic_sep);
		strcat(texttosend, send_to);
		strcat(texttosend, generic_sep);
		strcat(texttosend, text_message);
		if (argc == 7 || argc == 8)
			if (strlen(argv[6]))
			{
				strcat(texttosend, generic_sep);
				strcat(texttosend, lock_console);
			}
		if (argc == 8)
			if (strlen(argv[7]))
			{
				strcat(texttosend, generic_sep);
				strcat(texttosend, custom_id);
			}
		strcat(texttosend, "\0");

		if (option_debug==100)
			ast_log(LOG_NOTICE, "Calling send_sms_message...\n");
		
		ret = send_sms_message(NULL, fd, texttosend);
		if (ret == -1)
			return RESULT_SHOWUSAGE;
    }
    else
    	return RESULT_SHOWUSAGE;

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_lock_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_lock_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short ch;
	short group=0;
	struct dgv_pvt *tmp;
	struct dgv_pvt *start;

	if (argc != 4)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}
		return RESULT_SHOWUSAGE;
	}

	if (!strncmp(argv[2], "channel", 7))
	{
		ch = atoi(argv[3]);

		tmp = find_ch_port(ch);

		if (tmp)
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Locking channel (Channel %d)... \n\n", tmp->port);
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Bloqueando canal (Canal %d)... \n\n", tmp->port);
					break;
				default:
					ast_cli(fd, "\n       Locking channel (Channel %d)... \n\n", tmp->port);
			}

			ast_dgv_lock(&tmp->lock, tmp->port);
			{
				tmp->locked = 1;

				if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
					ast_log(LOG_WARNING, "dgv_lock_channel (%d): Can not write to Statistics file\n", tmp->port);
			}
			ast_dgv_unlock(&tmp->lock, tmp->port);
		}
		else //if (tmp)
			ast_cli(fd, "\n       Channel %d cannot be locked\n", ch);
	}
	else //if (!strncmp(argv[3], "channel", 7))
	{
		if (!strncmp(argv[2], "group", 5))
		{
			group = atoi(argv[3]);

			if (group > 0)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Locking group (Group %d)... \n\n", group);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Bloqueando grupo (Grupo %d)... \n\n", group);
						break;
					default:
						ast_cli(fd, "\n       Locking group (Group %d)... \n\n", group);
				}

				ast_dgv_lock(&iflock, 0);
				{
					start = iffirst;
					tmp = start;
					while (tmp)
					{
						if (group == tmp->group)
						{
							ast_dgv_lock(&tmp->lock, tmp->port);
							{
								tmp->locked = 1;

								if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
									ast_log(LOG_WARNING, "dgv_lock_channel (%d): Can not write to Statistics file\n", tmp->port);
							}
							ast_dgv_unlock(&tmp->lock, tmp->port);
						}

						tmp = tmp->next;
					} //while (tmp)
				}
				ast_dgv_unlock(&iflock, 0);
			}
			else //if (group > 0)
				ast_cli(fd, "\n       Group %d cannot be locked\n", group);
		}
		else //if (!strncmp(argv[3], "group", 5))
			return RESULT_SHOWUSAGE;
	} //if (!strncmp(argv[3], "channel", 7))

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_unlock_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_unlock_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short nForChannel, ch;
	short group=0;
	struct dgv_pvt *tmp;
	struct dgv_pvt *start;

	if (argc != 4)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}
		return RESULT_SHOWUSAGE;
	}

	if (!strncmp(argv[2], "channel", 7))
	{
		if (!strncmp(argv[3], "all", 3))
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Unlocking channel (All Channels)... \n\n");
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Desbloqueando canal (Todos Canais)... \n\n");
					break;
				default:
					ast_cli(fd, "\n       Unlocking channel (All Channels)... \n\n");
			}

			for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
			{
				tmp = find_ch_port(nForChannel);

				if (tmp)
				{
					ast_dgv_lock(&tmp->lock, tmp->port);
					{
						tmp->locked = 0;

						if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
							ast_log(LOG_WARNING, "dgv_unlock_channel (%d): Can not write to Statistics file\n", tmp->port);
					}
					ast_dgv_unlock(&tmp->lock, tmp->port);

				}
				else //if (tmp)
					ast_cli(fd, "\n       Channel %d cannot be unlocked\n", nForChannel);

			} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)
		}
		else //if (!strncmp(argv[3], "all", 3))
		{
			ch = atoi(argv[3]);

			tmp = find_ch_port(ch);

			if (tmp)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Unlocking channel (Channel %d)... \n\n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Desbloqueando canal (Canal %d)... \n\n", tmp->port);
						break;
					default:
						ast_cli(fd, "\n       Unlocking channel (Channel %d)... \n\n", tmp->port);
				}

				ast_dgv_lock(&tmp->lock, tmp->port);
				{
					tmp->locked = 0;

					if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
						ast_log(LOG_WARNING, "dgv_unlock_channel (%d): Can not write to Statistics file\n", tmp->port);
				}
				ast_dgv_unlock(&tmp->lock, tmp->port);
			}
			else //if (tmp)
				ast_cli(fd, "\n       Channel %d cannot be unlocked\n", ch);

		} //if (!strncmp(argv[3], "all", 3))
	}
	else //if (!strncmp(argv[3], "channel", 7))
	{
		if (!strncmp(argv[2], "group", 5))
		{
			if (!strncmp(argv[3], "all", 3))
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Unlocking group (All Groups)... \n\n");
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Desbloqueando grupo (Todos Grupos)... \n\n");
						break;
					default:
						ast_cli(fd, "\n       Unlocking group (All Groups)... \n\n");
				}

				for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
				{
					tmp = find_ch_port(nForChannel);

					if (tmp)
					{
						ast_dgv_lock(&tmp->lock, tmp->port);
						{
							tmp->locked = 0;

							if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
								ast_log(LOG_WARNING, "dgv_unlock_channel (%d): Can not write to Statistics file\n", tmp->port);
						}
						ast_dgv_unlock(&tmp->lock, tmp->port);

					}
					else //if (tmp)
						ast_cli(fd, "\n       Group %d cannot be unlocked\n", nForChannel);
				} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)
			}
			else //if (!strncmp(argv[3], "all", 3))
			{
				group = atoi(argv[3]);

				if (group > 0)
				{
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       Unlocking group (Group %d)... \n\n", group);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Desbloqueando grupo (Grupo %d)... \n\n", group);
							break;
						default:
							ast_cli(fd, "\n       Unlocking group (Group %d)... \n\n", group);
					}

					ast_dgv_lock(&iflock, 0);
					{
						start = iffirst;
						tmp = start;
						while (tmp)
						{
							if (group == tmp->group)
							{
								ast_dgv_lock(&tmp->lock, tmp->port);
								{
									tmp->locked = 0;

									if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
										ast_log(LOG_WARNING, "dgv_unlock_channel (%d): Can not write to Statistics file\n", tmp->port);
								}
								ast_dgv_unlock(&tmp->lock, tmp->port);
							}

							tmp = tmp->next;
						} //while (tmp)
					}
					ast_dgv_unlock(&iflock, 0);
				}
				else //if (group > 0)
					ast_cli(fd, "\n       Group %d cannot be unlocked\n", group);
			} //if (!strncmp(argv[3], "all", 3))
		}
		else //if (!strncmp(argv[3], "group", 5))
			return RESULT_SHOWUSAGE;
	} //if (!strncmp(argv[3], "channel", 7))

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_get_unread_sms_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_get_unread_sms_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short nForChannel, ch;
	short group=0;
	struct dgv_pvt *tmp;
	struct dgv_pvt *start;
	int ret;
	int res_sched;

	if (argc != 6)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}
		return RESULT_SHOWUSAGE;
	}

	if (!strncmp(argv[4], "channel", 7))
	{
		if (!strncmp(argv[5], "all", 3))
		{
			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "\n       Getting unread SMS messages from channel (All Channels)... \n\n");
					break;
				case 1://Portuguese
					ast_cli(fd, "\n       Solicitando mensagens SMS nao lidas do canal (Todos Canais)... \n\n");
					break;
				default:
					ast_cli(fd, "\n       Getting unread SMS messages from channel (All Channels)... \n\n");
			}

			for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
			{
				tmp = find_ch_port(nForChannel);

				if (tmp)
				{
					if (tmp->sig == DG_SIG_GSM)
					{
						ast_dgv_lock(&tmp->lock, tmp->port);
						{
							if (tmp->call_state != DG_AST_CALLING && tmp->sms_reserved == 0 && tmp->alarmed == 0 && tmp->reserved == 0)
							{
								tmp->reserved = 2;

								ret = dg_GSMClearAllSMS(tmp->port, 2); /* request all sim card messages again */
								if (ret != EXIT_SUCCESS)
									ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment\n\n", tmp->port);
							}
							else
							{
								ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment, it will be scheduled to %d ms\n\n", tmp->port, gsmclearsmstimeout);

								if (tmp->gsmclearsms_timeout_sched > -1)
								{
									if (option_debug==100)
										ast_log(LOG_DEBUG, "dgv_get_unread_sms_channel (%d) - deleting gsmclearsms schedule\n", tmp->port);

									res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
									if (res_sched == -1)
										if (option_debug==100)
											ast_log(LOG_DEBUG, "Failed to remove scheduler entry for port %d\n", tmp->port);

									tmp->gsmclearsms_timeout_sched = -1;
								}

								tmp->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, tmp);
							}
						}
						ast_dgv_unlock(&tmp->lock, tmp->port);
					}
				}
				else //if (tmp)
					ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed\n\n", nForChannel);

			} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)
		}
		else //if (!strncmp(argv[3], "all", 3))
		{
			ch = atoi(argv[5]);

			tmp = find_ch_port(ch);

			if (tmp)
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Getting unread SMS messages from channel (Channel %d)... \n\n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Solicitando mensagens SMS nao lidas do canal (Canal %d)... \n\n", tmp->port);
						break;
					default:
						ast_cli(fd, "\n       Getting unread SMS messages from channel (Channel %d)... \n\n", tmp->port);
				}

				if (tmp->sig == DG_SIG_GSM)
				{
					ast_dgv_lock(&tmp->lock, tmp->port);
					{
						if (tmp->call_state != DG_AST_CALLING && tmp->sms_reserved == 0 && tmp->alarmed == 0 && tmp->reserved == 0)
						{
							tmp->reserved = 2;

							ret = dg_GSMClearAllSMS(tmp->port, 2); /* request all sim card messages again */
							if (ret != EXIT_SUCCESS)
								ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment\n\n", tmp->port);
						}
						else
						{
							ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment, it will be scheduled to %d ms\n\n", tmp->port, gsmclearsmstimeout);

							if (tmp->gsmclearsms_timeout_sched > -1)
							{
								if (option_debug==100)
									ast_log(LOG_DEBUG, "dgv_get_unread_sms_channel (%d) - deleting gsmclearsms schedule\n", tmp->port);

								res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
								if (res_sched == -1)
									if (option_debug==100)
										ast_log(LOG_DEBUG, "Failed to remove scheduler entry for port %d\n", tmp->port);

								tmp->gsmclearsms_timeout_sched = -1;
							}

							tmp->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, tmp);
						}
					}
					ast_dgv_unlock(&tmp->lock, tmp->port);
				}
				else
					ast_cli(fd, "       Error: Port %d is not a GSM port!\n\n", tmp->port);
			}
			else //if (tmp)
				ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed\n\n", ch);

		} //if (!strncmp(argv[3], "all", 3))
	}
	else //if (!strncmp(argv[3], "channel", 7))
	{
		if (!strncmp(argv[4], "group", 5))
		{
			if (!strncmp(argv[5], "all", 3))
			{
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Getting unread SMS messages from group (All Groups)... \n\n");
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Solicitando mensagens SMS nao lidas do grupo (Todos Grupos)... \n\n");
						break;
					default:
						ast_cli(fd, "\n       Getting unread SMS messages from group (All Groups)... \n\n");
				}

				for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
				{
					tmp = find_ch_port(nForChannel);

					if (tmp)
					{
						if (tmp->sig == DG_SIG_GSM)
						{
							ast_dgv_lock(&tmp->lock, tmp->port);
							{
								if (tmp->call_state != DG_AST_CALLING && tmp->sms_reserved == 0 && tmp->alarmed == 0 && tmp->reserved == 0)
								{
									tmp->reserved = 2;

									ret = dg_GSMClearAllSMS(tmp->port, 2); /* request all sim card messages again */
									if (ret != EXIT_SUCCESS)
										ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment\n\n", tmp->port);
								}
								else
								{
									ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment, it will be scheduled to %d ms\n\n", tmp->port, gsmclearsmstimeout);

									if (tmp->gsmclearsms_timeout_sched > -1)
									{
										if (option_debug==100)
											ast_log(LOG_DEBUG, "dgv_get_unread_sms_channel (%d) - deleting gsmclearsms schedule\n", tmp->port);

										res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
										if (res_sched == -1)
											if (option_debug==100)
												ast_log(LOG_DEBUG, "Failed to remove scheduler entry for port %d\n", tmp->port);

										tmp->gsmclearsms_timeout_sched = -1;
									}

									tmp->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, tmp);
								}
							}
							ast_dgv_unlock(&tmp->lock, tmp->port);
						}
					}
					else //if (tmp)
						ast_cli(fd, "\n       SMS messages from Group %d cannot be readed\n\n", nForChannel);
				} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount; nForChannel++)
			}
			else //if (!strncmp(argv[3], "all", 3))
			{
				group = atoi(argv[5]);

				if (group > 0)
				{
					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       Getting unread SMS messages from group (Group %d)... \n\n", group);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Solicitando mensagens SMS nao lidas do grupo (Grupo %d)... \n\n", group);
							break;
						default:
							ast_cli(fd, "\n       Getting unread SMS messages from group (Group %d)... \n\n", group);
					}

					ast_dgv_lock(&iflock, 0);
					{
						start = iffirst;
						tmp = start;
						while (tmp)
						{
							if (group == tmp->group)
							{
								if (tmp->sig == DG_SIG_GSM)
								{
									ast_dgv_lock(&tmp->lock, tmp->port);
									{
										if (tmp->call_state != DG_AST_CALLING && tmp->sms_reserved == 0 && tmp->alarmed == 0 && tmp->reserved == 0)
										{
											tmp->reserved = 2;

											ret = dg_GSMClearAllSMS(tmp->port, 2); /* request all sim card messages again */
											if (ret != EXIT_SUCCESS)
												ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment\n\n", tmp->port);
										}
										else
										{
											ast_cli(fd, "\n       SMS messages from Channel %d cannot be readed at this moment, it will be scheduled to %d ms\n\n", tmp->port, gsmclearsmstimeout);

											if (tmp->gsmclearsms_timeout_sched > -1)
											{
												if (option_debug==100)
													ast_log(LOG_DEBUG, "dgv_get_unread_sms_channel (%d) - deleting gsmclearsms schedule\n", tmp->port);

												res_sched = ast_sched_del(sched, tmp->gsmclearsms_timeout_sched);
												if (res_sched == -1)
													if (option_debug==100)
														ast_log(LOG_DEBUG, "Failed to remove scheduler entry for port %d\n", tmp->port);

												tmp->gsmclearsms_timeout_sched = -1;
											}

											tmp->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, tmp);
										}
									}
									ast_dgv_unlock(&tmp->lock, tmp->port);
								}
								else
									ast_cli(fd, "       Error: Port %d is not a GSM port!\n\n", tmp->port);
							}

							tmp = tmp->next;
						} //while (tmp)
					}
					ast_dgv_unlock(&iflock, 0);
				}
				else //if (group > 0)
					ast_cli(fd, "\n       SMS messages from Group %d cannot be readed\n\n", group);
			} //if (!strncmp(argv[3], "all", 3))
		}
		else //if (!strncmp(argv[3], "group", 5))
			return RESULT_SHOWUSAGE;
	} //if (!strncmp(argv[3], "channel", 7))

	return RESULT_SUCCESS;
}

#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_csca_channel(int fd, int argc, const char * const *argv)
#else//#ifdef CC_AST_HAS_VERSION_1_8
	static int dgv_gsm_csca_channel(int fd, int argc, char **argv)
#endif//#ifdef CC_AST_HAS_VERSION_1_8
{
	short ch;
	struct dgv_pvt *tmp;
	int res = -1;
	char sca[30] = "";
	char szGSMCMD[10] = "";

	if (argc != 5 && argc != 6)
	{
		ast_cli(fd, "\n");

		switch (dgv_conf.consolelanguage)
		{
			case 0://English
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
				break;
			case 1://Portuguese
				ast_cli(fd, "       Erro: Voce passou somente %d parametros. \n", argc);
				break;
			default:
				ast_cli(fd, "       Error: You passed only %d args. \n", argc);
		}
		return RESULT_SHOWUSAGE;
	}

	ch = atoi(argv[4]);

	tmp = find_ch_port(ch);

	if (tmp)
	{
		if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
		{
			if (!strncmp(argv[3], "read", 4))
			{
				res = dg_GSMSendCommand(tmp->port, DEFAULT_GSM_READ_SMSC_ADDRESS);

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						ast_cli(fd, "\n       Reading SMSC address (Channel %d)... \n\n", tmp->port);
						break;
					case 1://Portuguese
						ast_cli(fd, "\n       Lendo Endereco SMSC (Canal %d)... \n\n", tmp->port);
						break;
					default:
						ast_cli(fd, "\n       Reading SMSC address (Channel %d)... \n\n", tmp->port);
				}
			}
			else
			{
				if (!strncmp(argv[3], "write", 5))
				{
					ast_copy_string(sca, argv[5], sizeof(sca));
					sprintf(szGSMCMD, "%s%s", DEFAULT_GSM_WRITE_SMSC_ADDRESS, sca);
					res = dg_GSMSendCommand(tmp->port, szGSMCMD);

					switch (dgv_conf.consolelanguage)
					{
						case 0://English
							ast_cli(fd, "\n       Writing SMSC address %s (Channel %d)... \n\n", argv[5], tmp->port);
							break;
						case 1://Portuguese
							ast_cli(fd, "\n       Escrevendo Endereco SMSC %s (Canal %d)... \n\n", argv[5], tmp->port);
							break;
						default:
							ast_cli(fd, "\n       Writing SMSC address %s (Channel %d)... \n\n", argv[5], tmp->port);
					}
				}
			}
		}
		else //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
		{
			ast_cli(fd, "\n");

			switch (dgv_conf.consolelanguage)
			{
				case 0://English
					ast_cli(fd, "       Error: Channel %d isn't a GSM Channel... \n", tmp->port);
					break;
				case 1://Portuguese
					ast_cli(fd, "       Erro: Canal %d nao eh um Canal GSM... \n", tmp->port);
					break;
				default:
					ast_cli(fd, "       Error: Channel %d isn't a GSM Channel... \n", tmp->port);
			}

			return RESULT_SHOWUSAGE;
		} //if (dg_GetPortInterface(tmp->port) == DG_GSM_INTERFACE)
	}
	else
	{
		ast_cli(fd, "\n       CSCA Read/Write error for Channel %d\n", ch);
		return RESULT_SHOWUSAGE;
	}

    if (res != 0)
    {
		ast_cli(fd, "\n       CSCA Read/Write error for Channel %d (res %d)\n", ch, res);
		return RESULT_SHOWUSAGE;
    }

	return RESULT_SUCCESS;
}

//-------------------------------------------------------------------------------------------
// Console Commands Help
//-------------------------------------------------------------------------------------------
static char show_channels_usage_en[] =
	"\n"
	"       Shows a list of available dgv channels. \n       If group is specified, show channels of this group. \n"
	"        |_ Usage: dgv show channels [group/all/json] \n"
	"\n";

static char show_channels_usage_br[] =
	"\n"
	"       Mostra a lista de canais disponiveis. \n       Caso especifique o grupo, serao mostrados os canais deste grupo. \n"
	"        |_ Uso: dgv show channels [group/all/json] \n"
	"\n";

static char show_cards_usage_en[] =
	"\n"
	"       Shows a list of available dgv cards. \n"
	"        |_ Usage: dgv show cards [all/json] \n"
	"\n";

static char show_cards_usage_br[] =
	"\n"
	"       Mostra a lista de placas disponiveis. \n"
	"        |_ Usage: dgv show cards [all/json] \n"
	"\n";

static char show_channel_usage_en[] =
	"\n"
	"       Detailed information about a given channel. \n"
	"        |_ Usage: dgv show channel <channel> \n"
	"\n";
	
static char show_channel_usage_br[] =
	"\n"
	"       Informacoes detalhadas de um determinado canal. \n"	
	"        |_ Uso: dgv show channel <channel> \n"
	"\n";
	
/*static char reset_usage_en[] =
	"\n"
	"       Resets all DigiVoice channels, hangup and restart all controls. \n"
	"        |_ Usage: dgv reset \n"	
	"\n";*/
	
/*static char reset_usage_br[] =
	"\n"
	"       Reseta todos os canais DigiVoice, desliga e restarta todos os controles. \n"
	"        |_ Uso: dgv reset \n"	
	"\n";*/

static char gsm_reset_usage_en[] =
	"\n"
	"       Resets a specific DigiVoice GSM channel, hangup and restart the GSM control. \n"
	"        |_ Usage: dgv gsm reset <channel/all> \n"
	"\n";

static char gsm_reset_usage_br[] =
	"\n"
	"       Reseta um canal DigiVoice GSM especifico, desliga e restarta o controle GSM. \n"
	"        |_ Uso: dgv gsm reset <channel/all> \n"
	"\n";
	
static char show_version_usage_en[] =
	"\n"
	"       Shows chan_dgv current version. \n"
	"        |_ Usage: dgv show version \n"	
	"\n";
	
static char show_version_usage_br[] =
	"\n"
	"       Mostra a versao corrente do chan_dgv. \n"
	"        |_ Uso: dgv show version \n"	
	"\n";
	
static char show_alarms_usage_en[] =
	"\n"
	"       Shows alarms from specific E1/GSM/FXO Card installed. \n"
	"        |_ Usage: dgv show alarms <card/all/json> \n"
	"\n";
	
static char show_alarms_usage_br[] =
	"\n"	
	"       Mostra alarmes de uma especifica Placa E1/GSM/FXO instalada. \n"
	"        |_ Uso: dgv show alarms <card/all/json> \n"
	"\n";

static char show_statistics_usage_en[] =
	"\n"
	"       Shows statistics of DigiVoice channels. \n       If group is specified, show statistics of this group. \n"
	"        |_ Usage: dgv show statistics [group/all/json] \n"
	"\n";

static char show_statistics_usage_br[] =
	"\n"
	"       Mostra estatisticas de uma lista de canais. \n       Caso especifique o grupo, serao mostradas as estatisticas deste grupo. \n"
	"        |_ Uso: dgv show statistics [group/all/json] \n"
	"\n";

static char reset_alarms_usage_en[] =
	"\n"
	"       Reset alarms from specific E1/FXO/GSM Card installed. \n"
	"        |_ Usage: dgv alarms reset <card/all> <span/channel/all> \n"
	"\n";
	
static char reset_alarms_usage_br[] =
	"\n"	
	"       Reseta alarmes de uma especifica Placa E1/FXO/GSM instalada. \n"
	"        |_ Uso: dgv alarms reset <card/all> <span/channel/all> \n"
	"\n";

static char reset_statistics_usage_en[] =
	"\n"
	"       Reset statistics from specific channel. \n"
	"        |_ Usage: dgv statistics reset <channel/all> \n"
	"\n";

static char reset_statistics_usage_br[] =
	"\n"
	"       Reseta estatisticas de um especifico canal. \n"
	"        |_ Uso: dgv statistics reset <channel/all> \n"
	"\n";

static char send_sms_usage_en[] =
	"\n"	
	"       Send a SMS message using a GSM channel installed. \n"
	"        |_ Usage: dgv send sms <resource> <send_to> <text_message> [lock_console] [custom_id] \n"
	"\n";

static char send_sms_usage_br[] =
	"\n"	
	"       Envia uma mensagem SMS utilizando um canal GSM instalado. \n"
	"        |_ Uso: dgv send sms <resource> <send_to> <text_message> [lock_console] [custom_id] \n"
	"\n";

static char lock_channel_usage_en[] =
	"\n"
	"       Lock a specific channel. \n"
	"        |_ Usage: dgv lock channel <channel> \n"
	"\n";

static char lock_channel_usage_br[] =
	"\n"
	"       Bloqueia um canal especifico. \n"
	"        |_ Uso: dgv lock channel <channel> \n"
	"\n";

static char unlock_channel_usage_en[] =
	"\n"
	"       Unlock a specific channel. \n"
	"        |_ Usage: dgv unlock channel <channel/all> \n"
	"\n";

static char unlock_channel_usage_br[] =
	"\n"
	"       Desbloqueia um canal especifico. \n"
	"        |_ Uso: dgv unlock channel <channel/all> \n"
	"\n";

static char lock_group_usage_en[] =
	"\n"
	"       Lock a specific group. \n"
    "        |_ Usage: dgv lock group <group> \n"
	"\n";

static char lock_group_usage_br[] =
	"\n"
	"       Bloqueia um grupo especifico. \n"
	"        |_ Uso: dgv lock group <group> \n"
	"\n";

static char unlock_group_usage_en[] =
	"\n"
	"       Unlock a specific group. \n"
	"        |_ Usage: dgv unlock group <group/all> \n"
	"\n";

static char unlock_group_usage_br[] =
	"\n"
	"       Desbloqueia um grupo especifico. \n"
	"        |_ Uso: dgv unlock group <group/all> \n"
	"\n";

static char get_unread_sms_channel_usage_en[] =
	"\n"
	"       Get unread SMS messages of a specific channel. \n"
	"        |_ Usage: dgv get unread sms channel <channel/all> \n"
	"\n";

static char get_unread_sms_channel_usage_br[] =
	"\n"
	"       Solicita mensagens SMS nao lidas de um canal especifico. \n"
	"        |_ Uso: dgv get unread sms channel <channel/all> \n"
	"\n";

static char get_unread_sms_group_usage_en[] =
	"\n"
	"       Get unread SMS messages of a specific group. \n"
	"        |_ Usage: dgv get unread sms group <group/all> \n"
	"\n";

static char get_unread_sms_group_usage_br[] =
	"\n"
	"       Solicita mensagens SMS nao lidas de um grupo especifico. \n"
	"        |_ Uso: dgv get unread sms group <group/all> \n"
	"\n";

static char gsm_csca_read_usage_en[] =
	"\n"
	"       Read GSM CSCA (SMSC Address) of a specific channel. \n"
	"        |_ Usage: dgv gsm csca read <channel> \n"
	"\n";

static char gsm_csca_read_usage_br[] =
	"\n"
	"       Le o GSM CSCA (Endereco SMSC) de um canal especifico. \n"
	"        |_ Usage: dgv gsm csca read <channel> \n"
	"\n";

static char gsm_csca_write_usage_en[] =
	"\n"
	"       Write GSM CSCA (SMSC Address) of a specific channel. \n"
	"        |_ Usage: dgv gsm csca write <channel> <service_center_address>\n"
	"\n";

static char gsm_csca_write_usage_br[] =
	"\n"
	"       Escreve o GSM CSCA (Endereco SMSC) de um canal especifico. \n"
	"        |_ Usage: dgv gsm csca write <channel> <service_center_address>\n"
	"\n";

//for testing purposes
static char debug_audio_usage_en[] =
	"\n"
	"       Enable/Disable audio debug (for testing purposes). \n"
	"        |_ Usage: dgv debug audio <ch> <on/off> \n"	
	"\n";
	
static char debug_audio_usage_br[] =
	"\n"
	"       Habilita/Desabilita debug de audio (para fins de teste). \n"	
	"        |_ Uso: dgv debug audio <ch> <on/off> \n"	
	"\n";

static char debug_voicerlib_usage_en[] =
	"\n"
	"       VoicerLib level(0..5) debug (for testing purposes). \n"
	"        |_ Usage: dgv debug voicerlib <card/all> <level> \n"
	"\n";
	
static char debug_voicerlib_usage_br[] =
	"\n"
	"       Nivel(0..5) de debug da VoicerLib (para fins de teste). \n"
	"        |_ Uso: dgv debug voicerlib <card/all> <level> \n"
	"\n";

static char debug_isdn_usage_en[] =
	"\n"
	"       Enable/Disable ISDN signaling debug (for testing purposes). \n"
	"        |_ Usage: dgv debug isdn <span/all> <on/off> \n"
	"\n";

static char debug_isdn_usage_br[] =
	"\n"
	"       Habilita/Desabilita debug de sinalizacao ISDN (para fins de teste). \n"
	"        |_ Uso: dgv debug isdn <span/all> <on/off> \n"
	"\n";
	
static char verbose_usage_en[] =
	"\n"
	"       Sets verbosity level to specific channel. <ch>=1..n or 'all', <level>=0..n, <on/off> (for testing purposes). \n"
	"        |_ Usage: dgv verbose <ch> <level> <on/off>\n\n"
	"       Verbosity level Descriptions: \n"
	"        | \n"
	"        |_ Level / Description \n"
	"        |_    0  - Disable dgv verbose. \n"	
	"        |_   10  - start_dial + dgv_new + dgv_request + dgv_call + dgv_hangup + dgv_fixup. \n"
	"        |_   20  - set_port_group + find_free_port + find_free_sms_channel + swap_sc + swap_sc_tech. \n"
	"        |_   30  - ISDN events. \n"
	"        |_   40  - ToneThread + StutterToneThread + SubChannelToneThread + BeepToneThread. \n"
	"        |_   50  - DigiVoice Hardware Events. \n"
	"        |_   60  - read_samples. \n"
	"        |_   70  - QueueFrame. \n"
	"        |_   80  - dgv_write. \n"
	"        |_   90  - dgv_gsm_reset + unload_module + load_statistics + reset_statistics. \n"
#ifdef CC_AST_HAS_SNMP
	"        |_  100  - dgv_indicate + dgv_sendtext + dgv_devicestate + dgv_snmp_show_alarms. \n"
#else//#ifdef CC_AST_HAS_SNMP
	"        |_  100  - dgv_indicate + dgv_sendtext + dgv_devicestate. \n"
#endif//#ifdef CC_AST_HAS_SNMP
	"        |_  110  - Pause/Release InputBuffer. \n"
	"\n";
	
static char verbose_usage_br[] =
	"\n"
	"       Seta um nivel de log para um canal especifico. <ch>=1..n or 'all', <level>=0..n, <on/off> (para fins de teste). \n"
	"        |_ Uso: dgv verbose <ch> <level> <on/off>\n\n"
	"       Descricao dos niveis de log: \n"
	"        | \n"
	"        |_ Nivel / Descricao \n"
	"        |_    0  - Desabilita dgv verbose. \n"	
	"        |_   10  - start_dial + dgv_new + dgv_request + dgv_call + dgv_hangup + dgv_fixup. \n"
	"        |_   20  - set_port_group + find_free_port + find_free_sms_channel + swap_sc + swap_sc_tech. \n"
	"        |_   30  - Eventos ISDN. \n"
	"        |_   40  - ToneThread + StutterToneThread + SubChannelToneThread + BeepToneThread. \n"
	"        |_   50  - Eventos Hardware DigiVoice. \n"
	"        |_   60  - read_samples. \n"
	"        |_   70  - QueueFrame. \n"
	"        |_   80  - dgv_write. \n"
	"        |_   90  - dgv_gsm_reset + unload_module + load_statistics + reset_statistics. \n"
#ifdef CC_AST_HAS_SNMP
	"        |_  100  - dgv_indicate + dgv_sendtext + dgv_devicestate + dgv_snmp_show_alarms. \n"
#else//#ifdef CC_AST_HAS_SNMP
	"        |_  100  - dgv_indicate + dgv_sendtext + dgv_devicestate. \n"
#endif//#ifdef CC_AST_HAS_SNMP
	"        |_  110  - Pausa/Libera InputBuffer. \n"
	"\n";
	
static char r2_log_usage_en[] =
	"\n"
	"       Turns on/off r2d/mfc logs in /var/log/voicerlib/debug-<ch> (for testing purposes). \n"
	"        |_ Usage: dgv r2 log <on/off/reload> \n"	
	"\n";
	
static char r2_log_usage_br[] =
	"\n"
	"       Habilita/Desabilita logs de r2d/mfc em /var/log/voicerlib/debug-<ch> (para fins de teste). \n"	
	"        |_ Uso: dgv r2 log <on/off/reload> \n"	
	"\n";

static char gsm_log_usage_en[] =
	"\n"
	"       Turns on/off gsm logs in /var/log/voicerlib/gsm-<ch>.log (for testing purposes). \n"
	"        |_ Usage: dgv gsm log <on/off> \n"
	"\n";
	
static char gsm_log_usage_br[] =
	"\n"
	"       Habilita/Desabilita logs de gsm em /var/log/voicerlib/gsm-<ch>.log (para fins de teste). \n"
	"        |_ Uso: dgv gsm log <on/off> \n"
	"\n";

static char echo_can_usage_en[] =
	"\n"
	"       Set echo cancellation manually with TAPS (0 - off, 1-32, 2-64, 3-128, 4-256, 5-512, 6-1024) (for testing purposes). \n"
#ifdef K_ECHO
	"        |_ Usage: dgv echo <ch> <type> <taps> \n"
#else//#ifdef K_ECHO
	"        |_ Usage: dgv echo <ch> <taps> \n"
#endif//#ifdef K_ECHO
	"\n";
	
static char echo_can_usage_br[] =
	"\n"
	"       Seta cancelamento de eco manualmente com TAPS (0 - off, 1-32, 2-64, 3-128, 4-256, 5-512, 6-1024) (para fins de teste). \n"
#ifdef K_ECHO
	"        |_ Uso: dgv echo <ch> <type> <taps> \n"
#else//#ifdef K_ECHO
	"        |_ Uso: dgv echo <ch> <taps> \n"
#endif//#ifdef K_ECHO
	"\n";
	
static char drop_txrx_usage_en[] =
	"\n"
	"       Force dropping TX/RX samples. <times> times on <ch> channel (for testing purposes). \n"
	"        |_ Usage: dgv drop tx/rx <ch> <times> \n"	
	"\n";
	
static char drop_txrx_usage_br[] =
	"\n"
	"       Remove amostras de TX/RX. <times> vezes em um <ch> canal (para fins de teste). \n"	
	"        |_ Uso: dgv drop tx/rx <ch> <times> \n"	
	"\n";

static char record_file_en[] =
	"\n"
	"       Please, don't use this function. \n"
	"        |_ Usage: dgv record <ch> \n"	
	"\n";
	
static char record_file_br[] =
	"\n"	
	"       Nao utilize esta funcao. \n"
	"        |_ Uso: dgv record <ch> \n"	
	"\n";

static char test_lock_usage_en[] =
	"\n"
	"       Please, don't use this function. \n"
	"\n";
	
static char test_lock_usage_br[] =
	"\n"	
	"       Nao utilize esta funcao. \n"
	"\n";

static char send_generic_cmd_usage_en[] =
	"\n"
	"       Please, don't use this function. \n"
	"\n";

static char send_generic_cmd_usage_br[] =
	"\n"
	"       Nao utilize esta funcao. \n"
	"\n";

#if !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)
	//-------------------------------------------------------------------------------------------
	//Console Definitions -> English 
	//-------------------------------------------------------------------------------------------
	static struct ast_cli_entry dgv_cli_en[] = {
		{ {"dgv", "show", "channels", NULL}, dgv_show_channels,
		  "Show active DigiVoice channels", show_channels_usage_en },

		{ {"dgv", "show", "cards", NULL}, dgv_show_cards,
		  "Show active DigiVoice cards", show_cards_usage_en },

		{ {"dgv", "show", "channel", NULL}, dgv_show_channel,
		  "Show detailed information about a given channel", show_channel_usage_en },
		  
		/*{ {"dgv", "reset", NULL, NULL}, dgv_reset,
		  "Reset all DigiVoice channels", reset_usage_en },*/
		  
		{ {"dgv", "gsm", "reset", NULL}, dgv_gsm_reset,
		  "Reset a specific GSM channel", gsm_reset_usage_en },

		{ {"dgv", "show", "version", NULL}, dgv_show_version,
		  "Show chan_dgv current version", show_version_usage_en },
		  
		{ {"dgv", "show", "alarms", NULL}, dgv_show_alarms,
		  "Shows alams from specific E1/GSM/FXO card installed", show_alarms_usage_en },
		  
		{ {"dgv", "show", "statistics", NULL}, dgv_show_statistics,
		  "Shows statistics of DigiVoice channels", show_statistics_usage_en },

		{ {"dgv", "alarms", "reset", NULL}, dgv_alarms_reset,
		  "Reset alarms from specific E1/FXO/GSM card installed", reset_alarms_usage_en },
		  
		{ {"dgv", "statistics", "reset", NULL}, dgv_statistics_reset,
		  "Reset statistics from specific channel", reset_statistics_usage_en },

		{ {"dgv", "send", "sms", NULL, NULL}, dgv_send_sms,
		  "Send a SMS message using a GSM channel installed", send_sms_usage_en },

		{ {"dgv", "lock", "channel", NULL}, dgv_lock_channel,
		  "Lock a specific channel", lock_channel_usage_en },

		{ {"dgv", "unlock", "channel", NULL}, dgv_unlock_channel,
		  "Unlock a specific channel", unlock_channel_usage_en },

		{ {"dgv", "lock", "group", NULL}, dgv_lock_channel,
		  "Lock a specific group", lock_group_usage_en },

		{ {"dgv", "unlock", "group", NULL}, dgv_unlock_channel,
		  "Unlock a specific group", unlock_group_usage_en },

		{ {"dgv", "get", "unread", "sms", "channel", NULL}, dgv_get_unread_sms_channel,
		  "Get unread SMS messages of a specific channel", get_unread_sms_channel_usage_en },

		{ {"dgv", "get", "unread", "sms", "group", NULL}, dgv_get_unread_sms_channel,
		  "Get unread SMS messages of a specific group", get_unread_sms_group_usage_en },

		{ {"dgv", "gsm", "csca", "read", NULL}, dgv_gsm_csca_channel,
		  "Read GSM CSCA (SMSC Address) of a specific channel", gsm_csca_read_usage_en },

		{ {"dgv", "gsm", "csca", "write", NULL}, dgv_gsm_csca_channel,
		  "Write GSM CSCA (SMSC Address) of a specific channel", gsm_csca_write_usage_en },

		//for testing purposes
		{ {"dgv", "debug", "audio", NULL}, dgv_debug_audio,
		  "Enable/Disable audio debug (for testing purposes)", debug_audio_usage_en },
		  
		{ {"dgv", "debug", "voicerlib", NULL}, dgv_debug_voicerlib,
	 	  "VoicerLib level debug (for testing purposes)", debug_voicerlib_usage_en },

		{ {"dgv", "debug", "isdn", NULL, NULL}, dgv_debug_isdn,
		  "Enable/Disable ISDN signaling debug (for testing purposes)", debug_isdn_usage_en },
		  
	    { {"dgv", "verbose", NULL, NULL, NULL}, dgv_verbose_channel,
	      "Sets verbosity level to specific channel (for testing purposes)", verbose_usage_en },
	      
	    { {"dgv", "r2", "log", NULL}, dgv_r2_log,
	      "Logs all R2D/MFC signalling into a file (for testing purposes)", r2_log_usage_en },

		{ {"dgv", "gsm", "log", NULL}, dgv_gsm_log,
		  "Logs all GSM signalling into a file (for testing purposes)", gsm_log_usage_en },
	      
	    { {"dgv", "echo", NULL, NULL}, dgv_echo_can,
	      "Turns echo cancellation on/off (for testing purposes)", echo_can_usage_en },
	      
	    { {"dgv", "drop", NULL, NULL, NULL}, dgv_drop,
	      "Dropping samples utility (for testing purposes)", drop_txrx_usage_en },

	    { {"dgv", "record", NULL}, dgv_record,
	      "Please, don't use this function (for testing purposes)", record_file_en },

	    { {"dgv", "test", NULL}, dgv_test_lock,
	      "Please, don't use this function (for testing purposes)", test_lock_usage_en },      

		{ {"dgv", "send", "cmd", NULL, NULL}, dgv_send_generic_cmd,
		  "Please, don't use this function (for testing purposes)", send_generic_cmd_usage_en },
	};
	
	//-------------------------------------------------------------------------------------------
	//Console Definitions -> Portuguese 
	//-------------------------------------------------------------------------------------------
	static struct ast_cli_entry dgv_cli_br[] = {
		{ {"dgv", "show", "channels", NULL}, dgv_show_channels,
		  "Mostra os canais DigiVoice ativos", show_channels_usage_br },

		{ {"dgv", "show", "cards", NULL}, dgv_show_cards,
		  "Mostra as placas DigiVoice ativas", show_cards_usage_br },

		{ {"dgv", "show", "channel", NULL}, dgv_show_channel,
		  "Mostra informacoes detalhadas de um determinado canal", show_channel_usage_br },
		  
		/*{ {"dgv", "reset", NULL, NULL}, dgv_reset,
		  "Reseta todos os canais DigiVoice", reset_usage_br },*/
		  
		{ {"dgv", "gsm", "reset", NULL}, dgv_gsm_reset,
		  "Reseta um canal GSM especifico", gsm_reset_usage_br },

		{ {"dgv", "show", "version", NULL}, dgv_show_version,
		  "Mostra a versao corrente do chan_dgv", show_version_usage_br },
		  
		{ {"dgv", "show", "alarms", NULL}, dgv_show_alarms,
		  "Mostra alarmes de uma especifica placa E1/GSM/FXO instalada", show_alarms_usage_br },
	
		{ {"dgv", "show", "statistics", NULL}, dgv_show_statistics,
		  "Mostra estatisticas de uma especifica placa instalada", show_statistics_usage_br },

		{ {"dgv", "alarms", "reset", NULL}, dgv_alarms_reset,
		  "Reseta alarmes de uma especifica placa E1/FXO/GSM instalada", reset_alarms_usage_br },
		  
		{ {"dgv", "statistics", "reset", NULL}, dgv_statistics_reset,
		  "Reseta estatisticas de um especifico canal", reset_statistics_usage_br },

		{ {"dgv", "send", "sms", NULL, NULL}, dgv_send_sms,
		  "Envia uma mensagem SMS utilizando um canal GSM instalado", send_sms_usage_br },
		  
		{ {"dgv", "lock", "channel", NULL}, dgv_lock_channel,
          "Bloqueia um canal especifico", lock_channel_usage_br },

		{ {"dgv", "unlock", "channel", NULL}, dgv_unlock_channel,
          "Desbloqueia um canal especifico", unlock_channel_usage_br },

  		{ {"dgv", "lock", "group", NULL}, dgv_lock_channel,
          "Bloqueia um grupo especifico", lock_group_usage_br },

  		{ {"dgv", "unlock", "group", NULL}, dgv_unlock_channel,
          "Desbloqueia um grupo especifico", unlock_group_usage_br },

  		{ {"dgv", "get", "unread", "sms", "channel", NULL}, dgv_get_unread_sms_channel,
  		  "Solicita mensagens SMS nao lidas de um canal especifico", get_unread_sms_channel_usage_br },

  		{ {"dgv", "get", "unread", "sms", "group", NULL}, dgv_get_unread_sms_channel,
  		  "Solicita mensagens SMS nao lidas de um grupo especifico", get_unread_sms_group_usage_br },

  		{ {"dgv", "gsm", "csca", "read", NULL}, dgv_gsm_csca_channel,
  		  "Le o GSM CSCA (Endereco SMSC) de um canal especifico", gsm_csca_read_usage_br },

  		{ {"dgv", "gsm", "csca", "write", NULL}, dgv_gsm_csca_channel,
  		  "Escreve o GSM CSCA (Endereco SMSC) de um canal especifico", gsm_csca_write_usage_br },

		//for testing purposes 
		{ {"dgv", "debug", "audio", NULL}, dgv_debug_audio,
		  "Habilita/Desabilita debug de audio (para fins de teste)", debug_audio_usage_br },
		  
		{ {"dgv", "debug", "voicerlib", NULL}, dgv_debug_voicerlib,
		  "Nivel de debug da VoicerLib (para fins de teste)", debug_voicerlib_usage_br },
		  
		{ {"dgv", "debug", "isdn", NULL, NULL}, dgv_debug_isdn,
		  "Habilita/Desabilita debug de sinalizacao ISDN (para fins de teste)", debug_isdn_usage_br },

	    { {"dgv", "verbose", NULL, NULL, NULL}, dgv_verbose_channel,
	      "Seta um nivel de log para um canal especifico (para fins de teste)", verbose_usage_br },
	      
	    { {"dgv", "r2", "log", NULL}, dgv_r2_log,
	      "Gera um arquivo de log com toda sinalizacao R2D/MFC (para fins de teste)", r2_log_usage_br },
	      
		{ {"dgv", "gsm", "log", NULL}, dgv_gsm_log,
		  "Gera um arquivo de log com toda sinalizacao GSM (para fins de teste)", gsm_log_usage_br },

	    { {"dgv", "echo", NULL, NULL}, dgv_echo_can,
	      "Habilita/Desabilita cancelamento de eco para efeito de testes (para fins de teste)", echo_can_usage_br },
	      
	    { {"dgv", "drop", NULL, NULL, NULL}, dgv_drop,
	      "Utilitario para remover amostras (para fins de teste)", drop_txrx_usage_br },

	    { {"dgv", "record", NULL}, dgv_record,
	      "Nao utilize esta funcao (para fins de teste)", record_file_br },           

	    { {"dgv", "test", NULL}, dgv_test_lock,
	      "Nao utilize esta funcao (para fins de teste)", test_lock_usage_br },

		{ {"dgv", "send", "cmd", NULL, NULL}, dgv_send_generic_cmd,
		  "Nao utilize esta funcao (para fins de teste)", send_generic_cmd_usage_br },
	};
#else//#if !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)
	static char *handle_cli_dgv_show_channels(struct ast_cli_entry *e, int cmd,	struct ast_cli_args *a)
	{
		switch (cmd)
		{
			case CLI_INIT:
				e->command = "dgv show channels";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
			    		e->usage = show_channels_usage_en;	
			    		break;    
					case 1://Portuguese
			    		e->usage = show_channels_usage_br;
			    		break;    
			    	default:
			    		e->usage = show_channels_usage_en;
	    		}
				return NULL;
			case CLI_GENERATE:
				return NULL;
		}

		if (dgv_show_channels(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_show_cards(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
		switch (cmd)
		{
			case CLI_INIT:
				e->command = "dgv show cards";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
			    		e->usage = show_cards_usage_en;
			    		break;
					case 1://Portuguese
			    		e->usage = show_cards_usage_br;
			    		break;
			    	default:
			    		e->usage = show_cards_usage_en;
	    		}
				return NULL;
			case CLI_GENERATE:
				return NULL;
		}

		if (dgv_show_cards(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_show_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
		switch (cmd)
		{
			case CLI_INIT:
				e->command = "dgv show channel";
	
				switch (dgv_conf.consolelanguage)
				{
			    	case 0://English
			    		e->usage = show_channel_usage_en;	
			    		break;    
			    	case 1://Portuguese
			    		e->usage = show_channel_usage_br;
			    		break;    
			    	default:
			    		e->usage = show_channel_usage_en;
	    		}
				return NULL;
			case CLI_GENERATE:
				return NULL;
		}
	
		if (dgv_show_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	/*static char *handle_cli_dgv_reset(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv reset";
	
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = reset_usage_en;	
						break;    
					case 1://Portuguese
						e->usage = reset_usage_br;
						break;    
					default:
						e->usage = reset_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;    
	    }
	    
		if (dgv_reset(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}*/
	
	static char *handle_cli_dgv_gsm_reset(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
		switch (cmd)
		{
			case CLI_INIT:
				e->command = "dgv gsm reset";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = gsm_reset_usage_en;
						break;
					case 1://Portuguese
						e->usage = gsm_reset_usage_br;
						break;
					default:
						e->usage = gsm_reset_usage_en;
				}
				return NULL;
			case CLI_GENERATE:
				return NULL;
		}

		if (dgv_gsm_reset(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_show_version(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv show version";
	
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = show_version_usage_en;	
						break;    
					case 1://Portuguese
						e->usage = show_version_usage_br;
						break;    
					default:
						e->usage = show_version_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;    
	    }
	    
	    if (dgv_show_version(a->fd, a->argc, a->argv))
	    	return CLI_SHOWUSAGE;
	    else
	    	return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_show_alarms(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv show alarms";
	
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = show_alarms_usage_en;	
						break;    
					case 1://Portuguese
						e->usage = show_alarms_usage_br;
						break;    
					default:
						e->usage = show_alarms_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_show_alarms(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_show_statistics(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv show statistics";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = show_statistics_usage_en;
						break;
					case 1://Portuguese
						e->usage = show_statistics_usage_br;
						break;
					default:
						e->usage = show_statistics_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_show_statistics(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_alarms_reset(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv alarms reset";
		    
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = reset_alarms_usage_en;	
						break;    
					case 1://Portuguese
						e->usage = reset_alarms_usage_br;
						break;    
					default:
						e->usage = reset_alarms_usage_en;
	    	    }
				return NULL;
		    
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_alarms_reset(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;    
		else
			return CLI_SUCCESS;
	}
	
	static char *handle_cli_dgv_statistics_reset(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv statistics reset";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = reset_statistics_usage_en;
						break;
					case 1://Portuguese
						e->usage = reset_statistics_usage_br;
						break;
					default:
						e->usage = reset_statistics_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_statistics_reset(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_send_sms(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv send sms";
		    
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = send_sms_usage_en;	
						break;    
					case 1://Portuguese
						e->usage = send_sms_usage_br;
						break;    
					default:
						e->usage = send_sms_usage_en;
	    	    }
				return NULL;
		    
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_send_sms(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;    
		else
			return CLI_SUCCESS;
	}	
	
	static char *handle_cli_dgv_lock_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv lock channel";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = lock_channel_usage_en;
						break;
					case 1://Portuguese
						e->usage = lock_channel_usage_br;
						break;
					default:
						e->usage = lock_channel_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_lock_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_unlock_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv unlock channel";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = unlock_channel_usage_en;
						break;
					case 1://Portuguese
						e->usage = unlock_channel_usage_br;
						break;
					default:
						e->usage = unlock_channel_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_unlock_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_lock_group(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv lock group";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = lock_group_usage_en;
						break;
					case 1://Portuguese
						e->usage = lock_group_usage_br;
						break;
					default:
						e->usage = lock_group_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_lock_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_unlock_group(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv unlock group";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = unlock_group_usage_en;
						break;
					case 1://Portuguese
						e->usage = unlock_group_usage_br;
						break;
					default:
						e->usage = unlock_group_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_unlock_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_get_unread_sms_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv get unread sms channel";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = get_unread_sms_channel_usage_en;
						break;
					case 1://Portuguese
						e->usage = get_unread_sms_channel_usage_br;
						break;
					default:
						e->usage = get_unread_sms_channel_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_get_unread_sms_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_get_unread_sms_group(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv get unread sms group";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = get_unread_sms_group_usage_en;
						break;
					case 1://Portuguese
						e->usage = get_unread_sms_group_usage_br;
						break;
					default:
						e->usage = get_unread_sms_group_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_get_unread_sms_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_gsm_csca_read_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv gsm csca read";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = gsm_csca_read_usage_en;
						break;
					case 1://Portuguese
						e->usage = gsm_csca_read_usage_br;
						break;
					default:
						e->usage = gsm_csca_read_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_gsm_csca_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_gsm_csca_write_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv gsm csca write";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = gsm_csca_write_usage_en;
						break;
					case 1://Portuguese
						e->usage = gsm_csca_write_usage_br;
						break;
					default:
						e->usage = gsm_csca_write_usage_en;
	    	    }
				return NULL;

			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_gsm_csca_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_debug_audio(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv debug audio";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = debug_audio_usage_en;
						break;    
					case 1://Portuguese
						e->usage = debug_audio_usage_br;
						break;    
					default:
						e->usage = debug_audio_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_debug_audio(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else 
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_debug_voicerlib(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv debug voicerlib";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = debug_voicerlib_usage_en;
						break;    
					case 1://Portuguese
						e->usage = debug_voicerlib_usage_br;
						break;    
					default:
						e->usage = debug_voicerlib_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_debug_voicerlib(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else 
			return CLI_SUCCESS;
	}	
	
	static char *handle_cli_dgv_debug_isdn(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv debug isdn";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = debug_isdn_usage_en;
						break;
					case 1://Portuguese
						e->usage = debug_isdn_usage_br;
						break;
					default:
						e->usage = debug_isdn_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_debug_isdn(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_verbose_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv verbose";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = verbose_usage_en;
						break;    
					case 1://Portuguese
						e->usage = verbose_usage_br;
						break;    
					default:
						e->usage = verbose_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_verbose_channel(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else 
			return CLI_SUCCESS;
	}
	
	static char *handle_cli_dgv_r2_log(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv r2 log";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = r2_log_usage_en;
						break;    
					case 1://Portuguese
						e->usage = r2_log_usage_br;
						break;    
					default:
						e->usage = r2_log_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_r2_log(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else 
			return CLI_SUCCESS;
	}
	
	static char *handle_cli_dgv_gsm_log(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv gsm log";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = gsm_log_usage_en;
						break;
					case 1://Portuguese
						e->usage = gsm_log_usage_br;
						break;
					default:
						e->usage = gsm_log_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_gsm_log(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_echo_can(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv echo";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = echo_can_usage_en;
						break;    
					case 1://Portuguese
						e->usage = echo_can_usage_br;
						break;    
					default:
						e->usage = echo_can_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }

	    if (dgv_echo_can(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else 
			return CLI_SUCCESS;
	}
		
	static char *handle_cli_dgv_drop(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv drop";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = drop_txrx_usage_en;
						break;    
					case 1://Portuguese
						e->usage = drop_txrx_usage_br;
						break;    
					default:
						e->usage = drop_txrx_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_drop(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_record(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv record";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = record_file_en;
						break;    
					case 1://Portuguese
						e->usage = record_file_br;
						break;    
					default:
						e->usage = record_file_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_record(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static char *handle_cli_dgv_test_lock(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv test";
				
				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = test_lock_usage_en;
						break;
					case 1://Portuguese
						e->usage = test_lock_usage_br;
						break;    
					default:
						e->usage = test_lock_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }
	    
		if (dgv_test_lock(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}
	
	static char *handle_cli_dgv_send_generic_cmd(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
	{
	    switch (cmd)
	    {
			case CLI_INIT:
				e->command = "dgv send";

				switch (dgv_conf.consolelanguage)
				{
					case 0://English
						e->usage = send_generic_cmd_usage_en;
						break;
					case 1://Portuguese
						e->usage = send_generic_cmd_usage_br;
						break;
					default:
						e->usage = send_generic_cmd_usage_en;
	    	    }
				return NULL;
			case CLI_GENERATE:
				return NULL;
	    }

		if (dgv_send_generic_cmd(a->fd, a->argc, a->argv))
			return CLI_SHOWUSAGE;
		else
			return CLI_SUCCESS;
	}

	static struct ast_cli_entry dgv_cli_en[] = { 
			AST_CLI_DEFINE(handle_cli_dgv_show_channels, "Show active DigiVoice channels"),
			AST_CLI_DEFINE(handle_cli_dgv_show_cards, "Show active DigiVoice cards"),
			AST_CLI_DEFINE(handle_cli_dgv_show_channel, "Show detailed information about a given channel"),
			/*AST_CLI_DEFINE(handle_cli_dgv_reset, "Reset all DigiVoice channels"),*/
			AST_CLI_DEFINE(handle_cli_dgv_gsm_reset, "Reset a specific GSM channel"),
			AST_CLI_DEFINE(handle_cli_dgv_show_version, "Show chan_dgv current version"),
			AST_CLI_DEFINE(handle_cli_dgv_show_alarms, "Shows alarms from specific E1/GSM/FXO Card installed"),
			AST_CLI_DEFINE(handle_cli_dgv_show_statistics, "Shows statistics of DigiVoice channels"),
			AST_CLI_DEFINE(handle_cli_dgv_alarms_reset, "Reset alarms from specific E1/FXO/GSM Card installed"),
			AST_CLI_DEFINE(handle_cli_dgv_statistics_reset, "Reset statistics from specific channel"),
			AST_CLI_DEFINE(handle_cli_dgv_send_sms, "Send a SMS message using a GSM channel installed"),
			AST_CLI_DEFINE(handle_cli_dgv_lock_channel, "Lock a specific channel"),
			AST_CLI_DEFINE(handle_cli_dgv_unlock_channel, "Unlock a specific channel"),
			AST_CLI_DEFINE(handle_cli_dgv_lock_group, "Lock a specific group"),
			AST_CLI_DEFINE(handle_cli_dgv_unlock_group, "Unlock a specific group"),
			AST_CLI_DEFINE(handle_cli_dgv_get_unread_sms_channel, "Get unread SMS messages of a specific channel"),
			AST_CLI_DEFINE(handle_cli_dgv_get_unread_sms_group, "Get unread SMS messages of a specific group"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_csca_read_channel, "Read GSM CSCA (SMSC Address) of a specific channel"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_csca_write_channel, "Write GSM CSCA (SMSC Address) of a specific channel"),

			//for testing purposes
			AST_CLI_DEFINE(handle_cli_dgv_debug_audio, "Enable/Disable audio debug (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_debug_voicerlib, "VoicerLib level debug (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_debug_isdn, "Enable/Disable ISDN signaling debug (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_verbose_channel, "Sets verbosity level to specific channel (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_r2_log, "Logs all R2D/MFC signalling into a file (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_log, "Logs all GSM signalling into a file (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_echo_can, "Turns echo cancelation on/off (for testing purposes)"),			
			AST_CLI_DEFINE(handle_cli_dgv_drop, "Dropping samples utility (for testing purposes)"), 
			AST_CLI_DEFINE(handle_cli_dgv_record, "Please, don't use this function (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_test_lock, "Please, don't use this function (for testing purposes)"),
			AST_CLI_DEFINE(handle_cli_dgv_send_generic_cmd, "Please, don't use this function (for testing purposes)")
	};
	
	static struct ast_cli_entry dgv_cli_br[] = { 
			AST_CLI_DEFINE(handle_cli_dgv_show_channels, "Mostra os canais DigiVoice ativos"),
			AST_CLI_DEFINE(handle_cli_dgv_show_cards, "Mostra as placas DigiVoice ativas"),
			AST_CLI_DEFINE(handle_cli_dgv_show_channel, "Mostra informacoes detalhadas de um determinado canal"),
			/*AST_CLI_DEFINE(handle_cli_dgv_reset, "Reseta todos os canais DigiVoice"),*/
			AST_CLI_DEFINE(handle_cli_dgv_gsm_reset, "Reseta um canal GSM especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_show_version, "Mostra a versao corrente do chan_dgv"),
			AST_CLI_DEFINE(handle_cli_dgv_show_alarms, "Mostra alarmes de uma especifica placa E1/GSM/FXO instalada"),
			AST_CLI_DEFINE(handle_cli_dgv_show_statistics, "Mostra estatisticas de uma lista de canais"),
			AST_CLI_DEFINE(handle_cli_dgv_alarms_reset, "Reseta alarmes de uma especifica placa E1/FXO/GSM instalada"),
			AST_CLI_DEFINE(handle_cli_dgv_statistics_reset, "Reseta estatisticas de um especifico canal"),
			AST_CLI_DEFINE(handle_cli_dgv_send_sms, "Envia uma mensagem SMS utilizando um canal GSM instalado"),
			AST_CLI_DEFINE(handle_cli_dgv_lock_channel, "Bloqueia um canal especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_unlock_channel, "Desbloqueia um canal especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_lock_group, "Bloqueia um grupo especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_unlock_group, "Desbloqueia um grupo especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_get_unread_sms_channel, "Solicita mensagens SMS nao lidas de um canal especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_get_unread_sms_group, "Solicita mensagens SMS nao lidas de um grupo especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_csca_read_channel, "Le o GSM CSCA (Endereco SMSC) de um canal especifico"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_csca_write_channel, "Escreve o GSM CSCA (Endereco SMSC) de um canal especifico"),

			//for testing purposes
			AST_CLI_DEFINE(handle_cli_dgv_debug_audio, "Habilita/Desabilita debug de audio (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_debug_voicerlib, "Nivel de debug da VoicerLib (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_debug_isdn, "Habilita/Desabilita debug de sinalizacao ISDN (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_verbose_channel, "Seta um nivel de log para um canal especifico (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_r2_log, "Gera um arquivo de log com toda sinalizacao R2D/MFC (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_gsm_log, "Gera um arquivo de log com toda sinalizacao GSM (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_echo_can, "Habilita/Desabilita cancelamento de eco para efeito de testes (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_drop, "Utilitario para remover amostras (para fins de teste)"), 
			AST_CLI_DEFINE(handle_cli_dgv_record, "Nao utilize esta funcao (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_test_lock, "Nao utilize esta funcao (para fins de teste)"),
			AST_CLI_DEFINE(handle_cli_dgv_send_generic_cmd, "Nao utilize esta funcao (para fins de teste)")
	};
#endif//#if !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)

static void *do_timer(void *data)
{
	int count;
	int res;
	struct timeval tv;
	struct timespec ts;

	for (;;) 
	{		
		res = ast_sched_wait(sched);
		if ((res > 1000) || (res < 0))
			res = 1000;
		tv = ast_tvadd(ast_tvnow(), ast_samp2tv(res, 1000));
		ts.tv_sec = tv.tv_sec;
		ts.tv_nsec = tv.tv_usec * 1000;
		
		pthread_testcancel();		
		
		ast_dgv_lock(&timer_pending_lock, 0);
		{
			ast_cond_timedwait(&timer_pending_cond, &timer_pending_lock, &ts);
		}		
		ast_dgv_unlock(&timer_pending_lock, 0);		
		
		pthread_testcancel();		
		count = ast_sched_runq(sched);
		
		if (count >= 20)
			ast_log(LOG_DEBUG, "chan_dgv: ast_sched_runq ran %d scheduled tasks all at once\n", count);		
	}
	
	return NULL;
}

//isdn support functions
#ifdef USE_LIBPRI
	//---------------------------------------------
	// Find dchans index based on card and span
	//---------------------------------------------
	int dg_pri_findindex(short card, short span)
	{
	    int i= 0;
    
    	for (i=0; i< dchans_count; i++) 
    	{
        	if (dchans[i].card==card && dchans[i].span==span) 
        	{
            	//found!
            	return i;
        	}
    	}
    	//not found
    	return -1;
	}

	//---------------------------------------------
	// Find dchans index based on pri handle
	//---------------------------------------------
	int dg_pri_findpri(struct pri *pri)
	{
    	int i=0;
    
	    for (i=0; i< dchans_count; i++) 
    	{
        	if (dchans[i].pri==pri) 
        	{
	            //found!
            	return i;
        	}
    	}
    	//not found
    	return -1;
	}
	
	//---------------------------------------------
	// Find dchans index based on fds[1]
	//---------------------------------------------
	int dg_pri_findfds(int fds)
	{
	    int i= 0;
	    
	    for (i=0; i< dchans_count; i++) 
	    {
        	if (dchans[i].fds[TX]==fds) 
        	{
	            //found!
    	        return i;
        	}
    	}
    	//not found
    	return -1;
	}

	//------------------------------------------------
	// dg_pri_to_port
	// converts pri channel (1-15,17-31,32-46,48-62) to voicerlib
	// defaults (1-30,31-60)
	//------------------------------------------------
	int dg_pri_to_port(int pri_channel)
	{
		if (pri_channel <= 15)
			return pri_channel;
		else
			if ((pri_channel >= 17) && (pri_channel <= 31))
				return (pri_channel - 1);
			else
				if ((pri_channel >= 32) && (pri_channel <= 46))
					return (pri_channel - 31);
				else
					if ((pri_channel >= 48) && (pri_channel <= 62))
						return (pri_channel - 32);
					else
						return 0;
	}
	
	//------------------------------------------------
	// port_to_pri
	// converts voicerlib port (1-30,31-60) to pri 
	// channel (1-15,17-31) 
	//------------------------------------------------
	int port_to_pri(int port)
	{
		if (port <= 15)
			return port;
		else
			if ((port >= 16) && (port <= 30))
				return (port + 1);
			else
			{
				port = port - 30;

				if (port <= 15)
					return port;
				else
					return (port + 1);
			}

		// channel (1-15,17-31,32-46,48-62)
		/*if (port <= 15)
			return port;
		else
			if ((port >= 16) && (port <= 45))
				return (port + 1);
			else
				if ((port >= 46) && (port <= 60))
					return (port + 2);
				else
					return 0;*/			
	}

	static struct dgv_pvt *fixup_pri_channel(int dc, struct dgv_pvt *p, q931_call *c, short clear_old_call)
	{
		int x;
		short span;
		struct dgv_pvt *tmp;

		if (!c)
		{
			if (!p)
				return NULL;

			return p;
		}

		if ((p) && (p->call == c))
			return p;

		for (x = 1; x <= dg_GetCardPortsCount(dchans[dc].card); x++)
		{
			if (x <= 30)
				span = 1;
			else
				span = 2;

			if (span == dchans[dc].span)
			{
				tmp = find_ch_port(x);

				if (!tmp)
					continue;

				if (tmp->call == c)
				{
					/* Found our call */
					if (p->port != tmp->port)
					{
						struct dgv_pvt *new = p, *old = tmp;

				        if (option_verbose >= 3)
			    	        ast_verbose(VERBOSE_PREFIX_3 "Moving call from channel %d to channel %d\n", old->port, new->port);

						if ((new->ast_ch) || (new->reserved))
						{
							ast_log(LOG_WARNING, "Can't fix up channel from %d to %d because %d is already in use\n", old->port, new->port, new->port);

							if (clear_old_call)
								old->call = NULL;

							return NULL;
						}

						ast_dgv_lock(&old->lock, old->port);
						{
							/* Fix it all up now */
							new->ast_ch = old->ast_ch;
							old->ast_ch = NULL;
							if (new->ast_ch)
							{
#ifdef CC_AST_HAS_VERSION_11
									char tmpdgvname[256];
									snprintf(tmpdgvname, sizeof(tmpdgvname), "DGV/%d", new->port);
									ast_channel_name_set(new->ast_ch, tmpdgvname);
#else//#ifdef CC_AST_HAS_VERSION_11
	#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
										ast_string_field_build(new->ast_ch, name, "DGV/%d", new->port);
	#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
										snprintf(new->ast_ch->name, sizeof(new->ast_ch->name), "DGV/%d", new->port);
	#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
#endif//#ifdef CC_AST_HAS_VERSION_11
									dgv_channel_tech_pvt_set(new->ast_ch, new);
							}
							else
								ast_log(LOG_WARNING, "Whoa, there's no ast_ch, and we're having to fix up channel %d to channel %d\n", old->port, new->port);

							new->call = old->call;
							old->call = NULL;

							new->pri = old->pri;
							//old->pri = NULL;

							new->reserved = 1;
							old->reserved = 0;

							new->call_state = old->call_state;
							old->call_state = DG_AST_NONE;

							ast_copy_string(new->exten, old->exten, sizeof(new->exten));
							ast_copy_string(new->rdnis, old->rdnis, sizeof(new->rdnis));
							ast_copy_string(new->dnid, old->dnid, sizeof(new->dnid));
							ast_copy_string(new->cid_num, old->cid_num, sizeof(new->cid_num));
							ast_copy_string(new->cid_name, old->cid_name, sizeof(new->cid_name));
							ast_copy_string(new->pri_cid_num, old->pri_cid_num, sizeof(new->pri_cid_num));
							ast_copy_string(new->pri_cid_name, old->pri_cid_name, sizeof(new->pri_cid_name));
						}
						ast_dgv_unlock(&old->lock, old->port);
					} //if (p->port != tmp->port)
					return p;
				} //if (tmp->call == c)
			} //if (span == dchans[dc].span)
		} //for (x = 1; x <= dg_GetCardPortsCount(dchans[dc].card); x++)

		ast_log(LOG_WARNING, "Call specified, but not found?\n");
		return NULL;
	}

	//--------------------------------------
	// Simple switch to isdn - overlap
	//--------------------------------------
	static void *ss_thread(void *data)
	{
	    struct dgv_pvt *p = data;
    	char exten[AST_MAX_EXTENSION]="";
    	int len = 0;
    	int res;
    	int timeout;
    	char ss_name[AST_CHANNEL_NAME];
    	int dc=0;

    	pthread_t threadid;
    	pthread_attr_t attr;

    	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    	if (p->ast_ch)
    	{   	
	        if (option_verbose >= 3) 
    	        ast_verbose(VERBOSE_PREFIX_3 "Starting simple switch on '%s'\n", dgv_channel_name(p->ast_ch));

	        ast_copy_string(ss_name, dgv_channel_name(p->ast_ch), sizeof(ss_name));

			/* Now loop looking for an extension */
        	ast_copy_string(exten, p->exten, sizeof(exten));
        	len = strlen(exten);
        	res = 0;       
      
        	while((len < AST_MAX_EXTENSION-1) && ast_matchmore_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num))
        	{
        		if (len && !ast_ignore_pattern(dgv_channel_context(p->ast_ch), exten))
                	dg_GenerateMF(p->port,GENERATE_OFF,0);
            	else
            	{
                	dg_GenerateMF(p->port,GENERATE_TONE1,0);
                
					if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						ast_log(LOG_NOTICE, "ss_thread GENERATE_TONE1 on %d \n", p->port);                                
    	        }    

        	    if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num))
        		{
	                timeout = matchdigittimeout;                
    	    	}    
        	    else
            	{
                	timeout = gendigittimeout;                
            	}
            
            	/*chan_zap.c send a AST_FRAME_VOICE in zt_read, here we call a thread to send it*/
        	    p->need_fv_thread=1;
    			pthread_attr_init(&attr);
    			pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);         
    	        if (ast_pthread_create(&threadid, &attr, fv_thread, p)>=0) 
        	    {           	
    				if (option_verbose >= 3)
                		ast_log(LOG_NOTICE,"fv_thread Created - port %s \n", dgv_channel_name(p->ast_ch));
	            }
    	        else                                
        	    {            	
            		ast_log(LOG_ERROR, "Error creating fv_thread - port %s \n", dgv_channel_name(p->ast_ch));
            	}
            	pthread_attr_destroy(&attr);	

	            res = ast_waitfordigit(p->ast_ch, timeout);
	            
        		if (!p->ast_ch)
        		{        			
        			if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
        				ast_log(LOG_WARNING, "ss_thread: p->ast_ch does not exist(ast_waitfordigit) on port %d\n", p->port);
        			return NULL;
        		}
	                                    
    	        if (res < 0) {
        	        ast_log(LOG_NOTICE, "ss_thread: waitfordigit returned < 0...\n");
        	        if (p->ast_ch)
        	        	ast_hangup(p->ast_ch);                
                	return NULL;
            	} else if (res) {
	                exten[len++] = res;
    	            exten[len] = '\0';
        	    } else
            	{            	
        	  		if (option_verbose >= 3)	         	
                		ast_verbose(VERBOSE_PREFIX_3 "ss_thread: Timeout on '%s'\n", dgv_channel_name(p->ast_ch));
	                break;
    	        }
        	    usleep(1);
        	}//while

        	dg_GenerateMF(p->port,GENERATE_OFF,0);
        
        	/* if no extension was received ('unspecified') or invalid extension on overlap call, use the 's' extension */
        	if ((ast_strlen_zero(exten)) || (!ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)))
        	{
	            if (option_verbose >= 3)
    	            ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of empty or invalid extension received on overlap call\n");
        	    exten[0] = 's';
            	exten[1] = '\0';
        	}   
                    
        	if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num))
        	{
        		ast_dgv_lock(&p->lock, p->port);
        		{
					if (p->pri)
					{
						if (p->call)
						{
							if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
								ast_log(LOG_NOTICE, "ss_thread: sending pri_proceeding port %d, pri_channel %d\n", p->port, p->pri_channel);

							dc = dg_pri_findpri(p->pri);

							if (!dgv_trylock_ccs(p, dc))
							{
								pri_proceeding(p->pri, p->call, p->pri_channel, !p->digital);
								p->proceeding = 1;

								if (option_verbose >= 3)
									ast_log(LOG_NOTICE, "Sending pri_proceeding %d/%d\n", p->pri_channel, p->port);

								dgv_releaselock_ccs(dc);
							}
							else
								ast_log(LOG_WARNING, "ss_thread: Unable to grab ccs_lock on port %d\n", p->port);
						}
						else
							if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
								ast_log(LOG_NOTICE, "ss_thread: did not call pri_proceeding because p->call is null on %d\n", p->port);
					}
        		}
				ast_dgv_unlock(&p->lock, p->port);

            	/* Start the real PBX */
        		dgv_channel_exten_set(p->ast_ch, exten);
            	ast_setstate(p->ast_ch, AST_STATE_RING);
            	p->call_state=DG_AST_RINGING;
            	p->need_fv_thread=0;

            	res = ast_pbx_run(p->ast_ch);
            	if (res) 
                	ast_log(LOG_WARNING, "PBX exited non-zero!\n");
        	} 
        	else 
        	{
            	ast_log(LOG_WARNING, "No such possible extension '%s' in context '%s'\n", exten, dgv_channel_context(p->ast_ch));
            	dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_UNALLOCATED);
            	ast_hangup(p->ast_ch);
            	p->exten[0] = '\0';
            	/* Since we send release complete here, we won't get one */
            	p->call = NULL;
	        }
    	} //if (p->ast_ch)
    
    	if (option_verbose >= 3) 
        	ast_verbose( VERBOSE_PREFIX_3 "Stopping simple switch on '%s'\n", ss_name);

    	return NULL;
	}

	//--------------------------------------
	// Handles CCS Information
	//--------------------------------------
	void ccs_handler(short card, short span, short size, void *SampleData)
	{
#ifdef DEBUG_PRI
			char data[CCS_DATA_SIZE*4];
			char data2[CCS_DATA_SIZE*4];
#endif//#ifdef DEBUG_PRI
    	unsigned char ccs_data[CCS_DATA_SIZE+16];
    	short x,ret;
    	int dc;

#ifdef DEBUG_PRI_STEP
			ast_log(LOG_NOTICE, "first step on ccs_handler \n");
#endif//#ifdef DEBUG_PRI_STEP
    	
		dc = dg_pri_findindex(card, span);

	    if (size>0)
    	{
#ifdef DEBUG_PRI_STEP
            	ast_log(LOG_NOTICE, "try ccs_lock on ccs_handler %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
			
        	ast_dgv_lock(&ccs_lock[dc], 0);
        	
#ifdef DEBUG_PRI_STEP
        		ccs_lock_count[dc]++;
            	ast_log(LOG_NOTICE, "ccs_lock on ccs_handler %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
        
#ifdef DEBUG_PRI
            	sprintf(data,"Entrou no CCS_Handler card=%d span=%d size=%d\n",card,span,size);
            	primsg(NULL, data);

            	//shows (card/framer/size)
            	sprintf(data2," RX (c: %d /spn: %d /size: %d): ", card, span, size);
#endif//#ifdef DEBUG_PRI
        
        	for (x=0; x< size; x++)
        	{
	            //sprintf(data,"%02x ",ccs_data[i]);
    	        ccs_data[x] = *((u8 *)SampleData + x);
#ifdef DEBUG_PRI
					sprintf(data,"%02x ",ccs_data[x]);
					strcat(data2,data);
#endif//#ifdef DEBUG_PRI
        	}
        	//append two more bytes to pass to libpri
        	ccs_data[size]=0;
        	ccs_data[size+1]=0;
    
        	//send data to lippri
        	if (dc>-1) {
#ifdef DEBUG_PRI
	            	sprintf(data,"CCS_Handler dado --> %s\n", data2);
	            	primsg(NULL, data);
#endif//#ifdef DEBUG_PRI
		            
	        	ret = write(dchans[dc].fds[TX], ccs_data, size+2);   //need to pass two more bytes to libpri
            	
	            if (ret<1)
	                ast_log(LOG_ERROR, "\nERROR WRITE %d\n",ret);
    	        //else
        	    //    printf("\nWriting to libpri %d ret = %d\n",dchans[dc].fds[TX],ret);
	        }
        	//sprintf(data,"Saiu do CCS_Handler card=%d span=%d size=%d\n",card,span,size);
        	//primsg(NULL, data);

#ifdef DEBUG_PRI_STEP
        		ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP
        	
        	ast_dgv_unlock(&ccs_lock[dc], 0);
        	
#ifdef DEBUG_PRI_STEP
            	ast_log(LOG_NOTICE, "ccs_unlock on ccs_handler %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
	    }

#ifdef DEBUG_PRI
        	ast_log(LOG_NOTICE, "ccs_handler exited \n");
#endif//#ifdef DEBUG_PRI

	}

	//Socket monitoring routine
	static void *dgfifo_thread(void *data)
	{
	    unsigned char ccs_data[CCS_DATA_SIZE-1];
    	short i,j;
    	short k;
    	short ret;
    	struct timeval tv;

    	dg_fd_set read_fds;
    	dg_fd_set master_fds;

#ifdef DEBUG_PRI
		    char data2[CCS_DATA_SIZE*4];
    		char data1[CCS_DATA_SIZE*4];
#endif//#ifdef DEBUG_PRI
    
    	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	    FD_ZERO(&master_fds);

	    for (i=0; i<dchans_count; i++) 
    	{   
	        if (dchans[i].fds[TX]>-1)
    	        FD_SET(dchans[i].fds[TX] , &master_fds);
    	}

    	ast_log(LOG_NOTICE, "Starting socket thread - fdmax=%d\n",fdmax);
    	thd_socket_running = 1;
    	while (thd_socket_running)
    	{
	        /* configure select() timeout to 100ms */
    		tv.tv_sec = 100;//0;
        	tv.tv_usec = 10000000;//100000;
        
        	//copy to local copy since read_fs will be modified by select()
        	FD_ZERO(&read_fds);
        	read_fds = master_fds;

#ifdef DEBUG_PRI_STEP
				ast_log(LOG_NOTICE, "Before dgfifo_thread select...\n");
#endif//#ifdef DEBUG_PRI_STEP

        	//waiting for ever
            ret = dg_select(fdmax+1, &read_fds, NULL, NULL, /*NULL*/ &tv );
        	if (ret == -1) {
        		ast_log(LOG_NOTICE, ">>>>>--> Select() returned < 0 - controling dgfifo_thread (isdn), ret %d\n", ret);
	            perror("select");
    	    }

#ifdef DEBUG_PRI_STEP
				ast_log(LOG_NOTICE, "After dgfifo_thread select...ret %d\n", ret);
#endif//#ifdef DEBUG_PRI_STEP

	        //printf("socket_thread: Saiu do select - ret=%d\n",ret);
    	    for (i=0; i<= fdmax; i++)
        	{
            	if (FD_ISSET(i, &read_fds))
            	{
            		// we got one!!
                	//printf("Preparando para dar READ\n",ret);
#ifdef DEBUG_PRI_STEP
						ast_log(LOG_NOTICE, "Before dgfifo_thread read...\n");
#endif//#ifdef DEBUG_PRI_STEP
					while ((ret = read(i, ccs_data, sizeof(ccs_data))) > 0)
	                {
	                    //send data to ccs
    	                k=dg_pri_findfds(i); //k receive array index
        	            if (k>-1) {
#ifdef DEBUG_PRI
		                        sprintf(data1,"dgfifo_thread: (%d) vai enviar CCS placa=%d span=%d size=%d\n",i, dchans[k].card, dchans[k].span,ret);
    		                    primsg(NULL, data1);
        	                	sprintf(data2," TX (c: %d /spn: %d /fds : %d): ", dchans[k].card,dchans[k].span,i);
            	            	for (j=0; j< ret; j++)
                	        	{
                    	        	sprintf(data1,"%02x ",ccs_data[j]);
                        	    	strcat(data2,data1);
	                        	}
	    	                    if (option_debug == 30)
    	    	                	ast_log(LOG_NOTICE,"%s\n", data2);
        	    	            strcat(data2,"\n");
            	    	        primsg(NULL, data2);

#ifdef DEBUG_PRI_STEP
									ast_log(LOG_NOTICE, "Before call dg_SendCCSBuffer...\n");
#endif//#ifdef DEBUG_PRI_STEP

#endif//#ifdef DEBUG_PRI

							ret-=2; //remove 2 bytes of the package (crc will be generated by the chip)

	                        j=dg_SendCCSBuffer(dchans[k].card, dchans[k].span, ccs_data, ret,NULL);

#ifdef DEBUG_PRI_STEP
	                        	ast_log(LOG_NOTICE, "After call dg_SendCCSBuffer...\n");
#endif//#ifdef DEBUG_PRI_STEP
                        
	                        if  (j!=DG_EXIT_SUCCESS) {
    	                        ast_log(LOG_ERROR, "Erro SendCCSBuffer cod %x\n",j);
        	                }
#ifdef DEBUG_PRI
                        		sprintf(data1,"dgfifo_thread (%d): ENVIOU CCS placa=%d span=%d size=%d\n",i, dchans[k].card, dchans[k].span,ret);
                        		primsg(NULL, data1);
#endif//#ifdef DEBUG_PRI
                    	}
                    	else
                    	{
                        	ast_log(LOG_ERROR, "socket ERROR %d - k = %d\n", i, k);
                    	}
                	} //while ((ret = read(i, ccs_data, sizeof(ccs_data))) > 0)
	            } //if (FD_ISSET(i, &read_fds))
    	    } //for (i = 0; i <= fdmax; i++)
	    } //while (thd_socket_running)

    	return 0;
	}

	static void apply_plan_to_number(char *buf, size_t size, const struct dgv_pvt *p, const char *number, const int plan)
	{
		switch (plan) {
			case PRI_INTERNATIONAL_ISDN:		/* Q.931 dialplan == 0x11 international dialplan => prepend international prefix digits */
				snprintf(buf, size, "%s%s", p->internationalprefix, number);
				break;
			case PRI_NATIONAL_ISDN:			/* Q.931 dialplan == 0x21 national dialplan => prepend national prefix digits */
				snprintf(buf, size, "%s%s", p->nationalprefix, number);
				break;
			case PRI_LOCAL_ISDN:			/* Q.931 dialplan == 0x41 local dialplan => prepend local prefix digits */
				snprintf(buf, size, "%s%s", p->localprefix, number);
				break;
			case PRI_PRIVATE:			/* Q.931 dialplan == 0x49 private dialplan => prepend private prefix digits */
				snprintf(buf, size, "%s%s", p->privateprefix, number);
				break;
			case PRI_UNKNOWN:			/* Q.931 dialplan == 0x00 unknown dialplan => prepend unknown prefix digits */
				snprintf(buf, size, "%s%s", p->unknownprefix, number);
				break;
			default:				/* other Q.931 dialplan => don't twiddle with callingnum */
				snprintf(buf, size, "%s", number);
				//0.9.8 break;
		}
	}

	//******************************************************************************************
	// event handler pri
	// Each event must be interpreted and translated to voicerlib events
	// 
	//******************************************************************************************
	static inline void pri_event_handle(struct pri *pri, pri_event *e, int dc)
	{
	    dg_event_data_structure EventContext;
    	short port,span;
    	int group_port[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short is_group[20] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
    	short backwards[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short roundrobin[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short serialnumber[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short serialnumber_port[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short talktime[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	short serialnumber_talktime[20] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    	//char data[256];
    
		char nothing[256];

	    static struct dgv_pvt *p;
	    char plancallingnum[256];

    	pthread_t threadid;
    	pthread_attr_t attr;

    	switch (e->gen.e)
    	{
			case PRI_EVENT_DCHAN_UP:
				ast_log(LOG_NOTICE, ">>>>>--> LINK UP (idx %d)\n", dc);
				//When link is up, all ports from this span will be restarted

				dchans[dc].alarmed = 0;

				/*if (!dchans[dc].resetting)
				{
					dchans[dc].resetting = 1;
					for (port=1; port<=dg_GetCardPortsCount(dchans[dc].card); port++)
					{
						if (port<=30)
							span=1;
						else
							span=2;

						if (span==dchans[dc].span)
						{
							p = find_ch_port(port);

							if (p)
							{
								ast_dgv_lock(&p->lock, p->port);
								{
									if (p->pri)
									{
										if (p->call)
										{
											pri_hangup(p->pri, p->call, -1);
											pri_destroycall(p->pri, p->call);
										}
									}
									//0.9.8 p->pri = NULL;
									p->call = NULL;
									if (p->ast_ch)
									{
										ast_log(LOG_NOTICE, ">>>>>--> Sending hangup to reset channels\n");
										//has a channel associated

										struct ast_frame f = { AST_FRAME_CONTROL, AST_CONTROL_HANGUP, };
										//ast_dgv_lock(&p->lock, p->port);
										//{
										QueueFrame(p, &f);
										//}
										//ast_dgv_unlock(&p->lock, p->port);
									}

									p->call_state=DG_AST_NONE;
									p->reserved=0;
								}
								ast_dgv_unlock(&p->lock, p->port);

								ast_log(LOG_NOTICE, ">>>>>--> Resetting port %d card %d span %d\n", port, dchans[dc].card,dchans[dc].span);
							}
						}
					}
					dchans[dc].resetting = 0;
				}*/

				EventContext.command = EV_E1_ALARM;
				EventContext.data = ALARM_LINKDOWN;
                EventContext.data_aux = OFF;
                EventContext.port = dchans[dc].span;
                EventContext.card = dchans[dc].card - 1;
                event_handler(&EventContext);

				break;
			case PRI_EVENT_DCHAN_DOWN:
				ast_log(LOG_WARNING, ">>>>>--> LINK DOWN (idx %d) PRI 2 (default): %s (%d) - dc = %d\n", dc, pri_event2str(e->gen.e), e->gen.e, dc);

				dchans[dc].alarmed = 1;

				if (!dchans[dc].resetting)
				{
					dchans[dc].resetting = 1;
					for (port=1; port<=dg_GetPortsCount(); port++)
					{
						if (dg_GetCardNumber(port) == dchans[dc].card)
						{
							if (dg_GetRelativeChannelNumber(port)<=30)
								span=1;
							else
								span=2;

							if (span==dchans[dc].span)
							{
								p = find_ch_port(port);

								if (p)
								{
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "B-channel %d down on span %d\n", p->port, dc);

									ast_dgv_lock(&p->lock, p->port);
									{
										if (p->pri)
										{
											if (p->call)
											{
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, "PRI_EVENT_DCHAN_DOWN - before pri_destroycall on %d\n", p->port);

												pri_hangup(p->pri, p->call, -1);
												pri_destroycall(p->pri, p->call);
												p->call = NULL;

												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, "PRI_EVENT_DCHAN_DOWN - after pri_destroycall on %d \n", p->port);
											}
											else
												ast_log(LOG_WARNING, "The PRI Call have not been destroyed\n");
										}

										/* Force soft hangup if appropriate */
										if (p->ast_ch)
										{
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, "PRI_EVENT_DCHAN_DOWN - before AST_SOFTHANGUP_DEV on %d \n", p->port);

#ifdef CC_AST_HAS_VERSION_11
												ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
												p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11

											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, "PRI_EVENT_DCHAN_DOWN - after AST_SOFTHANGUP_DEV on %d \n", p->port);
										}

										p->call_state = DG_AST_NONE;
										p->reserved = 0;
									}
									ast_dgv_unlock(&p->lock, p->port);
								} //if (p)
							} //if (span==dchans[dc].span)
						} //if (dg_GetCardNumber(port) == dchans[dc].card)
					} //for (port=1; port<=dg_GetPortsCount(); port++)
					dchans[dc].resetting = 0;
				}

				EventContext.command = EV_E1_ALARM;
				EventContext.data = ALARM_LINKDOWN;
                EventContext.data_aux = ON;
                EventContext.port = dchans[dc].span;
                EventContext.card = dchans[dc].card - 1;
                event_handler(&EventContext);

				break;
        	case PRI_EVENT_RESTART:
    	        ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_RESTART (idx %d)\n", dc);

				if (e->restart.channel > -1)
				{
                	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
                	     		        dg_pri_to_port(e->restart.channel) + ((dchans[dc].span-1)*30) );

               		p = find_ch_port(EventContext.port);

               		if (p)
               		{
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "B-channel %d restarted on span %d\n", p->port, dc);

						ast_dgv_lock(&p->lock, p->port);
						{
							if (p->pri)
							{
								if (p->call)
								{
			    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
			        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - before pri_destroycall on %d \n", p->port);

			    	                pri_destroycall(p->pri, p->call);
			    	                p->call = NULL;

			    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
			        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - after pri_destroycall on %d \n", p->port);
								}
							}

							/* Force soft hangup if appropriate */
							if (p->ast_ch)
							{
		    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
		        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - before AST_SOFTHANGUP_DEV on %d \n", p->port);

#ifdef CC_AST_HAS_VERSION_11
									ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
									p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11

		    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
		        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - after AST_SOFTHANGUP_DEV on %d \n", p->port);
	                        }

							p->call_state = DG_AST_NONE;
	                        p->reserved = 0;
						}
						ast_dgv_unlock(&p->lock, p->port);
               		}
               		else
               		{
						ast_log(LOG_WARNING, "Restart requested on odd/unavailable channel number %d on span %d\n", e->restart.channel, dc);
               		}
				}
				else
				{
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_2 "Restart on requested on entire span %d\n", dc);

	                for (port=1; port<=dg_GetCardPortsCount(dchans[dc].card); port++)
	                {
        	            if (port<=30)
            	            span=1;
                	    else
                    	    span=2;

                    	if (span==dchans[dc].span)
                    	{
                   			p = find_ch_port(port);

                   			if (p)
                   			{
								if (option_verbose >= 3)
									ast_verbose(VERBOSE_PREFIX_3 "B-channel %d restarted on span %d\n", p->port, dc);

								ast_dgv_lock(&p->lock, p->port);
								{
									if (p->pri)
									{
										if (p->call)
										{
					    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
					        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - before pri_destroycall on %d \n", p->port);

					    	                pri_destroycall(p->pri, p->call);
					    	                p->call = NULL;

					    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
					        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - after pri_destroycall on %d \n", p->port);
										}
									}

									/* Force soft hangup if appropriate */
									if (p->ast_ch)
									{
				    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
				        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - before AST_SOFTHANGUP_DEV on %d \n", p->port);

#ifdef CC_AST_HAS_VERSION_11
											ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
											p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11

				    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
				        	                ast_log(LOG_NOTICE, "PRI_EVENT_RESTART - before AST_SOFTHANGUP_DEV on %d \n", p->port);
		                            }

									p->call_state = DG_AST_NONE;
	    	                        p->reserved = 0;
								}
								ast_dgv_unlock(&p->lock, p->port);
                   			} //if (p)
                    	} //if (span==dchans[dc].span)
					} //for (port=1; port<=dg_GetCardPortsCount(dchans[dc].card); port++)
				}
				break;
        	case PRI_EVENT_INFO_RECEIVED:
				EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
									dg_pri_to_port(e->ring.channel) + ((dchans[dc].span-1)*30) );

				p = find_ch_port(EventContext.port);

				if (p)
				{
					if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						ast_log(LOG_NOTICE, "RECEBEU PRI_EVENT_INFO_RECEIVED-no ch=%d!!!!!!!\n", e->ring.channel);

					p = fixup_pri_channel(dc, p, e->ring.call, 0);
					if (!p)
					{
						ast_log(LOG_WARNING, "INFO received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->ring.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "INFO received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->ring.channel, dc, p->port);

					ast_dgv_lock(&p->lock, p->port);
					{
						/* queue DTMF frame if the PBX for this call was already started (we're forwarding INFORMATION further on */
						if (dchans[dc].overlapdial && p->call==e->ring.call && p->ast_ch)
						{
							int digitlen = strlen(e->ring.callednum);
							char digit;
							int i;

							for (i=0; i<digitlen; i++)
							{
								digit = e->ring.callednum[i];
								{
									struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = digit;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = digit;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
									f.src = "chan_dgv";
									f.datalen = 0;
									QueueFrame(p, &f);
								}
							}
						}
					}
					ast_dgv_unlock(&p->lock, p->port);
				}
				else
					ast_log(LOG_WARNING, "INFO received on unconfigured channel %d, card %d, span %d\n", e->ring.channel, dchans[dc].card, dc);
				break;
        	case PRI_EVENT_RING:
	            if (e->ring.channel == -1)
    	        {
                	if (e->ring.flexible)
                	{
                    	ast_dgv_lock(&iflock, 0);
                    	{
                        	p = find_free_port(group_port, is_group, backwards, dc, roundrobin, serialnumber, serialnumber_port, talktime, serialnumber_talktime, 0);	//dc indicates that function must search only inside the correct span
                    	}
                    	ast_dgv_unlock(&iflock, 0);

                    	if (p)
                    	{
							EventContext.port = p->port;

        	        		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
            					ast_log(LOG_NOTICE, "PRI_EVENT_RING 1 >>>>>--> flex=%d ch=%d p=%d\n", e->ring.flexible, e->ring.channel, p->port);
                    	}
                	}
                	else
                	{
                    	ast_log(LOG_WARNING, "RING no-flexible received, and there is no port available!\n");
                    	p = NULL;
                	}
            	}
            	else
            	{
                	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
                	     		        dg_pri_to_port(e->ring.channel) + ((dchans[dc].span-1)*30) );

               		p = find_ch_port(EventContext.port);

                	if (p)
                	{
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						{
							ast_log(LOG_NOTICE, "PRI_EVENT_RING 2 - dchans[dc].card: %d \n", dchans[dc].card);
							ast_log(LOG_NOTICE, "PRI_EVENT_RING 2 - e->ring.channel: %d \n", e->ring.channel);
							ast_log(LOG_NOTICE, "PRI_EVENT_RING 2 - dchans[dc].span: %d \n", dchans[dc].span);
            				ast_log(LOG_NOTICE, "PRI_EVENT_RING 2 - e->ring.flexible: %d \n", e->ring.flexible);
            				ast_log(LOG_NOTICE, "PRI_EVENT_RING 2 - EventContext.port: %d \n", EventContext.port);
						}
                	}
	            }

    	        //check if port is free
        	    if (p)
        	    {
            	    if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
#ifdef DEBUG_PRI
							ast_log(LOG_NOTICE, "pri_event_handle >>>>>--> (Card: %d Span: %d / pri=%x) PRI 2: %s (%d) - ch=%d, port=%d\n", dchans[dc].card, dchans[dc].span, (unsigned int)pri, pri_event2str(e->gen.e), e->gen.e, e->ring.channel, p->port);
#else//#ifdef DEBUG_PRI
							ast_log(LOG_NOTICE, "pri_event_handle >>>>>--> (Card: %d Span: %d / pri=XX) PRI 2: %s (%d) - ch=%d, port=%d\n", dchans[dc].card, dchans[dc].span,                    pri_event2str(e->gen.e), e->gen.e, e->ring.channel, p->port);
#endif//#ifdef DEBUG_PRI

					ast_dgv_lock(&p->lock, p->port);

					if ((p->ast_ch) || (p->reserved))
					{
						if (p->call == e->ring.call)
						{
							ast_log(LOG_WARNING, "Duplicate setup requested on port %d\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);

							break;
						}
						else
						{
							ast_log(LOG_WARNING, "PRI RING: Port %d is already taken\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);

							if (e->ring.flexible)
							{
								ast_dgv_lock(&iflock, 0);
								{
									p = find_free_port(group_port, is_group, backwards, dc, roundrobin, serialnumber, serialnumber_port, talktime, serialnumber_talktime, 0);	//dc indicates that function must search only inside the correct span
								}
								ast_dgv_unlock(&iflock, 0);

								if (p)
								{
									p->reserved = 1;

									if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
										ast_log(LOG_NOTICE, "PRI_EVENT_RING 3 >>>>>--> flex=%d ch=%d port=%d\n", e->ring.flexible, e->ring.channel, p->port);
								}
								else
								{
									ast_log(LOG_WARNING, "RING flexible received (3), but there is no port available, hanging up!\n");

									if (e->ring.flexible)
										pri_hangup(pri, e->ring.call, PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION);
									else
										pri_hangup(pri, e->ring.call, PRI_CAUSE_REQUESTED_CHAN_UNAVAIL);

									break;
								}
							}
							else
							{
								ast_log(LOG_WARNING, "RING no-flexible received and there is no port available!\n");
								pri_hangup(pri, e->ring.call, PRI_CAUSE_REQUESTED_CHAN_UNAVAIL);

								break;
							}
						}
					}
					else
						ast_dgv_unlock(&p->lock, p->port);

					ast_dgv_lock(&p->lock, p->port);

					/*
					if (e->ring.channel == -1)
						p = find_free_port(0, 0, dc, 0, 0);	//dc indicates that function must search only inside the correct span
					else
						p = find_ch_port(EventContext.port);*/

					if (p)
					{
						p->pri_channel = port_to_pri(p->ch_board);

						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_RING: Now our port is %d, pri_channel %d and ring_channel %d\n", p->port, p->pri_channel, e->ring.channel);
					}
					else
					{
						ast_log(LOG_WARNING, "RING received, but the port %d is not available, hanging up!\n", p->port);
						if (e->ring.flexible)
							pri_hangup(pri, e->ring.call, PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION);
						else
							pri_hangup(pri, e->ring.call, PRI_CAUSE_REQUESTED_CHAN_UNAVAIL);

						ast_dgv_unlock(&p->lock, p->port);

						break;
					}

					p->reserved = 1;
					p->pri = pri;
					p->call = e->ring.call;

					if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						ast_log(LOG_NOTICE, "PRI_RING: Saving the new call reference on port %d\n", p->port);

					if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						ast_log(LOG_NOTICE, "PRI_RING: callingnum: %s \n", e->ring.callingnum);

					apply_plan_to_number(plancallingnum, sizeof(plancallingnum), p, e->ring.callingnum, e->ring.callingplan);

					if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
						ast_log(LOG_NOTICE, "PRI_RING: number: %s %d %s\n", plancallingnum, e->ring.callingplan, e->ring.callednum);

					if (p->use_callerid)
					{
						if (!ast_strlen_zero(plancallingnum))
							ast_shrink_phone_number(plancallingnum);
						
						ast_copy_string(p->cid_num, plancallingnum, sizeof(p->cid_num));
						
						ast_copy_string(p->pri_cid_name, e->ring.callingname, sizeof(p->pri_cid_name));
						p->cid_ton = e->ring.callingplan; /* this is the callingplan (TON/NPI), e->ring.callingplan>>4 would be the TON */

						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "--- PRI_RING: cid_num: %s pri_cid_name:%s\n", p->cid_num, p->pri_cid_name);
					}
					else
					{
						p->cid_num[0] = '\0';
						p->pri_cid_name[0] = '\0';
						p->cid_ton = 0;

						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "--- PRI_RING: No CallerID information. \n");
					}

					apply_plan_to_number(p->rdnis, sizeof(p->rdnis), p,
										 e->ring.redirectingnum, e->ring.callingplanrdnis);
						
						//Reverse charging
#ifdef CC_PRI_HAS_REVERSECHARGE
						p->reverse_charging = e->ring.reversecharge;
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_EVENT_RING 1 - e->ring.reversecharge: %d\n", e->ring.reversecharge);
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE

					/* If immediate=yes go to s|1 */
					if (p->immediate)
					{
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of immediate=yes\n");
						p->exten[0] = 's';
						p->exten[1] = '\0';
					}
					/* Get called number */
					else if (!ast_strlen_zero(e->ring.callednum)) {
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "Getting extension %s  %s\n", e->ring.callednum, p->exten);
						ast_copy_string(p->exten, e->ring.callednum, sizeof(p->exten));
						ast_copy_string(p->dnid, e->ring.callednum, sizeof(p->dnid));
					} else
						p->exten[0] = '\0';

					/* Set DNID on all incoming calls -- even immediate */
					if (!ast_strlen_zero(e->ring.callednum))
						ast_copy_string(p->dnid, e->ring.callednum, sizeof(p->dnid));

					/* No number yet, but received "sending complete"? */
					if (e->ring.complete && (ast_strlen_zero(e->ring.callednum))) {
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of Complete received\n");
						p->exten[0] = 's';
						p->exten[1] = '\0';
					}

					if ( (!dchans[dc].overlapdial) &&
						 (!ast_strlen_zero(p->exten)) &&
						 (!ast_exists_extension(NULL, p->context, p->exten, 1, p->cid_num)) )
					{
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of invalid extension received\n");

						p->exten[0] = 's';
						p->exten[1] = '\0';
					}

					/* Make sure extension exists (or in overlap dial mode, can exist) */
					if ((dchans[dc].overlapdial &&
						 ast_canmatch_extension(NULL, p->context, p->exten, 1, p->cid_num)) ||
						 ast_exists_extension(NULL, p->context, p->exten, 1, p->cid_num) )
					{
						if (e->ring.complete || !dchans[dc].overlapdial)
						{
							pri_proceeding(pri, e->ring.call, p->pri_channel, 0);

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "Sending pri_proceeding - %d/%d\n", p->pri_channel, p->port);
						}
						else
						{
							pri_need_more_info(pri, e->ring.call, p->pri_channel, 1);
						
							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "Sending pri_need_more_info %d/%d\n", p->pri_channel, p->port);
						}

						if (dchans[dc].overlapdial && ast_matchmore_extension(NULL, p->context, p->exten, 1, p->cid_num))
						{
							//overlapdial
#ifdef DEBUG_PRI_STEP
                        		ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP
	                        		
                			ast_dgv_unlock(&ccs_lock[dc], 0);
	                			
#ifdef DEBUG_PRI_STEP
                            	ast_log(LOG_NOTICE, "ccs_unlock on pri_event_handle %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

                            ast_copy_string(p->cid_name, p->pri_cid_name, sizeof(p->cid_name));

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
                            	if (dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
							{
#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
									if (p->use_callerid)
									{
										ast_dgv_unlock(&p->lock, p->port);
										{
											if (p->ast_ch)
											{
												ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);

												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, "PRI_RING: ast_set_callerid(ss_thread) on port %d \n", p->port);
											}
										}
										ast_dgv_lock(&p->lock, p->port);
									}
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC

#ifdef USERUSER_INFO
									ast_dgv_unlock(&p->lock, p->port);
									{
										if ((p->ast_ch) && (!ast_strlen_zero(e->ring.useruserinfo)))
											pbx_builtin_setvar_helper(p->ast_ch, "DGV_USERUSERINFO", e->ring.useruserinfo);
									}
									ast_dgv_lock(&p->lock, p->port);
#endif//#ifdef USERUSER_INFO

								pthread_attr_init(&attr);
								pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
								if (ast_pthread_create(&threadid, &attr, ss_thread, p)>=0)
								{
									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "ss_thread Created - port %d \n", p->port);
								}
								else
								{
									ast_log(LOG_ERROR, "Error creating ss_thread - port %d \n", p->port);
									ast_hangup(p->ast_ch);
								}
								pthread_attr_destroy(&attr);

							}
							else
							{
								ast_log(LOG_ERROR,"Error creating dgv_new to overlapdial - port %d \n", p->port);
								pri_hangup(pri, e->ring.call, PRI_CAUSE_SWITCH_CONGESTION);
								p->call = NULL;
								p->reserved = 0;
							}

#ifdef DEBUG_PRI_STEP
            	               	ast_log(LOG_NOTICE, "try ccs_lock on pri_event_handle %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
            	    			
            	            ast_dgv_lock(&ccs_lock[dc], 0);
            	            	
#ifdef DEBUG_PRI_STEP
								ccs_lock_count[dc]++;
								ast_log(LOG_NOTICE, "ccs_lock on pri_event_handle %d step 1 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
							ast_dgv_unlock(&p->lock, p->port);
						}
						else //if (dchans[dc].overlapdial && ast_matchmore_extension(NULL, p->context, p->exten, 1, p->cid_num))
						{
#ifdef DEBUG_PRI_STEP
    	                    ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP
    	                    	
	                		ast_dgv_unlock(&ccs_lock[dc], 0);
	                			
#ifdef DEBUG_PRI_STEP
	                           	ast_log(LOG_NOTICE, "ccs_unlock on pri_event_handle %d step 2 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
    							
#ifdef USERUSER_INFO
								if (!ast_strlen_zero(e->ring.useruserinfo))
									ast_copy_string(p->useruserinfo, e->ring.useruserinfo, sizeof(p->useruserinfo));
								else
									p->useruserinfo[0] = '\0';
#endif//#ifdef USERUSER_INFO

							if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)))
								ast_log(LOG_NOTICE, "--- PRI_RING: before event_handler(EV_RINGS) \n");


							EventContext.port = p->port;
							EventContext.command = EV_RINGS;

							ast_dgv_unlock(&p->lock, p->port);

							event_handler(&EventContext);

							if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)))
								ast_log(LOG_NOTICE, "--- PRI_RING: after event_handler(EV_RINGS) \n");

#ifdef DEBUG_PRI_STEP
            	                ast_log(LOG_NOTICE, "try ccs_lock on pri_event_handle %d step 2 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
            	                	
            	            ast_dgv_lock(&ccs_lock[dc], 0);
            	            	
#ifdef DEBUG_PRI_STEP
								ccs_lock_count[dc]++;
								ast_log(LOG_NOTICE, "ccs_lock on pri_event_handle %d step 2 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
						} //if (dchans[dc].overlapdial && ast_matchmore_extension(NULL, p->context, p->exten, 1, p->cid_num))
					}
					else //if ((dchans[dc].overlapdial && ...
					{
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "Extension '%s' in context '%s' from '%s' does not exist. Rejecting call on channel %d\n",
										p->exten, p->context, p->cid_num, p->port/*p->logicalspan, p->prioffset, pri->span*/);
						pri_hangup(pri, e->ring.call, PRI_CAUSE_UNALLOCATED);

						p->call = NULL;
						p->exten[0] = '\0';
						p->reserved = 0;

						ast_dgv_unlock(&p->lock, p->port);
					} //if ((dchans[dc].overlapdial && ...
            	}
            	else //if (p)
            	{
            		ast_log(LOG_WARNING, "RING received (1/2), but there is no port available, hanging up!\n");

					if (e->ring.flexible)
						pri_hangup(pri, e->ring.call, PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION);
					else
						pri_hangup(pri, e->ring.call, PRI_CAUSE_REQUESTED_CHAN_UNAVAIL);
	            } //if (p)
    			break;
    	    case PRI_EVENT_RINGING: //network side
        	    EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            			            dg_pri_to_port(e->ringing.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

	            //check if port is free
	            if (p){
        	        if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_RINGING (idx %d) PRI 1: %s (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e);
                	//..q931_facility(pri, e->ringing.call);
                	//..pri_answer(pri, e->ringing.call, e->ringing.channel, 0);

					p = fixup_pri_channel(dc, p, e->ringing.call, 0);
					if (!p)
					{
						ast_log(LOG_WARNING, "RINGING received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->ringing.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "RINGING received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->ringing.channel, dc, p->port);

#ifdef USERUSER_INFO
						if (p->ast_ch && !ast_strlen_zero(e->ringing.useruserinfo))
						{
							struct ast_channel *owner = p->ast_ch;
							//ast_dgv_unlock(&p->lock, p->port);
							pbx_builtin_setvar_helper(owner, "DGV_USERUSERINFO", e->ringing.useruserinfo);
							//ast_dgv_lock(&p->lock, p->port);
						}
#endif//#ifdef USERUSER_INFO
            	}
	            else
					ast_log(LOG_WARNING, "Ringing requested on unconfigured channel %d, card %d, span %d\n", e->ringing.channel, dchans[dc].card, dc);
            	break;
			case PRI_EVENT_PROGRESS:
        	    EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            			            dg_pri_to_port(e->proceeding.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

	            //check if port is free
	            if (p)
	            {
        	        if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_PROGRESS (idx %d) PRI 1: %s (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e);

					p = fixup_pri_channel(dc, p, e->proceeding.call, 0);
					if (!p)
					{
						ast_log(LOG_WARNING, "PROGRESS received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->proceeding.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PROGRESS received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->proceeding.channel, dc, p->port);

					if ((!p->progress)
#ifdef PRI_PROGRESS_MASK
							|| (e->proceeding.progressmask & PRI_PROG_INBAND_AVAILABLE)
#else
							|| (e->proceeding.progress == 8)
#endif
						)
					{
						p->f.frametype = AST_FRAME_CONTROL;
#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass.integer = AST_CONTROL_PROGRESS;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass = AST_CONTROL_PROGRESS;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

						if (e->proceeding.cause > -1)
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "PROGRESS with cause code %d received on port %d\n", e->proceeding.cause, p->port);

							/* Work around broken, out of spec USER_BUSY cause in a progress message */
							if (e->proceeding.cause == AST_CAUSE_USER_BUSY)
							{
								if (p->ast_ch)
								{
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "PROGRESS with 'user busy' received on port %d, signalling AST_CONTROL_BUSY instead of AST_CONTROL_PROGRESS\n", p->port);

									dgv_channel_hangupcause_set(p->ast_ch, e->proceeding.cause);
#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								}
							}
						}

						ast_dgv_lock(&p->lock, p->port);
                        QueueFrame(p, &p->f);

						if (
#ifdef PRI_PROGRESS_MASK
							e->proceeding.progressmask & PRI_PROG_INBAND_AVAILABLE
#else
							e->proceeding.progress == 8
#endif
							)
						{
#ifdef CC_AST_HAS_VERSION_1_8
								p->f.subclass.integer = AST_CONTROL_PROGRESS;
#else//#ifdef CC_AST_HAS_VERSION_1_8
								p->f.subclass = AST_CONTROL_PROGRESS;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
							QueueFrame(p, &p->f);
						}
						p->progress = 1;
						ast_dgv_unlock(&p->lock, p->port);
					}
            	}
	            else
					ast_log(LOG_WARNING, "Progress requested on unconfigured channel %d, card %d, span %d\n", e->proceeding.channel, dchans[dc].card, dc);
            	break;
        	case PRI_EVENT_PROCEEDING:
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
                	                dg_pri_to_port(e->proceeding.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (p) {
                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, ">>>>>--> (idx %d) PRI_EVENT_PROCEEDING: %s (%d) - ch=%d\n", dc, pri_event2str(e->gen.e), e->gen.e, e->proceeding.channel);
            	}
            	break;
    		case PRI_EVENT_ANSWER:
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            			    		dg_pri_to_port(e->answer.channel) + ((dchans[dc].span-1)*30) );
            	
           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (p) {
		    		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
		    			ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_ANSWER (idx %d) PRI 2: %s (%d) (%d)\n", dc, pri_event2str(e->gen.e), e->gen.e, e->answer.channel);
            	
					p = fixup_pri_channel(dc, p, e->answer.call, 0);
					if (!p)
					{
						ast_log(LOG_WARNING, "PRI_EVENT_ANSWER received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->answer.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_EVENT_ANSWER received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->answer.channel, dc, p->port);

					ast_dgv_lock(&p->lock, p->port);
					{
#ifdef USERUSER_INFO
							if (p->ast_ch && !ast_strlen_zero(e->answer.useruserinfo))
							{
								struct ast_channel *owner = p->ast_ch;
								ast_dgv_unlock(&p->lock, p->port);
								pbx_builtin_setvar_helper(owner, "DGV_USERUSERINFO", e->answer.useruserinfo);
								ast_dgv_lock(&p->lock, p->port);
							}
#endif//#ifdef USERUSER_INFO

						EventContext.port = p->port;
						EventContext.command = EV_ANSWERED;
					}
					ast_dgv_unlock(&p->lock, p->port);

#ifdef DEBUG_PRI_STEP
						ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP

					ast_dgv_unlock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
						ast_log(LOG_NOTICE, "ccs_unlock on pri_event_handle %d step 3 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

					if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)))
						ast_log(LOG_NOTICE, "--- PRI_ANSWER: before event_handler(EV_ANSWERED)\n");

					event_handler(&EventContext);

					if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)))
						ast_log(LOG_NOTICE, "--- PRI_ANSWER: after event_handler(EV_ANSWERED)\n");

#ifdef DEBUG_PRI_STEP
						ast_log(LOG_NOTICE, "try ccs_lock on pri_event_handle %d step 3 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

					ast_dgv_lock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
						ccs_lock_count[dc]++;
						ast_log(LOG_NOTICE, "ccs_lock on pri_event_handle %d step 3 \n", ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
            	}
            	else
            		ast_log(LOG_WARNING, "Answer on unconfigured channel %d, card %d, span %d\n", e->answer.channel, dchans[dc].card, dc);
	    		break;
        	case PRI_EVENT_HANGUP:
            	//abs_port must consider the card/span
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->hangup.channel) + ((dchans[dc].span-1)*30) );
            	//EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card, e->hangup.channel + ((dchans[dc].span-1)*30) );
            	
           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (p)
            	{
                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP (idx %d) PRI 2: %s (%d) (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e, p->port);

					p = fixup_pri_channel(dc, p, e->hangup.call, 1);
					if (!p)
					{
						ast_log(LOG_WARNING, "PRI_EVENT_HANGUP received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->hangup.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->hangup.channel, dc, p->port);

            		ast_dgv_lock(&p->lock, p->port);
            		{
	                	if (!p->alreadyhungup)
	                	{
	                    	p->alreadyhungup = 1;
	                    	if (p->ast_ch)
	                    	{
	                    		dgv_channel_hangupcause_set(p->ast_ch, e->hangup.cause);
	                        
								if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	    	    	            	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause %d on port %d \n", e->hangup.cause, p->port);                    
	        	            
	        	    	        int pri_send_ast_control = 0;
	                        
	        	    	        switch (dgv_channel_state(p->ast_ch))
	        	    	        {
									case AST_STATE_BUSY:
									case AST_STATE_UP:
										if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
											ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with _state BUSY or UP on %d \n", p->port);
										pri_send_ast_control = 3;
										break;
	        	    	        	default:
										switch (e->hangup.cause)
										{
											case PRI_CAUSE_USER_BUSY:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_USER_BUSY on %d \n", p->port);
												pri_send_ast_control = 1;
												break;
											case PRI_CAUSE_CALL_REJECTED:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_CALL_REJECTED on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											case PRI_CAUSE_NETWORK_OUT_OF_ORDER:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_NETWORK_OUT_OF_ORDER on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											case PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											case PRI_CAUSE_SWITCH_CONGESTION:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_SWITCH_CONGESTION on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											case PRI_CAUSE_DESTINATION_OUT_OF_ORDER:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_DESTINATION_OUT_OF_ORDER on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											case PRI_CAUSE_NORMAL_TEMPORARY_FAILURE:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause PRI_CAUSE_NORMAL_TEMPORARY_FAILURE on %d \n", p->port);
												pri_send_ast_control = 2;
												break;
											default:
												if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
													ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP with cause default on %d \n", p->port);
												pri_send_ast_control = 3;
										} //switch (e->hangup.cause)
									break;
	        	    	        } //switch (dgv_channel_state(p->ast_ch))
	                        
	                    	    /*
		            	    	struct ast_frame f;
		            	    	f.src = "chan_dgv";
		            	    	f.datalen = 0;
		            	    	*/
		            	    	
	    	                	switch (pri_send_ast_control)
	        	            	{
		        	            	case 1://case PRI_CAUSE_USER_BUSY
						        		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	        	    						ast_log(LOG_NOTICE, "Sending AST_CONTROL_BUSY into PRI_EVENT_HANGUP on '%d' \n", p->port);
	            						                    	
						        		p->f.frametype = AST_FRAME_CONTROL;
#ifdef CC_AST_HAS_VERSION_1_8
											p->f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											p->f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
			                            QueueFrame(p, &p->f);
	            		        		break;
	                		    	case 2://case PRI_CAUSE_ALL
				        				if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            							ast_log(LOG_NOTICE, "Sending AST_CONTROL_CONGESTION into PRI_EVENT_HANGUP on '%d' \n", p->port);
	            					
				        				p->f.frametype = AST_FRAME_CONTROL;
#ifdef CC_AST_HAS_VERSION_1_8
											p->f.subclass.integer = AST_CONTROL_CONGESTION;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											p->f.subclass = AST_CONTROL_CONGESTION;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
	        	        	            QueueFrame(p, &p->f);
	                    				break;
		                    		case 3://case default
				        				if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
				        					ast_log(LOG_NOTICE, "Sending AST_FRAME_VOICE into PRI_EVENT_HANGUP on '%d' \n", p->port);
	
#ifdef CC_AST_HAS_VERSION_11
											ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
											p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11
	
		                	        	p->f.frametype = AST_FRAME_VOICE;

#ifdef CC_AST_HAS_VERSION_13
		                	        		p->f.subclass.format = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
												ast_format_copy(&p->f.subclass.format, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
													p->f.subclass.format = p->ast_ch->rawreadformat;
		#else//#ifdef CC_AST_HAS_VERSION_10
			#ifdef CC_AST_HAS_VERSION_1_8
														p->f.subclass.integer = p->ast_ch->rawreadformat;
			#else//#ifdef CC_AST_HAS_VERSION_1_8
														p->f.subclass = p->ast_ch->rawreadformat;
			#endif//#ifdef CC_AST_HAS_VERSION_1_8
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_VERSION_13
											if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_slin) == AST_FORMAT_CMP_EQUAL)
												p->f.samples = DATALEN;
											else
												if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_gsm) == AST_FORMAT_CMP_EQUAL)
													p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
												else
													if ((ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_alaw) == AST_FORMAT_CMP_EQUAL) ||
														(ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_ulaw) == AST_FORMAT_CMP_EQUAL))
														p->f.samples = DATALEN;
													else
						        						p->f.samples = DATALEN;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
												switch (ast_channel_rawreadformat(p->ast_ch)->id)
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat.id)
		#else//#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat)
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
											{
												case AST_FORMAT_SLINEAR:
													p->f.samples = DATALEN;
													break;
												case AST_FORMAT_GSM:
													p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
													break;
												case AST_FORMAT_ALAW:
												case AST_FORMAT_ULAW:
													p->f.samples = DATALEN;
													break;
												default:
													p->f.samples = DATALEN;
											} //switch (p->ast_ch->rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_13
	
#ifdef CC_AST_HAS_DATA_PTR				        				
											p->f.data.ptr = nothing + AST_FRIENDLY_OFFSET;
#else//#ifdef CC_AST_HAS_DATA_PTR				        				
											p->f.data = nothing + AST_FRIENDLY_OFFSET;
#endif//#ifdef CC_AST_HAS_DATA_PTR
										QueueFrame(p, &p->f);
	        	    	        		break;
	            	    	    	default:
				    	    			if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
		            						ast_log(LOG_WARNING, "Error, case default on pri_send_ast_control(%d), so nothing to do on '%d' \n", pri_send_ast_control, p->port);                    	
	                    		} //switch (pri_send_ast_control)                        
		                    }
	    	            }
	        	        else //if (!p->alreadyhungup)
	        	        {
		                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	    	                	ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP-no ch=%d chanpos=%d - step 3\n",e->hangup.channel, p->port);
		
	    	                if (p->pri)
	    	                {
	    	                	if (p->call)
	    	                	{
	    	                		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	    	                			ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP - calling pri_hangup... e->hangup.cause %d on port %d\n", e->hangup.cause, p->port);
	               	        
	    	                		pri_hangup(p->pri, p->call, e->hangup.cause);
	    	                		p->call = NULL;
	    	                	}
	    	                	else
    	                			ast_log(LOG_WARNING, "PRI_EVENT_HANGUP - did not call pri_hangup because p->call is null on %d\n", p->port);

	    	                }
	    	                else
	                			ast_log(LOG_WARNING, "PRI_EVENT_HANGUP - did not call pri_hangup because p->pri is null on %d\n", p->port);
	    	                
	    	                p->reserved = 0;
	                	} //if (!p->alreadyhungup)
                	
						if (e->hangup.cause == PRI_CAUSE_REQUESTED_CHAN_UNAVAIL)
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "Forcing restart of channel %d (card %d / span %d) since channel reported in use\n", e->hangup.channel,
											dchans[dc].card+1, dchans[dc].span+1);

							pri_reset(pri, e->hangup.channel);
							dchans[dc].resetting = 1;
						}

#ifdef USERUSER_INFO
							/*if (p->ast_ch && !ast_strlen_zero(e->hangup.useruserinfo))
							{
								struct ast_channel *owner = p->ast_ch;
								ast_dgv_unlock(&p->lock, p->port);
								pbx_builtin_setvar_helper(owner, "DGV_USERUSERINFO", e->hangup.useruserinfo);
								ast_dgv_lock(&p->lock, p->port);
							}*/
#endif//#ifdef USERUSER_INFO
            		}
            		ast_dgv_unlock(&p->lock, p->port);
            	}
            	else
                	ast_log(LOG_WARNING, "Hangup requested on unconfigured channel %d, card %d, span %d\n", e->hangup.channel, dchans[dc].card, dc);
	    		break;
    	    case PRI_EVENT_HANGUP_REQ: //network side
        	    //abs_port must consider the card/span
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->hangup.channel) + ((dchans[dc].span-1)*30) );
            	//EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card, e->hangup.channel + ((dchans[dc].span-1)*30) );
            	
           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (p)
            	{
	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
    	                ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ (idx %d) PRI 1: %s (%d) (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e, p->port);

					p = fixup_pri_channel(dc, p, e->hangup.call, 1);
					if (!p)
					{
						ast_log(LOG_WARNING, "PRI_EVENT_HANGUP_REQ received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->hangup.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP_REQ received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->hangup.channel, dc, p->port);

	                ast_dgv_lock(&p->lock, p->port);
	                {
	        	        if (p->ast_ch)
	            	    {
	        	        	dgv_channel_hangupcause_set(p->ast_ch, e->hangup.cause);
	                    
							if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	    	                	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause %x on port %d \n", e->hangup.cause, p->port);                    
	                    
	        	            int pri_send_ast_control = 0;
	                    
        	    	        switch (dgv_channel_state(p->ast_ch))
        	    	        {
								case AST_STATE_BUSY:
								case AST_STATE_UP:
									if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
										ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with _state BUSY or UP on %d \n", p->port);
									pri_send_ast_control = 3;
									break;
        	    	        	default:
									switch (e->hangup.cause)
									{
										case PRI_CAUSE_USER_BUSY:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_USER_BUSY on %d \n", p->port);
											pri_send_ast_control = 1;
											break;
										case PRI_CAUSE_CALL_REJECTED:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_CALL_REJECTED on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										case PRI_CAUSE_NETWORK_OUT_OF_ORDER:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_NETWORK_OUT_OF_ORDER on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										case PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_NORMAL_CIRCUIT_CONGESTION on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										case PRI_CAUSE_SWITCH_CONGESTION:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_SWITCH_CONGESTION on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										case PRI_CAUSE_DESTINATION_OUT_OF_ORDER:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_DESTINATION_OUT_OF_ORDER on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										case PRI_CAUSE_NORMAL_TEMPORARY_FAILURE:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause PRI_CAUSE_NORMAL_TEMPORARY_FAILURE on %d \n", p->port);
											pri_send_ast_control = 2;
											break;
										default:
											if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
												ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_REQ with cause default on %d \n", p->port);
											pri_send_ast_control = 3;
									} //switch (e->hangup.cause)
								break;
	        	    	    } //switch (dgv_channel_state(p->ast_ch))

	            	        /*
	            	    	struct ast_frame f;
	            	    	f.src = "chan_dgv";
	            	    	f.datalen = 0;
	            	    	*/
	            	    	
	                    	switch (pri_send_ast_control)
	                    	{
	                    		case 1://case PRI_CAUSE_USER_BUSY
				        			if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            						ast_log(LOG_NOTICE, "Sending AST_CONTROL_BUSY into PRI_EVENT_HANGUP_REQ on '%d' \n", p->port);
	            					                    	
				        			p->f.frametype = AST_FRAME_CONTROL;
#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
	                            	QueueFrame(p, &p->f);
	                    			break;
	                    		case 2://case PRI_CAUSE_ALL
				        			if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            						ast_log(LOG_NOTICE, "Sending AST_CONTROL_CONGESTION into PRI_EVENT_HANGUP_REQ on '%d' \n", p->port);
	            					
				        			p->f.frametype = AST_FRAME_CONTROL;
#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass.integer = AST_CONTROL_CONGESTION;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass = AST_CONTROL_CONGESTION;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
	    	                        QueueFrame(p, &p->f);
	                    			break;
	                    		case 3://case default
				        			if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            						ast_log(LOG_NOTICE, "Sending AST_FRAME_VOICE into PRI_EVENT_HANGUP_REQ on '%d' \n", p->port);
				        		
#ifdef CC_AST_HAS_VERSION_11
										ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
										p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11
	
	                        		p->f.frametype = AST_FRAME_VOICE;

#ifdef CC_AST_HAS_VERSION_13
	                        			p->f.subclass.format = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
											ast_format_copy(&p->f.subclass.format, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
												p->f.subclass.format = p->ast_ch->rawreadformat;
		#else//#ifdef CC_AST_HAS_VERSION_10
			#ifdef CC_AST_HAS_VERSION_1_8
													p->f.subclass.integer = p->ast_ch->rawreadformat;
			#else//#ifdef CC_AST_HAS_VERSION_1_8
													p->f.subclass = p->ast_ch->rawreadformat;
			#endif//#ifdef CC_AST_HAS_VERSION_1_8
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_VERSION_13
										if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_slin) == AST_FORMAT_CMP_EQUAL)
											p->f.samples = DATALEN;
										else
											if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_gsm) == AST_FORMAT_CMP_EQUAL)
												p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
											else
												if ((ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_alaw) == AST_FORMAT_CMP_EQUAL) ||
														(ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_ulaw) == AST_FORMAT_CMP_EQUAL))
													p->f.samples = DATALEN;
												else
													p->f.samples = DATALEN;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
												switch (ast_channel_rawreadformat(p->ast_ch)->id)
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat.id)
		#else//#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat)
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
										{
											case AST_FORMAT_SLINEAR:
												p->f.samples = DATALEN;
												break;
											case AST_FORMAT_GSM:
												p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
												break;
											case AST_FORMAT_ALAW:
											case AST_FORMAT_ULAW:
												p->f.samples = DATALEN;
												break;
											default:
												p->f.samples = DATALEN;
												//0.9.8 break;
										}//switch (p->ast_ch->rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_DATA_PTR
										p->f.data.ptr = nothing + AST_FRIENDLY_OFFSET;
#else//#ifdef CC_AST_HAS_DATA_PTR
										p->f.data = nothing + AST_FRIENDLY_OFFSET;
#endif//#ifdef CC_AST_HAS_DATA_PTR
									QueueFrame(p, &p->f);
	                    			break;
		                    	default:
					        		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	        	    					ast_log(LOG_WARNING, "Error, case default on pri_send_ast_control_REQ(%d), so nothing to do on '%d' \n", pri_send_ast_control, p->port);                    	
	                	    } //switch (pri_send_ast_control)
		                }
	    	            else //if (p->ast_ch)
	        	        {
	    	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	        	                ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP_REQ-no ch=%d chanpos=%d - hangup cause %x\n",e->hangup.channel, p->port, e->hangup.cause);
	
	    	                if (p->pri)
	    	                {
	    	                	if (p->call)
	    	                	{
	    	                		pri_hangup(p->pri, p->call, e->hangup.cause);
	    	    	                p->call = NULL;
	    	                	}
	    	                	else
    	                			ast_log(LOG_WARNING, "PRI_EVENT_HANGUP_REQ - did not call pri_hangup because p->call is null on %d\n", p->port);
	    	                }
	    	                else
	                			ast_log(LOG_WARNING, "PRI_EVENT_HANGUP_REQ - did not call pri_hangup because p->pri is null on %d\n", p->port);
	                	} //if (p->ast_ch)
                
						if (e->hangup.cause == PRI_CAUSE_REQUESTED_CHAN_UNAVAIL)
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "Forcing restart of channel %d (card %d / span %d) since channel reported in use\n", e->hangup.channel,
											dchans[dc].card+1, dchans[dc].span+1);

							pri_reset(pri, e->hangup.channel);
							dchans[dc].resetting = 1;
						} //if (e->hangup.cause == PRI_CAUSE_REQUESTED_CHAN_UNAVAIL)

#ifdef USERUSER_INFO
							/*if (p->ast_ch && !ast_strlen_zero(e->hangup.useruserinfo))
							{
								struct ast_channel *owner = p->ast_ch;
								ast_dgv_unlock(&p->lock, p->port);
								pbx_builtin_setvar_helper(owner, "DGV_USERUSERINFO", e->hangup.useruserinfo);
								ast_dgv_lock(&p->lock, p->port);
							}*/
#endif//#ifdef USERUSER_INFO
	                }
	                ast_dgv_unlock(&p->lock, p->port);
            	}
            	else
                	ast_log(LOG_WARNING, "Hangup REQ requested on unconfigured channel %d, card %d, span %d\n", e->hangup.channel, dchans[dc].card, dc);
	            break;
    	    case PRI_EVENT_HANGUP_ACK:
            	//abs_port must consider the card/span
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->hangup.channel) + ((dchans[dc].span-1)*30) );
            
           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (p) 
            	{
	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                   	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_ACK STEP 1 (idx %d) PRI 1: %s (%d) (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e, p->port);
	
					p = fixup_pri_channel(dc, p, e->hangup.call, 1);
					if (!p)
					{
						ast_log(LOG_WARNING, "PRI_EVENT_HANGUP_ACK received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->hangup.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI_EVENT_HANGUP_ACK received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->hangup.channel, dc, p->port);

	           		ast_dgv_lock(&p->lock, p->port);
	           		{
	                	p->call = NULL;
	                	dchans[dc].resetting = 0;
	                	p->reserved=0;
	                	
	                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                    	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_HANGUP_ACK STEP 2 (idx %d) PRI 1: %s (%d) (%d)\n", dc,pri_event2str(e->gen.e), e->gen.e, p->port);

						if (p->ast_ch)
		                    if (option_verbose >= 3)
	    	                    ast_verbose(VERBOSE_PREFIX_3 "Channel %d, card %d and span %d got hangup ACK\n", e->hangup.channel, dchans[dc].card, dc);

#ifdef USERUSER_INFO
							if (p->ast_ch && !ast_strlen_zero(e->hangup.useruserinfo))
							{
								struct ast_channel *owner = p->ast_ch;
								ast_dgv_unlock(&p->lock, p->port);
								pbx_builtin_setvar_helper(owner, "DGV_USERUSERINFO", e->hangup.useruserinfo);
								ast_dgv_lock(&p->lock, p->port);
							}
#endif//#ifdef USERUSER_INFO
            		}
            		ast_dgv_unlock(&p->lock, p->port);
            	}
            	else
            		ast_log(LOG_WARNING, "Hangup ACK requested on unconfigured channel %d, card %d, span %d\n", e->hangup.channel, dchans[dc].card, dc);
            	break;
			case PRI_EVENT_CONFIG_ERR:
				ast_log(LOG_WARNING, "PRI Error on span %d: %s\n", dc, e->err.err);
				break;
			case PRI_EVENT_RESTART_ACK:
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->restartack.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

            	//check if port is free
            	if (!p)
            	{
            		for (port=1; port<=dg_GetCardPortsCount(dchans[dc].card); port++)
            		{
            			if (dchans[dc].resetting)
            			{
							if (port<=30)
								span=1;
							else
								span=2;

							if (span==dchans[dc].span)
							{
								p = find_ch_port(port);
								if (p)
								{
									ast_dgv_lock(&p->lock, p->port);
									{
										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "Assuming restart ack is really for channel %d, card %d and span %d\n", e->restartack.channel, dchans[dc].card, dc);

										if (p->ast_ch)
										{
											ast_log(LOG_WARNING, "Got restart ack on channel %d with owner on card %d and span %d\n", e->restartack.channel, dchans[dc].card, dc);
#ifdef CC_AST_HAS_VERSION_11
												ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
												p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11
										}
										dchans[dc].resetting = 0;

										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "B-channel %d successfully restarted on card %d and span %d\n", e->restartack.channel, dchans[dc].card, dc);
									}
									ast_dgv_unlock(&p->lock, p->port);

									break;
								}
							}
            			}
            		} //for (port=1; port<=dg_GetCardPortsCount(dchans[dc].card); port++)
					if (!p)
						ast_log(LOG_WARNING, "Restart ACK requested on strange channel %d, card %d and span %d\n", e->restartack.channel, dchans[dc].card, dc);
            	}
            	else
            	{
            		ast_dgv_lock(&p->lock, p->port);
            		{
            			if (p->ast_ch)
            			{
            				ast_log(LOG_WARNING, "Got restart ack on channel %d, card %d and span %d with owner\n", e->restartack.channel, dchans[dc].card, dc);
#ifdef CC_AST_HAS_VERSION_11
								ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
								p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11
            			}
            			dchans[dc].resetting = 0;

            			if (option_verbose >= 3)
            				ast_verbose(VERBOSE_PREFIX_3 "B-channel %d successfully restarted on card %d and span %d\n", e->restartack.channel, dchans[dc].card, dc);
            		}
            		ast_dgv_unlock(&p->lock, p->port);
            	}
				break;
        	case PRI_EVENT_SETUP_ACK:
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->setup_ack.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

				if (p)
				{
                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, ">>>>>--> (idx %d) PRI_EVENT_SETUP_ACK: %s (%d) - ch=%d\n", dc, pri_event2str(e->gen.e), e->gen.e, e->setup_ack.channel);

					p = fixup_pri_channel(dc, p, e->setup_ack.call, 0);
					if (!p)
					{
						ast_log(LOG_WARNING, "SETUP_ACKNOWLEDGE received on channel %d, but fixup_pri_channel did not find a properly port at dc %d\n", e->setup_ack.channel, dc);
						break;
					}
					else
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "SETUP_ACKNOWLEDGE received on channel %d and fixup_pri_channel(%d, ...) got port %d\n", e->setup_ack.channel, dc, p->port);
				}
				else
            		ast_log(LOG_WARNING, "Received SETUP_ACKNOWLEDGE on unconfigured channel %d, card %d, span %d\n", e->setup_ack.channel, dchans[dc].card, dc);
            	break;
			case PRI_EVENT_NOTIFY:
            	EventContext.port = dg_GetAbsolutePortNumber(dchans[dc].card,
            						dg_pri_to_port(e->notify.channel) + ((dchans[dc].span-1)*30) );

           		p = find_ch_port(EventContext.port);

            	if (p)
            	{
	                if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                   	ast_log(LOG_NOTICE, ">>>>>--> PRI_EVENT_NOTIFY STEP 1 (idx %d) PRI 1: %s (%d) (%d)\n", dc, pri_event2str(e->gen.e), e->gen.e, p->port);

	                if (dchans[dc].discardremoteholdretrieval)
	                {
						ast_dgv_lock(&p->lock, p->port);
						{
							p->f.frametype = AST_FRAME_CONTROL;
							switch (e->notify.info)
							{
								case PRI_NOTIFY_REMOTE_HOLD:
#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass.integer = AST_CONTROL_HOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass = AST_CONTROL_HOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										QueueFrame(p, &p->f);
										break;
								case PRI_NOTIFY_REMOTE_RETRIEVAL:
#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										p->f.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										QueueFrame(p, &p->f);
										break;
							}
						}
						ast_dgv_unlock(&p->lock, p->port);
	                }
            	}
            	else
					ast_log(LOG_WARNING, "Received NOTIFY on unconfigured channel %d, card %d and span %d\n", e->notify.channel, dchans[dc].card, dc);
				break;
			default:
            	ast_log(LOG_NOTICE, ">>>>>--> (idx %d) PRI 2 (default): %s (%d) - dc = %d (case default)\n", dc, pri_event2str(e->gen.e), e->gen.e, dc);
		} //switch (e->gen.e)
	}

	//---------------------------------------------------------------
	// Function that handles dchannels data from pri
	//---------------------------------------------------------------
	static void *dchan(void *data)
	{
		// Joint D-channel
		struct timeval *next, tv;
		pri_event *e;
		dg_fd_set fds;
		int res;
    	int dc=0;
    	//int max_fds=0;
    	//short card,span;
    	dg_fd_set master_fds;

    	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "first step on dchan \n");
#endif//#ifdef DEBUG_PRI

    	/* stores pri file descriptor */
    	FD_ZERO(&master_fds);
    	for (dc=0; dc<dchans_count; dc++)
    	{
	        if (dchans[dc].fds[RX]>-1)
            	FD_SET(dchans[dc].fds[RX] , &master_fds);
    	}

		thd_pri_running = 1;

    	while (thd_pri_running)
    	{
        	/* runs through pris */
        	FD_ZERO(&fds);
        	fds = master_fds;

	        //send data to ccs
    		for (dc = 0; dc < dchans_count; dc++)
    		{
#ifdef DEBUG_PRI_STEP
					ast_log(LOG_NOTICE, "try ccs_lock on dchan %d, ccs_lock_count %d, step 1 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

    			ast_dgv_lock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
	        	ccs_lock_count[dc]++;
            	ast_log(LOG_NOTICE, "ccs_lock on dchan %d, ccs_lock_count %d, step 1 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

    			/* Find lowest available d-channel */
    			if (!dchans[dc].pri)
    			{
    				ast_log(LOG_ERROR, ">>>>>--> Error, !dchans[%d].pri", dc);
    				ast_dgv_unlock(&ccs_lock[dc], 0);
    				break;
    			}

				if ((next = pri_schedule_next(dchans[dc].pri)))
				{
					/* We need relative time here */
					tv = ast_tvsub(*next, ast_tvnow());
					if (tv.tv_sec < 0) {
						tv = ast_tv(0,0);
					}

					if (tv.tv_sec > 60)
					{
						tv = ast_tv(60, 0);
					}

				}
				else
				{
					/* Don't poll for more than 60 seconds */
					tv = ast_tv(60, 0);
				}

#ifdef DEBUG_PRI_STEP
					ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP

    			ast_dgv_unlock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
        		if (next)
        			ast_log(LOG_NOTICE, "ccs_unlock on dchan %d step 1, next, ccs_lock_count %d\n", dc, ccs_lock_count[dc]);
        		else
    				ast_log(LOG_NOTICE, "ccs_unlock on dchan %d step 1, ccs_lock_count %d\n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
    		}

    		res = dg_select(fdmax + 2, &fds, NULL, NULL, next ? &tv : NULL);

			if (res < 0)
	            ast_log(LOG_NOTICE, ">>>>>--> Select() returned < 0 - controling dchan (isdn), res %d\n", res);
			else
				if (res==0)
				{
#ifdef DEBUG_PRI_STEP
						ast_log(LOG_NOTICE, "dchan res==0\n");
#endif//#ifdef DEBUG_PRI_STEP

					//select returns timeout - no events
					for (dc=0; dc<dchans_count; dc++)
					{
#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "dchan res 0 for dc %d \n", dc);
#endif//#ifdef DEBUG_PRI_STEP

#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "try ccs_lock on dchan %d, ccs_lock_count %d, step 2, select %d \n", dc, ccs_lock_count[dc], res);
#endif//#ifdef DEBUG_PRI_STEP

						ast_dgv_lock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
						ccs_lock_count[dc]++;
						ast_log(LOG_NOTICE, "ccs_lock on dchan dc %d , ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

						/* Just a timeout, run the scheduler */
						if (dchans[dc].pri)
						{
#ifdef DEBUG_PRI_STEP
								ast_log(LOG_NOTICE, "dchan before pri_schedule_run, %d \n", dc);
#endif//#ifdef DEBUG_PRI_STEP

							e = pri_schedule_run(dchans[dc].pri);

							if (e)
							{
#ifdef DEBUG_PRI_STEP
									ast_log(LOG_NOTICE, "before pri_event_handle on dchan %d (timeout), ccs_lock_count %d, step 2, dc %d \n", dc, ccs_lock_count[dc], dc);
#endif//#ifdef DEBUG_PRI_STEP

								pri_event_handle(dchans[dc].pri, e, dc);

#ifdef DEBUG_PRI_STEP
									ast_log(LOG_NOTICE, "after pri_event_handle on dchan %d (timeout), ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
							} //if (e)

#ifdef DEBUG_PRI_STEP
								ast_log(LOG_NOTICE, "dchan after pri_schedule_run \n");
#endif//#ifdef DEBUG_PRI_STEP
						} //if (dchans[dc].pri)

#ifdef DEBUG_PRI_STEP
							ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP

						ast_dgv_unlock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "ccs_unlock on dchan %d, ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
					} //for (dc=0; dc<dchans_count; dc++)
#ifdef DEBUG_PRI_STEP
						ast_log(LOG_NOTICE, "dchan end with select==0 \n");
#endif//#ifdef DEBUG_PRI_STEP
				}
				else //if (res==0)
				{
					for (dc=0; dc<dchans_count; dc++)
					{
#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "dchan res %d for dc %d \n", res, dc);
#endif//#ifdef DEBUG_PRI_STEP

#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "try ccs_lock on dchan %d, ccs_lock_count %d, step 2, select %d \n", dc, ccs_lock_count[dc], res);
#endif//#ifdef DEBUG_PRI_STEP

						ast_dgv_lock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
						ccs_lock_count[dc]++;
						ast_log(LOG_NOTICE, "ccs_lock on dchan dc %d , ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP

						if (FD_ISSET(dchans[dc].fds[RX], &fds))
						{
#ifdef DEBUG_PRI_STEP
								ast_log(LOG_NOTICE, "dchan dc %d, step FD_ISSET \n", dc);
#endif//#ifdef DEBUG_PRI_STEP

							if (dchans[dc].pri)
							{
#ifdef DEBUG_PRI_STEP
									ast_log(LOG_NOTICE, "dchan dc %d, step (dchans[dc].pri) \n", dc);
#endif//#ifdef DEBUG_PRI_STEP

								while ((e = pri_check_event(dchans[dc].pri)))
								{
#ifdef DEBUG_PRI_STEP
										ast_log(LOG_NOTICE, "before pri_event_handle on dchan %d (ok), ccs_lock_count %d, step 2, dc %d \n", dc, ccs_lock_count[dc], dc);
#endif//#ifdef DEBUG_PRI_STEP

									pri_event_handle(dchans[dc].pri, e, dc);

#ifdef DEBUG_PRI_STEP
										ast_log(LOG_NOTICE, "after pri_event_handle on dchan %d (ok), ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
								} //while ((e = pri_check_event(dchans[dc].pri)))
							} //if (dchans[dc].pri)
						} //if (FD_ISSET(dchans[dc].fds[RX], &fds))

#ifdef DEBUG_PRI_STEP
							ccs_lock_count[dc]--;
#endif//#ifdef DEBUG_PRI_STEP

						ast_dgv_unlock(&ccs_lock[dc], 0);

#ifdef DEBUG_PRI_STEP
							ast_log(LOG_NOTICE, "ccs_unlock on dchan %d, ccs_lock_count %d, step 2 \n", dc, ccs_lock_count[dc]);
#endif//#ifdef DEBUG_PRI_STEP
					} //for (dc=0; dc<dchans_count; dc++)
				} //if (res==0)
		} //while (thd_pri_running)

		return NULL;

	} //static void *dchan(void *data)
#endif//#ifdef USE_LIBPRI

//------------------------------------------------------------------------------------
//thread used to outgoing call(fxs card), like ss_thread used in isdn with overlapdial
//------------------------------------------------------------------------------------
static void *fxss_thread(void *data)
{
	struct dgv_pvt *p = data;
	struct ast_channel *ast_ch_pickup_call = NULL;
    char exten[AST_MAX_EXTENSION]="";
    int len = 0;
    int res;    
    char fxss_name[AST_CHANNEL_NAME];
    
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_features_pickup_config *, pickup_cfg, NULL, ao2_cleanup);
#endif//#ifdef CC_AST_HAS_VERSION_12
    const char *pickupexten;

	pthread_t threadid;
	pthread_attr_t attr;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    if ((p) && (p->ast_ch) && (p->reserved))
    {
        if (option_verbose >= 3)
            ast_verbose(VERBOSE_PREFIX_3 "Starting fx simple switch on port %d\n", p->port);
        
        p->fxss_thread_running = 1;

        ast_dgv_lock(&p->lock, p->port);
        {
        	if ((p->ast_ch) && (p->reserved))
        		ast_copy_string(fxss_name, dgv_channel_name(p->ast_ch), sizeof(fxss_name));
        	else
        	{
				if (option_verbose >= 3) 
    				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on port %d (step 1)\n", p->port);
				
				ast_dgv_unlock(&p->lock, p->port);
				
				if (p->ast_ch)
				{
					ast_hangup(p->ast_ch);
					
					if (option_verbose >= 3)
    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because port %d already hungup (step 1)\n", p->port);					
				}
				
				p->fxss_thread_running = 0;				
				return NULL;
        	}
        }
        ast_dgv_unlock(&p->lock, p->port);

#ifdef CC_AST_HAS_VERSION_12
			ast_channel_lock(p->ast_ch);
			{
				pickup_cfg = ast_get_chan_features_pickup_config(p->ast_ch);
				if (!pickup_cfg) {
					ast_log(LOG_ERROR, "Unable to retrieve pickup configuration options. Unable to detect call pickup extension\n");
					pickupexten = "";
				} else {
					pickupexten = ast_strdupa(pickup_cfg->pickupexten);
				}
			}
			ast_channel_unlock(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_12
			pickupexten = ast_strdupa(ast_pickup_ext());
#endif//#ifdef CC_AST_HAS_VERSION_12

        /* Now loop looking for an extension */
        ast_copy_string(exten, p->exten, sizeof(exten));
        len = strlen(exten);
        res = 0;

		while ( (p->ast_ch) && (p->reserved) && (len < AST_MAX_EXTENSION-1) && (!p->immediate) &&				
		        ( (ast_matchmore_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)) ||
		          (!strncmp(exten, pickupexten, strlen(exten)))
		        )
		      )
        {
			if ((p->ast_ch) && (p->reserved))
			{
				if (len == 0)
					p->fv_thread_timeout = p->dialtone_timeout;
				else
        	        p->fv_thread_timeout = p->interdigit_timeout;

				/*chan_zap.c send a AST_FRAME_VOICE in zt_read, here we call a thread to send it*/
				p->need_fv_thread=1;
    			pthread_attr_init(&attr);
    			pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);    			
            	if (ast_pthread_create(&threadid, &attr, fv_thread, p)>=0)
            	{
	    			if (option_verbose >= 3)
    	            	ast_log(LOG_NOTICE,"fv_thread Created - port %s \n", fxss_name);
        	    }
            	else
	            {
    	        	ast_log(LOG_ERROR, "Error creating fv_thread - port %s \n", fxss_name);
        	    }        	    
        	    pthread_attr_destroy(&attr);        	    

        	    if ((p->ast_ch) && (p->reserved))
        	    	res = ast_waitfordigit(p->ast_ch, p->fv_thread_timeout);
        	    else
            	{
    				if (option_verbose >= 3)
        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s'(step 2)\n", fxss_name);
    				
					if (p->ast_ch)
					{
						ast_hangup(p->ast_ch);
						
						if (option_verbose >= 3)
	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 2)\n", fxss_name);    						
					}

    				p->fxss_thread_running = 0;    				
    				return NULL;
            	}

        	    ast_dgv_lock(&p->lock, p->port);
        	    {
        	    	if ((p->ast_ch) && (p->reserved))
					{
						if (dg_GetPortStatus(p->port) == spNone && p->sig != DG_SIG_CB_FXS)
						{
	    					if (option_verbose >= 3) 
	        					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 3)\n", fxss_name);
							
	    					ast_dgv_unlock(&p->lock, p->port);
	    					
	    					if (p->ast_ch)
	    						ast_hangup(p->ast_ch);
	    					
							p->fxss_thread_running = 0;
							return NULL;
						}
					}
					else
					{
						if (option_verbose >= 3) 
	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s' (step 3)\n", fxss_name);
						
						ast_dgv_unlock(&p->lock, p->port);
						
    					if (p->ast_ch)
    					{
    						ast_hangup(p->ast_ch);
    						
    						if (option_verbose >= 3)
    	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 4)\n", fxss_name);    						
    					}

						p->fxss_thread_running = 0;						
	    				return NULL;
					}
        	    }
				ast_dgv_unlock(&p->lock, p->port);

            	if (res < 0)
            	{
            		if (option_verbose >= 3)
            			ast_log(LOG_NOTICE, "fxss_thread: waitfordigit returned < 0...\n");

					if (p->ast_ch)
					{
						ast_hangup(p->ast_ch);
						
						if (option_verbose >= 3)
	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 5)\n", fxss_name);    						
					}

					p->fxss_thread_running = 0;
                	return NULL;
            	}
            	else 
            		if (res) 
            		{
            			if (strchr("#", res))
            			{
            				if (option_verbose >= 3)
            					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: digit(#) detected on '%s'\n", fxss_name);
            				break;
            			}

                		exten[len++] = res;
                		exten[len] = '\0';
            		}
            		else
            		{
        		    	if (option_verbose >= 3)
		        			ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Timeout on '%s'\n", fxss_name);
		                break;
            		}

					if (ast_ignore_pattern(p->context, exten))
					{
						dg_GenerateMF(p->port, GENERATE_TONE1, 0);
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: we got an ignore pattern, GENERATE_TONE1 again on '%s'\n", fxss_name);
					}
			}
			else
			{
            	if (option_verbose >= 3)
		        	ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: already hangup on '%s'\n", fxss_name);
                break;
			}
			usleep(1);
        } //while

		if (p)
		{
	        /* if no extension was received ('unspecified') on overlap call, use the 's' extension */
    	    if ((ast_strlen_zero(exten)) && (!p->immediate)) 
        	{
				if (dg_GetPortStatus(p->port) == spNone && p->sig == DG_SIG_CB_FXS)
				{
					if (option_verbose >= 3)
    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 6)\n", fxss_name);

					if (p->ast_ch)
					{
						ast_hangup(p->ast_ch);
						
						if (option_verbose >= 3)
	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 7)\n", fxss_name);						
					}
				}
				else
				{	
	            	if (option_verbose >= 3)
	                	ast_verbose(VERBOSE_PREFIX_3 "Calling dgv_indicate because of empty extension received on overlap call \n");           
	            
		            if (p->ast_ch)
	    	        {          
#ifdef CC_AST_HAS_INDICATE_DATA        
		            		dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION, NULL, 0);
#else//#ifdef CC_AST_HAS_INDICATE_DATA	
		            		dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA
	    	        }
				}
    	    }
        	else //if ((ast_strlen_zero(exten)) && (!p->immediate))
        	{
        		if ((p->ast_ch) && (p->reserved))
        		{
		        	if ((ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)) && (strcmp(exten, pickupexten)))
		        	{
	    	        	if (option_verbose >= 3)
    	    	        	ast_verbose(VERBOSE_PREFIX_3 "Starting the real PBX... \n");

	    	        	ast_dgv_lock(&p->lock, p->port);
	    	        	{
	    	        		if ((p->ast_ch) && (p->reserved))
	    	        		{
		    		    	    /* Start the real PBX */
								dgv_channel_exten_set(p->ast_ch, exten);

				        	    ast_setstate(p->ast_ch, AST_STATE_RING);
		    		        	p->call_state=DG_AST_RINGING;
	    	        		}
	    	        		else
	    	        		{
	    						if (option_verbose >= 3) 
	    	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s' (step 4)\n", fxss_name);
	    						
	    						ast_dgv_unlock(&p->lock, p->port);
	    						
    	    					if (p->ast_ch)
    	    					{
    	    						ast_hangup(p->ast_ch);
    	    						
    	    						if (option_verbose >= 3)
    	    	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 8)\n", fxss_name);    	    						
    	    					}
	    						
	    						p->fxss_thread_running = 0;
	    	    				return NULL;
	    	        		}
	    	        	}
	    	        	ast_dgv_unlock(&p->lock, p->port);

#ifndef CC_AST_HAS_VERSION_1_8
							if ((strlen(p->szPortID) > 1) && (p->ast_ch) && (p->reserved))
							{
								if (strlen(p->szPortName) > 1)
								{
									ast_set_callerid(p->ast_ch, p->szPortID, p->szPortName, NULL);
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s and name %s\n", p->port, p->szPortID, p->szPortName);
								}
								else
								{
									ast_set_callerid(p->ast_ch, p->szPortID, NULL, NULL);
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s \n", p->port, p->szPortID);
								}
							}
							else
							{
								if (option_verbose >= 3)
									ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): No callerid information. \n", p->port);
							}
#endif//#ifndef CC_AST_HAS_VERSION_1_8
	    	            p->need_fv_thread=0;

	    	        	if ((p->ast_ch) && (p->reserved))	    	        		
	    	        	{
	    	        		res = ast_pbx_run(p->ast_ch);
	    	        		
	    	        		if (res) 
	    	        			ast_log(LOG_WARNING, "PBX exited non-zero!\n");
	    	        	}
	    	        	else
	    	        	{
    						if (option_verbose >= 3) 
    	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s' (step 5)\n", fxss_name);
    						
	    					if (p->ast_ch)
	    					{
	    						ast_hangup(p->ast_ch);
	    						
	    						if (option_verbose >= 3)
	    	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 9)\n", fxss_name);    	    						
	    					}
    						
    						p->fxss_thread_running = 0;
    	    				return NULL;
	    	        	}
    		    	}
    		    	else //if ((ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)) && (strcmp(exten, pickupexten)))
    	    		{
    					if (!strcmp(exten, pickupexten))
    	    			{
    						if ((p->ast_ch) && (p->reserved))
    						{
    							short ret=0;

								//enable inputbuffer streaming with agc disabled
				                dg_PauseInputBuffer(p->port, DG_RELEASE);
				                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
				                	ast_log(LOG_WARNING, "fxss_thread: Releasing InputBuffer on port %d\n", p->port);

				                ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
				                if (ret != DG_EXIT_SUCCESS)
				                    ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n",p->port,ret);
				                else
				                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				                        ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n",p->port);

								p->call_state = DG_AST_CONVERSATION;

#ifndef CC_AST_HAS_VERSION_1_8
									if ((strlen(p->szPortID) > 1) && (p->ast_ch) && (p->reserved))
									{
										if (strlen(p->szPortName) > 1)
										{
											ast_set_callerid(p->ast_ch, p->szPortID, p->szPortName, NULL);
											if (option_verbose >= 3)
												ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s and name %s\n", p->port, p->szPortID, p->szPortName);
										}
										else
										{
											ast_set_callerid(p->ast_ch, p->szPortID, NULL, NULL);
											if (option_verbose >= 3)
												ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s \n", p->port, p->szPortID);
										}
									}
									else
									{
										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): No callerid information. \n", p->port);
									}
#endif//#ifndef CC_AST_HAS_VERSION_1_8

								if (option_verbose >= 3)
									ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): Trying to pickup a call...\n", p->port);

#ifdef CC_AST_HAS_VERSION_1_4
									ast_dgv_lock(&p->lock, p->port);
#endif//#ifdef CC_AST_HAS_VERSION_1_4

		    	        		if (!p->ast_ch)
		    	        		{
		    						if (option_verbose >= 3)
		    	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch, so stopping fx simple switch on '%s' (step 6)\n", fxss_name);

		    						p->fxss_thread_running = 0;
#ifdef CC_AST_HAS_VERSION_1_4
										ast_dgv_unlock(&p->lock, p->port);
#endif//#ifdef CC_AST_HAS_VERSION_1_4
		    	    				return NULL;
		    	        		}

								ast_ch_pickup_call = p->ast_ch;

    							if (ast_pickup_call(p->ast_ch))
    							{
    								if (option_verbose >= 3)
    									ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): No call pickup possible.\n", p->port);
    								
    								p->call_state = DG_AST_DIALING;
    								
#ifdef CC_AST_HAS_VERSION_1_4
										ast_dgv_unlock(&p->lock, p->port);
#endif//#ifdef CC_AST_HAS_VERSION_1_4

#ifdef CC_AST_HAS_INDICATE_DATA        
    									dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION, NULL, 0);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
								        dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA
    							}
    							else
    							{
    								if (option_verbose >= 3)
    			    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Call picked up, calling ast_hangup on '%s'...\n", fxss_name);
    								
#ifdef CC_AST_HAS_VERSION_1_4
										ast_dgv_unlock(&p->lock, p->port);
#endif//#ifdef CC_AST_HAS_VERSION_1_4

        	    					if (ast_ch_pickup_call)
        	    						ast_hangup(ast_ch_pickup_call);
        	    					else
        	    					{
    	        						if (option_verbose >= 3)
    	        	        				ast_verbose(VERBOSE_PREFIX_3 "!ast_ch_pickup_call, so stopping fx simple switch on '%s' (step 7)\n", fxss_name);
        	    					}
    							}
    							ast_ch_pickup_call = NULL;
    						}
    						else //if ((p->ast_ch) && (p->reserved))
    							ast_log(LOG_WARNING, "No such possible extension '%s' in context...\n", exten);
    	    			}
    					else //if (!strcmp(exten, pickupexten))
    					{
			            	if (option_verbose >= 3)
	    		    		{
	            				if (p->immediate)
	        	    			{
	            					if (option_verbose >= 3)        	    				
			            	    		ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of immediate option is enable \n");
		            			}            		
	    	        			else
	            				{
	            					if (option_verbose >= 3)            					
			    	            		ast_verbose(VERBOSE_PREFIX_3 "Going to extension s|1 because of empty extension received on overlap call \n");
		            			}	                	
			    	    	}
	    	    	
	    			        exten[0] = 's';
	        			    exten[1] = '\0';
	        			    
			        		if (((p->ast_ch) && (p->reserved)) && (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)))
				        	{
			    	        	ast_dgv_lock(&p->lock, p->port);
			    	        	{
			    	        		if ((p->ast_ch) && (p->reserved))
			    	        		{			        			
		    			        		/* Start the real PBX */
			    	    		    	dgv_channel_exten_set(p->ast_ch, exten);
		        		    
			    	    		    	ast_setstate(p->ast_ch, AST_STATE_RING);
			    	    		    	p->call_state=DG_AST_RINGING;
			    	        		}
			    	        		else
			    	        		{
			    						if (option_verbose >= 3) 
			    	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s' (step 8)\n", fxss_name);
			    						
			    						ast_dgv_unlock(&p->lock, p->port);
			    						
		    	    					if (p->ast_ch)
		    	    					{
		    	    						ast_hangup(p->ast_ch);
		    	    						
		    	    						if (option_verbose >= 3)
		    	    	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 10)\n", fxss_name);		    	    						
		    	    					}
			    						
			    						p->fxss_thread_running = 0;			    						
			    	    				return NULL;
			    	        		}
			    	        	}
			    	        	ast_dgv_unlock(&p->lock, p->port);

#ifndef CC_AST_HAS_VERSION_1_8
									if ((strlen(p->szPortID) > 1) && (p->ast_ch) && (p->reserved))
									{
										if (strlen(p->szPortName) > 1)
										{
											ast_set_callerid(p->ast_ch, p->szPortID, p->szPortName, NULL);
											if (option_verbose >= 3)
												ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s and name %s\n", p->port, p->szPortID, p->szPortName);
										}
										else
										{
											ast_set_callerid(p->ast_ch, p->szPortID, NULL, NULL);
											if (option_verbose >= 3)
												ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): set callerid %s \n", p->port, p->szPortID);
										}
									}
									else
									{
										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "fxss_thread (%d): No callerid information. \n", p->port);
									}
#endif//#ifndef CC_AST_HAS_VERSION_1_8
	    	    	            p->need_fv_thread=0;

	    	    	        	if ((p->ast_ch) && (p->reserved))
	    	    	        	{
	    	    	        		res = ast_pbx_run(p->ast_ch);
	    	    	        		
	    	    	        		if (res) 
	    	    	        			ast_log(LOG_WARNING, "PBX exited non-zero!\n");
		        		   		}
	    	    	        	else
	    	    	        	{
	        						if (option_verbose >= 3) 
	        	        				ast_verbose(VERBOSE_PREFIX_3 "!p->ast_ch or !p->reserved, so stopping fx simple switch on '%s' (step 9)\n", fxss_name);
	        						
        	    					if (p->ast_ch)
        	    					{
        	    						ast_hangup(p->ast_ch);
	        	    						
        	    						if (option_verbose >= 3)
        	    	    					ast_verbose(VERBOSE_PREFIX_3 "fxss_thread: Calling ast_hangup because '%s' already hungup (step 11)\n", fxss_name);	        	    						
        	    					}
	        						
	        						p->fxss_thread_running = 0;
	        	    				return NULL;
	    	    	        	}
			    	    	} 
			    	    	else//if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num))
		    		    	{    	    	
			    	    		if ((p->ast_ch) && (p->reserved))
	        			    	{
		    	    	    		ast_log(LOG_WARNING, "No such possible extension '%s' in context '%s' \n", exten, dgv_channel_context(p->ast_ch));
#ifdef CC_AST_HAS_INDICATE_DATA        
								        dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION, NULL, 0);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
								        dgv_indicate(p->ast_ch, AST_CONTROL_CONGESTION);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA            
	        		    		}
	            				else
	    	    			    	ast_log(LOG_WARNING, "No such possible extension '%s' in context... \n", exten);            		    	    
			    	    	} //if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num))
    					} //if (!strcmp(exten, pickupexten))
        			} //if ((ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), exten, 1, p->cid_num)) && (strcmp(exten, pickupexten)))
        		} //if (p->ast_ch)
        	} //if ((ast_strlen_zero(exten)) && (!p->immediate))
		} //if (p)	
    } //if ((p) && (p->ast_ch) && (p->reserved))
   
    if (option_verbose >= 3)
    {
    	if (p)
    		ast_verbose( VERBOSE_PREFIX_3 "Stopping fx simple switch on port %d\n", p->port);
    	else
    		ast_verbose( VERBOSE_PREFIX_3 "Stopping fx simple switch on 'unknown port'\n");
    }

	if (p)
		p->fxss_thread_running = 0;

    return NULL;
}

/*Send frametype as AST_FRAME_VOICE; ***zt_read*/
static void *fv_thread(void *data)
{
    struct dgv_pvt *p = data;

	char nothing[256];
	
	char fv_name[AST_CHANNEL_NAME];

#ifdef CC_AST_HAS_VERSION_13
		struct ast_format *fv_rawreadformat;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
			struct ast_format fv_rawreadformat;
	#else//#ifdef CC_AST_HAS_VERSION_10
			short fv_rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

    int i, timeout;

    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    if (p)
    {
    	ast_dgv_lock(&p->lock, p->port);
    	{
    		if (p->ast_ch)
    		{
    			timeout = (p->fv_thread_timeout + 1000) / 20;    		
    		
				ast_copy_string(fv_name, dgv_channel_name(p->ast_ch), sizeof(fv_name));

#ifdef CC_AST_HAS_VERSION_13
						fv_rawreadformat = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
							ast_format_copy(&fv_rawreadformat, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
							fv_rawreadformat = p->ast_ch->rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13
    		}
    		else
    		{
    			if ((option_debug == 30) || (option_debug == 50))
    				ast_log(LOG_WARNING, "fv_thread: p->ast_ch does not exist on port %d\n", p->port);
    			ast_dgv_unlock(&p->lock, p->port);
    			return NULL;
    		}
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    		
    	for (i=0; i<timeout; i++)
    	{
			if ((p) && (p->ast_ch) && (p->need_fv_thread))
			{
				struct ast_frame fr = { AST_FRAME_VOICE, };
#ifdef CC_AST_HAS_VERSION_10
					fr.subclass.format = fv_rawreadformat;
#else//#ifdef CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_1_8
						fr.subclass.integer = fv_rawreadformat;
	#else//#ifdef CC_AST_HAS_VERSION_1_8
						fr.subclass = fv_rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_10
				fr.datalen = 0;

#ifdef CC_AST_HAS_VERSION_13
					if (ast_format_cmp(fv_rawreadformat, ast_format_slin) == AST_FORMAT_CMP_EQUAL)
						fr.samples = DATALEN;
					else
						if (ast_format_cmp(fv_rawreadformat, ast_format_gsm) == AST_FORMAT_CMP_EQUAL)
							fr.samples = DATALEN * (DATALEN/GSM_DATALEN);
						else
							if ((ast_format_cmp(fv_rawreadformat, ast_format_alaw) == AST_FORMAT_CMP_EQUAL) ||
								(ast_format_cmp(fv_rawreadformat, ast_format_ulaw) == AST_FORMAT_CMP_EQUAL))
								fr.samples = DATALEN;
							else
								fr.samples = DATALEN;
#else//#ifdef CC_AST_HAS_VERSION_13
#ifdef CC_AST_HAS_VERSION_10
						switch (fv_rawreadformat.id)
#else//#ifdef CC_AST_HAS_VERSION_10
						switch (fv_rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_10
					{
						case AST_FORMAT_SLINEAR:
							fr.samples = DATALEN;
							break;
						case AST_FORMAT_GSM:
							fr.samples = DATALEN * (DATALEN/GSM_DATALEN);
							break;
						case AST_FORMAT_ALAW:
						case AST_FORMAT_ULAW:
							fr.samples = DATALEN;
							break;
						default:
							fr.samples = DATALEN;
							//0.9.8 break;
					} //switch (fv_rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_13
				
#ifdef CC_AST_HAS_DATA_PTR				
					fr.data.ptr = nothing + AST_FRIENDLY_OFFSET;
#else//#ifdef CC_AST_HAS_DATA_PTR				
					fr.data = nothing + AST_FRIENDLY_OFFSET;
#endif//#ifdef CC_AST_HAS_DATA_PTR				

				fr.src = "chan_dgv";

				if ((option_debug == 30) || (option_debug == 50))
					ast_verbose(VERBOSE_PREFIX_3 "Sending QueueFrame into fv_thread on '%s' \n", fv_name);        								

				if ((p) && (p->ast_ch) && (p->need_fv_thread))
				{
					ast_dgv_lock(&p->lock, p->port);
					{
						if (p->ast_ch)
							QueueFrame(p, &fr);
						else
			    		{
			    			if ((option_debug == 30) || (option_debug == 50))
			    				ast_log(LOG_WARNING, "fv_thread: p->ast_ch does not exist on '%s'\n", fv_name);
			    		}
					}
					ast_dgv_unlock(&p->lock, p->port);

            		usleep(20 * 1000L);
				}
				else
				{
					usleep(20 * 1000L);
					
					break;
				}
			}
			else //if ((p) && (p->ast_ch) && (p->need_fv_thread))
			{
				usleep(20 * 1000L);
				
				break;
			}
    	} //for (i=0; i<timeout; i++)
    }

	return NULL;
}

/*enable echo cancellation*/
static void *dialplan_echo_thread(void *data)
{
	struct dgv_pvt *p = data;
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12
	const char *dialplan_ec = NULL;
	const char *dialplan_et = NULL;

	char de_name[AST_CHANNEL_NAME];
    int i, nTimeout, ret;
    short nIsBridged = 0;
    int res_sched;

    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    if (p)
    {
		if (!p->ast_ch)
		{
			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "dialplan_echo_thread: p->ast_ch does not exist on port %d\n", p->port);
			return NULL;
		}
    	
		//enable callprogress during conversation
		ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
    	if (ret != DG_EXIT_SUCCESS)
        	ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n", p->port, ret);
		else
        	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            	ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n", p->port);    	
    	
    	ast_dgv_lock(&p->lock, p->port);
    	{
    		if (p->ast_ch)
    		{
    			nTimeout = 100;
				ast_copy_string(de_name, dgv_channel_name(p->ast_ch), sizeof(de_name));
    		}
    		else
    		{
    			if (option_verbose >= 3)
    				ast_log(LOG_WARNING, "dialplan_echo_thread: p->ast_ch does not exist on port %d\n", p->port);
    			ast_dgv_unlock(&p->lock, p->port);
    			return NULL;
    		}

    		/*if (p->call_state != DG_AST_CONVERSATION)
    		{
    			if (option_verbose >= 3)
    				ast_log(LOG_WARNING, "dialplan_echo_thread: call_state(%d) not in DG_AST_CONVERSATION on port %d\n", p->call_state, p->port);
    			ast_dgv_unlock(&p->lock, p->port);
    			return NULL;
    		}*/
    	}
    	ast_dgv_unlock(&p->lock, p->port);
    		
    	for (i=0; i<nTimeout; i++)
    	{
    		ast_dgv_lock(&p->lock, p->port);

			if ((p) && (p->ast_ch))
			{
				dgv_chan_tmp = dgv_bridged_channel(p->ast_ch);
				if (dgv_chan_tmp)
				{
					nIsBridged = 1;

                	ast_dgv_unlock(&p->lock, p->port);

                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                		ast_log(LOG_NOTICE, "dialplan_echo_thread: Before pbx_builtin_getvar_helper(DGV_ECHOTAPS), port %d\n", p->port);

                	if ((p->ast_ch) && (p->reserved))
                		dialplan_ec = pbx_builtin_getvar_helper(p->ast_ch, "DGV_ECHOTAPS");
                	else
                		ast_log(LOG_NOTICE, "dialplan_echo_thread: p->ast_ch does not exist on port %d\n", p->port);

                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                		ast_log(LOG_NOTICE, "dialplan_echo_thread: After pbx_builtin_getvar_helper(DGV_ECHOTAPS), port %d\n", p->port);                                	

                	ast_dgv_lock(&p->lock, p->port);				
					
					if (dialplan_ec)
					{
                		p->dialplan_echocan = atoi(dialplan_ec);

                		if (option_verbose >= 3)
                			ast_log(LOG_NOTICE, "Port %d Changing Echo Cancellation Taps %s to %s\n",
                					p->port, echo_taps_desc[p->echo_can], echo_taps_desc[p->dialplan_echocan]);

                		ast_dgv_unlock(&p->lock, p->port);

                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "dialplan_echo_thread: Before pbx_builtin_getvar_helper(DGV_ECHOTRAINING), port %d\n", p->port);

                        if (p->ast_ch)
							dialplan_et = pbx_builtin_getvar_helper(p->ast_ch, "DGV_ECHOTRAINING");
                        else
							ast_log(LOG_WARNING, "dialplan_echo_thread: p->ast_ch does not exist on port %d\n", p->port);

                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "dialplan_echo_thread: After pbx_builtin_getvar_helper(DGV_ECHOTRAINING), port %d\n", p->port);

                        ast_dgv_lock(&p->lock, p->port);

						if (dialplan_et)
						{
							p->dialplan_echotraining = atoi(dialplan_et);

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "Port %d Changing Echo Training %d to %d\n",
										p->port, p->echo_training, p->dialplan_echotraining);
						}
						else //if (dialplan_et)
							p->dialplan_echotraining = p->echo_training;
					}
					else //if (dialplan_ec)
					{
						p->dialplan_echocan = p->echo_can;
						p->dialplan_echotraining = p->echo_training;
					}
					ast_dgv_unlock(&p->lock, p->port);
					break;
				}
				else //if (dgv_chan_tmp)
				{
					ast_dgv_unlock(&p->lock, p->port);

					if (option_debug == 50)
						ast_verbose(VERBOSE_PREFIX_3 "dialplan_echo_thread: not yet, so sleeping and trying again on '%s' \n", de_name);

					if ((p) && (p->ast_ch))
					{
	            		usleep(10 * 1000L);
	            		//and try again
					}
					else //if ((p) && (p->ast_ch))
					{
		    			if (option_verbose >= 3)
		    				ast_log(LOG_WARNING, "dialplan_echo_thread(1): p || p->ast_ch does not exist on '%s' \n", de_name);				
		    			return NULL;
					} //if ((p) && (p->ast_ch))
				} //if (dgv_chan_tmp)
			}
			else //if ((p) && (p->ast_ch))
			{
				ast_dgv_unlock(&p->lock, p->port);

    			if (option_verbose >= 3)
    				ast_log(LOG_WARNING, "dialplan_echo_thread(2): p || p->ast_ch does not exist on '%s' \n", de_name);
    			return NULL;
			} //if ((p) && (p->ast_ch))
    	} //for (i=0; i<timeout; i++)

    	//ast_dgv_lock(&p->lock, p->port);
    	//{
		if ((p) && (p->ast_ch))
		{
			if (!nIsBridged)
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    				ast_log(LOG_NOTICE, "dialplan_echo_thread: not Bridged, setting default echo cancellation on '%s' \n", de_name);

				p->dialplan_echocan = p->echo_can;
				p->dialplan_echotraining = p->echo_training;
			}

			if (p->dialplan_echocan)
			{
#ifdef K_ECHO
					if (dg_EnableEchoCancelation(p->port, p->echo_type, p->dialplan_echocan, p->dialplan_echotraining, p->echo_nlp) != DG_EXIT_SUCCESS)
#else//#ifdef K_ECHO
					if (dg_EnableEchoCancelation(p->port, p->dialplan_echocan, p->dialplan_echotraining, p->echo_nlp) != DG_EXIT_SUCCESS)
#endif//#ifdef K_ECHO
					ast_log(LOG_ERROR, "Port %d ERROR ENABLING DSP ECHO CANCELLATION\n", p->port);
				else
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "Port %d Enabling(4) DSP Echo Cancellation - taps = %s\n", p->port, echo_taps_desc[p->dialplan_echocan]);

					if ((dg_GetCardInterface(p->board) == DG_DIGITAL_INTERFACE) && (p->echo_type == ECHO_HW_DSP))
					{
						//only echotraining
						if (p->all_echotraining_sched > -1)
						{
							res_sched = ast_sched_del(sched, p->all_echotraining_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

							p->all_echotraining_sched = -1;
						}
						p->all_echotraining_sched = ast_sched_add(sched, p->dialplan_echotraining + 5000, all_echotraining_callback, p);
						if (p->all_echotraining_sched < 0)
							ast_log(LOG_NOTICE, "Port %d ERROR Setting timeout to all_echotraining(1) \n", p->port);
					}
				}
			} //if (p->dialplan_echocan)
		}
		else //if ((p) && (p->ast_ch))
		{
			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "dialplan_echo_thread(3): p || p->ast_ch does not exist on '%s' \n", de_name);
		} //if ((p) && (p->ast_ch))
    	//}
    	//ast_dgv_unlock(&p->lock, p->port);
    } //if (p)

	return NULL;
}

/*enable vconf*/
static void *vconf_thread(void *data)
{
	struct dgv_pvt *p = data;
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12

	char v_name[AST_CHANNEL_NAME];
    int i, nTimeout, ret;
    short nIsBridged = 0;

    struct dgv_pvt *p_bridged;
	pthread_t threadid;
	pthread_attr_t attr;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    if (!p)
    {
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "vconf_thread: p does not exist!\n");

		return NULL;
    }

	if (!p->ast_ch)
	{
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "vconf_thread: p->ast_ch does not exist on port %d\n", p->port);

		goto go_out;
	}

	ast_dgv_lock(&p->lock, p->port);
	{
		if (p->ast_ch)
		{
			nTimeout = 100;
			ast_copy_string(v_name, dgv_channel_name(p->ast_ch), sizeof(v_name));
		}
		else
		{
			ast_dgv_unlock(&p->lock, p->port);

			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "vconf_thread: p->ast_ch does not exist on port %d\n", p->port);

			goto go_out;
		}
	}
	ast_dgv_unlock(&p->lock, p->port);

	for (i=0; i<nTimeout; i++)
	{
		ast_dgv_lock(&p->lock, p->port);

		if ((p) && (p->ast_ch))
		{
			dgv_chan_tmp = dgv_bridged_channel(p->ast_ch);
			if (dgv_chan_tmp)
			{
				nIsBridged = 1;

				ast_dgv_unlock(&p->lock, p->port);
				break;
			}
			else //if (dgv_chan_tmp)
			{
				ast_dgv_unlock(&p->lock, p->port);

				if (option_debug == 50)
					ast_verbose(VERBOSE_PREFIX_3 "vconf_thread: not yet, so sleeping and trying again on '%s' \n", v_name);

				if ((p) && (p->ast_ch))
				{
					usleep(10 * 1000L);
					//and try again
				}
				else //if ((p) && (p->ast_ch))
				{
					if (option_verbose >= 3)
						ast_log(LOG_WARNING, "vconf_thread(1): p || p->ast_ch does not exist on '%s' \n", v_name);

					goto go_out;
				} //if ((p) && (p->ast_ch))
			} //if (dgv_chan_tmp)
		}
		else //if ((p) && (p->ast_ch))
		{
			ast_dgv_unlock(&p->lock, p->port);

			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "vconf_thread(2): p || p->ast_ch does not exist on '%s' \n", v_name);

			goto go_out;
		} //if ((p) && (p->ast_ch))
	} //for (i=0; i<timeout; i++)

	if ((p) && (p->ast_ch))
	{
		if (!nIsBridged)
		{
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "vconf_thread: not Bridged, nothing to do on '%s' \n", v_name);
		}
		else
		{
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "vconf_thread (%d) - vconf detected a bridged channel...\n", p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				//check if there is a bridged channel with
				if (dgv_channel_tech_pvt(dgv_chan_tmp) != NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype, 3))
				{
					p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);
					if (p_bridged->vconf_pass_through)
					{
						if (p->vconf_port_bridged == p_bridged->port)
						{
							ret=dg_LocalBridgeConnect(p->port, p->vconf_port_bridged);
							if (ret!=DG_EXIT_SUCCESS)
								ast_log(LOG_ERROR, "Error connecting vconf for port %d and port %d - cod %x\n", p->port, p->vconf_port_bridged, ret);
							else
							{
								ast_log(LOG_WARNING, "vconf connected port %d and port %d\n", p->port, p->vconf_port_bridged);
								p->vconf_established = p_bridged->vconf_established = 1;
							}
						}
						else
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_WARNING, "vconf_thread (%d): port bridged must be %d (dropping vconf)\n", p->port, p->vconf_port_bridged);
					}
					else
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_WARNING, "vconf_thread (%d): %s must has a vconf enabled (dropping vconf)\n", p->port, dgv_channel_name(dgv_chan_tmp));
				}
				else
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_WARNING, "vconf_thread (%d): %s is not a DGV channel (dropping vconf)\n", p->port, dgv_channel_name(dgv_chan_tmp));
			}
			ast_dgv_unlock(&p->lock, p->port);
		} //if (!nIsBridged)
	}
	else //if ((p) && (p->ast_ch))
	{
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "vconf_thread(3): p || p->ast_ch does not exist on '%s' \n", v_name);
	} //if ((p) && (p->ast_ch))

go_out:
	if (p->vconf_established == 0)
	{
		pthread_attr_init(&attr);
		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
		if (ast_pthread_create(&threadid, &attr, dialplan_echo_thread, p)>=0)
		{
			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "dialplan_echo_thread Created(4) - port %d \n", p->port);
		}
		else
			ast_log(LOG_ERROR, "Error creating dialplan_echo_thread - port %d \n", p->port);
		pthread_attr_destroy(&attr);
	}

	return NULL;
}

/*wait for a ccb_pass_through*/
static void *ccb_thread(void *data)
{
	struct dgv_pvt *p = data;
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12

	char v_name[AST_CHANNEL_NAME];
    int i, nTimeout;

    struct dgv_pvt *p_bridged;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    if (!p)
    {
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "ccb_thread: p does not exist!\n");

		return NULL;
    }

	if (!p->ast_ch)
	{
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "ccb_thread: p->ast_ch does not exist on port %d\n", p->port);

		return NULL;
	}

	ast_dgv_lock(&p->lock, p->port);
	{
		if (p->ast_ch)
		{
			nTimeout = 10;
			ast_copy_string(v_name, dgv_channel_name(p->ast_ch), sizeof(v_name));
		}
		else
		{
			ast_dgv_unlock(&p->lock, p->port);

			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "ccb_thread: p->ast_ch does not exist on port %d\n", p->port);

			return NULL;
		}
	}
	ast_dgv_unlock(&p->lock, p->port);

	for (i=0; i<nTimeout; i++)
	{
		ast_dgv_lock(&p->lock, p->port);

		if ((p) && (p->ast_ch))
		{
			dgv_chan_tmp = dgv_bridged_channel(p->ast_ch);
			if (dgv_chan_tmp)
			{
				if (dgv_channel_tech_pvt(dgv_chan_tmp) != NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype, 3))
				{
					p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);

					if (p_bridged->call_state == DG_AST_CONVERSATION)
					{
						if (p_bridged->flashing > 0)
						{
							p_bridged->flashing = 0;
							p->flashing = 2;

							ast_dgv_unlock(&p->lock, p->port);

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "ccb_thread: ccb_pass_through received, calling CallBlock application on %d \n", p->port);

							dg_Flash(p->port, 1, 1500, 100);

							break;
						}
						else //if (p_bridged->flashing > 0)
						{
							ast_dgv_unlock(&p->lock, p->port);

							if (option_debug == 50)
								ast_verbose(VERBOSE_PREFIX_3 "ccb_thread: not yet, so sleeping and trying again on '%s' \n", v_name);

							if ((p) && (p->ast_ch))
							{
								usleep(500 * 1000L);
								//and try again
							}
							else //if ((p) && (p->ast_ch))
							{
								if (option_verbose >= 3)
									ast_log(LOG_WARNING, "ccb_thread(1): p || p->ast_ch does not exist on '%s' \n", v_name);

								break;
							} //if ((p) && (p->ast_ch))
						} //if (p_bridged->flashing > 0)
					}
					else
					{
						ast_dgv_unlock(&p->lock, p->port);

						ast_log(LOG_WARNING, "ccb_thread (%d): %s must be under Conversation (dropping ccb_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_WARNING, "ccb_thread (%d): p_bridged->call_state %d\n", p->port, p_bridged->call_state);

						break;
					}
				}
				else
				{
					ast_dgv_unlock(&p->lock, p->port);

					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_WARNING, "ccb_thread (%d): %s is not a DGV channel (dropping ccb_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));

					break;
				}
			}
			else //if (dgv_chan_tmp)
			{
				ast_dgv_unlock(&p->lock, p->port);

				if (option_verbose >= 3)
					ast_log(LOG_WARNING, "ccb_thread(1): p || p->ast_ch does not exist on '%s' \n", v_name);

				break;
			} //if (dgv_chan_tmp)
		}
		else //if ((p) && (p->ast_ch))
		{
			ast_dgv_unlock(&p->lock, p->port);

			if (option_verbose >= 3)
				ast_log(LOG_WARNING, "ccb_thread(2): p || p->ast_ch does not exist on '%s' \n", v_name);

			break;
		} //if ((p) && (p->ast_ch))
	} //for (i=0; i<timeout; i++)

	return NULL;
}

//thread for dg_GSMCheckSignalQuality
static void *gsm_getsignal_thread(void *data)
{
	struct dgv_pvt *p = data;
	short ret;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	if (!p)
	{
		ast_log(LOG_WARNING, "gsm_getsignal_thread: p does not exist!\n");
		return NULL;
	}

	if (option_debug==100)
		ast_log(LOG_DEBUG, "gsm_getsignal_thread on port %d\n", p->port);

	for (;;)
	{
    	if (!channel_up)
    	{
			if (option_debug==100)
				ast_log(LOG_DEBUG, "*** Channel not ready to request alarm! ***\n");

			p->gsm_getsignal_thread_id = 0;

			return NULL;
    	}

		ast_dgv_lock(&p->lock, p->port);
		{
			if (p->call_state != DG_AST_CALLING && p->sms_reserved == 0 && p->alarmed == 0)
			{
				ast_dgv_unlock(&p->lock, p->port);//Unlock this channel to avoid a VoicerLib deadlock

        		ret = dg_GSMCheckSignalQuality(p->port);
        		if (ret != DG_EXIT_SUCCESS)
        			ast_log(LOG_NOTICE, "Error requesting alarm notifications for card %d, port %d - error code %x\n", p->board, p->port, ret);

        		ast_dgv_lock(&p->lock, p->port);

	        	if (p->gsmgetsignal_timeout_sched > -1)
				{
	        		if (option_debug==100)
						ast_log(LOG_DEBUG, "Waiting alarm response for port %d\n", p->port);
				}
	        	else
	        	{
					if (option_debug==100)
						ast_log(LOG_DEBUG, "Creating gsmgetsignal schedule for port %d\n", p->port);

					p->gsmgetsignal_timeout_sched = ast_sched_add(sched, gsmgetsignaltimeout, gsmgetsignal_timeout_callback, p);

					if (option_debug==100)
						ast_log(LOG_DEBUG, "Alarm notifications requested for card %d, port %d (call_state %d, sms_reserved %d)\n", p->board, p->port, p->call_state, p->sms_reserved);
				}
			}
			else
			{
				if (option_debug==100)
					ast_log(LOG_DEBUG, "Cannot request alarm notifications for card %d, port %d (call_state %d, sms_reserved %d, alarmed %d)\n", p->board, p->port, p->call_state, p->sms_reserved, p->alarmed);
			}
		}
		ast_dgv_unlock(&p->lock, p->port);

		usleep(dgv_conf.gsm_signal_update_interval*1000L);
	}

	ast_dgv_lock(&p->lock, p->port);
	{
		p->gsm_getsignal_thread_id = 0;
	}
	ast_dgv_unlock(&p->lock, p->port);

    return NULL;
}

//----------------------------------------------------------------------
// sets individual card configuration
//----------------------------------------------------------------------
short set_card_config(char *s, short sync, short sync_alternative, short use_crc4)
{
	char *copy;
	char *piece;
	char *c=NULL;
	int start=0, finish=0, x;
    short num_spans=0,spn=0;
    
#ifdef USE_LIBPRI
    	char szTemp[255];
#endif//#ifdef USE_LIBPRI

	c = copy = ast_strdupa(s);
    if (!copy)
       return 0;

	while ((piece = strsep(&c, ","))) 
	{		
		if (sscanf(piece, "%d-%d", &start, &finish) == 2)
		{			
			//Range
		} 
		else 
		{
			if (sscanf(piece, "%d", &start)) 
			{
				//Just one
				finish = start;				
			} 
			else 
			{
				ast_log(LOG_ERROR, "Syntax error parsing set_card configuration '%s' at '%s'. Ignoring.\n", s, piece);
				continue;
			}
		}

		for (x = start; x <= finish; x++) 
		{
			if (x < 0 || x > dg_GetCardsCount()) 
			{
				ast_log(LOG_WARNING, "Ignoring invalid card %d (maximum port is %d)\n", x,dg_GetCardsCount());
			}
			else
			{
				if (dg_GetCardInterface(x)==DG_DIGITAL_INTERFACE) 
				{
					dgv_sync_conf[x - 1] = sync;
					ast_log(LOG_NOTICE,"card=%d sync_mode = %s (%d)\n", x, sync_mode_desc[dgv_sync_conf[x - 1]], dgv_sync_conf[x - 1]);

                    /* check which type of framer will be used */
                    if (dg_GetCardPortsCount(x)>30)
                        num_spans = 2;
                    else
                        num_spans = 1;

                    for (spn=0; spn < num_spans; spn++) 
                    {                    	
                        if (span_type[spn] == DG_SPAN_CCS) 
                        {
#ifdef USE_LIBPRI
	                            //set ccs callback - doesn't matter call several times
                            	dg_SetCCSCallback(ccs_handler);
                            	ast_log(LOG_NOTICE,"CCS TYPE card=%d span=%d\n",x,spn+1);
                            	dg_EnableCCSMode(x,spn+1);                         

                            	//starts driver to handle exchange data from libpri to voicerlib                            
                            	sprintf(szTemp, "/dev/dgvfifo%d", dev_idx++);
                            	dchans[dchans_count].fds[TX] = open(szTemp,O_RDWR  | O_NONBLOCK);
                            	ast_log(LOG_NOTICE,"Card %d - Span %d Creating %s\n", x,spn+1,szTemp);
	
                            	sprintf(szTemp,"/dev/dgvfifo%d",dev_idx++);
                            	dchans[dchans_count].fds[RX] = open(szTemp,O_RDWR | O_NONBLOCK);
                            	ast_log(LOG_NOTICE,"Card %d - Span %d Creating %s\n", x,spn+1,szTemp);
	
                            	if (dchans[dchans_count].fds[TX]<0 || dchans[dchans_count].fds[RX]<0) 
                            	{
	                                ast_log(LOG_ERROR, "Error opening /dev/dgvfifo! You need to install this device driver before!");
                                	return 0;
                            	}
                            	else
                            	{
	                                //stores card and span number
                                	dchans[dchans_count].card = x;
                                	dchans[dchans_count].span = spn + 1;
                                	dchans[dchans_count].pritype = span_pritype[spn];
                                	dchans[dchans_count].switchtype = span_switchtype[spn];
                                	dchans[dchans_count].overlapdial = span_overlapdial[spn];
                                	dchans[dchans_count].pridialplan = span_pridialplan[spn];
                                	dchans[dchans_count].prilocaldialplan = span_prilocaldialplan[spn];
                                	dchans[dchans_count].facilityenable = span_facilityenable[spn];
                                	dchans[dchans_count].discardremoteholdretrieval = span_discardremoteholdretrieval[spn];
	                                
                                	ast_log(LOG_NOTICE,"%d - Card %d - Span %d Creating fifo device TX=%d RX=%d\n", dchans_count, x, spn+1, dchans[dchans_count].fds[TX],
                                                                                                	dchans[dchans_count].fds[RX]);
	
                                	//save the greater file descriptor used in select()
                                	if (dchans[dchans_count].fds[TX] > fdmax)
	                                    fdmax = dchans[dchans_count].fds[RX];    //check for the last that was opened
    	                            
        	                        pri_set_error(prierr);

            	                    //initializes pri structure
                	                if (!(dchans[dchans_count].pri = pri_new(dchans[dchans_count].fds[RX], dchans[dchans_count].pritype, dchans[dchans_count].switchtype))) {
                    	                ast_log(LOG_ERROR,"Fatal error starting pri structure!");
                        	            return 0;
                            	    }
                                	else
                                	{
	                                    ast_log(LOG_NOTICE,"************** Pri started for span %d\n", spn+1);
	                                    
                                    	pri_set_message(primsg);

                                    	//set overlapdial state
                                    	pri_set_overlapdial(dchans[dchans_count].pri, dchans[dchans_count].overlapdial);
                                    	
                                    	pri_hangup_fix_enable(dchans[dchans_count].pri, 1);

                                    	ast_log(LOG_NOTICE, "setting card %d \n", dchans[dchans_count].card);                                    	
                                    	ast_log(LOG_NOTICE, "setting span %d \n", dchans[dchans_count].span);                                    	
                                    	ast_log(LOG_NOTICE, "setting pritype %d \n", dchans[dchans_count].pritype);
										ast_log(LOG_NOTICE, "setting switchtype %d \n", dchans[dchans_count].switchtype);
                                    	ast_log(LOG_NOTICE, "setting overlapdial %d \n", dchans[dchans_count].overlapdial);										
                                    	ast_log(LOG_NOTICE, "setting pridialplan %d \n", dchans[dchans_count].pridialplan);
                                    	ast_log(LOG_NOTICE, "setting prilocaldialplan %d \n", dchans[dchans_count].prilocaldialplan);
                                    	ast_log(LOG_NOTICE, "setting facilityenable %d \n", dchans[dchans_count].facilityenable);
                                    	ast_log(LOG_NOTICE, "setting discardremoteholdretrieval %d \n", dchans[dchans_count].discardremoteholdretrieval);
                                	}

	                                //increment counter
    	                            dchans_count++;
        	                    }
#endif//#ifdef USE_LIBPRI
                        }
                        else
                            ast_log(LOG_NOTICE,"CAS TYPE card=%d span=%d\n", x, spn+1);                            
                    } //spans
					
					dg_SetCardSyncMode(x, dgv_sync_conf[x - 1]);

					if ((sync_alternative == dgv_sync_conf[x - 1]) || (dgv_sync_conf[x - 1] == 3))
						sync_alternative = 0;

					dgv_sync_alternative_conf[x - 1] = sync_alternative;

					if (dgv_sync_alternative_conf[x - 1] > 0)
						ast_log(LOG_NOTICE, "card=%d sync_alternative = %s (%d)\n", x, sync_mode_desc[sync_alternative], sync_alternative);
					else
						ast_log(LOG_NOTICE, "card=%d sync_alternative not set\n", x);

                    //crc4 options
                    switch (use_crc4)
					{
                    	case 0:
                        	dg_SetE1CRC4Option(x, CFG_FRAMER_E1_A, 0);
                        	dg_SetE1CRC4Option(x, CFG_FRAMER_E1_B, 0);
                        	ast_log(LOG_NOTICE, "card=%d -> All CRC4 Disable \n",x);                    	
                    		break;                    		
                    	case 1:
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_A, 1);
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_B, 0);
                            ast_log(LOG_NOTICE, "card=%d -> CRC4 Enable E1-A and Disable E1-B \n",x);                    	
                    		break;                    		
                    	case 2:
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_A, 0);
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_B, 1);
                            ast_log(LOG_NOTICE, "card=%d -> CRC4 Disable E1-A and Enable E1-B \n",x);                    	
                    		break;                    		
                    	case 3:
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_A, 1);
                            dg_SetE1CRC4Option(x, CFG_FRAMER_E1_B, 1);
	                        ast_log(LOG_NOTICE, "card=%d -> All CRC4 Enable \n",x);                    	
                    		break;
					}                    		                    		                    		
			    }
			}
		}
	}
	return 1;
}

//----------------------------------------------------------------------
// sets individual port configuration
//----------------------------------------------------------------------
short set_port_config(short reload,
		              char *s,
		              char *port_id,
		              char *port_name,
		              char *context,
		              short category,
		              short category_pass_through,
		              short min_rx,
		              short max_rx,
		              short interdigit_timeout,
					  short txgain,
					  short rxgain,
					  short cid_pos,
					  short echo_can,
                      char *accountcode,
                      short signalling,
                      short b_subscriber_hangup,
                      short ccb_pass_through,
                      short auto_dgccb,
                      char *deny_prefix,
                      short vconf_pass_through,
                      short group_b_value,
                      short wait_group_b,
                      short silence_threshold,
                      char *musicclass,
                      char *callgroup,
                      char *pickupgroup,
                      short gen_dtmf,
                      short gen_ringback,
                      char *callprogress,
                      char *language,
                      short dtmf_to_asterisk,
                      short dialtone_timeout,
                      short wait_dialtone,
                      short gen_busy,
                      short drop_tx_samples_factor,
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
						  struct ast_jb_conf jbconf,
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
                      short relaxdtmf,
                      short echo_type,
                      short echo_training,
                      short echo_nlp,
                      short ring_type,
                      short flash_min_time,
                      short flash_max_time,
                      short immediate,
                      char *mailbox,
                      short fax_detection,
                      short fax_pass_through,
                      short r2_country,
                      short polarity,
                      short cid_detection_type,
                      short second_call,
                      char *second_call_digits,
                      short gsm_hide_number,
                      short gsm_sms_confirmation,
                      short gsm_sms_flashsms,
                      short gsm_displaycallwaiting,
                      char *gsm_pin_number)
{
	char *copy;
	char *piece;
	char *c=NULL;
	int start=0, finish=0, x, ret;
	int nForReset;
	int nForSMSChannel;
	int res_sched;
	short ch_board;
	struct dgv_pvt *p;	
	
	c = copy = ast_strdupa(s);
    if (!copy)
       return 0;

	while ((piece = strsep(&c, ","))) 
	{
		if (sscanf(piece, "%d-%d", &start, &finish) == 2)
		{
			//Range
		} 
		else 
			if (sscanf(piece, "%d", &start)) 
			{
				//Just one
				finish = start;
			} 
			else 
			{
				ast_log(LOG_ERROR, "Syntax error parsing set_port configuration '%s' at '%s'. Ignoring.\n", s, piece);
				continue;
			}

		for (x = start; x <= finish; x++) 
		{
			if (x < 1 || x > dg_GetPortsCount()) 
			{
				ast_log(LOG_WARNING, "Ignoring invalid port %d (maximum port is %d)\n", x,dg_GetPortsCount());
			}
			else
			{
				p = find_ch_port(x);
				
				if (p!=NULL)
				{					
					ast_copy_string(p->szPortID, port_id, sizeof(p->szPortID));
					ast_copy_string(p->szPortName, port_name, sizeof(p->szPortName));
					ast_copy_string(p->context, context, sizeof(p->context));
					ast_copy_string(p->accountcode, accountcode, sizeof(p->accountcode));
					p->category = category;
					p->category_pass_through = category_pass_through;
					//p->auto_r2dmfc = auto_r2dmfc;
					p->max_digits_rx = max_rx;
					p->interdigit_timeout = interdigit_timeout;
					p->min_digits_rx = min_rx;
					p->tx_gain = txgain;
					p->rx_gain = rxgain;
					p->ask_for_cid_position = cid_pos;
					p->echo_can = echo_can;
					p->echo_type = echo_type,
					p->echo_training = echo_training;
                    p->echo_nlp = echo_nlp;
                    p->drop_tx_samples_factor = drop_tx_samples_factor;
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
						memcpy(&p->jbconf, &jbconf, sizeof(p->jbconf));
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
                    p->relaxdtmf = relaxdtmf;
					p->last_dtmf = -1;
                    p->gen_ringback = gen_ringback;
                    p->gen_busy = gen_busy;
                    p->gen_dtmf = gen_dtmf;
                    p->dtmf_to_asterisk = dtmf_to_asterisk;
                    p->b_subscriber_hangup = b_subscriber_hangup;
                    p->ccb_pass_through = ccb_pass_through;
                    p->auto_dgccb = auto_dgccb;
                    ast_copy_string(p->deny_prefix, deny_prefix, sizeof(p->deny_prefix));
                    p->vconf_pass_through = vconf_pass_through;
                    p->group_b_value = group_b_value;
                    p->wait_group_b = wait_group_b;
                    p->r2_country = r2_country;
                    p->silence_threshold = silence_threshold;
					p->audio_debug = 0;
					p->faxhandled = 0;
					p->fax_tone_state = 0;
					p->fxo_bridged_dial = 0;

					if (reload<2)
						p->verbosity = 0;

					//fxs
					p->ring_type      = ring_type;
					p->flash_min_time = flash_min_time;
					p->flash_max_time = flash_max_time;
					p->immediate      = immediate;//fxs and pri channels
					ast_copy_string(p->mailbox, mailbox, sizeof(p->mailbox));
					p->second_call = second_call;
					ast_copy_string(p->second_call_digits, second_call_digits, sizeof(p->second_call_digits));
					p->scd_step = 0;
					
					//gsm
					p->gsm_hide_number = gsm_hide_number;
					p->gsm_sms_confirmation = gsm_sms_confirmation;
					p->gsm_sms_flashsms = gsm_sms_flashsms;
					p->gsm_displaycallwaiting = gsm_displaycallwaiting;
					ast_copy_string(p->gsm_pin_number, gsm_pin_number, sizeof(p->gsm_pin_number));
					p->gsm_need_reload = 0;
					p->gsm_start_log = 1;
#ifdef USE_LIBPRI
	                    //isdn specific configurations
    	                ast_copy_string(p->internationalprefix, internationalprefix, sizeof(p->internationalprefix));
        	            ast_copy_string(p->nationalprefix, nationalprefix, sizeof(p->nationalprefix));
            	        ast_copy_string(p->localprefix, localprefix, sizeof(p->localprefix));
                	    ast_copy_string(p->privateprefix, privateprefix, sizeof(p->privateprefix));
                    	ast_copy_string(p->unknownprefix, unknownprefix, sizeof(p->unknownprefix));
                    	
                    	//isdn controls
                    	p->alerting = 0;
                    	p->outgoing = 0;
                    	p->proceeding = 0;
                    	p->progress = 0;
                    	p->reverse_charging = -1;
                    	p->useruserinfo[0] = '\0';

                    	p->alerting_sig_timeout_sched = -1;
                    	p->proceeding_sig_timeout_sched = -1;
                    	p->progress_sig_timeout_sched = -1;
#endif//#ifdef USE_LIBPRI
 					p->use_callerid = use_callerid;
                    ast_copy_string(p->customcascfg, customcascfg, sizeof(p->customcascfg)); 					
 					
                    p->dialtone_timeout = dialtone_timeout;         //hardcoded - must be configured

                    p->wait_dialtone = wait_dialtone;
                    p->fax_detection = fax_detection;
                    if (dg_GetCardInterface(dg_GetCardNumber(p->port)) != DG_DIGITAL_INTERFACE)
                    	p->fax_pass_through = 0;
                    else
                    	p->fax_pass_through = fax_pass_through;
					p->polarity = polarity;
					p->cid_detection_type = cid_detection_type;

                    p->linetone_timeout_sched = -1;
                    p->digit_timeout_sched = -1;

                    if ((p->reserved == 0) && (p->call_state == DG_AST_NONE))
                    	p->ring_timeout_sched = -1;

                    p->all_echotraining_sched = -1;
                    p->second_call_sched = -1;
                    p->need_sc_ring_sched = -1;
                    p->senddtmf_dialtone_timeout_sched = -1;
                    p->reserved_timeout_sched = -1;
                    p->gsmerror_timeout_sched = -1;
                    p->alarmNDT_timeout_sched = -1;
                    
        			if (p->gsmgetsignal_timeout_sched > -1)
        			{
        				res_sched = ast_sched_del(sched, p->gsmgetsignal_timeout_sched);
        		    	if (res_sched == -1)
        		    		if (reload>0)
        		    			ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);
        			}
    				p->gsmgetsignal_timeout_sched = -1;

	        		if (p->gsmclearsms_timeout_sched > -1)//stop reserved timeout
	        		{
	        			res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
				    	if (res_sched == -1)
				    		if (reload>0)
				    			ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);
	        		}
	        		p->gsmclearsms_timeout_sched = -1;

        			if (p->gsmsendsms_timeout_sched > -1)
        			{
        				res_sched = ast_sched_del(sched, p->gsmsendsms_timeout_sched);
        		    	if (res_sched == -1)
        		    		if (reload>0)
        		    			ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);
        			}
    				p->gsmsendsms_timeout_sched = -1;

					p->sig = signalling;   /* signalling type */					
					
                    ast_copy_string(p->callgroupcfg, callgroup, sizeof(p->callgroupcfg));
                    ast_copy_string(p->pickupgroupcfg, pickupgroup, sizeof(p->pickupgroupcfg));
					p->callgroup = ast_get_group(callgroup);
                    p->pickupgroup = ast_get_group(pickupgroup);

                    if (p->auto_dgccb)
                    	p->flashing = 1;
                    else
                    	p->flashing = 0;

                    p->dialplan_echocan = 0;
                    p->dialplan_echotraining = 0;
                    for (nForReset = 0; nForReset < 20; nForReset++)
                    {
                    	p->dialplan_senddtmf[nForReset] = '\0';
                    	p->dialplan_group_b[nForReset] = '\0';	
                    }
                    
                    if (ast_strlen_zero(callprogress) && !ast_strlen_zero(dgv_conf.szDefaultCallProgressFile))
                        ast_copy_string(callprogress, dgv_conf.szDefaultCallProgressFile, sizeof(callprogress));
                    ast_copy_string(p->callprogress, callprogress, sizeof(p->callprogress));
                    
                    //language
                    if (!ast_strlen_zero(language))
                    	ast_copy_string(p->language, language, sizeof(p->language));
                    else                    	
                    	ast_copy_string(p->language, global_language, sizeof(p->language));

                    ast_copy_string(p->musicclass, musicclass, sizeof(p->musicclass));  /* music on hold */

					p->tlastdigit  = ast_tvnow();

					p->reserved = 0;
					p->sms_reserved = 0;
					p->sms_send_to[0] = '\0';
					p->sms_send_messagereference = -1;
					p->sms_send_custom_id[0] = '\0';
					p->sms_sent_status = 0;
					p->alarmed = 0;
					
					if (p->vconf_established != 1)
						p->vconf_established = 0;

					if (p->fax_established != 1)
						p->fax_established = 0;

					if (p->alarmNDT_timeout_sched > -1) 
					{
						res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->alarmNDT_timeout_sched = -1;
					}
					
	        		if (p->reserved_timeout_sched > -1)//stop reserved timeout
	        		{
	        			res_sched = ast_sched_del(sched, p->reserved_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

	        			p->reserved_timeout_sched = -1;
	        		}

	        		if (p->gsmerror_timeout_sched > -1)//stop reserved timeout
	        		{
	        			res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

	        			p->gsmerror_timeout_sched = -1;
	        		}

	        		if (p->gsmclearsms_timeout_sched > -1)//stop reserved timeout
	        		{
	        			res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

	        			p->gsmclearsms_timeout_sched = -1;
	        		}

					if (((p->call_state == DG_AST_NONE || !p->ast_ch) && reload==2) || (reload<2))
					{
						p->last_r2_signal = R2_IDLE;
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "Resetting last_r2_signal variable on port %d", p->port);
					}

					if (option_verbose >= 3) 
						ast_log(LOG_NOTICE, "port=%d id=%s context=%s - cat=%d max_rx=%d txgain=%d rxgain=%d echocan=%s\n", p->port, 
															 p->szPortID, p->context,p->category, p->max_digits_rx, p->tx_gain,
															 p->rx_gain, p->echo_can ? "yes":"no" );

					//gain
					dg_SetPortGain(p->port, PLAY_GAIN, p->tx_gain);
                    //set rec gain instead of rx_gain to avoid affect echo and agc algorythms
					dg_SetPortGain(p->port, REC_GAIN, p->rx_gain);
                    //RX Gain is always zero
                    //dg_SetPortGain(p->port, RX_GAIN, 0);//removed on 1.0.8_rc3
                    //dg_SetPortGain(p->port, TX_GAIN, 0);//removed on 1.0.8_rc3

                    //set relaxdtmf
                    if ((p->sig == DG_SIG_CB_FXO) || 
                  	    (dg_GetCardInterface(dg_GetCardNumber(p->port)) == DG_FX_INTERFACE && p->sig == DG_SIG_FXO))
                    	dg_SetTwist(p->port, 12, 0 );
                    else
                    {
	                    switch (p->relaxdtmf)
	                    {
	                        case 0:                        
	                            dg_SetTwist(p->port, 6 , 18);
								break;
	                        case 1:
	                            dg_SetTwist(p->port, 9 , 12);
	                            break;
	                        case 2:
	                            dg_SetTwist(p->port, 9 , 6 );
	                            break;
	                        case 3:
	                            dg_SetTwist(p->port, 12, 0 );
	                            break;
	                    }
                    }

                    //fxs configurations
					if (p->sig == DG_SIG_FXS)                   
						dg_SetFlash(p->port, p->flash_min_time, p->flash_max_time);
					
                    //Callprogress is started only in the beginning
                    //call progress to detect busy, etc
                    if (reload<2)
                    {
                        dg_DestroyCallProgress(p->port);
                        dg_DisableInputBuffer(p->port);
                    }

                    switch(p->sig)
                    {            
                        case DG_SIG_R2_NDIS_FIXED:
                        case DG_SIG_R2_NDIS_VARIABLE:
                            //ast_log(LOG_NOTICE, "port %d criando thread e1\n", p->port);
                            //Creates E1 Thread and configure all necessary data
                            //RETIRA!!!!!!!!!!!!!!!!!!!!!!!
                            if (reload<2)
                            {
                                if (p->vconf_pass_through)
								 {
									ch_board = dg_GetRelativeChannelNumber(p->port);

									if ((ch_board > 0) && (ch_board < 31))
										p->vconf_port_bridged = p->port + 30;
									else
										p->vconf_port_bridged = p->port - 30;
								 }

                                ret=dg_CreateE1Thread(p->port);
                            }
                            else
                                ret=DG_EXIT_SUCCESS;
    
                            if (ret!=DG_EXIT_SUCCESS)
                                ast_log(LOG_ERROR, "Error creating E1 thread for port %d - cod %x\n",p->port,ret);
                            else
                            {
                                dg_SetPortId(p->port, p->szPortID);
                                //set the same category  for all channels
                                dg_ConfigE1Thread(p->port,E1CFG_GROUP_II,p->category);
                                //set to receive x digits
                                //ast_log(LOG_ERROR, "MAX DIGITS E1 thread for port %d - value %x\n",p->port,p->max_digits_rx);
    
                                if (p->sig==DG_SIG_R2_NDIS_FIXED) 
                                {
                                    dg_ConfigE1Thread(p->port,E1CFG_GROUP_B, p->group_b_value);
                                    dg_ConfigE1Thread(p->port,E1CFG_MAXDIGITS_RX, p->max_digits_rx);
                                }
                                else
                                    if (p->sig==DG_SIG_R2_NDIS_VARIABLE)
                                    {
                                        dg_ConfigE1Thread(p->port,E1CFG_MAXDIGITS_RX, 0);
                                        dg_ConfigE1Thread(p->port,E1CFG_GROUP_B, 0);    
                                    }
                                //set to ask for cid after n digits
                                dg_ConfigE1Thread(p->port,E1CFG_SEND_ID_AFTERDIGIT,p->ask_for_cid_position);
                                //set to not generate ringback tones
                                dg_ConfigE1Thread(p->port,E1CFG_GENERATE_RINGBACK, 0);
                                //threshold during conversation
                                dg_ConfigE1Thread(p->port,SILENCE_THRESHOLD_AFTER,p->silence_threshold);
                                //set r2 country                               
                                dg_ConfigE1Thread(p->port, E1CFG_R2_COUNTRY, p->r2_country);
                            }
                            break;
                        case DG_SIG_PRI:
							if (p->vconf_pass_through)
							{
							    ch_board = dg_GetRelativeChannelNumber(p->port);

								if ((ch_board > 0) && (ch_board < 31))
									p->vconf_port_bridged = p->port + 30;
								else
									p->vconf_port_bridged = p->port - 30;
							}

                            //threshold during conversation
                            dg_SetSilenceThreshold(p->port, p->silence_threshold);                        	
                            break;
                        case DG_SIG_CUSTOMCAS:
                            //threshold during conversation
                            dg_SetSilenceThreshold(p->port, p->silence_threshold);
                            
                            //ast_log(LOG_NOTICE, "port %d criando thread cas\n", p->port);
                            //Creates Custom CAS Thread and configure all necessary data                            
                            if (reload<2)
                                ret=dg_CreateCustomCAS(p->port, p->customcascfg);
                            else
                                ret=DG_EXIT_SUCCESS;
    
                            if (ret!=DG_EXIT_SUCCESS)
                                ast_log(LOG_ERROR, "Error creating Custom CAS thread for port %d - cod %x\n",p->port,ret);
                            else                           	
                                dg_SetPortId(p->port, p->szPortID);
                                                                
                            break;
                        case DG_SIG_FXO:
                        	if ((dg_GetCardType(dg_GetCardNumber(p->port)) == VB0404FX) ||
                        	    (dg_GetCardType(dg_GetCardNumber(p->port)) == VB0404FX_R))
                        	{
								if (option_verbose >= 3)
									ast_log(LOG_NOTICE, "Changing signalling FXS to FXO on port %d\n", p->port);
								dg_SetFXCardType(p->port, DG_FX_TYPE_FXO);
                        	}
                        	
                        	if (p->cid_detection_type == 2)
                        		dg_EnableFSKDetection(p->port, DG_ENABLE, 0);
                        	else
                        		dg_EnableFSKDetection(p->port, DG_DISABLE, 0);
                        	
                        case DG_SIG_FXS:
                            //threshold during conversation/answer
                            dg_SetSilenceThreshold(p->port, p->silence_threshold);
                            break;
                        case DG_SIG_CB_FXO:
                        case DG_SIG_CB_FXS:                        
                            //Creates CB Thread and configure all necessary data
                            if (reload<2)
                                ret=dg_CreateCBThread(p->port);
                            else
                                ret=DG_EXIT_SUCCESS;
    
                            if (ret!=DG_EXIT_SUCCESS)
                                ast_log(LOG_ERROR, "Error creating CB FXO/FXS thread for port %d - cod %x\n",p->port,ret);
                            else
                            {
                                dg_SetPortId(p->port, p->szPortID);

                                if (p->sig == DG_SIG_CB_FXO)
                                	dg_ConfigCBThread(p->port, E1CFG_SIGTYPE, R2_TYPE_CB_FXO);
                                else
                                	dg_ConfigCBThread(p->port, E1CFG_SIGTYPE, R2_TYPE_CB_FXS);

                                //set to not generate ringback tones
                                dg_ConfigCBThread(p->port, E1CFG_GENERATE_RINGBACK, 0);
                                
                                //threshold during conversation
                                dg_SetSilenceThreshold(p->port, p->silence_threshold);
                            }
                            break;
                        case DG_SIG_GSM:
                            if (reload<2)
                            {
                            	p->alarmed = 2;
                            	
                            	if (!ast_strlen_zero(p->gsm_pin_number))
                            	{
                            		dg_GSMSetPinNumber(p->port, p->gsm_pin_number, NULL);
                            		if (option_verbose >= 3)
                            			ast_log(LOG_NOTICE, "GSM port=%d, PIN=%s\n", p->port, p->gsm_pin_number);
                            	}
                            	
                                ret=dg_CreateGSMThread(p->port);
                                if (ret!=DG_EXIT_SUCCESS)
                                	ast_log(LOG_ERROR, "Error creating GSM thread for port %d - cod %x\n", p->port, ret);
                            }
                            else
                            {
                            	if (((p->call_state == DG_AST_NONE) && (!p->ast_ch)) || ((p->call_state == DG_AST_CONVERSATION) && (p->ast_ch)))
                                	set_gsm_conf(p);
                            	else
                            	{
                            		if (option_verbose >= 3)
                            			ast_log(LOG_NOTICE, "GSM port %d is waiting for a scheduled reload...\n", p->port);
                            		p->gsm_need_reload = 1;
                            	}
                            }
                            
                            for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
    						{
    							if (p->smschannel[nForSMSChannel])
    							{
    								p->smschannel[nForSMSChannel]->verbosity |= p->verbosity;
    								p->smschannel[nForSMSChannel]->sig       = p->sig;
    								ast_copy_string(p->smschannel[nForSMSChannel]->context, p->context, sizeof(p->smschannel[nForSMSChannel]->context));
    								ast_copy_string(p->smschannel[nForSMSChannel]->accountcode, p->accountcode, sizeof(p->smschannel[nForSMSChannel]->accountcode));
    							}
    							else
    								ast_log(LOG_ERROR, "Unable to configure a smschannel on '%d'\n", p->port);
    						} //for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)                            

                            break;
                        //0.9.8 default:
            				//0.9.8 break;
                    }
                    
                    if (reload<2)
                    {
                        ret = dg_CreateCallProgress(p->port, p->callprogress);
                        if (ret != DG_EXIT_SUCCESS)
                        {
                            //thread creation problem
                            if (ret != 0x455)
                            {
                            	if (ret == 0x503)
                            	{
                            		if (option_verbose >= 3)
                            			ast_log(LOG_NOTICE, "CallProgress thread already running on port %d\n", p->port);
                            	}
                            	else
                            		ast_log(LOG_ERROR, "Failed to create CallProgress thread - port %d - error code = %d\n", p->port, ret);
                            }
                            else
                            {
                                //message to show callprogress problem
                                //ast_log(LOG_ERROR, "Failed to create CallProgress thread - error code = %d\n",ret);
                                if (option_verbose >= 3)
                                {
	                                ast_verbose(VERBOSE_PREFIX_3 "*********************************************\n");
    	                            ast_verbose(VERBOSE_PREFIX_3 "* Unable to find the configuration file for *\n");
        	                        ast_verbose(VERBOSE_PREFIX_3 "* call progress. Check if the file:         *\n");
            	                    ast_verbose(VERBOSE_PREFIX_3 "* %s \n", p->callprogress);
                	                ast_verbose(VERBOSE_PREFIX_3 "* is available. Without the correct file    *\n");
                    	            ast_verbose(VERBOSE_PREFIX_3 "* the channel will be unable to detect busy *\n");
                        	        ast_verbose(VERBOSE_PREFIX_3 "* and another call progress signal!!!!      *\n");
                            	    ast_verbose(VERBOSE_PREFIX_3 "*********************************************\n");
                                }
                            }
                        }	
                        //else
                        //     ast_log(LOG_ERROR, "CallProgress OK (%d) -  code = %d\n",p->port,ret);
                        // 
                        //check for specific configurations for FXO
                        switch(p->sig)
                        {
                            case DG_SIG_FXO:
                            case DG_SIG_CB_FXO:
                                //for fxo cards, enables CALLERID Detection
                                //we don't do autopickup
                            	dg_IdleAbort(p->port);
                                dg_IdleSettings(p->port,0,5,500,1,0,wtDTMF,500,20,"");
                                ret = dg_IdleStart(p->port);
                                if (ret!=DG_EXIT_SUCCESS) {
                                    ast_log(LOG_ERROR, "dg_IdleStart for port %d - cod %x\n",p->port,ret);
                                }
                                break;
                        }

                        //Enables InputBuffer at beginning 
                        if ((ret=dg_EnableInputBuffer(p->port,DG_DISABLE)) != DG_EXIT_SUCCESS)
                        {
                            if (ret==DG_ERROR_THREAD_ALREADY_RUNNING) 
                            {
                                /* try again */
                                dg_DisableInputBuffer(p->port);
                                ret=dg_EnableInputBuffer(p->port,DG_DISABLE);
                                if (ret!= DG_EXIT_SUCCESS) 
                                {
                                    ast_log(LOG_NOTICE, "Error creating InputBuffer port %d - cod =%x - state %x\n",
                                        p->port,ret, GetInputBufferStatus(p->port));
                                }

                            }
                        }

                        while(GetInputBufferStatus(p->port)!=INPUT_BUFFER_ON)
                        {
                            usleep(100);
                        }

						if ((supported_fmts & AST_FORMAT_GSM)!=0)
							dg_SetRecordFormat(p->port, ffGsm610);
						else
							if ((supported_fmts & AST_FORMAT_ULAW)!=0)
								dg_SetRecordFormat(p->port, ffWaveULaw);
							else
								if ((supported_fmts & AST_FORMAT_ALAW)!=0)
									dg_SetRecordFormat(p->port, ffWaveALaw);
								else
									dg_SetRecordFormat(p->port, ffWavePCM);
                    } //if (reload<2)
				}
			}
		}
	}
	return 0;
}

//----------------------------------------------------------------------
// Searchs groups and set port with the selected group
//----------------------------------------------------------------------
short set_port_group(short group, char *s)
{
	char *copy;
	char *piece;
	char *c=NULL;
	int start=0, finish=0, x;
	struct dgv_pvt *p;
	
	c = copy = ast_strdupa(s);
    if (!copy)
       return 0;

	while ((piece = strsep(&c, ","))) 
	{
		if (sscanf(piece, "%d-%d", &start, &finish) == 2)
		{
			//Range
		} 
		else 
			if (sscanf(piece, "%d", &start)) 
			{
				//Just one
				finish = start;
			} 
			else 
			{
				ast_log(LOG_ERROR, "Syntax error parsing group configuration '%s' at '%s'. Ignoring.\n", s, piece);
				continue;
			}

		for (x = start; x <= finish; x++) 
		{
			if (x < 0 || x > dg_GetPortsCount()) 
			{
				ast_log(LOG_WARNING, "Ignoring invalid group %d (maximum group is %d)\n", x,dg_GetPortsCount());
			}
			else
			{
				p = find_ch_port(x);
				
				if (p!=NULL)
				{
					p->group = group;
					if ((p->verbosity & DG_LOG_FIND_PORT) != 0)
						ast_log(LOG_NOTICE, "group=%d  port=%d\n", p->group, p->port);
				}
			}
		}
	}
	return 0;
}

//--------------------------------------------------------------------------
// This functions return de tone code based on tone str
//--------------------------------------------------------------------------
short tone_lookup(char *tone_str)
{
	if (strcasecmp(tone_str, "tone1")==0)
		return GENERATE_TONE1;
	else	
		return GENERATE_TONE2;
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
//Generic gettickcount
unsigned int dgv_gettick(void)
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec*1000L+tv.tv_usec/1000L;
}


//------------------------------------------------------------
// Convert audio format constants
//------------------------------------------------------------
static inline short ast2dgvformat(int ast_format)
{
#ifdef CC_AST_HAS_VERSION_13
		if (ast_format == ast_format_get_codec_id(ast_format_alaw))
			return ffWaveALaw;
		else
			if (ast_format == ast_format_get_codec_id(ast_format_slin))
				return ffWavePCM;
			else
				if (ast_format == ast_format_get_codec_id(ast_format_ulaw))
					return ffWaveULaw;
				else
					if (ast_format == ast_format_get_codec_id(ast_format_gsm))
						return ffGsm610;
					else
						return -1;
#else//#ifdef CC_AST_HAS_VERSION_13
		switch(ast_format) {
			case AST_FORMAT_ALAW:
				return ffWaveALaw;
			case AST_FORMAT_SLINEAR:
				return ffWavePCM;
			case AST_FORMAT_ULAW:
				return ffWaveULaw;
			case AST_FORMAT_GSM:
				return ffGsm610;
			default:
				return -1;
		}
#endif//#ifdef CC_AST_HAS_VERSION_13
}

//------------------------------------------------------------
// starting dial procedure
//------------------------------------------------------------
void start_dial(struct dgv_pvt *p)
{
    short ret=1;
    
    dg_event_data_structure EventContext;
    
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12
    struct dgv_pvt *p_bridged;    
        
    if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
        ast_log(LOG_NOTICE, "--------- start_dial ---------\n");

    /* Use category pass through */
	if (p->category_port_bridged > 0 && p->category_port_bridged <= dg_GetPortsCount())
	{
		p_bridged = find_ch_port(p->category_port_bridged);

		if (p_bridged)
			if (p_bridged->ast_ch)
				if (p_bridged->category_pass_through && p_bridged->sig == DG_SIG_R2_NDIS_FIXED)
				{
					if (!ast_strlen_zero(p_bridged->cid_category))
					{
						dg_ConfigE1Thread(p->port, E1CFG_GROUP_II, atoi(p_bridged->cid_category));

						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): Category Pass Through detected, changing my category to '%d'\n", p->port, atoi(p_bridged->cid_category));
					}
				}
	} //if (p->category_port_bridged > 0 && p->category_port_bridged <= dg_GetPortsCount())

    switch (p->sig) 
    {   	
        case DG_SIG_FXS:
        	break;
        case DG_SIG_CB_FXS:
        	break;
        case DG_SIG_FXO:
        case DG_SIG_CB_FXO:
			if (ast_strlen_zero(p->szNumberToDial))
	    	{
                if (option_verbose >= 3)
                	ast_log(LOG_NOTICE, "No number to dial, answering port %d \n", p->port);
                	
				p->fxo_bridged_dial = 1;
				
				ast_dgv_lock(&p->lock, p->port);
				{
	                if (!p->ast_ch)
	                {
	                	ast_log(LOG_WARNING, "start_dial: p->ast_ch does not exist(DG_SIG_FXO/CB_FXO) on port %d\n", p->port);
	                	p->call_state = DG_AST_NONE;	                	
	                	dg_HangUp(p->port);
	                	ast_dgv_unlock(&p->lock, p->port);
	                	return;
	                }

#ifdef CC_AST_HAS_VERSION_13
	                	ast_format_cap_remove_by_type(dgv_channel_nativeformats(p->ast_ch), AST_MEDIA_TYPE_UNKNOWN);
	                	ast_format_cap_append(dgv_channel_nativeformats(p->ast_ch), ast_format_alaw, 0);
						ast_set_read_format(p->ast_ch, ast_format_alaw);
						ast_set_write_format(p->ast_ch, ast_format_alaw);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
							struct ast_format ast_ch_fmt;
							ast_format_cap_remove_all(dgv_channel_nativeformats(p->ast_ch));
							ast_format_cap_add(dgv_channel_nativeformats(p->ast_ch), ast_format_set(&ast_ch_fmt, AST_FORMAT_ALAW, 0));
							ast_set_read_format_by_id(p->ast_ch, AST_FORMAT_ALAW);
							ast_set_write_format_by_id(p->ast_ch, AST_FORMAT_ALAW);
	#else//#ifdef CC_AST_HAS_VERSION_10
							p->ast_ch->nativeformats = AST_FORMAT_ALAW;
							ast_set_read_format(p->ast_ch, AST_FORMAT_ALAW);
							ast_set_write_format(p->ast_ch, AST_FORMAT_ALAW);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
	
	                //re-enable inputbuffer using G711 format
#ifdef CC_AST_HAS_VERSION_13
	                	dg_SetRecordFormat(p->port, ast2dgvformat(ast_format_get_codec_id(ast_format_alaw)));
#else//#ifdef CC_AST_HAS_VERSION_13
	                	dg_SetRecordFormat(p->port, ast2dgvformat(AST_FORMAT_ALAW));
#endif//#ifdef CC_AST_HAS_VERSION_13
	
	                //check if there is a bridged channel with
	                if ((dgv_chan_tmp=dgv_bridged_channel(p->ast_ch))!=NULL)
	                {
						//tell to bridged channel to stop playbuffer
	                    if (dgv_channel_tech_pvt(dgv_chan_tmp)!=NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype,3))
	                    {                    	
							p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);
	                        if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                        	ast_log(LOG_NOTICE, "start_dial: changing bridged format for %s \n", dgv_channel_name(dgv_chan_tmp));
	
	                        //change bridged channel
#ifdef CC_AST_HAS_VERSION_13
	                        	ast_format_cap_remove_by_type(dgv_channel_nativeformats(dgv_chan_tmp), AST_MEDIA_TYPE_UNKNOWN);
	                        	ast_format_cap_append(dgv_channel_nativeformats(dgv_chan_tmp), ast_format_alaw, 0);
	                        	ast_set_read_format(dgv_chan_tmp, ast_format_alaw);
	                        	ast_set_write_format(dgv_chan_tmp, ast_format_alaw);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
									struct ast_format chan_tmp_fmt;
									ast_format_cap_remove_all(dgv_channel_nativeformats(dgv_chan_tmp));
									ast_format_cap_add(dgv_channel_nativeformats(dgv_chan_tmp), ast_format_set(&chan_tmp_fmt, AST_FORMAT_ALAW, 0));
									ast_set_read_format_by_id(dgv_chan_tmp, AST_FORMAT_ALAW);
									ast_set_write_format_by_id(dgv_chan_tmp, AST_FORMAT_ALAW);
	#else//#ifdef CC_AST_HAS_VERSION_10
									dgv_chan_tmp->nativeformats = AST_FORMAT_ALAW;
									ast_set_read_format(dgv_chan_tmp, AST_FORMAT_ALAW);
									ast_set_write_format(dgv_chan_tmp, AST_FORMAT_ALAW);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
	                        //re-enable inputbuffer using G711 format
#ifdef CC_AST_HAS_VERSION_13
	                        	dg_SetRecordFormat(p_bridged->port, ast2dgvformat(ast_format_get_codec_id(ast_format_alaw)));
#else//#ifdef CC_AST_HAS_VERSION_13
	                        	dg_SetRecordFormat(p_bridged->port, ast2dgvformat(AST_FORMAT_ALAW));
#endif//#ifdef CC_AST_HAS_VERSION_13

	                    }
	                }
	                	
	               	ast_setstate(p->ast_ch, AST_STATE_DIALING);//only to asterisk log
				
					p->call_state = DG_AST_CALLING;
					ast_setstate(p->ast_ch, AST_STATE_RINGING);

	                EventContext.port = p->port;
	                EventContext.command = EV_ANSWERED;
				}
				ast_dgv_unlock(&p->lock, p->port);
					    
                event_handler(&EventContext);
                break;
	    	}
        default:
            //dialing        	
        	if (p->sig == DG_SIG_CB_FXO)
        		ret = dg_Dial(p->port, p->szNumberToDial, dgv_conf.afterdialpause, dtDirectMF);
        	else
        		if (p->sig != DG_SIG_GSM)
        			ret = dg_Dial(p->port, p->szNumberToDial, dgv_conf.afterdialpause, dgv_conf.default_dialtype);
        		else
        		{
        			ast_dgv_lock(&p->lock, p->port);
        			{
                        if (p->ast_ch)
                        {
                        	p->call_state = DG_AST_DIALING;
        	                ast_setstate(p->ast_ch, AST_STATE_DIALING);
        	                
        	                ret = dg_Dial(p->port, p->szNumberToDial, dgv_conf.afterdialpause, dtDTMF);
        	            }
        			}
        			ast_dgv_unlock(&p->lock, p->port);
        		}
        	
            if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                ast_log(LOG_NOTICE, "Dialing using port %d to %s\n", p->port, p->szNumberToDial);

			ast_dgv_lock(&p->lock, p->port);
			{
                if (!p->ast_ch)
                {
                	ast_log(LOG_WARNING, "start_dial: p->ast_ch does not exist(default) on port %d\n", p->port);
                	p->call_state = DG_AST_NONE;
                	dg_HangUp(p->port);
                	ast_dgv_unlock(&p->lock, p->port);
                	return;
                }
            
                if (p->sig != DG_SIG_GSM)
                {
		            if (ret == EXIT_SUCCESS)
		            {
		                p->call_state = DG_AST_DIALING;
		                ast_setstate(p->ast_ch, AST_STATE_DIALING);
		            }
		            else
		            {
		                //failed to dial
		                p->call_state = DG_AST_NONE;
		                ast_setstate(p->ast_ch, AST_STATE_BUSY);
		            }
                }
                
	            //0.9.8 break;
			}
			ast_dgv_unlock(&p->lock, p->port);
    }
}

#ifdef USE_LIBPRI
	static inline void start_pri_dial(struct dgv_pvt *p, struct ast_channel *ast)
	{
#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "start_pri_dial: STEP 1\n");
#endif//#ifdef DEBUG_PRI

		dg_event_data_structure EventContext;

		struct pri_sr *sr;

		unsigned int exclusive=0;
		int prilocaldialplan;
		int pridialplan;
		int dc=0;
		char *c;
#ifdef USERUSER_INFO
			const char *useruser;
#endif//#ifdef USERUSER_INFO

#ifdef CC_PRI_HAS_REVERSECHARGE
			const char *reversecharging;
			int reversecharging_value;
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE

		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
			ast_log(LOG_NOTICE, "--------- start_pri_dial ---------\n");

		if (p->sig == DG_SIG_PRI)
		{
#ifdef DEBUG_PRI
				ast_log(LOG_NOTICE, "start_pri_dial: ast_dgv_lock on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI

			if (!p->ast_ch)
			{
				ast_log(LOG_WARNING, "start_pri_dial: p->ast_ch does not exist(DG_SIG_PRI) on port %d\n", p->port);
				p->call_state = DG_AST_NONE;
				p->call = NULL;
				return;
			}

			if (!p->call)
			{
#ifdef DEBUG_PRI
					ast_log(LOG_NOTICE, "start_pri_dial: before dgv_trylock_ccs on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI

				dc = dg_pri_findpri(p->pri);

				if (!dgv_trylock_ccs(p, dc))
				{
#ifdef DEBUG_PRI
						ast_log(LOG_NOTICE, "start_pri_dial: before pri_new_call on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI

						p->call = pri_new_call(p->pri);

#ifdef DEBUG_PRI
						ast_log(LOG_NOTICE, "start_pri_dial: after pri_new_call on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI

					if (p->call==NULL)
					{
						//todo: creating error handle'
						p->call_state = DG_AST_NONE;

						if (p->ast_ch)
							ast_setstate(p->ast_ch, AST_STATE_BUSY);

						ast_log(LOG_WARNING, "start_pri_dial: Unable to create a pri call on port %d\n", p->port);
					}
					else
					{
						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI Dialing using port %d to %s\n",p->port,p->szNumberToDial);

						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							ast_log(LOG_NOTICE, "PRI Dialing with dc %d on port %d\n", dc, p->port);

						if (!(sr = pri_sr_new()))
							ast_log(LOG_WARNING, "Failed to allocate setup request channel %d\n", p->port);
						else
							if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
								ast_log(LOG_NOTICE, "pri_sr created on port %d\n", p->port);

						/* traditional behavior */
						if (dchans[dc].pritype == PRI_NETWORK)
							exclusive = 0;
						else
							exclusive = 1;

						p->pri_channel = port_to_pri(p->ch_board);
						pri_sr_set_channel(sr, p->pri_channel, exclusive, 1);

						if ((!p->pri) || (!p->call) || (!p->ast_ch))
						{
							ast_log(LOG_WARNING, "start_pri_dial: We think the call was disconnected, port %d\n", p->port);
							p->call_state = DG_AST_NONE;
							dgv_releaselock_ccs(dc);
							return;
						}

						p->digital = IS_DIGITAL(dgv_channel_transfercapability(p->ast_ch));

						if (p->vconf_pass_through)
							pri_sr_set_bearer(sr, PRI_TRANS_CAP_DIGITAL, p->digital ? -1 : PRI_LAYER_1_ALAW);
						else
							pri_sr_set_bearer(sr, p->digital ? PRI_TRANS_CAP_DIGITAL : dgv_channel_transfercapability(p->ast_ch), (p->digital ? -1 : PRI_LAYER_1_ALAW));

						if (dchans[dc].facilityenable)
							pri_facility_enable(p->pri);

						pridialplan = dchans[dc].pridialplan - 1;

						c = p->szNumberToDial;

						if (pridialplan == -2) { /* compute dynamically */
							if (strncmp(c + p->stripmsd, p->internationalprefix, strlen(p->internationalprefix)) == 0) {
								pridialplan = PRI_INTERNATIONAL_ISDN;
							} else if (strncmp(c + p->stripmsd, p->nationalprefix, strlen(p->nationalprefix)) == 0) {
								pridialplan = PRI_NATIONAL_ISDN;
							} else {
								pridialplan = PRI_LOCAL_ISDN;
							}
						}

						pri_sr_set_called(sr, p->szNumberToDial, pridialplan, 0);
						prilocaldialplan = dchans[dc].prilocaldialplan - 1;
						pri_sr_set_caller(sr, p->pri_cid_num, p->pri_cid_name, prilocaldialplan, p->pri_cid_num[0] ? PRES_ALLOWED_USER_NUMBER_NOT_SCREENED : PRES_NUMBER_NOT_AVAILABLE);
						//pri_sr_set_redirecting(sr, ast->cid.cid_rdnis, p->pri->localdialplan - 1, PRES_ALLOWED_USER_NUMBER_PASSED_SCREEN, redirect_reason);

						if (option_verbose >= 3)
						{
							ast_log(LOG_NOTICE, "PRI Dialing to %s using port %d pri_cid_num: %s pri_cid_name: %s \n", p->szNumberToDial, p->port, p->pri_cid_num, p->pri_cid_name);
							if (p->ast_ch)
								ast_verbose(VERBOSE_PREFIX_3 "Requested transfer capability: 0x%.2x - %s\n", dgv_channel_transfercapability(p->ast_ch), ast_transfercapability2str(dgv_channel_transfercapability(p->ast_ch)));
						}

#if defined(USERUSER_INFO) || defined(CC_PRI_HAS_REVERSECHARGE)
							ast_channel_lock(ast);
							{
#ifdef USERUSER_INFO
									/* User-user information */
									if (p->ast_ch)
										useruser = pbx_builtin_getvar_helper(p->ast_ch, "DGV_USERUSERINFO");
									if (useruser)
									{
										pri_sr_set_useruser(sr, useruser);

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "PRI Dialing using user-user information: '%s' on port %d\n", useruser, p->port);
									}
#endif//#ifdef USERUSER_INFO

#ifdef CC_PRI_HAS_REVERSECHARGE
									/* Reverse charging */
									if (p->ast_ch)
										reversecharging = pbx_builtin_getvar_helper(p->ast_ch, "DGV_REVERSECHARGE");
									if (reversecharging)
									{
										reversecharging_value = atoi(reversecharging);

										if (reversecharging_value == 1)
										{
											pri_sr_set_reversecharge(sr, PRI_REVERSECHARGE_REQUESTED);

											if (option_verbose >= 3)
												ast_log(LOG_NOTICE, "PRI Dialing using reverse charge request on port %d\n", p->port);
										}
									}
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE
							}
							ast_channel_unlock(ast);
#endif//#if defined(USERUSER_INFO) || defined(CC_PRI_HAS_REVERSECHARGE)

						if ((!p->pri) || (!p->call) || (!p->ast_ch))
						{
							ast_log(LOG_WARNING, "start_pri_dial: We think the call was disconnected, port %d\n", p->port);
							p->call_state = DG_AST_NONE;
							dgv_releaselock_ccs(dc);
							return;
						}

						if (pri_setup(p->pri, p->call, sr))
						{
							//error during pri_call
							ast_log(LOG_WARNING, "Unable to setup call to %s (using port %d)\n", p->szNumberToDial, p->port);
						}
						else
						{
							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "PRI Dialing using port %d to %s\n",p->port,p->szNumberToDial);

							if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
							{
								if (p->digital)
									ast_log(LOG_NOTICE, "PRI Dialing(%d): is digital \n", p->port);
								else
									ast_log(LOG_NOTICE, "PRI Dialing(%d): isn't digital \n", p->port);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): ch_board %d \n", p->port, p->ch_board);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): port_to_pri(p->ch_board) %d \n", p->port, p->pri_channel);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): exclusive %d \n", p->port, exclusive);

								ast_log(LOG_NOTICE, "PRI Dialing(%d): p->pri_cid_num %s \n", p->port, p->pri_cid_num);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): prilocaldialplan %d \n", p->port, prilocaldialplan);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): p->pri_cid_name %s \n", p->port, p->pri_cid_name);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): p->szNumberToDial %s \n", p->port, p->szNumberToDial);
								ast_log(LOG_NOTICE, "PRI Dialing(%d): pridialplan %d \n", p->port, pridialplan);
							}

							if (p->ast_ch)
								ast_setstate(p->ast_ch, AST_STATE_DIALING);

							//must call EV_AFTERDIAL to complete call
							p->call_state = DG_AST_DIALING;

							EventContext.port = p->port;
							EventContext.command = EV_AFTERDIAL;

							ast_dgv_unlock(&p->lock, p->port);
							{
								event_handler(&EventContext);
							}
							ast_dgv_lock(&p->lock, p->port);
						}

						pri_sr_free(sr);
					}

					dgv_releaselock_ccs(dc);
				} //if (!dgv_trylock_ccs(p))
				else
					ast_log(LOG_WARNING, "start_pri_dial: Unable to grab ccs_lock on port %d\n", p->port);
			}
			else //if (!p->call)
			{
				if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
					ast_log(LOG_WARNING, "start_pri_dial: port %d has already a p->call\n", p->port);
			} //if (!p->call)
		}
		else
			ast_log(LOG_WARNING, "start_pri_dial: Unable to process non ISDN signalling on port %d\n", p->port);
	}
#endif//#ifdef USE_LIBPRI

//------------------------------------------------------------
//set tone thread values
//------------------------------------------------------------
static short set_tone_thread_values(int port)
{
	struct dgv_pvt *p;
	int nForSleep;
	
	p = find_ch_port(port);
	
	if (p)
	{
		switch (p->tone_thread_type)
		{
		    case AST_CONTROL_RINGING:
				if ((p->verbosity & DG_LOG_THREADS) != 0)
		    		ast_log(LOG_NOTICE, "set_tone_thread_values: port %d - type = AST_CONTROL_RINGING \n", port);

				for (nForSleep = 0; nForSleep < 4; nForSleep++)
					p->tone_thread_timevalues[nForSleep] = dgv_conf.ringback_times[nForSleep];
					
				p->tone_thread_tonetype[0] = dgv_conf.ringbacktone1;
				p->tone_thread_tonetype[1] = dgv_conf.ringbacktone2;
				break;
			case AST_CONTROL_BUSY:
				if ((p->verbosity & DG_LOG_THREADS) != 0)
		    		ast_log(LOG_NOTICE, "set_tone_thread_values: port %d - type = AST_CONTROL_BUSY \n", port);

				for (nForSleep = 0; nForSleep < 4; nForSleep++)
					p->tone_thread_timevalues[nForSleep] = dgv_conf.busy_times[nForSleep];

				p->tone_thread_tonetype[0] = dgv_conf.busytone1;
				p->tone_thread_tonetype[1] = dgv_conf.busytone2;
				break;
		    default:      
		    	ast_log(LOG_WARNING, "set_tone_thread_values: port %d - no tone_thread_type, type = %d \n", port, p->tone_thread_type);
		} //switch (p->tone_thread_type)
		p->setting_tone_thread_values = 0;
	} //if (p)

	return 1;
}

//------------------------------------------------------------
// thread allowing ringback and busy tones
//------------------------------------------------------------
static void *ToneThread(void *param)
{
	struct dgv_pvt *p;
	short port = *(short *)param;
	short nForSleep;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	p = find_ch_port(port);
	
	if (!p)
		return NULL;

	ast_dgv_lock(&p->lock, p->port);
	{
		if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
			 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||
			 (p->tone_thread_running != 1) )
		{
			p->tone_thread_id = 0;
			p->tone_thread_running = 0;
			ast_dgv_unlock(&p->lock, p->port);
			if ((p->verbosity & DG_LOG_THREADS) != 0)
				ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(1)\n", port);
			dg_GenerateMF(port, GENERATE_OFF, 0);
			return NULL;
		}

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "ToneThread: Starting ToneThread on port %d\n", port);

		if (p->setting_tone_thread_values)
		{
			if ((p->verbosity & DG_LOG_THREADS) != 0)
				ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(1) on port %d\n", port);

			set_tone_thread_values(port);
		}
	}
	ast_dgv_unlock(&p->lock, p->port);
	
	if ( (p->sig == DG_SIG_R2_NDIS_FIXED || p->sig == DG_SIG_R2_NDIS_VARIABLE) && 
		 (p->tone_thread_type == AST_CONTROL_RINGING) )
	{
		for (nForSleep = 1; nForSleep <= 300; nForSleep++)
		{
			if (p->need_ringback == 1)
				break;
    	
			ast_dgv_lock(&p->lock, p->port);
			{
				if ((p->ast_ch==NULL) || (p->tone_thread_running != 1))
				{
					p->tone_thread_id = 0;
					p->tone_thread_running = 0;
					ast_dgv_unlock(&p->lock, p->port);
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(2)\n", port);
					
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);
	    	
			if ((p->verbosity & DG_LOG_THREADS) != 0)
				ast_log(LOG_NOTICE, "ToneThread: port %d - sleeping step 0\n", port);

	    	usleep(100*1000L);
		}
	}

	for (;;)
	{
		ast_dgv_lock(&p->lock, p->port);
		{
			if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||
				 (p->tone_thread_running != 1) )
			{
				p->tone_thread_id = 0;
				p->tone_thread_running = 0;
				ast_dgv_unlock(&p->lock, p->port);
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(3)\n", port);
				dg_GenerateMF(port, GENERATE_OFF, 0);
				return NULL;
			}
    	}
    	ast_dgv_unlock(&p->lock, p->port);

		ast_dgv_lock(&p->lock, p->port);
		{
			if (p->setting_tone_thread_values)
			{
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(2) on port %d\n", port);

				set_tone_thread_values(port);
			}
		}
		ast_dgv_unlock(&p->lock, p->port);
			
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "ToneThread: port %d - generating tone step 1\n", port);

		//turn on tone 1
		dg_GenerateMF(port, p->tone_thread_tonetype[0], 0);

		//wait for duration
        for (nForSleep = 1; nForSleep <= 10; nForSleep++)
        {
			ast_dgv_lock(&p->lock, p->port);
			{
				if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				     ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||			
					 (p->tone_thread_running != 1) )
				{
					p->tone_thread_id = 0;
					p->tone_thread_running = 0;
					ast_dgv_unlock(&p->lock, p->port);
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(4)\n", port);
					dg_GenerateMF(port, GENERATE_OFF, 0);
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			ast_dgv_lock(&p->lock, p->port);
			{				
				if (p->setting_tone_thread_values)
				{
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(3) on port %d\n", port);
	
					set_tone_thread_values(port);
				}			
			}
			ast_dgv_unlock(&p->lock, p->port);
			
			usleep(p->tone_thread_timevalues[0]*100L);
        }

		ast_dgv_lock(&p->lock, p->port);
		{        
			if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||			
				 (p->tone_thread_running != 1) )
			{
				p->tone_thread_id = 0;
				p->tone_thread_running = 0;
				ast_dgv_unlock(&p->lock, p->port);
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(5)\n", port);
				dg_GenerateMF(port, GENERATE_OFF, 0);
				return NULL;
			}
		}
		ast_dgv_unlock(&p->lock, p->port);
			
        if ((p->verbosity & DG_LOG_THREADS) != 0)
        	ast_log(LOG_NOTICE, "ToneThread: port %d - waiting time1\n",port);

		//turn off tone 1
		dg_GenerateMF(port, GENERATE_OFF, 0);

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "ToneThread: port %d - generating silence step 1\n",port);

		//wait for silence pause
	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
					 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||
					 (p->tone_thread_running != 1) )
				{
					p->tone_thread_id = 0;
					p->tone_thread_running = 0;
					ast_dgv_unlock(&p->lock, p->port);
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(6)\n", port);
					dg_GenerateMF(port, GENERATE_OFF, 0);
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->setting_tone_thread_values)
				{
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(4) on port %d\n", port);
	
					set_tone_thread_values(port);
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			usleep(p->tone_thread_timevalues[1]*100L);				
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||			
				 (p->tone_thread_running != 1) )
			{
				p->tone_thread_id = 0;
				p->tone_thread_running = 0;
				ast_dgv_unlock(&p->lock, p->port);
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(7)\n", port);
				dg_GenerateMF(port, GENERATE_OFF, 0);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);

		//turn on tone 2
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "ToneThread: port %d - generating tone step 1\n",port);

		dg_GenerateMF(port, p->tone_thread_tonetype[1], 0);

	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
					 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||
					 (p->tone_thread_running != 1) )
				{
					p->tone_thread_id = 0;
					p->tone_thread_running = 0;
					ast_dgv_unlock(&p->lock, p->port);
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(8)\n", port);
					dg_GenerateMF(port, GENERATE_OFF, 0);
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->setting_tone_thread_values)
				{
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(5) on port %d\n", port);
	
					set_tone_thread_values(port);
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			usleep(p->tone_thread_timevalues[2]*100L);
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||			
				 (p->tone_thread_running != 1) )
			{
				p->tone_thread_id = 0;
				p->tone_thread_running = 0;
				ast_dgv_unlock(&p->lock, p->port);
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(9)\n", port);
				dg_GenerateMF(port, GENERATE_OFF, 0);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);

		//turn off tone 2
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "ToneThread: port %d - generating silence step 2\n",port);

		dg_GenerateMF(port, GENERATE_OFF, 0);
			
	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
					 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||
					 (p->tone_thread_running != 1) )
				{
					p->tone_thread_id = 0;
					p->tone_thread_running = 0;
					ast_dgv_unlock(&p->lock, p->port);
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(10)\n", port);
					dg_GenerateMF(port, GENERATE_OFF, 0);
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->setting_tone_thread_values)
				{
					if ((p->verbosity & DG_LOG_THREADS) != 0)
						ast_log(LOG_NOTICE, "ToneThread: Calling set_tone_thread_values function(6) on port %d\n", port);

					set_tone_thread_values(port);
				}
			}
			ast_dgv_unlock(&p->lock, p->port);	
			
			usleep(p->tone_thread_timevalues[3]*100L);
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->reserved==0 && p->ast_ch==NULL) ||
				 ((p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) && p->ast_ch==NULL) ||			
				 (p->tone_thread_running != 1) )
			{
				p->tone_thread_id = 0;
				p->tone_thread_running = 0;
				ast_dgv_unlock(&p->lock, p->port);
				if ((p->verbosity & DG_LOG_THREADS) != 0)
					ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(11)\n", port);
				dg_GenerateMF(port, GENERATE_OFF, 0);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);
	} //for (;;)

	ast_dgv_lock(&p->lock, p->port);
	{
		p->tone_thread_id = 0;
		p->tone_thread_running = 0;
	}
	ast_dgv_unlock(&p->lock, p->port);
	if ((p->verbosity & DG_LOG_THREADS) != 0)
		ast_log(LOG_NOTICE, "****** ToneThread: port %d - end(12)\n", port);
	dg_GenerateMF(port, GENERATE_OFF, 0);
	return NULL;
}

//------------------------------------------------------------
// thread allowing stutter dial tone
//------------------------------------------------------------
static void *StutterToneThread(void *param)
{
	short port = *(short *)param;
	struct dgv_pvt *p;
	unsigned int time_values=100;
	
	short nForStutterTone;
	short nForSleep;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	p = find_ch_port(port);
		
	if (!p)
		return NULL;
	
	if ((!p->stuttertone_thread_running) || (p->tone_thread_running != 0))
	    return NULL;

	if ((p->verbosity & DG_LOG_THREADS) != 0)
	    ast_log(LOG_NOTICE, ">>>>> StutterToneThread Starting: port %d\n",port);

	if ((!p->stuttertone_thread_running) || (p->tone_thread_running != 0))
	    return NULL;

	for (nForStutterTone = 0; nForStutterTone < 3; nForStutterTone++)
	{
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "StutterToneThread: port %d - generating stutter tone step 1\n",port);

		//turn on tone 1
		dg_GenerateMF(port, GENERATE_TONE1, 0);

		//wait for duration
        for (nForSleep = 1; nForSleep <= 10; nForSleep++)
        {
        	if ((p->ast_ch == NULL) || (!p->stuttertone_thread_running) || (p->tone_thread_running != 0))		
			{		    
				dg_GenerateMF(port, GENERATE_OFF, 0);
	        	break;
			}
        	
			usleep(time_values*100L);
        }		
		
		//check if thread is already asked to terminate		
        if ((p->ast_ch == NULL) || (!p->stuttertone_thread_running) || (p->tone_thread_running != 0))		
		{		    
			dg_GenerateMF(port, GENERATE_OFF, 0);
	        break;
		}
		
        if ((p->verbosity & DG_LOG_THREADS) != 0)
            ast_log(LOG_NOTICE, "StutterToneThread: port %d - waiting time1\n",port);
            
		//turn off tone 1
		dg_GenerateMF(port, GENERATE_OFF, 0);

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "StutterToneThread: port %d - generating silence step 1\n",port);

		//wait for silence pause
        for (nForSleep = 1; nForSleep <= 10; nForSleep++)
        {
        	if ((p->ast_ch == NULL) || (!p->stuttertone_thread_running) || (p->tone_thread_running != 0))		
			{		    
				dg_GenerateMF(port, GENERATE_OFF, 0);
	        	break;
			}
			        	
			usleep(time_values*100L);				
        }
		
		if ((p->ast_ch == NULL) || (!p->stuttertone_thread_running) || (p->tone_thread_running != 0))			
		{
       	    dg_GenerateMF(port, GENERATE_OFF, 0);
            break;
		}
	}
	
	if ((p->ast_ch == NULL) || (!p->stuttertone_thread_running) || (p->tone_thread_running != 0))
    	dg_GenerateMF(port, GENERATE_OFF, 0);
	else
    	dg_GenerateMF(port, GENERATE_TONE1, 0);

	//turn off flags
	p->stuttertone_thread_running = 0;
	p->stuttertone_thread_id = 0;
	if ((p->verbosity & DG_LOG_THREADS) != 0)
		ast_log(LOG_NOTICE, "****** StutterToneThread: port %d - end! \n",port);

	return NULL;
}

//------------------------------------------------------------
// thread allowing subchannel ring tone
//------------------------------------------------------------
static void *SubChannelToneThread(void *param)
{
	struct dgv_pvt *p;
	short port = *(short *)param;
	short nForSleep;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	p = find_ch_port(port);
	
	if (!p->subchannel)
	{
		ast_log(LOG_WARNING, "****** SubChannelToneThread: port %d - end(0)\n", port);
		return NULL;
	}

	ast_dgv_lock(&p->lock, p->port);
	{
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "****** SubChannelToneThread: channel_index %d - port %d\n", p->channel_index, port);		

		if (p->subchannel)
		{
			 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
			      (p->subchanneltone_thread_running != 1) )
			 {
				CleanVarSubChannelToneThread(p, 1);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			 }
		}
		else
		{
			CleanVarSubChannelToneThread(p, 2);
			ast_dgv_unlock(&p->lock, p->port);
			return NULL;
		}

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "SubChannelToneThread: Starting SubChannelToneThread on port %d\n", port);
	}
	ast_dgv_unlock(&p->lock, p->port);
	
	for (nForSleep = 0; nForSleep < 4; nForSleep++)
		p->subchanneltone_thread_timevalues[nForSleep] = dgv_conf.subchannelring_times[nForSleep];

	p->subchanneltone_thread_tonetype[0] = dgv_conf.subchannelringtone1;
	p->subchanneltone_thread_tonetype[1] = dgv_conf.subchannelringtone2;

	for (;;)
	{
		ast_dgv_lock(&p->lock, p->port);
		{
			if (p->subchannel)
			{
				 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
				      (p->subchanneltone_thread_running != 1) )
				 {
					CleanVarSubChannelToneThread(p, 3);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				 }
			}
			else
			{
				CleanVarSubChannelToneThread(p, 4);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			}
    	}
    	ast_dgv_unlock(&p->lock, p->port);

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "SubChannelToneThread: port %d - generating tone step 1\n", port);

		//turn on tone 1
		dg_GenerateMF(port, p->subchanneltone_thread_tonetype[0], 0);

		//wait for duration
        for (nForSleep = 1; nForSleep <= 10; nForSleep++)
        {
			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->subchannel)
				{
					 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
					      (p->subchanneltone_thread_running != 1) )
					 {
						CleanVarSubChannelToneThread(p, 5);
						ast_dgv_unlock(&p->lock, p->port);
						return NULL;
					 }
				}
				else
				{
					CleanVarSubChannelToneThread(p, 6);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			usleep(p->subchanneltone_thread_timevalues[0]*100L);
        }

		ast_dgv_lock(&p->lock, p->port);
		{        
			if (p->subchannel)
			{
				 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
				      (p->subchanneltone_thread_running != 1) )
				 {
					CleanVarSubChannelToneThread(p, 7);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				 }
			}
			else
			{
				CleanVarSubChannelToneThread(p, 8);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			}
		}
		ast_dgv_unlock(&p->lock, p->port);
			
        if ((p->verbosity & DG_LOG_THREADS) != 0)
        	ast_log(LOG_NOTICE, "SubChannelToneThread: port %d - waiting time1\n",port);

		//turn off tone 1
		dg_GenerateMF(port, GENERATE_OFF, 0);

		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "SubChannelToneThread: port %d - generating silence step 1\n",port);

		//wait for silence pause
	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if (p->subchannel)
	    		{
	    			 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
	    			      (p->subchanneltone_thread_running != 1) )
	    			 {
	    				CleanVarSubChannelToneThread(p, 9);
	    				ast_dgv_unlock(&p->lock, p->port);
	    				return NULL;
	    			 }
	    		}
	    		else
	    		{
					CleanVarSubChannelToneThread(p, 10);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
	    		}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			usleep(p->subchanneltone_thread_timevalues[1]*100L);				
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if (p->subchannel)
			{
				 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
				      (p->subchanneltone_thread_running != 1) )
				 {
					CleanVarSubChannelToneThread(p, 11);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				 }
			}
			else
			{
				CleanVarSubChannelToneThread(p, 12);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);

		//turn on tone 2
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "SubChannelToneThread: port %d - generating tone step 1\n",port);

		dg_GenerateMF(port, p->subchanneltone_thread_tonetype[1], 0);

	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if (p->subchannel)
	    		{
	    			 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
	    			      (p->subchanneltone_thread_running != 1) )
	    			 {
	    				CleanVarSubChannelToneThread(p, 13);
	    				ast_dgv_unlock(&p->lock, p->port);
	    				return NULL;
	    			 }
	    		}
	    		else
	    		{
					CleanVarSubChannelToneThread(p, 14);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
	    		}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			usleep(p->subchanneltone_thread_timevalues[2]*100L);
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if (p->subchannel)
			{
				 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
				      (p->subchanneltone_thread_running != 1) )
				 {
					CleanVarSubChannelToneThread(p, 15);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				 }
			}
			else
			{
				CleanVarSubChannelToneThread(p, 16);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);

		//turn off tone 2
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "SubChannelToneThread: port %d - generating silence step 2\n",port);

		dg_GenerateMF(port, GENERATE_OFF, 0);
			
	    for (nForSleep = 1; nForSleep <= 10; nForSleep++)
	    {
	    	ast_dgv_lock(&p->lock, p->port);
	    	{
	    		if (p->subchannel)
	    		{
	    			 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
	    			      (p->subchanneltone_thread_running != 1) )
	    			 {
	    				CleanVarSubChannelToneThread(p, 17);
	    				ast_dgv_unlock(&p->lock, p->port);
	    				return NULL;
	    			 }
	    		}
	    		else
	    		{
					CleanVarSubChannelToneThread(p, 18);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
	    		}
	    	}
	    	ast_dgv_unlock(&p->lock, p->port);

			usleep(p->subchanneltone_thread_timevalues[3]*100L);
        }

	    ast_dgv_lock(&p->lock, p->port);
	    {
			if (p->subchannel)
			{
				 if ( ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->subchannel->reserved==0 && p->subchannel->ast_ch==NULL) ||
				      (p->subchanneltone_thread_running != 1) )
				 {
					CleanVarSubChannelToneThread(p, 19);
					ast_dgv_unlock(&p->lock, p->port);
					return NULL;
				 }
			}
			else
			{
				CleanVarSubChannelToneThread(p, 20);
				ast_dgv_unlock(&p->lock, p->port);
				return NULL;
			}
	    }
	    ast_dgv_unlock(&p->lock, p->port);
	} //for (;;)

	ast_dgv_lock(&p->lock, p->port);
	{
		p->subchanneltone_thread_id = 0;
		p->subchanneltone_thread_running = 0;
	}
	ast_dgv_unlock(&p->lock, p->port);
	if ((p->verbosity & DG_LOG_THREADS) != 0)
		ast_log(LOG_NOTICE, "****** SubChannelToneThread: port %d - end(11)\n", port);
	dg_GenerateMF(port, GENERATE_OFF, 0);
	return NULL;
}

int CleanVarSubChannelToneThread(struct dgv_pvt *p, int endStep)
{
	if (p)
	{
		p->subchanneltone_thread_id = 0;
		p->subchanneltone_thread_running = 0;
		if ((p->verbosity & DG_LOG_THREADS) != 0)
			ast_log(LOG_NOTICE, "****** SubChannelToneThread: port %d - end(%d)\n", p->port, endStep);
		
		if (p->beeptone_thread_id == 0)
			dg_GenerateMF(p->port, GENERATE_OFF, 0);
		else
		{
			if ((p->verbosity & DG_LOG_THREADS) != 0)
				ast_log(LOG_NOTICE, "****** SubChannelToneThread: port %d - beeptone_thread_id still running\n", p->port);
		}
		return 1;
	}
	else
	{
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "SubChannelToneThread: Failed to CleanVarSubChannelToneThread step %d - port XXX\n", endStep);
		return 0;
	}
}

//------------------------------------------------------------
// thread allowing a beep tone
//------------------------------------------------------------
static void *BeepToneThread(void *param)
{
	short port = *(short *)param;
	struct dgv_pvt *p;
	unsigned int time_values=100;
	
	short nForSleep;

	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	p = find_ch_port(port);
		
	if (!p)
		return NULL;

	if ((p->verbosity & DG_LOG_THREADS) != 0)
	    ast_log(LOG_NOTICE, ">>>>> BeepToneThread Starting: port %d\n",port);

	//turn on tone 1
	dg_GenerateMF(port, GENERATE_TONE1, 0);

	//wait for duration
    for (nForSleep = 1; nForSleep <= 50; nForSleep++)
	{
       	if (p->ast_ch == NULL)		
		{
			dg_GenerateMF(port, GENERATE_OFF, 0);
	       	break;
		}
		usleep(time_values*100L);
    }
		
	dg_GenerateMF(port, GENERATE_OFF, 0);

	//turn off flags
	p->beeptone_thread_id = 0;
	if ((p->verbosity & DG_LOG_THREADS) != 0)
		ast_log(LOG_NOTICE, "****** BeepToneThread: port %d - end! \n",port);

	return NULL;
}

//------------------------------------------------------------
// Callback that receives samples from card and passes to ast.
// size is the number of samples (not datalen)
//------------------------------------------------------------
void/*short new_vlib_change*/ read_samples(short port, short size, void *SampleData)
{
	//struct ast_frame fr;(1.0.0_rc5)

	struct dgv_pvt *p;

	int format_to_use;
#ifdef CC_AST_HAS_VERSION_13
		struct ast_format *afmt;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
			struct ast_format afmt;
	#else//#ifdef CC_AST_HAS_VERSION_10
			int afmt;
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
    char *readbuf;
    
    //int j;
	//short x;    
    //char sin_tx[1024];

#ifdef DEBUG_Q
	unsigned char ctmp;
  	char szTmp[1024];
	char char_tx[1024];
	char *tmp8;
#endif//#ifdef DEBUG_Q
	
	p = find_ch_port(port);
	
	if (p)
	{
        ast_dgv_lock(&p->lock, p->port);
        {
            if (!p->ast_ch)
            {
            	if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
            		ast_log(LOG_WARNING, "dgv_read: p->ast_ch does not exist on port %d\n", p->port);
            	ast_dgv_unlock(&p->lock, p->port);
            	return /*DG_ERROR_NOT_PLAYING new_vlib_change*/;
            }
            
			if (dgv_channel_state(p->ast_ch) != AST_STATE_UP)
			{
				//channel isn't up
	            if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
	                ast_log(LOG_WARNING, "dgv_read: port %d - ast_ch not UP - state is %d, but send voice...\n",p->port,dgv_channel_state(p->ast_ch));
			}

#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				if ((p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS) && (dgv_channel_state(p->ast_ch) != AST_STATE_UP))
				{
					if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
						ast_log(LOG_WARNING, "dgv_read: port %d - ast_ch not UP and our signalling is FXS, do not send audio\n", p->port);
					ast_dgv_unlock(&p->lock, p->port);
					return /*DG_ERROR_NOT_PLAYING new_vlib_change*/;
				}
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

			if (p->audio_debug && option_debug==60) //120
#ifdef CC_AST_HAS_VERSION_13
				ast_log(LOG_WARNING, "dgv_read: port %d read_last_format = %lld , readformat=%s - supported=%x - rawread=%s\n",port,p->read_last_format, ast_format_get_name(dgv_channel_readformat(p->ast_ch)), supported_fmts, ast_format_get_name(dgv_channel_rawreadformat(p->ast_ch)));
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
					ast_log(LOG_WARNING, "dgv_read: port %d read_last_format = %x , readformat=%s - supported=%x - rawread=%s\n",port,p->read_last_format, ast_getformatname(dgv_channel_readformat(p->ast_ch)), supported_fmts, ast_getformatname(dgv_channel_rawreadformat(p->ast_ch)));
	#else//#ifdef CC_AST_HAS_VERSION_10
					ast_log(LOG_WARNING, "dgv_read: port %d read_last_format = %x , readformat=%s - supported=%x - rawread=%s\n",port,p->read_last_format, ast_getformatname(p->ast_ch->readformat), supported_fmts, ast_getformatname(p->ast_ch->rawreadformat));
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
	        //if its fax but not SLINEAR, drop samples
#ifdef CC_AST_HAS_VERSION_13
					if (p->faxhandled && ast_format_cap_iscompatible_format(dgv_channel_nativeformats(p->ast_ch), ast_format_slin) == AST_FORMAT_CMP_NOT_EQUAL)
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
						struct ast_format tmpfmt;
						if (p->faxhandled && !ast_format_cap_iscompatible(dgv_channel_nativeformats(p->ast_ch), ast_format_set(&tmpfmt, AST_FORMAT_SLINEAR, 0)))
	#else//#ifdef CC_AST_HAS_VERSION_10
						if (p->faxhandled && p->ast_ch->nativeformats != AST_FORMAT_SLINEAR)
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
	        {
	            if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
	                ast_log(LOG_NOTICE, "dgv_read: port %d - Fax handled - forcing slin\n", p->port);

	            p->f.frametype = AST_FRAME_NULL;
	            ast_dgv_unlock(&p->lock, p->port);
	            return /*DG_ERROR_NOT_PLAYING new_vlib_change*/;
	        }			
	        //}
	        //ast_dgv_unlock(&p->lock, p->port);
	
	        //ast_dgv_lock(&p->lock, p->port);
	        //{        
	    	if (p->ast_ch)
#ifdef CC_AST_HAS_VERSION_13
	    			afmt = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
						ast_format_copy(&afmt, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
						afmt = p->ast_ch->rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13
	    	else
	    		afmt = p->pref_format;
	        //}
	        //ast_dgv_unlock(&p->lock, p->port);        	

#ifdef CC_AST_HAS_VERSION_13
	    		if (p->read_last_format == -1 || p->read_last_format != ast_format_get_codec_id(afmt))
				{
					p->read_last_format = ast_format_get_codec_id(afmt);
					format_to_use = ast2dgvformat(ast_format_get_codec_id(afmt));
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
					if (p->read_last_format == -1 || p->read_last_format != afmt.id)
					{
						p->read_last_format = afmt.id;
						format_to_use = ast2dgvformat(afmt.id);
	#else//#ifdef CC_AST_HAS_VERSION_10
					if (p->read_last_format == -1 || p->read_last_format != afmt)
					{
						p->read_last_format = afmt;
						format_to_use = ast2dgvformat(afmt);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
	            dg_SetRecordFormat(p->port, format_to_use);
	            if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
	            {
#ifdef CC_AST_HAS_VERSION_13
	                	ast_log(LOG_NOTICE, "dgv_read: port %d Assigning read format:%lld %s\n",p->port,p->read_last_format,ast2dgvformatname(p->read_last_format));
#else//#ifdef CC_AST_HAS_VERSION_13
	                	ast_log(LOG_NOTICE, "dgv_read: port %d Assigning read format:%x %s\n",p->port,p->read_last_format,ast2dgvformatname(p->read_last_format));
#endif//#ifdef CC_AST_HAS_VERSION_13
	            }
	        }

	        //copy data to internal buf[]
	        readbuf = ((char *)(p->buf)) + AST_FRIENDLY_OFFSET;
	        /*if (size < DGV_MAX_BUF)
            	memcpy(readbuf, SampleData , size);
        	else
            	memcpy(readbuf , SampleData , DGV_MAX_BUF-1);*/

	        p->f.frametype = AST_FRAME_VOICE;
	        p->f.src = ttype;
	        p->f.mallocd = 0;
	        p->f.delivery.tv_sec = 0;
	        p->f.delivery.tv_usec = 0;
	        p->f.offset = AST_FRIENDLY_OFFSET;
			//samples
	        p->f.samples = size;
			//subclass is the recordformat used
#ifdef CC_AST_HAS_VERSION_10
				p->f.subclass.format = afmt;
#else//#ifdef CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_1_8
					p->f.subclass.integer = afmt;
	#else//#ifdef CC_AST_HAS_VERSION_1_8
					p->f.subclass = afmt;
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_10

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				#ifdef CC_AST_HAS_TIMING_INFO_IN_FRAME
					ast_clear_flag(&p->f, AST_FRFLAG_HAS_TIMING_INFO);
				#else
					p->f.has_timing_info = 0;
				#endif//#ifdef CC_AST_HAS_TIMING_INFO_IN_FRAME 	
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

			//ast_log(LOG_NOTICE, "dgv_read: port %d sending format:%x\n",p->port,afmt);
#ifdef CC_AST_HAS_VERSION_13
				if (p->read_last_format == ast_format_get_codec_id(ast_format_slin))
#else//#ifdef CC_AST_HAS_VERSION_13
				if (p->read_last_format == AST_FORMAT_SLINEAR)
#endif//#ifdef CC_AST_HAS_VERSION_13
			{
				if (size<=(DATALEN*20)/2 /*|| p->faxhandled*/)     //400ms
				{
					if (size < ((DGV_MAX_BUF/2) - AST_FRIENDLY_OFFSET/2))
						memcpy(readbuf, SampleData, size*2);
					else
					{
						memcpy(readbuf, SampleData, DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1);
						size = DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1;
					}
	
#ifdef CC_AST_HAS_DATA_PTR
	                   	p->f.data.ptr = p->buf + AST_FRIENDLY_OFFSET;//(char *)readbuf;(1.0.0_rc4)
#else//#ifdef CC_AST_HAS_DATA_PTR	                    
	                   	p->f.data = p->buf + AST_FRIENDLY_OFFSET;//(char *)readbuf;(1.0.0_rc4)
#endif//#ifdef CC_AST_HAS_DATA_PTR	                    
					p->f.samples = size;
					p->f.datalen = size*2;

					//ast_log(LOG_NOTICE, "dgv_read(%d): slin datalen=%d samples=%d size=%d \n",p->port, p->f.datalen, p->f.samples, size);

					if (dg_GetPortStatus(p->port)==spDialing)
					{
						p->f.frametype = AST_FRAME_NULL;
#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass.integer = 0;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass = 0;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						p->f.datalen = 0;
						p->f.samples = 0;
						p->f.mallocd = 0;
						p->f.offset = 0;
						p->f.src = "chan_dgv";
#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data.ptr = NULL;
#else//#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data = NULL;
#endif//#ifdef CC_AST_HAS_DATA_PTR							
						if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
							ast_log(LOG_NOTICE, "Sending AST_FRAME_NULL because dg_GetPortStatus == spDialing on %d \n", p->port);
					}
						
					//ast_dgv_lock(&p->lock, p->port); {
						//QueueFrame(port, &p->f);
						QueueFrame(p, &p->f);
					//} ast_dgv_unlock(&p->lock, p->port);

				}
				else
				{
					 if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
						ast_log(LOG_NOTICE, "dgv_read(%d): dropping samples - too much delay datalen=%d delay=%d\n",p->port,
								size,size/8);
				}
			}
#ifdef CC_AST_HAS_VERSION_13
				else if (p->read_last_format == ast_format_get_codec_id(ast_format_gsm))
#else//#ifdef CC_AST_HAS_VERSION_13
				else if (p->read_last_format == AST_FORMAT_GSM)
#endif//#ifdef CC_AST_HAS_VERSION_13
			{
				if (size<=GSM_DATALEN*20)     //400ms
				{
					if (size < DGV_MAX_BUF - AST_FRIENDLY_OFFSET)
						memcpy(readbuf, SampleData, size);
					else
					{
						memcpy(readbuf, SampleData, DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1);
						size = DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1;
					}
#ifdef CC_AST_HAS_DATA_PTR
						p->f.data.ptr = p->buf + AST_FRIENDLY_OFFSET;//(short *)readbuf;(1.0.0_rc4)
#else//#ifdef CC_AST_HAS_DATA_PTR                        
						p->f.data = p->buf + AST_FRIENDLY_OFFSET;//(short *)readbuf;(1.0.0_rc4)
#endif//#ifdef CC_AST_HAS_DATA_PTR                        
					p->f.datalen = size; /*GSM_DATALEN * (size/GSM_DATALEN);*/
					p->f.samples = DATALEN * (size/GSM_DATALEN);

					if (option_debug==60 && ((p->verbosity & DG_LOG_READ_SAMPLES) != 0))
						//if (size>GSM_DATALEN)
						//{
						ast_log(LOG_NOTICE, "dgv_read(%d): gsm datalen=%d samples=%d size=%d \n",p->port, p->f.datalen, p->f.samples, size);
						//}

					if (dg_GetPortStatus(p->port)==spDialing)
					{
						p->f.frametype = AST_FRAME_NULL;
#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass.integer = 0;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass = 0;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						p->f.datalen = 0;
						p->f.samples = 0;
						p->f.mallocd = 0;
						p->f.offset = 0;
						p->f.src = "chan_dgv";
#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data.ptr = NULL;
#else//#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data = NULL;
#endif//#ifdef CC_AST_HAS_DATA_PTR
						if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
							ast_log(LOG_NOTICE, "Sending AST_FRAME_NULL because dg_GetPortStatus == spDialing on %d \n", p->port);
					}

					//ast_dgv_lock(&p->lock, p->port); {
						//QueueFrame(port, &p->f);
						QueueFrame(p, &p->f);
					//} ast_dgv_unlock(&p->lock, p->port);
				}
				else
				{
					if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
						ast_log(LOG_NOTICE, "dgv_read(%d): dropping samples - too much delay datalen=%d delay=%d\n",p->port,
								size,size*20/33);
				}

			}
#ifdef CC_AST_HAS_VERSION_13
				else if ((p->read_last_format == ast_format_get_codec_id(ast_format_alaw)) || (p->read_last_format == ast_format_get_codec_id(ast_format_ulaw)))
#else//#ifdef CC_AST_HAS_VERSION_13
				else if ((p->read_last_format == AST_FORMAT_ALAW) || (p->read_last_format == AST_FORMAT_ULAW))
#endif//#ifdef CC_AST_HAS_VERSION_13
			{
#ifdef DEBUG_Q
					fwrite(SampleData, 1, size, p->rec);
#endif//#ifdef DEBUG_Q
				//a_law and u_law waits for 160 samples with 160 datalen
				if (size<=DATALEN*20 && p->force_drop_rx==0 /*|| p->faxhandled*/ )     //400ms
				{
					if (size < DGV_MAX_BUF - AST_FRIENDLY_OFFSET)
					{
						memcpy(readbuf, SampleData, size);
						//if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
						//	ast_log(LOG_NOTICE, "dgv_read(%d): size < DGV_MAX_BUF on port %d \n", p->port);
					}
					else
					{
						memcpy(readbuf, SampleData, DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1);
						size = DGV_MAX_BUF - AST_FRIENDLY_OFFSET - 1;
						if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
							ast_log(LOG_NOTICE, "dgv_read(%d): size >= DGV_MAX_BUF \n", p->port);
					}

#ifdef CC_AST_HAS_DATA_PTR						
						p->f.data.ptr = p->buf + AST_FRIENDLY_OFFSET;//(short *)readbuf;(1.0.0_rc4)
#else//#ifdef CC_AST_HAS_DATA_PTR						
						p->f.data = p->buf + AST_FRIENDLY_OFFSET;//(short *)readbuf;(1.0.0_rc4)
#endif//#ifdef CC_AST_HAS_DATA_PTR						

					p->f.samples = size;
					p->f.datalen = size;

					//sin wave -
					/*if (p->audio_debug)
					{
						x=0;
						for (j=0; j<size; j++)
						{
							sin_tx[j] = senoide[x++];
							if (x==32) x=0;
						}
						p->f.data = (char *)sin_tx;
					}*/

					//ast_log(LOG_NOTICE, "dgv_read(%d): g711 datalen=%d samples=%d size=%d x=%d\n",p->port, p->f.datalen, p->f.samples, size, x);

					if (dg_GetPortStatus(p->port)==spDialing)
					{
						p->f.frametype = AST_FRAME_NULL;
#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass.integer = 0;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							p->f.subclass = 0;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						p->f.datalen = 0;
						p->f.samples = 0;
						p->f.mallocd = 0;
						p->f.offset = 0;
						p->f.src = "chan_dgv";
#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data.ptr = NULL;
#else//#ifdef CC_AST_HAS_DATA_PTR							
							p->f.data = NULL;
#endif//#ifdef CC_AST_HAS_DATA_PTR
						if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
							ast_log(LOG_NOTICE, "Sending AST_FRAME_NULL because dg_GetPortStatus == spDialing on %d \n", p->port);
					}

					//ast_dgv_lock(&p->lock, p->port); {
						//QueueFrame(port, &p->f);
						QueueFrame(p, &p->f);
					//} ast_dgv_unlock(&p->lock, p->port);
					if (option_debug == 60)
						ast_log(LOG_NOTICE, "dgv_read(%d): g711 datalen=%d samples=%d size=%d \n",p->port, p->f.datalen, p->f.samples, size);
				}
				else
				{
					if (p->force_drop_rx>0)
						p->force_drop_rx--;

					if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
						ast_log(LOG_NOTICE, "dgv_read(%d): ulaw/alaw dropping samples - too much delay datalen=%d delay=%d\n",p->port,
								size,size*8);
				}
			}
			else
			{
				if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
					ast_log(LOG_NOTICE, "dgv_read(%d): do not call QueueFrame function, no format selected\n", p->port);
			}
		
			p->samples_rx_count += p->f.samples;
			
#ifdef DEBUG_Q
/*			if ((p->verbosity & DG_LOG_READ_SAMPLES) != 0)
			{
    			if ((p->samples_rx_count%32767)==0)
				{
					ast_log(LOG_NOTICE, "dgv_read: Port %d rx_count=%d\n",p->port,p->samples_rx_count);
				}
			}*/
#endif//#ifdef DEBUG_Q

        }
		ast_dgv_unlock(&p->lock, p->port);		
	} //if (p)

	return /*DG_EXIT_SUCCESS new_vlib_change*/;
}

//------------------------------------------------------------------
// Queue frames to asterisk channel
//------------------------------------------------------------------
void QueueFrame(struct dgv_pvt *p, struct ast_frame *fp)
{
	struct ast_frame f;
	int trycnt;

	f = *fp;

	if (p)
	{
		//ast_log(LOG_NOTICE, "QueueFrame 2: Port %d\n",p->port);
		/* check if has a frame to queue to Asterisk */ 
		if ((f.frametype >= 0) && (f.frametype != AST_FRAME_NULL) && (p->ast_ch))
		{
			if (((p->verbosity & DG_LOG_AST_QUEUE_FRAME) != 0) && f.frametype!=AST_FRAME_VOICE)
#ifdef CC_AST_HAS_VERSION_1_8
						ast_verbose(VERBOSE_PREFIX_3 "%s: handle_owned: Prepared frame type[%d]subclass[%d], owner=[%s]\n",
									p->dev, f.frametype, f.subclass.integer, dgv_channel_name(p->ast_ch));
#else//#ifdef CC_AST_HAS_VERSION_1_8
						ast_verbose(VERBOSE_PREFIX_3 "%s: handle_owned: Prepared frame type[%d]subclass[%d], owner=[%s]\n",
									p->dev, f.frametype, f.subclass, dgv_channel_name(p->ast_ch));
#endif//#ifdef CC_AST_HAS_VERSION_1_8

			trycnt = 0;
			for (;;)
			{
				if ((p) && (p->ast_ch))
				{
#ifdef CC_AST_HAS_VERSION_1_8
						if (ast_channel_trylock(p->ast_ch))
#else//#ifdef CC_AST_HAS_VERSION_1_8
						#ifdef CC_AST_HAS_VERSION_1_6
							if (ast_dgv_trylock(&p->ast_ch->lock_dont_use, p->port))
						#else//#ifndef CC_AST_HAS_VERSION_1_6
							if (ast_dgv_trylock(&p->ast_ch->lock, p->port))
						#endif//#ifndef CC_AST_HAS_VERSION_1_6
#endif//#ifdef CC_AST_HAS_VERSION_1_8
					{
						#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
							DEADLOCK_AVOIDANCE(&p->lock);
						#else//#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
							ast_dgv_unlock(&p->lock, p->port);
							usleep(1);
							ast_dgv_lock(&p->lock, p->port);
						#endif//#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
						trycnt++;
                        if (trycnt>100)
                        {
    						if ((p->verbosity & DG_LOG_AST_QUEUE_FRAME) != 0)
#ifdef CC_AST_HAS_VERSION_1_8
									ast_verbose(VERBOSE_PREFIX_3 "chan_dgv handled_owned (%d): Too much delay to delivery frame %d - frame type[%d]subclass[%d]\n", p->port,trycnt,f.frametype, f.subclass.integer);
#else//#ifdef CC_AST_HAS_VERSION_1_8
									ast_verbose(VERBOSE_PREFIX_3 "chan_dgv handled_owned (%d): Too much delay to delivery frame %d - frame type[%d]subclass[%d]\n", p->port,trycnt,f.frametype, f.subclass);
#endif//#ifdef CC_AST_HAS_VERSION_1_8

    						break;
                        }
					}
					else
					{
                         if ((p->verbosity & DG_LOG_AST_QUEUE_FRAME) != 0)
                             ast_verbose(VERBOSE_PREFIX_3 "handled_owned : frame %d\n", trycnt);

						if (p && p->ast_ch!=NULL && fp!=NULL)
						    ast_queue_frame(p->ast_ch, &f);
						else
#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "QueueFrame: CHANNEL NULL Port %d - type[%d]subclass[%d]\n",p->port,f.frametype,f.subclass.integer);
#else//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "QueueFrame: CHANNEL NULL Port %d - type[%d]subclass[%d]\n",p->port,f.frametype,f.subclass);
#endif//#ifdef CC_AST_HAS_VERSION_1_8

						if (p->ast_ch==NULL)
						{
						    if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "handled_owned : ast_ch == NULL\n");

						    break;
						}

#ifdef CC_AST_HAS_VERSION_1_8
						ast_channel_unlock(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_1_8
							#ifdef CC_AST_HAS_VERSION_1_6
								ast_dgv_unlock(&p->ast_ch->lock_dont_use, p->port);
						    #else//#ifdef CC_AST_HAS_VERSION_1_6
								ast_dgv_unlock(&p->ast_ch->lock, p->port);
							#endif//#ifdef CC_AST_HAS_VERSION_1_6
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						break;
					}
				}
				else
					break;
			}
		}
	}
}

//------------------------------------------------------------
// This function handles digivoice voicerlib events
//------------------------------------------------------------
void event_handler(void *context_data)
{
	//struct dg_event_data_strucuture *EventContext;
	short ret=0;
    
	short ndis_size=0;
	char cAux;
	int res_sched;
	dg_event_data_structure *EventContext;
	struct dgv_pvt *p, *p_bridged, *p_smschannel;
    
	char szTemp[TAMSTR];
	char szGroupBReceived[TAMSTR];
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12
	char tmpfax[256];
	char tmpGSMMessage[300];
	int tmpGSMSignal;
	
	short nTempPort;

	//gsm
	char operator[20];
	char simcardid[40];
	int simstatus;
	char sca[30];

	//sms message
	int nForText;
	int nPosText;
	short nStepParse;
	char received_from[40];
    char date[20];
    char time[20];
    char text_message[200];
    char st_cod[20];
    char message_reference[20];
	
    //int fxss_len = 0;
	short nForPort=0;
#ifdef USE_LIBPRI    	
    	short span=0;
    	short dc=0;
#endif//#ifdef USE_LIBPRI

#ifdef CC_AST_HAS_VERSION_11
    	struct ast_party_id connected_id;
#endif//#ifdef CC_AST_HAS_VERSION_11

	const char *dialplan_ringtype;
	int dialplan_ringtype_value;
	int dialplan_ringtype_set;

    pthread_t threadid;
    pthread_attr_t attr;
    
    dg_signal_e1_thread_structure *p_e1_thread_structure;//pointer to voicerlib e1 thread structure

	if (!channel_up)
	{
        ast_log(LOG_WARNING, "*** Channel not ready to receive events! ***\n");
		return;
	}

	EventContext = ((dg_event_data_structure*)context_data);

	/* Copy received Data */
	switch (EventContext->command)
	{
		case EV_E1_ALARM:
			if (EventContext->port == 1)
				nTempPort = dg_GetAbsolutePortNumber(EventContext->card + 1, 1);
			else
				nTempPort = dg_GetAbsolutePortNumber(EventContext->card + 1, 31);

			p = find_ch_port(nTempPort);
			
			break;
		default:
			p = find_ch_port(EventContext->port);
		//0.9.8 break;
	}

    if (p==NULL)
    {
		switch (EventContext->command)
		{
			case EV_E1_ALARM:   	
				ast_log(LOG_NOTICE, "  -- Event 0x%x on card %d, port %d, data %x \n", EventContext->command, EventContext->card, EventContext->port, EventContext->data);
			default:
				ast_log(LOG_WARNING, "*** Channel not OK, can not receive events when p is null! ****\n");
				return;
		}
    }

	/*if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	{
		ast_log(LOG_NOTICE, "  -- Event 0x%x on port %d data=%x\n",EventContext->command,EventContext->port,EventContext->data);
	}*/

	switch (EventContext->command)
	{
        case EV_ERRORDETECTED:
        	switch (EventContext->data)
        	{
        		case ERROR_INTERRUPT:
        			ast_log(LOG_WARNING, "Interruption lost in DigiVoice card %d, please verify your hardware.\n", EventContext->card+1);
        			break;
        		case ERROR_1024TAPS:
        			ast_log(LOG_ERROR, "Error Enabling DSP Echo Cancellation (1024 TAPS) in DigiVoice card %d and port %d. Please verify your hardware/configuration.\n", EventContext->card+1, EventContext->data_aux);
        			break;
        		default:
        			ast_log(LOG_WARNING, "EV_ERRORDETECTED card %d, data %x, data_aux %x\n", EventContext->card+1, EventContext->data, EventContext->data_aux);
        			break;
        	}

            break;

        case EV_INFORECEIVED:
        	if (EventContext->data == INFO_VERSION)
        	{
            	if (option_verbose >= 3)
            		ast_verbose(VERBOSE_PREFIX_3 "Information of 'version number' received on card %d: %xh\n", EventContext->card+1, EventContext->data_aux);

				for (nForPort = 1; nForPort <= dg_GetCardPortsCount(EventContext->card+1); nForPort++)
				{
					p = find_ch_port(dg_GetAbsolutePortNumber(EventContext->card+1, nForPort));
					if (p)
						p->firmware_vr_board = EventContext->data_aux;
				}
        	}
        	else
        		if (EventContext->data == INFO_LECCOUNT)
        		{
        			if (option_verbose >= 3)
        				ast_verbose(VERBOSE_PREFIX_3 "Information of 'LEC count' received on card %d: %x\n", EventContext->card+1, EventContext->data_aux);

    				for (nForPort = 1; nForPort <= dg_GetCardPortsCount(EventContext->card+1); nForPort++)
    				{
    					p = find_ch_port(dg_GetAbsolutePortNumber(EventContext->card+1, nForPort));
    					if (p)
    						p->lec_count_board = EventContext->data_aux;
    				}
        		}
            break;

		case EV_E1_ALARM:
			//port means E1_A ou E1_B
			switch(EventContext->data)
		    {
					case ALARM_RSLIP:		//escorregamento					
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_RSLIP = EventContext->data_aux;

						ast_log(LOG_NOTICE, "Card %d E1-%d - RSLIP - count=%d \n", EventContext->card + 1, EventContext->port, EventContext->data_aux);
						manager_event(EVENT_FLAG_SYSTEM, "Alarm",
									  					 "CARD: %d\r\n"
									  					 "E1: %d\r\n"
									  					 "ALARM: RSLIP\r\n"
									  					 "STATUS: %d\r\n",
									  					 EventContext->card + 1, EventContext->port, EventContext->data_aux
									  );
						break;
					case ALARM_RAIS:		//alarme remoto
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_RAIS = EventContext->data_aux;
						
						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_RAIS++;						
						
						ast_log(LOG_NOTICE, "Card %d E1-%d - RAIS (%s) \n", EventContext->card + 1, EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
						manager_event(EVENT_FLAG_SYSTEM, "Alarm",
									  					 "CARD: %d\r\n"
														 "E1: %d\r\n"
                                    					 "ALARM: RAIS\r\n"
                                    					 "STATUS: %d\r\n",
									  					 EventContext->card + 1, EventContext->port, EventContext->data_aux
									  );
						break;
					case ALARM_AISS:		//indicacao de alarme
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_AISS = EventContext->data_aux;

						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_AISS++;						
						
						ast_log(LOG_NOTICE, "Card %d E1-%d - AISS (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
						manager_event(EVENT_FLAG_SYSTEM, "Alarm",
									  					 "CARD: %d\r\n"
														 "E1: %d\r\n"
                                    					 "ALARM: AISS\r\n"
                                    					 "STATUS: %d\r\n",
									  					 EventContext->card + 1, EventContext->port, EventContext->data_aux
									  );
						break;
					case ALARM_AIS16S:		//indicacao de alarme canal  16
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_AIS16S = EventContext->data_aux;					
					
						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_AIS16S++;						
						
						ast_log(LOG_NOTICE, "Card %d E1-%d - AIS16S (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
						manager_event(EVENT_FLAG_SYSTEM, "Alarm",
									  					 "CARD: %d\r\n"
														 "E1: %d\r\n"
                                    					 "ALARM: AIS16S\r\n"
                                    					 "STATUS: %d\r\n",
									  					 EventContext->card + 1, EventContext->port, EventContext->data_aux
									  );
						break;
					case ALARM_LOSS:		//perda de sinal
						if (EventContext->data_aux)//LOSS ON
						{
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_RSLIP = 0;
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_RAIS = 0;
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_AISS = 0;
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_AIS16S = 0;						
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_CRC4SYNC = 0;
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_MFSYNC = 0;
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_SYNC = 0;
#ifdef USE_LIBPRI
								dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_LOSS++;

							if ((EventContext->port == dgv_sync_conf[EventContext->card]) &&
								(dgv_sync_alternative_conf[EventContext->card] > 0) &&
								(dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_LOSS == 0))
							{
								ast_log(LOG_NOTICE, "Card %d E1-%d - ALARM_LOSS detected, changing sync_mode from %s (%d) to %s (%d)\n",
										EventContext->card + 1, EventContext->port,
										sync_mode_desc[dgv_sync_conf[EventContext->card]], dgv_sync_conf[EventContext->card],
										sync_mode_desc[dgv_sync_alternative_conf[EventContext->card]], dgv_sync_alternative_conf[EventContext->card]);

								dg_SetCardSyncMode(EventContext->card + 1, dgv_sync_alternative_conf[EventContext->card]);
							}
						}
						else
						{
							if ((EventContext->port == dgv_sync_conf[EventContext->card]) &&
								(dgv_sync_alternative_conf[EventContext->card] > 0) &&
								(dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_LOSS == 1))
							{
								ast_log(LOG_NOTICE, "Card %d E1-%d - ALARM_LOSS cleared, returning sync_mode to %s (%d)\n",
										EventContext->card + 1, EventContext->port,
										sync_mode_desc[dgv_sync_conf[EventContext->card]], dgv_sync_conf[EventContext->card]);

								dg_SetCardSyncMode(EventContext->card + 1, dgv_sync_conf[EventContext->card]);
							}
						}
					
						if (EventContext->port == 1)
							nTempPort = 1;
						else
							nTempPort = 31;
							
						for (nForPort = nTempPort; nForPort <= nTempPort + 29; nForPort++)
						{
							p = find_ch_port(dg_GetAbsolutePortNumber(EventContext->card + 1, nForPort));
	
							if (p)
								p->alarmed = EventContext->data_aux;
							else
							{
								ast_log(LOG_WARNING, "event_handler: p does not exist(EV_E1_ALARM) on card %d, relativeport %d\n", EventContext->card + 1, nForPort);
								break;
							}
						}
						
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_LOSS = EventContext->data_aux;

                        switch (p->sig)
                        {
#ifdef USE_LIBPRI
	                            case DG_SIG_PRI:
    	                            //find index
        	                        dc = dg_pri_findindex(EventContext->card+1,EventContext->port);
        
            	                    if (EventContext->data_aux)
            	                    {
                	                    ast_log(LOG_NOTICE, "Card %d E1-%d - ISDN LOSS (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
                                    
                    	                //alarmed - must block isdn
                        	            dchans[dc].alarmed = 1;
                            	        //ends all calls
                                	    if (!dchans[dc].resetting) 
                                	    {
                                    	    dchans[dc].resetting = 1;
                        					for (nForPort=1; nForPort<=dg_GetPortsCount(); nForPort++)
                        					{
                        						if (dg_GetCardNumber(nForPort) == dchans[dc].card)
                        						{
                        							if (dg_GetRelativeChannelNumber(nForPort)<=30)
                        								span=1;
                        							else
                        								span=2;
                                                
													if (span==dchans[dc].span)
													{
														p = find_ch_port(nForPort);

														if (p)
														{
															ast_dgv_lock(&p->lock, p->port);
															{
																if (p->pri)
																{
																	if (p->call)
																	{
																		if (!dgv_trylock_ccs(p, dc))
																		{
																			pri_hangup(p->pri, p->call, -1);
																			pri_destroycall(p->pri, p->call);

																			dgv_releaselock_ccs(dc);
																		}
																		else //if (!dgv_trylock_ccs(p), dc)
																			ast_log(LOG_WARNING, "ISDN LOSS: Unable to grab ccs_lock on port %d\n", p->port);
																	}
																}
																//0.9.8 p->pri = NULL;
																p->call = NULL;

																if (p->ast_ch)
																{
																	ast_log(LOG_NOTICE, ">>>>>--> Sending hangup to reset channels \n");
																	//has a channel associated
																	struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
																		f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
																		f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
																	f.src = "chan_dgv";
																	f.datalen = 0;
																	//ast_dgv_lock(&p->lock, p->port); {
																	QueueFrame(p, &f);
																	//} ast_dgv_unlock(&p->lock, p->port);
																	p->call_state=DG_AST_NONE;
																}
																else
																{
																	if (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0) || ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0))
																		ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_E1_ALARM) on port %d\n", p->port);
																}
															}
															ast_dgv_unlock(&p->lock, p->port);

															p->reserved=0;
															//dchans[dc].alarmed = 0;
															if (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0) || ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0))
																ast_log(LOG_NOTICE, ">>>>>--> Resetting port %d card %d span %d \n", nForPort, dchans[dc].card,dchans[dc].span);
														} //if (p)
													}
                        						}
                	                        }
                    	                    dchans[dc].resetting = 0;
                        	            }
                            	    } //if (EventContext->data_aux)
                                	else
	                                {
                                		//find index
                                		dc = dg_pri_findindex(EventContext->card+1,EventContext->port);

    	                                //alarm cleared
        	                            ast_log(LOG_NOTICE, "Card %d E1-%d - ISDN LOSS Cleared (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
            	                        dchans[dc].alarmed = 0;
                	                } //if (EventContext->data_aux)
                    	            break;
#endif//#ifdef USE_LIBPRI
                            default:
                                ast_log(LOG_NOTICE, "Card %d E1-%d - R2 LOSS (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
                                //0.9.8 break;
                        }
                        
                        manager_event(EVENT_FLAG_SYSTEM, "Alarm",
                                    				 	 "CARD: %d\r\n"
                                    					 "E1: %d\r\n"
                                    					 "ALARM: LOSS\r\n"
                                    					 "STATUS: %d\r\n",
                                    					 EventContext->card + 1, EventContext->port, EventContext->data_aux
                                      );                        
						break;
					case ALARM_CRC4SYNC:	//crc4
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_CRC4SYNC = EventContext->data_aux;
						
						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_CRC4SYNC++;						
						
						ast_log(LOG_NOTICE, "Card %d E1-%d - CRC4SYNC (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
                        manager_event(EVENT_FLAG_SYSTEM, "Alarm",
                                    				 	 "CARD: %d\r\n"
                                    					 "E1: %d\r\n"
                                    					 "ALARM: CRC4SYNC\r\n"
                                    					 "STATUS: %d\r\n",
                                    					 EventContext->card + 1, EventContext->port, EventContext->data_aux
                                      );						
						break;
					case ALARM_MFSYNC:		//sincronismo de multiquadro
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_MFSYNC = EventContext->data_aux;
						
						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_MFSYNC++;						
					
						ast_log(LOG_NOTICE, "Card %d E1-%d - MFSYNC (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
                        manager_event(EVENT_FLAG_SYSTEM, "Alarm",
                                    				 	 "CARD: %d\r\n"
                                    					 "E1: %d\r\n"
                                    					 "ALARM: MFSYNC\r\n"
                                    					 "STATUS: %d\r\n",
                                    					 EventContext->card + 1, EventContext->port, EventContext->data_aux
                                      );
						break;
					case ALARM_SYNC:		//sincronismo de quadro
						dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_SYNC = EventContext->data_aux;
						
						if (EventContext->data_aux)
							dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_SYNC++;						
					
						ast_log(LOG_NOTICE, "Card %d E1-%d - SYNC (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
                        manager_event(EVENT_FLAG_SYSTEM, "Alarm",
                                    				 	 "CARD: %d\r\n"
                                    					 "E1: %d\r\n"
                                    					 "ALARM: SYNC\r\n"
                                    					 "STATUS: %d\r\n",
                                    					 EventContext->card + 1, EventContext->port, EventContext->data_aux
                                      );
						break;
#ifdef USE_LIBPRI
						case ALARM_LINKDOWN:		//isdn link up/down
							dgv_e1alarms[EventContext->card][EventContext->port - 1].nALARM_LINKDOWN = EventContext->data_aux;

							if (EventContext->data_aux)
								dgv_e1alarmsCount[EventContext->card][EventContext->port - 1].nALARM_LINKDOWN++;

							ast_log(LOG_NOTICE, "Card %d E1-%d - LINKDOWN (%s) \n", EventContext->card+1,EventContext->port, showalarmstatus_desc[EventContext->data_aux]);
	                        manager_event(EVENT_FLAG_SYSTEM, "Alarm",
	                                    				 	 "CARD: %d\r\n"
	                                    					 "E1: %d\r\n"
	                                    					 "ALARM: LINKDOWN\r\n"
	                                    					 "STATUS: %d\r\n",
	                                    					 EventContext->card + 1, EventContext->port, EventContext->data_aux
	                                      );
							break;
#endif//#ifdef USE_LIBPRI
					default:
						ast_log(LOG_WARNING, "Card %d E1-%d - Alarm case default... \n", EventContext->card+1,EventContext->port);
		   	}
		   	break; 

        case EV_CALLERID:
            if (p && (p->sig==DG_SIG_FXO || p->sig==DG_SIG_CB_FXO || p->sig==DG_SIG_GSM))
            {
            	if (option_verbose >= 3)
            		ast_verbose(VERBOSE_PREFIX_3 "CallerID Received on port %d - callstate=%x\n", EventContext->port, p->call_state);            	
            	
				if ((p->sig==DG_SIG_GSM || (p->sig==DG_SIG_FXO && p->cid_detection_type == 2)) && p->ast_ch)
           		{
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "PBX already started, dropping EV_CALLERID on port %d\n", p->port);

           			break;
           		}

                dg_GetCallerId(p->port, szTemp);
                if (strlen(szTemp)>1)
                {
                	if ((p->sig==DG_SIG_GSM) || (p->sig==DG_SIG_FXO && p->cid_detection_type == 2))  
                		ast_copy_string(p->cid_num, szTemp, sizeof(p->cid_num));
                	else
                		ast_copy_string(p->cid_num, szTemp + 1, sizeof(p->cid_num));
                }
                else
                    p->cid_num[0] = '\0';
                
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                {
                    if (strlen(szTemp)>1)
                        ast_verbose(VERBOSE_PREFIX_3 "EV_CALLERID (Dgv/%d): CallerID Received - '%s'\n", p->port, p->cid_num);
                    else
                        ast_verbose(VERBOSE_PREFIX_3 "EV_CALLERID (Dgv/%d): No CallerID information.\n", p->port);
                }

                dg_GetNameId(p->port, szTemp);
                if (strlen(szTemp)>1)
                {
                	if (p->sig==DG_SIG_FXO && p->cid_detection_type == 2)
                		ast_copy_string(p->cid_name, szTemp, sizeof(p->cid_name));
                }
                else
                    p->cid_name[0] = '\0';
                
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                {
                    if (strlen(szTemp)>1)
                        ast_verbose(VERBOSE_PREFIX_3 "EV_CALLERID (Dgv/%d): CallerName Received - '%s'\n", p->port, p->cid_name);
                    else
                        ast_verbose(VERBOSE_PREFIX_3 "EV_CALLERID (Dgv/%d): No CallerName information.\n", p->port);
                }

                if ((p->sig == DG_SIG_GSM) || (p->sig==DG_SIG_FXO && p->cid_detection_type == 2))
                {
                	if ((p->call_state == DG_AST_NONE) || (p->call_state == DG_AST_RINGING))
                	{
                     	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                       		ast_log(LOG_NOTICE, "EV_CALLERID port %d - call_state == DG_AST_NONE \n", p->port);
            	    		
        				ast_dgv_lock(&p->lock, p->port);
        				{
        					if (option_verbose >= 3)
        						ast_verbose(VERBOSE_PREFIX_3 "Answering port %d\n", EventContext->port);
        						
        					p->call_state=DG_AST_RINGING;

     						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        						ast_log(LOG_NOTICE, "Calling dgv_new port %d using context=%s\n", p->port, p->context);

     						if (p->sig == DG_SIG_GSM)
     							p->cid_name[0] = '\0';

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
     							if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
							{
#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
									ast_dgv_unlock(&p->lock, p->port);

									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_CALLERID: Before ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

									if (p->ast_ch)
									{
										ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_CALLERID: DG_SIG_GSM, so ast_set_callerid on port %d \n", p->port);
									}
									else
										ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_CALLERID) on port %d\n", p->port);

									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_CALLERID: After ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

									ast_dgv_lock(&p->lock, p->port);
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
	        				}
        				}
        				ast_dgv_unlock(&p->lock, p->port);
                	} //if ((p->call_state == DG_AST_NONE) || (p->call_state == DG_AST_RINGING))
                } //if (p-sig == DG_SIG_GSM)
            } //if (p && (p->sig==DG_SIG_FXO || p->sig==DG_SIG_CB_FXO || p->sig==DG_SIG_GSM))

			break;
		case EV_E1CHANGESTATUS:
            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_E1CHANGESTATUS (%d) data %x\n",p->port,EventContext->data);

            switch (EventContext->data)
            {
                case C_SEIZURE:
                    //ask for first r2d number
                    if (p->sig==DG_SIG_R2_NDIS_VARIABLE && p->max_digits_rx>0)
                    {
                        /* first allocate channel to avoid double use -AST_STATE_PRERING*/
                        if (!p->reserved) 
                        {
                            if (p->call_state==DG_AST_NONE || p->call_state==DG_AST_WAITFORDIALTONE)
                            {
                                ast_dgv_lock(&p->lock, p->port);
                                {
                                    p->call_state=DG_AST_R2_SIGNALLING;

                                    p->reserved=1;	/* reserve channel */

                                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                        ast_log(LOG_NOTICE, ">> Seizure on port %d, reserved %d, variable\n", p->port, p->reserved);
                                }
                                ast_dgv_unlock(&p->lock, p->port);
                            }
                            dg_SetStartE1RxCount(p->port, 1);
                        }
                        else {
                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                ast_log(LOG_NOTICE, ">> Seizure on port %d - but it's already reserved - don't send seizure ack!\n",p->port);
                        }
                        
                    }
                    else
                    	if ((p->sig==DG_SIG_R2_NDIS_FIXED) && (p->call_state==DG_AST_NONE))
                    	{
                    		ast_dgv_lock(&p->lock, p->port);
                    		{
	                    		if (!p->reserved)
	                    		{
	                    			p->reserved=1;	/* reserve channel */

	                                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                                    ast_log(LOG_NOTICE, ">> Seizure on port %d, reserved %d, fixed\n", p->port, p->reserved);
	                    		}
	                            else
	                            	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                            		ast_log(LOG_NOTICE, ">> Seizure on port %d - but it's already reserved - don't send seizure ack!\n", p->port);
                    		}
                    		ast_dgv_unlock(&p->lock, p->port);
                    	}
                    break;
                case C_GROUP_II:
                    if (p->sig==DG_SIG_R2_NDIS_VARIABLE && p->call_state==DG_AST_R2_SIGNALLING)
                    {
                        //send group B
                        if (p->digit_timeout_sched > -1)
                        {
                            res_sched = ast_sched_del(sched, p->digit_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

                            p->digit_timeout_sched = -1;
                        }
                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        	ast_log(LOG_NOTICE,"E1 Events (%d): C_GROUP_II - Sending group B\n", p->port);

                        if (!p->wait_group_b)
                        {
                        	//automatically sending group B
                        	dg_R2SendGroupB(p->port, p->group_b_value);
                        }

                        ast_dgv_lock(&p->lock, p->port);
                        {
                            p->call_state=DG_AST_RINGING;
                            if (p)
                            {
                                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                    ast_log(LOG_NOTICE, "C_GROUP_II: Calling dgv_new port '%d', using context = '%s', cid_num = '%s'\n", p->port, p->context, p->cid_num);

                                p->cid_name[0] = '\0';

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
                                	if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
										if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
										if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
								{
#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
										ast_dgv_unlock(&p->lock, p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "C_GROUP_II: Before ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

										if (p->ast_ch)
											ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);
										else
											ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(C_GROUP_II) on port %d\n", p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "C_GROUP_II: After ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

										ast_dgv_lock(&p->lock, p->port);
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
								}
                            }
                            else
                            {
                            	if (option_verbose >= 3)
                                	ast_verbose(VERBOSE_PREFIX_3 "C_GROUP_II: No port available!\n");
                                p->call_state=DG_AST_NONE;
                            }
                        }
                        ast_dgv_unlock(&p->lock, p->port);							
                    }
                    break;
                case C_ID_RECEIVED:
                    if (p->sig==DG_SIG_R2_NDIS_VARIABLE && p->call_state==DG_AST_R2_SIGNALLING)
                    {
                        ast_dgv_lock(&p->lock, p->port); 
                        {
	                        //get callerid and set into asterisk channel
    	                    dg_GetCallerId(p->port, szTemp);
        	                if (strlen(szTemp)>1)
        	                	ast_copy_string(p->cid_num, szTemp + 1, sizeof(p->cid_num));
                    	    else
	                            p->cid_num[0] = '\0';
	                            
    	                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	                {            	                
            	                if (strlen(szTemp)>1)
                	                ast_verbose(VERBOSE_PREFIX_3 "Dgv/%d: E1 CallerID Event - '%s'\n", p->port, p->cid_num);
                    	    }
                        	dg_SetNextE1RxCount(p->port, 1);
                        } 
                        ast_dgv_unlock(&p->lock, p->port);
                    }
                    break;
                case C_NUMBER_RECEIVED:		/* Get called number */
                    dg_GetE1Number(p->port, szTemp);
                    ndis_size=strlen(szTemp);
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        ast_log(LOG_NOTICE,"E1 Events (%d): Partial Number received = %s\n", p->port, szTemp);
                
                    if (!ast_strlen_zero(szTemp))
                    {
                    	ast_copy_string(p->rdnis, szTemp, sizeof(p->rdnis));
                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                            ast_log(LOG_NOTICE, "C_NUMBER_RECEIVED: %s\n",p->rdnis);

                        ast_copy_string(p->exten, p->rdnis, sizeof(p->exten));
                    }
                    else
                    	ast_copy_string(p->exten, "s", sizeof(p->exten));

					if (!ast_strlen_zero(p->exten))
						ast_copy_string(p->dnid, p->exten, sizeof(p->dnid));

                    if (p->sig==DG_SIG_R2_NDIS_VARIABLE)   //not using rd2/mfc automatic
                    {
                        //manual R2DMFC
                        if (ndis_size == p->ask_for_cid_position)
                        {
                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                ast_log(LOG_NOTICE,"E1 Events (%d): Asking for CID...\n", p->port);
                            dg_R2AskForId(p->port);
                        }
                        else
                        {
                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                            	ast_log(LOG_NOTICE,"E1 Events (%d): Receiving digit count %d min=%d max=%d...\n", p->port, ndis_size,p->min_digits_rx, p->max_digits_rx);

                            if (ndis_size >= p->min_digits_rx && ndis_size<p->max_digits_rx /*&& !p->check_exten_after_timeout*/)
                            {
                                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                    ast_log(LOG_NOTICE,"E1 Events (%d): Checking extensions %s (%s@%s)...\n", p->port, szTemp, p->exten, p->context);

                                if (ast_matchmore_extension(NULL, p->context, p->exten, 1, p->cid_num))
                                {
                                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                        ast_log(LOG_NOTICE,"E1 Events (%d): Still has possible extensions. Waiting more digits...\n", p->port);

                                    dg_SetNextE1RxCount(p->port, 1);
                                }
                                else
                                    if (ast_exists_extension(NULL, p->context, p->exten, 1, p->cid_num))
                                    {
                                        //extension found! accept call
                                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                            ast_log(LOG_NOTICE,"E1 Events (%d): Extension (%s@%s) found!!!.\n", p->port, p->exten, p->context);

                                        //when send groupB you're saying that you receive enough numbers
                                        dg_R2AskForGroupII(p->port, DG_A3_COMPELED);
                                        //...
                                        //turnoff dtmf timer if it exists
                                        // 
                                        if (p->digit_timeout_sched > -1)
                                        {
                                            res_sched = ast_sched_del(sched, p->digit_timeout_sched);
                					    	if (res_sched == -1)
                					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

                                            p->digit_timeout_sched = -1;
                                        }
                                    }
                                    else
                                    {
                                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                            ast_log(LOG_NOTICE,"E1 Events (%d): No more possible extensions. Dropping call.\n", p->port);
                                        //turnoff timer
                                        if (p->digit_timeout_sched > -1)
                                        {
                                            res_sched = ast_sched_del(sched, p->digit_timeout_sched);
                					    	if (res_sched == -1)
                					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

                                            p->digit_timeout_sched = -1;
                                        }

                                        dg_HangUp(p->port);
                                        //....p->reserved=0;
                                        if ( (p->sig==DG_SIG_FXO)    || (p->sig==DG_SIG_CUSTOMCAS) || (p->sig==DG_SIG_FXS) ||
                                        	 (p->sig==DG_SIG_CB_FXS) || (p->sig==DG_SIG_CB_FXO)    || (p->sig==DG_SIG_GSM) )
                                            p->reserved=0;
                                        else
                                        {
                                            p->reserved=2;      //wait receiving idle
                                    	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                        	    ast_log(LOG_NOTICE, "E1 Events (%d): Changing reserved to 2.\n", p->port);
                                        }
                                    }
                            }
                            else
                                if (strlen(szTemp)==p->max_digits_rx)
                                {
                                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                        ast_log(LOG_NOTICE,"E1 Events (%d): Completed Number received = %s\n",  p->port, szTemp);
                                    dg_R2AskForGroupII(p->port, DG_A3_PULSED);
                                }
                                else
                                    dg_SetNextE1RxCount(p->port, 1);
                          }
                        }
                    break;
                case C_E1_IDLE:      //recebimento de idle para liberar o canal
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        ast_log(LOG_NOTICE, "C_E1_IDLE (%d): state=%x\n", p->port, p->call_state);

        			ast_dgv_lock(&p->lock, p->port);
        			{
            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_NOTICE, "C_E1_IDLE port %d, p->need_sc_ring %d\n", p->port, p->need_sc_ring);        				
        				
        				if (p->sig == DG_SIG_CB_FXS && p->need_sc_ring >= 1)
        				{
        					if (p->subchannel)	
        					{
        						if (p->ast_ch)	
        						{
    		    					if (p->need_sc_ring_sched > -1) 
    		    					{
    		    						res_sched = ast_sched_del(sched, p->need_sc_ring_sched);
    							    	if (res_sched == -1)
    							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

    		    						p->need_sc_ring_sched = -1;
    		    					}
    								p->need_sc_ring_sched = ast_sched_add(sched, 70, need_sc_ring_callback, p);
    								if (p->need_sc_ring_sched<0)
    									ast_log(LOG_WARNING, "ERROR Setting SecondCallRing callback on port %d\n", EventContext->port);
	                        	}
	                    		else
	                    		{
	                    			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                    				ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(C_E1_IDLE) on port %d\n", p->port);                    		
	                    		}
	                    	}
	                    	else
	                    	{
	                    		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                    			ast_log(LOG_WARNING, "event_handler: p->subchannel does not exist(C_E1_IDLE) on port %d\n", p->port);
	                    	}
        				}

		            	if (p->tone_thread_running == 1)
		            		p->tone_thread_running = 2;
		
		                if (p->stuttertone_thread_running == 1)
							p->stuttertone_thread_running = 0;        				
        			}
        			ast_dgv_unlock(&p->lock, p->port);

                    break;
                case C_B_ENDCALL:
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        ast_log(LOG_NOTICE, "C_B_ENDCALL (%d): state=%x\n",p->port,p->call_state);

                    //Disables Audio
                    //dg_DisableInputBuffer(p->port);
                    //dg_PauseInputBuffer(p->port, DG_PAUSE);removed on 1.0.4_rc2                    

                    if (p->call_state==DG_AST_CONVERSATION)
                    {
                        if (p->b_subscriber_hangup)
                        {

                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                ast_log(LOG_NOTICE, "C_B_ENDCALL (%d): End call\n",p->port);
                            //end call - send hangup to asterisk
                            //send hangup
                            //.......p->reserved=0;
                            if ( (p->sig==DG_SIG_FXO)    || (p->sig==DG_SIG_CUSTOMCAS) || (p->sig==DG_SIG_FXS) ||
                            	 (p->sig==DG_SIG_CB_FXS) || (p->sig==DG_SIG_CB_FXO)    || (p->sig==DG_SIG_GSM) )
                                p->reserved=0;
                            else
                            {
                                p->reserved=2;      //wait receiving idle
                        	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                            	    ast_log(LOG_NOTICE, "C_B_ENDCALL (%d): Changing reserved to 2.\n", p->port);
                            }

                            struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
                            f.src = "chan_dgv";
                            f.datalen = 0;
                            ast_dgv_lock(&p->lock, p->port); {
                                //QueueFrame(EventContext->port, &f);
                            	QueueFrame(p, &f);
                            } ast_dgv_unlock(&p->lock, p->port);
                            p->call_state=DG_AST_NONE;
                        }
                        else
                        {
                        	/* removed on 1.0.4_rc2
                            //check if there is a bridged channel with
                        	ast_dgv_lock(&p->lock, p->port);
                        	{
	                            if (p->ast_ch)
	                            {
	                                if ((dgv_chan_tmp=dgv_bridged_channel(p->ast_ch))!=NULL)
	                                {
	                                    //tell to bridged channel to stop playbuffer
	                                    if (dgv_chan_tmp->tech_pvt!=NULL && !strncmp(dgv_chan_tmp->name,type,3))
	                                    {
	                                        p_bridged = (struct dgv_pvt *)dgv_chan_tmp->tech_pvt;
	                                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                                                ast_log(LOG_NOTICE, "C_B_ENDCALL: put %s quite\n",p_bridged->dev);
	                                            dg_StopPlayBuffer(p_bridged->port);	                                            
	                                    }
	                                }
	                            }
	                            else
	                            {
	                            	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                            		ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(C_B_ENDCALL) on port %d\n", p->port);
	                            }
                        	}
                        	ast_dgv_unlock(&p->lock, p->port);
                        	*/
                        }
                    }
                    break;
                case C_ANSWERED:
                    if (p->call_state==DG_AST_CONVERSATION)
                    {
                        //b subscriber returns after C_B_ENDCALL
                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                            ast_log(LOG_NOTICE, "B Subscriber returns to call\n");
                        //dg_EnableInputBuffer(p->port, DG_DISABLE);
                        dg_PauseInputBuffer(p->port, DG_RELEASE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "C_ANSWERED: Releasing InputBuffer on port %d\n", p->port);
                    }
                    break;
			} //switch (EventContext->data)						
			
			if ((EventContext->data == C_ANSWERED) && (p->call_state==DG_AST_CALLING) && ((p->polarity == 1) || (p->polarity == 2)))
			{
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                	ast_log(LOG_NOTICE, "EV_E1CHANGESTATUS with data C_ANSWERED, calling EV_ANSWERED on port %d\n", p->port);
				
				//do not break, so EV_ANSWERED
			}
			else
				break;
		case EV_ANSWERED:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "EV_ANSWERED %d, call_state %d\n", EventContext->port, p->call_state);
			
			if (p->call_state==DG_AST_CALLING)
			{
				dg_DisableCallProgress(p->port);
				
				ast_dgv_lock(&p->lock, p->port);
				{
					//check if there is a bridged channel with
					if (!p->ast_ch)
					{
						ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_ANSWERED, step 1) on port %d\n", p->port);
						ast_dgv_unlock(&p->lock, p->port);
						break;
					}
					
					if (p->subchannel_audio_index == 0)
						p->subchannel_audio_index = 1;
					
					dg_DisableAnswerDetection(p->port);
					//dg_DisableCallProgress(p->port);						
					
#ifdef CC_AST_HAS_VERSION_12
						RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
						struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
					dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
					if (dgv_chan_bridged)
					{
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "EV_ANSWERED: Stopping music on hold on '%s'\n", dgv_channel_name(p->ast_ch));
						
						struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
							fmoh.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							fmoh.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						fmoh.src = "chan_dgv";													
						fmoh.datalen = 0;
						QueueFrame(p, &fmoh);
					}

					p->call_state = DG_AST_CONVERSATION;
					
					p->statistics.answer = ast_tvnow();
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "Starting to count call duration on port %d\n", p->port);

					struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
						f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
						f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
					f.src = "chan_dgv";
					f.datalen = 0;
					//ast_dgv_lock(&p->lock, p->port); {
					//QueueFrame(EventContext->port, &f);
					QueueFrame(p, &f);
					//} ast_dgv_unlock(&p->lock, p->port);

					if (!p->ast_ch)
					{
						ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_ANSWERED, step 2) on port %d\n", p->port);
						ast_dgv_unlock(&p->lock, p->port);
						break;
					}
					ast_setstate(p->ast_ch, AST_STATE_UP);
				}
				ast_dgv_unlock(&p->lock, p->port);
					
				/*
				ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
            	if (ret != DG_EXIT_SUCCESS)
                	ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n",p->port,ret);
				else
                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                    	ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n",p->port);
                */			

				dg_SetDetectionType(p->port,DETECT_DTMF,DG_ENABLE);

				//enable inputbuffer streaming with agc disabled
				//ret = dg_EnableInputBuffer(p->port,DG_DISABLE);
            	ret = dg_PauseInputBuffer(p->port, DG_RELEASE);
				if (ret != DG_EXIT_SUCCESS)
					ast_log(LOG_NOTICE, "Error releasing InputBuffer %d cod = %x - state %x\n",p->port,ret,
                            GetInputBufferStatus(p->port));
				else
	                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
	                	ast_log(LOG_WARNING, "EV_ANSWERED: Releasing InputBuffer on port %d\n", p->port);

				if (p->vconf_pass_through)
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_ANSWERED (%d) - vconf signalling detected...\n", p->port);

					pthread_attr_init(&attr);
					pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
					if (ast_pthread_create(&threadid, &attr, vconf_thread, p)>=0)
					{
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "vconf_thread Created(4) - port %d \n", p->port);
					}
					else
						ast_log(LOG_ERROR, "Error creating vconf_thread - port %d \n", p->port);
					pthread_attr_destroy(&attr);
				}
				else //if (p->vconf_pass_through)
				{
					pthread_attr_init(&attr);
					pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
					if (ast_pthread_create(&threadid, &attr, dialplan_echo_thread, p)>=0)
					{
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "dialplan_echo_thread Created(3) - port %d \n", p->port);
					}
					else
						ast_log(LOG_ERROR, "Error creating dialplan_echo_thread - port %d \n", p->port);
					pthread_attr_destroy(&attr);
				} //if (p->vconf_pass_through)

				if (p->sig == DG_SIG_GSM)
				{
					ast_dgv_lock(&p->lock, p->port);
						if (p->gsm_need_reload)
						{
							p->gsm_need_reload = 0;

							ast_dgv_unlock(&p->lock, p->port);

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "Starting scheduled reload on port %d\n", p->port);

							set_gsm_conf(p);
						}
						else
							ast_dgv_unlock(&p->lock, p->port);

						dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError = 0;
						dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError = 0;//Off

						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "Call completed, resetting Dial Error Flag on port %d\n", p->port);
				} //if (p->sig == DG_SIG_GSM)
			} //if (p->call_state==DG_AST_CALLING)
			break;			
		case EV_R2:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_verbose(VERBOSE_PREFIX_3 "Port %d R2D: %x\n",p->port, EventContext->data);
			
			p->last_r2_signal = EventContext->data;

            // New verbosity to save R2 signalling to file
            /*if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0) {
                sprintf(szTemp,"R2:%x",EventContext->data);
                save_log(p->port, szTemp);
            }*/

            /* check if is blocking collect calls */
            if (p->flashing == 2 && EventContext->data==R2_IDLE && p->sig != DG_SIG_CB_FXS)
            {
            	dg_Flash(p->port, 0, 0, 0);
            	
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                    ast_log(LOG_NOTICE, "EV_R2 (%x) - Calling hangup on port %d\n", EventContext->data, p->port);                	
            	
                /* Send hangup */
                struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
					f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
					f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
                f.src = "chan_dgv";
                f.datalen = 0;
                ast_dgv_lock(&p->lock, p->port); {
                    //QueueFrame(EventContext->port, &f);
                	QueueFrame(p, &f);
                } ast_dgv_unlock(&p->lock, p->port);
            }

            /* change the reserved flag*/
            if (EventContext->data==R2_IDLE)
            {
                if (p->reserved==2)
                {
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        ast_log(LOG_NOTICE, "EV_R2 (%d) - Changing reserved to 0 because receive %x\n", p->port, EventContext->data);
                    p->reserved = 0;
                    
            		if (p->reserved_timeout_sched > -1)//stop reserved timeout
            		{
            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_NOTICE, "EV_R2 (%d) - deleting reserved schedule\n", p->port);    	    		            	
    	    		            	
            			res_sched = ast_sched_del(sched, p->reserved_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            			p->reserved_timeout_sched = -1;
            		}                    
                }
            }

            if (p->sig == DG_SIG_R2_NDIS_VARIABLE && p->ccb_pass_through && p->call_state == DG_AST_CONVERSATION &&
            	(EventContext->data == R2_CLEAR_BACK || EventContext->data == R2_ANSWERED))
            {
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                    ast_log(LOG_NOTICE, "EV_R2 (%d) - Using ccb_pass_through, because we received a R2_CLEAR_BACK or R2_ANSWERED\n", p->port);
    			
    			ast_dgv_lock(&p->lock, p->port);
    			{
					if (!p->ast_ch)
					{
						ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_R2) on port %d(dropping ccb_pass_through)\n", p->port);
						ast_dgv_unlock(&p->lock, p->port);
						break;
					}

					switch (EventContext->data)
                	{
                		case R2_CLEAR_BACK:
                			if (p->flashing == 0)
                			{
                				p->flashing = 1;

                				if (option_verbose >= 3)
                					ast_log(LOG_NOTICE, "EV_R2: Setting R2_CLEAR_BACK - port %d \n", p->port);
                			}
                			else
                                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                    ast_log(LOG_NOTICE, "EV_R2 (%d) - flashing != 0, dropping ccb_pass_through\n", p->port);

                			break;
                		case R2_ANSWERED:
                			if (p->flashing == 1)
                			{
								p->flashing = 2;

								if (option_verbose >= 3)
									ast_log(LOG_NOTICE, "EV_R2: Setting R2_ANSWERED - port %d \n", p->port);
                			}
                			else
                                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                    ast_log(LOG_NOTICE, "EV_R2 (%d) - flashing != 1, dropping ccb_pass_through\n", p->port);

                			break;
                	} //switch (EventContext->data)
    			}
    			ast_dgv_unlock(&p->lock, p->port);
            } //if (p->sig == DG_SIG_R2_NDIS_VARIABLE && p->ccb_pass_through && p->call_state == DG_AST_CONVERSATION && ...

			break;			
		case EV_LINEREADY:
        	if (p)
        	{
        		if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
        		{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_LINEREADY port %d data: %x, call_state %d\n", p->port, EventContext->data, p->call_state);			

        			switch (p->call_state)
        			{
        				case DG_AST_NONE:
        					if (p->locked)
        					{
        						p->reserved=2;      //wait receiving idle

        			            if (option_verbose >= 3)
        			                ast_log(LOG_WARNING, "Port %d is locked!\n", p->port);

								if (p->gen_busy)
								{
									ast_dgv_lock(&p->lock, p->port);
									{
										switch (p->tone_thread_running)
										{
											case 0:
												p->tone_thread_running = 1;
												p->tone_thread_type = AST_CONTROL_BUSY;
												p->setting_tone_thread_values = 1;

												pthread_attr_init(&attr);
												pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
												if (pthread_create(&p->tone_thread_id, &attr, ToneThread, &(p->port)) < 0)
													p->tone_thread_running = 0;
												else
													if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
														ast_log(LOG_NOTICE, "EV_LINEREADY: tone_thread congestion created on port %d\n", p->port);
												pthread_attr_destroy(&attr);
												break;
											case 1:
												p->tone_thread_type = AST_CONTROL_BUSY;
												p->setting_tone_thread_values = 1;
												if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
													ast_log(LOG_NOTICE, "EV_LINEREADY: tone_thread congestion reconfigured on port %d\n", p->port);
												break;
											case 2:
												p->tone_thread_running = 1;
												p->tone_thread_type = AST_CONTROL_BUSY;
												p->setting_tone_thread_values = 1;
												if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
													ast_log(LOG_NOTICE, "EV_LINEREADY: tone_thread congestion re-enabled on port %d\n", p->port);
												break;
										} //switch (p->tone_thread_running)
									}
									ast_dgv_unlock(&p->lock, p->port);
								} //if (p->gen_busy)
								break;
							} //if (p->locked)

        					p->subchannel_audio_index = 1;
        					p->reserved=1;	/* reserve channel */

			            	if (strlen(p->szPortID) > 1)
			                	ast_copy_string(p->cid_num, p->szPortID, sizeof(p->cid_num));
			                else
			                    p->cid_num[0] = '\0';

			                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
			                {
			                	if (strlen(p->szPortID) > 1)
			                        ast_verbose(VERBOSE_PREFIX_3 "EV_LINEREADY (Dgv/%d): CallerID Detected - '%s'\n", p->port, p->cid_num);
			                    else
			                        ast_verbose(VERBOSE_PREFIX_3 "EV_LINEREADY (Dgv/%d): No CallerID information.\n", p->port);
			                }

			            	if (strlen(p->szPortName) > 1)
			                	ast_copy_string(p->cid_name, p->szPortName, sizeof(p->cid_name));
			                else
			                	p->cid_name[0] = '\0';

							//dynamically enable echo cancellation						
    	    			    if (p->echo_can)
			        	    {
#ifdef K_ECHO
									if (dg_EnableEchoCancelation(p->port, p->echo_type, p->echo_can, p->echo_training, p->echo_nlp) != DG_EXIT_SUCCESS)
#else//#ifdef K_ECHO
									if (dg_EnableEchoCancelation(p->port, p->echo_can, p->echo_training, p->echo_nlp) != DG_EXIT_SUCCESS)
#endif//#ifdef K_ECHO
            					{
		        			    	if (p->immediate)
									{    
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        			  						ast_log(LOG_NOTICE, "EV_LINEREADY (%d): Starting Immediate. \n", p->port);
        			  						
						            	if (option_verbose >= 3)
							                ast_verbose(VERBOSE_PREFIX_3 "Going to immediate on extension s|1 \n");						                
						    	            
						                //fxs immediate
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						            		if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    	    	    	            	{
    										pthread_attr_init(&attr);
    										pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);    	    	    	            		
	            		    	        	if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
    	            	    		        {
        	            	    		    	p->call_state = DG_AST_DIALING;
    											if (option_verbose >= 3)
                	            	    		ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
		                	            	}
	    		                	        else                                
    	    		                	    	ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
    	    		                	   	pthread_attr_destroy(&attr);
    		        	            	}
										else
    	    		        	        	ast_log(LOG_ERROR,"Error creating dgv_new to fxs immediate - port %d \n", p->port);
									}
									else// not p->immediate
									{         						
	            			        	ast_log(LOG_ERROR, "Port %d ERROR ENABLING DSP ECHO CANCELLATION \n", p->port);
	            			        	
					        		  	if (ast_app_has_voicemail(p->mailbox, NULL))
					        		  	{
											p->stuttertone_thread_running = 1;

        							    	pthread_attr_init(&attr);	
        							   		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);                
        									if (pthread_create(&p->stuttertone_thread_id, &attr, StutterToneThread, &(p->port)) < 0)
												p->stuttertone_thread_running = 0;
               								else
                   								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                       								ast_log(LOG_NOTICE, "EV_LINEREADY: stutter dial tone thread created port %d\n",p->port);                       			
               								pthread_attr_destroy(&attr);        		  		
        		  						}        		  		
        		  						else
        		  						{        		  											
			        						dg_GenerateMF(p->port, GENERATE_TONE1, 0);
			        						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_LINEREADY GENERATE_TONE1 on %d \n", p->port);	
        		  						}

    	    							dg_SetDetectionType(p->port, DETECT_DTMF, DG_ENABLE);
       								
										//fxs overlapdial
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    	    								if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    	    	                		{
											pthread_attr_init(&attr);
											pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);    	    	                			                                
	            	            			if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
                	            			{
	            	            				p->call_state = DG_AST_DIALING;
    											if (option_verbose >= 3)
                            	    				ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
	                            			}
    	                        			else                                
        	                    				ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
											pthread_attr_destroy(&attr);
            	            			}
										else
	    	                    		{
    		                    			ast_log(LOG_ERROR,"Error creating dgv_new to fxs overlapdial - port %d \n", p->port);
	        		               		}
            						}
            					}
								else
    	    			        {    	    
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "Port %d Enabling(1) DSP Echo Cancellation - taps = %s \n", p->port, echo_taps_desc[p->echo_can]);

									dg_PickUp(p->port, 64);//only used as a timer
			                	}
	        			    }
	        			    else //if (p->echo_can)
	        			    {
	        			    	if (p->sig == DG_SIG_CB_FXS)
	        			    	{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "PickingUp port %d without DSP Echo Cancellation \n", p->port);

									dg_PickUp(p->port, 64);//only used as a timer
	        			    	}
	        			    	else //if (p->sig == DG_SIG_CB_FXS)
	        			    	{
		        			    	if (p->immediate)
									{
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	        		  						ast_log(LOG_NOTICE, "EV_LINEREADY (%d): Starting Immediate. \n", p->port);
	        		  						
						            	if (option_verbose >= 3)
							                ast_verbose(VERBOSE_PREFIX_3 "Going to immediate on extension s|1 \n");						                
							                
						        	    //fxs immediate
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						            		if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	    	        	            	{
											pthread_attr_init(&attr);
											pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);    	        	            		
		            	    	        	if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
	    	            	    	        {
	        	            	    	    	p->call_state = DG_AST_DIALING;
	    										if (option_verbose >= 3)
	                	            	    		ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
			                	            }
	    		                	        else                                
	        		                	    	ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
											pthread_attr_destroy(&attr);
	    	        	            	}
										else
	    	    	        	        	ast_log(LOG_ERROR,"Error creating dgv_new to fxs immediate - port %d \n", p->port);
									}
									else// not p->immediate
									{
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	        		  						ast_log(LOG_NOTICE, "EV_LINEREADY (%d): Starting DIAL_TONE. \n", p->port);									
	        		  						        			    	
						        		if (ast_app_has_voicemail(p->mailbox, NULL))
						        		{
											p->stuttertone_thread_running = 1;
	
	        							    pthread_attr_init(&attr);	
	        							   	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);                
	        								if (pthread_create(&p->stuttertone_thread_id, &attr, StutterToneThread, &(p->port)) < 0)
												p->stuttertone_thread_running = 0;
	               							else
	                   							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                       							ast_log(LOG_NOTICE, "EV_LINEREADY: stutter dial tone thread created port %d\n",p->port);                       			
	               							pthread_attr_destroy(&attr);        		  		
	        		  					}        		  		
	        		  					else
	        		  					{        		  											
				        					dg_GenerateMF(p->port, GENERATE_TONE1, 0);
				        					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_LINEREADY GENERATE_TONE1 on %d \n", p->port);			        					
	        		  					}        							        		  						        			    	
	        		  						        			    	
		    	    					dg_SetDetectionType(p->port, DETECT_DTMF, DG_ENABLE);
	        						
										//fxs overlapdial
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
		    	    						if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	        	        	        	{
	        								pthread_attr_init(&attr);
											pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
		            	    	        	if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
	    	            	    	        {
	        	            	    	    	p->call_state = DG_AST_DIALING;
	    										if (option_verbose >= 3)
	                	            	    		ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
			                	            }
	    		                	        else                                
	        		                	    	ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
											pthread_attr_destroy(&attr);
		        	    	            }	
										else
	    	    	        	        	ast_log(LOG_ERROR,"Error creating dgv_new to fxs overlapdial - port %d \n", p->port);
									}
	        			    	} //if (p->sig == DG_SIG_CB_FXS)
	        			    } //if (p->echo_can)
        	                break;
						case DG_AST_CALLING:
							if ((p->ast_ch) && (p->sig == DG_SIG_FXS))
							{
								if (p->subchannel_audio_index == 0)
									p->subchannel_audio_index = 1;

								dg_DisableAnswerDetection(p->port);
								dg_DisableCallProgress(p->port);
								
								ast_dgv_lock(&p->lock, p->port);
								{
									if (p->ast_ch)
									{
#ifdef CC_AST_HAS_VERSION_12
											RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
											struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
										dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
										if (dgv_chan_bridged)
										{
											if (option_verbose >= 3)
												ast_log(LOG_NOTICE, "EV_LINEREADY: Stopping music on hold on '%s'\n", dgv_channel_name(p->ast_ch));
											
											struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
												fmoh.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
												fmoh.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
											fmoh.src = "chan_dgv";													
											fmoh.datalen = 0;
											QueueFrame(p, &fmoh);											
										}
									}
									else
										ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_LINEREADY) on port %d\n", p->port);
								}
								ast_dgv_unlock(&p->lock, p->port);

								//dynamically enable echo cancellation
    	    			    	if (p->echo_can)
			        	    	{
#ifdef K_ECHO
										if (dg_EnableEchoCancelation(p->port, p->echo_type, p->echo_can, p->echo_training, p->echo_nlp) != DG_EXIT_SUCCESS)
#else//#ifdef K_ECHO
										if (dg_EnableEchoCancelation(p->port, p->echo_can, p->echo_training, p->echo_nlp) != DG_EXIT_SUCCESS)
#endif//#ifdef K_ECHO
    	        					{
										ast_log(LOG_ERROR, "Port %d ERROR ENABLING DSP ECHO CANCELLATION \n", p->port);										    	        						
										p->call_state = DG_AST_CONVERSATION;
										
										struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										f.src = "chan_dgv";
										f.datalen = 0;
										ast_dgv_lock(&p->lock, p->port);
										{
											//QueueFrame(EventContext->port, &f);
											QueueFrame(p, &f);
											//} 
											//ast_dgv_unlock(&p->lock, p->port);

											if (p->ast_ch)
												ast_setstate(p->ast_ch, AST_STATE_UP);
											else
												ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_LINEREADY) on port %d\n", p->port);										
										}
										ast_dgv_unlock(&p->lock, p->port);
        	    					}
									else
    	    				        {
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "Port %d Enabling(2) DSP Echo Cancellation - taps = %s \n", p->port, echo_taps_desc[p->echo_can]);
										
										dg_PickUp(p->port, 64);//only used as a timer
			                		}
	        			    	}
	        			    	else
	        			    	{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        		  						ast_log(LOG_NOTICE, "EV_LINEREADY (%d): ast_setstate(AST_STATE_UP). \n", p->port);
        		  							        			    		
									p->call_state = DG_AST_CONVERSATION;
									
									struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
									f.src = "chan_dgv";
									f.datalen = 0;
									ast_dgv_lock(&p->lock, p->port); 
									{
										//QueueFrame(EventContext->port, &f);
										QueueFrame(p, &f);
										//} 
										//ast_dgv_unlock(&p->lock, p->port);

										if (p->ast_ch)
											ast_setstate(p->ast_ch, AST_STATE_UP);
										else
											ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_LINEREADY) on port %d\n", p->port);											
									} 
									ast_dgv_unlock(&p->lock, p->port);										
		        			    } 		

								//enable callprogress during conversation
								ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
                				if (ret != DG_EXIT_SUCCESS)
                    				ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n",p->port,ret);
                				else
                    				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        				ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n",p->port);								
								
								dg_SetDetectionType(p->port, DETECT_DTMF, DG_ENABLE);

			                	ret = dg_PauseInputBuffer(p->port, DG_RELEASE);
								if (ret != DG_EXIT_SUCCESS)
									ast_log(LOG_NOTICE, "Error releasing InputBuffer %d cod = %x - state %x\n",p->port,ret, GetInputBufferStatus(p->port));			
								else
									if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
										ast_log(LOG_WARNING, "EV_LINEREADY: Releasing InputBuffer on port %d\n", p->port);
							}
							break;       	              
        			} //switch (p->call_state)
        			break;
        		}
        		else //if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
        		{
        			if ((p->sig == DG_SIG_CB_FXO) && (p->call_state == DG_AST_NONE))
        			{
        				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        					ast_log(LOG_NOTICE, "EV_LINEREADY port %d data: %x \n", p->port, EventContext->data);        				
        			
        				//do not break, so EV_RINGS
        				
        			} //if (p->sig == DG_SIG_CB_FXO)
        			else
        			{
        				break;
        			}
        		} //if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
            }
		case EV_RINGS:
#ifdef DEBUG_PRI
				ast_log(LOG_NOTICE, "EV_RINGS: STEP 1...\n");
#endif//#ifdef DEBUG_PRI

        	if (option_verbose >= 3)
        		ast_verbose(VERBOSE_PREFIX_3 "Ringing on port %d - callstate=%x\n", EventContext->port, p->call_state);

        	switch (p->sig)
        	{
            	case DG_SIG_FXO:
            		dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1] = 0;
            		p->alarmed = 0;
            		
					if (p->alarmNDT_timeout_sched > -1) 
					{
						res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->alarmNDT_timeout_sched = -1;
					}
            	case DG_SIG_GSM:
            		if (p->ring_timeout_sched > -1)//stop ring timeout
            		{
            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_NOTICE, "EV_RINGS port %d - deleting ring schedule \n", p->port);    	    		            	
    	    		            	
            			res_sched = ast_sched_del(sched, p->ring_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            			p->ring_timeout_sched = -1;
            		}

            		//set ring timeout
            		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            			ast_log(LOG_NOTICE, "EV_RINGS port %d - Creating ring schedule \n", p->port);        	        			
        	        			
            		p->ring_timeout_sched = ast_sched_add(sched, ringtimeout, ring_timeout_callback, p);

            		if (p->sig == DG_SIG_GSM)
            		{
						if (p->alarmed == 99)
							p->alarmed = 0;

						if (p->gsmerror_timeout_sched > -1)
						{
	            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	            				ast_log(LOG_NOTICE, "EV_RINGS port %d - deleting gsmerror schedule\n", p->port);

							res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

							p->gsmerror_timeout_sched = -1;
						}
            		}

            		break;
            	case DG_SIG_FXS:
            		//TODO: ring timeout                	
            		break;
            	case DG_SIG_CB_FXS:
            		//TODO: ring timeout
            		break;            		
            	case DG_SIG_R2_NDIS_FIXED:
            	case DG_SIG_R2_NDIS_VARIABLE:
            		p->need_ringback = 1;
            		break;
            	case DG_SIG_CB_FXO:
            		dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1] = 0;
            		p->alarmed = 0;
            		
					if (p->alarmNDT_timeout_sched > -1) 
					{
						res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->alarmNDT_timeout_sched = -1;
					}            		
            		break;
        		//0.9.8 default:
        		//0.9.8 break;
        	}

        	switch (p->call_state)
        	{
               	case DG_AST_R2_SIGNALLING:
               		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
               			ast_log(LOG_NOTICE, "EV_RINGS port %d - call_state == DG_AST_R2_SIGNALLING \n", p->port);
    	    		
               		ast_dgv_lock(&p->lock, p->port);
               		{
               			p->reserved = 1;

               			if (p->ast_ch)
               				ast_setstate(p->ast_ch, AST_STATE_RING);
               			else
               				ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_RINGS) on port %d\n", p->port);
               		}
               		ast_dgv_unlock(&p->lock, p->port);                    
                
               		p->call_state=DG_AST_RINGING;
               		break;
               	case DG_AST_NONE:
               		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
               			ast_log(LOG_NOTICE, "EV_RINGS port %d - call_state == DG_AST_NONE\n", p->port);
    	    		
					//turnoff dtmf timer if it exists
					if (p->digit_timeout_sched > -1)
                    {
                        res_sched = ast_sched_del(sched, p->digit_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

                        p->digit_timeout_sched = -1;
                    }

					ast_dgv_lock(&p->lock, p->port);
					{
						p->reserved=1;	/* reserve channel */
						
						if ((p->sig==DG_SIG_GSM) || (p->sig==DG_SIG_FXO && p->cid_detection_type == 2))
						{
							p->call_state=DG_AST_RINGING;
							ast_dgv_unlock(&p->lock, p->port);

							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "Waiting for a CallerID information on port %d\n", p->port);

							break;
						}

						if (option_verbose >= 3)        	    								
							ast_verbose(VERBOSE_PREFIX_3 "Answering port %d\n", p->port);						
						
						p->call_state=DG_AST_RINGING;
					}
					ast_dgv_unlock(&p->lock, p->port);
						
					//do not break, so DG_AST_RINGING
               	case DG_AST_RINGING:
               		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
               			ast_log(LOG_NOTICE, "EV_RINGS port %d - call_state == DG_AST_RINGING\n", p->port);
               		
               		if (p->ast_ch)
               		{
						if (option_verbose >= 3)        	    								
							ast_verbose(VERBOSE_PREFIX_3 "PBX already started, dropping EV_RINGS on port %d\n", p->port);

               			break;
               		}	               			
					
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "Calling dgv_new port %d using context=%s\n", p->port, p->context);
					
					ast_dgv_lock(&p->lock, p->port);
					{
						//get callerid and set it into asterisk channel
                        if (p->sig!=DG_SIG_FXO && p->sig!=DG_SIG_CB_FXO && p->sig!=DG_SIG_PRI && p->sig!=DG_SIG_GSM)
                        {
                            dg_GetCallerId(p->port, szTemp);
                            if (strlen(szTemp)>1)
                            {
                            	p->cid_category[0] = szTemp[0];
                            	p->cid_category[1] = '\0';
                                ast_copy_string(p->cid_num, szTemp + 1, sizeof(p->cid_num));
                            }
                            else
                                p->cid_num[0] = '\0';
                        } //if (p->sig!=DG_SIG_FXO && p->sig!=DG_SIG_CB_FXO && p->sig!=DG_SIG_PRI && p->sig!=DG_SIG_GSM)

                        if (p->sig == DG_SIG_PRI)
                        	ast_copy_string(p->cid_name, p->pri_cid_name, sizeof(p->cid_name));
                        else
                        	p->cid_name[0] = '\0';

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
                        	if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
								if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						{
	                        if (p->sig!=DG_SIG_FXO && p->sig!=DG_SIG_CB_FXO && p->sig!=DG_SIG_PRI && p->sig!=DG_SIG_GSM)
	                        {
#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
									ast_dgv_unlock(&p->lock, p->port);

									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_RINGS: Before ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

									if (p->ast_ch)
										ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);
									else
										ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_RINGS) on port %d\n", p->port);

									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_RINGS: After ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

									ast_dgv_lock(&p->lock, p->port);
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC

	                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                        {
	    	                        if (strlen(szTemp)>1)
	        	                        ast_verbose(VERBOSE_PREFIX_3 "EV_RINGS (Dgv/%d): CallerID Received - '%s' \n", p->port, p->cid_num);
	            	                else
	                	                ast_verbose(VERBOSE_PREFIX_3 "EV_RINGS (Dgv/%d): No CallerID information.\n", p->port);
	                    		}
							}
							else //if (p->sig!=DG_SIG_FXO && p->sig!=DG_SIG_CB_FXO && p->sig!=DG_SIG_PRI && p->sig!=DG_SIG_GSM)
							{
								if (p->sig == DG_SIG_FXO || p->sig == DG_SIG_CB_FXO)
								{
#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
										ast_dgv_unlock(&p->lock, p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_RINGS: Before ast_set_callerid(%s), port %d\n", p->cid_num, p->port);

										if (p->ast_ch)
										{
											ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);

											if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_RINGS: DG_SIG_FXO/CB_FXO/GSM so ast_set_callerid on port %d \n", p->port);
										}
										else
											ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_RINGS) on port %d\n", p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_RINGS: After ast_set_callerid(%s), port %d\n", p->cid_num, p->port);
										
										ast_dgv_lock(&p->lock, p->port);
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
								}
								else //if (p->sig == DG_SIG_FXO || p->sig == DG_SIG_CB_FXO)
								{
									if (p->sig == DG_SIG_PRI)
									{
#ifdef USERUSER_INFO
											ast_dgv_unlock(&p->lock, p->port);

											if (p->ast_ch)
												if (!ast_strlen_zero(p->useruserinfo))
													pbx_builtin_setvar_helper(p->ast_ch, "DGV_USERUSERINFO", p->useruserinfo);

											ast_dgv_lock(&p->lock, p->port);
#endif//#ifdef USERUSER_INFO

#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
											ast_dgv_unlock(&p->lock, p->port);

											if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
												ast_log(LOG_NOTICE, "EV_RINGS: before try to set_callerid on port %d \n", p->port);

											if (p->ast_ch)
											{
												ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);

												if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
													ast_log(LOG_NOTICE, "EV_RINGS: !ast_matchmore_extension on DG_SIG_PRI so ast_set_callerid on port %d \n", p->port);
											}
											else
												ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_RINGS) on port %d\n", p->port);

											if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
												ast_log(LOG_NOTICE, "EV_RINGS: after try to set_callerid on port %d \n", p->port);

											ast_dgv_lock(&p->lock, p->port);
#endif//#ifndef CC_AST_HAS_EXT_CHAN_ALLOC
									} //if (p->sig == DG_SIG_PRI)
								} //if (p->sig == DG_SIG_FXO || p->sig == DG_SIG_CB_FXO)
							} //if (p->sig!=DG_SIG_FXO && p->sig!=DG_SIG_CB_FXO && p->sig!=DG_SIG_PRI && p->sig!=DG_SIG_GSM)
						} //if (dgv_new(p, AST_STATE_RING, p->context, START_PBX, 0)!=NULL)
					}
					ast_dgv_unlock(&p->lock, p->port);
					break;
        	} //switch (p->call_state)
        	break;
		case EV_FLASH:
        	if (p)
        	{
        		if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) 
        		{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_FLASH port %d data: %x \n", p->port, EventContext->data);
						
					if (p->call_state == DG_AST_CONVERSATION)
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "EV_FLASH port %d - Sending AST_FRAME_CONTROL \n", p->port);
														
						struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass.integer = AST_CONTROL_FLASH;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass = AST_CONTROL_FLASH;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						f.src = "chan_dgv";
						f.datalen = 0;
						ast_dgv_lock(&p->lock, p->port); 
						{
							//QueueFrame(EventContext->port, &f);
							QueueFrame(p, &f);
						}
						ast_dgv_unlock(&p->lock, p->port);
						
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_NOTICE, "EV_FLASH: AST_CONTROL_FLASH on port %d \n", p->port);

						if (features_conf.fxs_atxfer[0] != '\0')
						{            
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_NOTICE, "EV_FLASH: Sending %s to asterisk on port %d \n", features_conf.fxs_atxfer, p->port);						
							
							p->flashing = 1;
										
		                    int nAtxferLen = strlen(features_conf.fxs_atxfer);
							char cAtxfer;
                		    int nForLen;

		                    for (nForLen = 0; nForLen < nAtxferLen; nForLen++)
        		            {
								cAtxfer = features_conf.fxs_atxfer[nForLen];

								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_FLASH: Sending DTMF %d on port %d \n", cAtxfer, p->port);

								struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = cAtxfer;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = cAtxfer;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
        		                ast_dgv_lock(&p->lock, p->port);
                		        {
									QueueFrame(p, &f);
								}
                        		ast_dgv_unlock(&p->lock, p->port);                        
                    		}
						}
						else
						{
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_WARNING, "EV_FLASH: FXS transfer command not found!!! \n");
						}
					}
        		}
        	}
			break;            
        case EV_CALLING:
        	if (p)
        	{        
            	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                	ast_log(LOG_NOTICE, "EV_CALLING port %d data: %x - state=%x\n",p->port, EventContext->data, p->call_state);        		
        		
				switch (p->sig)
               	{
					case DG_SIG_R2_NDIS_FIXED:
    	            case DG_SIG_R2_NDIS_VARIABLE:    	            	
    					p_e1_thread_structure = dg_GetE1ThreadPointer(p->port);

    					if (p_e1_thread_structure)
    					{    						
    						if ( (p->group_b_received == p_e1_thread_structure->b_free_withoutbilling) ||
    						 	 (p->group_b_received == p_e1_thread_structure->b_number_unknown     ) ||
    							 (p->group_b_received == p_e1_thread_structure->b_number_changed     ) ||
    							 (p->group_b_received == p_e1_thread_structure->b_out_of_service     ) )
    						{
								//nothing to do
    						}
    						else
    						{
								if (p->call_state != DG_AST_NONE)
								{	
									struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
									f.src = "chan_dgv";
									f.datalen = 0;
									ast_dgv_lock(&p->lock, p->port); {
										//QueueFrame(EventContext->port, &f);
										QueueFrame(p, &f);
									} ast_dgv_unlock(&p->lock, p->port);
								} //if (p->call_state != DG_AST_NONE)
    						}
						} //if (p_e1_thread_structure)
    					else
    						ast_log(LOG_WARNING, "event_handler: p_e1_thread_structure does not exist(EV_CALLING) on port %d\n", p->port);
    						
            	        break;
					default:
						if (p->call_state != DG_AST_NONE)
						{	
							struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
							f.src = "chan_dgv";
							f.datalen = 0;
							ast_dgv_lock(&p->lock, p->port); {
								//QueueFrame(EventContext->port, &f);
								QueueFrame(p, &f);
							} ast_dgv_unlock(&p->lock, p->port);
						}
						//0.9.8 break;					
               	}     		
        	}
			break;
		case EV_FAX_CNG:/*Fax (T_FAX1 = CP_TONE2 = 1100Hz = CNG = DSP_FAXMODE_DETECT_CNG = 'f' = 1)*/
			if (p->fax_detection)
			{
				if (p->call_state != DG_AST_NONE)
				{
					ast_dgv_lock(&p->lock, p->port);
					{
						if (!p->ast_ch)
						{
							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_FAX_CNG) on port %d\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);
							break;
						}

						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "FAX CNG detected on port %d\n", p->port);

   	                    p->fax_tone_state = 1;
					}
					ast_dgv_unlock(&p->lock, p->port);
				} //if (p->call_state != DG_AST_NONE)
			}
			else //if (p->fax_detection)
			{
				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "FAX CNG detected(%d), but the FAX option is disabled\n", p->port);
			} //if (p->fax_detection)
			break;
		case EV_FAX_CED:/*Fax (T_FAX2 = CP_TONE3 = 2100Hz = CED = DSP_FAXMODE_DETECT_CED = 'e' = 2)*/
			if (p->fax_detection)
			{
				if (p->call_state != DG_AST_NONE)
				{
					ast_dgv_lock(&p->lock, p->port);
					{
						if (!p->ast_ch)
						{
							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_FAX_CED) on port %d\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);
							break;
						}

						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "FAX CED detected on port %d\n", p->port);

   	                    p->fax_tone_state = 2;
					}
					ast_dgv_unlock(&p->lock, p->port);
				} //if (p->call_state != DG_AST_NONE)
			}
			else //if (p->fax_detection)
			{
				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "FAX CED detected(%d), but the FAX option is disabled\n", p->port);
			} //if (p->fax_detection)
			break;
		case EV_FAX://fax handling
			if (p->fax_detection)
			{
				if (p->call_state != DG_AST_NONE)
				{
					ast_dgv_lock(&p->lock, p->port);
					{
						if (!p->ast_ch)
						{
							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_FAX) on port %d\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);
							break;
						}

						if (p->fax_pass_through)
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "FAX detected on port %d\n",p->port);

    	                    //check if there is a bridged channel with
        	                if ((dgv_chan_tmp = dgv_bridged_channel(p->ast_ch)) != NULL)
            	            {
            					if (dgv_channel_tech_pvt(dgv_chan_tmp) != NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype, 3))
            					{
            						p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);
            						if (p_bridged->fax_pass_through)
            						{
            							if (dg_GetCardNumber(p->port) == dg_GetCardNumber(p_bridged->port))
            							{
           									ret=dg_LocalBridgeConnect(p->port, p_bridged->port);

           									if (ret!=DG_EXIT_SUCCESS)
            									ast_log(LOG_ERROR, "Error connecting LocalBridge Fax for port %d and port %d - cod %x\n", p->port, p_bridged->port, ret);
            								else
            								{
            									if (option_verbose >= 3)
            										ast_verbose(VERBOSE_PREFIX_3 "LocalBridge Fax connected port %d and port %d\n", p->port, p_bridged->port);

            									p->fax_established = p_bridged->fax_established = 1;
            									p->fax_port_bridged = p_bridged->port;
            									p_bridged->fax_port_bridged = p->port;

                								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                									ast_log(LOG_NOTICE, "EV_FAX (%d): LocalBridge Fax connected, p->fax_port_bridged %d, p_bridged->fax_port_bridged %d\n", p->port, p->fax_port_bridged, p_bridged->fax_port_bridged);
            								}
            							}
            							else
            								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            									ast_log(LOG_WARNING, "EV_FAX (%d): port bridged (%d) must be at the same card (dropping fax_pass_through)\n", p->port, p_bridged->port);
            						}
            						else
            							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            								ast_log(LOG_WARNING, "EV_FAX (%d): %s must has a fax_pass_through enabled (dropping fax_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));
            					}
            					else
            						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            							ast_log(LOG_WARNING, "EV_FAX (%d): %s is not a DGV channel (dropping fax_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));
        	                }
        	                else //if ((dgv_chan_tmp = dgv_bridged_channel(p->ast_ch)) != NULL)
        	                {
        	                	ast_dgv_unlock(&p->lock, p->port);

        	                	if (option_verbose >= 3)
		        					ast_log(LOG_WARNING, "EV_FAX: no one is bridged on port %d (dropping fax_pass_through)\n", p->port);

        	                	break;
        	                } //if ((dgv_chan_tmp = dgv_bridged_channel(p->ast_ch)) != NULL)
						}
						else //if (p->fax_pass_through)
						{
			                switch (p->faxhandled)
		    	            {
		        	            case 0:
			                        /* first change codecs etc */
		    	                    p->faxhandled++;

									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "FAX detected on port %d\n",p->port);

		        	                /* disable echo cancellation when receiving fax */
		            	            if (p->echo_can)
		                	            dg_DisableEchoCancelation(p->port);

#ifdef CC_AST_HAS_VERSION_13
		            	            	ast_format_cap_remove_by_type(dgv_channel_nativeformats(p->ast_ch), AST_MEDIA_TYPE_UNKNOWN);
		            	            	ast_format_cap_append(dgv_channel_nativeformats(p->ast_ch), ast_format_slin, 0);
		            	            	ast_set_read_format(p->ast_ch, ast_format_slin);
		            	            	ast_set_write_format(p->ast_ch, ast_format_slin);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
											struct ast_format ast_ch_fmt;
											ast_format_cap_remove_all(dgv_channel_nativeformats(p->ast_ch));

											ast_format_cap_add(dgv_channel_nativeformats(p->ast_ch), ast_format_set(&ast_ch_fmt, AST_FORMAT_SLINEAR, 0));
											ast_set_read_format_by_id(p->ast_ch, AST_FORMAT_SLINEAR);
											ast_set_write_format_by_id(p->ast_ch, AST_FORMAT_SLINEAR);
	#else//#ifdef CC_AST_HAS_VERSION_10
											p->ast_ch->nativeformats = AST_FORMAT_SLINEAR;

											ast_set_read_format(p->ast_ch, AST_FORMAT_SLINEAR);
											ast_set_write_format(p->ast_ch, AST_FORMAT_SLINEAR);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

		                	        if (option_verbose >= 3)
		                    	    	ast_verbose(VERBOSE_PREFIX_3 "Codec changed(SLINEAR) on port %d\n",p->port);

		    	                    //check if there is a bridged channel with
		        	                if ((dgv_chan_tmp=dgv_bridged_channel(p->ast_ch))!=NULL)
		            	            {
		                	            //tell to bridged channel to stop playbuffer
		                    	        if (dgv_channel_tech_pvt(dgv_chan_tmp)!=NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype,3))
		                        	    {
		                            	    p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);

		                                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                                		ast_log(LOG_NOTICE, "EV_FAX: changing bridged format for %s \n", dgv_channel_name(dgv_chan_tmp));
		
		    	                            dg_DisableEchoCancelation(p_bridged->port);

		    	                            p_bridged->faxhandled++;

		    	                            //change bridged channel
#ifdef CC_AST_HAS_VERSION_13
												ast_format_cap_remove_by_type(dgv_channel_nativeformats(dgv_chan_tmp), AST_MEDIA_TYPE_UNKNOWN);
												ast_format_cap_append(dgv_channel_nativeformats(dgv_chan_tmp), ast_format_slin, 0);
												ast_set_read_format(dgv_chan_tmp, ast_format_slin);
												ast_set_write_format(dgv_chan_tmp, ast_format_slin);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
													struct ast_format ast_ch_fmt;
													ast_format_cap_remove_all(dgv_channel_nativeformats(dgv_chan_tmp));

													ast_format_cap_add(dgv_channel_nativeformats(dgv_chan_tmp), ast_format_set(&ast_ch_fmt, AST_FORMAT_SLINEAR, 0));
													ast_set_read_format_by_id(dgv_chan_tmp, AST_FORMAT_SLINEAR);
													ast_set_write_format_by_id(dgv_chan_tmp, AST_FORMAT_SLINEAR);
	#else//#ifdef CC_AST_HAS_VERSION_10
													dgv_chan_tmp->nativeformats = AST_FORMAT_SLINEAR;

													ast_set_read_format(dgv_chan_tmp, AST_FORMAT_SLINEAR);
													ast_set_write_format(dgv_chan_tmp, AST_FORMAT_SLINEAR);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

		    	                            //re-enable inputbuffer using PCM format
#ifdef CC_AST_HAS_VERSION_13
												dg_SetRecordFormat(p_bridged->port, ast2dgvformat(ast_format_get_codec_id(ast_format_slin)));
#else//#ifdef CC_AST_HAS_VERSION_13
												dg_SetRecordFormat(p_bridged->port, ast2dgvformat(AST_FORMAT_SLINEAR));
#endif//#ifdef CC_AST_HAS_VERSION_13
		    	                        }
		        	                }
		                        	break;
			                    case 1:
		    	                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		        	                    ast_log(LOG_NOTICE, "---> EV_FAX Step 2 port %d\n",p->port);

		    	                    /* first change codecs etc */
		        	                p->faxhandled++;

		    	                    //dg_DisableInputBuffer(p->port);
		        	                //........dg_PauseInputBuffer(p->port, DG_PAUSE);
		            	            dg_StopPlayBuffer(p->port);


		    	                    if (!dgv_channel_is_bridged(p->ast_ch) && strncasecmp(dgv_channel_exten(p->ast_ch), "fax", 3))
		        	                {
			                            snprintf(tmpfax, sizeof(tmpfax), "fax%s", dgv_channel_exten(p->ast_ch));
		    	                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		        	                        ast_verbose(VERBOSE_PREFIX_3 "Checking %s to specific fax extension %s@%s\n", dgv_channel_name(p->ast_ch), tmpfax, dgv_channel_context(p->ast_ch));
#ifdef CC_AST_HAS_VERSION_11
											if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), tmpfax, 1, S_COR(ast_channel_caller(p->ast_ch)->id.number.valid, ast_channel_caller(p->ast_ch)->id.number.str, NULL)))
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
												if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), tmpfax, 1, S_COR(p->ast_ch->caller.id.number.valid, p->ast_ch->caller.id.number.str, NULL)))
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), tmpfax, 1, p->ast_ch->cid.cid_num))
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
		                	            {
		                    	            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                        	            ast_verbose(VERBOSE_PREFIX_3 "Redirecting %s to specific fax extension %s\n", dgv_channel_name(p->ast_ch), tmpfax);
		                            	    /*endif*/
		                                	pbx_builtin_setvar_helper(p->ast_ch, "FAXEXTEN", dgv_channel_exten(p->ast_ch));
			                                if (ast_async_goto(p->ast_ch, dgv_channel_context(p->ast_ch), tmpfax, 1))
		    	                            {
		        	                            ast_log(LOG_WARNING, "Failed to async goto '%s' into '%s' of '%s'\n", dgv_channel_name(p->ast_ch), tmpfax, dgv_channel_context(p->ast_ch));
		            	                        //p->faxhandled=0;
		                	                    //p->ast_ch->nativeformats = p->nativeformats;
		                    	            }
		                        	        /*endif*/
			                            }
		    	                        else
		        	                    {
		            	                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                	                    ast_verbose(VERBOSE_PREFIX_3 "2nd Checking %s to specific fax extension %s@%s\n", dgv_channel_name(p->ast_ch), tmpfax, dgv_channel_context(p->ast_ch));


#ifdef CC_AST_HAS_VERSION_11
												if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), "fax", 1, S_COR(ast_channel_caller(p->ast_ch)->id.number.valid, ast_channel_caller(p->ast_ch)->id.number.str, NULL)))
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
													if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), "fax", 1, S_COR(p->ast_ch->caller.id.number.valid, p->ast_ch->caller.id.number.str, NULL)))
	#else//#ifdef CC_AST_HAS_VERSION_1_8
													if (ast_exists_extension(p->ast_ch, dgv_channel_context(p->ast_ch), "fax", 1, p->ast_ch->cid.cid_num))
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
		                        	        {
		                            	        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                                	        ast_verbose(VERBOSE_PREFIX_3 "Redirecting %s to fax extension\n", dgv_channel_name(p->ast_ch));
		                                    	/*endif*/
			                                    pbx_builtin_setvar_helper(p->ast_ch, "FAXEXTEN", dgv_channel_exten(p->ast_ch));
		    	                                if (ast_async_goto(p->ast_ch, dgv_channel_context(p->ast_ch), "fax", 1))
		        	                            {
		            	                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                	                            ast_log(LOG_WARNING, "Failed to async goto '%s' into fax of '%s'\n", dgv_channel_name(p->ast_ch), dgv_channel_context(p->ast_ch));

		    	                                    //p->faxhandled=0;
		        	                                //p->ast_ch->nativeformats = p->nativeformats;
		            	                        }
		                	                    /*endif*/
		                    	            }
		                        	        else
		                            	    {
		                                	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                                    	    ast_log(LOG_NOTICE, "Fax detected, but no fax extension\n");

		    	                                //p->faxhandled=0;
		        	                            //p->ast_ch->nativeformats = p->nativeformats;
		            	                    }
		                	            }
		                    	        /*endif*/
		                        	}

		    	                    //re-enable inputbuffer using PCM format
#ifdef CC_AST_HAS_VERSION_13
										dg_SetRecordFormat(p->port, ast2dgvformat(ast_format_get_codec_id(ast_format_slin)));
#else//#ifdef CC_AST_HAS_VERSION_13
										dg_SetRecordFormat(p->port, ast2dgvformat(AST_FORMAT_SLINEAR));
#endif//#ifdef CC_AST_HAS_VERSION_13
		    	                    break;
		        	            default:
		            	            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                	            ast_log(LOG_DEBUG, "Already in a fax extension, not redirecting\n");

		            	            //0.9.8 break;
		        	        } //switch (p->faxhandled)
						} //if (p->fax_pass_through)
					}					
					ast_dgv_unlock(&p->lock, p->port);					
				} //if (p->call_state != DG_AST_NONE)				
			}
			else //if (p->fax_detection)			
			{
				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "FAX detected(%d), but the FAX option is disabled\n", p->port);
			} //if (p->fax_detection)			
			break;
	  	case EV_AUDIO_SIGNAL:
			if (((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0) && p->audio_debug)
				ast_log(LOG_NOTICE, "EV_AUDIO_SIGNAL porta %d dado: %x\n",p->port, EventContext->data);
				
			if (p->last_dtmf!=-1 && EventContext->data==0x20) /* silence*/
			{	
			    p->last_dtmf=-1;

                dg_PauseInputBuffer(p->port, DG_RELEASE);
                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
                	ast_log(LOG_WARNING, "EV_AUDIO_SIGNAL: Releasing InputBuffer on port %d\n", p->port);
			}
				
			break;		
		case EV_LINEOFF:
        case EV_BUSY:
        	if (p)
        	{
        		if (EventContext->command == EV_LINEOFF)        			
        		{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_LINEOFF port %d call_state: %x\n",p->port, p->call_state);
        		}
        		else
        		{
        			if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
        			{
        				ast_dgv_lock(&p->lock, p->port);
        				{
        					if (p->ast_ch)
        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_BUSY);
        					else
        					{
        						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_BUSY) on port %d\n", p->port);
        					}
        				}
        				ast_dgv_unlock(&p->lock, p->port);
        			}
        			else
        			{
        				ast_dgv_lock(&p->lock, p->port);
        				{
	        				if (p->need_sc_ring >= 1)
	        				{
	        					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_BUSY: need_sc_ring = 1, dropping BUSY event on port %d\n", p->port);
	        					
	        					ast_dgv_unlock(&p->lock, p->port);
	        					
	        					break; //mainly switch break
	        				}
        				}
        				ast_dgv_unlock(&p->lock, p->port);
        			}
        			
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                		ast_log(LOG_NOTICE, "EV_BUSY port %d call_state: %x\n",p->port, p->call_state);	
        		}

        		if ((p->subchannel) && 
        			(((EventContext->command == EV_LINEOFF) && (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)) ||
        			 ((EventContext->command == EV_BUSY) && (p->sig == DG_SIG_CB_FXS))))
        		{
                	ast_dgv_lock(&p->lock, p->port);
                	{
            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            				ast_log(LOG_NOTICE, "EV_LINEOFF port %d channel_index %d, subchannel_audio_index %d\n", p->port, p->channel_index, p->subchannel_audio_index);

            			if (p->channel_index == 1)
            			{
			 				if (p->subchannel->ast_ch)
			 				{
			 					if (p->ast_ch)
			 					{
#ifdef CC_AST_HAS_VERSION_12
										RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
										struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
			 						int nExchangeCall = 0;
			 						
			 						dg_PauseInputBuffer(p->port, DG_PAUSE);
					                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
					                	ast_log(LOG_WARNING, "EV_LINEOFF: Pausing InputBuffer on port %d\n", p->port);
			 						dg_StopPlayBuffer(p->port);

			 						p->write_last_format=-1;
			 						
        	     					p->reserved=1;
        	     					
		        					switch (p->subchannel_audio_index)
		        					{
		        						case 1:
											dgv_chan_bridged = dgv_bridged_channel(p->subchannel->ast_ch);
		            						if (dgv_chan_bridged)
		            						{
								    			/*
								    			 * (Up) A -> B
								    			 *           C (MOH)
								    			 * B -> HungUp
								    			 */
		            							
		            							if (option_verbose >= 3)
		            								ast_log(LOG_NOTICE, "EV_LINEOFF: Exchanging call on port %d\n", p->port);		            							
		            							
		            							nExchangeCall = 2;
		            						}
		            						else
		            						{
	    						    			/*
	    						    			 * (Up) A -> B
	    						    			 *           C (Ring)
	    						    			 * B -> HungUp
	    						    			 */
		            							
		            							if (option_verbose >= 3)
		            								ast_log(LOG_WARNING, "EV_LINEOFF: dgv_chan_bridged does not exist on port %d\n", p->port);
		
		            							dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
			            						if (dgv_chan_bridged)
			            						{
			            							if (option_verbose >= 3)
			            								ast_log(LOG_NOTICE, "EV_LINEOFF: Exchanging call on port %d\n", p->port);			            							
			            							
			            							nExchangeCall = 2;
			            						}
			            						else
			            						{
			            							if (option_verbose >= 3)
			            								ast_log(LOG_WARNING, "EV_LINEOFF: dgv_chan_bridged does not exist on port %d\n", p->port);
			            						}
		            						}
		            						
		        							break;
		        						case 2:
							    			/*
							    			 * (MOH) A -> B
							    			 *            C (Up)
							    			 * B -> HungUp
							    			 */

		        							dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
		            						if (dgv_chan_bridged)
		            						{
		            							if (option_verbose >= 3)
		            								ast_log(LOG_NOTICE, "EV_LINEOFF: Exchanging call on port %d\n", p->port);		            							
		            							
		            							nExchangeCall = 1;
		            						}
		            						else
		            						{
		            							if (option_verbose >= 3)
		            								ast_log(LOG_WARNING, "EV_LINEOFF: dgv_chan_bridged does not exist on port %d\n", p->port);
		            						}
		
		        							break;
			        				} //switch (p->subchannel_audio_index)		        					

    		            			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    		            				ast_log(LOG_NOTICE, "EV_LINEOFF port %d, nExchangeCall %d, p->need_sc_ring %d\n", p->port, nExchangeCall, p->need_sc_ring);

		        					if (nExchangeCall >= 1)
            						{
            							if (p->sig == DG_SIG_CB_FXS && p->need_sc_ring == 0)
            							{
            								p->need_sc_ring = 1;
            								if (nExchangeCall == 2)
            									p->need_sc_ring++;
            								
            								ast_dgv_unlock(&p->lock, p->port);

            								break; //mainly switch break
            							}
            							else
            								p->need_sc_ring = 0;
            							
            							swap_sc(p, p->subchannel);
            							
            							ast_setstate(p->ast_ch, AST_STATE_RINGING);
            							
            	     					p->call_state = DG_AST_CALLING;
            	     					p->subchannel->call_state = DG_AST_HANGINGUP;            	     					

    									/* dialplan ring_type */
            	     					ast_dgv_unlock(&p->lock, p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_LINEOFF: Before pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

    									if (p->ast_ch)
    										dialplan_ringtype = pbx_builtin_getvar_helper(p->ast_ch, "DGV_RINGTYPE");
										else
											ast_log(LOG_NOTICE, "EV_LINEOFF: p->ast_ch does not exist on port %d\n", p->port);

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_LINEOFF: After pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

            	     					dialplan_ringtype_set = 0;
            	     					ast_dgv_lock(&p->lock, p->port);
    									if (dialplan_ringtype)
    									{
    										dialplan_ringtype_value = atoi(dialplan_ringtype);

    										switch (dialplan_ringtype_value)
    										{
												case 1:
												case 2:
												case 3:
													dg_SetRing(p->port, DG_ENABLE, dialplan_ringtype_value);
	    											dialplan_ringtype_set = 1;

	    											if (option_verbose >= 3)
	    												ast_log(LOG_NOTICE, "Dialplan variable(DGV_RINGTYPE) changed ring_type to %d on port %d\n", dialplan_ringtype_value, p->port);

	    											break;
    										}
    									}
    									if (dialplan_ringtype_set == 0)
    										dg_SetRing(p->port, DG_ENABLE, p->ring_type);

           	     						if (option_verbose >= 3)
           	     							ast_log(LOG_NOTICE, "EV_LINEOFF: Ring started on %d\n", p->port);
            		           			
            		           			if (nExchangeCall == 2)
            		           				swap_sc_tech(p, p->subchannel);

            							p->subchannel_audio_index = 1;
            							p->subchannel->subchannel_audio_index = 1;
            						}
		        					
		                        	if (p->tone_thread_running == 1)
		                        		p->tone_thread_running = 2;

    		                        if (p->stuttertone_thread_running == 1)
    									p->stuttertone_thread_running = 0;		        					
	        					
									struct ast_frame f = { AST_FRAME_CONTROL, };
									f.src = "chan_dgv";
									f.datalen = 0;
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
									QueueFrame(p->subchannel, &f);
										
									ast_dgv_unlock(&p->lock, p->port);
			        					
			            			break; //mainly switch break
                 				}
                 				else
                 				{
            						if (option_verbose >= 3)
            							ast_log(LOG_WARNING, "EV_LINEOFF: p->ast_ch does not exist on port %d\n", p->port);
            					}
             				}
             				else
             				{
        						if (option_verbose >= 3)
        							ast_log(LOG_WARNING, "EV_LINEOFF: p->subchannel->ast_ch does not exist on port %d\n", p->port);
        					}            				
            			} //if (p->channel_index == 1)
					}
					ast_dgv_unlock(&p->lock, p->port);
        		} //if ((p->subchannel) && (EventContext->command == EV_LINEOFF) && (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS))

				if ((EventContext->command == EV_BUSY) ||
				   ((EventContext->command == EV_LINEOFF) && (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)))
				{
					if (p->call_state != DG_AST_NONE)
					{
						struct ast_frame f = { AST_FRAME_CONTROL, };
						f.src = "chan_dgv";
						f.datalen = 0;
					
						if (p->sig == DG_SIG_FXS)
							p->reserved=0;
						
						switch (p->call_state)
						{
        		            case DG_AST_R2_SIGNALLING:
                	        
                    		    p->call_state = DG_AST_HANGINGUP;

								if (p->digit_timeout_sched > -1)
        		                {		
        		                    res_sched = ast_sched_del(sched, p->digit_timeout_sched);
        					    	if (res_sched == -1)
        					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

                		            p->digit_timeout_sched = -1;
                        		}
							
								//Disables Audio
								//dg_DisableInputBuffer(p->port);
		                        dg_PauseInputBuffer(p->port, DG_PAUSE);
				                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
				                	ast_log(LOG_WARNING, "EV_BUSY: Pausing InputBuffer on port %d\n", p->port);

	                        	p->reserved=0;
	                        	/*if (p->sig==DG_SIG_FXO)
		                            p->reserved=0;
		                        else
	    	                        p->reserved=2;      //wait receiving idle*/

								//send hangup								
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

								ast_dgv_lock(&p->lock, p->port); {
									//QueueFrame(EventContext->port, &f);
									QueueFrame(p, &f);
								} ast_dgv_unlock(&p->lock, p->port);
								p->call_state=DG_AST_NONE;
								//******************					
								//VERY IMPORTANT!!!
								//Here it is possible that p->ast_ch is
								//already NULL, then be careful!!!!!!
								//******************
			
								break;
							case DG_AST_CONVERSATION:
		                    case DG_AST_RINGING:
								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_BUSY port %d - Sending AST_CONTROL_HANGUP \n",p->port);										                 		                                                
		
								//Disables Audio
								//dg_DisableInputBuffer(p->port);
		                        dg_PauseInputBuffer(p->port, DG_PAUSE);
				                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
				                	ast_log(LOG_WARNING, "EV_BUSY: Pausing InputBuffer on port %d\n", p->port);
		
				                ast_dgv_lock(&p->lock, p->port);
				                {
				                	p->reserved=0;
		                        
				                	if (p->tone_thread_running == 1)
				                		p->tone_thread_running = 2;
									
				                	if (p->stuttertone_thread_running == 1)
				                		p->stuttertone_thread_running = 0;

				                	if (p->ast_ch)
				                	{
				                		p->call_state = DG_AST_HANGINGUP;

					                	//send hangup
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

										QueueFrame(p, &f);
				                	}
				                	else
				                	{
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_BUSY port %d - 'callstate=9' error avoided!\n", p->port);
				                	}
				                }
				                ast_dgv_unlock(&p->lock, p->port);

								//******************
								//VERY IMPORTANT!!!
								//Here it is possible that p->ast_ch is
								//already NULL, then be careful!!!!!!
								//******************
								break;
		                    case DG_AST_HANGINGUP:
        		                //do nothing
                		        break;
		                    case DG_AST_DIALING://only to fxs cards
		                    	if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
		                    	{		                    	
			                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		               		        	ast_log(LOG_NOTICE, "EV_LINEOFF port %d - Sending AST_CONTROL_HANGUP \n", p->port);

				                	ast_dgv_lock(&p->lock, p->port);
				                	{
				                		if (p->tone_thread_running == 1)
				                			p->tone_thread_running = 2;
				                	}
				                	ast_dgv_unlock(&p->lock, p->port);
	               		        	
		                        	if (p->stuttertone_thread_running == 1)
										p->stuttertone_thread_running = 0;	               		        	
	               		        	
									//send hangup
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

									ast_dgv_lock(&p->lock, p->port); 
									{
			               				if (p->ast_ch) 
			               					dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_UNALLOCATED);
			               				else
			               				{
			               					p->call_state = DG_AST_NONE;
			               					ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_LINEOFF) on port %d\n", p->port);
			               				}
			               				
										//QueueFrame(EventContext->port, &f);
										QueueFrame(p, &f);

										if ((!p->fxss_thread_running) && (p->ast_ch))
										{
											if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_LINEOFF port %d - Sending ast_hangup \n", p->port);
		               		        		
											ast_hangup(p->ast_ch);
										}
										else
											if (p->fxss_thread_running)
											{
												if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
													ast_log(LOG_NOTICE, "EV_LINEOFF port %d - fxss_thread still running\n", p->port);
											}
									} 
									ast_dgv_unlock(&p->lock, p->port);

	        		    			p->exten[0] = '\0';       		    			
	        		    			break;
		                    	}           			    	        			
							default:
								//send specific subclass
								if (p->sig == DG_SIG_GSM)
								{
									switch (EventContext->data)
									{
										case 404://NO CARRIER
										case 503://NO DIALTONE
											if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_BUSY port %d - Sending AST_CONTROL_CONGESTION\n", p->port);

											ast_dgv_lock(&p->lock, p->port);
					        				{
					        					if (p->ast_ch)
					        					{
					        						if (EventContext->data == 404)
					        						{
					        							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					        								ast_log(LOG_NOTICE, "EV_BUSY port %d - Setting cause AST_CAUSE_UNALLOCATED\n", p->port);

					        							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_UNALLOCATED);
					        						}
					        						else
					        						{
														if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
															ast_log(LOG_NOTICE, "EV_BUSY port %d - Setting cause AST_CAUSE_CONGESTION\n", p->port);

						        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
					        						}
					        					}
					        					else
					        					{
					        						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					        							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_BUSY) on port %d\n", p->port);
					        					}
					        				}
					        				ast_dgv_unlock(&p->lock, p->port);

#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_CONGESTION;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_CONGESTION;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

											if (p->call_state==DG_AST_CALLING || p->call_state==DG_AST_DIALING)
											{
												dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTotalDialError++;
												if  (dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError == 0)
												{
													dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError++;
													if (dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError >= max_ndt_count)
													{
														dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError = 0;
														dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError = 1;//On

														if (option_verbose >= 3)
															ast_log(LOG_NOTICE, "We tried to Dial more than %d times, setting Dial Error Flag on port %d\n", max_ndt_count, p->port);
													}
												}
											}

											break;
										case 486://BUSY
											if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_BUSY port %d - Sending AST_STATE_BUSY\n", p->port);

											ast_dgv_lock(&p->lock, p->port);
					        				{
					        					if (p->ast_ch)
					        					{
													if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
														ast_log(LOG_NOTICE, "EV_BUSY port %d - Setting cause AST_CAUSE_USER_BUSY\n", p->port);

					        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_USER_BUSY);
					        					}
					        					else
					        					{
					        						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					        							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_BUSY) on port %d\n", p->port);
					        					}
					        				}
					        				ast_dgv_unlock(&p->lock, p->port);

#ifdef CC_AST_HAS_VERSION_1_8
												f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
												f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

												dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError = 0;
												dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError = 0;//Off

												if (option_verbose >= 3)
													ast_log(LOG_NOTICE, "Busy detected, resetting Dial Error Flag on port %d\n", p->port);
											break;
										default:
											if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_BUSY port %d - Sending AST_CONTROL_CONGESTION\n", p->port);

											ast_dgv_lock(&p->lock, p->port);
					        				{
					        					if (p->ast_ch)
					        					{
													if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
														ast_log(LOG_NOTICE, "EV_BUSY port %d - Setting cause AST_CAUSE_CONGESTION\n", p->port);

					        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
					        					}
					        					else
					        					{
					        						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					        							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_BUSY) on port %d\n", p->port);
					        					}
					        				}
					        				ast_dgv_unlock(&p->lock, p->port);

#ifdef CC_AST_HAS_VERSION_1_8
												f.subclass.integer = AST_CONTROL_CONGESTION;
#else//#ifdef CC_AST_HAS_VERSION_1_8
												f.subclass = AST_CONTROL_CONGESTION;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

												dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError = 0;
												dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError = 0;//Off

												if (option_verbose >= 3)
													ast_log(LOG_NOTICE, "Busy detected, resetting Dial Error Flag on port %d\n", p->port);
											break;
									}
								}
								else
								{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_BUSY port %d - Sending AST_STATE_BUSY\n", p->port);

#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								}

								ast_dgv_lock(&p->lock, p->port); {
									//QueueFrame(EventContext->port, &f);
									QueueFrame(p, &f);
								} ast_dgv_unlock(&p->lock, p->port);
						
								//******************					
								//VERY IMPORTANT!!!
								//Here it is possible that p->ast_ch is
								//already NULL, then be careful!!!!!!
								//0.9.8 break;
						} //switch(p->call_state)
					}
					else //if (p->call_state != DG_AST_NONE)
					{
						if (p)
						{
							if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
							{
								//p->reserved=0; /* removed on 1.0.5_rc2 */
				                if (p->reserved==2)
				                {
				                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				                        ast_log(LOG_NOTICE, "EV_BUSY (%d) - Changing reserved to 0, because we received an EV_BUSY\n", p->port);
				                    p->reserved = 0;
				                }

								ast_dgv_lock(&p->lock, p->port);
								{
									if ((p->tone_thread_running == 1) || (p->stuttertone_thread_running == 1))
									{
										if (p->tone_thread_running == 1)
											p->tone_thread_running = 2;
	
				                        if (p->stuttertone_thread_running == 1)
											p->stuttertone_thread_running = 0;
										//p->ast_ch = NULL;									
			                        } //if ((p->tone_thread_running == 1) || (p->stuttertone_thread_running == 1))
								}
								ast_dgv_unlock(&p->lock, p->port);
							}
							else //if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
							{
								ast_dgv_lock(&p->lock, p->port);
								{
									if ((p->sig == DG_SIG_R2_NDIS_FIXED || p->sig == DG_SIG_R2_NDIS_VARIABLE) && (p->ast_ch == NULL))
									{
										switch (p->reserved)
										{
											case 1:
												if (p->last_r2_signal==R2_IDLE)
												{
													p->reserved=0;
													if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
														ast_log(LOG_NOTICE, "EV_BUSY (%d): R2_IDLE already received, changing reserved to 0.\n", p->port);
												}
												break;
											case 2:
												p->reserved=0;
												if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
													ast_log(LOG_NOTICE, "EV_BUSY: Changing reserved to 0 after dgv_hangup port %d \n", p->port);

												if (p->reserved_timeout_sched > -1)//stop reserved timeout
												{
													if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
														ast_log(LOG_NOTICE, "EV_BUSY (%d) - deleting reserved schedule\n", p->port);

													res_sched = ast_sched_del(sched, p->reserved_timeout_sched);
											    	if (res_sched == -1)
											    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

													p->reserved_timeout_sched = -1;
												}
												break;
										} //switch (p->reserved)
									}
									else
									{
										if (p->sig == DG_SIG_GSM)
										{
											dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nTempDialError = 0;
											dgv_gsmDECount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1].nFlagDialError = 0;//Off

											if (option_verbose >= 3)
												ast_log(LOG_NOTICE, "Busy detected, resetting Dial Error Flag on port %d\n", p->port);
										}
									}
								}
								ast_dgv_unlock(&p->lock, p->port);
							} //if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
						} //if (p)
					} //if (p->call_state != DG_AST_NONE)
				} //if ((EventContext->command == EV_BUSY) || ...

				if ((p->sig == DG_SIG_GSM) && (EventContext->command == EV_LINEOFF))
				{
					//use the same schedule to reload the channel...
					ast_dgv_lock(&p->lock, p->port);
					{
						if (p->gsm_need_reload)
						{
							p->gsm_need_reload = 0;

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "Starting scheduled reload on port %d\n", p->port);

							if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
							{
								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_LINEOFF (%d) - deleting gsmerror schedule\n", p->port);

								res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
						    	if (res_sched == -1)
						    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

								p->gsmerror_timeout_sched = -1;
							}

							p->alarmed=99;

							//set gsmerror timeout
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_NOTICE, "EV_LINEOFF (%d) - Creating gsmerror schedule\n", p->port);

							p->gsmerror_timeout_sched = ast_sched_add(sched, 2000, gsmerror_timeout_callback, p);

							if (p->gsmclearsms_timeout_sched > -1)
							{
								res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
						    	if (res_sched == -1)
						    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

								p->gsmclearsms_timeout_sched = -1;
							}
						}
					}
					ast_dgv_unlock(&p->lock, p->port);
				} //if ((p->sig == DG_SIG_GSM) && (EventContext->command == EV_LINEOFF))

				if ((p->sig == DG_SIG_FXO) && (EventContext->command == EV_LINEOFF) && (p->call_state == DG_AST_INCOMING_CALL) &&
					((dg_GetCardType(dg_GetCardNumber(p->port)) == VB0408PCIE)))
				{
					if (p->flashing == 1)
					{
						p->flashing = 2;

	           			if (option_verbose >= 3)
		    	           	ast_log(LOG_NOTICE, "EV_LINEOFF/EV_AFTERPICKUP: Calling CallBlock application on %d \n", p->port);

	           			dg_PauseInputBuffer(p->port, DG_RELEASE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "EV_LINEOFF/EV_AFTERPICKUP: Releasing InputBuffer on port %d\n", p->port);

						dg_Flash(p->port, 1, 1500, 100);
						break;
					}

        	    	switch(p->sig)
            	    {
                		case DG_SIG_FXO:
			    		    if (p->ring_timeout_sched > -1)//stop ring timeout
	    	    		    {
    		    		    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	                		ast_log(LOG_NOTICE, "EV_LINEOFF/EV_AFTERPICKUP port %d - deleting ring schedule \n", p->port);

        		        	    res_sched = ast_sched_del(sched, p->ring_timeout_sched);
    					    	if (res_sched == -1)
    					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            		            p->ring_timeout_sched = -1;
                		    }

	                    	break;
        		   	}

        	    	ast_dgv_lock(&p->lock, p->port);
        	    	{
        	    		if (p->ast_ch)
        	    			ast_setstate(p->ast_ch, AST_STATE_UP);
        	    		else
        	    			ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_LINEOFF/EV_AFTERPICKUP) on port %d\n", p->port);
        	    	}
        	    	ast_dgv_unlock(&p->lock, p->port);

					dg_SetDetectionType(p->port, dgv_conf.default_detectiontype, DG_ENABLE);

					//enable inputbuffer streaming with agc disabled
					if (p->flashing != 2)
					{
						dg_PauseInputBuffer(p->port, DG_RELEASE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "EV_LINEOFF/EV_AFTERPICKUP: Releasing InputBuffer on port %d\n", p->port);
					}
					else
	    		    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                		ast_log(LOG_NOTICE, "EV_LINEOFF/EV_AFTERPICKUP port %d - InputBuffer already released \n", p->port);

					pthread_attr_init(&attr);
					pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    	        	if (ast_pthread_create(&threadid, &attr, dialplan_echo_thread, p)>=0)
    		        {
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "dialplan_echo_thread Created(2) - port %d \n", p->port);
	            	}
        	        else
            	    	ast_log(LOG_ERROR, "Error creating dialplan_echo_thread - port %d \n", p->port);
            	   	pthread_attr_destroy(&attr);

					p->call_state = DG_AST_CONVERSATION;
				} //if ((p->sig == DG_SIG_FXO) && (EventContext->command == EV_LINEOFF) && (p->call_state == DG_AST_INCOMING_CALL) &&
        	} //if (p)
			break;
		case EV_DIALTONE:
			//dial tone was received, then dials
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "EV_DIALTONE port %d\n",p->port);

			if (p)
			{
			    dg_DisableCallProgress(p->port);
			    
                switch (p->sig)
                {
                    case DG_SIG_FXO:
                    case DG_SIG_CB_FXO:                        	
                    	dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1] = 0;
                    	p->alarmed = 0;
                    	
    					if (p->alarmNDT_timeout_sched > -1) 
    					{
    						res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

    						p->alarmNDT_timeout_sched = -1;
    					}
                    	break;
                }
			    
				if (p->call_state == DG_AST_WAITFORDIALTONE)
				{
                    ast_dgv_lock(&p->lock, p->port);
                    {
	                    if (p->linetone_timeout_sched > -1) //stop line tone timeout
    	                {
        	                res_sched = ast_sched_del(sched, p->linetone_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            	            p->linetone_timeout_sched = -1;
                	    }

    					if (!ast_strlen_zero(p->dialplan_senddtmf))				
    					{
    						p->call_state = DG_AST_CALLING;
    						
        					if (p->senddtmf_dialtone_timeout_sched > -1) 
        					{
        						res_sched = ast_sched_del(sched, p->senddtmf_dialtone_timeout_sched);
    					    	if (res_sched == -1)
    					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

        						p->senddtmf_dialtone_timeout_sched = -1;
        					}

        		   			dg_Dial(p->port, p->dialplan_senddtmf, 20, dtDirectMF);
				    		
        		   			if (option_verbose >= 3)
        		   	        	ast_log(LOG_NOTICE, "Dialing using port %d to %s\n", p->port, p->dialplan_senddtmf);
        		   			
        		   			ast_dgv_unlock(&p->lock, p->port);
        		   			
        		   			break; //mainly switch break
    					}
    		        }
                    ast_dgv_unlock(&p->lock, p->port); 

                    start_dial(p);                    
				}
                else
                {
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                        ast_log(LOG_NOTICE, "EV_DIALTONE port %d - another state=%x\n",p->port, p->call_state);
                }
			}
			break;
        case EV_AFTERPICKUP:
        case EV_AFTERFLASH:
#ifdef DEBUG_PRI        	
        		ast_log(LOG_NOTICE, "EV_AFTERPICKUP: STEP 1...\n");
#endif//#ifdef DEBUG_PRI
        	
        	if (EventContext->command == EV_AFTERPICKUP)
        	{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_AFTERPICKUP port %d \n", p->port);
        	}		
        	else
        	{	
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
               		ast_log(LOG_NOTICE, "EV_AFTERFLASH port %d \n", p->port);	
        	}
        
            if ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && (EventContext->command == EV_AFTERPICKUP))
            {
				switch (p->call_state)
				{
					case DG_AST_NONE:			
				        if (p->immediate)
						{    
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        						ast_log(LOG_NOTICE, "EV_AFTERPICKUP (%d): Starting Immediate. \n", p->port);
        			  						
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "Going to immediate on extension s|1 \n");						                
						    	            
							//fxs immediate
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
								if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    	    	    		{
								pthread_attr_init(&attr);
								pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	            				if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
    	            			{
        	            			p->call_state = DG_AST_DIALING;
    								if (option_verbose >= 3)
                	        			ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
		                		}
	    		        		else                                
    	    		    			ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
								pthread_attr_destroy(&attr);
    	    	    		}
							else
    	    					ast_log(LOG_ERROR,"Error creating dgv_new to fxs immediate - port %d \n", p->port);
						}
						else// not p->immediate
						{				
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        		  				ast_log(LOG_NOTICE, "EV_AFTERPICKUP (%d): Starting DIAL_TONE. \n", p->port);
		        		  		
		        		  	if (ast_app_has_voicemail(p->mailbox, NULL))
        				  	{
								p->stuttertone_thread_running = 1;

        		    			pthread_attr_init(&attr);	
                				pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);                
                				if (pthread_create(&p->stuttertone_thread_id, &attr, StutterToneThread, &(p->port)) < 0)
									p->stuttertone_thread_running = 0;
               					else
                   					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                       					ast_log(LOG_NOTICE, "EV_AFTERPICKUP: stutter dial tone thread created port %d\n",p->port);                       			
		               			pthread_attr_destroy(&attr);        		  		
        				  	}        		  		
        				  	else
        				  	{
        				  		dg_GenerateMF(p->port, GENERATE_TONE1, 0);
			        			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_AFTERPICKUP GENERATE_TONE1 on %d \n", p->port);			        			
        				  	}
			        	
							dg_SetDetectionType(p->port, DETECT_DTMF, DG_ENABLE);
        
							//fxs overlapdial
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
								if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if (dgv_new(p,AST_STATE_RESERVED, p->context, DONT_START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
        					{
								pthread_attr_init(&attr);
								pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
								if (ast_pthread_create(&threadid, &attr, fxss_thread, p)>=0) 
            					{
            						p->call_state = DG_AST_DIALING;
    								if (option_verbose >= 3)
                						ast_log(LOG_NOTICE,"fxss_thread Created - port %d \n", p->port);                                	                                    	
								}
								else                                
        							ast_log(LOG_ERROR, "Error creating fxss_thread - port %d \n", p->port);
								pthread_attr_destroy(&attr);
							}
							else
	    					{
	    						ast_log(LOG_ERROR,"Error creating dgv_new to fxs overlapdial - port %d \n", p->port);
							}
						}
						break;
					case DG_AST_CALLING:
						if (p->sig == DG_SIG_FXS)
						{
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	        		  			ast_log(LOG_NOTICE, "EV_AFTERPICKUP (%d): ast_setstate(AST_STATE_UP). \n", p->port);

							p->call_state = DG_AST_CONVERSATION;
							
							struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
							f.src = "chan_dgv";
							f.datalen = 0;
							ast_dgv_lock(&p->lock, p->port);
							{
								QueueFrame(p, &f);
								
								if (p->ast_ch)
									ast_setstate(p->ast_ch, AST_STATE_UP);
								else
									ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERPICKUP) on port %d\n", p->port);
							}
							ast_dgv_unlock(&p->lock, p->port);
						}//if (p->sig == DG_SIG_FXS)
						else						
						{
							ast_dgv_lock(&p->lock, p->port);
							{
								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								{
									ast_log(LOG_NOTICE, "EV_AFTERPICKUP: p->szPortID = %s \n", p->szPortID);
									if (p->ast_ch)
#ifdef CC_AST_HAS_VERSION_11
											connected_id = dgv_channel_connected_effective_id(p->ast_ch);
											ast_log(LOG_NOTICE, "EV_AFTERPICKUP: connected_id.number.str = %s \n", (connected_id.number.valid) ? connected_id.number.str : "NULL");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
												ast_log(LOG_NOTICE, "EV_AFTERPICKUP: p->ast_ch->connected.id.number.str = %s \n", S_COR(p->ast_ch->connected.id.number.valid, p->ast_ch->connected.id.number.str, NULL));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
												ast_log(LOG_NOTICE, "EV_AFTERPICKUP: p->ast_ch->cid.cid_num = %s \n", p->ast_ch->cid.cid_num);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
									ast_log(LOG_NOTICE, "EV_AFTERPICKUP: p->cid_num = %s \n", p->cid_num);
								}
							
#ifdef CC_AST_HAS_VERSION_11
		        					if (p->ast_ch)
		        					{
		        						connected_id = dgv_channel_connected_effective_id(p->ast_ch);

		        						if ((connected_id.number.valid) &&
										    (!ast_strlen_zero(connected_id.number.str)) &&
										    (ast_isphonenumber(connected_id.number.str)))
#else//#ifdef CC_AST_HAS_VERSION_11
		        					if ((p->ast_ch) &&
	#ifdef CC_AST_HAS_VERSION_1_8
		        							(p->ast_ch->connected.id.number.valid) &&
		        							(!ast_strlen_zero(p->ast_ch->connected.id.number.str)) &&
		        							(ast_isphonenumber(p->ast_ch->connected.id.number.str)))
	#else//#ifdef CC_AST_HAS_VERSION_1_8
											(!ast_strlen_zero(p->cid_num)) &&
											(ast_isphonenumber(p->cid_num)))
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
		                    	{
		        					char sCIDToCBFXS[AST_MAX_EXTENSION] = "";
	        					
		                    		sCIDToCBFXS[0] = 'A';
		                    		sCIDToCBFXS[1] = '1';
#ifdef CC_AST_HAS_VERSION_11
										strncpy(&sCIDToCBFXS[2], connected_id.number.str, strlen(connected_id.number.str));
										sCIDToCBFXS[strlen(sCIDToCBFXS)] = 'C';
										sCIDToCBFXS[strlen(connected_id.number.str) + 3] = '\0';
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
											strncpy(&sCIDToCBFXS[2], p->ast_ch->connected.id.number.str, strlen(p->ast_ch->connected.id.number.str));
											sCIDToCBFXS[strlen(sCIDToCBFXS)] = 'C';
											sCIDToCBFXS[strlen(p->ast_ch->connected.id.number.str) + 3] = '\0';
	#else//#ifdef CC_AST_HAS_VERSION_1_8
											strncpy(&sCIDToCBFXS[2], p->cid_num, strlen(p->cid_num));
											sCIDToCBFXS[strlen(sCIDToCBFXS)] = 'C';
											sCIDToCBFXS[strlen(p->cid_num) + 3] = '\0';
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
									dg_Dial(p->port, sCIDToCBFXS, 20, dtDirectMF);
	
		            				if (option_verbose >= 3)
		                				ast_log(LOG_NOTICE, "EV_AFTERPICKUP: Sending CallerID(%s) on ChannelBank FXS port %d \n", sCIDToCBFXS, p->port);
		                    	}
		                    	else
		                    	{
		                    		if (p->need_sc_ring == 0)
		                    		{
										struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										f.src = "chan_dgv";
										f.datalen = 0;
										QueueFrame(p, &f);
		                    		}
		                    		else
		                    			p->need_sc_ring = 0;

		            				if (option_verbose >= 3)
		                				ast_log(LOG_NOTICE, "EV_AFTERPICKUP: Ring started on %d \n", p->port);
		                    	}
#ifdef CC_AST_HAS_VERSION_11
		        					}
#endif//#ifdef CC_AST_HAS_VERSION_11
							}
							ast_dgv_unlock(&p->lock, p->port);
						}//if (p->sig == DG_SIG_FXS)
			
						break;	
				}
			}
			else//if ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && (EventContext->command == EV_AFTERPICKUP))
			{
				//AFTERPICKUP or AFTERFLASH is used only to incoming calls	
				if (p->call_state == DG_AST_INCOMING_CALL)
				{
					if (p->flashing == 1)
					{
						p->flashing = 2;

	           			if (option_verbose >= 3)
		    	           	ast_log(LOG_NOTICE, "EV_AFTERPICKUP: Calling CallBlock application on %d \n", p->port);

	           			dg_PauseInputBuffer(p->port, DG_RELEASE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "EV_AFTERPICKUP: Releasing InputBuffer on port %d\n", p->port);
	           			
						dg_Flash(p->port, 1, 1500, 100);
						break;
					}

        	    	switch(p->sig)
            	    {
                		case DG_SIG_FXO:
                		//case DG_SIG_CB_FXO:	
                		case DG_SIG_GSM: 
			    		    if (p->ring_timeout_sched > -1)//stop ring timeout
	    	    		    {
    		    		    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
        	                		ast_log(LOG_NOTICE, "EV_AFTERPICKUP port %d - deleting ring schedule \n", p->port);

        		        	    res_sched = ast_sched_del(sched, p->ring_timeout_sched);
    					    	if (res_sched == -1)
    					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            		            p->ring_timeout_sched = -1;
                		    }

	                    	break;
	                    //0.9.8 default:
	                    	//0.9.8 break;
        		   	}
				
        	    	ast_dgv_lock(&p->lock, p->port);
        	    	{
        	    		if (p->ast_ch)
        	    			ast_setstate(p->ast_ch, AST_STATE_UP);
        	    		else
        	    			ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERPICKUP) on port %d\n", p->port);
        	    	}
        	    	ast_dgv_unlock(&p->lock, p->port);
        	    	
					//call progress to detect busy, etc
					/* todo: configuration to this */
					dg_SetDetectionType(p->port, dgv_conf.default_detectiontype, DG_ENABLE);
			
					//enable inputbuffer streaming with agc disabled
					if (p->flashing != 2)
					{
						dg_PauseInputBuffer(p->port, DG_RELEASE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "EV_AFTERPICKUP: Releasing InputBuffer on port %d\n", p->port);
					}
					else
	    		    	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                		ast_log(LOG_NOTICE, "EV_AFTERPICKUP port %d - InputBuffer already released \n", p->port);							

                	/*
                	ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
                	if (ret != DG_EXIT_SUCCESS)
	                    ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n",p->port,ret);
    	            else
        	            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            	            ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n",p->port);
                	*/
                	
					pthread_attr_init(&attr);
					pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    	        	if (ast_pthread_create(&threadid, &attr, dialplan_echo_thread, p)>=0)
    		        {
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "dialplan_echo_thread Created(1) - port %d \n", p->port);
	            	}
        	        else
            	    	ast_log(LOG_ERROR, "Error creating dialplan_echo_thread - port %d \n", p->port);
            	   	pthread_attr_destroy(&attr);

    				if (p->sig == DG_SIG_GSM)
    				{
    					ast_dgv_lock(&p->lock, p->port);
    						if (p->gsm_need_reload)
    						{
    							p->gsm_need_reload = 0;

    							ast_dgv_unlock(&p->lock, p->port);

    							if (option_verbose >= 3)
    								ast_log(LOG_NOTICE, "Starting scheduled reload on port %d\n", p->port);

    							set_gsm_conf(p);
    						}
    						else
    							ast_dgv_unlock(&p->lock, p->port);
    				} //if (p->sig == DG_SIG_GSM)

					p->call_state = DG_AST_CONVERSATION;

					if (p->sig == DG_SIG_R2_NDIS_FIXED && p->ccb_pass_through && p->flashing == 0)
		            {
		                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                    ast_log(LOG_NOTICE, "EV_AFTERPICKUP (%d) - Using ccb_pass_through...\n", p->port);

		    			ast_dgv_lock(&p->lock, p->port);
		    			{
							//check if there is a bridged channel with
							if (!p->ast_ch)
							{
								ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERPICKUP) on port %d(dropping ccb_pass_through)\n", p->port);
								ast_dgv_unlock(&p->lock, p->port);
								break;
							}

		        			if ((dgv_chan_tmp = dgv_bridged_channel(p->ast_ch)) != NULL)
		    	            {
		            	        if (dgv_channel_tech_pvt(dgv_chan_tmp) != NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype, 3))
		                	    {
		                    	    p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);

		                    	    if (p_bridged->sig == DG_SIG_R2_NDIS_VARIABLE && p_bridged->call_state == DG_AST_CONVERSATION && p_bridged->ccb_pass_through)
		                    	    {
		                    	    	if (p_bridged->flashing > 0)
		                    	    	{
		                    	    		p_bridged->flashing = 0;
	                						p->flashing = 2;

		                	           		if (option_verbose >= 3)
		                	           			ast_log(LOG_NOTICE, "EV_AFTERPICKUP: ccb_pass_through received, calling CallBlock application on %d \n", p->port);

											dg_Flash(p->port, 1, 1500, 100);
		                    	    	}
		                    	    	else //if (p_bridged->flashing > 0)
		                    	    	{
		                    	    		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_NOTICE, "EV_AFTERPICKUP (%d) - ccb not detected...\n", p->port);

											pthread_attr_init(&attr);
											pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
											if (ast_pthread_create(&threadid, &attr, ccb_thread, p)>=0)
											{
												if (option_verbose >= 3)
													ast_log(LOG_NOTICE, "ccb_thread Created - port %d\n", p->port);
											}
											else
												ast_log(LOG_ERROR, "Error creating ccb_thread - port %d\n", p->port);
											pthread_attr_destroy(&attr);
		                    	    	} //if (p_bridged->flashing > 0)
		                        	}
		                        	else
		                        	{
		                        		ast_log(LOG_WARNING, "EV_AFTERPICKUP (%d): %s must be Variable and under Conversation (dropping ccb_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));
		                        		if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		                        			ast_log(LOG_WARNING, "EV_AFTERPICKUP (%d): p_bridged->sig %s, p_bridged->call_state %d, p_bridged->ccb_pass_through %d\n", p->port, sig2str(p_bridged->sig), p_bridged->call_state, p_bridged->ccb_pass_through);
		                        	}
		                        }
		            	        else
		            	        	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		            	        		ast_log(LOG_WARNING, "EV_AFTERPICKUP (%d): %s is not a DGV channel (dropping ccb_pass_through)\n", p->port, dgv_channel_name(dgv_chan_tmp));
			                }
		        			else
		        				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
		        					ast_log(LOG_WARNING, "EV_AFTERPICKUP: no one is bridged on port %d (dropping ccb_pass_through)\n", p->port);
		    			}
		    			ast_dgv_unlock(&p->lock, p->port);
		            } //if (p->sig == DG_SIG_R2_NDIS_FIXED && p->ccb_pass_through && p->flashing == 0)
				} //if (p->call_state == DG_AST_INCOMING_CALL)
			} //if ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && (EventContext->command == EV_AFTERPICKUP))
			break;
		case EV_GROUP_B:
			if (p)
			{
				p->group_b_received = EventContext->data;
				
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GROUP_B port %d, group_b_received %d \n", EventContext->port, p->group_b_received);
				
		        //Cause
				ast_dgv_lock(&p->lock, p->port);
				{
			        if (p->ast_ch)
			        {
				        if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
				        {
				        	if (p->group_b_received > 0)
				        	{
				        		sprintf(szGroupBReceived, "%d", p->group_b_received);
				    			pbx_builtin_setvar_helper(p->ast_ch, "DGV_GROUP_B_RECEIVED", szGroupBReceived);

				    			if (option_verbose >= 3)
				    				ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): Group B Received - '%s', use DGV_GROUP_B_RECEIVED variable to read it!\n", p->port, szGroupBReceived);
				        	}

	    					p_e1_thread_structure = dg_GetE1ThreadPointer(p->port);

	    					if (p_e1_thread_structure)
	    					{
	    						if ( (p->group_b_received == p_e1_thread_structure->b_free_calling       ) ||
	    						 	 (p->group_b_received == p_e1_thread_structure->b_free_withoutbilling) ||
	    							 (p->group_b_received == p_e1_thread_structure->b_collectcall        ) )
	    							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_NORMAL);
	    						else if (p->group_b_received == p_e1_thread_structure->b_busy)
	    							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_BUSY);
	    						else if (p->group_b_received == p_e1_thread_structure->b_congestion)
	    							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
	    						else if ( (p->group_b_received == p_e1_thread_structure->b_number_unknown) ||
		    						 	  (p->group_b_received == p_e1_thread_structure->b_number_changed) ||
		    							  (p->group_b_received == p_e1_thread_structure->b_out_of_service) )
	    							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_UNALLOCATED);
	    						else
	    							dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_FAILURE);
							} //if (p_e1_thread_structure)
	    					else
	    						ast_log(LOG_WARNING, "event_handler: p_e1_thread_structure does not exist(EV_GROUP_B) on port %d\n", p->port);				        	

			    	        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
			        	        ast_log(LOG_NOTICE, "EV_GROUP_B: Setting cause (%d), on port %d \n", dgv_channel_hangupcause(p->ast_ch), p->port);

		    	        	if (p->group_b_port_bridged > 0 && p->group_b_port_bridged <= dg_GetPortsCount())
		    	        	{
		    				    ret = -1;

		    	        		p_bridged = find_ch_port(p->group_b_port_bridged);

		    	        		if (p_bridged)
		    	        			if (p_bridged->ast_ch)
		    	        				if (p_bridged->sig == DG_SIG_R2_NDIS_VARIABLE)
		    	        					if (p_bridged->call_state == DG_AST_R2_SIGNALLING || p_bridged->call_state == DG_AST_RINGING)
		    	        						if (p_bridged->wait_group_b)
		    	        							if (p->group_b_received >= 1 && p->group_b_received <= 8)
		    	        							{
		    	        								dg_R2SendGroupB(p_bridged->port, p->group_b_received);

		    	        								if (option_verbose >= 3)
		    	        									ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): Group B Received, sending R2 Group B '%d' to bridged port %d\n", p->port, p->group_b_received, p_bridged->port);

		    	        								ret = 0;
		    	        							}

		    	        	    if (ret != 0)
		    	        			if (option_verbose >= 3)
		    	        			    ast_log(LOG_NOTICE, "EV_GROUP_B did not send R2 Group B...\n");
		    	        	} //if (p->group_b_port_bridged > 0 && p->group_b_port_bridged <= dg_GetPortsCount())
				        } //if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
			        } //if (p->ast_ch)
			        else
			        	ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_GROUP_B) on port %d\n", p->port);			        	
				}
				ast_dgv_unlock(&p->lock, p->port);
			}
			break;
		case EV_AFTERDIAL:
#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "EV_AFTERDIAL: STEP 1...\n");
#endif//#ifdef DEBUG_PRI

			if (p)
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_AFTERDIAL %d, call_state %d\n", EventContext->port, p->call_state);

    			if (p->call_state == DG_AST_CALLING)
    			{
    				switch (p->sig)
    				{
    					case DG_SIG_FXS:
							/* dialplan ring_type */
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_NOTICE, "EV_AFTERDIAL: Before pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

							if (p->ast_ch)
								dialplan_ringtype = pbx_builtin_getvar_helper(p->ast_ch, "DGV_RINGTYPE");
							else
								ast_log(LOG_NOTICE, "EV_AFTERDIAL: p->ast_ch does not exist on port %d\n", p->port);

							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_NOTICE, "EV_AFTERDIAL: After pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

	     					dialplan_ringtype_set = 0;
							if (dialplan_ringtype)
							{
								dialplan_ringtype_value = atoi(dialplan_ringtype);

								switch (dialplan_ringtype_value)
								{
									case 1:
									case 2:
									case 3:
										dg_SetRing(p->port, DG_ENABLE, dialplan_ringtype_value);
										dialplan_ringtype_set = 1;

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "Dialplan variable(DGV_RINGTYPE) changed ring_type to %d on port %d\n", dialplan_ringtype_value, p->port);

										break;
								}
							}
							if (dialplan_ringtype_set == 0)
								dg_SetRing(p->port, DG_ENABLE, p->ring_type);
    					case DG_SIG_CB_FXS:
    	           			if (option_verbose >= 3)        				
        	           			ast_log(LOG_NOTICE, "EV_AFTERDIAL: Ring started on %d \n", p->port);
    	           			
    						struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
								f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
    						f.src = "chan_dgv";
    						f.datalen = 0;
    						ast_dgv_lock(&p->lock, p->port); 
    						{
    							QueueFrame(p, &f);
    						}
    						ast_dgv_unlock(&p->lock, p->port);
    						break;
    				} //switch (p->sig)
    			} //if (p->call_state == DG_AST_CALLING)

    			if (p->call_state==DG_AST_DIALING)
				{
    				ast_dgv_lock(&p->lock, p->port);
					{
						if (!p->ast_ch)
						{
							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);
							ast_dgv_unlock(&p->lock, p->port);
							break;
						}

#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
							switch (p->sig)
							{
								case DG_SIG_R2_NDIS_FIXED:
								case DG_SIG_R2_NDIS_VARIABLE:
								case DG_SIG_FXO:
								case DG_SIG_CB_FXO:
								case DG_SIG_GSM:
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_AFTERDIAL %d, sending AST_CONTROL_PROGRESS\n", EventContext->port);

									struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass.integer = AST_CONTROL_PROGRESS;
#else//#ifdef CC_AST_HAS_VERSION_1_8
										f.subclass = AST_CONTROL_PROGRESS;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
									f.src = "chan_dgv";
									f.datalen = 0;
									QueueFrame(p, &f);

									break;
							}
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

						//enable callprogress to wait, busy, calling, answer, etc...
	               		switch (p->sig)
	               		{
		                   	case DG_SIG_R2_NDIS_FIXED:
	    	               	case DG_SIG_R2_NDIS_VARIABLE:
		    					p_e1_thread_structure = dg_GetE1ThreadPointer(p->port);

		    					if (p_e1_thread_structure)
		    					{
									const char *dialplan_sd = NULL;

                                	ast_dgv_unlock(&p->lock, p->port);

                                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                		ast_log(LOG_NOTICE, "EV_AFTERDIAL: Before pbx_builtin_getvar_helper(DGV_SENDDTMF), port %d\n", p->port);

                                	if (p->ast_ch)
                                		dialplan_sd = pbx_builtin_getvar_helper(p->ast_ch, "DGV_SENDDTMF");
                                	else
                                		ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);

                                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                		ast_log(LOG_NOTICE, "EV_AFTERDIAL: After pbx_builtin_getvar_helper(DGV_SENDDTMF), port %d\n", p->port);

                                	ast_dgv_lock(&p->lock, p->port);
                                	
            						if (!p->ast_ch)
            						{
            							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);
            							break;
            						}

									if (dialplan_sd)
									{
										ast_copy_string(p->dialplan_senddtmf, dialplan_sd, sizeof(p->dialplan_senddtmf));
										
				                		if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "EV_AFTERDIAL: Setting DGV_SENDDTMF to %s on port %d\n", p->dialplan_senddtmf, p->port);

			    						if ( (p->group_b_received == p_e1_thread_structure->b_free_calling) ||
				    						 (p->group_b_received == p_e1_thread_structure->b_free_withoutbilling) ||
			    							 (p->group_b_received == p_e1_thread_structure->b_collectcall) )
			    						{
					                		p->call_state = DG_AST_WAITFORDIALTONE;
					                		ast_setstate(p->ast_ch, AST_STATE_RINGING);
					                		
					                		ast_dgv_unlock(&p->lock, p->port);
					                		{
						                		dg_DisableCallProgress(p->port);
						                		if (p->wait_dialtone)
						                		{
							                		if (option_verbose >= 3)
							                			ast_log(LOG_NOTICE, "Group B received(%d), trying to detect DIALTONE on port %d\n", p->group_b_received, p->port);
							                		
						                			ret = dg_EnableCallProgress(p->port, CP_ENABLE_LINETONE_OR_BUSY);
						                			if (ret != DG_EXIT_SUCCESS)
						                				ast_log(LOG_WARNING, "Failed to *enable* CallProgress thread - error code = %d\n", ret);
						                		}
					                		}
					                		ast_dgv_lock(&p->lock, p->port);
					                		
											if (!p->ast_ch)
											{
												ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);
												break;
											}

		    		    					if (p->senddtmf_dialtone_timeout_sched > -1) 
		    		    					{
		    		    						res_sched = ast_sched_del(sched, p->senddtmf_dialtone_timeout_sched);
		    							    	if (res_sched == -1)
		    							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

		    		    						p->senddtmf_dialtone_timeout_sched = -1;
		    		    					}
		    								p->senddtmf_dialtone_timeout_sched = ast_sched_add(sched, p->dialtone_timeout, senddtmf_dialtone_timeout_callback, p);
		    								if (p->senddtmf_dialtone_timeout_sched < 0)
		    									ast_log(LOG_WARNING, "ERROR Setting dialtone timeout callback on port %d\n", p->port);

		    								break;
			    						}
									}
									else //if (dialplan_sd)
									{
										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "event_handler: dialplan variable(DGV_SENDDTMF) does not exist(EV_AFTERDIAL) on port %d\n", p->port);
									} //if (dialplan_sd)

		    						if ( (p->group_b_received == p_e1_thread_structure->b_free_calling) ||
		    						 	 (p->group_b_received == p_e1_thread_structure->b_busy        ) ||
		    						 	 (p->group_b_received == p_e1_thread_structure->b_congestion  ) ||
		    							 (p->group_b_received == p_e1_thread_structure->b_collectcall ) )
		    						{
							    		p->call_state = DG_AST_CALLING;
						    			ast_setstate(p->ast_ch, AST_STATE_RINGING);
		    						}
		    						else if ( (p->group_b_received == p_e1_thread_structure->b_free_withoutbilling) ||
			    						  	  (p->group_b_received == p_e1_thread_structure->b_number_unknown     ) ||
			    						 	  (p->group_b_received == p_e1_thread_structure->b_number_changed     ) ||
			    							  (p->group_b_received == p_e1_thread_structure->b_out_of_service     ) )
		    						{
										const char *dialplan_gbr = NULL;
										char dialplan_group_number[20] = "";

										sprintf(dialplan_group_number, "DGV_GROUP_B_%d", p->group_b_received);
										
	                                	ast_dgv_unlock(&p->lock, p->port);
	
	                                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                                		ast_log(LOG_NOTICE, "EV_AFTERDIAL: Before pbx_builtin_getvar_helper(%s), port %d\n", dialplan_group_number, p->port);
	
	                                	if (p->ast_ch)
	                                		dialplan_gbr = pbx_builtin_getvar_helper(p->ast_ch, dialplan_group_number);
	                                	else
	                                		ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);
	
	                                	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                                		ast_log(LOG_NOTICE, "EV_AFTERDIAL: After pbx_builtin_getvar_helper(%s), port %d\n", dialplan_group_number, p->port);
	
	                                	ast_dgv_lock(&p->lock, p->port);
	                                	
	            						if (!p->ast_ch)
	            						{
	            							ast_log(LOG_WARNING, "event_handler: p->ast_ch does not exist(EV_AFTERDIAL) on port %d\n", p->port);
	            							break;
	            						}
	
										if (dialplan_gbr)
										{
											ast_copy_string(p->dialplan_group_b, dialplan_gbr, sizeof(p->dialplan_group_b));
											
					                		if (option_verbose >= 3)
												ast_log(LOG_NOTICE, "EV_AFTERDIAL: Setting %s to '%s' on port %d\n", dialplan_group_number, p->dialplan_group_b, p->port);
					                		
			    							struct ast_frame f = { AST_FRAME_CONTROL, };
			    							f.src = "chan_dgv";
			    							f.datalen = 0;
			    							
			    							if (!strcmp(p->dialplan_group_b, "BUSY"))
			    							{
			    	        					if (p->ast_ch)
			    	        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_BUSY);
			    								
						                		if (option_verbose >= 3)
						                			ast_log(LOG_NOTICE, "Group B received(%d), Sending AST_STATE_BUSY on port %d\n", p->group_b_received, p->port);
			    								
			    								//send busy
#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass.integer = AST_CONTROL_BUSY;
#else//#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass = AST_CONTROL_BUSY;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
			    								QueueFrame(p, &f);
			    								break;
			    							}
			    							else
			    								if (!strcmp(p->dialplan_group_b, "CONGESTION"))
			    								{
				    	        					if (p->ast_ch)
				    	        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
			    									
							                		if (option_verbose >= 3)
							                			ast_log(LOG_NOTICE, "Group B received(%d), Sending AST_CONTROL_CONGESTION on port %d\n", p->group_b_received, p->port);
				    								
				    								//send congestion
#ifdef CC_AST_HAS_VERSION_1_8
														f.subclass.integer = AST_CONTROL_CONGESTION;
#else//#ifdef CC_AST_HAS_VERSION_1_8
														f.subclass = AST_CONTROL_CONGESTION;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
				    								QueueFrame(p, &f);
				    								break;
			    								}
			    								else
			    									if (!strcmp(p->dialplan_group_b, "HANGUP"))
			    									{
					    	        					if (p->ast_ch)
					    	        						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_NORMAL);
			    										
								                		if (option_verbose >= 3)
								                			ast_log(LOG_NOTICE, "Group B received(%d), Sending AST_CONTROL_HANGUP on port %d\n", p->group_b_received, p->port);										                 		                                                
			    				
			    										//send hangup
#ifdef CC_AST_HAS_VERSION_1_8
															f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
															f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8

		    											QueueFrame(p, &f);
		    											break;
			    									}
			    									else //if (!strcmp(p->dialplan_group_b, "HANGUP"))
			    									{
			    										//nothing to do
			    										
								                		if (option_verbose >= 3)
								                			ast_log(LOG_NOTICE, "Group B received(%d), but %s was not set correctly on port %d (use BUSY, CONGESTION or HANGUP)\n", 
								                					p->group_b_received, dialplan_group_number, p->port);
			    									} //if (!strcmp(p->dialplan_group_b, "HANGUP"))

										} //if (dialplan_gbr)

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "event_handler: dialplan variable(%s) does not exist(EV_AFTERDIAL), sending AST_CONTROL_ANSWER on port %d\n", dialplan_group_number, p->port);
										
										p->call_state = DG_AST_CONVERSATION;
					
										struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										f.src = "chan_dgv";
										f.datalen = 0;
										//ast_dgv_lock(&p->lock, p->port); 
										//{
										//QueueFrame(EventContext->port, &f);
										QueueFrame(p, &f);
										//} 
										//ast_dgv_unlock(&p->lock, p->port);

										ast_setstate(p->ast_ch, AST_STATE_UP);
		
					                	ret = dg_PauseInputBuffer(p->port, DG_RELEASE);
										if (ret != DG_EXIT_SUCCESS)
											ast_log(LOG_NOTICE, "Error releasing InputBuffer %d cod = %x - state %x\n", p->port, ret,
                                					GetInputBufferStatus(p->port));			
										else
											if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
												ast_log(LOG_WARNING, "EV_AFTERDIAL: Releasing InputBuffer on port %d\n", p->port);
			    					}
		    						else
		    						{
							    		p->call_state = DG_AST_CALLING;							    		
							    		ast_setstate(p->ast_ch, AST_STATE_RINGING);		    							
		    						}
								} //if (p_e1_thread_structure)
		    					else
		    						ast_log(LOG_WARNING, "event_handler: p_e1_thread_structure does not exist(EV_AFTERDIAL) on port %d\n", p->port);				        	

	            	            break;
	    	               	case DG_SIG_GSM:
								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "EV_AFTERDIAL %d, sending AST_CONTROL_RINGING\n", EventContext->port);

								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								QueueFrame(p, &f);

								//do not break, so DG_SIG_PRI
	           	    	    case DG_SIG_PRI:
							    p->call_state = DG_AST_CALLING;
						    	ast_setstate(p->ast_ch, AST_STATE_RINGING);
						    	
	           					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	           						ast_log(LOG_NOTICE, "EV_AFTERDIAL %d, ast_setstate AST_STATE_RINGING\n", EventContext->port);

		                       	break;
			                case DG_SIG_CUSTOMCAS:
	    	                case DG_SIG_FXO:
	    	                case DG_SIG_CB_FXO:
								if ((p->polarity == 0) || (p->polarity == 2))
								{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_AFTERDIAL: Enabling Answer detection on %d \n", p->port);
									
									ast_setstate(p->ast_ch, AST_STATE_RINGING);
									
									ast_dgv_unlock(&p->lock, p->port);
									{
				    				    ret = dg_EnableCallProgress(p->port, CP_ENABLE_ALL);
									    if (ret==EXIT_SUCCESS)
									    {
						   					ret = dg_EnableAnswerDetection(p->port);
						   					if (ret==EXIT_SUCCESS)
												p->call_state = DG_AST_CALLING;																   						
						   					else
						   						ast_log(LOG_ERROR, "ERROR FXO Enabling AnswerDetection on %d \n", p->port);					    					    
								    	}
									    else
									    	ast_log(LOG_ERROR, "ERROR FXO Enabling CallProgress on %d \n", p->port);
									}
									ast_dgv_lock(&p->lock, p->port);
									
									if (ret != EXIT_SUCCESS)
										if (p->ast_ch)
										{
											ast_log(LOG_WARNING, "EV_AFTERDIAL: Setting AST_STATE_BUSY on %d \n", p->port);											
											ast_setstate(p->ast_ch, AST_STATE_BUSY);
										}
								}
								else //if ((p->polarity == 0) && (p->polarity == 2))
								{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_AFTERDIAL: Not Enabling Answer detection on %d because polarity = %d \n", p->port, p->polarity);

									p->call_state = DG_AST_CALLING;
									ast_setstate(p->ast_ch, AST_STATE_RINGING);									
								} //if ((p->polarity == 0) && (p->polarity == 2))
		                       	break;
	    	   	        } //switch (p->sig)
					}
					ast_dgv_unlock(&p->lock, p->port);
				} //if (p->call_state==DG_AST_DIALING)
			} //if (p)
			break;
        case EV_SILENCE:
            if(EventContext->data==1)
            {
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                    ast_verbose(VERBOSE_PREFIX_3 "SILENCE on port %d - data=%x\n",EventContext->port,EventContext->data);

                //check if there is a bridged channel with, removed on 0.9.3
                /*if (p->ast_ch)
                {                	
                    if ((dgv_chan_tmp=dgv_bridged_channel(p->ast_ch))!=NULL)
                    {
                        //tell to bridged channel to stop playbuffer                                     
                        if (dgv_chan_tmp->tech_pvt!=NULL && !strncmp(dgv_chan_tmp->name, type, 3))
                        {      
                            p_bridged = (struct dgv_pvt *)dgv_chan_tmp->tech_pvt;
                            
                            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                                 ast_log(LOG_NOTICE, "(%d) SILENCE Detected: tells to %s drop tx samples\n",p->port, p_bridged->dev);
   
   							//drop tx samples
							p_bridged->force_drop_tx = 1;						
                        }
                    }
                }*/
            }
            else
            {
                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                    ast_verbose(VERBOSE_PREFIX_3 "AUDIO on port %d - data=%x\n",EventContext->port,EventContext->data);
            }
            break;
        case EV_DTMF:
			if (p)
			{
	            cAux = EventContext->data;
	            
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "Port %d Digit %c\n", EventContext->port, cAux);

	            ast_dgv_lock(&p->lock, p->port); 
	            {
		            /*if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0) {
		                sprintf(szTemp,"MF: %c",cAux);
		                save_log(p->port, szTemp);
		            }*/

					if (p->sig==DG_SIG_R2_NDIS_VARIABLE && p->call_state==DG_AST_R2_SIGNALLING)
					{
						//saves last digit received time
						p->tlastdigit = ast_tvnow();
						if (p->digit_timeout_sched > -1)
						{
		                    res_sched = ast_sched_del(sched, p->digit_timeout_sched);
					    	if (res_sched == -1)
					    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

		                    p->digit_timeout_sched = -1;
		                }

						p->digit_timeout_sched = ast_sched_add(sched, p->interdigit_timeout, digit_timeout_callback, p);
		
						if (p->digit_timeout_sched<0)
							ast_log(LOG_NOTICE, "Port %d ERROR Setting timeout\n",EventContext->port);

					}
		            //ast_sched_runq(sched); --> 0.9.6_rc9

					if ((p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && p->call_state == DG_AST_DIALING)
					{
						dg_GenerateMF(p->port, GENERATE_OFF, 0);

						struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass.integer = EventContext->data;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass = EventContext->data;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
						f.src = "chan_dgv";
						f.datalen = 0;
				    	//ast_dgv_lock(&p->lock, p->port); 
						//{
							//QueueFrame(EventContext->port, &f);
							QueueFrame(p, &f);
						//}
						//ast_dgv_unlock(&p->lock, p->port);
	                }
					
					if (p->ast_ch)
					{
						//transfer DTMF only if it isnt in conversation and not bridged
						if (dgv_channel_state(p->ast_ch) == AST_STATE_UP) // && !dgv_channel_is_bridged(p->ast_ch))
						{
	                        //comentado na v0.7.4 - isso estava causando paradas no audio durante a conversacao
							//dg_DisableInputBuffer(p->port);
	                        //dg_PauseInputBuffer(p->port, DG_PAUSE);
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_log(LOG_NOTICE, "Detecting DTMF to port %d - data %x\n",EventContext->port,cAux);

							short ast_send_dtmf = 1;

							if (p->subchannel && p->second_call && (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS) &&
								p->call_state == DG_AST_CONVERSATION && p->subchannel->call_state == DG_AST_CALLING)
							{
		    					if (p->second_call_sched > -1) 
		    					{
		    						res_sched = ast_sched_del(sched, p->second_call_sched);
							    	if (res_sched == -1)
							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

		    						p->second_call_sched = -1;
		    					}
								
								if (EventContext->data == p->second_call_digits[p->scd_step])
								{
									ast_send_dtmf = 0;

									if (strlen(p->second_call_digits) == (p->scd_step + 1))
									{
										p->sc_save_digits[0] = '\0';
										p->sc_save_digits[1] = '\0';
										p->sc_save_digits[2] = '\0';
										p->sc_save_digits[3] = '\0';
										p->sc_save_digits[4] = '\0';										
										
										p->scd_step = 0;
										
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "EV_DTMF: SecondCall digit detected, trying to exchange call on port %d\n", p->port);

										if ((p->ast_ch) && (p->subchannel->ast_ch))
										{											
											dg_PauseInputBuffer(p->port, DG_PAUSE);
							                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
							                	ast_log(LOG_WARNING, "EV_DTMF: Pausing InputBuffer on port %d\n", p->port);
											dg_StopPlayBuffer(p->port);

											ast_setstate(p->ast_ch, AST_STATE_DOWN);
											ast_setstate(p->subchannel->ast_ch, AST_STATE_DOWN);

											p->write_last_format=-1;
											p->subchannel->write_last_format=-1;

#ifdef CC_AST_HAS_VERSION_12
												RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
												struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
											dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
											if (dgv_chan_bridged)
											{
#ifdef CC_AST_HAS_VERSION_12
													RAII_VAR(struct ast_channel *, dgv_subchan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
													struct ast_channel *dgv_subchan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
												dgv_subchan_bridged = dgv_bridged_channel(p->subchannel->ast_ch);
												if (dgv_subchan_bridged)
												{
													if (option_verbose >= 3)
														ast_log(LOG_NOTICE, "EV_DTMF: Stopping music on hold on '%s'\n", dgv_channel_name(p->subchannel->ast_ch));
													
													struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
														fmoh.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
														fmoh.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
													fmoh.src = "chan_dgv";													
													fmoh.datalen = 0;
													QueueFrame(p->subchannel, &fmoh);
												}
												
												if (option_verbose >= 3)
													ast_log(LOG_NOTICE, "EV_DTMF: Exchanging call on port %d\n", p->port);
												
												if (option_verbose >= 3)
													ast_log(LOG_NOTICE, "EV_DTMF: Starting music on hold on '%s'\n", dgv_channel_name(p->ast_ch));

												struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
													fmoh.subclass.integer = AST_CONTROL_HOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
													fmoh.subclass = AST_CONTROL_HOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
												fmoh.src = "chan_dgv";
												
#ifdef CC_AST_HAS_DATA_PTR												
													fmoh.data.ptr = DEFAULT_MOHINTERPRET;
#else//#ifdef CC_AST_HAS_DATA_PTR													
													fmoh.data = DEFAULT_MOHINTERPRET;
#endif//#ifdef CC_AST_HAS_DATA_PTR													
													
												fmoh.datalen = strlen(DEFAULT_MOHINTERPRET) + 1;
												QueueFrame(p, &fmoh);

												swap_sc(p, p->subchannel);

												if (p->subchannel_audio_index == 1)
												{
													p->subchannel_audio_index = 2;
													p->subchannel->subchannel_audio_index = 2;
												}
												else
												{
													p->subchannel_audio_index = 1;
													p->subchannel->subchannel_audio_index = 1;
												}

												struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
												f.src = "chan_dgv";
												f.datalen = 0;
												QueueFrame(p, &f);

												p->call_state = DG_AST_CONVERSATION;
												ast_setstate(p->ast_ch, AST_STATE_UP);
												
												p->subchannel->call_state = DG_AST_CALLING;
												ast_setstate(p->subchannel->ast_ch, AST_STATE_UP);

												if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
													ast_log(LOG_NOTICE, "EV_DTMF (%d): ast_setstate(AST_STATE_UP). \n", p->port);
											}
											else
											{
												if (option_verbose >= 3)//if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
													ast_log(LOG_WARNING, "event_handler: dgv_chan_bridged does not exist(EV_DTMF) on port %d\n", p->port);
											}
										}
										else
										{
											if (option_verbose >= 3)//if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
												ast_log(LOG_WARNING, "event_handler: p->subchannel->ast_ch does not exist(EV_DTMF) on port %d\n", p->port);

						                    int nSCDialLen = strlen(p->sc_save_digits);
											char cSCDial;
				                		    int nForLen;

						                    for (nForLen = 0; nForLen < nSCDialLen; nForLen++)
				        		            {
						                    	cSCDial = p->sc_save_digits[nForLen];

												if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
													ast_log(LOG_NOTICE, "EV_DTMF: Sending DTMF %d on port %d \n", cSCDial, p->port);

												struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass.integer = cSCDial;
#else//#ifdef CC_AST_HAS_VERSION_1_8
													f.subclass = cSCDial;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
												f.src = "chan_dgv";
												f.datalen = 0;
												QueueFrame(p, &f);
				                    		}

						                    ast_send_dtmf = 1;

											p->sc_save_digits[0] = '\0';
											p->sc_save_digits[1] = '\0';
											p->sc_save_digits[2] = '\0';
											p->sc_save_digits[3] = '\0';
											p->sc_save_digits[4] = '\0';
										}
									}
									else
									{
										p->sc_save_digits[strlen(p->sc_save_digits)] = EventContext->data;

										if (strlen(p->sc_save_digits) != 5)
											p->sc_save_digits[strlen(p->sc_save_digits)] = '\0';

										p->scd_step++;
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "EV_DTMF: SecondCall digit detected, waiting for %c on port %d\n",
													p->second_call_digits[p->scd_step], p->port);

										p->second_call_sched = ast_sched_add(sched, 3000, second_call_callback, p);
										if (p->second_call_sched<0)
											ast_log(LOG_WARNING, "ERROR Setting SecondCall digit timeout on port %d\n", EventContext->port);
									}
								}
								else
								{
									if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
										ast_log(LOG_NOTICE, "EV_DTMF: digit %d != second_call_digits %d on %d\n",
												EventContext->data, p->second_call_digits[p->scd_step], p->port);
									p->scd_step = 0;
									
				                    int nSCDialLen = strlen(p->sc_save_digits);
									char cSCDial;
		                		    int nForLen;

				                    for (nForLen = 0; nForLen < nSCDialLen; nForLen++)
		        		            {
				                    	cSCDial = p->sc_save_digits[nForLen];

										if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
											ast_log(LOG_NOTICE, "EV_DTMF: Sending DTMF %d on port %d \n", cSCDial, p->port);

										struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = cSCDial;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = cSCDial;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										f.src = "chan_dgv";
										f.datalen = 0;
										QueueFrame(p, &f);
		                    		}

				                    ast_send_dtmf = 1;

									p->sc_save_digits[0] = '\0';
									p->sc_save_digits[1] = '\0';
									p->sc_save_digits[2] = '\0';
									p->sc_save_digits[3] = '\0';
									p->sc_save_digits[4] = '\0';
								}
							} //if (p->subchannel && p->second_call && (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS))

							if ( ( (p->dtmf_to_asterisk==1) ||
							       (p->dtmf_to_asterisk==2 && !dgv_channel_is_bridged(p->ast_ch)) ||
							       (p->flashing==1 && (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS)) ) && 
							       (ast_send_dtmf==1))
							{
								struct ast_frame f = { AST_FRAME_DTMF, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = EventContext->data;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = EventContext->data;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
				    			//ast_dgv_lock(&p->lock, p->port); 
				    			//{
								//QueueFrame(EventContext->port, &f);
				    			QueueFrame(p, &f);
							    //}
							    //ast_dgv_unlock(&p->lock, p->port);
							}	
							/* stores dtmf to generate on EV_AUDIODETECTED event */ 
							p->last_dtmf = EventContext->data;
						}
						else
						{
						    p->last_dtmf = -1;
						}
						
						if (p->call_state == DG_AST_CONVERSATION && !dgv_channel_is_bridged(p->ast_ch))
						{
	                        if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                            ast_log(LOG_NOTICE, "Port %d Digit %c - stopplaybuffer\n",EventContext->port, cAux);
	
							dg_StopPlayBuffer(p->port);	//avoiding noise
						}
					}
					else //if (p->ast_ch)
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "event_handler: p->ast_ch does not exist(EV_DTMF) on port %d\n", p->port);
					} //if (p->ast_ch)
				}
				ast_dgv_unlock(&p->lock, p->port);
			}
			else //if (p)
			{
				if (option_verbose >= 3)
					ast_log(LOG_WARNING, "Unable to locate port %d structure!\n", EventContext->port);

			} //if (p)
			
			break;
			
		//GSM EVENTS
		case EV_GSMERROR:
			dg_GSMGetMessage(EventContext->port, tmpGSMMessage);

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMERROR '%s' on port %d\n", tmpGSMMessage, EventContext->port);

			if (p->sms_reserved)
				manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
												 "SendTo: %s\r\n"
												 "Group: %d\r\n"
												 "Port: %d\r\n"
												 "SIMCardID: %s\r\n"
												 "Status: %s\r\n"
												 "Error: %s\r\n"
												 "CustomID: %s\r\n",
												 p->sms_send_to, p->group, p->port, p->gsm_simcard_id, "ERROR", tmpGSMMessage, p->sms_send_custom_id
							  );

			if (!strncmp(tmpGSMMessage, "+CME ERROR: call barred", 23))
			{
	       		if (option_verbose >= 3)
	       			ast_log(LOG_NOTICE, "EV_GSMERROR, 'CALL BARRED' message received, restarting GSM port %d\n", EventContext->port);

	       		ast_dgv_lock(&p->lock, p->port);
	       		{
					if (p->ast_ch)
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "EV_GSMERROR port %d - Setting cause AST_CAUSE_CONGESTION\n", p->port);

						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
						ast_queue_control(p->ast_ch, AST_CONTROL_CONGESTION);
					}
					p->alarmed = 0;
	       		}
    			ast_dgv_unlock(&p->lock, p->port);
			}

			if ((EventContext->data == GSM_SMS3) ||
				(EventContext->data == GSM_SMS5) ||
				(EventContext->data == GSM_LIST))
			{
				ast_log(LOG_WARNING, "GSM Error received, the SIM Card %d is probably full. Try to clear it.\n", p->port);
				break;
			}

            //do not break, so EV_GSMTIMEOUT						
		case EV_GSMTIMEOUT:
			if (EventContext->command == EV_GSMTIMEOUT)
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GSMTIMEOUT on port %d\n", EventContext->port);
            
			if (EventContext->command == EV_GSMTIMEOUT)
				if (p->sms_reserved)
					manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
													 "SendTo: %s\r\n"
													 "Group: %d\r\n"
													 "Port: %d\r\n"
													 "SIMCardID: %s\r\n"
													 "Status: %s\r\n"
													 "TimeOut(ms): %d\r\n"
													 "CustomID: %s\r\n",
													 p->sms_send_to, p->group, p->port, p->gsm_simcard_id, "TIMEOUT", gsmgenerictimeout, p->sms_send_custom_id
								  );

            ast_dgv_lock(&p->lock, p->port);
            {
				if (p->sms_reserved)
				{
					p->sms_reserved = 0;
					p->sms_send_to[0] = '\0';
					p->sms_send_messagereference = -1;
					p->sms_send_custom_id[0] = '\0';
					if (EventContext->command == EV_GSMTIMEOUT)
						p->sms_sent_status = -1;
					else
					{
						for (nForText = 0; nForText <= sizeof(p->sms_sent_tmpMessageError) - 1; nForText++)
							p->sms_sent_tmpMessageError[nForText] = '\0';
						ast_copy_string(p->sms_sent_tmpMessageError, tmpGSMMessage, sizeof(p->sms_sent_tmpMessageError));
						p->sms_sent_status = -2;
					}
				}

				if (p->reserved)
				{
					if (p->ast_ch)
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "EV_GSMTIMEOUT port %d - Setting cause AST_CAUSE_CONGESTION\n", p->port);

						dgv_channel_hangupcause_set(p->ast_ch, AST_CAUSE_CONGESTION);
						ast_queue_control(p->ast_ch, AST_CONTROL_CONGESTION);
					}
				}

				if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMTIMEOUT (%d) - deleting gsmerror schedule\n", p->port);

					res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
					if (res_sched == -1)
						ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmerror_timeout_sched = -1;
				}

				p->alarmed=99;

				//set gsmerror timeout
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GSMTIMEOUT (%d) - Creating gsmerror schedule\n", p->port);

				p->gsmerror_timeout_sched = ast_sched_add(sched, gsmerrortimeout, gsmerror_timeout_callback, p);

				if (p->gsmclearsms_timeout_sched > -1)
				{
					res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
					if (res_sched == -1)
						ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmclearsms_timeout_sched = -1;
				}
            }
            ast_dgv_unlock(&p->lock, p->port);

			break;
		case EV_GSMSMSRECEIVED:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSRECEIVED, receiving SMS message on port %d\n", EventContext->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->alarmed == 99)
					p->alarmed = 0;
			}
			ast_dgv_unlock(&p->lock, p->port);

			if (p->gsmerror_timeout_sched > -1)
			{
    			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    				ast_log(LOG_NOTICE, "EV_GSMSMSRECEIVED port %d - deleting gsmerror schedule\n", p->port);

				res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
		    	if (res_sched == -1)
		    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

				p->gsmerror_timeout_sched = -1;
			}

			dg_GSMGetSMS(EventContext->port, tmpGSMMessage);
			
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSRECEIVED '%s' on port %d\n", tmpGSMMessage, EventContext->port);			
			
			for (nForText = 0; nForText <= sizeof(received_from) - 1; nForText++)
				received_from[nForText] = '\0';
			
			for (nForText = 0; nForText <= sizeof(date) - 1; nForText++)
				date[nForText] = '\0';
			
			for (nForText = 0; nForText <= sizeof(time) - 1; nForText++)
				time[nForText] = '\0';
			
			for (nForText = 0; nForText <= sizeof(text_message) - 1; nForText++)
				text_message[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(st_cod) - 1; nForText++)
				st_cod[nForText] = '\0';

			nStepParse = 1;
			
			for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
			{				
				switch (nStepParse)
				{
					case 1:
						if (tmpGSMMessage[nForText] == ',')
							nStepParse++;

						break;
					case 2:
						nStepParse++;
						nPosText = 0;

						break;
					case 3:
						if (tmpGSMMessage[nForText] != '"')
						{
							received_from[nPosText] = tmpGSMMessage[nForText];							
							nPosText++;
							received_from[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 4:
						if (tmpGSMMessage[nForText] == '"')
						{
							nStepParse++;
							nPosText = 0;
						}						

						break;
					case 5:						
						if (tmpGSMMessage[nForText] != ',')
						{							
							date[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							date[nPosText] = '\0';
						}
						else
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 6:
						if (tmpGSMMessage[nForText] != '-')
						{
							time[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							time[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;						
					case 7:
						if (tmpGSMMessage[nForText] == '"')
						{
							nStepParse++;
							nPosText = 0;							
						}
						
						break;
					case 8:
						text_message[nPosText] = tmpGSMMessage[nForText];
						nPosText++;
						text_message[nPosText] = '\0';
						
						break;					
				} //switch (nStepParse)				
			} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
						
			if ((text_message[1] == '\r') || (text_message[1] == '\n'))
			{
				nPosText = 0;
				
				for (nForText = 2; nForText <= strlen(text_message) - 1; nForText++)
				{					
					text_message[nPosText] = text_message[nForText];
					nPosText++;				
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)
				
				text_message[nPosText] = '\0';				
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))
						
			if ((text_message[0] == '\r') || (text_message[0] == '\n'))
			{
				nPosText = 0;
				
				for (nForText = 1; nForText <= strlen(text_message) - 1; nForText++)
				{					
					text_message[nPosText] = text_message[nForText];
					nPosText++;					
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)
				
				text_message[nPosText] = '\0';				
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))
			
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSRECEIVED - '%s', '%s', '%s', '%s' on port %d\n", received_from, date, time, text_message, EventContext->port);

       		ast_dgv_lock(&p->lock, p->port);
			{
	       		if (option_verbose >= 3)
	       			ast_verbose(VERBOSE_PREFIX_3 "SMS received on port %d\n", EventContext->port);

	       		manager_event(EVENT_FLAG_SYSTEM, "SMS",
	                        				 	 "Group: %d\r\n"
	                        				 	 "Port: %d\r\n"
	                        				 	 "From: %s\r\n"
	                        				 	 "Date: %s\r\n"
	                        					 "Time: %s\r\n"
	                        					 "Text: %s\r\n"
	                        					 "--END EVENT--\r\n",
	                        					 p->group, p->port, received_from, date, time, text_message
	                          );
						
				//p->call_state=DG_AST_RINGING;
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "Calling dgv_new port %d using context=%s\n", p->port, p->context);
						
				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "Going to extension %s|1\n", DEFAULT_SMS_EXTEN);

				p_smschannel = find_free_sms_channel(p);
				if (p_smschannel)
				{
					p_smschannel->sms_reserved = 1;
					ast_copy_string(p_smschannel->exten, DEFAULT_SMS_EXTEN, sizeof(p_smschannel->exten));

					if (strlen(received_from) > 1)
						ast_copy_string(p_smschannel->cid_num, received_from, sizeof(p_smschannel->cid_num));
					else
						p->cid_num[0] = '\0';

					ast_copy_string(p_smschannel->sms_received_from     , received_from         , sizeof(p_smschannel->sms_received_from));
					ast_copy_string(p_smschannel->sms_received_simcardid, p->gsm_simcard_id     , sizeof(p_smschannel->sms_received_simcardid));
					ast_copy_string(p_smschannel->sms_received_date     , date                  , sizeof(p_smschannel->sms_received_date));
					ast_copy_string(p_smschannel->sms_received_time     , time                  , sizeof(p_smschannel->sms_received_time));
					ast_copy_string(p_smschannel->sms_received_text     , text_message          , sizeof(p_smschannel->sms_received_text));
					ast_copy_string(p_smschannel->sms_received_st_cod   , st_cod                , sizeof(p_smschannel->sms_received_st_cod));
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
					{
						//nothing to do
					}
					else
						ast_log(LOG_ERROR, "Error creating dgv_new to GSM SMS - port %d \n", p->port);
				}
				else
					ast_log(LOG_WARNING, "Error finding a free sms channel for port %d, dropping this message.\n", p->port);
			}
			ast_dgv_unlock(&p->lock, p->port);
			
			break;
		case EV_GSMSMSCONFIRMATION:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSCONFIRMATION, receiving SMS confirmation message on port %d\n", EventContext->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->alarmed == 99)
					p->alarmed = 0;

				if (p->gsmerror_timeout_sched > -1)
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMSMSCONFIRMATION port %d - deleting gsmerror schedule\n", p->port);

					res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
					if (res_sched == -1)
						ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmerror_timeout_sched = -1;
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			dg_GSMGetSMSConfirmation(EventContext->port, tmpGSMMessage);

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSCONFIRMATION '%s' on port %d\n", tmpGSMMessage, EventContext->port);

			for (nForText = 0; nForText <= sizeof(received_from) - 1; nForText++)
				received_from[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(date) - 1; nForText++)
				date[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(time) - 1; nForText++)
				time[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(text_message) - 1; nForText++)
				text_message[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(st_cod) - 1; nForText++)
				st_cod[nForText] = '\0';

			for (nForText = 0; nForText <= sizeof(message_reference) - 1; nForText++)
				message_reference[nForText] = '\0';

			nStepParse = 1;

			for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
			{
				switch (nStepParse)
				{
					case 1:
						if (tmpGSMMessage[nForText] == ',')
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 2:
						if (tmpGSMMessage[nForText] != ',')
						{
							message_reference[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							message_reference[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 3:
						nStepParse++;
						nPosText = 0;

						break;
					case 4:
						if (tmpGSMMessage[nForText] != '"')
						{
							received_from[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							received_from[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 5:
						if (tmpGSMMessage[nForText] == '"')
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 6:
						if (tmpGSMMessage[nForText] != ',')
						{
							date[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							date[nPosText] = '\0';
						}
						else
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 7:
						if (tmpGSMMessage[nForText] != '-')
						{
							time[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							time[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 8:
						if (tmpGSMMessage[nForText] == ',')
							nStepParse++;

						break;
					case 9:
						if (tmpGSMMessage[nForText] == '"')
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 10:
						if (tmpGSMMessage[nForText] != '-')
						{
							text_message[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							text_message[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 11:
						if (tmpGSMMessage[nForText] == ',')
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 12:
						if (nPosText <= 2)
						{
							st_cod[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							st_cod[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 13:
						break;
				} //switch (nStepParse)
			} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)

			if ((text_message[1] == '\r') || (text_message[1] == '\n'))
			{
				nPosText = 0;

				for (nForText = 2; nForText <= strlen(text_message) - 1; nForText++)
				{
					text_message[nPosText] = text_message[nForText];
					nPosText++;
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)

				text_message[nPosText] = '\0';
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))

			if ((text_message[0] == '\r') || (text_message[0] == '\n'))
			{
				nPosText = 0;

				for (nForText = 1; nForText <= strlen(text_message) - 1; nForText++)
				{
					text_message[nPosText] = text_message[nForText];
					nPosText++;
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)

				text_message[nPosText] = '\0';
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSCONFIRMATION - '%s', '%s', '%s', '%s', '%s', '%s' on port %d\n", received_from, date, time, text_message, st_cod, message_reference, EventContext->port);

       		ast_dgv_lock(&p->lock, p->port);
			{
	       		if (option_verbose >= 3)
	       			ast_verbose(VERBOSE_PREFIX_3 "SMS confirmation received on port %d\n", EventContext->port);

	       		manager_event(EVENT_FLAG_SYSTEM, "SMSConfirmation",
	                        				 	 "Group: %d\r\n"
	                        				 	 "Port: %d\r\n"
	                        				 	 "From: %s\r\n"
	                        				 	 "Date: %s\r\n"
	                        					 "Time: %s\r\n"
	                        					 "Text: %s\r\n"
	                        					 "StCod: %s\r\n"
	                        					 "MessageID: %s\r\n"
	                        					 "--END EVENT--\r\n",
	                        					 p->group, p->port, received_from, date, time, text_message, st_cod, message_reference
	                          );

				//p->call_state=DG_AST_RINGING;
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "Calling dgv_new port %d using context=%s\n", p->port, p->context);

				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "Going to extension %s|1\n", DEFAULT_SMS_ACK_EXTEN);

				p_smschannel = find_free_sms_channel(p);
				if (p_smschannel)
				{
					p_smschannel->sms_reserved = 1;
					ast_copy_string(p_smschannel->exten, DEFAULT_SMS_ACK_EXTEN, sizeof(p_smschannel->exten));

					if (strlen(received_from) > 1)
						ast_copy_string(p_smschannel->cid_num, received_from, sizeof(p_smschannel->cid_num));
					else
						p->cid_num[0] = '\0';

					ast_copy_string(p_smschannel->sms_received_from     , received_from    , sizeof(p_smschannel->sms_received_from));
					ast_copy_string(p_smschannel->sms_received_simcardid, p->gsm_simcard_id, sizeof(p_smschannel->sms_received_simcardid));
					ast_copy_string(p_smschannel->sms_received_date     , date             , sizeof(p_smschannel->sms_received_date));
					ast_copy_string(p_smschannel->sms_received_time     , time             , sizeof(p_smschannel->sms_received_time));
					ast_copy_string(p_smschannel->sms_received_text     , text_message     , sizeof(p_smschannel->sms_received_text));
					ast_copy_string(p_smschannel->sms_received_st_cod   , st_cod           , sizeof(p_smschannel->sms_received_st_cod));
					ast_copy_string(p_smschannel->sms_received_mr       , message_reference, sizeof(p_smschannel->sms_received_mr));

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
					{
						//nothing to do
					}
					else
						ast_log(LOG_ERROR, "Error creating dgv_new to GSM SMS confirmation - port %d \n", p->port);
				}
				else
					ast_log(LOG_WARNING, "Error finding a free sms channel for port %d, dropping this confirmation message.\n", p->port);
			}
			ast_dgv_unlock(&p->lock, p->port);

			break;
		case EV_GSMUSSDRECEIVED:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMUSSDRECEIVED, receiving USSD message on port %d\n", EventContext->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->alarmed == 99)
					p->alarmed = 0;

				if (p->gsmerror_timeout_sched > -1)
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMUSSDRECEIVED port %d - deleting gsmerror schedule\n", p->port);

					res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
					if (res_sched == -1)
						ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmerror_timeout_sched = -1;
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			dg_GSMGetUSSD(EventContext->port, tmpGSMMessage);

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMUSSDRECEIVED '%s' on port %d\n", tmpGSMMessage, EventContext->port);

			for (nForText = 0; nForText <= sizeof(text_message) - 1; nForText++)
				text_message[nForText] = '\0';

			nStepParse = 1;

			for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
			{
				switch (nStepParse)
				{
					case 1:
						if (tmpGSMMessage[nForText] == ',')
							nStepParse++;

						break;
					case 2:
						if (tmpGSMMessage[nForText] == '"')
						{
							nStepParse++;
							nPosText = 0;
						}

						break;
					case 3:
						if (tmpGSMMessage[nForText] != '"')
						{
							text_message[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							text_message[nPosText] = '\0';
						}
						else
							nStepParse++;

						break;
					case 4:
						break;
				} //switch (nStepParse)
			} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)

			if ((text_message[1] == '\r') || (text_message[1] == '\n'))
			{
				nPosText = 0;

				for (nForText = 2; nForText <= strlen(text_message) - 1; nForText++)
				{
					text_message[nPosText] = text_message[nForText];
					nPosText++;
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)

				text_message[nPosText] = '\0';
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))

			if ((text_message[0] == '\r') || (text_message[0] == '\n'))
			{
				nPosText = 0;

				for (nForText = 1; nForText <= strlen(text_message) - 1; nForText++)
				{
					text_message[nPosText] = text_message[nForText];
					nPosText++;
				} //for (nForText = 0; nForText <= strlen(text_message) - 1; nForText++)

				text_message[nPosText] = '\0';
			} //if ((text_message[1] == '\r') || (text_message[1] == '\n'))

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMUSSDRECEIVED - '%s' on port %d\n", text_message, EventContext->port);

       		ast_dgv_lock(&p->lock, p->port);
			{
	       		if (option_verbose >= 3)
	       			ast_verbose(VERBOSE_PREFIX_3 "USSD received on port %d\n", EventContext->port);

	       		manager_event(EVENT_FLAG_SYSTEM, "USSD",
	                        				 	 "Group: %d\r\n"
	                        				 	 "Port: %d\r\n"
	                        					 "Text: %s\r\n"
	                        					 "--END EVENT--\r\n",
	                        					 p->group, p->port, text_message
	                          );

				//p->call_state=DG_AST_RINGING;
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "Calling dgv_new port %d using context=%s\n", p->port, p->context);

				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "Going to extension %s|1\n", DEFAULT_USSD_EXTEN);

				p_smschannel = find_free_sms_channel(p);
				if (p_smschannel)
				{
					p_smschannel->sms_reserved = 1;
					ast_copy_string(p_smschannel->exten, DEFAULT_USSD_EXTEN, sizeof(p_smschannel->exten));

					p->cid_num[0] = '\0';

					ast_copy_string(p_smschannel->sms_received_simcardid, p->gsm_simcard_id     , sizeof(p_smschannel->sms_received_simcardid));
					ast_copy_string(p_smschannel->ussd_received_text    , text_message          , sizeof(p_smschannel->ussd_received_text));
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
						if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL, NULL)!=NULL)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0, NULL)!=NULL)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							if (dgv_new(p_smschannel, AST_STATE_RING, p_smschannel->context, START_PBX, 0)!=NULL)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
					{
						//nothing to do
					}
					else
						ast_log(LOG_ERROR, "Error creating dgv_new to GSM USSD - port %d \n", p->port);
				}
				else
					ast_log(LOG_WARNING, "Error finding a free ussd channel for port %d, dropping this message.\n", p->port);
			}
			ast_dgv_unlock(&p->lock, p->port);

			break;
		case EV_GSMMESSAGE:
			dg_GSMGetMessage(EventContext->port, tmpGSMMessage);

			if (!strncmp(tmpGSMMessage, "+COPS:", 6))
			{
				for (nForText = 0; nForText <= sizeof(operator) - 1; nForText++)
					operator[nForText] = '\0';

				nStepParse = 1;

				for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
				{
					switch (nStepParse)
					{
						case 1:
							if (tmpGSMMessage[nForText] == ',')
								nStepParse++;

							break;
						case 2:
							if (tmpGSMMessage[nForText] == ',')
								nStepParse++;

							break;
						case 3:
							nStepParse++;
							nPosText = 0;

							break;
						case 4:
							if (tmpGSMMessage[nForText] != '"')
							{
								operator[nPosText] = tmpGSMMessage[nForText];
								nPosText++;
								operator[nPosText] = '\0';
							}

							break;
					} //switch (nStepParse)
				} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)

				if (strlen(operator) > 1)
				{
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "Operator '%s' detected on port %d\n", operator, EventContext->port);

					ast_copy_string(p->gsm_operator_name, operator, sizeof(p->gsm_operator_name));

					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMMESSAGE, operator '%s' detected on port %d\n", p->gsm_operator_name, EventContext->port);
				}

				dg_GSMSendCommand(p->port, DEFAULT_GSM_REQ_SIMID);
			} //if (!strncmp(tmpGSMMessage, "+COPS:", 6))

			if (!strncmp(tmpGSMMessage, "^SCID:", 6))
			{
				for (nForText = 0; nForText <= sizeof(simcardid) - 1; nForText++)
					simcardid[nForText] = '\0';

				nStepParse = 1;

				for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
				{
					switch (nStepParse)
					{
						case 1:
							if (tmpGSMMessage[nForText] == ':')
								nStepParse++;

							break;
						case 2:
							nStepParse++;
							nPosText = 0;

							break;
						case 3:
							simcardid[nPosText] = tmpGSMMessage[nForText];
							nPosText++;
							simcardid[nPosText] = '\0';

							break;
					} //switch (nStepParse)
				} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)

				if (strlen(simcardid) > 1)
				{
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "SIM Card ID '%s' detected on port %d\n", simcardid, EventContext->port);

					ast_copy_string(p->gsm_simcard_id, simcardid, sizeof(p->gsm_simcard_id));

					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMMESSAGE, sim card id '%s' detected on port %d\n", p->gsm_simcard_id, EventContext->port);
				}
			} //if (!strncmp(tmpGSMMessage, "^SCID:", 6))

			if (!strncmp(tmpGSMMessage, "^SCKS: 1,", 9))
			{
				simstatus = atoi(&tmpGSMMessage[9]);

				p->gsm_start_log = simstatus;

				if (simstatus == 0)
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMMESSAGE, sim card not detected on port %d\n", EventContext->port);

					dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] = 100;
					dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1] = 99;
				}
				else
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMMESSAGE, sim card detected on port %d\n", EventContext->port);

					if (dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] == 100)
					{
						dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] = 99;
						dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1] = 99;
					}
				}
			} //if (!strncmp(tmpGSMMessage, "^SCKS: 0,", 9))

			if (!strncmp(tmpGSMMessage, "+CSCA:", 6))
			{
				for (nForText = 0; nForText <= sizeof(sca) - 1; nForText++)
					sca[nForText] = '\0';

				nStepParse = 1;

				for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
				{
					switch (nStepParse)
					{
						case 1:
							if (tmpGSMMessage[nForText] == '"')
								nStepParse++;

							break;
						case 2:
							nStepParse++;
							nPosText = 0;

						case 3:
							if (tmpGSMMessage[nForText] != '"')
							{
								sca[nPosText] = tmpGSMMessage[nForText];
								nPosText++;
								sca[nPosText] = '\0';
							}
							else
								nStepParse++;

							break;
						case 4:
							break;
					} //switch (nStepParse)
				} //for (nForText = 0; nForText <= strlen(tmpGSMMessage) - 1; nForText++)

				if (strlen(sca) > 1)
				{
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "SMSC address '%s' received on port %d\n", sca, EventContext->port);

		       		manager_event(EVENT_FLAG_SYSTEM, "SMSC",
		                        				 	 "Group: %d\r\n"
		                        				 	 "Port: %d\r\n"
		                        					 "SCA: %s\r\n",
		                        					 p->group, p->port, sca
		                          );
				}
			} //if (!strncmp(tmpGSMMessage, "+CSCA:", 6))

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
			{
				ast_log(LOG_NOTICE, "EV_GSMMESSAGE '%s' on port %d\n", tmpGSMMessage, EventContext->port);

				if ((strncmp(tmpGSMMessage, ">", 1)) &&
					(strncmp(tmpGSMMessage, " ", 1)) &&
					(strncmp(tmpGSMMessage, "OK", 2)) &&
					(strncmp(tmpGSMMessage, "RING", 4)) &&
					(strncmp(tmpGSMMessage, "BUSY", 4)) &&
					(strncmp(tmpGSMMessage, "ATE0", 4)) &&
					(strncmp(tmpGSMMessage, "AT&F", 4)) &&
					(strncmp(tmpGSMMessage, "ERROR", 5)) &&
					(strncmp(tmpGSMMessage, "^SMGO", 5)) &&
					(strncmp(tmpGSMMessage, "+CDS:", 5)) &&
					(strncmp(tmpGSMMessage, "+CSQ:", 5)) &&
					(strncmp(tmpGSMMessage, "+CLIP:", 6)) &&
					(strncmp(tmpGSMMessage, "+CCWA:", 6)) &&
					(strncmp(tmpGSMMessage, "+CMTI:", 6)) &&
					(strncmp(tmpGSMMessage, "+CMGR:", 6)) &&
					(strncmp(tmpGSMMessage, "+CMGL:", 6)) &&
					(strncmp(tmpGSMMessage, "+CSMS:", 6)) &&
					(strncmp(tmpGSMMessage, "+CPMS:", 6)) &&
					(strncmp(tmpGSMMessage, "^SCKS:", 6)) &&
					(strncmp(tmpGSMMessage, "+CSCA:", 6)) &&
					(strncmp(tmpGSMMessage, "+CMGS:", 6)) &&
					(strncmp(tmpGSMMessage, "+COPS:", 6)) &&
					(strncmp(tmpGSMMessage, "^SCID:", 6)) &&
					(strncmp(tmpGSMMessage, "AT+CMGS", 7)) &&
					(strncmp(tmpGSMMessage, "AT+CPMS", 7)) &&
					(strncmp(tmpGSMMessage, "^SYSSTART", 9)) &&
					(strncmp(tmpGSMMessage, "NO CARRIER", 10)) &&
					(strncmp(tmpGSMMessage, "+CME ERROR", 10)) &&
					(strncmp(tmpGSMMessage, "+CMS ERROR", 10)) &&
					(strncmp(tmpGSMMessage, "NO DIALTONE", 11)) &&
					(strncmp(tmpGSMMessage, "+CPIN: READY", 12)) &&
					(strncmp(tmpGSMMessage, "NO DIAL TONE", 12)) &&
					(strncmp(tmpGSMMessage, "+CPIN: SIM PIN", 14)) &&
					(strncmp(tmpGSMMessage, "+CPIN: SIM PUK", 14)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-C PIN", 15)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-C PUK", 15)) &&
					(strncmp(tmpGSMMessage, "+CPIN: SIM PIN2", 15)) &&
					(strncmp(tmpGSMMessage, "+CPIN: SIM PUK2", 15)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-NS PIN", 16)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-NS PUK", 16)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-SP PIN", 16)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-SP PUK", 16)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-NET PUK", 17)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-SIM PIN", 17)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-SIM PUK", 17)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-FSIM PIN", 18)) &&
					(strncmp(tmpGSMMessage, "+CPIN: PH-FSIM PUK", 18)))
						ast_log(LOG_WARNING, "EV_GSMMESSAGE '%s', WRONG MESSAGE or SMS Text received on port %d\n", tmpGSMMessage, EventContext->port);
			}

			break;
		case EV_GSMSMSSENT:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSSENT, receiving response on port %d\n", EventContext->port);

			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->gsmsendsms_timeout_sched > -1)
				{
					res_sched = ast_sched_del(sched, p->gsmsendsms_timeout_sched);
					if (res_sched == -1)
						ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmsendsms_timeout_sched = -1;
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			dg_GSMGetMessage(EventContext->port, tmpGSMMessage);

			for (nForText = 0; nForText <= sizeof(message_reference) - 1; nForText++)
				message_reference[nForText] = '\0';

			nPosText = 0;

			for (nForText = 6; nForText <= strlen(tmpGSMMessage) - 1; nForText++)
			{
				message_reference[nPosText] = tmpGSMMessage[nForText];
				nPosText++;
				message_reference[nPosText] = '\0';
				if (tmpGSMMessage[nForText] == ',')
					break;
			}

			p->sms_send_messagereference = atoi(message_reference);

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                ast_log(LOG_NOTICE, "EV_GSMSMSSENT, Message Reference (MR) '%s'/'%d' on port %d\n", message_reference, p->sms_send_messagereference, EventContext->port);
			
       		if (option_verbose >= 3)
       			ast_verbose(VERBOSE_PREFIX_3 "SMS message sent with success on port %d\n", EventContext->port);

       		manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
                        				 	 "SendTo: %s\r\n"
                        				 	 "Group: %d\r\n"
                        					 "Port: %d\r\n"
                        					 "SIMCardID: %s\r\n"
                        					 "Status: %s\r\n"
                        					 "MessageID: %d\r\n"
                        					 "CustomID: %s\r\n",
                        					 p->sms_send_to, p->group, p->port, p->gsm_simcard_id, "OK", p->sms_send_messagereference, p->sms_send_custom_id
                          );

			p->sms_reserved = 0;
			p->sms_send_to[0] = '\0';
			/* p->sms_send_messagereference = -1; *** Do not do it, we need to provide this MR to console events *** */
			p->sms_send_custom_id[0] = '\0';
			p->sms_sent_status = 1;

			update_statistics_sms(p);

			break;
		case EV_GSMRETURNOK:
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "EV_GSMRETURNOK '%d' on port %d\n", EventContext->data, EventContext->port);

			if (EventContext->data == GSM_CLEAR)
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GSMRETURNOK '%s' on port %d\n", "GSM_CLEAR", EventContext->port);

				ast_dgv_lock(&p->lock, p->port);
				{
					if (p->reserved == 2)
					{
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "Read and Clear All SMS process finished, now port %d is no long reserved\n", EventContext->port);

						p->reserved = 0;
					}
				}
				ast_dgv_unlock(&p->lock, p->port);
			}

			break;
		case EV_GSMREADY:
           	if (EventContext->data == 1)
           	{
           		p->gsm_start_log = 1;

           		switch (p->alarmed)
           		{
           			case 1:
                   		if (option_verbose >= 3)
                   		{
                   			ast_log(LOG_NOTICE, "EV_GSMREADY on port %d with data: MODULE RECONFIGURED OK\n", EventContext->port);
                   		
                   			ast_log(LOG_NOTICE, "Requesting alarm notifications... \n\n");
                   		}

    					ret = dg_GSMCheckSignalQuality(EventContext->port);
    					if (ret != DG_EXIT_SUCCESS)
    						ast_log(LOG_NOTICE, "Error requesting alarm notifications port %d - error code %x\n", EventContext->port, ret);
    					else
    					{
    						if (p->gsm_getsignal_thread_id == 0)
    						{
								pthread_attr_init(&attr);
								pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
								if (ast_pthread_create(&p->gsm_getsignal_thread_id, &attr, gsm_getsignal_thread, p)>=0)
								{
									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "gsm_getsignal_thread Created - port %d\n", p->port);
								}
								else
									ast_log(LOG_ERROR, "Error creating gsm_getsignal_thread - port %d\n", p->port);
								pthread_attr_destroy(&attr);
    						}
    						else
								if (option_verbose >= 3)
									ast_log(LOG_NOTICE, "gsm_getsignal_thread was already created - port %d\n", p->port);
    					}

    					if (p->gsmclearsms_timeout_sched > -1)
    					{
    						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    							ast_log(LOG_NOTICE, "EV_GSMREADY (%d) - deleting gsmclearsms schedule\n", p->port);

    						res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
    						if (res_sched == -1)
    							ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

    						p->gsmclearsms_timeout_sched = -1;
    					}

    					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    						ast_log(LOG_NOTICE, "EV_GSMREADY (%d) - Creating gsmclearsms schedule\n", p->port);

    					p->gsmclearsms_timeout_sched = ast_sched_add(sched, gsmclearsmstimeout, gsmclearsms_timeout_callback, p);

           				break;
           			case 0://Failsafe (Unexpected)
           			case 99://Failsafe (Unexpected)
           			case 2:
                    	set_gsm_conf(p);
                    	
                   		if (option_verbose >= 3)
                   			ast_log(LOG_NOTICE, "EV_GSMREADY on port %d with data: MODULE START OK\n", EventContext->port);
                   		
                   		break;           		
           		}
           	}
           	else
           	{
           		if ((p->gsm_start_log) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
           			ast_log(LOG_WARNING, "EV_GSMREADY on port %d with data: MODULE START ERROR, do not use this port!!!\n", EventContext->port);

           		p->reserved = /* p->sms_reserved = */ 1;

           		if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMREADY (%d) - deleting gsmerror schedule\n", p->port);

					res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
			    	if (res_sched == -1)
			    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmerror_timeout_sched = -1;
				}

				p->alarmed=99;

				//set gsmerror timeout
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GSMREADY (%d) - Creating gsmerror schedule\n", p->port);

				p->gsmerror_timeout_sched = ast_sched_add(sched, gsmerrortimeout, gsmerror_timeout_callback, p);

				if (p->gsmclearsms_timeout_sched > -1)
				{
					res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
			    	if (res_sched == -1)
			    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

					p->gsmclearsms_timeout_sched = -1;
				}

				dg_GSMSendCommand(p->port, DEFAULT_GSM_REQ_SIMSTATUS);
           	}

            break;
		case EV_GSMSIGNALQUALITY:
			ast_dgv_lock(&p->lock, p->port);
			{
				if (p->reserved == 2)
				{
					//probably a +CME ERROR: SIM busy during clear SMS
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_WARNING, "Ignoring EV_GSMSIGNALQUALITY during clear SMS, it is probably a +CME ERROR: SIM busy! (port %d, alarmed %d, reserved %d, sinal0 %d, sinal1 %d)\n",
								p->port,
								p->alarmed,
								p->reserved,
								dgv_gsmsignal[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1][0],
								dgv_gsmsignal[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1][1]);

					ast_dgv_unlock(&p->lock, p->port);
					break;
				}
			}
			ast_dgv_unlock(&p->lock, p->port);

			if (p->gsmgetsignal_timeout_sched > -1)
			{
				res_sched = ast_sched_del(sched, p->gsmgetsignal_timeout_sched);
		    	if (res_sched == -1)
		    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

				p->gsmgetsignal_timeout_sched = -1;
			}

			dg_GSMGetCallQuality(EventContext->port, tmpGSMMessage);

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "EV_GSMSIGNALQUALITY (CallQuality) '%s' on port %d\n", tmpGSMMessage, EventContext->port);

			sscanf(tmpGSMMessage, "%d", &dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1]);

			dg_GSMGetSignalQuality(EventContext->port, tmpGSMMessage);
			
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "EV_GSMSIGNALQUALITY (SignalQuality) '%s' on port %d\n", tmpGSMMessage, EventContext->port);
			
			sscanf(tmpGSMMessage, "%d", &tmpGSMSignal);

			if (tmpGSMSignal == 99)
			{
	            ast_dgv_lock(&p->lock, p->port);
	            {
					if (p->sms_reserved)
					{
						p->sms_reserved = 0;
						p->sms_send_to[0] = '\0';
						p->sms_send_messagereference = -1;
						p->sms_send_custom_id[0] = '\0';
						p->sms_sent_status = 0;
					}

					if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "EV_GSMSIGNALQUALITY (%d) - deleting gsmerror schedule\n", p->port);

						res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->gsmerror_timeout_sched = -1;
					}

					p->alarmed=99;

					//set gsmerror timeout
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMSIGNALQUALITY (%d) - Creating gsmerror schedule\n", p->port);

					p->gsmerror_timeout_sched = ast_sched_add(sched, gsmerrortimeout, gsmerror_timeout_callback, p);

					if (p->gsmclearsms_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->gsmclearsms_timeout_sched = -1;
					}
	            }
	            ast_dgv_unlock(&p->lock, p->port);
			}
			else
			{
				dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] = tmpGSMSignal;

				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_log(LOG_NOTICE, "EV_GSMSIGNALQUALITY CallQuality %d and SignalQuality %d on port %d\n", dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1], dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0], EventContext->port);

				if (p->alarmed != 0)
					dg_GSMSendCommand(p->port, DEFAULT_GSM_REQ_OPERATOR);

				p->alarmed = 0;
			}

			break;
		case EV_GSMSIM:
			ast_dgv_lock(&p->lock, p->port);
			{
				if (EventContext->data == 1)
				{
					if (option_verbose >= 3)
						ast_log(LOG_NOTICE, "EV_GSMSIM received, new SIM card detected on port %d\n", EventContext->port);

					dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] = 0;
					dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1] = 99;
				}
				else
				{
					if (p->alarmed == 0)
					{
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "EV_GSMSIM received, SIM card removed on port %d\n", EventContext->port);

						dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][0] = 100;
						dgv_gsmsignal[(dg_GetCardNumber(EventContext->port)) - 1][(dg_GetRelativeChannelNumber(EventContext->port)) - 1][1] = 99;
					}
				}

				p->alarmed = 0;

				if (p->sms_reserved)
				{
					p->sms_reserved = 0;
					p->sms_send_to[0] = '\0';
					p->sms_send_messagereference = -1;
					p->sms_send_custom_id[0] = '\0';
					p->sms_sent_status = 0;
				}

				if (p->alarmed == 0)
				{
					if (p->gsmerror_timeout_sched > -1)//stop gsmerror timeout
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "EV_GSMSIM (%d) - deleting gsmerror schedule\n", p->port);

						res_sched = ast_sched_del(sched, p->gsmerror_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->gsmerror_timeout_sched = -1;
					}

					p->alarmed=99;

					//set gsmerror timeout
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "EV_GSMSIM (%d) - Creating gsmerror schedule\n", p->port);

					p->gsmerror_timeout_sched = ast_sched_add(sched, gsmsimdetectedtimeout, gsmerror_timeout_callback, p);

					if (p->gsmclearsms_timeout_sched > -1)
					{
						res_sched = ast_sched_del(sched, p->gsmclearsms_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->gsmclearsms_timeout_sched = -1;
					}
				} //if (p->alarmed == 0)
			}
			ast_dgv_unlock(&p->lock, p->port);

			break;
	}
}

//-------------------------------------------------------------------------------------------
// Locate channel passing digivoice port
//-------------------------------------------------------------------------------------------
static struct dgv_pvt *find_ch_port(short port)
{
	struct dgv_pvt *tmp;

	if (!iflast)
		return NULL;

	tmp = iffirst; /* beginning of list */
	while (tmp)
	{
		//specific port
		if (tmp->port == port)
			break;

		tmp = tmp->next;
	}

	return tmp;
}

//-------------------------------------------------------------------------------------------
// Find avaiable port on digivoice port list
//-------------------------------------------------------------------------------------------
static struct dgv_pvt *find_free_port(int group_port[], short is_group[], short backwards[], short dc, short use_roundrobin[], short use_serialnumber[], short use_serialnumber_port[], short use_talktime[], short use_serialnumber_talktime[], short find_sms_reserved)
{
	struct dgv_pvt *tmp, *ftmp, *tttmp = NULL;
	short nForGroupPort;

	if (!iflast)
	{
		if (option_debug == 20)
			ast_log(LOG_NOTICE, "find_free_port: !iflast(first group_port %d, is_group %d, backwards %d, dc %d, use_roundrobin %d, use_serialnumber %d, use_talktime %d and find_sms_reserved %d\n",
					group_port[0], is_group[0], backwards[0], dc, use_roundrobin[0], use_serialnumber[0], use_talktime[0], find_sms_reserved);
		return NULL;
	}

	for (nForGroupPort = 0; nForGroupPort <= NUM_MAXDIALOPT - 1; nForGroupPort++)
	{
		if (group_port[nForGroupPort] == -1)
			break;

		if (is_group[nForGroupPort] == 1)
		{
			if (use_serialnumber[nForGroupPort] == 1)
			{
				if (option_debug == 20)
					ast_log(LOG_NOTICE, "find_free_port: finding free port on serial_number %05d\n", group_port[nForGroupPort]);

				if (backwards[nForGroupPort])
					tmp = iflast; /* get top of list */
				else
					tmp = iffirst; /* beginning of list */
			}
			else
				if (use_roundrobin[nForGroupPort] == 0)
				{
					if (option_debug == 20)
						ast_log(LOG_NOTICE, "find_free_port: finding free port on group %d\n", group_port[nForGroupPort]);

					if (backwards[nForGroupPort])
						tmp = iflast; /* get top of list */
					else
						tmp = iffirst; /* beginning of list */
				}
				else //if (use_roundrobin[nForGroupPort] == 0)
				{
					if (option_debug == 20)
						ast_log(LOG_NOTICE, "find_free_port: finding roundrobin free port on group %d\n", group_port[nForGroupPort]);

					if (backwards[nForGroupPort])
					{
						tmp = round_robin[group_port[nForGroupPort]] ? round_robin[group_port[nForGroupPort]]->previous : iflast;
						if (!tmp)
						{
							tmp = iflast;
							if (option_debug == 20)
								ast_log(LOG_NOTICE, "find_free_port: !tmp, roundrobin and backwards, now tmp->port = %d\n", tmp->port);
						}
						else
						{
							if (group_port[nForGroupPort] > 0)
								while (tmp)
								{
									if (tmp->group == group_port[nForGroupPort])
										break;

									if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
										ast_log(LOG_NOTICE, "find_free_port: my group %d is != of group %d, roundrobin and backwards, now tmp->port = %d\n", group_port[nForGroupPort], tmp->group, tmp->port);

									tmp = tmp->previous;

									if (!tmp)
									{
										tmp = iflast;
										break;
									}
								}
						}
					}
					else
					{
						tmp = round_robin[group_port[nForGroupPort]] ? round_robin[group_port[nForGroupPort]]->next : iffirst;
						if (!tmp)
						{
							tmp = iffirst;
							if (option_debug == 20)
								ast_log(LOG_NOTICE, "find_free_port: !tmp, roundrobin and !backwards, now tmp->port = %d\n", tmp->port);
						}
						else
						{
							if (group_port[nForGroupPort] > 0)
								while (tmp)
								{
									if (tmp->group == group_port[nForGroupPort])
										break;

									if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
										ast_log(LOG_NOTICE, "find_free_port: my group %d is != of group %d, roundrobin and !backwards, now tmp->port = %d\n", group_port[nForGroupPort], tmp->group, tmp->port);

									tmp = tmp->next;

									if (!tmp)
									{
										tmp = iffirst;
										break;
									}
								}
						}
					}
				} //if (use_roundrobin[nForGroupPort] == 0)
		}
		else //if (is_group[nForGroupPort] == 1)
		{
			if (option_debug == 20)
				ast_log(LOG_NOTICE, "find_free_port: Verifying if port %d is free\n", group_port[nForGroupPort]);

            tmp = find_ch_port(group_port[nForGroupPort]);
            if (tmp)
            {
            	if (((tmp->ast_ch==NULL && !tmp->reserved && !find_sms_reserved) || (find_sms_reserved)) && !tmp->alarmed && !tmp->locked && !tmp->sms_reserved)
            	{
					if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
						ast_log(LOG_NOTICE, "Channel allocated port=%d\n", tmp->port);

					return tmp;
            	}
    			else
    			{
    				if ( (tmp->sig == DG_SIG_FXS || tmp->sig == DG_SIG_CB_FXS) && (tmp->second_call) && (!tmp->subchannel) && (tmp->ast_ch) )
    				{
        				if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
      						ast_log(LOG_NOTICE, "Port=%d already reserved, but we will try to use second_call\n", tmp->port);
    					return tmp;
    				}
    				else
    				{
						if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
						{
							if (!find_sms_reserved)
								ast_log(LOG_NOTICE, "Port=%d already reserved\n", tmp->port);
							else
								ast_log(LOG_NOTICE, "SMS_Port=%d already reserved\n", tmp->port);
						}

						tmp = NULL;
						continue;
    				}
    			}
            }
            else
            {
            	tmp = NULL;
            	continue;
            }
		} //if (is_group[nForGroupPort] == 1)

		ftmp=tmp;

		while (tmp)
		{
			if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
			{
				if (use_serialnumber[nForGroupPort] == 0)
					ast_log(LOG_NOTICE, "find_free_port: first step(port %d at group %d)\n", tmp->port, group_port[nForGroupPort]);
				else
					ast_log(LOG_NOTICE, "find_free_port: first step(port %d at serial_number %05d)\n", tmp->port, group_port[nForGroupPort]);
			}

			/* check free port */
			//if (tmp->ast_ch==NULL && !tmp->reserved) //not reserved
			//if (((tmp->ast_ch==NULL && !tmp->reserved && !find_sms_reserved) || (!tmp->sms_reserved && find_sms_reserved)) && !tmp->alarmed && !tmp->locked)
			if (((tmp->ast_ch==NULL && !tmp->reserved && !find_sms_reserved) || (find_sms_reserved)) && !tmp->alarmed && !tmp->locked && !tmp->sms_reserved)
			{
				if (dc<0)
				{
					//default search, without libpri
					if (group_port[nForGroupPort]>0)
					{
						//group
						if (use_serialnumber[nForGroupPort] == 0)
						{
							if (tmp->group == group_port[nForGroupPort])
							{
								if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
									ast_log(LOG_NOTICE, "Channel allocated port=%d group=%d\n",tmp->port, tmp->group);

								if (use_roundrobin[nForGroupPort])
								{
									round_robin[group_port[nForGroupPort]] = tmp;
									if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
										ast_log(LOG_NOTICE, "find_free_port: now round_robin[%d]->port = %d\n", group_port[nForGroupPort], round_robin[group_port[nForGroupPort]]->port);
								}

								if (use_talktime[nForGroupPort])
								{
									if (!tttmp)
										tttmp = tmp;

									if (backwards[nForGroupPort])
									{
										if (((!find_sms_reserved) && (tmp->statistics.total_time.tv_sec > tttmp->statistics.total_time.tv_sec)) ||
											(( find_sms_reserved) && (tmp->statistics.sms_sent > tttmp->statistics.sms_sent)))
										{
											tttmp = tmp;

											if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
												ast_log(LOG_NOTICE, "find_free_port: now tttmp->port = %d\n", tttmp->port);
										}
									}
									else
									{
										if (((!find_sms_reserved) && (tmp->statistics.total_time.tv_sec < tttmp->statistics.total_time.tv_sec)) ||
											(( find_sms_reserved) && (tmp->statistics.sms_sent < tttmp->statistics.sms_sent)))
										{
											tttmp = tmp;

											if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
												ast_log(LOG_NOTICE, "find_free_port: now tttmp->port = %d\n", tttmp->port);
										}
									}
								}
								else
									return tmp;
							}
						}
						else
						{
							if (tmp->serial_num == group_port[nForGroupPort] && use_serialnumber_port[nForGroupPort] != 0)
							{
								if (tmp->ch_board == use_serialnumber_port[nForGroupPort])
								{
									if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
										ast_log(LOG_NOTICE, "Channel allocated port=%d serial_number=%05d and serial_number_port=%d\n", tmp->port, tmp->serial_num, use_serialnumber_port[nForGroupPort]);

									return tmp;
								}
							}
							else
							{
								if (tmp->serial_num == group_port[nForGroupPort] && use_serialnumber_talktime[nForGroupPort] != 0)
								{
									if (!tttmp)
										tttmp = tmp;

									if (use_serialnumber_talktime[nForGroupPort] == 2) //like T
									{
										if (((!find_sms_reserved) && (tmp->statistics.total_time.tv_sec > tttmp->statistics.total_time.tv_sec)) ||
											(( find_sms_reserved) && (tmp->statistics.sms_sent > tttmp->statistics.sms_sent)))
										{
											tttmp = tmp;

											if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
												ast_log(LOG_NOTICE, "find_free_port: now tttmp->port = %d\n", tttmp->port);
										}
									}
									else //like t
									{
										if (((!find_sms_reserved) && (tmp->statistics.total_time.tv_sec < tttmp->statistics.total_time.tv_sec)) ||
											(( find_sms_reserved) && (tmp->statistics.sms_sent < tttmp->statistics.sms_sent)))
										{
											tttmp = tmp;

											if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
												ast_log(LOG_NOTICE, "find_free_port: now tttmp->port = %d\n", tttmp->port);
										}
									}
								}
								else
									if (tmp->serial_num == group_port[nForGroupPort])
									{
										if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
											ast_log(LOG_NOTICE, "Channel allocated port=%d serial_number=%05d\n", tmp->port, tmp->serial_num);

										return tmp;
									}
							}
						}
					}
					else
					{
						//any port
						if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
							ast_log(LOG_NOTICE, "Channel allocated port=%d\n",tmp->port);

						return tmp;
					}
				}
				else
				{
#ifdef USE_LIBPRI
						//libpri specific search
						if (dchans[dc].card == tmp->board)
						{
							if (dchans[dc].span == 1 && tmp->ch_board<=30)
								return tmp;
							else
								if (dchans[dc].span == 2 && tmp->ch_board>30)
									return tmp;
						}
#endif//#ifdef USE_LIBPRI
				}
			}
			else
			{
				if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
				{
					if (!find_sms_reserved)
						ast_log(LOG_NOTICE, "Port=%d already reserved\n", tmp->port);
					else
						ast_log(LOG_NOTICE, "SMS_Port=%d already reserved\n", tmp->port);
				}
			}

			if (backwards[nForGroupPort])
			{
				tmp = tmp->previous;

				//Fix rotating on round-robin (By DB)
				if (!tmp)
				{
					if ((use_talktime[nForGroupPort]) || (use_serialnumber_talktime[nForGroupPort] != 0))
						return tttmp;

					tmp = iflast;
				}
			}
			else
			{
				tmp = tmp->next;

				//Fix rotating on round-robin (By DB)
				if (!tmp)
				{
					if ((use_talktime[nForGroupPort]) || (use_serialnumber_talktime[nForGroupPort] != 0))
						return tttmp;

					tmp = iffirst;
				}
			}

			//Fix rotating on round-robin (By DB)
			if (tmp == ftmp)
				tmp = NULL;

		} //while (tmp)
	} //for (nForGroupPort = 0; nForGroupPort <= NUM_MAXDIALOPT - 1; nForGroupPort++)

	if ((use_talktime[nForGroupPort]) || (use_serialnumber_talktime[nForGroupPort] != 0))
		return tttmp;
	else
		return tmp;
}

//-------------------------------------------------------------------------------------------
// Find avaiable smschannel port
//-------------------------------------------------------------------------------------------
static struct dgv_pvt *find_free_sms_channel(struct dgv_pvt *tmp)
{
	struct dgv_pvt *sms_tmp;

	if (tmp)
	{
		if (!tmp->iflastsmschannel)
		{
			if (option_debug == 20)
				ast_log(LOG_NOTICE, "find_free_sms_channel: !tmp->iflastsmschannel(port %d)\n", tmp->port);
			return NULL;
		}

		if (option_debug == 20)
			ast_log(LOG_NOTICE, "find_free_sms_channel: finding free sms channel(port %d)\n", tmp->port);

		sms_tmp = tmp->iffirstsmschannel; /* beginning of list */

	    while (sms_tmp)
	    {
	    	if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
	    		ast_log(LOG_NOTICE, "find_free_sms_channel: first step(p(%d)->smschannel[%d])\n", tmp->port, sms_tmp->port);
	    	
	        /* check free port */
	        if (sms_tmp->ast_ch==NULL && !sms_tmp->sms_reserved) //not reserved        
	        {
	        	if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
	        		ast_log(LOG_NOTICE, "Channel allocated p(%d)->smschannel[%d]\n", tmp->port, sms_tmp->port);	
	            break;
	        }
	        else
	        {
	            if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
	           		ast_log(LOG_NOTICE, "p(%d)->smschannel[%d] already reserved\n", tmp->port, sms_tmp->port);
	        }

	        sms_tmp = sms_tmp->next;
	    } //while (sms_tmp)
	}
	else //if (tmp)
	{
		ast_log(LOG_WARNING, "find_free_sms_channel: Invalid port...\n");
		return NULL;
	} //if (tmp)

	return sms_tmp;
}

//-------------------------------------------------------------------------------------------
// Create private structure for each channel
//-------------------------------------------------------------------------------------------
static struct dgv_pvt *initialize_ports(short port)
{
	struct dgv_pvt *tmp;
	int nForSMSChannel;

	tmp = (struct dgv_pvt *)calloc(1, sizeof *tmp);
	
    dg_signal_e1_thread_structure *p_e1_thread_structure;//pointer to voicerlib e1 thread structure

	if (!tmp)
		return NULL;

	ast_dgv_lock(&iflock, 0);
	{
		tmp->board = dg_GetCardNumber(port);
		tmp->serial_num = 0;
		tmp->firmware_vr_board = -1;
		tmp->lec_count_board = -1;
		tmp->irq_num = 0;
		tmp->ch_board = dg_GetRelativeChannelNumber(port);
		tmp->group = -1;	//default value - no group
		tmp->port = port;
		tmp->vconf_port_bridged = 0;
		tmp->write_last_format=-1;		//initializes with invalid format
		tmp->read_last_format=-1;
		tmp->ast_ch = NULL;
		tmp->subchannel = NULL;
		tmp->channel_index = 1;
		tmp->subchannel_audio_index = 0;
		tmp->tone_thread_id = 0;			//control thread to ringback and busyback tones
		tmp->stuttertone_thread_id = 0;   	//control thread to stutter dial tone
		tmp->subchanneltone_thread_id = 0;  //control thread to subchannel ring tone
		tmp->beeptone_thread_id = 0;        //control thread to beep tone before exchange call
		tmp->samples_rx_count = 0;		    //counter for received samples

        tmp->force_drop_rx = 0;
        tmp->force_drop_tx = 0;

		//Assume default values to avoid null strings and unsetted values 
		tmp->category = 1;
		tmp->category_pass_through = 0;
		tmp->max_digits_rx = 4;
		tmp->tx_gain = -6;
		tmp->rx_gain = 0;
		tmp->ask_for_cid_position = 2;
		tmp->echo_can = 2;  //64taps
		tmp->echo_type = ECHO_HW_DSP;
		tmp->echo_training = 800;
        tmp->echo_nlp = 1;
        tmp->drop_tx_samples_factor = TX_CUT_FACTOR;
        tmp->relaxdtmf=1;
        tmp->gen_ringback = 1;
        tmp->gen_busy = 1;
        tmp->gen_dtmf = 2;
        tmp->dtmf_to_asterisk = 2;
        tmp->b_subscriber_hangup=0;
        tmp->ccb_pass_through=0;
        tmp->auto_dgccb=0;
        tmp->deny_prefix[0]='\0';
        
		p_e1_thread_structure = dg_GetE1ThreadPointer(tmp->port);

		if (p_e1_thread_structure)
			tmp->group_b_value = p_e1_thread_structure->b_free_calling;
		else
		{
			ast_log(LOG_WARNING, "initialize_ports: p_e1_thread_structure does not exist on port %d\n", tmp->port);
			tmp->group_b_value = MFC_B_1;/* MFC_B_1 = default Brazil, old = B_FREE_CALLING */
		}

        tmp->r2_country = R2_COUNTRY_BR;//default Brazil
        tmp->silence_threshold = -18;
        tmp->dialtone_timeout = 3000;
        tmp->wait_dialtone = 1;
        tmp->fax_detection = 1;
        tmp->polarity = 2;
        tmp->cid_detection_type = 1;
        
    	//ids strings
        tmp->exten[0]        = '\0';
        tmp->rdnis[0]        = '\0';
        tmp->dnid[0]         = '\0';
        tmp->cid_num[0]      = '\0';
        tmp->cid_name[0]     = '\0';
        tmp->pri_cid_num[0]  = '\0';
        tmp->pri_cid_name[0] = '\0';
        tmp->cid_category[0] = '\0';

        //fxs
        tmp->ring_type           = 1;
        tmp->flash_min_time      = 80;
        tmp->flash_max_time      = 650;
        tmp->immediate           = 0;//fxs and pri channels
        tmp->fxss_thread_running = 0;
        tmp->fv_thread_timeout   = tmp->dialtone_timeout + 1000;
        tmp->need_fv_thread      = 0;
        tmp->need_sc_ring        = 0;
        tmp->need_ringback       = 0;        
        
        //gsm
        tmp->gsm_getsignal_thread_id = 0;
        ast_copy_string(tmp->gsm_operator_name, "Unknown", sizeof(tmp->gsm_operator_name));
        ast_copy_string(tmp->gsm_simcard_id, "Unknown", sizeof(tmp->gsm_simcard_id));
        tmp->gsm_hide_number = 0;
        tmp->gsm_sms_confirmation = 0;
        tmp->gsm_sms_flashsms = 0;
        tmp->gsm_displaycallwaiting = 1;
        tmp->gsm_pin_number[0] = '\0';
        tmp->gsm_need_reload = 0;
        tmp->gsm_start_log = 1;
        tmp->sms_sent_tmpMessageError[0] = '\0';
        tmp->smschannel_index = 0;
        for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
        	tmp->smschannel[nForSMSChannel] = NULL;

        tmp->sms_received_from[0]      = '\0';
        tmp->sms_received_simcardid[0] = '\0';
        tmp->sms_received_date[0]      = '\0';
        tmp->sms_received_time[0]      = '\0';
        tmp->sms_received_text[0]      = '\0';
        tmp->sms_received_st_cod[0]    = '\0';
        tmp->sms_received_mr[0]        = '\0';
        tmp->ussd_received_text[0]     = '\0';

        ast_copy_string(tmp->mailbox, "1234", sizeof(tmp->mailbox));
        ast_copy_string(tmp->callprogress, dgv_conf.szDefaultCallProgressFile, sizeof(tmp->callprogress));
        ast_copy_string(tmp->szPortID, "1122334455", sizeof(tmp->szPortID));
        ast_copy_string(tmp->szPortName, "none", sizeof(tmp->szPortName));
        ast_copy_string(tmp->context, "default", sizeof(tmp->context));
        ast_copy_string(tmp->accountcode, "none", sizeof(tmp->accountcode));

		//gain
		dg_SetPortGain(tmp->port, PLAY_GAIN, tmp->tx_gain);
        //set rec gain instead of rx_gain to avoid affect echo and agc algorythms
		dg_SetPortGain(tmp->port, REC_GAIN, tmp->rx_gain);
        //RX Gain is always zero
        //dg_SetPortGain(tmp->port, RX_GAIN, 0);//removed on 1.0.8_rc3
        //dg_SetPortGain(tmp->port, TX_GAIN, 0);//removed on 1.0.8_rc3
		
		//E1 specific configuration
		dg_SetPortId(tmp->port, tmp->szPortID);

		//mutexes
		ast_mutex_init(&tmp->lock);
	  	ast_mutex_init(&tmp->play_lock);
		ast_mutex_init(&tmp->record_lock);

	}
	ast_dgv_unlock(&iflock, 0);

	return tmp;
}

//-------------------------------------------------------------------------------------------
// Try to alloc Asterisk channel to desired port
//-------------------------------------------------------------------------------------------
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_1_8
		static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability, const char *linkedid)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
		static struct ast_channel *dgv_new(struct dgv_pvt *i, int state, char *context, short startpbx, int transfercapability)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
{
	struct ast_channel *tmp;
    //int flags;
	char szSMSChannel[10];
	short nForText;

#ifdef CC_PRI_HAS_REVERSECHARGE
		char szReverseChargingReceived[3];
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE

	for (nForText = 0; nForText <= sizeof(szSMSChannel) - 1; nForText++)
		szSMSChannel[nForText] = '\0';

	if ((i->sig == DG_SIG_GSM) && (i->channel_index == 2))
		sprintf(szSMSChannel, "%s%d", DEFAULT_SMS_CHANNEL, i->smschannel_index + 1);

#ifdef CC_AST_HAS_EXT_CHAN_ALLOC
    tmp = ast_channel_alloc(1, state, i->cid_num, i->cid_name,
#ifdef CC_AST_HAS_EXT2_CHAN_ALLOC
    	i->accountcode, i->dnid, i->context,
		#ifdef CC_AST_HAS_VERSION_1_8
			#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    			assignedids, requestor,
    		#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
    			linkedid,
			#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
		#endif//#ifdef CC_AST_HAS_VERSION_1_8
    	i->amaflags,
#endif//#ifdef CC_AST_HAS_EXT2_CHAN_ALLOC
	"%s/%d%s", ttype, i->port, szSMSChannel);
#else//#ifdef CC_AST_HAS_EXT_CHAN_ALLOC
	tmp = ast_channel_alloc(1);
#endif//#ifdef CC_AST_HAS_EXT_CHAN_ALLOC

	if (tmp)
    {
		dgv_channel_tech_set(tmp, &dgv_tech);
#if !defined(CC_AST_HAS_VERSION_1_4) && !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)
        	snprintf(dgv_channel_name(tmp), sizeof(tmp->name), "%s/%d%s", ttype, i->port, szSMSChannel);
#endif//#if !defined(CC_AST_HAS_VERSION_1_4) && !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)
        
        //..asterisk new versions ast trunk ast_string_field_build(tmp, tmp->name, "%s/%d", ttype, i->port);
        if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
            ast_log(LOG_NOTICE, "Trying to allocate %s\n", dgv_channel_name(tmp));
   
        /* lock physical port storing channel pointer into pvt structure */
        i->ast_ch = tmp;
        dgv_channel_tech_pvt_set(tmp, i);
        ast_copy_string(i->dev, dgv_channel_name(tmp), sizeof(i->dev));

        if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
#ifdef DEBUG_PRI
				ast_log(LOG_NOTICE, ">>>>>>>>dgv_new %s - address=%x\n", dgv_channel_name(tmp), (int)tmp);
#else//#ifdef DEBUG_PRI
				ast_log(LOG_NOTICE, ">>>>>>>>dgv_new %s - address=XX\n", dgv_channel_name(tmp)          );
#endif//#ifdef DEBUG_PRI

		switch (i->sig)
		{
			case DG_SIG_R2_NDIS_FIXED:
			case DG_SIG_R2_NDIS_VARIABLE:
			case DG_SIG_CUSTOMCAS:
			case DG_SIG_CB_FXO:
			case DG_SIG_CB_FXS:
				if (!ast_strlen_zero(i->cid_category))
				{
					pbx_builtin_setvar_helper(tmp, "DGV_CATEGORY_RECEIVED", i->cid_category);
					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): Category Received - '%s', use DGV_CATEGORY_RECEIVED variable to read it!\n", i->port, i->cid_category);
				}
				break;
			case DG_SIG_GSM:
				if (!ast_strlen_zero(i->sms_received_from))
				{
					pbx_builtin_setvar_helper(tmp, "DGV_SMS_FROM", i->sms_received_from);
					pbx_builtin_setvar_helper(tmp, "DGV_SMS_SIMCARDID", i->sms_received_simcardid);
					pbx_builtin_setvar_helper(tmp, "DGV_SMS_DATE", i->sms_received_date);
					pbx_builtin_setvar_helper(tmp, "DGV_SMS_TIME", i->sms_received_time);
		        	pbx_builtin_setvar_helper(tmp, "DGV_SMS_TEXT", i->sms_received_text);
					if (option_verbose >= 3)
					{
						if (ast_strlen_zero(i->sms_received_st_cod))
							ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Received, use DGV_SMS_FROM, DGV_SMS_SIMCARDID, DGV_SMS_DATE, DGV_SMS_TIME and DGV_SMS_TEXT variables to read it!\n", i->port);
						else
						{
							if (ast_strlen_zero(i->sms_received_mr))
							{
								pbx_builtin_setvar_helper(tmp, "DGV_SMS_STCOD", i->sms_received_st_cod);
								ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Received, use DGV_SMS_FROM, DGV_SMS_SIMCARDID, DGV_SMS_DATE, DGV_SMS_TIME, DGV_SMS_TEXT and DGV_SMS_STCOD variables to read it!\n", i->port);
							}
							else
							{
								pbx_builtin_setvar_helper(tmp, "DGV_SMS_STCOD", i->sms_received_st_cod);
								pbx_builtin_setvar_helper(tmp, "DGV_SMS_MESSAGEID", i->sms_received_mr);
								ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Received, use DGV_SMS_FROM, DGV_SMS_SIMCARDID, DGV_SMS_DATE, DGV_SMS_TIME, DGV_SMS_TEXT, DGV_SMS_STCOD and DGV_SMS_MESSAGEID variables to read it!\n", i->port);
							}
						}
					}
				}
				else
				{
					if (!ast_strlen_zero(i->ussd_received_text))
					{
						pbx_builtin_setvar_helper(tmp, "DGV_USSD_SIMCARDID", i->sms_received_simcardid);
			        	pbx_builtin_setvar_helper(tmp, "DGV_USSD_TEXT", i->ussd_received_text);
						if (option_verbose >= 3)
							ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): USSD Received, use DGV_USSD_SIMCARDID and DGV_USSD_TEXT variables to read it!\n", i->port);
					}
				}
				break;
			case DG_SIG_PRI:
#ifdef CC_PRI_HAS_REVERSECHARGE
					if (!i->outgoing)
					{
						sprintf(szReverseChargingReceived, "%d", i->reverse_charging);

						if (!ast_strlen_zero(szReverseChargingReceived))
						{
							pbx_builtin_setvar_helper(tmp, "DGV_REVERSECHARGE", szReverseChargingReceived);
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): Reverse Charge Value is '%s', use DGV_REVERSECHARGE variable to read it!\n", i->port, szReverseChargingReceived);
						}
					}
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE
			case DG_SIG_FXO:
			case DG_SIG_FXS:
				//nothing to do
				break;
		} //switch (i->sig)

        //copy account code if it is configured
        if (!ast_strlen_zero(i->accountcode))
        {
#ifdef CC_AST_HAS_VERSION_11
				ast_channel_accountcode_set(tmp, i->accountcode);
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
						ast_log(LOG_NOTICE, ">>>>>>>>dgv_new %s - has stringfield \n", dgv_channel_name(tmp));
					ast_string_field_set(tmp, accountcode, i->accountcode);
	#else//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					ast_copy_string(tmp->accountcode, i->accountcode, sizeof(tmp->accountcode));
	#endif//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
#endif//#ifdef CC_AST_HAS_VERSION_11
        }

        //copy language structure to channel
        if (!ast_strlen_zero(i->language))
        {
#ifdef CC_AST_HAS_VERSION_11
				ast_channel_language_set(tmp, i->language);
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					ast_string_field_set(tmp, language, i->language);
	#else//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					ast_copy_string(tmp->language, i->language, sizeof(tmp->language));
	#endif//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
#endif//#ifdef CC_AST_HAS_VERSION_11
        }
    	/* all native formats supported */

#ifdef CC_AST_HAS_VERSION_13
			if ((supported_fmts & AST_FORMAT_GSM)!=0)
				ast_format_cap_append(dgv_channel_nativeformats(tmp), ast_format_gsm, 0);
			if ((supported_fmts & AST_FORMAT_ULAW)!=0)
				ast_format_cap_append(dgv_channel_nativeformats(tmp), ast_format_ulaw, 0);
			if ((supported_fmts & AST_FORMAT_ALAW)!=0)
				ast_format_cap_append(dgv_channel_nativeformats(tmp), ast_format_alaw, 0);
			if ((supported_fmts & AST_FORMAT_SLIN)!=0)
				ast_format_cap_append(dgv_channel_nativeformats(tmp), ast_format_slin, 0);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
				ast_format_cap_from_old_bitfield(dgv_channel_nativeformats(tmp), supported_fmts);
	#else//#ifdef CC_AST_HAS_VERSION_10
				tmp->nativeformats = supported_fmts;
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

    	/* save native formats into pvt structure */
    	i->nativeformats = supported_fmts;

        i->faxhandled = 0;		                 //reset fax flag
        i->fax_tone_state = 0;
        i->fxo_bridged_dial = 0;                 //reset fxo bridged dial flag
        if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
            ast_log(LOG_NOTICE, ">>>>>>>>dgv_new %s - Resetting faxhandled and fxo_bridged_dial \n", dgv_channel_name(tmp));

        //preferencial format 
        //todo: make custom order in digivoice.conf

#ifdef CC_AST_HAS_VERSION_13
			if ((supported_fmts & AST_FORMAT_GSM)!=0)
				i->pref_format = ast_format_gsm;
			else
				if ((supported_fmts & AST_FORMAT_ULAW)!=0)
					i->pref_format = ast_format_ulaw;
				else
					if ((supported_fmts & AST_FORMAT_ALAW)!=0)
						i->pref_format = ast_format_alaw;
					else
						i->pref_format = ast_format_slin;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
				if ((supported_fmts & AST_FORMAT_GSM)!=0)
					ast_format_set(&i->pref_format, AST_FORMAT_GSM, 0);
				else
					if ((supported_fmts & AST_FORMAT_ULAW)!=0)
						ast_format_set(&i->pref_format, AST_FORMAT_ULAW, 0);
					else
						if ((supported_fmts & AST_FORMAT_ALAW)!=0)
							ast_format_set(&i->pref_format, AST_FORMAT_ALAW, 0);
						else
							ast_format_set(&i->pref_format, AST_FORMAT_SLINEAR, 0);
	#else//#ifdef CC_AST_HAS_VERSION_10
				if ((supported_fmts & AST_FORMAT_GSM)!=0)
					i->pref_format = AST_FORMAT_GSM;
				else
					if ((supported_fmts & AST_FORMAT_ULAW)!=0)
						i->pref_format = AST_FORMAT_ULAW;
					else
						if ((supported_fmts & AST_FORMAT_ALAW)!=0)
							i->pref_format = AST_FORMAT_ALAW;
						else
							i->pref_format = AST_FORMAT_SLINEAR;
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

        if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
#ifdef CC_AST_HAS_VERSION_13
        		ast_log(LOG_NOTICE, "dgv_new %s - preferencial format is %x\n", dgv_channel_name(tmp), ast_format_get_codec_id(i->pref_format));
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
					ast_log(LOG_NOTICE, "dgv_new %s - preferencial format is %x\n", dgv_channel_name(tmp), i->pref_format.id);
	#else//#ifdef CC_AST_HAS_VERSION_10
					ast_log(LOG_NOTICE, "dgv_new %s - preferencial format is %x\n", dgv_channel_name(tmp), i->pref_format);
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

#if !defined(CC_AST_HAS_VERSION_1_4) && !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)
        	tmp->type = ttype;
#endif//#if !defined(CC_AST_HAS_VERSION_1_4) && !defined(CC_AST_HAS_VERSION_1_6) && !defined(CC_AST_HAS_VERSION_1_8)

#ifdef CC_AST_HAS_VERSION_13
			ast_channel_set_writeformat(tmp, i->pref_format); /* requested write format (playback) */
			ast_channel_set_readformat(tmp, i->pref_format);  /* requested read format (record) */
			ast_channel_set_rawwriteformat(tmp, i->pref_format);
			ast_channel_set_rawreadformat(tmp, i->pref_format);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
				ast_format_copy(ast_channel_writeformat(tmp), &i->pref_format); /* requested write format (playback) */
				ast_format_copy(ast_channel_readformat(tmp), &i->pref_format);  /* requested read format (record) */
				ast_format_copy(ast_channel_rawwriteformat(tmp), &i->pref_format);
				ast_format_copy(ast_channel_rawreadformat(tmp), &i->pref_format);
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
					ast_format_copy(&tmp->writeformat, &i->pref_format); /* requested write format (playback) */
					ast_format_copy(&tmp->readformat, &i->pref_format);  /* requested read format (record) */
					ast_format_copy(&tmp->rawwriteformat, &i->pref_format);
					ast_format_copy(&tmp->rawreadformat, &i->pref_format);
		#else//#ifdef CC_AST_HAS_VERSION_10
					tmp->writeformat = i->pref_format; /* requested write format (playback) */
					tmp->readformat = i->pref_format;  /* requested read format (record) */
					tmp->rawwriteformat = i->pref_format;
					tmp->rawreadformat = i->pref_format;
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_VERSION_11
			ast_channel_transfercapability_set(tmp, transfercapability);
#else//#ifdef CC_AST_HAS_VERSION_11
			tmp->transfercapability = transfercapability;
#endif//#ifdef CC_AST_HAS_VERSION_11

        //reset internal buffer to hold samples
        memset(i->buf, 0, sizeof(i->buf));

        /* copy music class for this channel */
        if (!ast_strlen_zero(i->musicclass))
        {
#ifdef CC_AST_HAS_VERSION_11
				ast_channel_musicclass_set(tmp, i->musicclass);
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					ast_string_field_set(tmp, musicclass, i->musicclass);
	#else//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
					ast_copy_string(tmp->musicclass, i->musicclass, sizeof(tmp->musicclass));
	#endif//#ifdef CC_AST_HAS_STRINGFIELD_IN_CHANNEL
#endif//#ifdef CC_AST_HAS_VERSION_11
        }

        if (i->callgroup!=0)
        {
            /* Only FXS signalled stuff can be picked up */
#ifdef CC_AST_HAS_VERSION_11
				ast_channel_callgroup_set(tmp, i->callgroup);
#else//#ifdef CC_AST_HAS_VERSION_11
				tmp->callgroup = i->callgroup;
#endif//#ifdef CC_AST_HAS_VERSION_11
        }        
        
        if (i->pickupgroup!=0)
        {
            /* Only FXS signalled stuff can be picked up */
#ifdef CC_AST_HAS_VERSION_11
				ast_channel_pickupgroup_set(tmp, i->pickupgroup);
#else//#ifdef CC_AST_HAS_VERSION_11
				tmp->pickupgroup = i->pickupgroup;
#endif//#ifdef CC_AST_HAS_VERSION_11
        }

        i->read_last_format = -1;

        //set context to use
#ifdef CC_AST_HAS_VERSION_11
			ast_channel_context_set(tmp, i->context);
#else//#ifdef CC_AST_HAS_VERSION_11
			ast_copy_string(tmp->context, context, sizeof(tmp->context));
#endif//#ifdef CC_AST_HAS_VERSION_11

        /* Copy DNIS to channel structure */
        if (!ast_strlen_zero(i->exten))
        	dgv_channel_exten_set(tmp, i->exten);
    
        ast_setstate(tmp, state);

#ifdef USE_LIBPRI
	        i->alreadyhungup = 0;
    	    i->stripmsd = 1;            //todo: customizable
#endif//#ifdef USE_LIBPRI

        //ast_dgv_lock(&usecnt_lock);
        //usecnt++;
        //ast_dgv_unlock(&usecnt_lock);
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
       		/* Configure the new channel jb */
       		ast_jb_configure(tmp, &i->jbconf);

			ast_atomic_fetchadd_int(&usecnt, 1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
        	ast_dgv_lock(&usecnt_lock, 0);
        	usecnt++;
        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
    
        ast_update_use_count();

#ifdef CC_AST_HAS_VERSION_12
			ast_channel_unlock(tmp);
#endif//#ifdef CC_AST_HAS_VERSION_12

        if (startpbx) {
            if (ast_pbx_start(tmp)) {
                ast_log(LOG_WARNING, "Unable to start PBX on %s\n", dgv_channel_name(tmp));
                ast_hangup(tmp);
                i->reserved=0;
                tmp = NULL;
            }
        }
        if ((i->verbosity & DG_LOG_START_END_CALL) != 0)
            ast_log(LOG_NOTICE, "Channel allocated %s\n", dgv_channel_name(tmp));
    }
    else
    {
        i->ast_ch = NULL;
        ast_log(LOG_WARNING, "Unable to allocate channel structure\n");
    }

  	return tmp;
}

//-------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------
#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause)
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_11
		static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_channel *requestor, const char *data, int *cause)
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
			static struct ast_channel *dgv_request(const char *type, struct ast_format_cap *cap, const struct ast_channel *requestor, void *data, int *cause)
		#else//#ifdef CC_AST_HAS_VERSION_10
			#ifdef CC_AST_HAS_VERSION_1_8
				static struct ast_channel *dgv_request(const char *type, format_t format, const struct ast_channel *requestor, void *data, int *cause)
			#else//#ifdef CC_AST_HAS_VERSION_1_8
				static struct ast_channel *dgv_request(const char *type, int format, void *data, int *cause)
			#endif//#ifdef CC_AST_HAS_VERSION_1_8
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
{
	struct dgv_pvt *p;
	struct ast_channel *tmp = NULL;
	char *name = strdup(data ? (char *)data : "");
	char *s;
	char *more_data;
	char *piece_data;
	int group_port[NUM_MAXDIALOPT];
	short is_group[NUM_MAXDIALOPT];
	short backwards[NUM_MAXDIALOPT];
	short roundrobin[NUM_MAXDIALOPT];
	short serialnumber[NUM_MAXDIALOPT];
	short serialnumber_port[NUM_MAXDIALOPT];
	short talktime[NUM_MAXDIALOPT];
	short serialnumber_talktime[NUM_MAXDIALOPT];
	short nForData;
	short nCntGroup_Port;

	char *prefix_data;

#ifdef USE_LIBPRI
	    short dc=0;
    	short spn=0;
#endif//#ifdef USE_LIBPRI

#ifdef DEBUG_Q
		char szTemp[255];
#endif//#ifdef DEBUG_Q

#ifdef CC_AST_HAS_VERSION_13
			struct ast_str *captmp = ast_str_alloca(64);

			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "Asked to get a channel %s/%s using format %s\n", type, name, ast_format_cap_get_names(cap, &captmp));

			if (!(ast_format_cap_iscompatible(cap, dgv_tech.capabilities)))
			{
				ast_log(LOG_NOTICE, "Asked to get a channel of unsupported format '%s'\n", ast_format_cap_get_names(cap, &captmp));
				*cause = AST_CAUSE_BEARERCAPABILITY_NOTAVAIL;
				return NULL;
			}
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
			char captmp[256];

			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "Asked to get a channel %s/%s using format %s\n", type, name, ast_getformatname_multiple(captmp, sizeof(captmp), cap));

			if (!(ast_format_cap_has_joint(cap, dgv_tech.capabilities)))
			{
				ast_log(LOG_NOTICE, "Asked to get a channel of unsupported format '%s'\n", ast_getformatname_multiple(captmp, sizeof(captmp), cap));
				*cause = AST_CAUSE_BEARERCAPABILITY_NOTAVAIL;
				return NULL;
			}
	#else//#ifdef CC_AST_HAS_VERSION_10
			int oldformat;

			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "Asked to get a channel %s/%s using format %s\n", type, name, ast2dgvformatname(format));

			oldformat = format;
			format &= supported_fmts;
			if (!format) {
				ast_log(LOG_NOTICE, "Asked to get a channel of unsupported format '%s'\n", ast2dgvformatname(oldformat));
				return NULL;
			}
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13

	for (nForData = 0; nForData <= NUM_MAXDIALOPT - 1; nForData++)
	{
		group_port[nForData] = -1;
		is_group[nForData] = -1;
		backwards[nForData] = 0;
		roundrobin[nForData] = 0;
		serialnumber[nForData] = 0;
		serialnumber_port[nForData] = 0;
		talktime[nForData] = 0;
		serialnumber_talktime[nForData] = 0;
	}

   	//check if it's group or port
    s = strsep(&name, "/");
    if (s)
    {
    	nCntGroup_Port = 0;
		more_data = ast_strdupa(s);
		while ((piece_data = strsep(&more_data, "+")))
		{
	        //check for group syntax
	        if (toupper(piece_data[0]) == 'G' || toupper(piece_data[0])=='R' || toupper(piece_data[0])=='S' || toupper(piece_data[0])=='T')
	        {
				if (option_verbose >= 3)
					ast_log(LOG_NOTICE, "New Group detected: %s\n", piece_data);

				is_group[nCntGroup_Port] = 1;
				group_port[nCntGroup_Port] = atoi(piece_data+1);

	            if (piece_data[0] == 'G' || piece_data[0] == 'R' || piece_data[0] == 'S' || piece_data[0] == 'T')//upper letter use backward search
	                backwards[nCntGroup_Port] = 1;
	            else
	                backwards[nCntGroup_Port] = 0;

	            if (toupper(piece_data[0])=='R')
	            	roundrobin[nCntGroup_Port] = 1;

	            if (toupper(piece_data[0])=='S')
	            {
	            	serialnumber[nCntGroup_Port] = 1;

	            	if (toupper(piece_data[6])=='C')
	            		serialnumber_port[nCntGroup_Port] = atoi(piece_data+7);
	            	else
	            		serialnumber_port[nCntGroup_Port] = 0;

	            	if (piece_data[6]=='T')
	            		serialnumber_talktime[nCntGroup_Port] = 2; //like a backward
	            	else
	            		if (piece_data[6]=='t')
	            			serialnumber_talktime[nCntGroup_Port] = 1; //not a backward
	            		else
	            			serialnumber_talktime[nCntGroup_Port] = 0; //disabled
	            }

	            if (toupper(piece_data[0])=='T')
	            	talktime[nCntGroup_Port] = 1;
	        }
	        else
	        {
				if (option_verbose >= 3)
					ast_log(LOG_NOTICE, "New Port detected: %s\n", piece_data);

				is_group[nCntGroup_Port] = 0;
				group_port[nCntGroup_Port] = atoi(piece_data);
	        }
			nCntGroup_Port++;
		}
    }
    else
    {
        if (option_verbose >= 3)
            ast_log(LOG_NOTICE, "Invalid port -> %d\n", 0);
    }

    //keep protected
    ast_dgv_lock(&iflock, 0);
    {
        p = find_free_port(group_port, is_group, backwards, -1, roundrobin, serialnumber, serialnumber_port, talktime, serialnumber_talktime, 0);	//force port to -1 to find using group only

		//specific port must check if it's free
		if (p)
		{
			if ( (p->ast_ch && p->sig != DG_SIG_FXS && p->sig != DG_SIG_CB_FXS) ||
				 (p->ast_ch && p->subchannel) ||
				 (p->ast_ch && (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && !p->second_call) )
				p = NULL;		//force null to raise busy message
		}

        if (p && !p->reserved)
        {
        	if (name)
        	{
				prefix_data = ast_strdupa(name);
				if (prefix_data)
				{
					if (strlen(p->deny_prefix) > 0)
						if (!strncmp(p->deny_prefix, prefix_data, strlen(p->deny_prefix)))
						{
							ast_log(LOG_WARNING, "dgv_request: The number (%s) was blocked by deny_prefix configuration '%s' on port %d!\n", prefix_data, p->deny_prefix, p->port);
							p = NULL;
							*cause = AST_CAUSE_CONGESTION;

							ast_dgv_unlock(&iflock, 0);
							return tmp;
						}
				}
				else
					if (option_debug >= 1)
						ast_log(LOG_DEBUG, "Invalid prefix -> %d\n", 0);
        	}

        	if ((p->sig != DG_SIG_FXS) && (p->sig != DG_SIG_CB_FXS))
        		p->subchannel_audio_index = 1;
            p->reserved = 1;
            
            //setting audio format for recording
#ifdef CC_AST_HAS_VERSION_10
				if (p->read_last_format != dgv_format_cap_to_old_bitfield(cap))
				{
					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
#ifdef CC_AST_HAS_VERSION_13
							ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n", p->port, ast_format_cap_get_names(cap, &captmp));
#else//#ifdef CC_AST_HAS_VERSION_13
							ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n", p->port, ast_getformatname_multiple(captmp, sizeof(captmp), cap));
#endif//#ifdef CC_AST_HAS_VERSION_13

					//format was changed since last time
					dg_PauseInputBuffer(p->port, DG_PAUSE);
					if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
						ast_log(LOG_WARNING, "dgv_request: Pausing InputBuffer on port %d\n", p->port);

					//assigns file format
#ifdef CC_AST_HAS_VERSION_13
						p->read_last_format = dgv_format_cap_to_old_bitfield(cap);
#else//#ifdef CC_AST_HAS_VERSION_13
						p->read_last_format = ast_format_id_to_old_bitfield(dgv_format_cap_to_old_bitfield(cap));
#endif//#ifdef CC_AST_HAS_VERSION_13

					dg_SetRecordFormat(p->port, ast2dgvformat(p->read_last_format));
				}
#else//#ifdef CC_AST_HAS_VERSION_10
				if (p->read_last_format != format)
				{
					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
						ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n",p->port, ast2dgvformatname(format));

					//format was changed since last time
					//dg_DisableInputBuffer(p->port);
					dg_PauseInputBuffer(p->port, DG_PAUSE);
					if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
						ast_log(LOG_WARNING, "dgv_request: Pausing InputBuffer on port %d\n", p->port);

					//assigns file format
					p->read_last_format = format;

					dg_SetRecordFormat(p->port, ast2dgvformat(p->read_last_format));
				}
#endif//#ifdef CC_AST_HAS_VERSION_10
            
            if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                ast_log(LOG_NOTICE, "dgv_request: Calling dgv_new port %d using context=%s\n",p->port,p->context);

#ifdef USE_LIBPRI
            	//set pri structure to this channel
            	if (p->ch_board > 30)
	                spn = 2;
            	else
	                spn = 1;
            	dc = dg_pri_findindex(p->board, spn);
            	if (dc>=0) {
	                p->pri = dchans[dc].pri;
            	}
            	p->outgoing = 1;
#endif//#ifdef USE_LIBPRI

            	if (strlen(p->szPortID) > 1)
                	ast_copy_string(p->cid_num, p->szPortID, sizeof(p->cid_num));
                else
                    p->cid_num[0] = '\0';

                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                {
                	if ((!ast_strlen_zero(p->szPortID)) && (ast_isphonenumber(p->szPortID)))
                        ast_verbose(VERBOSE_PREFIX_3 "dgv_request (Dgv/%d): CallerID already detected - '%s'\n", p->port, p->cid_num);
                    else
                        ast_verbose(VERBOSE_PREFIX_3 "dgv_request (Dgv/%d): No CallerID information detected yet.\n", p->port);
                }

                if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
                {
                	if (strlen(p->szPortName) > 1)
                    	ast_copy_string(p->cid_name, p->szPortName, sizeof(p->cid_name));
                    else
                        p->cid_name[0] = '\0';
                }
                else
                	p->cid_name[0] = '\0';

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
                	tmp = dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, assignedids, requestor);
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_11
						tmp = dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, requestor ? ast_channel_linkedid(requestor) : "");
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_1_8
							tmp = dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0, requestor ? requestor->linkedid : "");
		#else//#ifdef CC_AST_HAS_VERSION_1_8
							tmp = dgv_new(p, AST_STATE_RESERVED, p->context, DONT_START_PBX, 0);
		#endif//#ifdef CC_AST_HAS_VERSION_1_8
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL

#ifdef DEBUG_Q
	            sprintf(szTemp,"/tmp/reclog%d.raw", p->port);
    	        p->rec = fopen(szTemp,"w");
        	    if (p->rec==NULL)
            	    ast_log(LOG_ERROR, "Error opening test file!!!");	
                
            	sprintf(szTemp,"/tmp/playlog%d.raw", p->port);
            	p->play = fopen(szTemp,"w");
            	if (p->play==NULL)
	                ast_log(LOG_ERROR, "Error opening test file!!!");	
#endif//#ifdef DEBUG_Q	
        }
        else //if (p && !p->reserved)
        {
        	if ( (p) &&
        		 (!p->subchannel) &&
        		 (p->second_call) &&
        		 (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS) && 
        		 (p->call_state == DG_AST_CONVERSATION || p->call_state == DG_AST_CALLING) )
        	{
	            if (option_verbose >= 3)
	                ast_log(LOG_WARNING, "dgv_request: Trying to allocate a subchannel on port %d\n", p->port);

				p->subchannel = initialize_ports(p->port);
				if (p->subchannel)
				{
					p->subchannel->subchannel             = p;
					p->subchannel->channel_index          = 2;
					p->subchannel->subchannel_audio_index = 1;
					p->subchannel->reserved               = 1;
					p->subchannel->call_state             = DG_AST_NONE;
					p->subchannel->verbosity              |= p->verbosity;
					p->subchannel->sig                    = p->sig;
					p->subchannel->callgroup              = p->callgroup;
					p->subchannel->pickupgroup            = p->pickupgroup;

		            //setting audio format for recording
#ifdef CC_AST_HAS_VERSION_10
						if (p->subchannel->read_last_format != dgv_format_cap_to_old_bitfield(cap))
						{
							if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
#ifdef CC_AST_HAS_VERSION_13
									ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n", p->subchannel->port, ast_format_cap_get_names(cap, &captmp));
#else//#ifdef CC_AST_HAS_VERSION_13
									ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n", p->subchannel->port, ast_getformatname_multiple(captmp, sizeof(captmp), cap));
#endif//#ifdef CC_AST_HAS_VERSION_13

							//assigns file format
							p->subchannel->read_last_format = dgv_format_cap_to_old_bitfield(cap);
						}
#else//#ifdef CC_AST_HAS_VERSION_10
						if (p->subchannel->read_last_format != format)
						{
							if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
								ast_log(LOG_NOTICE, "dgv_request: (%d) Req. new format: %s\n", p->subchannel->port, ast2dgvformatname(format));

							//format was changed since last time
							//dg_PauseInputBuffer(p->subchannel->port, DG_PAUSE);

							//assigns file format
							p->subchannel->read_last_format = format;

							//dg_SetRecordFormat(p->subchannel->port, ast2dgvformat(p->subchannel->read_last_format));
						}
#endif//#ifdef CC_AST_HAS_VERSION_10

		            if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
		                ast_log(LOG_NOTICE, "dgv_request: Calling dgv_new port %d using context=%s\n", p->subchannel->port, p->subchannel->context);

		            if ((!ast_strlen_zero(p->szPortID)) && (ast_isphonenumber(p->szPortID)))
	            	{
	            		ast_copy_string(p->subchannel->szPortID, p->szPortID, sizeof(p->subchannel->szPortID));
	                	ast_copy_string(p->subchannel->cid_num, p->subchannel->szPortID, sizeof(p->subchannel->cid_num));
	            	}
	                else
	                    p->subchannel->cid_num[0] = '\0';

	                if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                {
	                	if (strlen(p->subchannel->szPortID) > 1)
	                        ast_verbose(VERBOSE_PREFIX_3 "dgv_request (Dgv/%d): CallerID already detected - '%s'\n", p->subchannel->port, p->subchannel->cid_num);
	                    else
	                        ast_verbose(VERBOSE_PREFIX_3 "dgv_request (Dgv/%d): No CallerID information detected yet.\n", p->subchannel->port);
	                }
	                if (p->sig == DG_SIG_FXS || p->sig == DG_SIG_CB_FXS)
	                {
	                	if (!ast_strlen_zero(p->szPortName))
	                	{
	                		ast_copy_string(p->subchannel->szPortName, p->szPortName, sizeof(p->subchannel->szPortName));
	                		ast_copy_string(p->subchannel->cid_name, p->subchannel->szPortName, sizeof(p->subchannel->cid_name));
	                	}
	                    else
	                    	p->subchannel->cid_name[0] = '\0';
	                }
	                else
	                	p->subchannel->cid_name[0] = '\0';

#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	                	tmp = dgv_new(p->subchannel, AST_STATE_RESERVED, p->subchannel->context, DONT_START_PBX, 0, assignedids, requestor);
#else//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
	#ifdef CC_AST_HAS_VERSION_11
							tmp = dgv_new(p->subchannel, AST_STATE_RESERVED, p->subchannel->context, DONT_START_PBX, 0, requestor ? ast_channel_linkedid(requestor) : "");
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_1_8
								tmp = dgv_new(p->subchannel, AST_STATE_RESERVED, p->subchannel->context, DONT_START_PBX, 0, requestor ? requestor->linkedid : "");
		#else//#ifdef CC_AST_HAS_VERSION_1_8
								tmp = dgv_new(p->subchannel, AST_STATE_RESERVED, p->subchannel->context, DONT_START_PBX, 0);
		#endif//#ifdef CC_AST_HAS_VERSION_1_8
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_ASSIGNEDIDS_IN_CHANNEL
				}
				else //if (p->subchannel)
				{
					ast_log(LOG_ERROR, "Unable to register a subchannel on '%d'\n", p->port);
				} //if (p->subchannel)
        	}
        	else
        	{
				if (option_verbose >= 3)
					for (nForData = 0; nForData <= NUM_MAXDIALOPT - 1; nForData++)
						if (group_port[nForData] != -1)
						{
							if (is_group[nForData] == 1)
							{
								if (serialnumber[nForData] == 1)
								{
									if (serialnumber_port[nForData] != 0)
										ast_log(LOG_WARNING, "dgv_request: All ports of serial_number %05d with port %d are in use or reserved!\n", group_port[nForData], serialnumber_port[nForData]);
									else
										ast_log(LOG_WARNING, "dgv_request: All ports of serial_number %05d are in use or reserved!\n", group_port[nForData]);
								}
								else
									ast_log(LOG_WARNING, "dgv_request: All ports of group %d are in use or reserved!\n", group_port[nForData]);
							}
							else
								ast_log(LOG_WARNING, "dgv_request: Port %d in use or reserved!\n", group_port[nForData]);
						}

				if ((nCntGroup_Port > 1) || (is_group[0] == 1))
					*cause = AST_CAUSE_CONGESTION;
				else
				{
					if (name)
						*cause = AST_CAUSE_CONGESTION;
					else
						*cause = AST_CAUSE_BUSY;
				}
        	}
        } //if (p && !p->reserved)
    }
    ast_dgv_unlock(&iflock, 0);
   
  	//free(name);
	return tmp;
}

//-------------------------------------------------------------------------------------------
// Send desired digit to card
//-------------------------------------------------------------------------------------------
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	static int dgv_digit_begin(struct ast_channel *ast, char digit)
	{
		struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);
	
		if (p)
		{	
 	    	char szDigit[2];
        
       		szDigit[0] = digit;
       		szDigit[1] = '\0';        
        
        	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
            	ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit_begin: Asked to play digit[%s]\n", p->dev, szDigit);
    	}
      
    	return 0;
	}
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
	static int dgv_digit(struct ast_channel *ast, char digit, unsigned int duration)
#else//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
	static int dgv_digit(struct ast_channel *ast, char digit)
#endif//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
{
	struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);

	struct dgv_pvt *p_bridged;

#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_bridge_ch, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_bridge_ch;
#endif//#ifdef CC_AST_HAS_VERSION_12

	short nGenDTMF = 1;

	if (!p)
	{
		if (option_verbose >= 3)
			ast_log(LOG_NOTICE, "dgv_digit(%s): !p \n", dgv_channel_name(ast));
		return 0;
	}

	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
		ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: Asked to play digit with duration [%d]\n", p->dev, duration);
#else//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)
		ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: Asked to play digit \n", p->dev);
#endif//#if (defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && defined(CC_AST_HAS_SEND_DIGIT_END_DURATION)

	char szDigit[2];

	ast_dgv_lock(&p->lock, p->port);
	{
		if (!p->faxhandled)
		{
			if (p->gen_dtmf == 1)
			{
				dgv_bridge_ch = dgv_bridged_channel(ast);
				if (dgv_bridge_ch != NULL)
				{
					if (dgv_channel_tech_pvt(dgv_bridge_ch) != NULL && !strncmp(dgv_channel_name(dgv_bridge_ch), ttype, 3))
					{
						p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_bridge_ch);

						if ( (p_bridged->dtmf_to_asterisk == 1) &&
#ifdef CC_AST_HAS_VERSION_13
								(p->read_last_format != ast_format_get_codec_id(ast_format_gsm) && p_bridged->read_last_format != ast_format_get_codec_id(ast_format_gsm)) &&
								(p->write_last_format != ast_format_get_codec_id(ast_format_gsm) && p_bridged->write_last_format != ast_format_get_codec_id(ast_format_gsm)) )
#else//#ifdef CC_AST_HAS_VERSION_13
								(p->read_last_format != AST_FORMAT_GSM && p_bridged->read_last_format != AST_FORMAT_GSM) &&
								(p->write_last_format != AST_FORMAT_GSM && p_bridged->write_last_format != AST_FORMAT_GSM) )
#endif//#ifdef CC_AST_HAS_VERSION_13
						{
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
								ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: Don't Dial DTMF because we are bridged!\n", p->dev);

							ast_dgv_unlock(&p->lock, p->port);

							return 0;
						}
						else
						{
							if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
#ifdef CC_AST_HAS_VERSION_13
									ast_log(LOG_NOTICE, "%s: dgv_digit: variables read:, d2* %d, p->r %lld, b->r %lld, p->w %lld, b->w %lld\n",
#else//#ifdef CC_AST_HAS_VERSION_13
									ast_log(LOG_NOTICE, "%s: dgv_digit: variables read:, d2* %d, p->r %d, b->r %d, p->w %d, b->w %d\n",
#endif//#ifdef CC_AST_HAS_VERSION_13
										p->dev,
										p_bridged->dtmf_to_asterisk,
										p->read_last_format,
										p_bridged->read_last_format,
										p->write_last_format,
										p_bridged->write_last_format);
						}
					}
					else
					{
						if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
							ast_log(LOG_NOTICE, "%s: dgv_digit: %s is not a DGV channel\n", p->dev, dgv_channel_name(dgv_bridge_ch));
					}
				}
				else
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "%s: dgv_digit: not bridged!\n", p->dev);
				}
			}
			else
			{
				if (p->gen_dtmf == 2 && !dgv_channel_is_bridged(ast))
				{
					//nothing to do
				}
				else
					nGenDTMF = 0;
			}

			if (nGenDTMF)
				szDigit[0] = digit;
			else
				szDigit[0] = ',';

			szDigit[1] = '\0';

			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: Asked to play digit[%s]\n", p->dev, szDigit);

			//to allow
			if (dg_GetPortInterface(p->port)==DG_DIGITAL_INTERFACE || dg_GetPortInterface(p->port)==DG_GSM_INTERFACE)
			{
				if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
					ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: DIRECT Digit[%s]\n", p->dev, szDigit);

				//if ((dg_GetPortStatus(p->port)!=spDialing) && (dgv_channel_state(ast) == AST_STATE_UP))
				dg_Dial(p->port, szDigit, 20, dtDirectMF);
				//else
				//	ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: We are dialing and not up yet, Dropping digit[%s]\n", p->dev, szDigit);
			}
			else
			{
				if ((p->sig==DG_SIG_FXO || p->sig==DG_SIG_CB_FXO) && (p->fxo_bridged_dial))
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: Don't Dial DTMF [%s] because fxo on bridged dial \n", p->dev, szDigit);
				}
				else
				{
					if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_digit: DTMF [%s]\n", p->dev, szDigit);

					dg_Dial(p->port, szDigit, 20, dgv_conf.default_dialtype);
				}
			}
		}
		else
		{
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_verbose(VERBOSE_PREFIX_3 "dgv_digit: FAX Handling, Dropping DTMF on port %d \n", p->port);
		}

	}
	ast_dgv_unlock(&p->lock, p->port);

	return 0;
}
	
//-------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------
#ifdef CC_AST_HAS_VERSION_11
	static int dgv_call(struct ast_channel *ast, const char *dest, int timeout)
#else//#ifdef CC_AST_HAS_VERSION_11
	static int dgv_call(struct ast_channel *ast, char *dest, int timeout)
#endif//#ifdef CC_AST_HAS_VERSION_11
{
#ifdef DEBUG_PRI	
		ast_log(LOG_NOTICE, "dgv_call: STEP 1...\n");
#endif//#ifdef DEBUG_PRI

#ifdef CC_AST_HAS_VERSION_11
    	struct ast_party_id connected_id;
#endif//#ifdef CC_AST_HAS_VERSION_11

	struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);
	struct dgv_pvt *p_bridged;
	char *c;
#ifdef CC_AST_HAS_VERSION_12
		RAII_VAR(struct ast_channel *, dgv_chan_tmp, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
		struct ast_channel *dgv_chan_tmp;
#endif//#ifdef CC_AST_HAS_VERSION_12
	short ret;
	char sCIDToFXS[AST_MAX_EXTENSION] = "";
	const char *dialplan_bridge_channel = NULL;
	char dgv_bridge_channel[AST_MAX_EXTENSION] = "";
	char dgv_bridge_tmp_port[5] = "";
	short nForText;
	short nForTempText = 0;
	int tmp_port;

	const char *dialplan_ringtype;
	int dialplan_ringtype_value;
	int dialplan_ringtype_set;

	if (!p)
	{
		if (option_verbose >= 3)		
			ast_log(LOG_NOTICE, "dgv_call(%s): !p \n", dgv_channel_name(ast));
		return 0;		
	}

	ast_dgv_lock(&p->lock, p->port);
	{
    	if (!p->ast_ch)
    	{
    		p->call_state = DG_AST_NONE;
    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
    			ast_log(LOG_WARNING, "dgv_call: p->ast_ch does not exist on port %d\n", p->port);
    		ast_dgv_unlock(&p->lock, p->port);
    		return -1;
    	}
    	
#ifdef CC_AST_HAS_VERSION_11
			connected_id = dgv_channel_connected_effective_id(p->ast_ch);
#endif//#ifdef CC_AST_HAS_VERSION_11

    	c = strchr(dest, '/');
    	if (((c) || (!c && (p->sig == DG_SIG_FXO || p->sig == DG_SIG_CB_FXO))) && ((p->sig != DG_SIG_FXS) && (p->sig != DG_SIG_CB_FXS)))
    	{
            if ((dgv_channel_state(ast) != AST_STATE_DOWN) && (dgv_channel_state(ast) != AST_STATE_RESERVED)) {
                ast_log(LOG_WARNING, "dgv_call called on %s, neither down or reserved\n", dgv_channel_name(ast));//...down nor reserverd
                ast_dgv_unlock(&p->lock, p->port);
                return -1;
            }

			if (!c && (p->sig==DG_SIG_FXO || p->sig==DG_SIG_CB_FXO))
				p->szNumberToDial[0] = '\0';
			else
			{
    			c++;
    			sprintf(p->szNumberToDial, "%s", c);
			}
			
            if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                ast_log(LOG_NOTICE, "dgv_call: Using card/port %d to dialing to %s - timeout: %d \n", p->port,c, timeout);

    		switch(p->sig)
    		{
    			case DG_SIG_CUSTOMCAS:    			
    			case DG_SIG_FXO:
    			case DG_SIG_CB_FXO:
	                ret = dg_EnableCallProgress(p->port,CP_ENABLE_LINETONE_OR_BUSY);
	                if (ret != DG_EXIT_SUCCESS)
	                {
	                    //could not enabling call progress
	                    dg_DisableCallProgress(p->port);
	                    ast_log(LOG_WARNING, "Failed to *enable* CallProgress thread - error code = %d\n",ret);
	                    ast_dgv_unlock(&p->lock, p->port);
	                    return -1;
	                }
    				break;
    		}
   
    		//copies cid to port_id to allow port to send cid foward
    		if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE)) 
    		{
				if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                {
		        	ast_log(LOG_NOTICE, "dgv_call: p->szPortID = %s \n", p->szPortID);
#ifdef CC_AST_HAS_VERSION_11
						ast_log(LOG_NOTICE, "dgv_call: connected_id.number.str = %s \n", (connected_id.number.valid) ? connected_id.number.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
							ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->connected.id.number.str = %s \n", S_COR(p->ast_ch->connected.id.number.valid, p->ast_ch->connected.id.number.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->cid.cid_num = %s \n", p->ast_ch->cid.cid_num);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
	    	        ast_log(LOG_NOTICE, "dgv_call: p->cid_num = %s \n", p->cid_num);
                }
                
				if ((!ast_strlen_zero(p->cid_num)) && (ast_isphonenumber(p->cid_num)))
				{
					//nothing to do
				}
                else
                {
#ifdef CC_AST_HAS_VERSION_11
						if ((connected_id.number.valid) &&
							(!ast_strlen_zero(connected_id.number.str)) &&
							(ast_isphonenumber(connected_id.number.str)))
							ast_copy_string(p->cid_num, connected_id.number.str, sizeof(p->cid_num));
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
							if ((p->ast_ch->connected.id.number.valid) &&
								(!ast_strlen_zero(p->ast_ch->connected.id.number.str)) &&
								(ast_isphonenumber(p->ast_ch->connected.id.number.str)))
								ast_copy_string(p->cid_num, p->ast_ch->connected.id.number.str, sizeof(p->cid_num));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							if ((!ast_strlen_zero(p->ast_ch->cid.cid_num)) && (ast_isphonenumber(p->ast_ch->cid.cid_num)))
								ast_copy_string(p->cid_num, p->ast_ch->cid.cid_num, sizeof(p->cid_num));
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
                }

                if ((!ast_strlen_zero(p->cid_num)) && (ast_isphonenumber(p->cid_num)))
                	ast_shrink_phone_number(p->cid_num);
                
            	dg_SetPortId(p->port, p->cid_num);

				if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                {
		        	ast_log(LOG_NOTICE, "dgv_call: now p->szPortID = %s \n", p->szPortID);
#ifdef CC_AST_HAS_VERSION_11
						ast_log(LOG_NOTICE, "dgv_call: now connected_id.number.str = %s \n", (connected_id.number.valid) ? connected_id.number.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
							ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->connected.id.number.str = %s \n", S_COR(p->ast_ch->connected.id.number.valid, p->ast_ch->connected.id.number.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->cid.cid_num = %s \n", p->ast_ch->cid.cid_num);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
	    	        ast_log(LOG_NOTICE, "dgv_call: now p->cid_num = %s \n", p->cid_num);
                }

				/* DGV bridge channel information */
				if (p->ast_ch)
					dialplan_bridge_channel = pbx_builtin_getvar_helper(p->ast_ch, "DGV_BRIDGE_CHANNEL");

				p->group_b_port_bridged = 0;
				p->category_port_bridged = 0;

				if (dialplan_bridge_channel)
				{
					ast_copy_string(dgv_bridge_channel, dialplan_bridge_channel, sizeof(dgv_bridge_channel));

					//is a DGV channel?
					if (!strncmp(dgv_bridge_channel, "DGV/", 4))
					{
						for (nForText = 4; nForText <= strlen(dgv_bridge_channel) - 1; nForText++)
						{
							dgv_bridge_tmp_port[nForTempText] = dgv_bridge_channel[nForText];
							nForTempText++;
						}
						dgv_bridge_tmp_port[nForTempText] = '\0';

						tmp_port = atoi(dgv_bridge_tmp_port);

						if (tmp_port > 0 && tmp_port <= dg_GetPortsCount())
						{
							p->group_b_port_bridged = tmp_port;
							p->category_port_bridged = tmp_port;

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "dgv_call: Call originated by channel '%d' on port %d\n", p->group_b_port_bridged, p->port);
						}
						else
							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "dgv_call: Invalid channel '%s' set as the call source on port %d\n", dgv_bridge_channel, p->port);
					}
					else
						if (option_verbose >= 3)
							ast_log(LOG_NOTICE, "dgv_call (%d): %s is not a DGV channel\n", p->port, dgv_bridge_channel);
				}
    		} //if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))

#ifdef USE_LIBPRI  
	    		p->outgoing = 1;    		
	    		
	    		//copies cid to port_id to allow port to send cid foward
	    		if (p->sig == DG_SIG_PRI)
	    		{
					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                {
			        	ast_log(LOG_NOTICE, "dgv_call: p->szPortID = %s \n", p->szPortID);
#ifdef CC_AST_HAS_VERSION_11
							ast_log(LOG_NOTICE, "dgv_call: connected_id.number.str = %s \n", (connected_id.number.valid) ? connected_id.number.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->connected.id.number.str = %s \n", S_COR(p->ast_ch->connected.id.number.valid, p->ast_ch->connected.id.number.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->cid.cid_num = %s \n", p->ast_ch->cid.cid_num);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
						ast_log(LOG_NOTICE, "dgv_call: p->cid_num = %s \n", p->cid_num);
		    	        ast_log(LOG_NOTICE, "dgv_call: p->pri_cid_num = %s \n", p->pri_cid_num);
	                }
	                
					if ((!ast_strlen_zero(p->cid_num)) && (ast_isphonenumber(p->cid_num)))
					{
						ast_copy_string(p->pri_cid_num, p->cid_num, sizeof(p->pri_cid_num));
					}
	                else
	                {
#ifdef CC_AST_HAS_VERSION_11
						if ((connected_id.number.valid) &&
							(!ast_strlen_zero(connected_id.number.str)) &&
							(ast_isphonenumber(connected_id.number.str)))
							ast_copy_string(p->pri_cid_num, connected_id.number.str, sizeof(p->pri_cid_num));
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
								if ((p->ast_ch->connected.id.number.valid) &&
									(!ast_strlen_zero(p->ast_ch->connected.id.number.str)) &&
									(ast_isphonenumber(p->ast_ch->connected.id.number.str)))
									ast_copy_string(p->pri_cid_num, p->ast_ch->connected.id.number.str, sizeof(p->pri_cid_num));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								if ((!ast_strlen_zero(p->ast_ch->cid.cid_num)) && (ast_isphonenumber(p->ast_ch->cid.cid_num)))
									ast_copy_string(p->pri_cid_num, p->ast_ch->cid.cid_num, sizeof(p->pri_cid_num));
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
	                	else
	                		ast_copy_string(p->pri_cid_num, p->cid_num, sizeof(p->pri_cid_num));
	                }

	                if (!ast_strlen_zero(p->pri_cid_num))
	                	ast_shrink_phone_number(p->pri_cid_num);
	                else
	                	p->pri_cid_num[0] = '\0';
	                
	            	dg_SetPortId(p->port, p->pri_cid_num);

					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                {
			        	ast_log(LOG_NOTICE, "dgv_call: now p->szPortID = %s \n", p->szPortID);
#ifdef CC_AST_HAS_VERSION_11
						ast_log(LOG_NOTICE, "dgv_call: now connected_id.number.str = %s \n", (connected_id.number.valid) ? connected_id.number.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->connected.id.number.str = %s \n", S_COR(p->ast_ch->connected.id.number.valid, p->ast_ch->connected.id.number.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->cid.cid_num = %s \n", p->ast_ch->cid.cid_num);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
		    	        ast_log(LOG_NOTICE, "dgv_call: now p->cid_num = %s \n", p->cid_num);
		    	        ast_log(LOG_NOTICE, "dgv_call: now p->pri_cid_num = %s \n", p->pri_cid_num);
	                }
					
					//cid name
					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                {
#ifdef CC_AST_HAS_VERSION_11
							ast_log(LOG_NOTICE, "dgv_call: connected_id.name.str = %s \n", (connected_id.name.valid) ? connected_id.name.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->connected.id.name.str = %s \n", S_COR(p->ast_ch->connected.id.name.valid, p->ast_ch->connected.id.name.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: p->ast_ch->cid.cid_name = %s \n", p->ast_ch->cid.cid_name);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: p->pri_cid_name = %s \n", p->pri_cid_name);
#endif//#ifdef CC_AST_HAS_VERSION_11
	                }

#ifdef CC_AST_HAS_VERSION_11
						if ((connected_id.name.valid) &&
							(!ast_strlen_zero(connected_id.name.str)))
							ast_copy_string(p->pri_cid_name, connected_id.name.str, sizeof(p->pri_cid_name));
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
							if ((p->ast_ch->connected.id.name.valid) &&
								(!ast_strlen_zero(p->ast_ch->connected.id.name.str)))
								ast_copy_string(p->pri_cid_name, p->ast_ch->connected.id.name.str, sizeof(p->pri_cid_name));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
							if (!ast_strlen_zero(p->ast_ch->cid.cid_name))
								ast_copy_string(p->pri_cid_name, p->ast_ch->cid.cid_name, sizeof(p->pri_cid_name));
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
					else
						p->pri_cid_name[0] = '\0';

					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
					{
#ifdef CC_AST_HAS_VERSION_11
							ast_log(LOG_NOTICE, "dgv_call: now connected_id.name.str = %s \n", (connected_id.name.valid) ? connected_id.name.str : "<Unknown Caller>");
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->connected.id.name.str = %s \n", S_COR(p->ast_ch->connected.id.name.valid, p->ast_ch->connected.id.name.str, "<Unknown Caller>"));
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								ast_log(LOG_NOTICE, "dgv_call: now p->ast_ch->cid.cid_name = %s \n", p->ast_ch->cid.cid_name);
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
					    ast_log(LOG_NOTICE, "dgv_call: now p->pri_cid_name = %s \n", p->pri_cid_name);
					}
	    		}
#endif//#ifdef USE_LIBPRI    		
		    			
    		//turn off tonethread of the bridged channel
    		if ((dgv_chan_tmp=dgv_bridged_channel(ast))!=NULL)
    		{
                if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                    ast_log(LOG_NOTICE, " bridged -- Port %d dgv_call - tone thread canceled\n",p->port);
    
    			//tell to bridged channel to stop playbuffer
    			if (dgv_channel_tech_pvt(dgv_chan_tmp)!=NULL && !strncmp(dgv_channel_name(dgv_chan_tmp), ttype,3))
    			{
    				p_bridged = (struct dgv_pvt *)dgv_channel_tech_pvt(dgv_chan_tmp);
    				if (p_bridged->tone_thread_running == 1)
    				{
    					p_bridged->tone_thread_running = 2;

    					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
    						ast_log(LOG_NOTICE, "  -- Port %d dgv_call - tone thread canceled\n",p_bridged->port);
    				}
    			}
    		}
            p->call_state = DG_AST_WAITFORDIALTONE;
            
            switch(p->sig)
            {
                case DG_SIG_R2_NDIS_FIXED:
                case DG_SIG_R2_NDIS_VARIABLE:
                case DG_SIG_CUSTOMCAS:
                case DG_SIG_FXO:
                case DG_SIG_CB_FXO:
                	//set dialtone timeout
                	p->linetone_timeout_sched = ast_sched_add(sched, p->dialtone_timeout, dialtone_timeout_callback, p);
                    break;
            	case DG_SIG_GSM:
            		//set dialtone timeout
            		p->linetone_timeout_sched = ast_sched_add(sched, 100, dialtone_timeout_callback, p);
            		break;	                		
                //0.9.8 default:
                    //0.9.8 break;
            }
            
            if (p->sig != DG_SIG_GSM)
            {
            	//pickup port
            	dg_PickUp(p->port, 20);

            	if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
            		ast_log(LOG_NOTICE, "dgv_call: dg_PickedUp Port %d \n",p->port);

#ifdef USE_LIBPRI
					if (p->sig == DG_SIG_PRI)
						start_pri_dial(p, ast);
#endif//#ifdef USE_LIBPRI
            }
    	}
    	else
    	{
			if (p)
			{	
				if (p->sig == DG_SIG_FXS)
        		{
            		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
                		ast_log(LOG_NOTICE, "dgv_call: Incoming call(DG_SIG_FXS) on port %d \n", p->port);

        			if (p->call_state == DG_AST_NONE)
        			{
        				ast_setstate(p->ast_ch, AST_STATE_RINGING);
        				p->call_state = DG_AST_CALLING;
        				
        				if (p->channel_index != 2)
        				{
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
								if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
									ast_log(LOG_NOTICE, "dgv_call %d, sending AST_CONTROL_PROGRESS\n", p->port);

								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_PROGRESS;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_PROGRESS;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								QueueFrame(p, &f);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#ifdef CC_AST_HAS_VERSION_11
								if ((p->ast_ch) && (!ast_strlen_zero(connected_id.number.str)) && (ast_isphonenumber(connected_id.number.str)))
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
									if ((p->ast_ch) && (!ast_strlen_zero(p->ast_ch->connected.id.number.str)) && (ast_isphonenumber(p->ast_ch->connected.id.number.str)))
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if ((p->ast_ch) && (!ast_strlen_zero(p->ast_ch->cid.cid_num)) && (ast_isphonenumber(p->ast_ch->cid.cid_num)))
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
	                    	{
	                    		sCIDToFXS[0] = 'A';
	                    		sCIDToFXS[1] = '1';
#ifdef CC_AST_HAS_VERSION_11
									strncpy(&sCIDToFXS[2], connected_id.number.str, strlen(connected_id.number.str));
									sCIDToFXS[strlen(sCIDToFXS)] = 'C';
									sCIDToFXS[strlen(connected_id.number.str) + 3] = '\0';
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
										strncpy(&sCIDToFXS[2], p->ast_ch->connected.id.number.str, strlen(p->ast_ch->connected.id.number.str));
										sCIDToFXS[strlen(sCIDToFXS)] = 'C';
										sCIDToFXS[strlen(p->ast_ch->connected.id.number.str) + 3] = '\0';
	#else//#ifdef CC_AST_HAS_VERSION_1_8
										strncpy(&sCIDToFXS[2], p->ast_ch->cid.cid_num, strlen(p->ast_ch->cid.cid_num));
										sCIDToFXS[strlen(sCIDToFXS)] = 'C';
										sCIDToFXS[strlen(p->ast_ch->cid.cid_num) + 3] = '\0';
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
								dg_Dial(p->port, sCIDToFXS, 20, dtTone);
	
	            				if (option_verbose >= 3)							
	                				ast_log(LOG_NOTICE, "dgv_call: Sending CallerID(%s) on FXS port %d \n", sCIDToFXS, p->port);        				
	                    	}
	                    	else
	                    	{
								/* dialplan ring_type */
    	     					ast_dgv_unlock(&p->lock, p->port);

								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "dgv_call: Before pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

								if (p->ast_ch)
									dialplan_ringtype = pbx_builtin_getvar_helper(p->ast_ch, "DGV_RINGTYPE");
								else
									ast_log(LOG_NOTICE, "dgv_call: p->ast_ch does not exist on port %d\n", p->port);

								if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
									ast_log(LOG_NOTICE, "dgv_call: After pbx_builtin_getvar_helper(DGV_RINGTYPE), port %d\n", p->port);

    	     					dialplan_ringtype_set = 0;
    	     					ast_dgv_lock(&p->lock, p->port);
								if (dialplan_ringtype)
								{
									dialplan_ringtype_value = atoi(dialplan_ringtype);

									switch (dialplan_ringtype_value)
									{
										case 1:
										case 2:
										case 3:
											dg_SetRing(p->port, DG_ENABLE, dialplan_ringtype_value);
											dialplan_ringtype_set = 1;

											if (option_verbose >= 3)
												ast_log(LOG_NOTICE, "Dialplan variable(DGV_RINGTYPE) changed ring_type to %d on port %d\n", dialplan_ringtype_value, p->port);

											break;
									}
								}
								if (dialplan_ringtype_set == 0)
									dg_SetRing(p->port, DG_ENABLE, p->ring_type);

								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								//ast_dgv_lock(&p->lock, p->port); 
								//{
								QueueFrame(p, &f);
								//}
								//ast_dgv_unlock(&p->lock, p->port);						       				
	        					
	            				if (option_verbose >= 3)
	                				ast_log(LOG_NOTICE, "dgv_call: Ring started on %d \n", p->port);                   		
	                    	}
        				}
        				else //if (p->channel_index != 2)
        				{
        					if (p->subchannel)
        					{
	        					pthread_attr_t attr;
	        					
	        					p->subchannel->subchanneltone_thread_running = 1;
	        					
	        					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	        						ast_log(LOG_NOTICE, "dgv_call: channel_index %d, subchannel->channel_index %d - port %d\n",
	        								p->channel_index, p->subchannel->channel_index, p->port);

	                    		pthread_attr_init(&attr);
	                        	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	                        	if (pthread_create(&p->subchannel->subchanneltone_thread_id, &attr, SubChannelToneThread, &(p->port)) < 0)
	        						p->subchannel->subchanneltone_thread_running = 0;
	        	               	else
	        	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_START_END_CALL) != 0))
	                	               	ast_log(LOG_NOTICE, "dgv_call: subchanneltone_thread created on port %d\n", p->port);
	                    	   	pthread_attr_destroy(&attr);
	
								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								QueueFrame(p, &f);
	
	            				if (option_verbose >= 3)
	                				ast_log(LOG_NOTICE, "dgv_call: SubChannel Ring started on %d \n", p->port);
        					}
        					else //if (p->subchannel)
        					{
        						ast_log(LOG_ERROR, "dgv_call(%d): subchannel not connected! \n", p->port);
        					} //if (p->subchannel)
        				} //if (p->channel_index != 2)
        			}
        			else   
        			{             		
            			ast_log(LOG_WARNING, "dgv_call: Port %d in use or reserved! \n", p->port);
        			}
        		} //if (p->sig == DG_SIG_FXS)
				else
				{
					if (p->sig == DG_SIG_CB_FXS)
					{
	            		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                		ast_log(LOG_NOTICE, "dgv_call: Incoming call(DG_SIG_CB_FXS) on port %d \n", p->port);

        				if (p->channel_index != 2)
	        			{
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
								if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
									ast_log(LOG_NOTICE, "dgv_call %d, sending AST_CONTROL_PROGRESS\n", p->port);

								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_PROGRESS;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_PROGRESS;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								QueueFrame(p, &f);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

#ifdef CC_AST_HAS_VERSION_11
								if ((!ast_strlen_zero(connected_id.number.str)) &&
									(ast_isphonenumber(connected_id.number.str)))
								{
									ast_shrink_phone_number(connected_id.number.str);
									ast_copy_string(p->cid_num, connected_id.number.str, sizeof(p->cid_num));
								}
#else//#ifdef CC_AST_HAS_VERSION_11
	#ifdef CC_AST_HAS_VERSION_1_8
									if ((!ast_strlen_zero(p->ast_ch->connected.id.number.str)) &&
										(ast_isphonenumber(p->ast_ch->connected.id.number.str)))
									{
										ast_shrink_phone_number(p->ast_ch->connected.id.number.str);
										ast_copy_string(p->cid_num, p->ast_ch->connected.id.number.str, sizeof(p->cid_num));
									}
	#else//#ifdef CC_AST_HAS_VERSION_1_8
									if ((!ast_strlen_zero(p->ast_ch->cid.cid_num)) && (ast_isphonenumber(p->ast_ch->cid.cid_num)))
									{
										ast_shrink_phone_number(p->ast_ch->cid.cid_num);
										ast_copy_string(p->cid_num, p->ast_ch->cid.cid_num, sizeof(p->cid_num));
									}
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_11
		            		else
		            			p->cid_num[0] = '\0';

		            		p->cid_name[0] = '\0';
		            		
#ifndef CC_AST_HAS_VERSION_1_8
								ast_set_callerid(p->ast_ch, p->cid_num, p->cid_name, NULL);

								if (option_verbose >= 3)
		                    		ast_verbose(VERBOSE_PREFIX_3 "dgv_call (%d): set callerid %s\n", p->port, p->cid_num);
#endif//#ifndef CC_AST_HAS_VERSION_1_8
		            		
		        			if (p->call_state == DG_AST_NONE)
		        			{
		        				ast_setstate(p->ast_ch, AST_STATE_RINGING);
		        				p->call_state = DG_AST_CALLING;
		        				
		        				dg_PickUp(p->port, 300);
		        			}
		        			else
		        			{
		            			ast_log(LOG_WARNING, "dgv_call: Port %d in use or reserved! \n", p->port);
		        			}
	        			}
        				else //if (p->channel_index != 2)
        				{
        					if (p->subchannel)
        					{
	        					pthread_attr_t attr;
	        					
			        			if (p->call_state == DG_AST_NONE)
			        			{
			        				ast_setstate(p->ast_ch, AST_STATE_RINGING);
			        				p->call_state = DG_AST_CALLING;
			        			}
	        					
	        					p->subchannel->subchanneltone_thread_running = 1;
	        					
	        					if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	        						ast_log(LOG_NOTICE, "dgv_call: channel_index %d, subchannel->channel_index %d - port %d\n",
	        								p->channel_index, p->subchannel->channel_index, p->port);

	                    		pthread_attr_init(&attr);
	                        	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	                        	if (pthread_create(&p->subchannel->subchanneltone_thread_id, &attr, SubChannelToneThread, &(p->port)) < 0)
	        						p->subchannel->subchanneltone_thread_running = 0;
	        	               	else
	        	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_START_END_CALL) != 0))
	                	               	ast_log(LOG_NOTICE, "dgv_call: subchanneltone_thread created on port %d\n", p->port);
	                    	   	pthread_attr_destroy(&attr);
	
								struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass.integer = AST_CONTROL_RINGING;
#else//#ifdef CC_AST_HAS_VERSION_1_8
									f.subclass = AST_CONTROL_RINGING;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
								f.src = "chan_dgv";
								f.datalen = 0;
								QueueFrame(p, &f);
	
	            				if (option_verbose >= 3)
	                				ast_log(LOG_NOTICE, "dgv_call: SubChannel Ring started on %d \n", p->port);
        					}
        					else //if (p->subchannel)
        					{
        						ast_log(LOG_ERROR, "dgv_call(%d): subchannel not connected! \n", p->port);
        					} //if (p->subchannel)
        				} //if (p->channel_index != 2)
					}//if (p->sig == DG_SIG_CB_FXS)
				}//if (p->sig == DG_SIG_FXS)
			}  				
    	}	
	}
	ast_dgv_unlock(&p->lock, p->port);

	return 0;
}

//-------------------------------------------------------------------------------------------
// Writes a frame of voice data to a digivoice channel 
//-------------------------------------------------------------------------------------------
static int dgv_write(struct ast_channel *ast, struct ast_frame *frame)
{
#ifdef DEBUG_Q
		unsigned char ctmp;
		char szTmp[1024];
		char char_tx[1024];
		char *tmp8;
#endif//#ifdef DEBUG_Q
	short x = 0;
	int remaining_space, total_space;

	struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);

	if (!p)
	{
		ast_log(LOG_WARNING, "dgv_write: where is p?\n");
		return 0;
	}
	
	if ((!frame) || (frame->frametype != AST_FRAME_VOICE) /*|| p->faxhandled > 0 */) {  /* don't play anything when in fax stage 1 s*/
		dg_StopPlayBuffer(p->port);
		if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
			ast_verbose("%s: dgv_write: Don't know how to handle from type %d\n", dgv_channel_name(ast), frame->frametype);
		//ast_dgv_unlock(&p->lock, p->port);
		return 0;
	}
	else 
		if (!p->ast_ch) 
		{
			dg_StopPlayBuffer(p->port);
            if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                ast_log(LOG_WARNING, "Dropping frame since there is no active owner...\n");
            //ast_dgv_unlock(&p->lock, p->port);
			return 0;
		}
		else 
			if ((dgv_channel_state(ast) != AST_STATE_UP) && (dgv_channel_state(ast) != AST_STATE_RING))
			{
				dg_StopPlayBuffer(p->port);
				if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
#ifdef CC_AST_HAS_VERSION_1_8
						ast_verbose("%s: dgv_write: Attempt to Write frame type[%d]subclass[%d] on not up chan(state[%d]) and channel_index %d\n", dgv_channel_name(ast), frame->frametype, frame->subclass.integer,dgv_channel_state(ast), p->channel_index);
#else//#ifdef CC_AST_HAS_VERSION_1_8
						ast_verbose("%s: dgv_write: Attempt to Write frame type[%d]subclass[%d] on not up chan(state[%d]) and channel_index %d\n", dgv_channel_name(ast), frame->frametype, frame->subclass,dgv_channel_state(ast), p->channel_index);
#endif//#ifdef CC_AST_HAS_VERSION_1_8
				//ast_dgv_unlock(&p->lock, p->port);
				return 0;
			}
			else
				if ((p->subchannel_audio_index != p->channel_index))
				{
					if (((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0) && option_debug == 80)
						ast_verbose("%s: dgv_write: Attempt to Write frame on subchannel_audio_index %d with channel_index %d, dropping...\n", dgv_channel_name(ast), p->subchannel_audio_index, p->channel_index);
					//ast_dgv_unlock(&p->lock, p->port);
					return 0;
				}

	/* Return if it's not valid data */
#ifdef CC_AST_HAS_DATA_PTR	
		if (!frame->data.ptr || !frame->datalen)
#else//#ifdef CC_AST_HAS_DATA_PTR		
		if (!frame->data || !frame->datalen)
#endif//#ifdef CC_AST_HAS_DATA_PTR		
	{
		//ast_dgv_unlock(&p->lock, p->port);
		return 0;
	}

	if (p->gen_ringback)
	{
		if (p->tone_thread_running == 1)
		{
			if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
				ast_log(LOG_WARNING, "Turning off tone_thread, dgv_write received on %d...\n", p->port);

			p->tone_thread_running = 2;
		} //if (p->tone_thread_running == 1)
	}

	//check for format to play
#ifdef CC_AST_HAS_VERSION_13
		if (p->write_last_format != ast_format_get_codec_id(frame->subclass.format))
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_1_8
			if (p->write_last_format != frame->subclass.integer)
	#else//#ifdef CC_AST_HAS_VERSION_1_8
			if (p->write_last_format != frame->subclass)
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_13
	{
		//format was changed since last time
		ast_dgv_lock(&p->play_lock, p->port);
		{
			dg_StopPlayBuffer(p->port);
		}
		ast_dgv_unlock(&p->play_lock, p->port);
		
		//assigns file format
#ifdef CC_AST_HAS_VERSION_13
			p->write_last_format = ast_format_get_codec_id(frame->subclass.format);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_1_8
				p->write_last_format = frame->subclass.integer;
	#else//#ifdef CC_AST_HAS_VERSION_1_8
				p->write_last_format = frame->subclass;
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_13
		dg_SetPlayFormat(p->port, ast2dgvformat(p->write_last_format));
		
		if (p->subchannel)
			p->subchannel->write_last_format = p->write_last_format;

		if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
#ifdef CC_AST_HAS_VERSION_13
				ast_log(LOG_NOTICE, "dgv_write: (%d) Assigning new format:%lld %s (%d)\n",
                        			p->port,p->write_last_format,ast2dgvformatname(p->write_last_format),
                        			ast2dgvformat(p->write_last_format));
#else//#ifdef CC_AST_HAS_VERSION_13
				ast_log(LOG_NOTICE, "dgv_write: (%d) Assigning new format:%x %s (%d)\n",
									p->port,p->write_last_format,ast2dgvformatname(p->write_last_format),
									ast2dgvformat(p->write_last_format));
#endif//#ifdef CC_AST_HAS_VERSION_13
	}

	//if (p->audio_debug && option_debug==80)
    //    ast_verbose("%s: dgv_write: Attempt to Write frame type[%d]subclass[%d] (state[%d]) - format %x\n",ast->name, frame->frametype, frame->subclass,dgv_channel_state(ast),p->write_last_format);

	switch(ast2dgvformat(p->write_last_format))
	{
		case ffGsm610:case ffWaveALaw:case ffWaveULaw:
#ifdef DEBUG_Q
				fwrite(frame->data, frame->datalen,1, p->play);
				
				/*szTmp[0]=0;
				tmp8 = (char *)frame->data;
				strcat(szTmp, "\n");
				for (x=0;x<frame->datalen;x++)
				{
					ctmp = *(char* )(tmp8 + x);
					sprintf(char_tx,"%02x ", ctmp);
					strcat(szTmp, char_tx);
					if (x==32) strcat(szTmp, "\n");
				}
				ast_log(LOG_NOTICE, "dgv_write: %s\n",szTmp);*/
#endif//#ifdef DEBUG_Q

            //if (option_debug==80)
            //    ast_log(LOG_NOTICE, "dgv_write: port %d samples=%d datalen=%d\n",p->port,frame->samples,frame->datalen);

            remaining_space=0;

            /* check buffer size first - passing 0 as size only returns the current remaining_space */
            ast_dgv_lock(&p->play_lock, p->port); 
            {                	                	
            	if (dg_GetPortStatus(p->port)!=spDialing)
#ifdef CC_AST_HAS_DATA_PTR
                		x=dg_PlayBuffer(p->port, (char*)frame->data.ptr, 0, &remaining_space);
#else//#ifdef CC_AST_HAS_DATA_PTR
            			x=dg_PlayBuffer(p->port, (char*)frame->data, 0, &remaining_space);
#endif//#ifdef CC_AST_HAS_DATA_PTR            			
                else
                    if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                    	ast_log(LOG_NOTICE, "Canceled dg_PlayBuffer because dg_GetPortStatus == spDialing on %d \n", p->port);
            }
            ast_dgv_unlock(&p->play_lock, p->port);
               
            if (p->audio_debug && option_debug == 80 /*&& remaining_space!=6144*/)
#ifdef CC_AST_HAS_VERSION_13
            		ast_log(LOG_NOTICE, "dgv_write: port %d samples=%d datalen=%d fmt=%lld\n - fr.subclass=%x - remaining space=%d\n",p->port,frame->samples,
#else//#ifdef CC_AST_HAS_VERSION_13
            		ast_log(LOG_NOTICE, "dgv_write: port %d samples=%d datalen=%d fmt=%x\n - fr.subclass=%x - remaining space=%d\n",p->port,frame->samples,
#endif//#ifdef CC_AST_HAS_VERSION_13
#ifdef CC_AST_HAS_VERSION_1_8
							frame->datalen,p->write_last_format,frame->subclass.integer, remaining_space);
#else//#ifdef CC_AST_HAS_VERSION_1_8
							frame->datalen,p->write_last_format,frame->subclass, remaining_space);
#endif//#ifdef CC_AST_HAS_VERSION_1_8

            /* Tests to keep playbuffer less than 100ms*/
            if (ast2dgvformat(p->write_last_format)==ffGsm610)      /*   gsm    */
            {
                total_space = WRITE_BUFFER_LINES*SAMPLES_SIZE_GSM;
                //gsm*8 means > 160ms
                if (((total_space-remaining_space) > SAMPLES_SIZE_GSM*p->drop_tx_samples_factor) || p->force_drop_tx>0  )   {
                    if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                        ast_log(LOG_NOTICE, "playbuffer (%d): too much delay gsm - %d ms. Dropping samples. %d %d \n",p->port,
                                                                                ((total_space-remaining_space)*20)/33, 
                                                                                p->drop_tx_samples_factor,
                                                                                p->force_drop_tx);
                    dg_StopPlayBuffer(p->port);
                    p->force_drop_tx=0;
                }
                else
                {
                    //send current buffer
                    ast_dgv_lock(&p->play_lock, p->port); 
                    {
                    	if (dg_GetPortStatus(p->port)!=spDialing)
#ifdef CC_AST_HAS_DATA_PTR                    		
                    			x=dg_PlayBuffer(p->port, (char*)frame->data.ptr, frame->datalen, NULL);
#else//#ifdef CC_AST_HAS_DATA_PTR                    	
                    			x=dg_PlayBuffer(p->port, (char*)frame->data, frame->datalen, NULL);
#endif//#ifdef CC_AST_HAS_DATA_PTR                    			
                    	else	
                    		if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                        		ast_log(LOG_NOTICE, "Canceled dg_PlayBuffer because dg_GetPortStatus == spDialing on %d \n", p->port);                        		
                    }
                    ast_dgv_unlock(&p->play_lock, p->port);
                }
            }
            else
            {
                 total_space = WRITE_BUFFER_LINES*SAMPLES_SIZE;
                 //DATALEN*8 means > 160ms
                 if ( ( ((total_space-remaining_space) > DATALEN*p->drop_tx_samples_factor) || p->force_drop_tx>0 ) && (!p->faxhandled) )
                 {
                     if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                     {
                         ast_log(LOG_NOTICE, "playbuffer (%d): too much delay g711 - %d ms. - total=%d remaining=%d - Dropping samples.\n",p->port,
                                                                                     (total_space-remaining_space)/8,
                                                                                        total_space, remaining_space);
                     }
                     //stops buffer to drop all buffer content
                     dg_StopPlayBuffer(p->port);
                     p->force_drop_tx=0;
                 }
                 else
                 {
                     //send current buffer
                     ast_dgv_lock(&p->play_lock, p->port); 
                     {
                     	if (dg_GetPortStatus(p->port)!=spDialing)	
#ifdef CC_AST_HAS_DATA_PTR                     		
								x=dg_PlayBuffer(p->port, (char*)frame->data.ptr, frame->datalen, NULL);
#else//#ifdef CC_AST_HAS_DATA_PTR
								x=dg_PlayBuffer(p->port, (char*)frame->data, frame->datalen, NULL);
#endif//#ifdef CC_AST_HAS_DATA_PTR
                     	else
                     		if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                        		ast_log(LOG_NOTICE, "Canceled dg_PlayBuffer because dg_GetPortStatus == spDialing on %d \n", p->port);	
                     } 
                     ast_dgv_unlock(&p->play_lock, p->port);
                 }

            }

			if (x!=DG_EXIT_SUCCESS)
			{
				ast_log(LOG_NOTICE, "playbuffer: full buffer ret=%d - samples=%d datalen=%d remaining=%d\n",x,frame->samples,frame->datalen,remaining_space);
			}	

			break;
		case ffWavePCM:
	        if (p->audio_debug)
		        ast_log(LOG_NOTICE, "dgv_write: wavepcm samples=%d datalen=%d\n",frame->samples,frame->datalen);

            remaining_space=0;

			ast_dgv_lock(&p->play_lock, p->port); 
			{
				if (dg_GetPortStatus(p->port)!=spDialing)
#ifdef CC_AST_HAS_DATA_PTR					
			  			dg_PlayBuffer(p->port, (short*)frame->data.ptr, 0, &remaining_space);
#else//#ifdef CC_AST_HAS_DATA_PTR				
						dg_PlayBuffer(p->port, (short*)frame->data, 0, &remaining_space);
#endif//#ifdef CC_AST_HAS_DATA_PTR					
			  	else
                    if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                        ast_log(LOG_NOTICE, "Canceled dg_PlayBuffer because dg_GetPortStatus == spDialing on %d \n", p->port);    			  		
			} 
			ast_dgv_unlock(&p->play_lock, p->port);

            total_space = WRITE_BUFFER_LINES*SAMPLES_SIZE;
            // DATALEN*8 means > 160ms
            if ( ( ((total_space-remaining_space) > DATALEN*p->drop_tx_samples_factor) || p->force_drop_tx>0 ) && (!p->faxhandled) )
            {
                if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                    ast_log(LOG_NOTICE, "playbuffer (%d): too much delay slin - %d ms. Dropping samples.\n",p->port,
                                                                        (total_space-remaining_space)/8);

                dg_StopPlayBuffer(p->port);
                p->force_drop_tx=0;
            }
            else
            {
                ast_dgv_lock(&p->play_lock, p->port); 
                {
                	if (dg_GetPortStatus(p->port)!=spDialing)
#ifdef CC_AST_HAS_DATA_PTR
                			dg_PlayBuffer(p->port, (short*)frame->data.ptr, frame->samples, NULL);
#else//#ifdef CC_AST_HAS_DATA_PTR
                			dg_PlayBuffer(p->port, (short*)frame->data, frame->samples, NULL);
#endif//#ifdef CC_AST_HAS_DATA_PTR
                	else
                	   	if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
                        	ast_log(LOG_NOTICE, "Canceled dg_PlayBuffer because dg_GetPortStatus == spDialing on %d \n", p->port);
                } 
                ast_dgv_unlock(&p->play_lock, p->port);
            }
			break;
		default:
			if ((p->verbosity & DG_LOG_WRITE_SAMPLES) != 0)
#ifdef CC_AST_HAS_VERSION_13
					ast_log(LOG_WARNING, "write_last_format(%d) not supported, please check codec %lld\n", p->port, p->write_last_format);
#else//#ifdef CC_AST_HAS_VERSION_13
					ast_log(LOG_WARNING, "write_last_format(%d) not supported, please check codec %x\n", p->port, p->write_last_format);
#endif//#ifdef CC_AST_HAS_VERSION_13
	}
	return 0;
}


//-------------------------------------------------------------------------------------------
// Hangup procedure
//-------------------------------------------------------------------------------------------
static int dgv_hangup(struct ast_channel *ast)
{
#ifdef DEBUG_PRI	
		ast_log(LOG_NOTICE, "dgv_hangup: STEP 1...\n");	
#endif//#ifdef DEBUG_PRI
		
    struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);
    struct dgv_pvt *p_bridged;

    int nForReset;
    int res_sched;
    int ret;

	pthread_attr_t attr;

#ifdef USE_LIBPRI
		int dc;
#endif//#ifdef USE_LIBPRI

	short trycnt = 0;
	short exec_lock = 0;

	if (!p)
	{
		if (option_verbose >= 3)		
			ast_log(LOG_NOTICE, "dgv_hangup(%s): !p \n", dgv_channel_name(ast));
		return 0;		
	}    

	for (;;)
	{
		if (p)
		{
			if (exec_lock)
			{
				ast_dgv_lock(&p->lock, p->port);
				break;
			}
			else
				if (ast_dgv_trylock(&p->lock, p->port))
				{
					usleep(1 * 1000L);
					trycnt++;

					if (trycnt>=1000)
					{
						trycnt = 0;
						exec_lock = 1;

						#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
							#ifdef CC_AST_HAS_VERSION_1_6
								DEADLOCK_AVOIDANCE(&ast->lock_dont_use);
							#else//#ifndef CC_AST_HAS_VERSION_1_6
								CHANNEL_DEADLOCK_AVOIDANCE(ast);
							#endif//#ifndef CC_AST_HAS_VERSION_1_6
						#else//#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
							#ifdef CC_AST_HAS_VERSION_1_6
								ast_dgv_unlock(&ast->lock_dont_use, p->port);
								usleep(1);
								ast_dgv_lock(&ast->lock_dont_use, p->port);
							#else//#ifdef CC_AST_HAS_VERSION_1_6
								#ifdef CC_AST_HAS_VERSION_1_8
									ast_channel_unlock(ast);
									usleep(1);
									ast_channel_lock(ast);
								#else//#ifdef CC_AST_HAS_VERSION_1_8
									ast_dgv_unlock(&ast->lock, p->port);
									usleep(1);
									ast_dgv_lock(&ast->lock, p->port);
								#endif//#ifdef CC_AST_HAS_VERSION_1_8
							#endif//#ifdef CC_AST_HAS_VERSION_1_6
						#endif//#if (defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)) && !defined(DEBUG_LOCKS)
					}
				}
				else
					break;
		}
	}

	/* here we are already locked! */
	{
    	if ((!p->ast_ch) || (!dgv_channel_tech(ast)))
    	{
    		if (!dgv_channel_tech(ast))
    			ast_log(LOG_WARNING, "dgv_hangup: channel does not have tech?\n");    			
    		else    		
    			ast_log(LOG_WARNING, "dgv_hangup: channel not connected?\n");

	    	//ids strings
			p->exten[0]        = '\0';
	    	p->rdnis[0]        = '\0';
	    	p->dnid[0]         = '\0';
	    	p->cid_num[0]      = '\0';
	    	p->cid_name[0]     = '\0';
	    	p->pri_cid_num[0]  = '\0';
	    	p->pri_cid_name[0] = '\0';
	    	p->cid_category[0] = '\0';

	        p->sms_received_from[0]      = '\0';
	        p->sms_received_simcardid[0] = '\0';
	        p->sms_received_date[0]      = '\0';
	        p->sms_received_time[0]      = '\0';
	        p->sms_received_text[0]      = '\0';
	        p->sms_received_st_cod[0]    = '\0';
	        p->sms_received_mr[0]        = '\0';
	        p->ussd_received_text[0]     = '\0';

	        p->reserved=0;
	        p->group_b_port_bridged = 0;
	        p->category_port_bridged = 0;

	    	p->call_state = DG_AST_NONE;
	    	
	        p->write_last_format=-1;		//initializes with invalid format
	        p->read_last_format=-1;

	        p->ast_ch = NULL;
	        dgv_channel_tech_pvt_set(ast, NULL);

            if (p->auto_dgccb)
            	p->flashing = 1;
            else
            	p->flashing = 0;

	        p->need_fv_thread = 0;        
	        p->need_sc_ring = 0;
	        p->need_ringback = 0;
	        for (nForReset = 0; nForReset < 20; nForReset++)
	        {
	        	p->dialplan_senddtmf[nForReset] = '\0';
	        	p->dialplan_group_b[nForReset] = '\0';
	        }

	        dg_HangUp(p->port);
	        
			ast_dgv_unlock(&p->lock, p->port);
			return 0;
		}

    	if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
    		ast_verbose(VERBOSE_PREFIX_3 "Hanging up Port %d \n",p->port);
	
#ifdef DEBUG_Q
			if (p->rec) fclose(p->rec);
			if (p->play) fclose(p->play);
#endif//#ifdef DEBUG_Q	
	
    	ast_setstate(ast, AST_STATE_DOWN);

    	if (p->tone_thread_running == 1)
    	{
    		p->tone_thread_running = 2;

    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
        		ast_log(LOG_NOTICE, "  -- Port %d hangup - tone thread canceled\n",p->port);
    	}
    	
    	if (p->stuttertone_thread_running == 1)
    	{
    		p->stuttertone_thread_running = 0;
    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
        		ast_log(LOG_NOTICE, "  -- Port %d hangup - stutter tone thread canceled\n",p->port);
    	}

    	if (p->subchanneltone_thread_running == 1)
    	{
    		p->subchanneltone_thread_running = 0;
    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
        		ast_log(LOG_NOTICE, "  -- Port %d hangup - subchannel ring tone thread canceled\n", p->port);
    	}

    	if (p->subchannel)
    	{
    		if (p->subchannel->subchanneltone_thread_running == 1)
    		{
    			p->subchannel->subchanneltone_thread_running = 0;
    			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
    				ast_log(LOG_NOTICE, "  -- Port %d hangup - subchannel ring tone thread canceled\n", p->port);	
    		}

			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				ast_log(LOG_NOTICE, "Port %d, channel_index %d, subchannel_audio_index %d\n", p->port, p->channel_index, p->subchannel_audio_index);

			switch (p->channel_index)
			{
				case 1:
					switch (p->subchannel_audio_index)
					{
						case 0:
			    			/*
			    			 * (Ring) A -> B
			    			 *             C (Ring)
			    			 * A -> HungUp
			    			 */
							
		 					dg_PauseInputBuffer(p->port, DG_PAUSE);
			                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
			                	ast_log(LOG_WARNING, "dgv_hangup: Pausing InputBuffer on port %d\n", p->port);
		 					dg_StopPlayBuffer(p->port);
		 					
			 				if (p->subchannel->ast_ch)
			 				{
			 					if (p->ast_ch)
			 					{
									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "dgv_hangup: Dropping first call on port %d\n", p->port);

									swap_sc(p, p->subchannel);

									dgv_channel_tech_pvt_set(p->ast_ch, dgv_channel_tech_pvt(p->subchannel->ast_ch));
			 					}
			 					else
			 					{
									if (option_verbose >= 3)
										ast_log(LOG_WARNING, "dgv_hangup: p->ast_ch does not exist on port %d\n", p->port);			 						
			 					}
			 				}
			 				else
			 				{
								if (option_verbose >= 3)
									ast_log(LOG_WARNING, "dgv_hangup: p->subchannel->ast_ch does not exist on port %d\n", p->port);
							}
			 			
			    	        p->subchannel->ast_ch = NULL;
			    	        dgv_channel_tech_pvt_set(ast, NULL);
			    	    	p->subchannel->subchannel = NULL;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        	ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        	ast_dgv_lock(&usecnt_lock, 0);
					        	usecnt--;
					        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        ast_update_use_count();

							ast_dgv_unlock(&p->lock, p->port);

							ast_mutex_destroy(&p->subchannel->record_lock);
							ast_mutex_destroy(&p->subchannel->play_lock);
							ast_mutex_destroy(&p->subchannel->lock);

							free(p->subchannel);
							p->subchannel = NULL;
							
				    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				        		ast_log(LOG_NOTICE, "Port %d hungup - subchannel was freed\n", p->port);
							
							break;
						case 1:
		     				if (p->subchannel->ast_ch)
		     				{
		     					if (p->ast_ch)
		     					{
		     						dg_PauseInputBuffer(p->port, DG_PAUSE);
					                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
					                	ast_log(LOG_WARNING, "dgv_hangup: Pausing InputBuffer on port %d\n", p->port);
		     						dg_StopPlayBuffer(p->port);

		     						p->write_last_format=-1;
		     						
		                    		pthread_attr_init(&attr);
		                        	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
		                        	if (!pthread_create(&p->beeptone_thread_id, &attr, BeepToneThread, &(p->port)) < 0)
		        	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_START_END_CALL) != 0))
		                	               	ast_log(LOG_NOTICE, "dgv_hangup: beeptone_thread created on port %d\n", p->port);
		                    	   	pthread_attr_destroy(&attr);

#ifdef CC_AST_HAS_VERSION_12
										RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
										struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
		     						dgv_chan_bridged = dgv_bridged_channel(p->subchannel->ast_ch);
		     						if (dgv_chan_bridged)
		     						{
						    			/*
						    			 * (Up) A -> B
						    			 *           C (MOH)
						    			 * A -> HungUp
						    			 */

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Stopping music on hold on '%s'\n", dgv_channel_name(p->ast_ch));
										
										struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											fmoh.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											fmoh.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										fmoh.src = "chan_dgv";													
										fmoh.datalen = 0;
										QueueFrame(p->subchannel, &fmoh);										

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Exchanging call on port %d\n", p->port);
	
										swap_sc(p, p->subchannel);

										dgv_channel_tech_pvt_set(p->ast_ch, dgv_channel_tech_pvt(p->subchannel->ast_ch));
				     					
				     					dg_PauseInputBuffer(p->port, DG_RELEASE);
						                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
						                	ast_log(LOG_WARNING, "dgv_hangup: Releasing InputBuffer on port %d\n", p->port);
									}
									else
									{
						    			/*
						    			 * (Up) A -> B
						    			 *           C (Ring)
						    			 * A -> HungUp
						    			 */

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: dgv_chan_bridged does not exist on port %d\n", p->port);
										
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Exchanging call on port %d\n", p->port);

										swap_sc(p, p->subchannel);
	
										dgv_channel_tech_pvt_set(p->ast_ch, dgv_channel_tech_pvt(p->subchannel->ast_ch));
										
										dg_PauseInputBuffer(p->port, DG_RELEASE);
						                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
						                	ast_log(LOG_WARNING, "dgv_hangup: Releasing InputBuffer on port %d\n", p->port);
										
										struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass.integer = AST_CONTROL_ANSWER;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											f.subclass = AST_CONTROL_ANSWER;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										f.src = "chan_dgv";
										f.datalen = 0;
										QueueFrame(p, &f);

										if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
											ast_log(LOG_NOTICE, "dgv_hangup (%d): ast_setstate(AST_STATE_UP). \n", p->port);										
									}
		     					}
		     					else
		     					{
									if (option_verbose >= 3)
										ast_log(LOG_WARNING, "dgv_hangup: p->ast_ch does not exist on port %d\n", p->port);
		     					}
		     				}
		     				else
		     				{
								if (option_verbose >= 3)
									ast_log(LOG_WARNING, "dgv_hangup: p->subchannel->ast_ch does not exist on port %d\n", p->port);
							}
		     			
		        	        p->subchannel->ast_ch = NULL;
		        	        dgv_channel_tech_pvt_set(ast, NULL);
		        	    	p->subchannel->subchannel = NULL;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        	ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        	ast_dgv_lock(&usecnt_lock, 0);
		    		        	usecnt--;
		    		        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        ast_update_use_count();

		    				ast_dgv_unlock(&p->lock, p->port);

		    				ast_mutex_destroy(&p->subchannel->record_lock);
		    				ast_mutex_destroy(&p->subchannel->play_lock);
		    				ast_mutex_destroy(&p->subchannel->lock);

		    				free(p->subchannel);		    				
		    				p->subchannel = NULL;
		    				
				    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				        		ast_log(LOG_NOTICE, "Port %d hungup - subchannel was freed\n", p->port);		    				

		    				break;
						case 2:
			    			/*
			    			 * (MOH) A -> B
			    			 *            C (Up)
			    			 * A -> HungUp
			    			 */
							
 	     					dg_PauseInputBuffer(p->port, DG_PAUSE);
			                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
			                	ast_log(LOG_WARNING, "dgv_hangup: Pausing InputBuffer on port %d\n", p->port);
 	     					dg_StopPlayBuffer(p->port);
 	     					
 							p->write_last_format=-1;
 							p->read_last_format=-1;

			 				if (p->subchannel->ast_ch)
			 				{
			 					if (p->ast_ch)
			 					{
#ifdef CC_AST_HAS_VERSION_12
										RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
										struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
			 						dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
	     							if (dgv_chan_bridged)
	     							{
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Dropping first call on port %d\n", p->port);

	     								p->subchannel_audio_index = 1;

	     								swap_sc_tech(p, p->subchannel);

	     		     					dg_PauseInputBuffer(p->port, DG_RELEASE);
	    				                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
	    				                	ast_log(LOG_WARNING, "dgv_hangup: Releasing InputBuffer on port %d\n", p->port);
	     							}
	     							else
	     							{
	     								if (option_verbose >= 3)
	     									ast_log(LOG_WARNING, "dgv_hangup: dgv_chan_bridged does not exist on port %d\n", p->port);
	     							}
			 					}
			 					else
			 					{
									if (option_verbose >= 3)
										ast_log(LOG_WARNING, "dgv_hangup: p->ast_ch does not exist on port %d\n", p->port);			 						
			 					}
			 				}
			 				else
			 				{
								if (option_verbose >= 3)
									ast_log(LOG_WARNING, "dgv_hangup: p->subchannel->ast_ch does not exist on port %d\n", p->port);
							}

     	        	        p->subchannel->ast_ch = NULL;
     	        	        dgv_channel_tech_pvt_set(ast, NULL);
     	        	    	p->subchannel->subchannel = NULL;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
     	    		        	ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
     	    		        	ast_dgv_lock(&usecnt_lock, 0);
     	    		        	usecnt--;
     	    		        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
     	    		        ast_update_use_count();

     	    				ast_dgv_unlock(&p->lock, p->port);

     	    				ast_mutex_destroy(&p->subchannel->record_lock);
     	    				ast_mutex_destroy(&p->subchannel->play_lock);
     	    				ast_mutex_destroy(&p->subchannel->lock);

     	    				free(p->subchannel);     	    				
     	    				p->subchannel = NULL;

				    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				        		ast_log(LOG_NOTICE, "Port %d hungup - subchannel was freed\n", p->port);

     	    				break;
					} //switch (p->subchannel_audio_index)
					break;
				case 2:
					switch (p->subchannel_audio_index)
					{
						case 1:
			    			/*
			    			 * (Up) A -> B
			    			 *           C (Ring)
			    			 * C -> HungUp
			    			 */
							
			    			/*
			    			 * (Ring) A -> B
			    			 *             C (Ring)
			    			 * C -> HungUp
			    			 */
							
			    			/*
			    			 * (Up) A -> B
			    			 *           C (MOH)
			    			 * C -> HungUp
			    			 */

							if (option_verbose >= 3)
								ast_log(LOG_NOTICE, "dgv_hangup: Dropping second call on port %d\n", p->port);
							
			    	        p->ast_ch = NULL;
			    	        dgv_channel_tech_pvt_set(ast, NULL);
			    	    	p->subchannel->subchannel = NULL;
			    			p->subchannel = NULL;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        	ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        	ast_dgv_lock(&usecnt_lock, 0);
					        	usecnt--;
					        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
					        ast_update_use_count();

							ast_dgv_unlock(&p->lock, p->port);

							ast_mutex_destroy(&p->record_lock);
							ast_mutex_destroy(&p->play_lock);
							ast_mutex_destroy(&p->lock);

							free(p);
							
				    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				        		ast_log(LOG_NOTICE, "Port %d hungup - channel was freed\n", p->port);

							break;
						case 2:
			    			/*
			    			 * (MOH) A -> B
			    			 *            C (Up)
			    			 * C -> HungUp
			    			 */							
							
		     				if (p->subchannel->ast_ch)
		     				{							
			     				if (p->ast_ch)
			     				{
			     					dg_PauseInputBuffer(p->port, DG_PAUSE);
					                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
					                	ast_log(LOG_WARNING, "dgv_hangup: Pausing InputBuffer on port %d\n", p->port);
			     					dg_StopPlayBuffer(p->port);
	
									p->subchannel->write_last_format=-1;
									
		                    		pthread_attr_init(&attr);
		                        	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
		                        	if (!pthread_create(&p->beeptone_thread_id, &attr, BeepToneThread, &(p->port)) < 0)
		        	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_START_END_CALL) != 0))
		                	               	ast_log(LOG_NOTICE, "dgv_hangup: beeptone_thread created on port %d\n", p->port);
		                    	   	pthread_attr_destroy(&attr);									

#ifdef CC_AST_HAS_VERSION_12
										RAII_VAR(struct ast_channel *, dgv_chan_bridged, NULL, ast_channel_cleanup);
#else//#ifdef CC_AST_HAS_VERSION_12
										struct ast_channel *dgv_chan_bridged;
#endif//#ifdef CC_AST_HAS_VERSION_12
									dgv_chan_bridged = dgv_bridged_channel(p->ast_ch);
									if (dgv_chan_bridged)
									{
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Stopping music on hold on '%s'\n", dgv_channel_name(p->ast_ch));
										
										struct ast_frame fmoh = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
											fmoh.subclass.integer = AST_CONTROL_UNHOLD;
#else//#ifdef CC_AST_HAS_VERSION_1_8
											fmoh.subclass = AST_CONTROL_UNHOLD;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
										fmoh.src = "chan_dgv";													
										fmoh.datalen = 0;
										QueueFrame(p, &fmoh);										
										
										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "dgv_hangup: Exchanging call on port %d\n", p->port);
	
										swap_sc(p, p->subchannel);
										
										p->subchannel->subchannel_audio_index = 1;
										
				     					dg_PauseInputBuffer(p->port, DG_RELEASE);
						                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
						                	ast_log(LOG_WARNING, "dgv_hangup: Releasing InputBuffer on port %d\n", p->port);
									}
									else
									{
										if (option_verbose >= 3)
											ast_log(LOG_WARNING, "dgv_hangup: dgv_chan_bridged does not exist on port %d\n", p->port);
									}
			     				}
			     				else
			     				{
									if (option_verbose >= 3)
										ast_log(LOG_WARNING, "dgv_hangup: p->ast_ch does not exist on port %d\n", p->port);
								}
		     				}
		     				else
		     				{
								if (option_verbose >= 3)
									ast_log(LOG_WARNING, "dgv_hangup: p->subchannel->ast_ch does not exist on port %d\n", p->port);
							}		     				
		     			
		        	        p->ast_ch = NULL;
		        	        dgv_channel_tech_pvt_set(ast, NULL);
		        	    	p->subchannel->subchannel = NULL;
		        			p->subchannel = NULL;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        	ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        	ast_dgv_lock(&usecnt_lock, 0);
		    		        	usecnt--;
		    		        	ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		    		        ast_update_use_count();
		    		        
		    				ast_dgv_unlock(&p->lock, p->port);

		    				ast_mutex_destroy(&p->record_lock);
		    				ast_mutex_destroy(&p->play_lock);
		    				ast_mutex_destroy(&p->lock);

		    				free(p);
		    				
				    		if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				        		ast_log(LOG_NOTICE, "Port %d hungup - channel was freed\n", p->port);		    				

		    				break;
					} //switch (p->subchannel_audio_index)
					break;
			} //switch (p->channel_index)

			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				ast_verbose(VERBOSE_PREFIX_3 "Port %d subchannel hungup\n", p->port);

			return 0;
    	} //if (p->subchannel)

    	if ((p->sig == DG_SIG_GSM) && (p->channel_index == 2))
    	{
			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				ast_log(LOG_NOTICE, "Port %d, channel_index %d, smschannel_index %d\n", p->port, p->channel_index, p->smschannel_index);

	        p->sms_received_from[0]      = '\0';
	        p->sms_received_simcardid[0] = '\0';
	        p->sms_received_date[0]      = '\0';
	        p->sms_received_time[0]      = '\0';
	        p->sms_received_text[0]      = '\0';
	        p->sms_received_st_cod[0]    = '\0';
	        p->sms_received_mr[0]        = '\0';
	        p->ussd_received_text[0]     = '\0';

			p->sms_reserved=0;
			
			p->call_state = DG_AST_NONE;
			p->ast_ch = NULL;
			dgv_channel_tech_pvt_set(ast, NULL);

			ast_dgv_unlock(&p->lock, p->port);

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				ast_dgv_lock(&usecnt_lock, 0);
				usecnt--;
				ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			ast_update_use_count();
    		
			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				ast_verbose(VERBOSE_PREFIX_3 "Port %d smschannel hungup\n", p->port);

    		return 0;
    	}

    	ast_dgv_lock(&p->play_lock, p->port);
    	{
    		dg_StopPlayBuffer(p->port);
    	}
    	ast_dgv_unlock(&p->play_lock, p->port);
    	
    	ast_dgv_lock(&p->record_lock, p->port);
    	{
    		//dg_DisableInputBuffer(p->port);
            dg_PauseInputBuffer(p->port, DG_PAUSE);
            if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
            	ast_log(LOG_WARNING, "dgv_hangup: Pausing InputBuffer on port %d\n", p->port);
    	}
    	ast_dgv_unlock(&p->record_lock, p->port);

        if (p->echo_can)
        {
            dg_DisableEchoCancelation(p->port);
    		if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
				ast_log(LOG_NOTICE, "  -- Port %d Disabling Echo Cancellation \n",p->port);            
        }

    	dg_DisableAnswerDetection(p->port);

#ifdef USE_LIBPRI
    		//isdn controls
    		p->alerting = 0;
    		p->outgoing = 0;
        	p->proceeding = 0;
        	p->progress = 0;
        	p->reverse_charging = -1;
        	p->useruserinfo[0] = '\0';
    	
#ifdef DEBUG_PRI    		
    			ast_log(LOG_NOTICE, "dgv_hangup: STEP 2 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI    			

			//last parameter is the cause - need to be setted
    	    if (p->pri)
        	{
				if (p->ch_board > 30)
					dc = dg_pri_findindex(p->board, 2);
				else
					dc = dg_pri_findindex(p->board, 1);

#ifdef USERUSER_INFO
					const char *useruser = pbx_builtin_getvar_helper(ast,"DGV_USERUSERINFO");
#endif//#ifdef USERUSER_INFO
    	    	if (!dgv_trylock_ccs(p, dc))
    	    	{
    	    		if (p->alreadyhungup)
	            	{
	                	if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                    	ast_log(LOG_NOTICE, "dgv_hangup: Already hungup... Calling hangup once, and clearing call on %d\n", p->port);

#ifdef USERUSER_INFO
							if (p->call)
								pri_call_set_useruser(p->call, useruser);
#endif//#ifdef USERUSER_INFO
	                    
	                	if (p->call)
	                	{
	                		if (dgv_channel_hangupcause(p->ast_ch) != PRI_CAUSE_INVALID_CALL_REFERENCE)
	                			pri_hangup(p->pri, p->call, -1);
	                		else
	                			ast_log(LOG_ERROR, "dgv_hangup: did not call pri_hangup because hangupcause is %d on %d(alreadyhungup)\n", dgv_channel_hangupcause(p->ast_ch), p->port);
	                	}
	                	else
	                		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                			ast_log(LOG_WARNING, "dgv_hangup: did not call pri_hangup because p->call is null on %d (alreadyhungup)\n", p->port);
	
	                	//0.9.8 p->pri = NULL;
	                	p->call = NULL;
	            	} //if (p->alreadyhungup)
	            	else
	            	{
		                const char *cause = pbx_builtin_getvar_helper(ast, "PRI_CAUSE");
	    	            int icause = dgv_channel_hangupcause(ast) ? dgv_channel_hangupcause(ast) : -1;
	                
	        	        if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            	        ast_log(LOG_NOTICE, "dgv_hangup: Not yet hungup... Calling hangup once with cause %s, icause (%d), dgv_channel_hangupcause(ast) %d and clearing call\n", cause, icause, dgv_channel_hangupcause(ast));
	                    
#ifdef USERUSER_INFO
							if (p->call)
								pri_call_set_useruser(p->call, useruser);
#endif//#ifdef USERUSER_INFO

		                p->alreadyhungup = 1;
	                
	    	            if (cause)
	    	            {
	        	            if (atoi(cause))
	        	            {
	            	            icause = atoi(cause);
	            	            if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	            	            	ast_log(LOG_NOTICE, "dgv_hangup: Setting icause = %d\n", icause);            	            
	        	            }
	                	}
	
	                	if (p->call)
	                	{
	            	        if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                	        ast_log(LOG_NOTICE, "dgv_hangup: Calling pri_hangup... icause %d on port %d \n", icause, p->port);
	                		
	    	            	pri_hangup(p->pri, p->call, icause);
	    	            	
	    	            	//p->call = NULL; 0.9.7_rc3
	                	}
	                	else
	                		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
	                			ast_log(LOG_WARNING, "dgv_hangup: did not call pri_hangup because p->call is null on %d (!alreadyhungup)\n", p->port);
	            	} //if (p->alreadyhungup)
	            	
	            	dgv_releaselock_ccs(dc);
    	    	} 
    	    	else 
    	    		ast_log(LOG_WARNING, "dgv_hangup: Unable to grab ccs_lock on port %d\n", p->port);
        	}
    	    else
    	        if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
        	        ast_log(LOG_NOTICE, "dgv_hangup: !p->pri on port %d \n", p->port);
#endif//#ifdef USE_LIBPRI

		if ((p->sig == DG_SIG_FXS) || (p->sig == DG_SIG_CB_FXS)) 
		{
			p->subchannel_audio_index = 0;
			//if (p->ast_ch)
			//{								 
			switch (p->call_state)
			{
				case DG_AST_CALLING:
	                if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
						ast_log(LOG_NOTICE, "dgv_hangup: Disabling SetRing on port %d \n", p->port);
										
	                if (p->sig == DG_SIG_FXS)
	                	dg_SetRing(p->port, DG_DISABLE, p->ring_type);
	                else
	                	dg_HangUp(p->port);
	                
					p->reserved=0;
					//p->ast_ch = NULL;											
					break;						
				case DG_AST_CONVERSATION:
				case DG_AST_RINGING:
					if (option_verbose >= 3)
    					ast_log(LOG_NOTICE, "Calling dgv_indicate with AST_CONTROL_BUSY on port %d \n", p->port);
    
#ifdef CC_AST_HAS_INDICATE_DATA
						dgv_indicate(p->ast_ch, AST_CONTROL_BUSY, NULL, 0);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
						dgv_indicate(p->ast_ch, AST_CONTROL_BUSY);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA
					break;
				case DG_AST_DIALING:
					if (p->sig == DG_SIG_CB_FXS)
					{
						p->reserved=0;
						if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
		                	ast_log(LOG_NOTICE, "dgv_hangup: Changing reserved to 0 on DG_SIG_CB_FXS when DIALING port %d \n", p->port);							
						break;
					}
				case DG_AST_HANGINGUP:
	                if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
	                	ast_log(LOG_NOTICE, "dgv_hangup: Nothing to do when DIALING or HANGINGUP on port %d \n", p->port);
	                
	                	//ast_log(LOG_NOTICE, "dgv_hangup: Setting p->ast_ch = NULL on port %d \n", p->port);												
					//p->ast_ch = NULL;
					break;			
				default:
	                if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
						ast_log(LOG_NOTICE, "dgv_hangup: p->call_state = %d on port %d \n", p->call_state, p->port);
											
	                //0.9.8 break;						
			}
			//}
		}
		else //if ((p->sig == DG_SIG_FXS) || (p->sig == DG_SIG_CB_FXS))
		{
			if (option_verbose >= 3)
               	ast_log(LOG_NOTICE, "Calling dg_HangUp on port %d\n", p->port);

#ifdef DEBUG_PRI
				ast_log(LOG_NOTICE, "dgv_hangup: STEP 3 on port %d\n", p->port);			
#endif//#ifdef DEBUG_PRI

			dg_HangUp(p->port);

			if (dg_GetCardInterface(dg_GetCardNumber(p->port)) == DG_FX_INTERFACE && p->sig == DG_SIG_FXO)
				dg_SetTwist(p->port, 12, 0);

		} //if ((p->sig == DG_SIG_FXS) || (p->sig == DG_SIG_CB_FXS))
		
#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "dgv_hangup: STEP 4 on port %d\n", p->port);		
#endif//#ifdef DEBUG_PRI

		update_statistics_call(p);
		
    	//ids strings
		p->exten[0]        = '\0';
    	p->rdnis[0]        = '\0';
    	p->dnid[0]         = '\0';
    	p->cid_num[0]      = '\0';
    	p->cid_name[0]     = '\0';
    	p->pri_cid_num[0]  = '\0';
    	p->pri_cid_name[0] = '\0';
    	p->cid_category[0] = '\0';
    
#ifdef DEBUG_PRI
    		ast_log(LOG_NOTICE, "dgv_hangup: STEP 5 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI
    	
		if (p->reserved_timeout_sched > -1)//stop reserved timeout
		{
			if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
				ast_log(LOG_NOTICE, "dgv_hangup (%d) - deleting reserved schedule\n", p->port);    	    		            	
		            	
			res_sched = ast_sched_del(sched, p->reserved_timeout_sched);
	    	if (res_sched == -1)
	    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

			p->reserved_timeout_sched = -1;
		}    		
    		
        if (p->reserved>0)
        {
            if ((p->sig==DG_SIG_FXO) || (p->sig==DG_SIG_CB_FXO) || (p->sig==DG_SIG_CUSTOMCAS) || (p->sig==DG_SIG_GSM) || (p->sig==DG_SIG_PRI) ||
            	(p->second_call && p->sig==DG_SIG_CB_FXS && dg_GetPortStatus(p->port) == spNone) ||
            	((p->sig==DG_SIG_R2_NDIS_FIXED || p->sig==DG_SIG_R2_NDIS_VARIABLE) && (p->call_state==DG_AST_CALLING || p->call_state==DG_AST_WAITFORDIALTONE)))
                p->reserved=0;
            else
            {
            	if ((p->last_r2_signal==R2_IDLE) && 
            		(p->sig==DG_SIG_R2_NDIS_FIXED || p->sig==DG_SIG_R2_NDIS_VARIABLE))
            	{
	                p->reserved=0;
	        	    if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
	            	    ast_log(LOG_NOTICE, "dgv_hangup (%d): R2_IDLE already received, changing reserved to 0.\n", p->port);
            	}
            	else
            	{            	
	                p->reserved=2;      //wait receiving idle
	                
	        	    if (p->sig==DG_SIG_R2_NDIS_FIXED || p->sig==DG_SIG_R2_NDIS_VARIABLE)	        	    	
	        	    {
		        	    if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
		            	    ast_log(LOG_NOTICE, "dgv_hangup (%d): Changing reserved to 2 because last_r2_signal = %x\n", p->port, p->last_r2_signal);
		        	    
	        	    	//set reserved timeout
	        	    	if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
	        	    		ast_log(LOG_NOTICE, "dgv_hangup (%d) - Creating reserved schedule\n", p->port);
	        		
	        			p->reserved_timeout_sched = ast_sched_add(sched, reservedtimeout, reserved_timeout_callback, p);
	        	    }
	        	    else
		        	    if (((p->verbosity & DG_LOG_START_END_CALL) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
		            	    ast_log(LOG_NOTICE, "dgv_hangup (%d): Changing reserved to 2\n", p->port);
            	}
            }            
        }
        
        switch (p->sig)
        {
            case DG_SIG_FXO:
            case DG_SIG_CB_FXO:                        	
                if (dgv_fxoNDTCount[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1] >= max_ndt_count)
                {
                	p->alarmed = 1;
                	
					if (p->alarmNDT_timeout_sched > -1) 
					{
						res_sched = ast_sched_del(sched, p->alarmNDT_timeout_sched);
				    	if (res_sched == -1)
				    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

						p->alarmNDT_timeout_sched = -1;
					}
					p->alarmNDT_timeout_sched = ast_sched_add(sched, alarmNDTtimeout, alarmNDT_timeout_callback, p);
					if (p->alarmNDT_timeout_sched < 0)
						ast_log(LOG_WARNING, "ERROR Setting alarmNDT timeout callback on port %d\n", p->port);
                }
            	break;
            case DG_SIG_R2_NDIS_FIXED:
            case DG_SIG_R2_NDIS_VARIABLE:
            	p->group_b_port_bridged = 0;
            	p->category_port_bridged = 0;
            case DG_SIG_PRI:
            	if (p->vconf_established)
            	{
					ret=dg_LocalBridgeDisconnect(p->port, p->vconf_port_bridged);
                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
						ast_log(LOG_NOTICE, "Disconnecting vconf for port %d and port %d - cod %x\n", p->port, p->vconf_port_bridged, ret);

            		p_bridged = find_ch_port(p->vconf_port_bridged);
            	    if (p_bridged)
    					p_bridged->vconf_established = 0;

					p->vconf_established = 0;
            	}
            	break;
        }

		if (p->fax_established)
		{
			ret=dg_LocalBridgeDisconnect(p->port, p->fax_port_bridged);
			if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
				ast_log(LOG_NOTICE, "LocalBridge Fax disconnected, port %d and port %d - cod %x\n", p->port, p->fax_port_bridged, ret);

			p_bridged = find_ch_port(p->fax_port_bridged);
			if (p_bridged)
			{
				p_bridged->fax_established = 0;
				p_bridged->fax_port_bridged = 0;
			}

			p->fax_established = 0;
			p->fax_port_bridged = 0;
		}
        
#ifdef DEBUG_PRI        
        	ast_log(LOG_NOTICE, "dgv_hangup: STEP 6 on port %d\n", p->port);        
#endif//#ifdef DEBUG_PRI

    	p->call_state = DG_AST_NONE;
    	
        p->write_last_format=-1;		//initializes with invalid format
        p->read_last_format=-1;

#ifdef DEBUG_PRI        
        	ast_log(LOG_NOTICE, "dgv_hangup: STEP 7 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI
        
        /* unlink digivoice port from channel */
		p->ast_ch = NULL;

		dgv_channel_tech_pvt_set(ast, NULL);

        if (p->auto_dgccb)
        	p->flashing = 1;
        else
        	p->flashing = 0;

        p->need_fv_thread = 0;        
        p->need_sc_ring = 0;
        p->need_ringback = 0;
        for (nForReset = 0; nForReset < 20; nForReset++)
        {
        	p->dialplan_senddtmf[nForReset] = '\0';
        	p->dialplan_group_b[nForReset] = '\0';
        }
                
#ifdef DEBUG_PRI        
        	ast_log(LOG_NOTICE, "dgv_hangup: STEP 8 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI
        
        if (p->linetone_timeout_sched > -1) //stop line tone timeout
        {
            res_sched = ast_sched_del(sched, p->linetone_timeout_sched);
	    	if (res_sched == -1)
	    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

            p->linetone_timeout_sched = -1;
        }

#ifdef DEBUG_PRI
        	ast_log(LOG_NOTICE, "dgv_hangup: STEP 9 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI
	}
	ast_dgv_unlock(&p->lock, p->port);

	dg_DisableCallProgress(p->port);
			
#ifdef DEBUG_PRI    
    		ast_log(LOG_NOTICE, "dgv_hangup: STEP 10 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI    

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		ast_atomic_fetchadd_int(&usecnt, -1);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		ast_dgv_lock(&usecnt_lock, 0);
		usecnt--;
		ast_dgv_unlock(&usecnt_lock, 0);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

/*    ast_dgv_lock(&usecnt_lock); {
		usecnt--;
	} ast_dgv_unlock(&usecnt_lock);*/

#ifdef DEBUG_PRI
		ast_log(LOG_NOTICE, "dgv_hangup: STEP 11 on port %d\n", p->port);		
#endif//#ifdef DEBUG_PRI		

	ast_update_use_count();

#ifdef DEBUG_PRI	
		ast_log(LOG_NOTICE, "dgv_hangup: STEP 12 on port %d\n", p->port);
#endif//#ifdef DEBUG_PRI
	
	dg_GenerateMF(p->port, GENERATE_OFF, 0);
		
	if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
        ast_verbose(VERBOSE_PREFIX_3 "Port %d hungup\n",p->port);

	return 0;
}

//-------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------
static int dgv_answer(struct ast_channel *ast)
{
#ifdef DEBUG_Q
    	char szTemp[255];
#endif//#ifdef DEBUG_Q

#ifdef DEBUG_PRI   	
    	ast_log(LOG_NOTICE, "dgv_answer: STEP 1...\n");
#endif//#ifdef DEBUG_PRI
    	
#ifdef CC_PRI_HAS_REVERSECHARGE    	
    	char nothing[256];
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE
    	
	struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);

	int ret=0;
	//int trycnt;

#ifdef USE_LIBPRI
		int dc;
#endif//#ifdef USE_LIBPRI

	if (!p)
	{
		if (option_verbose >= 3)		
			ast_log(LOG_NOTICE, "dgv_answer(%s): !p \n", dgv_channel_name(ast));
		return 0;		
	}
	
	/*
	trycnt = 0;
	for (;;)
	{	
		if (ast_dgv_trylock(&p->lock, p->port))						
		{
			usleep(1);
			trycnt++;
	        if (trycnt > trycnt_avoiddl)
	        {
        		ast_log(LOG_WARNING, "dgv_answer: Too much delay to unlock channel, dropping callback_event on port %d\n", p->port);
#ifdef DEBUG_LAST_LOCK	        		
        			ast_log(LOG_WARNING, "dgv_answer(lock_last_res %d): line %05d(%s), port %03d, mutex '%s'\n",
        					p->lock_last_res,
        					p->lock_last_lineno, 
        					p->lock_last_func, 
        					p->lock_last_port, 
        					p->lock_last_mutex_name);
#endif//#ifdef DEBUG_LAST_LOCK        		
        		return 0;
			}
		}
		else //if (ast_dgv_trylock(&p->lock, p->port))
		*/
	
	ast_dgv_lock(&p->lock, p->port);
	{
    	if (!p->ast_ch)
    	{
    		p->call_state = DG_AST_NONE;
    		if (p->tone_thread_running == 1)
    			p->tone_thread_running = 2;
    		
    		if (p->stuttertone_thread_running == 1)
    			p->stuttertone_thread_running = 0;

    	    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    	    	ast_log(LOG_WARNING, "dgv_answer: p->ast_ch does not exist on port %d\n", p->port);
    		ast_dgv_unlock(&p->lock, p->port);
    		return 0;
    	}    	
    	
    	if (p->call_state==DG_AST_RINGING)
    	{
    	    /* stops tone thread */
    		if (p->tone_thread_running == 1)
    			p->tone_thread_running = 2;

    		if (p->stuttertone_thread_running == 1)
    			p->stuttertone_thread_running = 0;

            if (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS)
            {
        		p->statistics.answer = ast_tvnow();
				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "Starting to count call duration on port %d\n", p->port);

				ast_setstate(p->ast_ch, AST_STATE_UP);

				//enable inputbuffer streaming with agc disabled
                dg_PauseInputBuffer(p->port, DG_RELEASE);
                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
                	ast_log(LOG_WARNING, "dgv_answer: Releasing InputBuffer on port %d\n", p->port);

                p->call_state = DG_AST_CONVERSATION;
                
                ast_dgv_unlock(&p->lock, p->port);
                {
	                ret = dg_EnableCallProgress(p->port, CP_ENABLE_BUSY_OR_FAX);
	                if (ret != DG_EXIT_SUCCESS)
	                    ast_log(LOG_NOTICE, "EnableCallProgress port %d - error %d\n",p->port,ret);
	                else
	                    if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
	                        ast_log(LOG_NOTICE, "EnableCallProgress CP_ENABLE_BUSY_OR_FAX port %d - OK\n",p->port);
                }
                ast_dgv_lock(&p->lock, p->port);					
            }
            else //if (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS)
            {
				if (p->subchannel_audio_index == 0)
					p->subchannel_audio_index = 1;
				
				p->call_state=DG_AST_INCOMING_CALL;
            	if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
                	ast_log(LOG_NOTICE, "Port %d picking up\n",p->port);
                
#ifdef DEBUG_Q
		   			sprintf(szTemp,"/tmp/reclog%d.raw", p->port);
   					p->rec = fopen(szTemp,"w");
   					if (p->rec==NULL)
   						ast_log(LOG_ERROR, "Error opening test file!!!");	
    					
	   				sprintf(szTemp,"/tmp/playlog%d.raw", p->port);
   					p->play = fopen(szTemp,"w");
   					if (p->play==NULL)
	   					ast_log(LOG_ERROR, "Error opening test file!!!");	
#endif//#ifdef DEBUG_Q		
		
#ifdef USE_LIBPRI
   					if (p->sig == DG_SIG_PRI)
   					{
						if (p->ch_board > 30)
							dc = dg_pri_findindex(p->board, 2);
						else
							dc = dg_pri_findindex(p->board, 1);

						//Reverse charging
#ifdef CC_PRI_HAS_REVERSECHARGE
							if (p->flashing)
							{
								if (p->reverse_charging == 1)
								{
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "'Reverse Charging Indication' detected on port %d, dropping call...\n", p->port);
									
			                    	if (p->ast_ch)
			                    	{
			                    		dgv_channel_hangupcause_set(p->ast_ch, PRI_CAUSE_CALL_REJECTED);
			                        
				                        /*
				            	    	struct ast_frame f;
				            	    	f.src = "chan_dgv";
				            	    	f.datalen = 0;
				            	    	*/
				            	    	
				        				if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
				        					ast_log(LOG_NOTICE, "Sending AST_FRAME_VOICE into dgv_answer on '%d'\n", p->port);

#ifdef CC_AST_HAS_VERSION_11
											ast_channel_softhangup_internal_flag_add(p->ast_ch, AST_SOFTHANGUP_DEV);
#else//#ifdef CC_AST_HAS_VERSION_11
											p->ast_ch->_softhangup |= AST_SOFTHANGUP_DEV;
#endif//#ifdef CC_AST_HAS_VERSION_11
	
		                	        	p->f.frametype = AST_FRAME_VOICE;

#ifdef CC_AST_HAS_VERSION_13
	                        				p->f.subclass.format = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
												ast_format_copy(&p->f.subclass.format, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
													p->f.subclass.format = p->ast_ch->rawreadformat;
		#else//#ifdef CC_AST_HAS_VERSION_10
			#ifdef CC_AST_HAS_VERSION_1_8
														p->f.subclass.integer = p->ast_ch->rawreadformat;
			#else//#ifdef CC_AST_HAS_VERSION_1_8
														p->f.subclass = p->ast_ch->rawreadformat;
			#endif//#ifdef CC_AST_HAS_VERSION_1_8
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_VERSION_13
											if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_slin) == AST_FORMAT_CMP_EQUAL)
												p->f.samples = DATALEN;
											else
												if (ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_gsm) == AST_FORMAT_CMP_EQUAL)
													p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
												else
													if ((ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_alaw) == AST_FORMAT_CMP_EQUAL) ||
															(ast_format_cmp(ast_channel_rawreadformat(p->ast_ch), ast_format_ulaw) == AST_FORMAT_CMP_EQUAL))
														p->f.samples = DATALEN;
													else
														p->f.samples = DATALEN;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
												switch (ast_channel_rawreadformat(p->ast_ch)->id)
	#else//#ifdef CC_AST_HAS_VERSION_11
		#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat.id)
		#else//#ifdef CC_AST_HAS_VERSION_10
													switch (p->ast_ch->rawreadformat)
		#endif//#ifdef CC_AST_HAS_VERSION_10
	#endif//#ifdef CC_AST_HAS_VERSION_11
											{
												case AST_FORMAT_SLINEAR:
													p->f.samples = DATALEN;
													break;
												case AST_FORMAT_GSM:
													p->f.samples = DATALEN * (DATALEN/GSM_DATALEN);
													break;
												case AST_FORMAT_ALAW:
												case AST_FORMAT_ULAW:
													p->f.samples = DATALEN;
													break;
												default:
													p->f.samples = DATALEN;
											} //switch (p->ast_ch->rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_DATA_PTR				        				
											p->f.data.ptr = nothing + AST_FRIENDLY_OFFSET;
#else//#ifdef CC_AST_HAS_DATA_PTR				        				
											p->f.data = nothing + AST_FRIENDLY_OFFSET;
#endif//#ifdef CC_AST_HAS_DATA_PTR
											QueueFrame(p, &p->f);
				                    } //if (p->ast_ch)

					                ast_dgv_unlock(&p->lock, p->port);
					                return 0;
								}
								else
								{
									if (option_verbose >= 3)
										ast_verbose(VERBOSE_PREFIX_3 "No 'Reverse Charging Indication' detected on port %d, answering...\n", p->port);
								}
							}
#else//#ifdef CC_PRI_HAS_REVERSECHARGE
							if (p->flashing)												           			
								ast_log(LOG_WARNING, "CallBlock application not supported, please contact DigiVoice Technical Support.\n");							
#endif//#ifdef CC_PRI_HAS_REVERSECHARGE

						if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
    		    	        ast_log(LOG_NOTICE,"Answering PRI - pri_channel %d and port %d\n", p->pri_channel, p->port);
	        	        
		    	        if (p->pri)
		    	        {
		    	        	if (p->call)
		    	        	{
		    	        		if (!dgv_trylock_ccs(p, dc))
		    	        		{
		    	        			p->proceeding = 1;
		    	        			pri_answer(p->pri, p->call, p->pri_channel, 0);

									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "Sending pri_answer %d/%d\n", p->pri_channel, p->port);

		    	        			dgv_releaselock_ccs(dc);
		    	        		}
		    	        		else
		    	        			ast_log(LOG_WARNING, "dgv_answer: Unable to grab ccs_lock on port %d\n", p->port);
		    	        	}
		    	        	else
		    	        		if ((p->verbosity & DG_LOG_ISDN_EVENTS) != 0)
		    	        			ast_log(LOG_WARNING, "dgv_answer did not call pri_answer because p->call is null %d\n", p->port);
		    	        }
   					} //if (p->sig == DG_SIG_PRI)
#endif//#ifdef USE_LIBPRI

				if (dg_GetCardInterface(dg_GetCardNumber(p->port)) == DG_FX_INTERFACE && p->sig == DG_SIG_FXO)
				{
	                switch (p->relaxdtmf)
	                {
	                    case 0:                        
	                        dg_SetTwist(p->port, 6 , 18);
							break;
	                    case 1:
	                        dg_SetTwist(p->port, 9 , 12);
	                        break;
	                    case 2:
	                        dg_SetTwist(p->port, 9 , 6 );
	                        break;
	                    case 3:
	                        dg_SetTwist(p->port, 12, 0 );
	                        break;
	                }
				}

				if (p->flashing != 0)
   		            dg_PickUp(p->port, 1500);
				else
				{
		            if (p->max_digits_rx==0)
    		            dg_PickUp(p->port, 20);
        		    else
            		    dg_PickUp(p->port, 1000);
				}
				
	            if ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0)
    	            ast_log(LOG_NOTICE, "dgv_answer: dg_PickedUp Port %d \n",p->port);
            } //if (p->sig==DG_SIG_FXS || p->sig==DG_SIG_CB_FXS)
    	} //if (p->call_state==DG_AST_RINGING)		
	}
	ast_dgv_unlock(&p->lock, p->port);
	    	
			/*ast_dgv_unlock(&p->lock, p->port);
			break;
		} //if (ast_dgv_trylock(&p->lock, p->port))
	} //for (;;)*/	

	return 0;
}

static int dgv_fixup(struct ast_channel *oldchan, struct ast_channel *newchan)
{
#ifdef DEBUG_PRI	
		ast_log(LOG_NOTICE, "dgv_fixup: STEP 1...\n");
#endif//#ifdef DEBUG_PRI

	struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(newchan);
	
	if (!p)
	{
		if (option_verbose >= 3)		
			ast_log(LOG_NOTICE, "dgv_fixup(%s): !p \n", dgv_channel_name(oldchan));
		return 0;
	}
	
	ast_dgv_lock(&p->lock, p->port);
	
		if (!p->ast_ch)
		{
			if ((p->verbosity & DG_LOG_START_END_CALL) != 0)
				ast_log(LOG_WARNING, "dgv_fixup: p->ast_ch does not exist on port %d\n", p->port);
			ast_dgv_unlock(&p->lock, p->port);
			return 0;
		}

		if (option_verbose >= 3)
			ast_verbose(VERBOSE_PREFIX_3 "New owner for port %d is %s (Old: %s)\n", p->port, dgv_channel_name(newchan), dgv_channel_name(oldchan));
		
		if (p->ast_ch == oldchan)
		{
			p->ast_ch = newchan;
		}
		
		if (dgv_channel_state(newchan) == AST_STATE_RINGING)
		{
			if (option_verbose >= 3)
				ast_verbose(VERBOSE_PREFIX_3 "%s: dgv_fixup Calling dgv_indicate\n", p->dev);

			ast_dgv_unlock(&p->lock, p->port);        
#ifdef CC_AST_HAS_INDICATE_DATA        
        		dgv_indicate(newchan, AST_CONTROL_RINGING, NULL, 0);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
        		dgv_indicate(newchan, AST_CONTROL_RINGING);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA                
		}
		else
			ast_dgv_unlock(&p->lock, p->port);

	return 0;
}

static int dgv_sendtext(struct ast_channel *ast, const char *text)
{
	char *text_received = strdup(text ? (char *)text : "");
	
    if (option_verbose >= 3)
    	ast_verbose(VERBOSE_PREFIX_3 "dgv_sendtext, a text was received: '%s'\n", text_received);	
	
	if (option_debug==100)
		ast_log(LOG_NOTICE, "Calling send_sms_message...\n");

	return send_sms_message(ast, -1, text_received);
}

static int send_sms_message(struct ast_channel *ast, int fd, char *text)
{
	struct dgv_pvt *p;
	char *text_received = strdup(text ? (char *)text : "");
	char *s;
	char *more_data;
	char *piece_data;
	int group_port[NUM_MAXDIALOPT];
	short is_group[NUM_MAXDIALOPT];
	short backwards[NUM_MAXDIALOPT];
	short roundrobin[NUM_MAXDIALOPT];
	short serialnumber[NUM_MAXDIALOPT];
	short serialnumber_port[NUM_MAXDIALOPT];
	short talktime[NUM_MAXDIALOPT];
	short serialnumber_talktime[NUM_MAXDIALOPT];
	short nForData;
	short nCntGroup_Port;
	short waitforret=0;
	int nForWaitForRet=0;

	int ret, ret_vlib;
	short step_keep_fd=0;

	char header[20];
    char resource[20];
	char send_to[40];
	char text_message[200];
	char lock_dialplan_console[5];
	char custom_id[40];

	char sz_pgroup[TAMSTR];
	char sz_pport[TAMSTR];
	char sz_simcardid[TAMSTR];
	char sz_vlibgsmerr[TAMSTR];
	char sz_messagereference[TAMSTR];
	char sz_gsm_error_timeout[TAMSTR];
	char sz_ret[TAMSTR];
	char sz_find_port_err_msg[TAMSTR];

    if (option_verbose >= 3)
    	ast_verbose(VERBOSE_PREFIX_3 "A text was received: '%s'\n", text);	

	header[0]                = '\0';
    resource[0]              = '\0';
	send_to[0]               = '\0';
	text_message[0]          = '\0';
	lock_dialplan_console[0] = '\0';
	custom_id[0]             = '\0';

	sz_pgroup[0]            = '\0';
	sz_pport[0]             = '\0';
	sz_simcardid[0]         = '\0';
	sz_vlibgsmerr[0]        = '\0';
	sz_messagereference[0]  = '\0';
	sz_gsm_error_timeout[0] = '\0';
	sz_ret[0]               = '\0';
	sz_find_port_err_msg[0] = '\0';
    
	s = strsep(&text_received, generic_sep);
	if ((s) && (strlen(s) > 0))
	{
		if (!strncmp(s, sms_header, sizeof(sms_header)))
		{
			if (option_verbose >= 3)
			    ast_verbose(VERBOSE_PREFIX_3 "dgv_sendtext called by DgSendSMS application\n");			

			ast_copy_string(header, s, sizeof(header));
			
			if (option_debug==100)
				ast_log(LOG_NOTICE, "header received: '%s'\n", header);
			
			s = strsep(&text_received, generic_sep);
			if ((s) && (strlen(s) > 0))
			{				
				ast_copy_string(resource, s, sizeof(resource));
				
				if (option_debug==100)
					ast_log(LOG_NOTICE, "resource received: '%s'\n", resource);				
				
				s = strsep(&text_received, generic_sep);
				if ((s) && (strlen(s) > 0))
				{
					ast_copy_string(send_to, s, sizeof(send_to));
					
					if (option_debug==100)
						ast_log(LOG_NOTICE, "send_to received: '%s'\n", send_to);
					
					s = strsep(&text_received, generic_sep);
					if ((s) && (strlen(s) > 0))
					{
						ast_copy_string(text_message, s, 161);
						
						if (option_debug==100)
							ast_log(LOG_NOTICE, "text_message received: '%s'\n", text_message);

						/*
						ConvCodeGSM((unsigned char *)text_message);

						if (option_debug==100)
							ast_log(LOG_NOTICE, "text_message after character conversion: '%s'\n", text_message);
						*/

						s = strsep(&text_received, generic_sep);
						if ((s) && (strlen(s) > 0))
						{
							ast_copy_string(lock_dialplan_console, s, sizeof(lock_dialplan_console));

							if (option_debug==100)
								ast_log(LOG_NOTICE, "lock_dialplan_console received: '%s'\n", lock_dialplan_console);

							if (!strncmp(lock_dialplan_console, sms_lock, sizeof(sms_lock)))
								waitforret = 1;
							else
								if (option_verbose >= 3)
								    ast_log(LOG_NOTICE, "dgv_sendtext, did not set the lock_dialplan_console parameter, do not lock dialplan/console to this message.\n");

							s = strsep(&text_received, generic_sep);
							if ((s) && (strlen(s) > 0))
							{
								ast_copy_string(custom_id, s, 40);

								if (option_debug==100)
									ast_log(LOG_NOTICE, "custom_id received: '%s'\n", custom_id);
							}
							else //if ((s) && (strlen(s) > 0))
							{
								if (option_verbose >= 3)
								    ast_log(LOG_NOTICE, "dgv_sendtext, did not receive the custom_id parameter, do not generate events with custom_id parameter to this message.\n");
							} //if ((s) && (strlen(s) > 0))
						}
						else //if ((s) && (strlen(s) > 0))
						{
							if (option_verbose >= 3)
							    ast_log(LOG_NOTICE, "dgv_sendtext, did not receive the lock_dialplan_console parameter, do not lock dialplan/console to this message.\n");
						} //if ((s) && (strlen(s) > 0))
					}
					else //if ((s) && (strlen(s) > 0))
					{
						if (option_verbose >= 3)
						    ast_log(LOG_WARNING, "dgv_sendtext, did not receive the text_message parameter, dropping this message.\n");
						
						return -1;
					} //if ((s) && (strlen(s) > 0))					
				}
				else //if ((s) && (strlen(s) > 0))
				{
					if (option_verbose >= 3)
					    ast_log(LOG_WARNING, "dgv_sendtext, did not receive the send_to parameter, dropping this message.\n");
					
					return -1;
				} //if ((s) && (strlen(s) > 0))				
			}
			else //if ((s) && (strlen(s) > 0))
			{
				if (option_verbose >= 3)
				    ast_log(LOG_WARNING, "dgv_sendtext, did not receive the resource parameter, dropping this message.\n");
				
				return -1;
			} //if ((s) && (strlen(s) > 0))
		}
		else //if (!strncmp(s, sms_header, sizeof(sms_header)))
		{
			if (option_verbose >= 3)
			    ast_verbose(VERBOSE_PREFIX_3 "dgv_sendtext called by another application, dropping this message.\n");
			
			return -1;
		} //if (!strncmp(s, sms_header, sizeof(sms_header)))		
    }
    else //if ((s) && (strlen(s) > 0))
    {
		if (option_verbose >= 3)
		    ast_log(LOG_WARNING, "dgv_sendtext, did not receive header XXX parameter, dropping this message.\n");
		
		return -1;
    } //if ((s) && (strlen(s) > 0))

	for (nForData = 0; nForData <= NUM_MAXDIALOPT - 1; nForData++)
	{
		group_port[nForData] = -1;
		is_group[nForData] = -1;
		backwards[nForData] = 0;
		roundrobin[nForData] = 0;
		serialnumber[nForData] = 0;
		serialnumber_port[nForData] = 0;
		talktime[nForData] = 0;
		serialnumber_talktime[nForData] = 0;
	}

	nCntGroup_Port = 0;
	more_data = ast_strdupa(resource);
	while ((piece_data = strsep(&more_data, "+")))
	{
		//check for group syntax
		if (toupper(piece_data[0]) == 'G' || toupper(piece_data[0])=='R' || toupper(piece_data[0])=='S' || toupper(piece_data[0])=='T')
		{
			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "New SMS_Group detected: %s\n", piece_data);

			is_group[nCntGroup_Port] = 1;
			group_port[nCntGroup_Port] = atoi(piece_data+1);

			if (piece_data[0] == 'G' || piece_data[0] == 'R' || piece_data[0] == 'S' || piece_data[0] == 'T')//upper letter use backward search
				backwards[nCntGroup_Port] = 1;
			else
				backwards[nCntGroup_Port] = 0;

			if (toupper(piece_data[0])=='R')
				roundrobin[nCntGroup_Port] = 1;

			if (toupper(piece_data[0])=='S')
			{
				serialnumber[nCntGroup_Port] = 1;

            	if (toupper(piece_data[6])=='C')
            		serialnumber_port[nCntGroup_Port] = atoi(piece_data+7);
            	else
            		serialnumber_port[nCntGroup_Port] = 0;

            	if (piece_data[6]=='T')
            		serialnumber_talktime[nCntGroup_Port] = 2; //like a backward
            	else
            		if (piece_data[6]=='t')
            			serialnumber_talktime[nCntGroup_Port] = 1; //not a backward
            		else
            			serialnumber_talktime[nCntGroup_Port] = 0; //disabled
			}

            if (toupper(piece_data[0])=='T')
            	talktime[nCntGroup_Port] = 1;
		}
		else
		{
			if (option_verbose >= 3)
				ast_log(LOG_NOTICE, "New SMS_Port detected: %s\n", piece_data);

			is_group[nCntGroup_Port] = 0;
			group_port[nCntGroup_Port] = atoi(piece_data);
		}
		nCntGroup_Port++;
	}

    //keep protected
    ast_dgv_lock(&iflock, 0);
    {
        p = find_free_port(group_port, is_group, backwards, -1, roundrobin, serialnumber, serialnumber_port, talktime, serialnumber_talktime, 1);	//force port to -1 to find using group only

        if (p && !p->sms_reserved && !p->alarmed)
        {
    		sprintf(sz_pgroup, "%d", p->group);
    		sprintf(sz_pport, "%d", p->port);
    		sprintf(sz_simcardid, "%s", p->gsm_simcard_id);
    		sprintf(sz_vlibgsmerr, "%s", p->sms_sent_tmpMessageError);
    		sprintf(sz_messagereference, "%d", p->sms_send_messagereference);

            if (p->sig != DG_SIG_GSM)
            {
                if (option_verbose >= 3)
                    ast_log(LOG_WARNING, "dgv_sendtext: Port %d is not a GSM port!\n", p->port);
                
                ast_dgv_unlock(&iflock, 0);
                
                ret = -1;

	       		manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
	                        				 	 "SendTo: %s\r\n"
	                        				 	 "Group: %d\r\n"
	                        					 "Port: %d\r\n"
	                        					 "SIMCardID: %s\r\n"
	                        					 "Status: %s\r\n"
	                        					 "Error: %s\r\n"
	                        					 "CustomID: %s\r\n",
	                        					 send_to, p->group, p->port, p->gsm_simcard_id, "ERROR", "PORT_NOTGSM", custom_id
	                          );

	       		if (ast)
	       		{
	       			ret = 0;

	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "ERROR");
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_ERROR", "PORT_NOTGSM");
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_ERROR and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
	       		}
	       		else
	       		{
	       			if (fd >= 0)
	       			{
	       				ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

		        		ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "ERROR");
		        		ast_cli(fd, "        |_ DGV_SMSSENT_ERROR: %s\n", "PORT_NOTGSM");
		        		ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
		        		ast_cli(fd, "\n");
	       			}
	       		}

                return ret;
            }

        	if (option_verbose >= 3)
        		ast_log(LOG_NOTICE, "dgv_sendtext(%d): Sending SMS Message to '%s'\n", p->port, send_to);
        	
        	ret_vlib = dg_GSMSendSMS(p->port, send_to, text_message);
        	if (ret_vlib != DG_EXIT_SUCCESS)
        	{
                if (option_verbose >= 3)
                    ast_log(LOG_WARNING, "dgv_sendtext: Port %d got error '%d' during Send SMS Message!\n", p->port, ret_vlib);

	       		ret = -1;

	       		manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
	                        				 	 "SendTo: %s\r\n"
	                        				 	 "Group: %d\r\n"
	                        					 "Port: %d\r\n"
	                        					 "SIMCardID: %s\r\n"
	                        					 "Status: %s\r\n"
	                        					 "Error: VLIB_%d\r\n"
	                        					 "CustomID: %s\r\n",
	                        					 send_to, p->group, p->port, p->gsm_simcard_id, "ERROR", ret_vlib, custom_id
	                          );

	       		if (ast)
	       		{
	       			ret = 0;

	       			sprintf(sz_ret, "VLIB_%d", ret_vlib);

	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
	        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "ERROR");
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_ERROR", sz_ret);
					pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_ERROR and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
	       		}
	       		else
	       		{
	       			if (fd >= 0)
	       			{
	       				ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

		        		ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "ERROR");
		        		ast_cli(fd, "        |_ DGV_SMSSENT_ERROR: VLIB_%d\n", ret_vlib);
		        		ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
		        		ast_cli(fd, "\n");
	       			}
	       		}
        	}
        	else
        	{
            	p->sms_reserved = 1;
            	ast_copy_string(p->sms_send_to, send_to, sizeof(p->sms_send_to));
            	ast_copy_string(p->sms_send_custom_id, custom_id, sizeof(p->sms_send_custom_id));
            	p->sms_sent_status = 0;

        		ret = 0;

        		if (waitforret)
        		{
        			ast_dgv_unlock(&iflock, 0);

        			if (option_verbose >= 3)
        				ast_log(LOG_NOTICE, "dgv_sendtext, waiting sending the SMS Message.\n");

                	for (nForWaitForRet = 1; nForWaitForRet <= (gsmerrortimeout / 1000); nForWaitForRet++)
                	{
                		if (p->sms_reserved == 0)
                		{
							switch (p->sms_sent_status)
							{
								case -1: /* Timeout */
									if (ast)
									{
										sprintf(sz_gsm_error_timeout, "%d", gsmgenerictimeout);

										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "TIMEOUT");
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_TIMEOUT", sz_gsm_error_timeout);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_TIMEOUT and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
									}
									else
									{
										if (fd >= 0)
										{
											ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

											ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
											ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
											ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
											ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
											ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "TIMEOUT");
											ast_cli(fd, "        |_ DGV_SMSSENT_TIMEOUT(ms): %d\n", gsmgenerictimeout);
											ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
											ast_cli(fd, "\n");
										}
									}

									break;
								case -2: /* Error */
									if (ast)
									{
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "ERROR");
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_ERROR", sz_vlibgsmerr);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_ERROR and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
									}
									else
									{
										if (fd >= 0)
										{
											ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

											ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
											ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
											ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
											ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
											ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "ERROR");
											ast_cli(fd, "        |_ DGV_SMSSENT_ERROR: %s\n", p->sms_sent_tmpMessageError);
											ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
											ast_cli(fd, "\n");
										}
									}

									break;
								case 1: /* Success */
									if (ast)
									{
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "OK");
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_MESSAGEID", sz_messagereference);
										pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

										if (option_verbose >= 3)
											ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_MESSAGEID and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
									}
									else
									{
										if (fd >= 0)
										{
											ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

											ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
											ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
											ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
											ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
											ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "OK");
											ast_cli(fd, "        |_ DGV_SMSSENT_MESSAGEID: %d\n", p->sms_send_messagereference);
											ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
											ast_cli(fd, "\n");
										}
									}

									p->sms_send_messagereference = -1; /* Reset this variable, this is the last time we need to use */
									break;
							}

                			break;
                		}

                		sleep(1);

                		if (option_debug==100)
                			ast_log(LOG_NOTICE, "waiting for sms command response, step: '%d'\n", nForWaitForRet);

						if (fd >= 0)
						{
							switch (step_keep_fd)
							{
								case 0:
									ast_cli(fd, "%s\b", "|");
									step_keep_fd = 1;
									break;
								case 1:
									ast_cli(fd, "%s\b", "/");
									step_keep_fd = 2;
									break;
								case 2:
									ast_cli(fd, "%s\b", "-");
									step_keep_fd = 3;
									break;
								case 3:
									ast_cli(fd, "%s\b", "\\");
									step_keep_fd = 0;
									break;
							}
						}
                	}

               		if (fd >= 0)
               			ast_cli(fd, "%s\b", "");

                	ast_dgv_lock(&iflock, 0);

                	if ((p->sms_reserved == 1) && ((nForWaitForRet - 1) == (gsmerrortimeout / 1000)))
                	{
                        if (option_verbose >= 3)
                            ast_log(LOG_WARNING, "dgv_sendtext: Port %d got timeout during Send SMS Message!\n", p->port);

        	       		p->sms_reserved = 0;
        	       		p->sms_send_to[0] = '\0';
        	       		p->sms_send_messagereference = -1;
        	       		p->sms_send_custom_id[0] = '\0';
        	       		p->sms_sent_status = 0;

        	       		ret = -1;

						manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
														 "SendTo: %s\r\n"
														 "Group: %d\r\n"
														 "Port: %d\r\n"
														 "SIMCardID: %s\r\n"
														 "Status: %s\r\n"
														 "TimeOut(ms): %d\r\n"
														 "CustomID: %s\r\n",
														 send_to, p->group, p->port, p->gsm_simcard_id, "TIMEOUT", gsmerrortimeout, custom_id
									  );

						if (ast)
						{
							ret = 0;

							sprintf(sz_gsm_error_timeout, "%d", gsmerrortimeout);

							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "TIMEOUT");
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_TIMEOUT", sz_gsm_error_timeout);
							pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_TIMEOUT and DGV_SMSSENT_CUSTOMID variables to read it!\n", p->port);
						}
						else
						{
							if (fd >= 0)
							{
								ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", p->port);

								ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
								ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %d\n", p->group);
								ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", p->port);
								ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", p->gsm_simcard_id);
								ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "TIMEOUT");
								ast_cli(fd, "        |_ DGV_SMSSENT_TIMEOUT(ms): %d\n", gsmerrortimeout);
								ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
								ast_cli(fd, "\n");
							}
						}
                	}
        		}
        		else //if (waitforret)
        		{
    	        	if (p->gsmsendsms_timeout_sched > -1)
    				{
    	        		if (option_debug==100)
    						ast_log(LOG_WARNING, "Waiting for sms command response for port %d\n", p->port);
    				}
    	        	else
    	        	{
    					if (option_debug==100)
    						ast_log(LOG_DEBUG, "Creating gsmsendsms schedule for port %d\n", p->port);

    					p->gsmsendsms_timeout_sched = ast_sched_add(sched, gsmerrortimeout, gsmsendsms_timeout_callback, p);
    				}
        		} //if (waitforret)
        	}
        }
        else //if (p && !p->sms_reserved && !p->alarmed)
        {
            if (p)
            {
				if (p->alarmed)
				{
					if (dgv_gsmsignal[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1][0] == 99)
					{
						sprintf(sz_find_port_err_msg, "%s", "NO_SIGNAL_RECEIVED");
						if (option_verbose >= 3)
							ast_log(LOG_WARNING, "dgv_sendtext: SMS_Port %d Error, no signal received!\n", p->port);
					}
					else
						if (dgv_gsmsignal[(dg_GetCardNumber(p->port)) - 1][(dg_GetRelativeChannelNumber(p->port)) - 1][0] == 100)
						{
							sprintf(sz_find_port_err_msg, "%s", "NO_SIM_CARD_DETECTED");
							if (option_verbose >= 3)
								ast_log(LOG_WARNING, "dgv_sendtext: SMS_Port %d Error, no sim card detected!\n", p->port);
						}
						else
						{
							sprintf(sz_find_port_err_msg, "%s", "PORT_ALARMED");
							if (option_verbose >= 3)
								ast_log(LOG_WARNING, "dgv_sendtext: SMS_Port %d Error, port alarmed!\n", p->port);
						}
				}
				else
				{
					if (p->sms_reserved)
					{
						sprintf(sz_find_port_err_msg, "%s", "PORT_RESERVED");
						if (option_verbose >= 3)
							ast_log(LOG_WARNING, "dgv_sendtext: SMS_Port %d Error, port in use or reserved!\n", p->port);
					}
				}
            }
            else
            {
				if (option_verbose >= 3)
					for (nForData = 0; nForData <= NUM_MAXDIALOPT - 1; nForData++)
						if (group_port[nForData] != -1)
						{
							if (is_group[nForData] == 1)
							{
								if (serialnumber[nForData] == 1)
									ast_log(LOG_WARNING, "dgv_sendtext: All ports of serial_number %05d are in use or reserved!\n", group_port[nForData]);
								else
									ast_log(LOG_WARNING, "dgv_sendtext: All ports of group %d are in use or reserved!\n", group_port[nForData]);

							}
							else
								ast_log(LOG_WARNING, "dgv_sendtext: SMS_Port %d Error, port invalid!\n", group_port[nForData]);
						}

				if ((nCntGroup_Port > 1) || (is_group[0] == 1))
					sprintf(sz_find_port_err_msg, "%s", "GROUP_RESERVED");
				else
					sprintf(sz_find_port_err_msg, "%s", "PORT_INVALID");
            }

            ret = -1;

       		manager_event(EVENT_FLAG_SYSTEM, "SMSSent",
                        				 	 "SendTo: %s\r\n"
                        				 	 "Group: %s\r\n"
                        					 "Port: %d\r\n"
                        					 "SIMCardID: %s\r\n"
                        					 "Status: %s\r\n"
                        					 "Error: %s\r\n"
                        					 "CustomID: %s\r\n",
                        					 send_to, "null", group_port[0], "null", "ERROR", sz_find_port_err_msg, custom_id
                          );

       		if (ast)
       		{
       			ret = 0;

        		sprintf(sz_pgroup, "%s", "null");
        		sprintf(sz_pport, "%d", group_port[0]);
        		sprintf(sz_simcardid, "%s", "null");

        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SEND_TO", send_to);
        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_GROUP", sz_pgroup);
        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_PORT", sz_pport);
        		pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_SIMCARDID", sz_simcardid);
				pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_STATUS", "ERROR");
				pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_ERROR", sz_find_port_err_msg);
				pbx_builtin_setvar_helper(ast, "DGV_SMSSENT_CUSTOMID", custom_id);

				if (option_verbose >= 3)
					ast_verbose(VERBOSE_PREFIX_3 "(Dgv/%d): SMS Sent response, use DGV_SMSSENT_SEND_TO, DGV_SMSSENT_GROUP, DGV_SMSSENT_PORT, DGV_SMSSENT_SIMCARDID, DGV_SMSSENT_STATUS, DGV_SMSSENT_ERROR and DGV_SMSSENT_CUSTOMID variables to read it!\n", group_port[0]);
       		}
       		else
       		{
       			if (fd >= 0)
       			{
       				ast_cli(fd, "       (Dgv/%d): SMS Sent response, use below variables to read it!\n", group_port[0]);

	        		ast_cli(fd, "        |_ DGV_SMSSENT_SEND_TO: %s\n", send_to);
	        		ast_cli(fd, "        |_ DGV_SMSSENT_GROUP: %s\n", "null");
	        		ast_cli(fd, "        |_ DGV_SMSSENT_PORT: %d\n", group_port[0]);
	        		ast_cli(fd, "        |_ DGV_SMSSENT_SIMCARDID: %s\n", "null");
	        		ast_cli(fd, "        |_ DGV_SMSSENT_STATUS: %s\n", "ERROR");
	        		ast_cli(fd, "        |_ DGV_SMSSENT_ERROR: %s\n", sz_find_port_err_msg);
	        		ast_cli(fd, "        |_ DGV_SMSSENT_CUSTOMID: %s\n", custom_id);
	        		ast_cli(fd, "\n");
       			}
       		}
        } //if (p && !p->sms_reserved && !p->alarmed)
    }
    ast_dgv_unlock(&iflock, 0);			

	return ret;
}

static int set_gsm_conf(struct dgv_pvt *p)
{
	p->alarmed = 1;

	dg_DisableGSMThread(p->port);

	dg_ConfigGSMThread(p->port, GSMCFG_ID_RESTRICTION, p->gsm_hide_number);	
	dg_ConfigGSMThread(p->port, GSMCFG_CALL_WAITING_ENABLE, DG_DISABLE);
	dg_ConfigGSMThread(p->port, GSMCFG_DISPLAY_CALL_WAITING_ENABLE, p->gsm_displaycallwaiting);
	dg_ConfigGSMThread(p->port, GSMCFG_ANSWER_TIMEOUT, gsmgenerictimeout);
	dg_ConfigGSMThread(p->port, GSMCFG_MESSAGE_CONFIRMATION, p->gsm_sms_confirmation);
	dg_ConfigGSMThread(p->port, GSMCFG_FLASHSMS_ENABLE, p->gsm_sms_flashsms);
	dg_ConfigGSMThread(p->port, GSMCFG_USSD_ENABLE, DG_ENABLE);

	if (option_verbose >= 3) 
		ast_log(LOG_NOTICE, "GSM port=%d, %s my number\n", p->port, p->gsm_hide_number ? "hide" : "show");    								
	
	dg_EnableGSMThread(p->port);
	
	return 0;
}

//-------------------------------------------------------------------------------------------
// Rotina para tratar o que o asterisk passa para a placa
// os sinais devem ser utilizados para gerar RINGBACK, ocupado, etc...
//-------------------------------------------------------------------------------------------
#ifdef CC_AST_HAS_INDICATE_DATA
	static int dgv_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen)
#else//#ifdef CC_AST_HAS_INDICATE_DATA
	static int dgv_indicate(struct ast_channel *ast, int condition)
#endif//#ifdef CC_AST_HAS_INDICATE_DATA
	{
#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "dgv_indicate: STEP 1...\n");
#endif//#ifdef DEBUG_PRI
		
		struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(ast);
	
    	pthread_attr_t attr;

		short ret;
#if defined CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_13
			struct ast_format *fv_rawreadformat;
#else//#ifdef CC_AST_HAS_VERSION_13
			struct ast_format fv_rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_13
#else//#ifdef CC_AST_HAS_VERSION_10
		short fv_rawreadformat;
#endif//#ifdef CC_AST_HAS_VERSION_10

#ifdef USE_LIBPRI
			int dc;
			int res_sched;
			char *s;
			int pause_before_send;
			char *text_received = ast_strdupa(data ? data : "");
#endif//#ifdef USE_LIBPRI

		if (!p)
		{
			if (option_verbose >= 3)		
				ast_log(LOG_NOTICE, "dgv_indicate(%s): !p \n", dgv_channel_name(ast));
			return 0;		
		}
		
		if ((p->sig == DG_SIG_GSM) && (p->channel_index == 2))
		{
			if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
				ast_log(LOG_NOTICE, "dgv_indicate: port %d, channel_index %d, smschannel_index %d (dropping dgv_indicate)\n", p->port, p->channel_index, p->smschannel_index);
			
			return 0;
		}	

		ast_dgv_lock(&p->lock, p->port);
		{
			if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
				ast_log(LOG_NOTICE, "dgv_indicate: condition = %d - port=%d - state=%x\n", condition, p->port, p->call_state);

#ifdef USE_LIBPRI
				if (p->ch_board > 30)
					dc = dg_pri_findindex(p->board, 2);
				else
					dc = dg_pri_findindex(p->board, 1);
#endif//#ifdef USE_LIBPRI

	    	switch(condition)
	    	{
	        	case -1:
	            	if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	                	ast_log(LOG_NOTICE, "dgv_indicate -1 : - port %d - flag %d\n",p->port,p->tone_thread_running);
	                
	        		//ast_dgv_lock(&p->lock, p->port);
	        		//{
	        		if (p->tone_thread_running == 1)
	                   	p->tone_thread_running = 2;
	            	//}
	                //ast_dgv_unlock(&p->lock, p->port);        		
	
		            //check if it is UP to enable samples
	            	if ((p->call_state == DG_AST_CONVERSATION) || (p->subchannel && p->call_state == DG_AST_CALLING)) 	            		
	            	{
		                //ast_dgv_lock(&p->lock, p->port);
		                //{				  
	                    //ret = dg_EnableInputBuffer(p->port,DG_DISABLE);
	                    ret = dg_PauseInputBuffer(p->port, DG_RELEASE);
	                	//}
		                //ast_dgv_unlock(&p->lock, p->port);
	                	if (ret != DG_EXIT_SUCCESS)
		                    ast_log(LOG_WARNING, "Error creating InputBuffer - port %d - cod %d\n",p->port, ret);			
	                	else
	                		if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
	                			ast_log(LOG_WARNING, "dgv_indicate: Releasing InputBuffer on port %d\n", p->port);
	            	}
	            	break;
	        	case AST_CONTROL_CONGESTION:                
	            	if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	                	ast_log(LOG_NOTICE, "dgv_indicate (%d): Congestion - generate busy\n",p->port);
	
	            	if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
	            		dgv_channel_hangupcause_set(ast, AST_CAUSE_CONGESTION);
	
	                dg_signal_e1_thread_structure *p_e1_thread_structure;//pointer to voicerlib e1 thread structure            	
	            	
					p_e1_thread_structure = dg_GetE1ThreadPointer(p->port);

					if (p_e1_thread_structure)
					{   
						if (!p->wait_group_b)
						{
							if (dg_R2SendGroupB(p->port, p_e1_thread_structure->b_busy)==DG_EXIT_SUCCESS)
								if (option_verbose >= 3)
									ast_log(LOG_NOTICE, "dgv_indicate: SENDING CONGESTION\n");
						}
					} //if (p_e1_thread_structure)
					else
						ast_log(LOG_WARNING, "dgv_indicate: p_e1_thread_structure does not exist(AST_CONTROL_CONGESTION) on port %d\n", p->port);
	            	
					//if (p->ast_ch)
					//    ast_setstate(p->ast_ch, AST_STATE_BUSY);
	
		            /*f.frametype = AST_FRAME_CONTROL;
	    	        f.subclass = AST_CONTROL_HANGUP;
	        	    ast_dgv_lock(&p->lock, p->port); {
	            	    QueueFrame(p->port, &f);
	            	} ast_dgv_unlock(&p->lock, p->port);*/
	
	            	if (p->gen_busy)
	            	{
	            		//ast_dgv_lock(&p->lock, p->port);
	            		//{
            			switch (p->tone_thread_running)
            			{
            				case 0:
            					p->tone_thread_running = 1;
                	        	p->tone_thread_type = AST_CONTROL_BUSY;
                	        	p->setting_tone_thread_values = 1;
                	        	
                        		pthread_attr_init(&attr);
                            	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
                            	if (pthread_create(&p->tone_thread_id, &attr, ToneThread, &(p->port)) < 0)
            						p->tone_thread_running = 0;
            	               	else
            	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                    	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread congestion created on port %d\n", p->port);
                        	   	pthread_attr_destroy(&attr);
            					break;
            				case 1:
            	        		p->tone_thread_type = AST_CONTROL_BUSY;
            	        		p->setting_tone_thread_values = 1;
            	        		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread congestion reconfigured on port %d\n", p->port);
            					break;
            				case 2:
            					p->tone_thread_running = 1;
        	        			p->tone_thread_type = AST_CONTROL_BUSY;
        	        			p->setting_tone_thread_values = 1;
        	        			if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
        	        				ast_log(LOG_NOTICE, "dgv_indicate: tone_thread congestion re-enabled on port %d\n", p->port);
            					break;
            			} //switch (p->tone_thread_running)        			
	            		//}
	            		//ast_dgv_unlock(&p->lock, p->port);
					} //if (p->gen_busy)
	            	break;
	        	case AST_CONTROL_RINGING:
	            	//ast_dgv_lock(&p->lock, p->port);
	            	//{
                	//dg_DisableInputBuffer(p->port);
                    if ((!p->subchannel) && (p->call_state!=DG_AST_CONVERSATION))
                    {
                        dg_PauseInputBuffer(p->port, DG_PAUSE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "dgv_indicate: Pausing InputBuffer on port %d\n", p->port);

						if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
							ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_RINGING received on port %d, pausing input buffer thread\n", p->port);
                    }
                    else
                    	if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
							ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_RINGING received on port %d, we are in conversation, do not pause input buffer thread\n", p->port);

	                if (!p->ast_ch)
	                {
	                	ast_log(LOG_WARNING, "dgv_indicate: p->ast_ch does not exist(AST_CONTROL_RINGING) on port %d\n", p->port);
	                	ast_dgv_unlock(&p->lock, p->port);
	                	return 0;
	                }
                	
#ifdef USE_LIBPRI
						if (datalen > 0)
						{
							s = strsep(&text_received, generic_sep);
							if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_indicate_header, sizeof(cmd_indicate_header))))
							{
								if (option_verbose >= 3)
									ast_verbose(VERBOSE_PREFIX_3 "dgv_indicate called by DgSendISDNSig application\n");

								pause_before_send = 0;

								s = strsep(&text_received, generic_sep);
								if ((s) && (strlen(s) > 0))
								{
									pause_before_send = atoi(s);

									if (option_debug==100)
										ast_log(LOG_NOTICE, "pause_before_send received: '%d'\n", pause_before_send);

        		            		if (p->alerting_sig_timeout_sched > -1)
									{
										if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
											ast_log(LOG_NOTICE, "app_DgSendISDNSig: Deleting isdn alerting signal schedule on port %d\n", p->port);

										res_sched = ast_sched_del(sched, p->alerting_sig_timeout_sched);
								    	if (res_sched == -1)
								    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

										p->alerting_sig_timeout_sched = -1;
									}

									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "app_DgSendISDNSig: Creating isdn alerting signal schedule on port %d (after %d ms)\n", p->port, pause_before_send);

									p->alerting_sig_timeout_sched = ast_sched_add(sched, pause_before_send, alerting_timeout_callback, p);

									ast_dgv_unlock(&p->lock, p->port);
									return 0;
								}
							}
						}

						if ((!p->alerting) && p->sig==DG_SIG_PRI && p->pri && !p->outgoing && (dgv_channel_state(p->ast_ch) != AST_STATE_UP))
						{
							if (p->pri)
							{
								if (p->call)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "dgv_indicate: sending pri_acknowledge port %d, pri_channel %d\n", p->port, p->pri_channel);
									
									if (!dgv_trylock_ccs(p, dc))
									{						
										pri_acknowledge(p->pri, p->call, p->pri_channel, !p->digital);

										if (option_verbose >= 3)
											ast_log(LOG_NOTICE, "Sending pri_acknowledge %d/%d\n", p->pri_channel, p->port);

										dgv_releaselock_ccs(dc);
									}
									else
										ast_log(LOG_WARNING, "dgv_indicate: Unable to grab ccs_lock on port %d\n", p->port);						
								}
								else
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "dgv_indicate: did not call pri_acknowledge because p->call is null on %d\n", p->port);
										
							}
							p->alerting = 1;
						}
#endif//#ifdef USE_LIBPRI           	
            	
		            if (p->call_state==DG_AST_R2_SIGNALLING)
	    	        {
		            	if (!p->wait_group_b)
		            		dg_R2SendGroupB(p->port, p->group_b_value);
	            	}
	            	else //if (p->call_state==DG_AST_R2_SIGNALLING)
	            	{
		                if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	    	                ast_log(LOG_NOTICE, "dgv_indicate: tonethread %x %x\n", dgv_channel_state(p->ast_ch), p->call_state);

	        	        if ((dgv_channel_state(p->ast_ch) == AST_STATE_UP || dgv_channel_state(p->ast_ch) == AST_STATE_RING) && p->gen_ringback)
	            	    {
	                        if ((!p->subchannel) && (p->call_state!=DG_AST_CONVERSATION))
	                        {
	                            dg_PauseInputBuffer(p->port, DG_PAUSE);
				                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
				                	ast_log(LOG_WARNING, "dgv_indicate: Pausing InputBuffer on port %d\n", p->port);

	    						if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	    							ast_log(LOG_NOTICE, "dgv_indicate: tone_thread will be started, pausing input buffer thread on port %d\n", p->port);
	                        }
	                        else
	                        	if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	    							ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_RINGING received on port %d, we are in conversation, do not pause input buffer thread\n", p->port);

                			switch (p->tone_thread_running)
                			{
                				case 0:
                					p->tone_thread_running = 1;
                    	        	p->tone_thread_type = condition;
                    	        	p->setting_tone_thread_values = 1;
                    	        	
                            		pthread_attr_init(&attr);
                                	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
                                	if (pthread_create(&p->tone_thread_id, &attr, ToneThread, &(p->port)) < 0)
                						p->tone_thread_running = 0;
                	               	else
                	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                        	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing created on port %d\n", p->port);
                            	   	pthread_attr_destroy(&attr);
                					break;
                				case 1:
                	        		p->tone_thread_type = condition;
                	        		p->setting_tone_thread_values = 1;
                	        		p->need_ringback = 1; //if we are reconfiguring it, so enable thread...
                	        		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                    	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing reconfigured on port %d\n", p->port);
                					break;
                				case 2:
                					p->tone_thread_running = 1;
            	        			p->tone_thread_type = condition;
            	        			p->setting_tone_thread_values = 1;
            	        			if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
            	        				ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing re-enabled on port %d\n", p->port);
                					break;
                			} //switch (p->tone_thread_running)
	        	        } //if ((dgv_channel_state(p->ast_ch) == AST_STATE_UP || dgv_channel_state(p->ast_ch) == AST_STATE_RING) && p->gen_ringback)
	            	} //if (p->call_state==DG_AST_R2_SIGNALLING)
		            //}
		            //ast_dgv_unlock(&p->lock, p->port);
	            	break;
	        	case AST_CONTROL_BUSY:
	        		//ast_dgv_lock(&p->lock, p->port); 
	        		//{
        			if (!p->ast_ch)
        			{
	                	ast_log(LOG_WARNING, "dgv_indicate: p->ast_ch does not exist(AST_CONTROL_BUSY) on port %d\n", p->port);
	                	if (p->call_state==DG_AST_R2_SIGNALLING)
	                	{
	                		dg_PauseInputBuffer(p->port, DG_PAUSE);
	                		if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
	                			ast_log(LOG_WARNING, "dgv_indicate: Pausing InputBuffer on port %d\n", p->port);
	                	}
	                	ast_dgv_unlock(&p->lock, p->port);
	                	return 0;        				
        			}

	        		if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
	        			dgv_channel_hangupcause_set(ast, AST_CAUSE_USER_BUSY);
	        		
		            if (p->call_state==DG_AST_R2_SIGNALLING)
	    	        {
	                   	//dg_DisableInputBuffer(p->port);
	                   	dg_PauseInputBuffer(p->port, DG_PAUSE);
		                if ((p->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
		                	ast_log(LOG_WARNING, "dgv_indicate: Pausing InputBuffer on port %d\n", p->port);

	                    dg_signal_e1_thread_structure *p_e1_thread_structure;//pointer to voicerlib e1 thread structure            	
	                	
	    				p_e1_thread_structure = dg_GetE1ThreadPointer(p->port);

	    				if (p_e1_thread_structure)
	    				{   
	    					if (!p->wait_group_b)
	    					{
	    						if (dg_R2SendGroupB(p->port, p_e1_thread_structure->b_busy)==DG_EXIT_SUCCESS)
	    							if (option_verbose >= 3)
	    								ast_log(LOG_NOTICE, "dgv_indicate: SENDING CONGESTION\n");
	    					}
	    				} //if (p_e1_thread_structure)
	    				else
	    					ast_log(LOG_WARNING, "dgv_indicate: p_e1_thread_structure does not exist(AST_CONTROL_BUSY) on port %d\n", p->port);	                	

	                	//dg_R2SendGroupB(p->port, B_FREE_CALLING);
	                	//dgv_answer(p->ast_ch);
	
	                   	ast_setstate(p->ast_ch, AST_STATE_BUSY);
	                   	if (option_verbose >= 3)
	                    	ast_log(LOG_NOTICE, "dgv_indicate: BUSY %x %x\n",dgv_channel_state(p->ast_ch), p->call_state);
	                
	            	    //...p->reserved=0;
	                	if ( (p->sig==DG_SIG_FXO) || (p->sig==DG_SIG_CB_FXO) || (p->sig==DG_SIG_CUSTOMCAS) ||
	                		 (p->sig==DG_SIG_FXS) || (p->sig==DG_SIG_CB_FXS) || (p->sig==DG_SIG_PRI) ||
	                		 (p->sig==DG_SIG_GSM) )
	                    	p->reserved=0;
	                	else
	                	{
	                    	p->reserved=2;      //wait receiving idle
	                    	if (((p->verbosity & DG_LOG_AST_INDICATE) != 0) || ((p->verbosity & DG_LOG_GENERAL_EVENTS) != 0))
	                    	    ast_log(LOG_NOTICE, "dgv_indicate (%d): Changing reserved to 2.\n", p->port);
	                	}
	
	                	struct ast_frame f = { AST_FRAME_CONTROL, };
#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass.integer = AST_CONTROL_HANGUP;
#else//#ifdef CC_AST_HAS_VERSION_1_8
							f.subclass = AST_CONTROL_HANGUP;
#endif//#ifdef CC_AST_HAS_VERSION_1_8
	                	f.src = "chan_dgv";
	                	f.datalen = 0;
	                	//ast_dgv_lock(&p->lock, p->port); {
		                QueueFrame(p, &f);
	    	            //} ast_dgv_unlock(&p->lock, p->port);
	        	    }
	            	else //if (p->call_state==DG_AST_R2_SIGNALLING)
	            	{
	                	if (p->gen_busy)
	                	{
	            			switch (p->tone_thread_running)
	            			{
	            				case 0:
	            					p->tone_thread_running = 1;
	                	        	p->tone_thread_type = condition;
	                	        	p->setting_tone_thread_values = 1;
	                	        	
	                        		pthread_attr_init(&attr);
	                            	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	                            	if (pthread_create(&p->tone_thread_id, &attr, ToneThread, &(p->port)) < 0)
	            						p->tone_thread_running = 0;
	            	               	else
	            	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
	                    	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread busy created on port %d\n", p->port);
	                        	   	pthread_attr_destroy(&attr);
	            					break;
	            				case 1:
	            	        		p->tone_thread_type = condition;
	            	        		p->setting_tone_thread_values = 1;
	            	        		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
	                	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread busy reconfigured on port %d\n", p->port);
	            					break;
	            				case 2:
	            					p->tone_thread_running = 1;
	        	        			p->tone_thread_type = condition;
	        	        			p->setting_tone_thread_values = 1;
	        	        			if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
	        	        				ast_log(LOG_NOTICE, "dgv_indicate: tone_thread busy re-enabled on port %d\n", p->port);
	            					break;
	            			} //switch (p->tone_thread_running)
	                	} //if (p->gen_busy)
	                	//schedule to hangup within 10s anyway
	                	//ast_channel_setwhentohangup(p->ast_ch, 10);  removed on version 0.9.5_rc5
	            	} //if (p->call_state==DG_AST_R2_SIGNALLING)
	        		//}
	        		//ast_dgv_unlock(&p->lock, p->port);
	            	break;
	    		case AST_CONTROL_PROCEEDING:
	    			if (option_verbose >= 3)
	    				ast_log(LOG_NOTICE, "Received AST_CONTROL_PROCEEDING on port %d\n", p->port);
#ifdef USE_LIBPRI
					if (datalen > 0)
					{
						s = strsep(&text_received, generic_sep);
						if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_indicate_header, sizeof(cmd_indicate_header))))
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "dgv_indicate called by DgSendISDNSig application\n");

							pause_before_send = 0;

							s = strsep(&text_received, generic_sep);
							if ((s) && (strlen(s) > 0))
							{
								pause_before_send = atoi(s);

								if (option_debug==100)
									ast_log(LOG_NOTICE, "pause_before_send received: '%d'\n", pause_before_send);

    		            		if (p->proceeding_sig_timeout_sched > -1)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "app_DgSendISDNSig: Deleting isdn proceeding signal schedule on port %d\n", p->port);

									res_sched = ast_sched_del(sched, p->proceeding_sig_timeout_sched);
							    	if (res_sched == -1)
							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

									p->proceeding_sig_timeout_sched = -1;
								}

								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "app_DgSendISDNSig: Creating isdn proceeding signal schedule on port %d (after %d ms)\n", p->port, pause_before_send);

								p->proceeding_sig_timeout_sched = ast_sched_add(sched, pause_before_send, proceeding_timeout_callback, p);

								ast_dgv_unlock(&p->lock, p->port);
								return 0;
							}
						}
					}

	    			if (!p->proceeding && p->sig==DG_SIG_PRI && p->pri && !p->outgoing)
	    			{
	    				if (p->pri)
	    				{		
							if (p->call)
							{
								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "dgv_indicate: sending pri_proceeding port %d, pri_channel %d\n", p->port, p->pri_channel);
								
								if (!dgv_trylock_ccs(p, dc))
		    					{
		    						pri_proceeding(p->pri, p->call, p->pri_channel, !p->digital);

									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "Sending pri_proceeding %d/%d\n", p->pri_channel, p->port);

		    						dgv_releaselock_ccs(dc);
		    					}
		    					else
		    						ast_log(LOG_WARNING, "dgv_indicate: Unable to grab ccs_lock on port %d\n", p->port);
							}
							else
								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "dgv_indicate: did not call pri_proceeding because p->call is null on %d\n", p->port);		    					
	    				}
	    				p->proceeding = 1;
	    			}
#endif//#ifdef USE_LIBPRI
	    			break;
	    		case AST_CONTROL_PROGRESS:
	    			if (option_verbose >= 3)
	    				ast_log(LOG_NOTICE, "Received AST_CONTROL_PROGRESS on port %d\n", p->port);

	    			if (p->ast_ch)
					{
#ifdef CC_AST_HAS_VERSION_13
							fv_rawreadformat = ast_channel_rawreadformat(p->ast_ch);
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_11
								ast_format_copy(&fv_rawreadformat, ast_channel_rawreadformat(p->ast_ch));
	#else//#ifdef CC_AST_HAS_VERSION_11
								fv_rawreadformat = p->ast_ch->rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_11
#endif//#ifdef CC_AST_HAS_VERSION_13
						struct ast_frame fr = { AST_FRAME_VOICE, };
#ifdef CC_AST_HAS_VERSION_10
							fr.subclass.format = fv_rawreadformat;
#else//#ifdef CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_1_8
								fr.subclass.integer = fv_rawreadformat;
	#else//#ifdef CC_AST_HAS_VERSION_1_8
								fr.subclass = fv_rawreadformat;
	#endif//#ifdef CC_AST_HAS_VERSION_1_8
#endif//#ifdef CC_AST_HAS_VERSION_10

#ifdef CC_AST_HAS_VERSION_13
							if (ast_format_cmp(fv_rawreadformat, ast_format_slin) == AST_FORMAT_CMP_EQUAL)
							{
								fr.samples = DATALEN;
								fr.datalen = DATALEN*2;
							}
							else
								if (ast_format_cmp(fv_rawreadformat, ast_format_gsm) == AST_FORMAT_CMP_EQUAL)
								{
									fr.samples = DATALEN;
									fr.datalen = GSM_DATALEN;
								}
								else
									if ((ast_format_cmp(fv_rawreadformat, ast_format_alaw) == AST_FORMAT_CMP_EQUAL) ||
											(ast_format_cmp(fv_rawreadformat, ast_format_ulaw) == AST_FORMAT_CMP_EQUAL))
									{
										fr.samples = DATALEN;
										fr.datalen = DATALEN;
									}
									else
									{
										fr.samples = DATALEN;
										fr.datalen = DATALEN;
									}
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
								switch (fv_rawreadformat.id)
	#else//#ifdef CC_AST_HAS_VERSION_10
								switch (fv_rawreadformat)
	#endif//#ifdef CC_AST_HAS_VERSION_10
							{
								case AST_FORMAT_SLINEAR:
									fr.samples = DATALEN;
									fr.datalen = DATALEN*2;
									break;
								case AST_FORMAT_GSM:
									fr.samples = DATALEN;
									fr.datalen = GSM_DATALEN;
									break;
								case AST_FORMAT_ALAW:
								case AST_FORMAT_ULAW:
									fr.samples = DATALEN;
									fr.datalen = DATALEN;
									break;
								default:
									fr.samples = DATALEN;
									fr.datalen = DATALEN;
							} //switch (fv_rawreadformat)
#endif//#ifdef CC_AST_HAS_VERSION_13

#ifdef CC_AST_HAS_DATA_PTR
							fr.data.ptr = p->buf + AST_FRIENDLY_OFFSET;
#else//#ifdef CC_AST_HAS_DATA_PTR
							fr.data = p->buf + AST_FRIENDLY_OFFSET;
#endif//#ifdef CC_AST_HAS_DATA_PTR

						fr.src = "chan_dgv";

						if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
							ast_verbose(VERBOSE_PREFIX_3 "Sending QueueFrame into dgv_indicate on port %d\n", p->port);

						QueueFrame(p, &fr);
					}
	    			else //if (p->ast_ch)
	                	ast_log(LOG_WARNING, "dgv_indicate: p->ast_ch does not exist(AST_CONTROL_PROGRESS) on port %d\n", p->port);

	    			if (!p->subchannel)
    					if (p->subchannel_audio_index == 0)
    						p->subchannel_audio_index = 1;

#ifdef USE_LIBPRI
					if (datalen > 0)
					{
						s = strsep(&text_received, generic_sep);
						if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_indicate_header, sizeof(cmd_indicate_header))))
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "dgv_indicate called by DgSendISDNSig application\n");

							pause_before_send = 0;

							s = strsep(&text_received, generic_sep);
							if ((s) && (strlen(s) > 0))
							{
								pause_before_send = atoi(s);

								if (option_debug==100)
									ast_log(LOG_NOTICE, "pause_before_send received: '%d'\n", pause_before_send);

    		            		if (p->progress_sig_timeout_sched > -1)
								{
									if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
										ast_log(LOG_NOTICE, "app_DgSendISDNSig: Deleting isdn progress signal schedule on port %d\n", p->port);

									res_sched = ast_sched_del(sched, p->progress_sig_timeout_sched);
							    	if (res_sched == -1)
							    		ast_log(LOG_WARNING, "Failed to remove scheduler entry for port %d\n", p->port);

									p->progress_sig_timeout_sched = -1;
								}

								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "app_DgSendISDNSig: Creating isdn progress signal schedule on port %d (after %d ms)\n", p->port, pause_before_send);

								p->progress_sig_timeout_sched = ast_sched_add(sched, pause_before_send, progress_timeout_callback, p);

								ast_dgv_unlock(&p->lock, p->port);
								return 0;
							}
						}
					}

	    			p->digital = 0;	/* Digital-only calls isn't allows any inband progress messages */
	    			if (!p->progress && p->sig==DG_SIG_PRI && p->pri && !p->outgoing)
	    			{
	    				if (p->pri)
	    				{		
							if (p->call)
							{
								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "dgv_indicate: sending pri_progress port %d, pri_channel %d\n", p->port, p->pri_channel);
								
								if (!dgv_trylock_ccs(p, dc))
		    					{	    						
		    						pri_progress(p->pri, p->call, p->pri_channel, 1);

									if (option_verbose >= 3)
										ast_log(LOG_NOTICE, "Sending pri_progress %d/%d\n", p->pri_channel, p->port);

		    						dgv_releaselock_ccs(dc);
		    					}
		    					else
		    						ast_log(LOG_WARNING, "dgv_indicate: Unable to grab ccs_lock on port %d\n", p->port);
							}
							else
								if (((p->verbosity & DG_LOG_ISDN_EVENTS) != 0) || (((p->verbosity & DG_LOG_AST_INDICATE) != 0)))
									ast_log(LOG_NOTICE, "dgv_indicate: did not call pri_progress because p->call is null on %d\n", p->port);		    					
	    				}
	    				p->progress = 1;
	    			}
#endif//#ifdef USE_LIBPRI

	                if (!p->ast_ch)
	                {
	                	ast_log(LOG_WARNING, "dgv_indicate: p->ast_ch does not exist(AST_CONTROL_PROGRESS) on port %d\n", p->port);
	                	ast_dgv_unlock(&p->lock, p->port);
	                	return 0;
	                }	    			
	    			
	                if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
    	                ast_log(LOG_NOTICE, "dgv_indicate: tonethread %x %x\n", dgv_channel_state(p->ast_ch), p->call_state);

        	        if ((dgv_channel_state(p->ast_ch) == AST_STATE_UP || dgv_channel_state(p->ast_ch) == AST_STATE_RING) && p->gen_ringback)
            	    {
            			switch (p->tone_thread_running)
            			{
            				case 0:
            					p->tone_thread_running = 1;
                	        	p->tone_thread_type = AST_CONTROL_RINGING;
                	        	p->setting_tone_thread_values = 1;
                	        	
                        		pthread_attr_init(&attr);
                            	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
                            	if (pthread_create(&p->tone_thread_id, &attr, ToneThread, &(p->port)) < 0)
            						p->tone_thread_running = 0;
            	               	else
            	               		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                    	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing created on port %d\n", p->port);
                        	   	pthread_attr_destroy(&attr);
            					break;
            				case 1:
            	        		p->tone_thread_type = AST_CONTROL_RINGING;
            	        		p->setting_tone_thread_values = 1;
            	        		if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
                	               	ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing reconfigured on port %d\n", p->port);
            					break;
            				case 2:
            					p->tone_thread_running = 1;
        	        			p->tone_thread_type = AST_CONTROL_RINGING;
        	        			p->setting_tone_thread_values = 1;
        	        			if (((p->verbosity & DG_LOG_THREADS) != 0) || ((p->verbosity & DG_LOG_AST_INDICATE) != 0))
        	        				ast_log(LOG_NOTICE, "dgv_indicate: tone_thread ringing re-enabled on port %d\n", p->port);
            					break;
            			} //switch (p->tone_thread_running)
        	        } //if ((dgv_channel_state(p->ast_ch) == AST_STATE_UP || dgv_channel_state(p->ast_ch) == AST_STATE_RING) && p->gen_ringback)
	    			break;
	        	case AST_CONTROL_ANSWER:
		        	if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)//if (option_verbose >= 3)
	    	        	ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_ANSWER condition = %d\n",condition);
		        	
		        	if ((p->sig == DG_SIG_R2_NDIS_FIXED) || (p->sig == DG_SIG_R2_NDIS_VARIABLE))
		        		dgv_channel_hangupcause_set(ast, AST_CAUSE_NORMAL);
	        	    break;                             
	        	case AST_CONTROL_HOLD:
					if (p)
					{		
		        		if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	            			ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_HOLD condition = %d\n",condition);				
					
						if (option_verbose >= 3)
	    	        		ast_log(LOG_NOTICE, "dgv_indicate: ast_moh_start on port %d \n", p->port);
	            		
#ifdef CC_AST_HAS_INDICATE_DATA                        
							ast_moh_start(ast, data, DEFAULT_MOHINTERPRET);
#else//#ifdef CC_AST_HAS_INDICATE_DATA
							ast_moh_start(ast, DEFAULT_MOHINTERPRET);
#endif//#ifdef CC_AST_HAS_INDICATE_DATA
					}						            
	            	break;       
	        	case AST_CONTROL_UNHOLD:
					if (p)
					{
	        			if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	            			ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_UNHOLD condition = %d\n",condition);
	            							
						if (option_verbose >= 3)
	            			ast_log(LOG_NOTICE, "dgv_indicate: ast_moh_stop on port %d \n", p->port);

						ast_moh_stop(ast);            		
					}			                      
	            	break;
	        	case AST_CONTROL_VIDUPDATE://?????????????????????????
		        	if (option_verbose >= 3)
	    	        	ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_VIDUPDATE condition = %d\n",condition);
	        	    /*ast_dgv_lock(&p->lock, p->port);
	        	    {
	            		dg_StopPlayBuffer(p->port);
	            	}
	        	    ast_dgv_unlock(&p->lock, p->port);*/
	            	break;
				case AST_CONTROL_FLASH:
					if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)//if (option_verbose >= 3)
	            		ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_FLASH condition = %d\n",condition);
					if (p)
					{		
						if ((p->verbosity & DG_LOG_AST_INDICATE) != 0)
	            			ast_log(LOG_NOTICE, "dgv_indicate: AST_CONTROL_FLASH on port %d \n", p->port);
					}				                    
	            	break;
	    	}
		}
		ast_dgv_unlock(&p->lock, p->port);
			
		return 0;
	}

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	#ifdef CC_AST_HAS_VERSION_11
		static int dgv_devicestate(const char *data)
	#else//#ifdef CC_AST_HAS_VERSION_11
		static int dgv_devicestate(void *data)
	#endif//#ifdef CC_AST_HAS_VERSION_11
	{
		/* make sure data is not null. Maybe unnecessary, but better be safe */
		char *text_received = ast_strdupa(data ? data : "");
		char *port_received = ast_strdupa(data ? data : "");
		struct dgv_pvt *p;
		short port=1;

		struct dgv_pvt *start;
		char *s;
		char header[20];
	    char resource[20];
		short group=-1;
		int res_total_time = 0;

		int res = 0;

		if (option_debug==100)
			ast_log(LOG_DEBUG, "A text was received: '%s'\n", text_received);

		s = strsep(&text_received, generic_sep);
		if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_header, sizeof(cmd_header))))
		{
			if (option_verbose >= 3)
				ast_verbose(VERBOSE_PREFIX_3 "dgv_devicestate called by DgGetStatistics application\n");

			header[0]   = '\0';
			resource[0] = '\0';

			ast_copy_string(header, s, sizeof(header));

			if (option_debug==100)
				ast_log(LOG_NOTICE, "header received: '%s'\n", header);

			s = strsep(&text_received, generic_sep);
			if ((s) && (strlen(s) > 0))
			{
				ast_copy_string(resource, s, sizeof(resource));

				if (option_debug==100)
					ast_log(LOG_NOTICE, "resource received: '%s'\n", resource);

				//check for group syntax
				if (toupper(resource[0]) == 'G' || toupper(resource[0])== 'R')
				{
					if (option_verbose >= 3)
						ast_log(LOG_NOTICE, "Statistics_Group %s\n", resource);

					group = atoi(resource+1);

					ast_dgv_lock(&iflock, 0);
					{
						start = iffirst;
						p = start;
						while (p)
						{
							if (group == p->group)
							{
								ast_dgv_lock(&p->lock, p->port);
								{
									res_total_time = res_total_time + p->statistics.total_time.tv_sec;
								}
								ast_dgv_unlock(&p->lock, p->port);
							}

							p = p->next;
						} //while (p)
					}
					ast_dgv_unlock(&iflock, 0);
				}
				else
				{
					port = atoi(resource);
					if (option_verbose >= 3)
						ast_log(LOG_NOTICE, "Statistics_Port %d\n", port);

					p = find_ch_port(port);

					if (p)
					{
						ast_dgv_lock(&p->lock, p->port);
						{
							res_total_time = p->statistics.total_time.tv_sec;
						}
						ast_dgv_unlock(&p->lock, p->port);
					}
					else
						ast_log(LOG_WARNING, "%s is not a Dgv channel\n", resource);
				}
			}
			else //if ((s) && (strlen(s) > 0))
				if (option_verbose >= 3)
					ast_log(LOG_WARNING, "dgv_devicestate, did not receive the resource parameter, dropping this message.\n");

			return res_total_time;
		}
		else //if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_header, sizeof(cmd_header))))
		{
			res = AST_DEVICE_INVALID;

			/* make sure data is not null. Maybe unnecessary, but better be safe */
			port = atoi(port_received);

			if (option_debug==100)
				ast_log(LOG_DEBUG, "Checking device state for port %d\n", port);

			p = find_ch_port(port);
			if (p)
			{
				if (p->alarmed > 0 || p->locked > 0)
					res = AST_DEVICE_UNAVAILABLE;
				else
				{
					switch (p->reserved)
					{
						case 0:
							if (p->call_state == DG_AST_CALLING)
								res = AST_DEVICE_RINGING;
							else
								if (p->call_state == DG_AST_NONE)
									res = AST_DEVICE_NOT_INUSE;
								else
									res = AST_DEVICE_INUSE;
							break;
						case 1:
							if (p->call_state == DG_AST_CALLING)
								res = AST_DEVICE_RINGINUSE;
							else
								res = AST_DEVICE_INUSE;
							break;
						case 2:
							if ((p->sig == DG_SIG_FXS) || (p->sig == DG_SIG_CB_FXS))
								res = AST_DEVICE_NOT_INUSE;
							else
								res = AST_DEVICE_BUSY;
							break;
					} //switch (p->reserved)
				} //if (p->alarmed > 0 || p->locked > 0)
			} //if (p)

			return res;
		} //if (((s) && (strlen(s) > 0)) && (!strncmp(s, cmd_header, sizeof(cmd_header))))
	}

	#ifdef CC_AST_HAS_SNMP
	    static int dgv_snmp_show_alarms(const char *type, int card, char *alarms)
		{
#ifdef DEBUG_PRI
			ast_log(LOG_NOTICE, "dgv_snmp_show_alarms: STEP 1...\n");
#endif//#ifdef DEBUG_PRI

			struct dgv_pvt *tmp;
			short nForPort;

			char temp_textalarm[127]; //less than original alarm variable (256)
			char textalarm[255]; //less than original alarm variable (256)

	    	if (!channel_up)
	    	{
				if (option_debug==100)
					ast_log(LOG_DEBUG, "*** Channel not ready to answer! ***\n");

				return -1;
	    	}

	    	ast_dgv_lock(&iflock, 0);

			if (!strncmp(type, ttype, 3))
			{
				if (card == 0)
				{
					if (option_debug==100)
						ast_log(LOG_DEBUG, "Checking DigiVoice cards count...\n");

					ast_dgv_unlock(&iflock, 0);

					return dg_GetCardsCount();
				}
				else //if (card == 0)
				{
					if ((card < 0) || (card > dg_GetCardsCount()))
					{
						if (option_debug==100)
							ast_log(LOG_DEBUG, "Card %d does not exist!\n", card);

						ast_dgv_unlock(&iflock, 0);

						return -1;
					}

					if (option_debug==100)
						ast_log(LOG_DEBUG, "Checking DigiVoice alarms (card %d)...\n", card);

					textalarm[0] = '\0';

					switch (dg_GetCardInterface(card))
					{
						case DG_DIGITAL_INTERFACE:
							if (dg_GetCardPortsCount(card) == 60)
							{
								temp_textalarm[0] = '\0';

								sprintf(temp_textalarm, "<"
														"CARD:%d,DG_DIGITAL_INTERFACE,E1;"
														"SPAN:%d;"
														"ALARM_LOSS:%d,%d;"
														"ALARM_RAIS:%d,%d;"
														"ALARM_RSLIP:%d"
														">",
														card,
														1,
														dgv_e1alarms[card - 1][0].nALARM_LOSS, dgv_e1alarmsCount[card - 1][0].nALARM_LOSS,
														dgv_e1alarms[card - 1][0].nALARM_RAIS, dgv_e1alarmsCount[card - 1][0].nALARM_RAIS,
														dgv_e1alarms[card - 1][0].nALARM_RSLIP
										);

								strcat(textalarm, temp_textalarm);
								temp_textalarm[0] = '\0';

								sprintf(temp_textalarm, "<"
														"CARD:%d,DG_DIGITAL_INTERFACE,E1;"
														"SPAN:%d;"
														"ALARM_LOSS:%d,%d;"
														"ALARM_RAIS:%d,%d;"
														"ALARM_RSLIP:%d"
														">",
														card,
														2,
														dgv_e1alarms[card - 1][1].nALARM_LOSS, dgv_e1alarmsCount[card - 1][1].nALARM_LOSS,
														dgv_e1alarms[card - 1][1].nALARM_RAIS, dgv_e1alarmsCount[card - 1][1].nALARM_RAIS,
														dgv_e1alarms[card - 1][1].nALARM_RSLIP
										);

								strcat(textalarm, temp_textalarm);
							}
							else //if (dg_GetCardPortsCount(c) == 60)
							{
								sprintf(textalarm, "<"
												   "CARD:%d,DG_DIGITAL_INTERFACE,E1;"
												   "SPAN:%d;"
												   "ALARM_LOSS:%d,%d;"
												   "ALARM_RAIS:%d,%d;"
												   "ALARM_RSLIP:%d"
												   ">",
												   card,
												   1,
												   dgv_e1alarms[card - 1][0].nALARM_LOSS, dgv_e1alarmsCount[card - 1][0].nALARM_LOSS,
												   dgv_e1alarms[card - 1][0].nALARM_RAIS, dgv_e1alarmsCount[card - 1][0].nALARM_RAIS,
												   dgv_e1alarms[card - 1][0].nALARM_RSLIP
										);
							} //if (dg_GetCardPortsCount(c) == 60)

							break;
						case DG_GSM_INTERFACE:
							temp_textalarm[0] = '\0';

							ast_copy_string(textalarm, "<", sizeof(textalarm));
							sprintf(temp_textalarm, "CARD:%d,DG_GSM_INTERFACE", card);
							strcat(textalarm, temp_textalarm);

							for (nForPort = 1; nForPort <= dg_GetCardPortsCount(card); nForPort++)
							{
								tmp = find_ch_port(dg_GetAbsolutePortNumber(card, nForPort));
								if (tmp)
								{
									sprintf(temp_textalarm, ";CHANNEL:%d,%d,%d",
											nForPort,
											(((dgv_gsmsignal[card - 1][nForPort - 1][0] >= 99) || (dgv_gsmsignal[card - 1][nForPort - 1][0] <= 0)) ? 0 : (-113 + (2 * dgv_gsmsignal[card - 1][nForPort - 1][0]))),
											(tmp->alarmed < 2) ? tmp->alarmed : 1);

									strcat(textalarm, temp_textalarm);
								}
								else
								{
									if (option_debug==100)
										ast_log(LOG_DEBUG, "Cannot check DigiVoice alarms for card %d, port %d when p is null\n", card, nForPort);

									break;
								}
							} //for (p = 1; p <= dg_GetCardPortsCount(c); p++)

							strcat(textalarm, ">");

							break;
						default:
							if (option_debug==100)
								ast_log(LOG_DEBUG, "Card %d is not an E1/GSM card!\n", card);

							ast_dgv_unlock(&iflock, 0);

							return -1;
					} //switch (dg_GetCardInterface(c))

					ast_copy_string(alarms, textalarm, sizeof(textalarm));

					ast_dgv_unlock(&iflock, 0);

					return 0;
				} //if (card == 0)
			}
			else
			{
				if (option_debug==100)
					ast_log(LOG_DEBUG, "%s is not a DigiVoice channel\n", type);

				ast_dgv_unlock(&iflock, 0);

				return -1;
			}
		}
	#endif//#ifdef CC_AST_HAS_SNMP

	#ifdef CC_AST_HAS_FAX
		static int dgv_faxstate(struct ast_channel *chan)
		{
			if (chan)
				if (!ast_strlen_zero(dgv_channel_name(chan)))
				{
					char *name_received = strdup(dgv_channel_name(chan) ? (char *)dgv_channel_name(chan) : "");

					if (option_verbose >= 3)
						ast_verbose(VERBOSE_PREFIX_3 "dgv_faxstate, get fax state command was received on '%s'\n", name_received);

					if (!strncmp(name_received, ttype, 3))
					{
						struct dgv_pvt *p = (struct dgv_pvt *)dgv_channel_tech_pvt(chan);

						if (p)
						{
							if (option_verbose >= 3)
								ast_verbose(VERBOSE_PREFIX_3 "dgv_faxstate is sending state %d on port %d.\n", p->fax_tone_state, p->port);

							return p->fax_tone_state;
						}
					}
					else
						ast_log(LOG_WARNING, "%s is not a Dgv channel\n", name_received);
				}

			return -1;
		}
	#endif//#ifdef CC_AST_HAS_FAX
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

static int load_statistics(void)
{
	FILE *f;
	char szFile[100];

	struct dgv_pvt *tmp;

	short nForChannel;
	short nStep;

	char buftotal[80];
    char szData[TAMSTR];

	for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)
	{
		nStep = 0;

		tmp = find_ch_port(nForChannel);

		if (!tmp)
		{
			ast_log(LOG_WARNING, "load_statistics: tmp %d does not exist\n", nForChannel);
			return 0;
		}

		if (option_verbose >= 3)
			ast_verbose(VERBOSE_PREFIX_3 "Loading statistics of port %d.\n", tmp->port);

		ast_dgv_lock(&tmp->lock, tmp->port);
		{
			//loading file control
			sprintf(szFile, "/var/lib/voicerlib/statistics/dgv_%d", tmp->port);
			if ((f = fopen(szFile, "r")) == NULL)
			{
				ast_log(LOG_WARNING, "load_statistics (%d): Can not open Statistics file (%s)\n", tmp->port, szFile);
				reset_statistics(tmp);
			}
			else
			{
				while (fgets(szData, TAMSTR - 1, f) != NULL)
				{
					if ((tmp->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
						ast_log(LOG_NOTICE, "load_statistics (%d): Statistics loaded (Step %d): '%s'\n", tmp->port, nStep, szData);

					switch (nStep)
					{
						case 0:
							memset(&tmp->statistics.answer, 0, sizeof(tmp->statistics.answer));
							break;
						case 1:
							tmp->locked = atoi(szData);
							if (tmp->locked != 0 && tmp->locked != 1)
								tmp->locked = 0;
							break;
						case 2:
							tmp->statistics.total_time.tv_sec = atoi(szData);
							break;
						case 3:
							tmp->statistics.calls_answered = atoi(szData);
							break;
						case 4:
							tmp->statistics.sms_sent = atoi(szData);
							break;
						case 5:
							ast_copy_string(tmp->statistics.last_reset, szData, sizeof(tmp->statistics.last_reset));
							break;
					} //switch (nStep)

					nStep++;
				} //while (fgets(szData, TAMSTR - 1, f) != NULL)

				fclose(f);
			}

			if ((tmp->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
			{
				snprintf(buftotal, sizeof(buftotal), "%ld", (long) tmp->statistics.total_time.tv_sec);

				ast_log(LOG_NOTICE, "load_statistics (%d): Statistics loaded (Last Step %d): Locked %d, TotalTime %s, CallsAnswered %d, SMSSent %d, LastReset %s\n",
						tmp->port,
						nStep,
						tmp->locked,
						buftotal,
						tmp->statistics.calls_answered,
						tmp->statistics.sms_sent,
						tmp->statistics.last_reset);
			}
		}
		ast_dgv_unlock(&tmp->lock, tmp->port);
	} //for (nForChannel = 1; nForChannel <= dg_GetPortsCount(); nForChannel++)

	return 1;
}

static void update_statistics_call(void *data)
{
	struct dgv_pvt *tmp = data;

	char buftotal[80];
	char buflast[80];
	time_t end;

    if (!tmp)
    {
		ast_log(LOG_WARNING, "update_statistics_call: p does not exist\n");
		return;
    }

	if (tmp->statistics.answer.tv_sec)
	{
		if (option_verbose >= 3)
			ast_verbose(VERBOSE_PREFIX_3 "Updating statistics (call) on port %d\n", tmp->port);

		time(&end);

		tmp->statistics.calls_answered++;
		tmp->statistics.last_answered.tv_sec = end - tmp->statistics.answer.tv_sec;
		tmp->statistics.total_time.tv_sec = tmp->statistics.last_answered.tv_sec + tmp->statistics.total_time.tv_sec;

		if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
			ast_log(LOG_WARNING, "update_statistics_call (%d): Can not write to Statistics file\n", tmp->port);

		if ((tmp->verbosity & DG_LOG_START_END_CALL) != 0)
		{
			snprintf(buflast, sizeof(buflast), "%ld", (long) tmp->statistics.last_answered.tv_sec);
			snprintf(buftotal, sizeof(buftotal), "%ld", (long) tmp->statistics.total_time.tv_sec);

			ast_log(LOG_NOTICE, "update_statistics_call (%d): Statistics updated to: LastAnsweredTime %s, TotalTime %s, CallsAnswered %d\n",
					tmp->port,
					buflast,
					buftotal,
					tmp->statistics.calls_answered);
		}
	}
	else
		if ((tmp->verbosity & DG_LOG_START_END_CALL) != 0)
			ast_log(LOG_NOTICE, "update_statistics_call (%d): No answer time found, statistics will not be updated...\n", tmp->port);

	memset(&tmp->statistics.answer, 0, sizeof(tmp->statistics.answer));

	return;
}

static void update_statistics_sms(void *data)
{
	struct dgv_pvt *tmp = data;

    if (!tmp)
    {
		ast_log(LOG_WARNING, "update_statistics_sms: p does not exist\n");
		return;
    }

	if (option_verbose >= 3)
		ast_verbose(VERBOSE_PREFIX_3 "Updating statistics (sms) on port %d\n", tmp->port);

	tmp->statistics.sms_sent++;

	if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
		ast_log(LOG_WARNING, "update_statistics_sms (%d): Can not write to Statistics file\n", tmp->port);

	if ((tmp->verbosity & DG_LOG_START_END_CALL) != 0)
	{
		ast_log(LOG_NOTICE, "update_statistics_sms (%d): Statistics updated to: SMSSent %d\n",
				tmp->port,
				tmp->statistics.sms_sent);
	}

	return;
}

static void reset_statistics(void *data)
{
	struct dgv_pvt *tmp = data;

	char buftotal[80];
	char buflast[80];

    if (!tmp)
    {
		if (option_verbose >= 3)
			ast_log(LOG_WARNING, "reset_statistics: p does not exist\n");

		return;
    }

	if (option_verbose >= 3)
		ast_verbose(VERBOSE_PREFIX_3 "Resetting statistics on port %d.\n", tmp->port);

	if (tmp->locked != 0 && tmp->locked != 1)
		tmp->locked = 0;
	memset(&tmp->statistics.answer, 0, sizeof(tmp->statistics.answer));
	memset(&tmp->statistics.last_answered, 0, sizeof(tmp->statistics.last_answered));
	memset(&tmp->statistics.total_time, 0, sizeof(tmp->statistics.total_time));
	tmp->statistics.calls_answered = 0;
	tmp->statistics.sms_sent = 0;
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		struct ast_tm tm;
		struct timeval now = ast_tvnow();
		ast_localtime(&now, &tm, NULL);
		ast_strftime(tmp->statistics.last_reset, sizeof(tmp->statistics.last_reset), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		time_t t;
		struct tm tm;
		time(&t);
		ast_localtime(&t, &tm, NULL);
		strftime(tmp->statistics.last_reset, sizeof(tmp->statistics.last_reset), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

	if (!statistics_to_file(tmp->port, tmp->locked, tmp->statistics.total_time, tmp->statistics.calls_answered, tmp->statistics.sms_sent, tmp->statistics.last_reset))
		ast_log(LOG_WARNING, "reset_statistics (%d): Can not write to Statistics file\n", tmp->port);

	if ((tmp->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)
	{
		snprintf(buflast, sizeof(buflast), "%ld", (long) tmp->statistics.last_answered.tv_sec);
		snprintf(buftotal, sizeof(buftotal), "%ld", (long) tmp->statistics.total_time.tv_sec);

		ast_log(LOG_NOTICE, "reset_statistics (%d): Statistics reseted to: LastAnsweredTime %s, TotalTime %s, CallsAnswered %d, SMSSent %d\n",
				tmp->port,
				buflast,
				buftotal,
				tmp->statistics.calls_answered,
				tmp->statistics.sms_sent);
	}

	return;
}

//-------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------
static int __unload_module(int reload)
{
	struct dgv_pvt *p;
	short ret;
	short nForCard;
	int nForSMSChannel;
	short ch_board;
	
	ast_log(LOG_NOTICE, "  -- Unload DigiVoice module\n");
	
	//unregister CLI commands
	switch (dgv_conf.consolelanguage)
	{
		case 0://English
			ast_cli_unregister_multiple(dgv_cli_en, sizeof(dgv_cli_en) / sizeof(dgv_cli_en[0]));
			break;
		case 1://Portuguese
			ast_cli_unregister_multiple(dgv_cli_br, sizeof(dgv_cli_br) / sizeof(dgv_cli_br[0]));
			break;
		default:
			ast_cli_unregister_multiple(dgv_cli_en, sizeof(dgv_cli_en) / sizeof(dgv_cli_en[0]));
			//0.9.8 break;		
	}					

	/* First, take us out of the channel loop */
	if (reload==0)
		ast_channel_unregister(&dgv_tech);

	if (!ast_dgv_lock(&iflock, 0)) 
	{
		/* Hangup all interfaces if they have an owner */
		p = iflast;
		while (p) 
		{
			if (p->ast_ch)
			{
				ast_softhangup(p->ast_ch, AST_SOFTHANGUP_APPUNLOAD);
	            p->reserved=0;
				ast_log(LOG_WARNING, "ast_softhangup for port %d\n",p->port);		
			}
			p = p->previous;
		}
		ast_dgv_unlock(&iflock, 0);
	} else {
		ast_log(LOG_WARNING, "Unable to lock the monitor\n");
		return -1;
	}

  	//destroy channel linked list
	ast_dgv_lock(&iflock, 0); 
  	{
		/* Destroy all the interfaces and free their memory */
		while(iflast) {
			p = iflast;		    
			ast_mutex_destroy(&p->record_lock);
			ast_mutex_destroy(&p->play_lock);
			ast_mutex_destroy(&p->lock);
            switch (p->sig)
            {
                case DG_SIG_R2_NDIS_FIXED:
                case DG_SIG_R2_NDIS_VARIABLE:
                    ret = dg_DestroyE1Thread(p->port);
                    if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)) {
                        ast_log(LOG_NOTICE, "Destroying E1 thread for port %d\n",p->port);		
                    }
                    break;
                case DG_SIG_PRI:
                    break;
                case DG_SIG_CUSTOMCAS:
                    ret = dg_DestroyCustomCAS(p->port);
                    if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)) {
                        ast_log(LOG_NOTICE, "Destroying Custom CAS thread for port %d\n",p->port);		
                    }
                    break;
                case DG_SIG_FXO:
                	dg_IdleAbort(p->port);
                    break;
    			case DG_SIG_CB_FXO:
    				dg_IdleAbort(p->port);
    			case DG_SIG_CB_FXS:    				
        			ret=dg_DestroyCBThread(p->port);
                    if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)) {
                        ast_log(LOG_NOTICE, "Destroying CB FXO/FXS thread for port %d\n",p->port);		
                    }
                    break;
    			case DG_SIG_GSM:
                    ret = dg_DestroyGSMThread(p->port);
                    if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0)) {
                        ast_log(LOG_NOTICE, "Destroying GSM thread for port %d\n",p->port);		
                    }
                    
        			for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
        				if (p->smschannel[nForSMSChannel])
        				{
        					ast_mutex_destroy(&p->smschannel[nForSMSChannel]->lock);		
        					free(p->smschannel[nForSMSChannel]);
        				}
        			p->iflastsmschannel = NULL;

                    break;
                //0.9.8 default:
    				//0.9.8 break;
            } //switch (p->sig)

        	if ((p->vconf_pass_through) || (p->fax_pass_through))
        	{
				ch_board = dg_GetRelativeChannelNumber(p->port);

				if ((ch_board > 0) && (ch_board < 31))
				{
					ret=dg_LocalBridgeDisconnect(p->port, p->vconf_port_bridged);
					if (ret==DG_EXIT_SUCCESS && ((p->verbosity & DG_LOG_LOAD_RESET_UNLOAD) != 0))
						ast_log(LOG_NOTICE, "Disconnecting vconf for port %d and port %d - cod %x\n", p->port, p->vconf_port_bridged, ret);
				}
        	}

			//calls all destructor functions
			dg_DisableInputBuffer(p->port);
			dg_StopPlayBuffer(p->port);
            dg_DestroyCallProgress(p->port);

			iflast = iflast->previous;

			if (p->ast_ch)
				dgv_channel_tech_pvt_set(p->ast_ch, NULL);

			free(p);
		}
		iflast = NULL;		
    } ast_dgv_unlock(&iflock, 0);
#ifdef USE_LIBPRI
	    if (dchans_count>0) {
    	    thd_socket_running = 0;
        	thd_pri_running = 0;

        	for (ret=0; ret<dchans_count; ret++) {
	            //closes ccs device drivers
            	close(dchans[ret].fds[TX]);
            	close(dchans[ret].fds[RX]);
        	}
    	}

		if (thd_sock)
		{
			pthread_cancel(thd_sock);
			pthread_kill(thd_sock, SIGURG);
			pthread_join(thd_sock, NULL);
		}
		thd_sock = AST_PTHREADT_STOP;

		if (thd_pri)
		{
			pthread_cancel(thd_pri);
			pthread_kill(thd_pri, SIGURG);
			pthread_join(thd_pri, NULL);
		}
		thd_pri = AST_PTHREADT_STOP;
#endif//#ifdef USE_LIBPRI
	    
		if (timer_thread) 
		{
			pthread_cancel(timer_thread);
			pthread_kill(timer_thread, SIGURG);
			pthread_join(timer_thread, NULL);
		}
		timer_thread = AST_PTHREADT_STOP;

	/* stops voicerlib engine */
	if (reload==0) {
		if (dg_DriverEnabled())
		{
    		//sets alarmmode for each card
    		for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
    		{
    			if (dg_GetCardInterface(nForCard) == DG_DIGITAL_INTERFACE)
    				dg_SetAlarmMode(nForCard, ALARM_MANUAL_NOTIFY);
    		}			

			ret = dg_ShutdownVoicerlib();
            if (ret!=DG_EXIT_SUCCESS)
            	ast_log(LOG_NOTICE, "  -- Shutdown VoicerLib, ret = %d\n", ret);
            else
            	ast_log(LOG_NOTICE, "  -- Shutdown VoicerLib\n");
		}
		ast_mutex_destroy(&iflock);
	}
	
#ifdef CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_13
			ao2_ref(dgv_tech.capabilities, -1);
	#else//#ifdef CC_AST_HAS_VERSION_13
			ast_format_cap_destroy(dgv_tech.capabilities);
	#endif//#ifdef CC_AST_HAS_VERSION_13

		ast_sched_context_destroy(sched);
#endif//#ifdef CC_AST_HAS_VERSION_10

  	ast_log(LOG_NOTICE, "  -- module unloaded\n");

	return 0;
}

int unload_module(void)
{
	channel_up = 0;
	ast_log(LOG_NOTICE, "unload_module: starting...\n");
	return __unload_module(0);
}

//------------------------------------------------------------
//starts all services and read configuration file
//------------------------------------------------------------
int setup_dgv(int reload)
{
	short ret, ch, nForCard, nForSpan, nForSMSChannel;
	struct ast_config *cfg;
	struct ast_variable *v;
	struct dgv_pvt *tmp;
	char szTemp[TAMSTR];
	char szContext[AST_MAX_CONTEXT] = "default\0";
	char szPort_ID[AST_MAX_EXTENSION] = "";
	char szPort_Name[AST_MAX_EXTENSION] = "";
	char szAccountCode[AST_MAX_ACCOUNT_CODE] = "";		//Account code
	short group=-1, sync, sync_alternative=0;
	short category=1,category_pass_through=0,txgain=0,rxgain=0,max_rx=4,min_rx=1,cid_pos=0,interdigit_timeout=3000;
	short echo_can=2, echo_type=ECHO_HW_DSP, echo_training=800, echo_nlp=1;
    short gen_dtmf=2;
    short dialtone_timeout=3000;
    short wait_dialtone=1;
    short fax_detection=1;
    short fax_pass_through=1;
    short polarity=2;
    short cid_detection_type=1;
    short dtmf_to_asterisk = 2;
    short gen_ringback=1;
    short gen_busy=1;
	//short auto_r2dmfc=1; //auto is default
	short signalling=0;
    short b_subscriber_hangup=0;
    short ccb_pass_through=0;
    short auto_dgccb=0;
    char deny_prefix[AST_MAX_EXTENSION] = "";
    short vconf_pass_through=0;
    short group_b_value=MFC_B_1;/* MFC_B_1 = default Brazil, old = B_FREE_CALLING */
    short wait_group_b=0;
    short r2_country=R2_COUNTRY_BR;//default Brazil
    short use_crc4=0;
    short silence_threshold=-36;
    char musicclass[MAX_MUSICCLASS] = "";
    
    char callprogress[TAMSTR] = "cp_default.cfg\0";
    char language[MAX_LANGUAGE] = "pt_BR\0";
    short drop_tx_samples_factor = TX_CUT_FACTOR;
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		struct ast_jb_conf jbconf;
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
    short relaxdtmf=1;
    
    //fxs
    int ring_type=1;
    int flash_min_time=80;
    int flash_max_time=650;
    int immediate=0;//fxs and pri channels     
    char mailbox[AST_MAX_EXTENSION] = "1234\0";
	short second_call=0;
	char second_call_digits[5] = "";

	char cur_callgroup[AST_MAX_EXTENSION] = "";
    char cur_pickupgroup[AST_MAX_EXTENSION] = "";
    
    //gsm
    short gsm_hide_number=0;
    short gsm_sms_confirmation=0;
    short gsm_sms_flashsms=0;
    short gsm_displaycallwaiting=1;
    char gsm_pin_number[5] = "";
    
    pthread_attr_t attr;

    unsigned int span_index;
    
    //arguments from span option
    int num_args;
    char *args[10];
    char tmpSerialNumber[100];
    char tmpIRQNumber[10];

	szAccountCode[0]='\0';
	
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		struct ast_flags config_flags = {reload ==  1 ? 0 : 0};
		cfg = ast_config_load(config, config_flags);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		cfg = ast_config_load(config);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	
	/* We *must* have a config file otherwise stop immediately */
	if (!cfg) {
		ast_log(LOG_ERROR, "***********************************************************************\n");
        ast_log(LOG_ERROR, "FATAL ERROR: Unable to load /etc/asterisk/digivoice.conf (File exists?)\n");
		ast_log(LOG_ERROR, "***********************************************************************\n");
		return -1;
	}  

	/* Make sure we can register our channel type */
	if (reload==0) 
	{
		//default values for codecs
#ifdef CC_AST_HAS_VERSION_10
	#ifdef CC_AST_HAS_VERSION_12
				if (!(dgv_tech.capabilities = ast_format_cap_alloc(0))) {
	#else//#ifdef CC_AST_HAS_VERSION_12
				if (!(dgv_tech.capabilities = ast_format_cap_alloc())) {
	#endif//#ifdef CC_AST_HAS_VERSION_12
				return AST_MODULE_LOAD_FAILURE;
			}

	#ifdef CC_AST_HAS_VERSION_13
				ast_format_cap_append(dgv_tech.capabilities, ast_format_alaw, 0);
	#else//#ifdef CC_AST_HAS_VERSION_13
				struct ast_format dgv_tech_fmt;
				ast_format_cap_add(dgv_tech.capabilities, ast_format_set(&dgv_tech_fmt, AST_FORMAT_ALAW, 0));
	#endif//#ifdef CC_AST_HAS_VERSION_13
#else//#ifdef CC_AST_HAS_VERSION_10
			dgv_tech.capabilities = AST_FORMAT_ALAW;
#endif//#ifdef CC_AST_HAS_VERSION_10

		if (ast_channel_register(&dgv_tech)) {
			ast_log(LOG_ERROR, "Unable to register channel class %s\n", ttype);
			__unload_module(0);
			return -1;
		}
	}

	dgv_conf.allow_slinear=0;
	dgv_conf.allow_ulaw=0;
	dgv_conf.allow_alaw=1;
	dgv_conf.allow_gsm=0;

    //language console
    dgv_conf.consolelanguage = 0;
    
    //fxs_transfer
    features_conf.fxs_atxfer[0] = '\0';
    features_conf.fxs_atxfer[1] = '\0';
    features_conf.fxs_atxfer[2] = '\0';
    features_conf.fxs_atxfer[3] = '\0';
    features_conf.fxs_atxfer[4] = '\0';            

    second_call_digits[0] = '\0';
    second_call_digits[1] = '\0';
    second_call_digits[2] = '\0';
    second_call_digits[3] = '\0';
    second_call_digits[4] = '\0';    

    dgv_conf.deny_prefix[0] = '\0';

	for (nForCard = 0; nForCard <= MAX_CARDS - 1; nForCard++)
		dgv_sync_conf[nForCard] = dgv_sync_alternative_conf[nForCard] = 0;

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		/* Copy the default jb config over p->jbconf */
		memcpy(&jbconf, &default_jbconf, sizeof(jbconf));
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

	//default values
	dgv_conf.auto_scan_port = 1;

	//set default interval
	dgv_conf.gsm_signal_update_interval = 5000;

	/* parse configuration file options */
	ast_dgv_lock(&iflock, 0); {
		v = ast_variable_browse(cfg, "general");
		while (v)
		{
			if (strcasecmp(v->name, "auto_port") == 0) {
				dgv_conf.auto_scan_port = atoi(v->value);
				if (dgv_conf.auto_scan_port>=0 && dgv_conf.auto_scan_port<=1)
					ast_log(LOG_NOTICE,"Dgv Channel: scanning ports mode: [%s]\n",auto_scan_port_desc[dgv_conf.auto_scan_port]);
				else
					ast_log(LOG_NOTICE,"Dgv Channel: No scanning ports mode found. Assuming automatic mode\n");
			}
			else if (strcasecmp(v->name, "allow_slinear") == 0) {
				dgv_conf.allow_slinear = atoi(v->value);
				ast_log(LOG_NOTICE,"Dgv Channel: SLINEAR support %s\n", dgv_conf.allow_slinear ? "enabled" : "disabled");
			}
			else if (strcasecmp(v->name, "allow_ulaw") == 0) {
				dgv_conf.allow_ulaw = atoi(v->value);
				ast_log(LOG_NOTICE,"Dgv Channel: U_LAW support %s\n", dgv_conf.allow_ulaw ? "enabled" : "disabled");
			}
			else if (strcasecmp(v->name, "allow_alaw") == 0) {
				dgv_conf.allow_alaw = atoi(v->value);
				ast_log(LOG_NOTICE,"Dgv Channel: A_LAW support %s\n", dgv_conf.allow_alaw ? "enabled" : "disabled");
			}
			else if (strcasecmp(v->name, "allow_gsm") == 0) {
				dgv_conf.allow_gsm = 0;//atoi(v->value);
				//ast_log(LOG_NOTICE,"Dgv Channel: GSM support %s\n", dgv_conf.allow_gsm ? "enabled" : "disabled");
			}
			else if (strcasecmp(v->name, "consolelanguage") == 0) {
				if (strcasecmp(v->value, "en")==0)
					dgv_conf.consolelanguage = 0;
				else
					dgv_conf.consolelanguage = 1;

				ast_log(LOG_NOTICE, "Dgv Channel: Console Language - %s \n", consolelanguage_desc[dgv_conf.consolelanguage]);				
			}

			v = v->next; /* next item */
		} /* end of while(v)*/

		//if no codec selected, set a default
		if (!dgv_conf.allow_slinear && !dgv_conf.allow_ulaw && !dgv_conf.allow_alaw && !dgv_conf.allow_gsm)
		{
			dgv_conf.allow_alaw = 1;
			ast_log(LOG_NOTICE, "Dgv Channel: no codec selected, setting A_LAW as default codec\n");
		}

		//register CLI commands	
		switch (dgv_conf.consolelanguage)
		{
			case 0://English		
				ast_cli_register_multiple(dgv_cli_en, sizeof(dgv_cli_en) / sizeof(dgv_cli_en[0]));
				break;
			case 1://Portuguese		
				ast_cli_register_multiple(dgv_cli_br, sizeof(dgv_cli_br) / sizeof(dgv_cli_br[0]));
				break;
			default:		
				ast_cli_register_multiple(dgv_cli_en, sizeof(dgv_cli_en) / sizeof(dgv_cli_en[0]));
				//0.9.8 break;
		}

		/* types to use with digivoice cards */
		v = ast_variable_browse(cfg, "allportsconfig");
		while (v)
		{
			if (strcasecmp(v->name, "default_callprogress") == 0) {
				ast_copy_string(dgv_conf.szDefaultCallProgressFile, v->value, sizeof(dgv_conf.szDefaultCallProgressFile));
			}
			else if (strcasecmp(v->name, "detectiontype") == 0) {
				dgv_conf.default_detectiontype = atoi(v->value);
			}
			else if (strcasecmp(v->name, "dialtype") == 0) {
				dgv_conf.default_dialtype = atoi(v->value);
			}
			else if (strcasecmp(v->name, "afterdialpause") == 0) {
				dgv_conf.afterdialpause = atoi(v->value);
			}
			else if (strcasecmp(v->name, "ringbacktone1") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szRingbacktone1Name, v->value, sizeof(dgv_conf.szRingbacktone1Name));
				dgv_conf.ringbacktone1 = tone_lookup((char *)v->value);
			}
			else if (strcasecmp(v->name, "ringbacktone2") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szRingbacktone2Name, v->value, sizeof(dgv_conf.szRingbacktone2Name));
				dgv_conf.ringbacktone2 = tone_lookup((char *)v->value);				
			}
			else if (strcasecmp(v->name, "busytone1") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szBusytone1Name, v->value, sizeof(dgv_conf.szBusytone1Name));
				dgv_conf.busytone1 = tone_lookup((char *)v->value);
			}
			else if (strcasecmp(v->name, "busytone2") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szBusytone2Name, v->value, sizeof(dgv_conf.szBusytone2Name));
				dgv_conf.busytone2 = tone_lookup((char *)v->value);
			}
			else if (strcasecmp(v->name, "subchannelringtone1") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szSubchannelringtone1Name, v->value, sizeof(dgv_conf.szSubchannelringtone1Name));
				dgv_conf.subchannelringtone1 = tone_lookup((char *)v->value);
			}
			else if (strcasecmp(v->name, "subchannelringtone2") == 0) {
				//get configuration name
				ast_copy_string(dgv_conf.szSubchannelringtone2Name, v->value, sizeof(dgv_conf.szSubchannelringtone2Name));
				dgv_conf.subchannelringtone2 = tone_lookup((char *)v->value);
			}
			else if (strcasecmp(v->name, "ringbacktimes") == 0) {
				//get configuration name
				ast_copy_string(szTemp, v->value, sizeof(szTemp));
				sscanf(szTemp,"%u,%u,%u,%u",&dgv_conf.ringback_times[0],
																		&dgv_conf.ringback_times[1],
																		&dgv_conf.ringback_times[2],
																		&dgv_conf.ringback_times[3]);

			}
			else if (strcasecmp(v->name, "busytimes") == 0) {
				//get configuration name
				ast_copy_string(szTemp, v->value, sizeof(szTemp));
				sscanf(szTemp,"%d,%d,%d,%d",&dgv_conf.busy_times[0],
																		&dgv_conf.busy_times[1],
																		&dgv_conf.busy_times[2],
																		&dgv_conf.busy_times[3]);

			}
			else if (strcasecmp(v->name, "subchannelringtimes") == 0) {
				//get configuration name
				ast_copy_string(szTemp, v->value, sizeof(szTemp));
				sscanf(szTemp,"%d,%d,%d,%d",&dgv_conf.subchannelring_times[0],
					   &dgv_conf.subchannelring_times[1],
					   &dgv_conf.subchannelring_times[2],
					   &dgv_conf.subchannelring_times[3]);

			}
			else if (strcasecmp(v->name, "deny_prefix") == 0) {
				if (!strcasecmp(v->value, "no"))
					dgv_conf.deny_prefix[0] = '\0';
				else {
					int x, isnum;
					char clearstr[AST_MAX_EXTENSION-1];
					ast_copy_string(clearstr, v->value, sizeof(clearstr));
					isnum = 1;
					if (strlen(clearstr) == 0)
						isnum = 0;
					else
						for (x = 0; x < strlen(clearstr); x++) {
							if (!isdigit(clearstr[x])) {
								isnum = 0;
								break;
							}
						}
					if (isnum)
						ast_copy_string(dgv_conf.deny_prefix, v->value, sizeof(dgv_conf.deny_prefix));
					else
						dgv_conf.deny_prefix[0] = '\0';
				}
				ast_log(LOG_NOTICE, "Dgv Channel: Prefix Denied %s \n", (strlen(dgv_conf.deny_prefix) > 0) ? dgv_conf.deny_prefix : "<None>");

				//set default deny_prefix before specific port configuration
				ast_copy_string(deny_prefix, dgv_conf.deny_prefix, sizeof(deny_prefix));
            }
			else if (strcasecmp(v->name, "gsm_signalupdateinterval") == 0) {
				dgv_conf.gsm_signal_update_interval = atoi(v->value);
				if (dgv_conf.gsm_signal_update_interval < 2000 || dgv_conf.gsm_signal_update_interval > 600000)//2s or 10m
					dgv_conf.gsm_signal_update_interval = 5000;
			}
			v = v->next; /* next item */
		} /* end of while(v)*/


		if (option_verbose >= 3)
		{
			ast_log(LOG_NOTICE,"Configurations for Dgv Channel:\n");
			ast_log(LOG_NOTICE,"default detectiontype = %d\n",dgv_conf.default_detectiontype);
			/*ast_log(LOG_NOTICE,"ringbacktone1 = %s (%d)\n",dgv_conf.szRingbacktone1Name, dgv_conf.ringbacktone1);
			ast_log(LOG_NOTICE,"ringbacktone2 = %s (%d)\n",dgv_conf.szRingbacktone2Name, dgv_conf.ringbacktone2);
			ast_log(LOG_NOTICE,"busytone1 = %s (%d)\n",dgv_conf.szBusytone1Name, dgv_conf.busytone1);
			ast_log(LOG_NOTICE,"busytone2 = %s (%d)\n",dgv_conf.szBusytone2Name, dgv_conf.busytone2);
	
			for (ch=0;ch<3;ch++)
				ast_log(LOG_NOTICE,"ringback_times[%d] = %d\n",ch, dgv_conf.ringback_times[ch]);
			for (ch=0;ch<3;ch++)
				ast_log(LOG_NOTICE,"busy_times[%d] = %d\n",ch, dgv_conf.ringback_times[ch]);
	*/
	
			ast_log(LOG_NOTICE,"default_call_progress = %s\n",dgv_conf.szDefaultCallProgressFile);
	
		}
	
		//set audio codecs capabilities
#ifdef CC_AST_HAS_VERSION_13
			if (dgv_conf.allow_slinear)
				ast_format_cap_append(dgv_tech.capabilities, ast_format_slin, 0);
			if (dgv_conf.allow_ulaw)
				ast_format_cap_append(dgv_tech.capabilities, ast_format_ulaw, 0);
			if (dgv_conf.allow_alaw)
				ast_format_cap_append(dgv_tech.capabilities, ast_format_alaw, 0);
			if (dgv_conf.allow_gsm)
				ast_format_cap_append(dgv_tech.capabilities, ast_format_gsm, 0);
			supported_fmts = (dgv_conf.allow_slinear ? (AST_FORMAT_SLINEAR) : 0) |
							 (dgv_conf.allow_ulaw ? (AST_FORMAT_ULAW) : 0) |
							 (dgv_conf.allow_alaw ? (AST_FORMAT_ALAW) : 0) |
							 (dgv_conf.allow_gsm ? (AST_FORMAT_GSM) : 0) ;
#else//#ifdef CC_AST_HAS_VERSION_13
	#ifdef CC_AST_HAS_VERSION_10
				struct ast_format dgv_tech_fmt;

				if (dgv_conf.allow_slinear)
					ast_format_cap_add(dgv_tech.capabilities, ast_format_set(&dgv_tech_fmt, AST_FORMAT_SLINEAR, 0));
				if (dgv_conf.allow_ulaw)
					ast_format_cap_add(dgv_tech.capabilities, ast_format_set(&dgv_tech_fmt, AST_FORMAT_ULAW, 0));
				if (dgv_conf.allow_alaw)
					ast_format_cap_add(dgv_tech.capabilities, ast_format_set(&dgv_tech_fmt, AST_FORMAT_ALAW, 0));
				if (dgv_conf.allow_gsm)
					ast_format_cap_add(dgv_tech.capabilities, ast_format_set(&dgv_tech_fmt, AST_FORMAT_GSM, 0));
				supported_fmts = (dgv_conf.allow_slinear ? ast_format_id_to_old_bitfield(AST_FORMAT_SLINEAR) : 0) |
					         	 (dgv_conf.allow_ulaw ? ast_format_id_to_old_bitfield(AST_FORMAT_ULAW) : 0) |
					         	 (dgv_conf.allow_alaw ? ast_format_id_to_old_bitfield(AST_FORMAT_ALAW) : 0) |
					         	 (dgv_conf.allow_gsm ? ast_format_id_to_old_bitfield(AST_FORMAT_GSM) : 0) ;
	#else//#ifdef CC_AST_HAS_VERSION_10
				dgv_tech.capabilities = (dgv_conf.allow_slinear ? AST_FORMAT_SLINEAR : 0) |
					                	(dgv_conf.allow_ulaw ? AST_FORMAT_ULAW : 0) |
					                	(dgv_conf.allow_alaw ? AST_FORMAT_ALAW : 0) |
					                	(dgv_conf.allow_gsm ? AST_FORMAT_GSM : 0) ;
				supported_fmts = dgv_tech.capabilities;
	#endif//#ifdef CC_AST_HAS_VERSION_10
#endif//#ifdef CC_AST_HAS_VERSION_13
		ast_log(LOG_NOTICE,"audio capabilities = %x\n",supported_fmts);

	} ast_dgv_unlock(&iflock, 0); /* config file parse end-block */	

	/* try to start voicerlib engine */
	if (reload==0) 
	{
		if ((ret = dg_StartVoicerlib(NULL)) != EXIT_SUCCESS)
		{
				ast_log(LOG_ERROR, "\nError starting voicerlib (code 0x%x)\n",ret);
				switch(ret)
				{
					case DG_ERROR_MEMORY_ALLOCATION:

						ast_log(LOG_NOTICE, "Cause: Memory allocation failed (low memory or /dev/vlibd* could not be created)\n");
						break;
					case DG_ERROR_FIRMWARE_NOT_FOUND:
						ast_log(LOG_NOTICE, "Cause: Firmware files not found at /var/lib/voicerlib/firmware!\n");
						break;
				}
				__unload_module(0);
				return -1;
		}
		else
			ast_log(LOG_NOTICE, "Number of ports detected = %d\n", dg_GetPortsCount());

        digivoice_sleep(500);

#ifdef CC_AST_HAS_VERSION_10
			sched = ast_sched_context_create();
#else//#ifdef CC_AST_HAS_VERSION_10
			sched = sched_context_create();
#endif//#ifdef CC_AST_HAS_VERSION_10

		if (!sched) {
			ast_log(LOG_ERROR, "Unable to create schedule context.\n");
			return -1;
		}
		else
		{
			ast_cond_init(&timer_pending_cond, NULL);
			
            pthread_attr_init(&attr);	
            pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);                
			if (pthread_create(&timer_thread, &attr, do_timer, NULL) < 0) {
				ast_log(LOG_ERROR, "Unable to start timer thread.\n");
			}
            pthread_attr_destroy(&attr);	
		}

	}

    /* Set default spans to CAS */
    for(span_index=0;span_index<NUM_SPANS;span_index++) {
            span_type[span_index] = DG_SPAN_CAS;
    }

	//E1 configurations
	ast_dgv_lock(&iflock, 0); {
		v = ast_variable_browse(cfg, "e1_config");
		while (v)
		{
			if (strcasecmp(v->name, "e1_sync") == 0) 
			{
				//get configuration name
				sync = atoi(v->value);
				//check values
				if (sync<1 || sync > 3)	//out of range sets internal as default
				{
						ast_log(LOG_WARNING,"e1_sync out of range.Using SYNC_INTERNAL as default\n");
						sync = SYNC_INTERNAL;
				}
			}
			else
			if (strcasecmp(v->name, "e1_sync_alternative") == 0)
			{
				//get configuration name
				sync_alternative = atoi(v->value);
				//check values
				if (sync_alternative < 1 || sync_alternative > 3)	//out of range sets no sync as default
				{
					ast_log(LOG_WARNING, "e1_sync_alternative out of range. Using NO SYNC as default\n");
					sync_alternative = 0;
				}
			}
			else
            if (strcasecmp(v->name, "use_crc4") == 0) {

                use_crc4 = atoi(v->value);

                if (use_crc4<0 || use_crc4>3) {
                    use_crc4 = 0;      //wrong value means that don't use it
                }
            }
            else
                if (strcasecmp(v->name, "span") == 0) {

                    //span option in cas has only
                    num_args = ast_app_separate_args((char *)v->value, ',' , args, 10);
                    //ast_log(LOG_WARNING,"Num Args = %d\n", num_args);

                    //sscanf(v->value,"%d,%s,%s,%s",&span_index,span_str,span_pritype_str,span_switchtype_str);
                    if (num_args>0) {

                        span_index = atoi(args[DG_SPAN_NUMBER]);
                        if (span_index && span_index<NUM_SPANS) {
                            if (!args[DG_FRAME_TYPE])
                                span_type[span_index-1] = DG_SPAN_CAS;
                            else
                              if (strcasecmp(args[DG_FRAME_TYPE], "ccs")==0) {
                                span_type[span_index-1] = DG_SPAN_CCS;
#ifdef USE_LIBPRI
	                                if (args[DG_SIDE]) {  //net/cpe
    	                                if (strcasecmp(args[DG_SIDE], "net")==0) {
        	                               span_pritype[span_index-1] = PRI_NETWORK;
            	                        }
                	                    else if (strcasecmp(args[DG_SIDE], "cpe")==0) {
                    	                   span_pritype[span_index-1] = PRI_CPE;                                       
                        	            }
                            	    }
                                	else
                                    	span_pritype[span_index-1] = PRI_CPE;

	                                	//switchtype
    	                            if (args[DG_SWITCHTYPE]) {                                    
        	                           	if (!strcasecmp(args[DG_SWITCHTYPE], "national"))
            	        					span_switchtype[span_index-1] = PRI_SWITCH_NI2;
                    					else if (!strcasecmp(args[DG_SWITCHTYPE], "ni1"))
                    						span_switchtype[span_index-1] = PRI_SWITCH_NI1;
                    					else if (!strcasecmp(args[DG_SWITCHTYPE], "dms100"))
	                    					span_switchtype[span_index-1] = PRI_SWITCH_DMS100;
    	                				else if (!strcasecmp(args[DG_SWITCHTYPE], "4ess"))
        	            					span_switchtype[span_index-1] = PRI_SWITCH_ATT4ESS;
            	        				else if (!strcasecmp(args[DG_SWITCHTYPE], "5ess"))
                	    					span_switchtype[span_index-1] = PRI_SWITCH_LUCENT5E;
                    					else if (!strcasecmp(args[DG_SWITCHTYPE], "euroisdn"))
                    						span_switchtype[span_index-1] = PRI_SWITCH_EUROISDN_E1;
                    					else if (!strcasecmp(args[DG_SWITCHTYPE], "qsig"))
                    						span_switchtype[span_index-1] = PRI_SWITCH_QSIG;
                    					else {
                                        	ast_log(LOG_WARNING, "Unknown switchtype '%s' - assuming euroisdn\n", args[3]);
	                                        span_switchtype[span_index-1] = PRI_SWITCH_EUROISDN_E1;
    	                                }
        	                        }
            	                    else
            	                    {
                                    	ast_log(LOG_WARNING, "Not defined switchtype '%s' - assuming euroisdn\n", args[3]);            	                    	
                	                    span_switchtype[span_index-1] = PRI_SWITCH_EUROISDN_E1;
            	                    }

                    	            if (args[DG_OVERLAPDIAL]) {  //overlapdial
                        	            if (!strcasecmp(args[DG_OVERLAPDIAL], "overlapdial")) 
                            	            span_overlapdial[span_index-1] = 1;
                                	    else
                                    	    span_overlapdial[span_index-1] = 0;
                                	}
                                	else
	                                    span_overlapdial[span_index-1] = 0;

	                                if (args[DG_PRIDIALPLAN]) {  //pridialplan
    	                				if (!strcasecmp(args[DG_PRIDIALPLAN], "national")) {
        	            					span_pridialplan[span_index-1] = PRI_NATIONAL_ISDN + 1;
            	        				} else if (!strcasecmp(args[DG_PRIDIALPLAN], "unknown")) {
                	    					span_pridialplan[span_index-1] = PRI_UNKNOWN + 1;
                    					} else if (!strcasecmp(args[DG_PRIDIALPLAN], "private")) {
                    						span_pridialplan[span_index-1] = PRI_PRIVATE + 1;
                    					} else if (!strcasecmp(args[DG_PRIDIALPLAN], "international")) {
                    						span_pridialplan[span_index-1] = PRI_INTERNATIONAL_ISDN + 1;
                    					} else if (!strcasecmp(args[DG_PRIDIALPLAN], "local")) {
                    						span_pridialplan[span_index-1] = PRI_LOCAL_ISDN + 1;
	                    	 			} else if (!strcasecmp(args[DG_PRIDIALPLAN], "dynamic")) {
    	                 					span_pridialplan[span_index-1] = -1;
        	            				} 
            	                        else
                	                        span_pridialplan[span_index-1] = PRI_NATIONAL_ISDN + 1;
                    	            }
	
    	                            if (args[DG_PRILOCALDIALPLAN]) {  //prilocaldialplan
        	                            if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "national")) {
            	                            span_prilocaldialplan[span_index-1] = PRI_NATIONAL_ISDN + 1;
                	                    } else if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "unknown")) {
                    	                    span_prilocaldialplan[span_index-1] = PRI_UNKNOWN + 1;
                        	            } else if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "private")) {
                            	            span_prilocaldialplan[span_index-1] = PRI_PRIVATE + 1;
                                	    } else if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "international")) {
                                    	    span_prilocaldialplan[span_index-1] = PRI_INTERNATIONAL_ISDN + 1;
	                                    } else if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "local")) {
    	                                    span_prilocaldialplan[span_index-1] = PRI_LOCAL_ISDN + 1;
        	                            } else if (!strcasecmp(args[DG_PRILOCALDIALPLAN], "dynamic")) {
            	                            span_prilocaldialplan[span_index-1] = -1;
                	                    } 
                    	                else
                        	                span_prilocaldialplan[span_index-1] = PRI_NATIONAL_ISDN + 1;
                            	    }

                    	            if (args[DG_FACILITYENABLE]) {  //facilityenable
                        	            if (!strcasecmp(args[DG_FACILITYENABLE], "facilityenable"))
                            	            span_facilityenable[span_index-1] = 1;
                                	    else
                                    	    span_facilityenable[span_index-1] = 0;
                                	}
                                	else
	                                    span_facilityenable[span_index-1] = 0;

                    	            if (args[DG_DISCARDREMOTEHOLDRETRIEVAL]) {
                        	            if (!strcasecmp(args[DG_DISCARDREMOTEHOLDRETRIEVAL], "discardremoteholdretrieval"))
                            	            span_discardremoteholdretrieval[span_index-1] = 1;
                                	    else
                                    	    span_discardremoteholdretrieval[span_index-1] = 0;
                                	}
                                	else
	                                    span_discardremoteholdretrieval[span_index-1] = 0;
#endif//#ifdef USE_LIBPRI
                            }
                            else
                                span_type[span_index-1] = DG_SPAN_CAS;
    
                        }
                    }//num_args>0
                    else
                    {
                        //wrong arguments
                        //assume cas span to span[0]
                        span_type[0] = DG_SPAN_CAS;
                    }
                }
            else
			if (strcasecmp(v->name, "cards") == 0) 
			{
				//set card confifurations - need to restart
                
                if (reload<2) 
                {
                    if (set_card_config((char *)v->value, sync, sync_alternative, use_crc4)==0)//with no cast(char *), warning: passing argument 1 of ‘set_card_config’ discards qualifiers from pointer target type(only 1_6 * version)
                    {
                        ast_log(LOG_ERROR, "\n************************************************\nFATAL ERROR - Please check [e1_config]\nsection at /etc/asterisk/digivoice.conf and\nif dgvfifo device driver is loaded.\n************************************************\n");
                        
    					//sets alarmmode for each card
    					for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
    					{
    						if (dg_GetCardInterface(nForCard) == DG_DIGITAL_INTERFACE)
    							dg_SetAlarmMode(nForCard, ALARM_MANUAL_NOTIFY);
    					}                       
                        
                        dg_ShutdownVoicerlib();
                        return -1;

                    }

                }
			}
			v = v->next; /* next item */
		} /* end of while(v)*/
	} ast_dgv_unlock(&iflock, 0); /* config file parse end-block */	

#ifdef USE_LIBPRI
    	//starts socket thread if there are spans in ccs mode
    	if (dchans_count>0 && reload<2) {
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				if (ast_pthread_create_background(&thd_sock, NULL, dgfifo_thread, NULL))
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				if (ast_pthread_create(&thd_sock, NULL, dgfifo_thread, NULL))
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			{
	            ast_log(LOG_ERROR,"Error starting ccs fifo control thread\n");
			}

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				if (ast_pthread_create_background(&thd_pri, NULL, dchan, NULL))
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				if (ast_pthread_create(&thd_pri, NULL, dchan, NULL))
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			{
					ast_log(LOG_ERROR,"ERROR: Cannot start dchannel control thread!!!\n");
			}
    	}
#endif//#ifdef USE_LIBPRI
	ast_log(LOG_NOTICE, "Loading ports........................ \n");
	//create private structure for each port
	if (reload==0) 
	{
		for (ch=1 ; ch<=dg_GetPortsCount() ; ch++) 
		{
			tmp = initialize_ports(ch);
			if (tmp)
			{
				if (ch==1) iffirst = tmp;	//save bottom list address

				/* port allocated */
				if (iflast!=NULL) iflast->next = tmp;
				tmp->previous = iflast;
				iflast = tmp;
			}
			else 
			{
				ast_log(LOG_ERROR, "Unable to register channel '%d'\n", ch);
			}
		}

		ast_log(LOG_NOTICE, "Loading Serial Numbers............... \n");
		tmpSerialNumber[0] = '\0';
		for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
		{
			dg_ReadCode(nForCard, (short *)tmpSerialNumber);
			ast_log(LOG_NOTICE, "Card %d - Real Serial Number: %s\n", nForCard, tmpSerialNumber);
			for (ch = 1; ch <= dg_GetCardPortsCount(nForCard); ch++)
			{
				tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard, ch));
				if (tmp)
				{
					if (!ast_strlen_zero(tmpSerialNumber))
						tmp->serial_num = atoi(tmpSerialNumber);
					else
						tmp->serial_num = 0;
				}
			}
		}

		ast_log(LOG_NOTICE, "Loading IRQ Numbers............... \n");
		tmpIRQNumber[0] = '\0';
		for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
		{
			dg_ReadIRQNumber(nForCard, tmpIRQNumber);
			ast_log(LOG_NOTICE, "Card %d - Real IRQ Number: %s\n", nForCard, tmpIRQNumber);
			for (ch = 1; ch <= dg_GetCardPortsCount(nForCard); ch++)
			{
				tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard, ch));
				if (tmp)
				{
					if (!ast_strlen_zero(tmpIRQNumber))
						tmp->irq_num = atoi(tmpIRQNumber);
					else
						tmp->irq_num = 0;
				}
			}
		}

		for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
			if (dg_GetCardInterface(nForCard) == DG_GSM_INTERFACE)
				for (ch = 1; ch <= dg_GetCardPortsCount(nForCard); ch++)
				{
					tmp = find_ch_port(dg_GetAbsolutePortNumber(nForCard, ch));
					if (tmp)
					{
						for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
						{
							tmp->smschannel[nForSMSChannel] = initialize_ports(tmp->port);
							if (tmp->smschannel[nForSMSChannel])
							{
								tmp->smschannel[nForSMSChannel]->smschannel[nForSMSChannel] = tmp;
								tmp->smschannel[nForSMSChannel]->channel_index              = 2;								
								tmp->smschannel[nForSMSChannel]->call_state                 = DG_AST_NONE;
								tmp->smschannel[nForSMSChannel]->smschannel_index           = nForSMSChannel; 

								if (nForSMSChannel==0) tmp->iffirstsmschannel = tmp->smschannel[nForSMSChannel];

								/* port allocated */
								if (tmp->iflastsmschannel!=NULL) tmp->iflastsmschannel->next = tmp->smschannel[nForSMSChannel];
								tmp->smschannel[nForSMSChannel]->previous = tmp->iflastsmschannel;
								tmp->iflastsmschannel = tmp->smschannel[nForSMSChannel];
							}
							else
								ast_log(LOG_ERROR, "Unable to register a smschannel on '%d'\n", tmp->port);
						} //for (nForSMSChannel = 0; nForSMSChannel <= NUM_SMSCHANNEL - 1; nForSMSChannel++)
					}
					else //if (tmp)
						ast_log(LOG_ERROR, "Unable to register smschannel '%d' of card '%d'\n", ch, nForCard);
				} //for (ch = 1; ch <= dg_GetCardPortsCount(nForCard); ch++)
	} //if (reload==0)
	
	ast_log(LOG_NOTICE, "Waiting synchronization....................\n");
	//digivoice_sleep(3000);

    //sets alarmmode for each card
    ast_log(LOG_NOTICE, "SetAlarmMode to ALARM_AUTOMATIC_NOTIFY \n");
    for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)
    {
		//date and time
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			struct ast_tm tm;
			struct timeval now = ast_tvnow();
			ast_localtime(&now, &tm, NULL);
		    for (nForSpan = 0; nForSpan <= 3; nForSpan++)
		    	ast_strftime(LastAlarmReset[nForCard][nForSpan], sizeof(LastAlarmReset[nForCard][nForSpan]), "%F %T", &tm);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
			time_t t;
			struct tm tm;
			time(&t);
			ast_localtime(&t, &tm, NULL);//old localtime_r
		    for (nForSpan = 0; nForSpan <= 3; nForSpan++)
		    	strftime(LastAlarmReset[nForCard][nForSpan], sizeof(LastAlarmReset[nForCard][nForSpan]), "%F %T", &tm);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
    	
    	switch (dg_GetCardInterface(nForCard + 1))
    	{
    		case DG_DIGITAL_INTERFACE:
    			dg_SetAlarmMode(nForCard + 1, ALARM_AUTOMATIC_NOTIFY);
     
		        for (nForSpan = 0; nForSpan <= 1; nForSpan++)
		        {
		        	//reset e1alarm status      
					dgv_e1alarms[nForCard][nForSpan].nALARM_LOSS     = 0;   
					dgv_e1alarms[nForCard][nForSpan].nALARM_AISS     = 0;
					dgv_e1alarms[nForCard][nForSpan].nALARM_AIS16S   = 0;
					dgv_e1alarms[nForCard][nForSpan].nALARM_SYNC     = 0;
					dgv_e1alarms[nForCard][nForSpan].nALARM_MFSYNC   = 0;
					dgv_e1alarms[nForCard][nForSpan].nALARM_RAIS     = 0;  
		  			dgv_e1alarms[nForCard][nForSpan].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
						dgv_e1alarms[nForCard][nForSpan].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI
					dgv_e1alarms[nForCard][nForSpan].nALARM_RSLIP    = 0;			
					
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_LOSS     = 0;   
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_AISS     = 0;
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_AIS16S   = 0;
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_SYNC     = 0;
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_MFSYNC   = 0;
					dgv_e1alarmsCount[nForCard][nForSpan].nALARM_RAIS     = 0;  
		  			dgv_e1alarmsCount[nForCard][nForSpan].nALARM_CRC4SYNC = 0;
#ifdef USE_LIBPRI
						dgv_e1alarmsCount[nForCard][nForSpan].nALARM_LINKDOWN = 0;
#endif//#ifdef USE_LIBPRI
		        } //for (nForSpan = 0; nForSpan <= 1; nForSpan++)	        
		        
    		case DG_FXO_INTERFACE:
    		case DG_FX_INTERFACE:		        
		        for (ch = 0; ch <= 59 ; ch++) 
		        	dgv_fxoNDTCount[nForCard][ch] = 0;
		        
		        break;
    		case DG_GSM_INTERFACE:
		        for (nForSpan = 0; nForSpan <= 3; nForSpan++)
		        {
					//reset gsmsignal status
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;
					dgv_gsmsignal[nForCard][nForSpan][0] = 0;

					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;
					dgv_gsmsignal[nForCard][nForSpan][1] = 99;

					dgv_gsmDECount[nForCard][nForSpan].nTotalDialError = 0;
					dgv_gsmDECount[nForCard][nForSpan].nTempDialError = 0;
					dgv_gsmDECount[nForCard][nForSpan].nFlagDialError = 0;
		        } //for (nForSpan = 0; nForSpan <= 1; nForSpan++)
		        
	  			break;
    	} //switch (dg_GetCardInterface(nForCard + 1)
    } //for (nForCard = 0; nForCard <= dg_GetCardsCount() - 1; nForCard++)
  
    //Enable card framers
    //e1 card initializations
    
    //enable first the E1-a off all cards

    
/*    for (card = 1; card <= dg_GetCardsCount(); card++)
    {
        if ((dg_GetCardInterface(card) == DG_DIGITAL_INTERFACE)) 
        {
            ast_log(LOG_NOTICE, "Enabling Framer A of card %d...............\n", card);
            dg_EnableFramer(card, CFG_FRAMER_E1_A);
        }
    }
    
    digivoice_sleep(5000);*/
    //enable first the E1-a off all cards
    
/*    for (card = 1; card <= dg_GetCardsCount(); card++)
    {
        if ((dg_GetCardInterface(card) == DG_DIGITAL_INTERFACE)) 
        {
            ast_log(LOG_NOTICE, "Enabling Framer B of card %d...............\n", card);
            dg_EnableFramer(card, CFG_FRAMER_E1_B);
        }
    }*/
    digivoice_sleep(500);

	//--------------------------------------------------
	//check individual port configuration
	//--------------------------------------------------
	ast_dgv_lock(&iflock, 0); {
		v = ast_variable_browse(cfg, "port_config");
		while (v)
		{
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				ast_jb_read_conf(&jbconf, v->name, v->value);
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

			if (strcasecmp(v->name, "port_id") == 0) {
				ast_copy_string(szPort_ID, v->value, sizeof(szPort_ID));
			}
			if (strcasecmp(v->name, "port_name") == 0) {
				ast_copy_string(szPort_Name, v->value, sizeof(szPort_Name));
			}
			else
			if (strcasecmp(v->name, "category") == 0) {
				category = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "category_pass_through") == 0) {
				category_pass_through = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "max_digits_rx") == 0) {
				max_rx = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "min_digits_rx") == 0) {
				min_rx = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "interdigit_timeout") == 0) {
				interdigit_timeout = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "tx_gain") == 0) {
				txgain = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "rx_gain") == 0) {
				rxgain = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "cid_pos") == 0) {
				cid_pos = atoi(v->value);
			}
            else
			if (strcasecmp(v->name, "echocan") == 0) {
				echo_can = atoi(v->value);
			}
            else
			if (strcasecmp(v->name, "echotype") == 0) {
				if (!strcasecmp(v->value, "dsp")) {
					echo_type = ECHO_HW_DSP;
				}
				else if (!strcasecmp(v->value, "swlec_vlibd")) {
					echo_type = ECHO_SW_LEC;
				}
				else if (!strcasecmp(v->value, "swlec")) {
					echo_type = ECHO_SW_LEC_U;
				}
				else
					echo_type = ECHO_HW_DSP;
			}
			else
			if (strcasecmp(v->name, "echotraining") == 0) {
				echo_training = atoi(v->value);
			}
			else
            if (strcasecmp(v->name, "echonlp") == 0) {
                echo_nlp = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "drop_tx_samples_factor") == 0) {
                drop_tx_samples_factor = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "relaxdtmf") == 0) {
                relaxdtmf = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "wait_dialtone") == 0) {
                wait_dialtone = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "dialtone_timeout") == 0) {
                dialtone_timeout = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "gen_ringback") == 0) {
                gen_ringback = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "gen_busy") == 0) {
                gen_busy = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "gen_dtmf") == 0) {
                gen_dtmf = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "dtmf_to_asterisk") == 0) {
                dtmf_to_asterisk = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "silence_threshold") == 0) {
                silence_threshold = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "group_b_value") == 0) {
                group_b_value = atoi(v->value);

                //wrong value sets B_FREE_CALLING
                if (group_b_value < MFC_B_1 || group_b_value >  MFC_B_8) 
                    group_b_value = MFC_B_1;/* MFC_B_1 = default Brazil, old = B_FREE_CALLING */
            }
            else
            if (strcasecmp(v->name, "wait_group_b") == 0) {
            	wait_group_b = atoi(v->value);
            }
			else
            if (strcasecmp(v->name, "r2_country") == 0) {
            	if (!strcasecmp(v->value, "br"))
            		r2_country = R2_COUNTRY_BR;
            	else
            		if (!strcasecmp(v->value, "ar"))
            			r2_country = R2_COUNTRY_AR;
            		else 
            			if (!strcasecmp(v->value, "mx"))
            				r2_country = R2_COUNTRY_MX;
            			else
            				r2_country = R2_COUNTRY_BR;//default Brazil
            }
			else				
            if (strcasecmp(v->name, "b_subscriber_hangup") == 0) {
                b_subscriber_hangup = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "ccb_pass_through") == 0) {
            	ccb_pass_through = atoi(v->value);
            }
            else
			if (strcasecmp(v->name, "auto_dgccb") == 0) {
				auto_dgccb = atoi(v->value);
			}
			else
            if (strcasecmp(v->name, "deny_prefix") == 0) {
				if (!strcasecmp(v->value, "no"))
					deny_prefix[0] = '\0';
				else {
					int x, isnum;
					char clearstr[AST_MAX_EXTENSION-1];
					ast_copy_string(clearstr, v->value, sizeof(clearstr));
					isnum = 1;
					if (strlen(clearstr) == 0)
						isnum = 0;
					else
						for (x = 0; x < strlen(clearstr); x++) {
							if (!isdigit(clearstr[x])) {
								isnum = 0;
								break;
							}
						}
					if (isnum)
						ast_copy_string(deny_prefix, v->value, sizeof(deny_prefix));
				}
            }
            else
            if (strcasecmp(v->name, "vconf_pass_through") == 0) {
            	vconf_pass_through = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "callprogress") == 0) {
                 if (!ast_strlen_zero(v->value))
                     ast_copy_string(callprogress, v->value, sizeof(callprogress));
                 else
                	 ast_copy_string(callprogress, dgv_conf.szDefaultCallProgressFile, sizeof(callprogress));
            }
            else
            if (strcasecmp(v->name, "language") == 0) {
                if (!ast_strlen_zero(v->value))
                	ast_copy_string(language, v->value, sizeof(language));
                else
                	ast_copy_string(language, global_language, sizeof(language));
            }
            else
			if (strcasecmp(v->name, "accountcode") == 0) {
				if (!ast_strlen_zero(v->value))
					ast_copy_string(szAccountCode, v->value, sizeof(szAccountCode));
				else
					szAccountCode[0]='\0';
			}
            else
            if (!strcasecmp(v->name, "musiconhold")) 
            {
                ast_copy_string(musicclass, v->value, sizeof(musicclass));
            }
			else
			if (!strcasecmp(v->name, "callgroup")) 
            {
				ast_copy_string(cur_callgroup, v->value, sizeof(cur_callgroup));                                
            } 
            else				
            if (!strcasecmp(v->name, "pickupgroup")) 
            {
            	ast_copy_string(cur_pickupgroup, v->value, sizeof(cur_pickupgroup));
            } 
            else 
			if (strcasecmp(v->name, "context") == 0) {
                   
				ast_copy_string(szContext, v->value, sizeof(szContext));

                if (szContext[0]=='\0')
					ast_copy_string(szContext, "default", sizeof(szContext));
			}
			else
			if (strcasecmp(v->name, "signalling") == 0) {
                
                if (!strcasecmp(v->value, "r2mfc_ndis_fixed")) {
                   signalling = DG_SIG_R2_NDIS_FIXED;
                }
                else if (!strcasecmp(v->value, "r2mfc_ndis_variable")) {
                   signalling = DG_SIG_R2_NDIS_VARIABLE;
                }
#ifdef USE_LIBPRI
                	else if (!strcasecmp(v->value, "isdn")) {
                   		signalling = DG_SIG_PRI;
                	}
#endif//#ifdef USE_LIBPRI
                else if (!strcasecmp(v->value, "customcas")) {
                   signalling = DG_SIG_CUSTOMCAS;
                }
                else if (!strcasecmp(v->value, "fxo")) {
                   signalling = DG_SIG_FXO;
                }
                else if (!strcasecmp(v->value, "fxs")) {
                   signalling = DG_SIG_FXS;              	
                }
                else if (!strcasecmp(v->value, "channelbank_fxs")) {
                   signalling = DG_SIG_CB_FXS;
                }
                else if (!strcasecmp(v->value, "channelbank_fxo")) {
                   signalling = DG_SIG_CB_FXO;              	
                }
                else if (!strcasecmp(v->value, "gsm")) {
                   signalling = DG_SIG_GSM;
                }
                else
                   signalling = DG_SIG_FXO; /* unkown signalling - set default FXO ?? */
			}
			else
#ifdef USE_LIBPRI
	            if (!strcasecmp(v->name, "internationalprefix")) 
    	        {
        	        ast_copy_string(internationalprefix, v->value, sizeof(internationalprefix));
            	}
            	else
            	if (!strcasecmp(v->name, "nationalprefix")) {
	                ast_copy_string(nationalprefix, v->value, sizeof(nationalprefix));
            	} 
            	else
            	if (!strcasecmp(v->name, "localprefix")) {
	                ast_copy_string(localprefix, v->value, sizeof(localprefix));
            	} 
            	else 
            	if (!strcasecmp(v->name, "privateprefix")) {
	                ast_copy_string(privateprefix, v->value, sizeof(privateprefix));
            	}
            	else
            	if (!strcasecmp(v->name, "unknownprefix")) {
	                ast_copy_string(unknownprefix, v->value, sizeof(unknownprefix));
				} 
				else
#endif//#ifdef USE_LIBPRI			
			if (!strcasecmp(v->name, "usecallerid")) {
				use_callerid = ast_true(v->value);
			}
			else			
            if (strcasecmp(v->name, "auto_r2dmfc") == 0) {
                ast_log(LOG_WARNING, 
                "\n+-------------------------------------------------------------+\n"
                "|   WARNING !!!                                               |\n"
                "|   You have an old version of digivoice.conf.                |\n"
                "|   'auto_r2dmfc' was replaced by 'signalling' configuration. |\n"
                "|   Please, refer to CHANGELOG to update your digivoice.conf. |\n"
                "+-------------------------------------------------------------+\n");

                /*__unload_module(0);
                return -1;*/
                if (atoi(v->value))
                    signalling = DG_SIG_R2_NDIS_FIXED; 
                else
                    signalling = DG_SIG_R2_NDIS_VARIABLE;                 
            }
            else
            if (!strcasecmp(v->name, "customcascfg")) 
            {           	
                ast_copy_string(customcascfg, v->value, sizeof(customcascfg));
            }
            else
            if (strcasecmp(v->name, "ring_type") == 0) 
            {
                ring_type = atoi(v->value);                            		
            }
            else
            if (strcasecmp(v->name, "flash_min_time") == 0) 
            {
                flash_min_time = atoi(v->value);            		
            }
            else
            if (strcasecmp(v->name, "flash_max_time") == 0) 
            {
                flash_max_time = atoi(v->value);            		
            }
            else
			if (!strcasecmp(v->name, "immediate")) 
			{
				immediate = ast_true(v->value);
			}
            else
            if (!strcasecmp(v->name, "mailbox")) 
            {
                ast_copy_string(mailbox, v->value, sizeof(mailbox));
            }
            else
            if (!strcasecmp(v->name, "faxdetection")) 
            {
                fax_detection = atoi(v->value);
            }
            else
            if (strcasecmp(v->name, "fax_pass_through") == 0) {
            	fax_pass_through = atoi(v->value);
            }
            else
			if (!strcasecmp(v->name, "polarity"))
			{
				polarity = atoi(v->value);
			}
			else
			if (!strcasecmp(v->name, "cid_type"))
			{
				cid_detection_type = atoi(v->value);
			}
			else
            if (!strcasecmp(v->name, "second_call_digits"))
            {
                ast_copy_string(second_call_digits, v->value, sizeof(second_call_digits));
                if (!ast_strlen_zero(second_call_digits))
                	second_call = 1;
                else
                	second_call = 0;
            }
            else			
            if (strcasecmp(v->name, "hide_number") == 0) 
            {
                gsm_hide_number = atoi(v->value);                            		
            }			
            else
			if (strcasecmp(v->name, "sms_confirmation") == 0)
			{
				gsm_sms_confirmation = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "sms_flashsms") == 0)
			{
				gsm_sms_flashsms = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "displaycallwaiting") == 0)
			{
				gsm_displaycallwaiting = atoi(v->value);
			}
			else
            if (strcasecmp(v->name, "pin_number") == 0) 
            {
            	ast_copy_string(gsm_pin_number, v->value, sizeof(gsm_pin_number));
            }			
            else
			if (strcasecmp(v->name, "ports") == 0) 
			{
				//set port - reload=0 new one
				//1 - already allocated
				set_port_config(reload,
						        (char *)v->value,
						        szPort_ID,
						        szPort_Name,
						        szContext,
						        category,
						        category_pass_through,
						        min_rx,
						        max_rx,
						        interdigit_timeout,
						        txgain,rxgain,
				 			    cid_pos,
				 			    echo_can,
				 			    szAccountCode,
				 			    signalling,
				 			    b_subscriber_hangup,
				 			    ccb_pass_through,
				 			    auto_dgccb,
				 			    deny_prefix,
				 			    vconf_pass_through,
				 			    group_b_value,
				 			    wait_group_b,
				 			    silence_threshold,
				 			    musicclass,
				 			    cur_callgroup,
				 			    cur_pickupgroup,
				 			    gen_dtmf,
				 			    gen_ringback,
				 			    callprogress,
				 			    language,
				 			    dtmf_to_asterisk,
				 			    dialtone_timeout,
				 			    wait_dialtone,
				 			    gen_busy,
				 			    drop_tx_samples_factor,
#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
									jbconf,
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
				 			    relaxdtmf,
				 			    echo_type,
				 			    echo_training,
				 			    echo_nlp,
                                ring_type,
                                flash_min_time,
                                flash_max_time,
                                immediate,
                                mailbox,
                                fax_detection,
                                fax_pass_through,
                                r2_country,
                                polarity,
                                cid_detection_type,
                                second_call,
                                second_call_digits,
                                gsm_hide_number,
                                gsm_sms_confirmation,
                                gsm_sms_flashsms,
                                gsm_displaycallwaiting,
                                gsm_pin_number);
			}
			v = v->next; /* next item */
		} /* end of while(v)*/
	} ast_dgv_unlock(&iflock, 0); /* config file parse end-block */

	//check group configuration
	ast_dgv_lock(&iflock, 0);
	{
		tmp = iffirst;
		while (tmp)
		{
			ast_dgv_lock(&tmp->lock, tmp->port);
			{
				tmp->group = -1;
				if ((tmp->verbosity & DG_LOG_FIND_PORT) != 0)
					ast_log(LOG_NOTICE, "resetting group=%d at port=%d\n", tmp->group, tmp->port);
			}
			ast_dgv_unlock(&tmp->lock, tmp->port);

			tmp = tmp->next;
		}

		v = ast_variable_browse(cfg, "groups");
		while (v)
		{
			if (strcasecmp(v->name, "group") == 0) {
				group = atoi(v->value);
			}
			else
			if (strcasecmp(v->name, "ports") == 0) {
				//find port
				set_port_group(group, (char *)v->value);
			}
			v = v->next; /* next item */
		} /* end of while(v)*/
	} ast_dgv_unlock(&iflock, 0); /* config file parse end-block */	

	if (reload==0) 
	{
		//set event callback handler function
		dg_SetEventCallback(event_handler,&event_context);
		//input buffer callback
		dg_SetAudioInputCallback(read_samples);
		//DEFAULT TYPE OF GSM files
		dg_SetGSMMode(GSM_RAW);
		
		memset(round_robin, 0, sizeof(round_robin));
		
		digivoice_sleep(500);

		for (ch=1; ch<=dg_GetPortsCount(); ch++)
		{
			dg_PauseInputBuffer(ch, DG_PAUSE);

			tmp = find_ch_port(ch);
			if (tmp)
				if ((tmp->verbosity & DG_LOG_PAUSEINPUTBUFFER) != 0)
					ast_log(LOG_WARNING, "setup_dgv: Pausing InputBuffer on port %d\n", tmp->port);
		}
	}

	ast_config_destroy(cfg);
	
	/* We *must* have a features.conf file to transfer a call */
#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		struct ast_flags config_flags_features = {reload ==  1 ? 0 : 0};
		cfg = ast_config_load(featurescfg, config_flags_features);
#else//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
		cfg = ast_config_load(featurescfg);
#endif//#if defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)

	if (!cfg)
	{
		ast_log(LOG_ERROR, "**********************************************************************\n");
        ast_log(LOG_ERROR, "FATAL ERROR: Unable to load /etc/asterisk/features.conf (File exists?)\n");
		ast_log(LOG_ERROR, "**********************************************************************\n");		
	}
	
	ast_dgv_lock(&iflock, 0); 
	{
		v = ast_variable_browse(cfg, "featuremap");
		while (v)
		{
            if (!strcasecmp(v->name, "atxfer"))         	
                ast_copy_string(features_conf.fxs_atxfer, v->value, sizeof(features_conf.fxs_atxfer));            

			v = v->next; /* next item */
		} /* end of while(v)*/
		
		if (features_conf.fxs_atxfer[0] == '\0')
			ast_log(LOG_WARNING, "Dgv Channel: FXS transfer command not found!!! \n");
		else              
			ast_log(LOG_NOTICE, "Dgv Channel: atxfer: %s \n", features_conf.fxs_atxfer);		
	} 
	ast_dgv_unlock(&iflock, 0); /* config file parse end-block */

	ast_config_destroy(cfg);	

	ast_dgv_lock(&iflock, 0);
	{
		if (!load_statistics())
		{
			ast_log(LOG_ERROR, "************************************************************\n");
			ast_log(LOG_ERROR, "FATAL ERROR: Unable to load Statistics Files (Files exists?)\n");
			ast_log(LOG_ERROR, "************************************************************\n");
		}
	}
	ast_dgv_unlock(&iflock, 0);

	channel_up = 1;
	
	for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
	{
		if (dg_GetCardInterface(nForCard) == DG_DIGITAL_INTERFACE)
		{
			ast_log(LOG_NOTICE, "Requesting alarm notifications... \n\n");
			dg_GetAlarmStatus(nForCard);
		}
	}

	for (nForCard = 1; nForCard <= dg_GetCardsCount(); nForCard++)
	{
		if (dg_GetCardType(nForCard) == DGV_2E1F || dg_GetCardType(nForCard) == DGV_1E1F)
		{
			ast_log(LOG_NOTICE, "Loading Firmware version for DGV-2E1F/DGV-1E1F card %d............... \n", nForCard);

			if (dg_GetVersion(nForCard) != EXIT_SUCCESS)
				ast_log(LOG_NOTICE, "Card %d - Error getting firmware version\n", nForCard);

			if (dg_GetLECInfo(nForCard) != EXIT_SUCCESS)
				ast_log(LOG_NOTICE, "Card %d - Error getting LEC count\n", nForCard);
		}
	}

	return 0;
}

//-------------------------------------------------------------------------------------------
// This functions is called by ast_register_atexit 
//-------------------------------------------------------------------------------------------
void dgv_cleanup(void)
{
	channel_up = 0;
	ast_log(LOG_NOTICE, "dgv_cleanup: Start VoicerLib Shutdown...\n");
	__unload_module(0);
}

//-------------------------------------------------------------------------------------------
//
//-------------------------------------------------------------------------------------------
int load_module()
{
	int nForMutex;

	ast_log(LOG_NOTICE, "load_module: starting for Dgv\n");
	ast_register_atexit(dgv_cleanup);
	ast_mutex_init(&iflock);
    ast_mutex_init(&timer_pending_lock);
#ifdef USE_LIBPRI
    for (nForMutex = 0; nForMutex <= (MAX_CARDS*NUM_SPANS_PER_CARD) - 1; nForMutex++)
    {
	    ast_mutex_init(&ccs_lock[nForMutex]);

#ifdef DEBUG_PRI_STEP
	    	ccs_lock_count[nForMutex] = 0;
#endif//#ifdef DEBUG_PRI_STEP
    }
#endif//#ifdef USE_LIBPRI
	return setup_dgv(0);
}

int reload(void)
{
	channel_up = 0;
	ast_log(LOG_NOTICE, "Starting reload procedure for Dgv...\n");
	usleep(300*1000);
	setup_dgv(2);
	
	return 0;
}

/* this function isn't used but freepbx voicemail drop the call
if ast_read isn't created */
static struct ast_frame  *dgv_read(struct ast_channel *ast)
{	
	static struct ast_frame f = { AST_FRAME_NULL, };
	f.src = ttype;
	return &f;
}

#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, tdesc,
#ifdef CC_AST_HAS_VERSION_13
						.support_level = AST_MODULE_SUPPORT_EXTENDED,
#endif//#ifdef CC_AST_HAS_VERSION_13
    			    .load = load_module,
       				.unload = unload_module,
       				.reload = reload,
           			);
#else//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
	//-------------------------------------------------------------------------------------------
	//
	//-------------------------------------------------------------------------------------------
	int usecount()
	{
		return usecnt;
	}
	
	//-------------------------------------------------------------------------------------------
	//
	//-------------------------------------------------------------------------------------------
	char *description()
	{
		return (char *) desc;
	}
	
	//-------------------------------------------------------------------------------------------
	//
	//-------------------------------------------------------------------------------------------
	char *key()
	{
		return ASTERISK_GPL_KEY;
	}
#endif//#if defined(CC_AST_HAS_VERSION_1_4) || defined(CC_AST_HAS_VERSION_1_6) || defined(CC_AST_HAS_VERSION_1_8)
